import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/database/entity/notification.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';


class NotificationDao{
  Database _db;
  String _tableName = 'notifications';

  Future open() async {
    _db = await openDatabase(
      join(await getDatabasesPath(), 'systemNotifications.db'),
      onCreate: (db, version) {
        return db.execute(
          "CREATE TABLE IF NOT EXISTS $_tableName(`campaignId` INTEGER PRIMARY KEY AUTOINCREMENT, `appId` TEXT, `campaignTypeId` INTEGER, `scheduleAt` INTEGER NOT NULL, `title` TEXT, `description` TEXT, `url` TEXT, `targetId` INTEGER, `target` TEXT, `status` TEXT, `brief` TEXT)",
        );
      },
      version: 1,
    );
    try {
      _db.rawQuery("PRAGMA encoding ='UTF-8'");
      _db.rawQuery( "PRAGMA journal_mode ='delete'");
    }catch (e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

  }

  Future<void> insert(Notification notification) async {
    await _db.insert(
      _tableName,
      notification.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<Notification> findById(int id) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'campaignId=?',
        whereArgs: [id]
    );
    if (maps.isNotEmpty) {
      Notification notification = new Notification();
      notification.fromMap(maps.first);
      return notification;
    }else
      return null;
  }

  Future<int> deleteById( int id ) async{
    return await _db.delete(
        _tableName,
        where: 'campaignId=?',
        whereArgs: [id]
    );
  }

  Future<void> update( Notification notification ) async{
    await _db.update(
        _tableName,
        notification.toMap(),
        where: 'campaignId=?',
        whereArgs: [notification.campaignId],
        conflictAlgorithm: ConflictAlgorithm.replace
    );
  }

  Future<List<Notification>> notifications(int limit, int offset) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        limit: limit,
        offset: offset,
        orderBy: "campaignId DESC"
    );
    return List.generate( maps.length, (i) {
      Notification notification = new Notification();
      notification.fromMap(maps[i]);
      return notification;
    });

  }

  Future<int> lastId( ) async{
    final List<Map<String, dynamic>> maps = await _db.rawQuery("SELECT campaignId FROM $_tableName ORDER BY campaignId DESC LIMIT 1");
    if( maps.isNotEmpty){
      return maps.first['campaignId'];
    }else
      return 0;
  }

  Future<int> unreadCount() async{
    return Sqflite.firstIntValue( await _db.rawQuery("SELECT COUNT(*) FROM $_tableName WHERE status='" + Notification.STATUS_UNREAD + "'") );
  }

  Future<void> delete() async{
    // _db.delete(_tableName);
    await _db.rawDelete("Delete from " + _tableName);
    close();
  }

  Future close() async => _db.close();
}