
import 'dart:convert';
import 'package:easy_localization/easy_localization.dart';
import 'package:equatable/equatable.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';


class PickupCreateRequest extends Equatable{

  String promoCode;
  String branchGuid;
  DeliveryContactItem pickup;
  int totalWays;
  bool specialCase;
  bool refundInAdvanced;
  double refundAmount;
  DateTime refundDate;
  String lang;
  String remark;
  String photo;
  List<String> images;
  String guid;
  DeliveryContactItem assignee;
  List<DeliveryListItem> items;
  double pickupCharges;
  bool refunded;
  String status;
  DateTime pickupOn;
  DateTime cancelOn;
  String trackingId;
  DateTime createdOn;
  DateTime updatedOn;
  int completedCount;
  double promoAmount;
  bool carRequired;
  double cashToCollect;
  bool preRefundRemind;
  double preRefundAmount;
  String preRefundMessage;
  String refundOption;
  bool approved;
  double cashAdvanceAmount;
  bool cashAdvanceRelease;
  DateTime cashAdvanceReleaseDate;

  @override
  String toString() => this.guid;

  PickupCreateRequest({this.promoCode, this.branchGuid, this.pickup, this.totalWays,
    this.specialCase, this.refundInAdvanced, this.refundAmount, this.refundDate, this.lang,
    this.remark, this.photo, this.images, this.guid, this.assignee,
    this.items, this.pickupCharges, this.refunded, this.status,
  this.pickupOn, this.cancelOn, this.trackingId, this.createdOn, this.updatedOn, this.completedCount,
  this.carRequired, this.cashToCollect, this.promoAmount, this.preRefundRemind,
  this.preRefundAmount, this.preRefundMessage, this.refundOption, this.approved,
  this.cashAdvanceAmount, this.cashAdvanceRelease, this.cashAdvanceReleaseDate});

  @override
  List<Object> get props => [this.promoCode, this.branchGuid, this.pickup, this.totalWays,
    this.specialCase, this.refundInAdvanced, this.refundAmount, this.refundDate, this.lang,
    this.remark, this.photo, this.images, this.guid, this.assignee,
    this.items, this.pickupCharges, this.refunded, this.status,
    this.pickupOn, this.cancelOn, this.trackingId, this.createdOn, this.updatedOn, this.completedCount,
  carRequired, cashToCollect, promoAmount, preRefundRemind, preRefundAmount, preRefundMessage,
  refundOption, approved, cashAdvanceAmount, cashAdvanceRelease, cashAdvanceReleaseDate];


  Map<String, dynamic> toMap(){
    Map<String, dynamic> data = {
      'promoCode': promoCode,
      'branchGuid': branchGuid,
      'pickup': pickup!=null? pickup.toMap():null,
      'totalWays': totalWays,
      'specialCase': specialCase,
      'refundInAdvanced': refundInAdvanced,
      'refundAmount': refundAmount,
      'refundDate': refundDate!=null? new DateFormat('yyyy-MM-dd').format(refundDate) : null,
      'lang': lang,
      'remark': remark,
      'photo': photo,
      'images': images,
      'guid': guid,
      'promoAmount': promoAmount,
      'carRequired': carRequired,
      'cashToCollect':cashToCollect,
      'refundOption':refundOption,
      'approved':approved,
      'cashAdvanceAmount':cashAdvanceAmount,
      'assignee': assignee!=null? assignee.toMap():null,
    };
    return data;
  }

  Future<bool> fromJson(String json) async{
    if( json == null) return false;

    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      totalWays = data["totalWays"];
      specialCase = data["specialCase"];
      refundInAdvanced = data["refundInAdvanced"];
      refundAmount = data["refundAmount"];
      refundDate = data["refundDate"]!=null? DateTime.parse(data["refundDate"]) : null;
      remark = data["remark"];
      guid = data["guid"];
      status = data["status"];
      trackingId = data["trackingId"];
      pickupCharges = data["pickupCharges"];
      refunded = data["refunded"];
      completedCount = data["completedCount"];
      createdOn = data["pickupOn"]!=null? DateTime.parse(data["pickupOn"]) : null;
      updatedOn = data["cancelOn"]!=null? DateTime.parse(data["cancelOn"]) : null;
      createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
      updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
      preRefundRemind = data["preRefundRemind"]??false;
      preRefundAmount = data["preRefundAmount"]??0.0;
      preRefundMessage = data["preRefundMessage"];
      refundOption = data["refundOption"];
      approved = data["approved"];
      cashAdvanceRelease = data["cashAdvanceRelease"]??false;
      cashAdvanceAmount = data["cashAdvanceAmount"]??0;
      cashAdvanceReleaseDate = data["cashAdvanceReleaseDate"]!=null? DateTime.parse(data["cashAdvanceReleaseDate"]) : null;


      if( data["pickup"] !=null){
        pickup = new DeliveryContactItem();
        pickup.fromMap( data["pickup"] );
      }

      if( data["assignee"] != null) {
        assignee = new DeliveryContactItem();
        assignee.fromMap(data["assignee"]);
      }

      final deliveryLitData = data["items"] as List;
      items = new List<DeliveryListItem>();
      deliveryLitData.forEach((element) {
        DeliveryListItem deliveryListItem = new DeliveryListItem();
        deliveryListItem.fromMap(element);
        items.add(deliveryListItem);
      });
      return true;
    }else{
      return false;
    }

    return false;
  }
}