import 'dart:convert';
import 'dart:ffi';
import 'package:equatable/equatable.dart';



class AddressBookListResponse extends Equatable{

  List<AddressBookItem> items;
  int totalRecords;


  AddressBookListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<AddressBookItem>();
      addressBookListData.forEach((element) {
        AddressBookItem addressBookItem = new AddressBookItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class AddressBookItem extends Equatable{

  String personGuid;
  String personId;
  String name;
  String address;
  String street;
  String townshipName;
  String townshipGuid;
  String addressType;
  String primaryPhone;
  String otherPhones;
  String email;
  double latitude;
  double longitude;
  String comment;
  bool active;
  DateTime updatedOn;
  String guid;
  String avatar;
  String stateGuid;
  bool primaryContact;

  AddressBookItem({this.personGuid, this.name, this.address, this.street,
  this.townshipName, this.townshipGuid, this.addressType, this.primaryPhone,
  this.otherPhones, this.email, this.latitude, this.longitude,
  this.comment, this.active, this.updatedOn, this.guid,
  this.avatar, this.stateGuid, this.primaryContact, this.personId});
  @override
  List<Object> get props => [personGuid, name, address, street,
    townshipName, townshipGuid, addressType, primaryPhone,
    otherPhones, email, latitude, longitude,
    comment, active, updatedOn, guid,
    avatar, stateGuid, primaryContact, this.personId];
  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    personGuid = data["personGuid"];
    personId = data["personId"];
    name = data["name"];
    address = data["address"];
    street = data["street"];
    townshipName = data["townshipName"];
    townshipGuid = data["townshipGuid"];
    addressType = data["addressType"];
    primaryPhone = data["primaryPhone"];
    otherPhones = data["otherPhones"];
    email = data["email"];
    latitude = data["latitude"];
    longitude = data["longitude"];
    comment = data["comment"];
    active = data["active"];
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    guid = data["guid"];
    avatar = data["avatar"];
    stateGuid = data["stateGuid"];
    primaryContact = data["primaryContact"];
  }

  Map<String, dynamic> toMap(){
    return {
      'personGuid': personGuid,
      'name': name,
      'address': address,
      'street': street,
      'townshipName': townshipName,
      'townshipGuid': townshipGuid,
      'addressType': addressType,
      'primaryPhone': primaryPhone,
      'otherPhones': otherPhones,
      'email': email,
      'latitude': latitude,
      'longitude': longitude,
      'comment': comment,
      'active': active,
      'updatedOn': null,
      'guid': guid,
      'avatar': avatar,
      'stateGuid': stateGuid,
      'primaryContact': primaryContact,
    };
  }

}