

import 'dart:convert';
import 'package:equatable/equatable.dart';


class BankListResponse extends Equatable{

  List<BankItem> items;
  int totalRecords;


  BankListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<BankItem>();
      addressBookListData.forEach((element) {
        BankItem addressBookItem = new BankItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class BankItem extends Equatable{

  String name;
  String description;
  String address;
  String phone;
  int country;
  String type;
  bool active;
  String guid;
  String website;
  DateTime createdOn;
  DateTime updatedOn;

  BankItem({this.name, this.description, this.address, this.phone, this.country,
    this.type, this.active, this.guid, this.website, this.createdOn, this.updatedOn});

  @override
  List<Object> get props => [name, description, address, phone, country,
    type, active, guid, website, createdOn, updatedOn];

  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    name = data["name"];
    description = data["description"];
    address = data["address"];
    phone = data["phone"];
    country = data["country"];
    type = data["type"];
    active = data["active"];
    guid = data["guid"];
    website = data["website"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
  }
}