import 'dart:convert';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:equatable/equatable.dart';


class DeliveryDetailResponse extends Equatable{

  String guid;
  String trackingId;
  String status;
  double totalCharges;
  String deliveryChargeOption;
  String itemPriceOption;
  double labourCost;
  double transportCost;
  double extraCost;
  String extraPaidBy;
  bool specialCase;
  bool labourRequired;
  DateTime createdOn;
  DateTime updatedOn;
  DateTime wayOn;
  DateTime pickupOn;
  DateTime doneOn;
  DateTime returnOn;
  DateTime cancelOn;
  List<DeliveryContactItem> contacts;
  List<DeliveryItem> items;
  String message;
  double promotion;
  bool refundInAdvanced;
  double refundAmount;
  DateTime refundDate;
  bool approved;
  String deliveryMethods;
  String pickupStatus;
  String remark;

  DeliveryDetailResponse({this.guid, this.trackingId, this.status, this.totalCharges,
  this.deliveryChargeOption, this.itemPriceOption, this.labourCost, this.transportCost,
  this.extraCost, this.extraPaidBy, this.specialCase, this.labourRequired,
  this.createdOn, this.updatedOn, this.wayOn, this.pickupOn, this.doneOn,
  this.returnOn, this.cancelOn, this.contacts, this.items, this.message,
  this.promotion, this.refundInAdvanced, this.refundAmount, this.refundDate,
  this.approved, this.deliveryMethods, this.pickupStatus, this.remark});

  @override
  List<Object> get props => [guid, trackingId, status, totalCharges,
    deliveryChargeOption, itemPriceOption, labourCost, transportCost,
    extraCost, extraPaidBy, specialCase, labourRequired,
    createdOn, updatedOn, wayOn, pickupOn, doneOn,
    returnOn, cancelOn, contacts, items, message, promotion, refundInAdvanced,
    refundAmount, refundDate, approved, deliveryMethods, pickupStatus, remark];

  @override
  String toString() => 'Way ID=${this.trackingId}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      guid = data["guid"];
      trackingId = data["trackingId"];
      status = data["status"];
      totalCharges = data["totalCharges"];
      deliveryChargeOption = data["deliveryChargeOption"];
      itemPriceOption = data["itemPriceOption"];
      labourCost = data["labourCost"];
      transportCost = data["transportCost"];
      extraCost = data["extraCost"];
      extraPaidBy = data["extraPaidBy"];
      specialCase = data["specialCase"];
      labourRequired = data["labourRequired"];
      message = data["message"];
      promotion = data["promotion"]??0.0;
      createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
      updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
      wayOn = data["wayOn"]!=null? DateTime.parse(data["wayOn"]) : null;
      pickupOn = data["pickupOn"]!=null? DateTime.parse(data["pickupOn"]) : null;
      doneOn = data["doneOn"]!=null? DateTime.parse(data["doneOn"]) : null;
      returnOn = data["returnOn"]!=null? DateTime.parse(data["returnOn"]) : null;
      cancelOn = data["cancelOn"]!=null? DateTime.parse(data["cancelOn"]) : null;
      refundInAdvanced = data["refundInAdvanced"];
      refundAmount = data["refundAmount"];
      refundDate = data["refundDate"]!=null? DateTime.parse(data["refundDate"]) : null;
      approved = data["approved"];
      deliveryMethods = data["deliveryMethods"];
      pickupStatus = data["pickupStatus"];
      remark = data["remark"]??'';

      final deliveryItems = data["items"] as List;
      items = new List<DeliveryItem>();
      deliveryItems.forEach((element) {
        DeliveryItem deliveryItem = new DeliveryItem();
        deliveryItem.fromMap(element);
        items.add(deliveryItem);
      });
      final contactItems = data["contacts"] as List;
      contacts = new List<DeliveryContactItem>();
      contactItems.forEach((element) {
        DeliveryContactItem deliveryContactItem = new DeliveryContactItem();
        deliveryContactItem.fromMap(element);
        contacts.add(deliveryContactItem);
      });
      return true;

    }else{
      return false;
    }
  }
}