import 'dart:convert';
import 'package:equatable/equatable.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';



class DeliveryListResponse extends Equatable{

  List<DeliveryListItem> items;
  int totalRecords;


  DeliveryListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final deliveryLitData = data["items"] as List;
      items = new List<DeliveryListItem>();
      deliveryLitData.forEach((element) {
        DeliveryListItem deliveryListItem = new DeliveryListItem();
        deliveryListItem.fromMap(element);
        items.add(deliveryListItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class DeliveryListItem extends Equatable{

  String contentGUID;
  String trackingId;
  String status;
  DateTime createdOn;
  DateTime updatedOn;
  double totalCharges;
  double forShowCharges;
  int totalItems;
  String zoneName;
  String zoneGUID;
  // OS pickup info
  String osName;
  String osAddress;
  DateTime pickupDate;
  String osGUID;
  String osContactGUID;
  // Customer drop-off info
  String customerName;
  String customerAddress;
  DateTime dropOffDate;
  String customerGUID;
  String customerContactGUID;
  // pickup person info
  String pickupPersonName;
  String pickupPersonGUID;
  String pickupPersonAssignGUID;
  double pickupPersonLatitude;
  double pickupPersonLongitude;
  // drop-off person info
  String dropOffPersonName;
  String dropOffPersonGUID;
  String dropOffPersonAssignGUID;
  double dropOffPersonLatitude;
  double dropOffPersonLongitude;
  double labourCost;
  double transportCost;
  double extraCost;
  double totalDeliveryCharges;
  double totalItemPrice;
  double promotionAmount;
  double dueAmount;

  String osPrimaryPhone;
  String osOtherPhones;
  String osComment;
  double osLatitude;
  double osLongitude;
  String osPersonId;
  String osTownshipName;
  String osAddressType;
  String customerPrimaryPhone;
  String customerOtherPhones;
  String customerComment;
  double customerLatitude;
  double customerLongitude;
  String customerPersonId;
  String customerTownshipName;
  String customerAddressType;
  String pickupPersonPrimaryPhone;
  String pickupPersonOtherPhones;
  String pickupPersonAvatar;
  String dropOffPersonPrimaryPhone;
  String dropOffPersonOtherPhones;
  String dropOffPersonAvatar;
  bool refundInAdvanced;
  double refundAmount;
  DateTime refundDate;
  String name;
  String townshipGuid;
  int totalWays;
  String images;
  int itemCount;
  int addressCount;
  int completedCount;
  double pickupCharges;
  bool refunded;
  DeliveryItem deliveryItem;
  double totalCashToCollect;
  bool checked;
  String highestMessage;

  String roleName;
  String refundOption;
  bool carRequired;
  String inboundStationGuid;
  String inboundStationName;
  String outboundStationGuid;
  String outboundStationName;
  String inboundBucketGuid;
  String inboundBucketName;
  String inboundBucketStatus;
  String outboundBucketGuid;
  String outboundBucketName;
  String outboundBucketStatus;
  String scheduleGuid;
  Duration scheduleTime;
  String codTransferred;
  String codCollected;
  String deliveryMethods;
  String customerSignature;
  String dropOffPersonSignature;
  bool isSelected;
  double cashAdvanceAmount;
  bool cashAdvanceRelease;
  DateTime cashAdvanceReleaseDate;
  String remark;

  DeliveryListItem({this.contentGUID, this.trackingId, this.status, this.createdOn,
  this.updatedOn, this.totalCharges, this.totalItems, this.zoneName, this.zoneGUID,
  this.osName, this.osAddress, this.pickupDate, this.osGUID, this.osContactGUID,
  this.customerName, this.customerAddress, this.dropOffDate, this.customerGUID,
  this.customerContactGUID, this.pickupPersonName, this.pickupPersonGUID,
  this.pickupPersonAssignGUID, this.pickupPersonLatitude, this.pickupPersonLongitude,
  this.dropOffPersonName, this.dropOffPersonGUID, this.dropOffPersonAssignGUID,
  this.dropOffPersonLatitude, this.dropOffPersonLongitude,
  this.labourCost, this.transportCost, this.extraCost,
  this.totalDeliveryCharges, this.totalItemPrice, this.promotionAmount,this.dueAmount,
  this.osPrimaryPhone, this.osOtherPhones, this.osComment, this.osLatitude,
  this.osLongitude, this.customerPrimaryPhone, this.customerOtherPhones,
  this.customerComment, this.customerLatitude, this.customerLongitude,
  this.pickupPersonPrimaryPhone, this.pickupPersonOtherPhones, this.pickupPersonAvatar,
  this.dropOffPersonPrimaryPhone, this.dropOffPersonOtherPhones, this.dropOffPersonAvatar,
  this.refundInAdvanced, this.refundAmount, this.refundDate, this.forShowCharges,
  this.name, this.townshipGuid, this.totalWays, this.images, this.itemCount, this.addressCount,
  this.completedCount, this.pickupCharges, this.refunded, this.deliveryItem,
  this.osPersonId, this.osTownshipName, this.customerPersonId, this.customerTownshipName,
  this.osAddressType, this.customerAddressType, this.totalCashToCollect, this.checked,
    this.highestMessage, this.roleName, this.refundOption, this.carRequired, this.inboundStationGuid,
  this.inboundStationName, this.outboundStationGuid, this.outboundStationName,
  this.inboundBucketGuid, this.inboundBucketName, this.inboundBucketStatus,
  this.outboundBucketGuid, this.outboundBucketName, this.outboundBucketStatus,
  this.scheduleGuid, this.scheduleTime, this.codTransferred, this.codCollected,
  this.deliveryMethods, this.customerSignature, this.dropOffPersonSignature,
    this.isSelected, this.cashAdvanceAmount, this.cashAdvanceRelease, this.cashAdvanceReleaseDate,
  this.remark});


  @override
  List<Object> get props => [contentGUID, trackingId, status, createdOn,
    updatedOn, totalCharges, totalItems, zoneName, zoneGUID,
    osName, osAddress, pickupDate, osGUID, osContactGUID,
    customerName, customerAddress, dropOffDate, customerGUID,
    customerContactGUID, pickupPersonName, pickupPersonGUID,
    pickupPersonAssignGUID, pickupPersonLatitude, pickupPersonLongitude,
    dropOffPersonName, dropOffPersonGUID, dropOffPersonAssignGUID, dropOffPersonLatitude,
    dropOffPersonLongitude, labourCost, transportCost, extraCost,
    totalDeliveryCharges, totalItemPrice, promotionAmount, dueAmount,
    osPrimaryPhone, osOtherPhones, osComment, osLatitude,
    osLongitude, customerPrimaryPhone, customerOtherPhones,
    customerComment, customerLatitude, customerLongitude,
    pickupPersonPrimaryPhone, pickupPersonOtherPhones, pickupPersonAvatar,
    dropOffPersonPrimaryPhone, dropOffPersonOtherPhones, dropOffPersonAvatar,
    refundInAdvanced, refundAmount, refundDate, forShowCharges,
    name, townshipGuid, totalWays, images, itemCount, addressCount,
    completedCount, pickupCharges, refunded, deliveryItem,
    osPersonId, osTownshipName, customerPersonId, customerTownshipName,
    osAddressType, customerAddressType, totalCashToCollect, checked, highestMessage,
    roleName, refundOption, carRequired, inboundStationGuid,
    inboundStationName, outboundStationGuid, outboundStationName,
    inboundBucketGuid, inboundBucketName, inboundBucketStatus,
    outboundBucketGuid, outboundBucketName, outboundBucketStatus,
    scheduleGuid, scheduleTime, codTransferred, codCollected,
    deliveryMethods, customerSignature, dropOffPersonSignature, isSelected,
    cashAdvanceAmount, cashAdvanceRelease, cashAdvanceReleaseDate, remark];

  @override
  String toString() => this.status;

  fromMap( Map<String, dynamic> data ){
    contentGUID = data["contentGUID"];
    trackingId = data["trackingId"];
    status = data["status"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    totalCharges = data["totalCharges"];
    forShowCharges = data["forShowCharges"];
    totalItems = data["totalItems"];
    zoneName = data["zoneName"];
    zoneGUID = data["zoneGUID"];
    osName = data["osName"];
    osAddress = data["osAddress"];
    pickupDate = data["pickupDate"]!=null? DateTime.parse(data["pickupDate"]) : null;
    osGUID = data["osGUID"];
    osContactGUID = data["osContactGUID"];
    customerName = data["customerName"];
    customerAddress = data["customerAddress"];
    dropOffDate =data["dropOffDate"]!=null? DateTime.parse(data["dropOffDate"]) : null;
    customerGUID = data["customerGUID"];
    customerContactGUID = data["customerContactGUID"];
    pickupPersonName = data["pickupPersonName"];
    pickupPersonGUID = data["pickupPersonGUID"];
    pickupPersonAssignGUID = data["pickupPersonAssignGUID"];
    pickupPersonLatitude = data["pickupPersonLatitude"];
    pickupPersonLongitude = data["pickupPersonLongitude"];
    dropOffPersonName = data["dropOffPersonName"];
    dropOffPersonGUID = data["dropOffPersonGUID"];
    dropOffPersonAssignGUID = data["dropOffPersonAssignGUID"];
    dropOffPersonLatitude = data["dropOffPersonLatitude"];
    pickupPersonLongitude = data["pickupPersonLongitude"];
    labourCost = data["labourCost"];
    transportCost = data["transportCost"];
    extraCost = data["extraCost"];
    totalDeliveryCharges = data["totalDeliveryCharges"]??0.0;
    totalItemPrice = data["totalItemPrice"]??0.0;
    promotionAmount = data["promotionAmount"]??0.0;
    dueAmount = data["dueAmount"];
    osPrimaryPhone = data["osPrimaryPhone"];
    osOtherPhones = data["osOtherPhones"];
    osComment = data["osComment"];
    osLatitude =data["osLatitude"];
    osLongitude = data["osLongitude"];
    customerPrimaryPhone = data["customerPrimaryPhone"];
    customerOtherPhones = data["customerOtherPhones"];
    customerComment = data["customerComment"];
    customerLatitude = data["customerLatitude"];
    customerLongitude = data["customerLongitude"];
    pickupPersonPrimaryPhone = data["pickupPersonPrimaryPhone"];
    pickupPersonOtherPhones = data["pickupPersonOtherPhones"];
    pickupPersonAvatar = data["pickupPersonAvatar"];
    dropOffPersonPrimaryPhone = data["dropOffPersonPrimaryPhone"];
    dropOffPersonOtherPhones = data["dropOffPersonOtherPhones"];
    dropOffPersonAvatar = data["dropOffPersonAvatar"];
    refundInAdvanced = data["refundInAdvanced"];
    refundAmount = data["refundAmount"]??0.0;
    refundDate = data["refundDate"]!=null? DateTime.parse(data["refundDate"]) : null;
    name = data["name"];
    townshipGuid = data["townshipGuid"];
    totalWays = data["totalWays"];
    images = data["images"];
    itemCount = data["itemCount"]??0;
    addressCount = data["addressCount"]??0;
    completedCount = data["completedCount"]??0;
    pickupCharges = data["pickupCharges"];
    refunded = data["refunded"];
    osPersonId = data["osPersonId"];
    osTownshipName = data["osTownshipName"];
    customerPersonId = data["customerPersonId"];
    customerTownshipName = data["customerTownshipName"];
    if( data["deliveryItem"] != null){
      deliveryItem = new DeliveryItem();
      deliveryItem.fromMap(data["deliveryItem"]);
    }
    osAddressType = data["osAddressType"];
    customerAddressType = data["customerAddressType"];
    totalCashToCollect = data["totalCashToCollect"];
    checked = data["checked"];
    highestMessage = data["highestMessage"];
    roleName = data["roleName"];
    refundOption = data["refundOption"];
    carRequired = data["carRequired"];
    inboundStationGuid = data["inboundStationGuid"];
    inboundStationName = data["inboundStationName"];
    outboundStationGuid = data["outboundStationGuid"];
    outboundStationName = data["outboundStationName"];
    inboundBucketGuid = data["inboundBucketGuid"];
    inboundBucketName = data["inboundBucketName"];
    inboundBucketStatus = data["inboundBucketStatus"];
    outboundBucketGuid = data["outboundBucketGuid"];
    outboundBucketName = data["outboundBucketName"];
    outboundBucketStatus = data["outboundBucketStatus"];
    scheduleGuid = data["scheduleGuid"];
    scheduleTime =data["scheduleTime"]!=null?
    Duration(hours: int.parse(data["scheduleTime"].toString().substring(0, data["scheduleTime"].toString().indexOf(':'))),
        minutes: int.parse(data["scheduleTime"].toString().substring(data["scheduleTime"].toString().indexOf(':') + 1))
    ) : null;
    codTransferred = data["codTransferred"];
    codCollected = data["codCollected"];
    deliveryMethods = data["deliveryMethods"];
    customerSignature = data["customerSignature"];
    dropOffPersonSignature = data["dropOffPersonSignature"];
    isSelected = false;
    cashAdvanceRelease = data["refundInAdvanced"]??false;
    cashAdvanceAmount = data["cashAdvanceAmount"]??0.0;
    cashAdvanceReleaseDate = data["cashAdvanceReleaseDate"]!=null? DateTime.parse(data["cashAdvanceReleaseDate"]) : null;
    remark = data["remark"];
  }

}