import 'dart:convert';
import 'dart:ffi';
import 'package:equatable/equatable.dart';



class DepartmentListResponse extends Equatable{

  List<DepartmentItem> items;
  int totalRecords;


  DepartmentListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<DepartmentItem>();
      addressBookListData.forEach((element) {
        DepartmentItem addressBookItem = new DepartmentItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class DepartmentItem extends Equatable{

  String guid;
  String name;
  String phone;
  String email;
  String address;
  String description;
  DateTime createdOn;
  DateTime updatedOn;
  String parentName;
  String parentGUID;
  bool visible;
  bool deletable;
  String avartarPhoto;
  String coverPhoto;
  String companyId;
  String website;
  String facebook;
  String about;

  DepartmentItem({this.guid, this.name, this.phone, this.email,
  this.address, this.description, this.createdOn, this.updatedOn,
  this.parentName, this.parentGUID, this.visible, this.deletable,
  this.avartarPhoto, this.coverPhoto, this.companyId, this.website,
  this.facebook, this.about});

  @override
  List<Object> get props => [guid, name, phone, email,
    address, description, createdOn, updatedOn,
    parentName, parentGUID, visible, deletable,
    avartarPhoto, coverPhoto, companyId, website,
    facebook, about];

  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    guid = data["guid"];
    name = data["name"];
    phone = data["phone"];
    email = data["email"];
    address = data["address"];
    description = data["description"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    parentName = data["parentName"];
    parentGUID = data["parentGUID"];
    visible = data["visible"];
    deletable = data["deletable"];
    avartarPhoto = data["avartarPhoto"];
    coverPhoto = data["coverPhoto"];
    companyId = data["companyId"];
    website = data["website"];
    facebook = data["facebook"];
    about = data["about"];
  }
}