import 'dart:convert';
import 'package:equatable/equatable.dart';



class EmployeeClearanceListResponse extends Equatable{

  List<EmployeeCollectedItem> items;
  int totalRecords;
  double totalCollected;
  double collectedPrepaid;
  int totalWays;


  EmployeeClearanceListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<EmployeeCollectedItem>();
      addressBookListData.forEach((element) {
        EmployeeCollectedItem addressBookItem = new EmployeeCollectedItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      totalCollected = data["totalCollected"]??0;
      collectedPrepaid = data["collectedPrepaid"]??0;
      totalWays = data["totalWays"]??0;
      return true;
    }else{
      return false;
    }
  }
}

class EmployeeCollectedItem extends Equatable{

  String employeeName;
  String employeeGuid;
  String zoneName;
  String zoneGuid;
  DateTime wayDate;
  double collectedAmount;
  String status;
  String description;
  DateTime updatedOn;
  String guid;
  bool isClearNow;

  EmployeeCollectedItem({this.employeeName, this.employeeGuid, this.zoneName,
    this.zoneGuid, this.wayDate, this.collectedAmount, this.status,
    this.description, this.updatedOn, this.guid});

  @override
  List<Object> get props => [employeeName, employeeGuid, zoneName,
    zoneGuid, wayDate, collectedAmount, status, description, updatedOn, guid];

  @override
  String toString() => this.employeeName;

  fromMap( Map<String, dynamic> data ){
    employeeName = data["employeeName"];
    employeeGuid = data["employeeGuid"];
    zoneName = data["zoneName"];
    zoneGuid = data["zoneGuid"];
    wayDate = data["wayDate"]!=null? DateTime.parse(data["wayDate"]) : null;
    collectedAmount = data["collectedAmount"];
    status = data["status"];
    description = data["description"];
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    guid = data["guid"];
    isClearNow = false;
  }
}