import 'dart:convert';
import 'package:equatable/equatable.dart';



class OSDepositRefundListResponse extends Equatable{

  List<DepositRefundItem> items;
  int totalRecords;


  OSDepositRefundListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<DepositRefundItem>();
      addressBookListData.forEach((element) {
        DepositRefundItem addressBookItem = new DepositRefundItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class DepositRefundItem extends Equatable{

  String ownerGuid;
  String ownerName;
  String approverGuid;
  String approverName;
  double amount;
  String methods;
  String description;
  DateTime updatedOn;
  String guid;
  bool systemGenerated;
  double afterBalance;
  double lastBalance;
  double balance;

  DepositRefundItem({this.ownerGuid, this.ownerName, this.approverGuid, this.approverName,
  this.amount, this.methods, this.description, this.updatedOn, this.guid,
    this.systemGenerated, this.afterBalance, this.lastBalance, this.balance});

  @override
  List<Object> get props => [ownerGuid, ownerName, approverGuid, approverName,
    amount, methods, description, updatedOn, guid, systemGenerated,
    afterBalance, lastBalance, balance];

  @override
  String toString() => this.ownerName;

  fromMap( Map<String, dynamic> data ){
    ownerGuid = data["ownerGuid"];
    ownerName = data["ownerName"];
    approverGuid = data["approverGuid"];
    approverName = data["approverName"];
    amount = data["amount"];
    methods = data["methods"];
    description = data["description"];
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    guid = data["guid"];
    systemGenerated = data["systemGenerated"];
    afterBalance = data["afterBalance"]??0;
    lastBalance = data["lastBalance"]??0;
    balance = data["balance"]??0;
  }
}