

import 'dart:convert';
import 'package:equatable/equatable.dart';


class PersonBankListResponse extends Equatable{

  List<PersonBankItem> items;
  int totalRecords;


  PersonBankListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final addressBookListData = data["items"] as List;
      items = new List<PersonBankItem>();
      addressBookListData.forEach((element) {
        PersonBankItem addressBookItem = new PersonBankItem();
        addressBookItem.fromMap(element);
        items.add(addressBookItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class PersonBankItem extends Equatable{

  String bankName;
  String bankGuid;
  String personName;
  String personGuid;
  String accountHolderName;
  String accountNumber;
  String accountType;
  String description;
  bool active;
  String guid;
  DateTime createdOn;
  DateTime updatedOn;

  PersonBankItem({this.bankName, this.bankGuid, this.personName, this.personGuid,
    this.accountHolderName, this.accountNumber, this.accountType, this.description,
    this.active, this.guid, this.createdOn, this.updatedOn});

  @override
  List<Object> get props => [bankName, bankGuid, personName, personGuid,
    accountHolderName, accountNumber, accountType, description,
    active, guid, createdOn, updatedOn];

  @override
  String toString() => this.bankName;

  fromMap( Map<String, dynamic> data ){
    bankName = data["bankName"];
    bankGuid = data["bankGuid"];
    personName = data["personName"];
    personGuid = data["personGuid"];
    accountHolderName = data["accountHolderName"];
    accountNumber = data["accountNumber"];
    accountType = data["accountType"];
    description = data["description"];
    active = data["active"];
    guid = data["guid"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
  }

  Map<String, dynamic> toMap(){
    Map<String, dynamic> data = {
      'personGuid': personGuid,
      'bankGuid': bankGuid,
      'accountHolderName': accountHolderName,
      'accountNumber': accountNumber,
      'accountType': accountType,
      'active': active,
      'description': description,
      'guid': guid,
    };
    return data;
  }
}