import 'dart:convert';
import 'package:equatable/equatable.dart';



class POSOrderListResponse extends Equatable{

  List<POSOrderListItem> items;
  int totalRecords;


  POSOrderListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final deliveryLitData = data["items"] as List;
      items = new List<POSOrderListItem>();
      deliveryLitData.forEach((element) {
        POSOrderListItem posOrderListItem = new POSOrderListItem();
        posOrderListItem.fromMap(element);
        items.add(posOrderListItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class POSOrderListItem extends Equatable{

  int orderId;
  DateTime orderDate;
  String orderGuid;
  String buyerName;
  String townshipName;
  String emailAddress;
  String deliveryAddress;
  int totalProduct;
  int totalQuantity;
  int totalAmount;
  String contactNo;
  String staffName;
  int staffId;
  int paidAmount;

  POSOrderListItem({this.orderId, this.orderDate, this.orderGuid, this.buyerName,
    this.townshipName, this.emailAddress, this.deliveryAddress, this.totalProduct,
  this.totalQuantity, this.totalAmount, this.contactNo, this.staffName, this.staffId,
  this.paidAmount});

  @override
  List<Object> get props => [orderId, orderDate, orderGuid, buyerName,
    townshipName, emailAddress, deliveryAddress, totalProduct,
    totalQuantity, totalAmount, contactNo, staffName, staffId, paidAmount];

  @override
  String toString() => this.buyerName;

  fromMap( Map<String, dynamic> data ){
    orderId = data["orderId"];
    orderDate = data["orderDate"]!=null? DateTime.parse(data["orderDate"]) : null;
    orderGuid = data["orderGuid"];
    buyerName = data["buyerName"];
    townshipName = data["townshipName"];
    emailAddress = data["emailAddress"];
    deliveryAddress = data["deliveryAddress"];
    totalProduct = data["totalProduct"];
    totalQuantity = data["totalQuantity"];
    totalAmount = data["totalAmount"];
    contactNo = data["contactNo"];
    staffName = data["staffName"];
    staffId = data["staffId"];
    paidAmount = data["paidAmount"];
  }
}