import 'dart:convert';
import 'package:equatable/equatable.dart';



class POSProductListResponse extends Equatable{

  List<POSProductListItem> items;
  int totalRecords;


  POSProductListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final deliveryLitData = data["items"] as List;
      items = new List<POSProductListItem>();
      deliveryLitData.forEach((element) {
        POSProductListItem posOrderListItem = new POSProductListItem();
        posOrderListItem.fromMap(element);
        items.add(posOrderListItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class POSProductListItem extends Equatable{

  int id;
  String productCode;
  String displayName;
  int priceBuy;
  int priceSell;
  int stockQty;
  String description;
  String guid;
  List<String> imageGuid=[];
  List<String> categoryGuid=[];

  POSProductListItem({this.id, this.productCode, this.displayName, this.priceBuy,
    this.priceSell, this.stockQty, this.description, this.guid, this.imageGuid,
  this.categoryGuid});

  @override
  List<Object> get props => [id, productCode, displayName, priceBuy,
    priceSell, stockQty, description, guid, imageGuid, categoryGuid];

  @override
  String toString() => this.displayName;

  fromMap( Map<String, dynamic> data ){
    id = data["id"];
    productCode = data["productCode"];
    displayName = data["displayName"];
    priceBuy = data["priceBuy"];
    priceSell = data["priceSell"];
    stockQty = data["stockQty"];
    description = data["description"];
    guid = data["guid"];
  }

  Map<String, dynamic> toMap(){
    Map<String, dynamic> data = {
      'id': id,
      'productCode': productCode,
      'displayName': displayName,
      'priceBuy': priceBuy,
      'priceSell': priceSell,
      'stockQty': stockQty,
      'description': description,
      'productGuid': guid,
      'imageGuid':imageGuid,
      'categoryGuid':categoryGuid
    };
    return data;
  }

}


class POSProductDetailItem extends Equatable{

  int id;
  String productCode;
  String displayName;
  int priceBuy;
  int priceSell;
  int stockQty;
  String productGuid;
  String description;
  List<String> imageGuid=[];
  List<String> categoryGuid=[];
  String primaryImageGuid;
  List<String> categoryName=[];

  POSProductDetailItem({this.id, this.productCode, this.displayName, this.priceBuy,
    this.priceSell, this.stockQty, this.description, this.productGuid, this.imageGuid,
    this.categoryGuid, this.primaryImageGuid, this.categoryName});

  @override
  List<Object> get props => [id, productCode, displayName, priceBuy,
    priceSell, stockQty, description, productGuid, imageGuid, categoryGuid,
    primaryImageGuid, categoryName];

  @override
  String toString() => this.displayName;

  fromJson( String json ){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    Map<String, dynamic> data = resultData["data"];
    id = data["id"];
    productCode = data["productCode"];
    displayName = data["displayName"];
    priceBuy = data["priceBuy"];
    priceSell = data["priceSell"];
    stockQty = data["stockQty"];
    description = data["description"];
    productGuid = data["productGuid"];
    imageGuid = data["imageGuid"]!=null? List.castFrom(data["imageGuid"]): null;
    categoryGuid = data["categoryGuid"]!=null? List.castFrom(data["categoryGuid"]) : null;
    primaryImageGuid = data["primaryImageGuid"];
    categoryName = data["categoryName"] !=null? List.castFrom(data["categoryName"]) : null;
  }
}