import 'dart:convert';
import 'package:equatable/equatable.dart';



class POSSupplierListResponse extends Equatable{

  List<PosSupplierListItem> items;
  int totalRecords;


  POSSupplierListResponse({this.items, this.totalRecords});
  @override
  List<Object> get props => [items, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final deliveryLitData = data["items"] as List;
      items = new List<PosSupplierListItem>();
      deliveryLitData.forEach((element) {
        PosSupplierListItem posOrderListItem = new PosSupplierListItem();
        posOrderListItem.fromMap(element);
        items.add(posOrderListItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class PosSupplierListItem extends Equatable{
  int supplierID;
  String name;
  String address;
  String city;
  String region;
  String countryName;
  String phone1;
  String phone2;
  String fax;
  DateTime createdOn;
  String guid;

  PosSupplierListItem({this.supplierID, this.name, this.address,
    this.city, this.region, this.countryName, this.phone1,
    this.phone2, this.fax, this.createdOn, this.guid});

  @override
  List<Object> get props => [supplierID, name, address,
    city, region, countryName, phone1,
    phone2, fax, createdOn, guid];

  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    supplierID = data["supplierID"];
    name = data["name"];
    address = data["address"];
    city = data["city"];
    region = data["region"];
    countryName = data["countryName"];
    phone1 = data["phone1"];
    phone2 = data["phone2"];
    fax = data["fax"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    guid = data["guid"];
  }

  Map<String, dynamic> toMap(){
    Map<String, dynamic> data = {
      'id': supplierID,
      'name': name,
      'address': address,
      'city': city,
      'region': region,
      'countryName': countryName,
      'phone1': phone1,
      'phone2': phone2,
      'fax':fax,
      'guid':guid
    };
    return data;
  }
}