import 'dart:convert';
import 'package:equatable/equatable.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';



class POSOrderDetailResponseV3 extends Equatable{

  String sellerGuid;
  String sellerName;
  String buyerGuid;
  String buyerName;
  String pickupGuid;
  String pickupAddressBookGuid;
  double totalAmount;
  double paidAmount;
  String discountType;
  double discountAmount;
  double deliveryCharges;
  String status;
  String shipment;
  String remark;
  String guid;
  DateTime createdOn;
  DateTime updatedOn;
  int createdBy;
  String createdByName;
  int updatedBy;
  String updatedByName;
  List<SaleOrderDetailItem> items;
  bool cod;
  DeliveryContactItem contact;
  String message;
  String salesId;
  String paidWith;
  String paymentTransactionId;
  String paymentRef1;
  String paymentRef2;
  bool validPayment;
  String scanningData;
  int paymentTransactionRef;
  String accountName;
  String accountGuid;
  DateTime paymentReceiveOn;
  String businessName;
  String businessPhone;
  String businessGuid;
  double transactionAmount;
  String orderAs;


  POSOrderDetailResponseV3({this.sellerGuid, this.sellerName, this.buyerGuid, this.buyerName,
    this.pickupGuid, this.pickupAddressBookGuid, this.totalAmount, this.paidAmount, this.discountType, this.discountAmount,
    this.deliveryCharges, this.status, this.shipment, this.remark, this.guid, this.createdOn, this.updatedOn, this.createdBy,
    this.createdByName, this.updatedBy, this.updatedByName, this.items, this.cod, this.contact, this.message, this.salesId, this.paidWith,
    this.paymentTransactionId, this.paymentRef1, this.paymentRef2, this.validPayment, this.scanningData,
    this.paymentTransactionRef, this.accountName, this.accountGuid, this.paymentReceiveOn,
    this.businessName, this.businessPhone, this.businessGuid, this.transactionAmount, this.orderAs});

  @override
  List<Object> get props => [sellerGuid, sellerName, buyerGuid, buyerName,
    pickupGuid, pickupAddressBookGuid, totalAmount, paidAmount, discountType, discountAmount,
    deliveryCharges, status, shipment, remark, guid, createdOn, updatedOn, createdBy,
    createdByName, updatedBy, updatedByName, items, cod, contact, message, salesId, paidWith,
    paymentTransactionId, paymentRef1, paymentRef2, validPayment, scanningData,
    paymentTransactionRef, accountName, accountGuid, paymentReceiveOn,
    businessName, businessPhone, businessGuid, transactionAmount, orderAs];

  @override
  String toString() => 'Guid=${this.guid}';

  bool fromJson(String json){

    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];
      final deliveryLitData = data["items"] as List;
      items = new List<SaleOrderDetailItem>();
      deliveryLitData.forEach((element) {
        SaleOrderDetailItem posOrderListItem = new SaleOrderDetailItem();
        posOrderListItem.fromMap(element);
        items.add(posOrderListItem);
      });
      Map<String, dynamic> contactData = data["contact"];
      contact = new DeliveryContactItem();
      contact.fromMap(contactData);

      sellerGuid = data["sellerGuid"];
      sellerName = data["sellerName"];
      buyerGuid = data["buyerGuid"];
      buyerName = data["buyerName"];
      pickupGuid = data["pickupGuid"];
      pickupAddressBookGuid = data["pickupAddressBookGuid"];
      totalAmount = data["totalAmount"];
      paidAmount = data["paidAmount"];
      discountType = data["discountType"];
      discountAmount = data["discountAmount"];
      deliveryCharges = data["deliveryCharges"];
      status = data["status"];
      shipment = data["shipment"];
      remark = data["remark"];
      guid = data["guid"];
      createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
      updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
      createdBy = data["createdBy"];
      createdByName = data["createdByName"];
      updatedBy = data["updatedBy"];
      updatedByName = data["updatedByName"];
      cod = data["cod"];
      message = data["message"];
      salesId = data["salesId"];
      paidWith = data["paidWith"];
      paymentTransactionId = data["paymentTransactionId"];
      paymentRef1 = data["paymentRef1"];
      paymentRef2 = data["paymentRef2"];
      validPayment = data["validPayment"]??false;
      scanningData = data["scanningData"];
      paymentTransactionRef = data["paymentTransactionRef"];
      accountName = data["accountName"];
      accountGuid = data["accountGuid"];
      paymentReceiveOn = data["paymentReceiveOn"]!=null? DateTime.parse(data["paymentReceiveOn"]) : null;
      businessName = data["businessName"];
      businessPhone = data["businessPhone"];
      businessGuid = data["businessGuid"];
      transactionAmount = data["transactionAmount"];
      orderAs = data["orderAs"];
      return true;
    }else{
      return false;
    }
  }
}

class SaleOrderDetailItem extends Equatable{

  String productGuid;
  String productName;
  String productType;
  String saleOrderGuid;
  String deliveryGuid;
  String deliveryTracking;
  String deliveryStatus;
  String deliveryPerson;
  String deliveryPersonGuid;
  DateTime deliveryDate;
  String previewLabel;
  String previewPhoto1;
  String previewPhoto2;
  int quantity;
  double price;
  double originalPrice;
  String status;
  String condition;
  String shipment;
  bool cod;
  String paidWith;
  String paymentTransactionId;
  String paymentRef1;
  String paymentRef2;
  bool validPayment;
  String remark;
  String guid;
  DateTime createdOn;
  DateTime updatedOn;
  int createdBy;
  String createdByName;
  int updatedBy;
  String updatedByName;
  String discountType;
  double discountAmount;
  int deliveryBatchId;
  DateTime endOn;
  String purchaseDetailGuid;
  DateTime resultDate;


  SaleOrderDetailItem({this.productGuid, this.productName, this.productType, this.saleOrderGuid,
    this.deliveryGuid, this.deliveryTracking, this.deliveryStatus, this.deliveryPerson,
    this.deliveryPersonGuid, this.deliveryDate, this.previewLabel, this.previewPhoto1, this.previewPhoto2,
    this.quantity, this.price, this.originalPrice, this.status, this.condition, this.shipment, this.cod, this.paidWith,
    this.paymentTransactionId, this.paymentRef1, this.paymentRef2, this.validPayment, this.remark,
    this.guid, this.createdOn, this.updatedOn, this.createdBy, this.createdByName, this.updatedBy,
    this.updatedByName, this.discountType, this.discountAmount, this.deliveryBatchId, this.endOn,
    this.purchaseDetailGuid, this.resultDate});

  @override
  List<Object> get props => [productGuid, productName, productType, saleOrderGuid,
    deliveryGuid, deliveryTracking, deliveryStatus, deliveryPerson,
    deliveryPersonGuid, deliveryDate, previewLabel, previewPhoto1, previewPhoto2,
    quantity, price, originalPrice, status, condition, shipment, cod, paidWith,
    paymentTransactionId, paymentRef1, paymentRef2, validPayment, remark,
    guid, createdOn, updatedOn, createdBy, createdByName, updatedBy,
    updatedByName, discountType, discountAmount, deliveryBatchId, endOn,
    purchaseDetailGuid, resultDate];

  @override
  String toString() => this.productName;

  fromMap( Map<String, dynamic> data ){
    productGuid = data["productGuid"];
    productName = data["productName"];
    productType = data["productType"];
    saleOrderGuid = data["saleOrderGuid"];
    deliveryGuid = data["deliveryGuid"];
    deliveryTracking = data["deliveryTracking"];
    deliveryStatus = data["deliveryStatus"];
    deliveryPerson = data["deliveryPerson"];
    deliveryPersonGuid = data["deliveryPersonGuid"];
    deliveryDate = data["deliveryDate"]!=null? DateTime.parse(data["deliveryDate"]) : null;
    previewLabel = data["previewLabel"];
    previewPhoto1 = data["previewPhoto1"];
    previewPhoto2 = data["previewPhoto2"];
    quantity = data["quantity"];
    price = data["price"];
    originalPrice = data["originalPrice"];
    status = data["status"];
    condition = data["condition"];
    shipment = data["shipment"];
    cod = data["cod"];
    paidWith = data["paidWith"];
    paymentTransactionId = data["paymentTransactionId"];
    paymentRef1 = data["paymentRef1"];
    paymentRef2 = data["paymentRef2"];
    validPayment = data["validPayment"];
    remark = data["remark"];
    guid = data["guid"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    createdBy = data["createdBy"];
    createdByName = data["createdByName"];
    updatedBy = data["updatedBy"];
    updatedByName = data["updatedByName"];
    discountType = data["discountType"];
    discountAmount = data["discountAmount"];
    deliveryBatchId = data["deliveryBatchId"];
    endOn = data["endOn"]!=null? DateTime.parse(data["endOn"]) : null;
    purchaseDetailGuid = data["purchaseDetailGuid"];
    resultDate = data["resultDate"]!=null? DateTime.parse(data["resultDate"]) : (data["endOn"]!=null? DateTime.parse(data["endOn"]) : null);
  }
}

