import 'dart:convert';
import 'package:equatable/equatable.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';

import 'address_book_list_response.dart';



class RouteListByNavigationResponse extends Equatable{

  String sourceStationGuid;
  String sourceStationName;
  double sourceLatitude;
  double sourceLongitude;
  String targetStationGuid;
  String targetStationName;
  double targetLatitude;
  double targetLongitude;
  double carDistance;
  List<LatLng> carRoutePoints;
  double bikeDistance;
  List<LatLng> bikeRoutePoints;
  double footDistance;
  List<LatLng> footRoutePoints;

  RouteListByNavigationResponse({this.sourceStationGuid, this.sourceStationName,
    this.sourceLatitude, this.sourceLongitude, this.targetStationGuid,
    this.targetStationName, this.targetLatitude, this.targetLongitude,
    this.carDistance, this.carRoutePoints, this.bikeDistance,
    this.bikeRoutePoints, this.footDistance, this.footRoutePoints});

  @override
  List<Object> get props => [sourceStationGuid, sourceStationName,
    sourceLatitude, sourceLongitude, targetStationGuid,
    targetStationName, targetLatitude, targetLongitude,
    carDistance, carRoutePoints, bikeDistance,
    bikeRoutePoints, footDistance, footRoutePoints];

  @override
  String toString() => 'Distance=${this.carDistance}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      sourceStationGuid = data["sourceStationGuid"];
      sourceStationName = data["sourceStationName"];
      sourceLatitude = data["sourceLatitude"];
      sourceLongitude = data["sourceLongitude"];
      targetStationGuid = data["targetStationGuid"];
      targetStationName = data["targetStationName"];
      targetLatitude = data["targetLatitude"];
      targetLongitude = data["targetLongitude"];
      carDistance = data["carDistance"];
      // carRoutePoints = data["carRoutePoints"];
      bikeDistance = data["bikeDistance"];
      // bikeRoutePoints = data["bikeRoutePoints"];
      footDistance = data["footDistance"];
      // footRoutePoints = data["footRoutePoints"];

      final carRoutePointList = data["carRoutePoints"]!=null? data["carRoutePoints"] as List : null;
      if( carRoutePointList !=null ){
        carRoutePoints = [];
        carRoutePointList.forEach((element) {
          carRoutePoints.add(LatLng(element[0], element[1]));
        });
      }

      final bikeRoutePointList = data["bikeRoutePoints"]!=null? data["bikeRoutePoints"] as List : null;
      if( bikeRoutePointList !=null ){
        bikeRoutePoints = [];
        bikeRoutePointList.forEach((element) {
          bikeRoutePoints.add(LatLng(element[0], element[1]));
        });
      }

      final footRoutePointList = data["footRoutePoints"]!=null? data["footRoutePoints"] as List : null;
      if( footRoutePointList !=null ){
        footRoutePoints = [];
        footRoutePointList.forEach((element) {
          footRoutePoints.add(LatLng(element[0], element[1]));
        });
      }
      return true;
    }else{
      return false;
    }
  }
}