import 'dart:convert';
import 'package:equatable/equatable.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';

import 'address_book_list_response.dart';



class RouteListByTrackingIdResponse extends Equatable{

  List<RouteListItem> items;
  int completed;
  int active;
  AddressBookItem currentStationContact;


  RouteListByTrackingIdResponse({this.items, this.completed, this.active});
  @override
  List<Object> get props => [items, completed, active];
  @override
  String toString() => 'Total records=${this.items.length}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];
      final routes = data["items"] as List;
      items = new List<RouteListItem>();
      routes.forEach((element) {
        RouteListItem routeListItem = new RouteListItem();
        routeListItem.fromMap(element);
        items.add(routeListItem);
      });

      completed = data["completed"];
      active = data["active"];
      currentStationContact = new AddressBookItem();
      currentStationContact.fromMap(data["currentStationContact"]);

      return true;
    }else{
      return false;
    }
  }
}

class RouteListItem extends Equatable{

  String pickupGuid;
  String pickupBucketGuid;
  String pickupBucketName;
  DateTime pickupDate;
  String deliverGuid;
  String deliverStationGuid;
  String deliverStationName;
  String deliverScheduleGuid;
  String deliverScheduleName;
  String deliverBucketGuid;
  String deliverBucketName;
  String deliverAddressBookGuid;
  String deliverAddressType;
  DateTime deliDate;
  String deliverymanGuid;
  String deliverymanName;
  String deliverymanId;
  int routeOrder;
  String wayType;
  String guid;
  String action;
  String status;
  double carDistance;
  List<LatLng> carRoutePoints;
  double bikeDistance;
  List<LatLng> bikeRoutePoints;
  double footDistance;
  List<LatLng> footRoutePoints;
  double latitude;
  double longitude;
  bool completed;
  DateTime createdOn;
  DateTime updatedOn;
  DateTime wayOn;
  DateTime doneOn;

  RouteListItem({this.pickupGuid, this.pickupBucketGuid,
  this.pickupBucketName, this.pickupDate, this.deliverGuid, this.deliverStationGuid,
  this.deliverStationName, this.deliverScheduleGuid, this.deliverScheduleName,
  this.deliverBucketGuid, this.deliverBucketName, this.deliverAddressBookGuid,
  this.deliverAddressType, this.deliDate, this.deliverymanGuid, this.deliverymanName,
  this.deliverymanId,this.routeOrder, this.wayType, this.guid, this.action,
  this.status, this.carDistance, this.carRoutePoints, this.bikeDistance,
  this.bikeRoutePoints, this.footDistance, this.footRoutePoints, this.latitude,
  this.longitude, this.completed, this.createdOn, this.updatedOn, this.wayOn,
  this.doneOn});
  @override
  List<Object> get props => [pickupGuid, pickupBucketGuid,
    pickupBucketName, pickupDate, deliverGuid, deliverStationGuid,
    deliverStationName, deliverScheduleGuid, deliverScheduleName,
    deliverBucketGuid, deliverBucketName, deliverAddressBookGuid,
    deliverAddressType, deliDate, deliverymanGuid, deliverymanName,
    deliverymanId, routeOrder, wayType, guid, action,
    status, carDistance, carRoutePoints, bikeDistance,
    bikeRoutePoints, footDistance, footRoutePoints, latitude,
    longitude, completed, createdOn, updatedOn, wayOn, doneOn];
  @override
  String toString() => this.status;

  fromMap( Map<String, dynamic> data ){
    pickupGuid = data["pickupGuid"];
    pickupBucketGuid = data["pickupBucketGuid"];
    pickupBucketName = data["pickupBucketName"];
    pickupDate = data["pickupDate"]!=null? DateTime.parse(data["pickupDate"]) : null;
    deliverGuid = data["deliverGuid"];
    deliverStationGuid = data["deliverStationGuid"];
    deliverStationName = data["deliverStationName"];
    deliverScheduleGuid = data["deliverScheduleGuid"];
    deliverScheduleName = data["deliverScheduleName"];
    deliverBucketGuid = data["deliverBucketGuid"];
    deliverBucketName = data["deliverBucketName"];
    deliverAddressBookGuid = data["deliverAddressBookGuid"];
    deliverAddressType = data["deliverAddressType"];
    deliDate = data["deliDate"]!=null? DateTime.parse(data["deliDate"]) : null;
    deliverymanGuid = data["deliverymanGuid"];
    deliverymanName = data["deliverymanName"];
    deliverymanId = data["deliverymanId"];
    routeOrder = data["routeOrder"];
    wayType = data["wayType"];
    guid = data["guid"];
    action = data["action"];
    status = data["status"];
    carDistance = data["carDistance"];
    // carRoutePoints = data["carRoutePoints"];
    bikeDistance = data["bikeDistance"];
    // bikeRoutePoints = data["bikeRoutePoints"];
    footDistance = data["footDistance"];
    // footRoutePoints = data["footRoutePoints"];
    latitude = data["latitude"];
    longitude = data["longitude"];
    completed = data["completed"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : null;
    wayOn = data["wayOn"]!=null? DateTime.parse(data["wayOn"]) : null;
    doneOn = data["doneOn"]!=null? DateTime.parse(data["doneOn"]) : null;

    final carRoutePointList = data["carRoutePoints"]!=null? data["carRoutePoints"] as List : null;
    if( carRoutePointList !=null ){
      carRoutePoints = [];
      carRoutePointList.forEach((element) {
        carRoutePoints.add(LatLng(element[0], element[1]));
      });
    }

    final bikeRoutePointList = data["bikeRoutePoints"]!=null? data["bikeRoutePoints"] as List : null;
    if( bikeRoutePointList !=null ){
      bikeRoutePoints = [];
      bikeRoutePointList.forEach((element) {
        bikeRoutePoints.add(LatLng(element[0], element[1]));
      });
    }

    final footRoutePointList = data["footRoutePoints"]!=null? data["footRoutePoints"] as List : null;
    if( footRoutePointList !=null ){
      footRoutePoints = [];
      footRoutePointList.forEach((element) {
        footRoutePoints.add(LatLng(element[0], element[1]));
      });
    }

  }
}