import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'hrm_base_client.dart';



class AddressBookService{

  String _url = "/v3/address_book/";
  UserService _userService = new UserService();


  Future<AddressBookListResponse> listAll(AddressBookRequest request ) async{
    AddressBookListResponse response;

    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list?all=true",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new AddressBookListResponse();
    if(!response.fromJson(httpResponse.data))
      response = null;
    return response;
  }

  Future<AddressBookListResponse> list(AddressBookRequest request ) async{
    AddressBookListResponse response;

    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new AddressBookListResponse();
    if(!response.fromJson(httpResponse.data))
      response = null;
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> update(Map<String, dynamic> request ) async{

    String accessToken = await _userService.getToken( );
    String jsonString = await putRequestWithAccessToken(
        "${_url}update",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<AddressBookListResponse> listPublic(AddressBookRequest request ) async{
    AddressBookListResponse response;
    String jsonString = await postRequest(
        "${_url}list_public",
        request.toMap() );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new AddressBookListResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<AddressBookListResponse> listPOIForPickup(AddressBookRequest request ) async{
    AddressBookListResponse response;
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list_pickup_poi",
        request.toMap(), accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new AddressBookListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<AddressBookListResponse> listPOIForDelivery(AddressBookRequest request ) async{
    AddressBookListResponse response;
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list_delivery_poi",
        request.toMap(), accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new AddressBookListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }
}