
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/bank_list_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/person_bank_list_response.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';


class BankService{
  String _uri = "/v1/bank/";

  Future<BankListResponse> listBank( int first, int max, String orderBy,bool asc,
      String keyword,[String branchGuid=''] ) async{
    BankListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String httpResponse = await getRequestWithAccessToken(
        "${_uri}list_banks?first=$first&max=$max&orderBy=$orderBy&asc=$asc&keyword=$keyword&branchGuid=$branchGuid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse);

    response = new BankListResponse();
    response.fromJson(httpResponse);
    return response;
  }

  Future<PersonBankListResponse> listPersonBank( int first, int max, String orderBy,bool asc,
      String keyword,[String personGuid=''] ) async{
    PersonBankListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String httpResponse = await getRequestWithAccessToken(
        "${_uri}list_by_people?first=$first&max=$max&orderBy=$orderBy&asc=$asc&keyword=$keyword&personGuid=$personGuid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse);

    response = new PersonBankListResponse();
    response.fromJson(httpResponse);
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> update(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    String httpResponse = await putRequestWithAccessToken(
        "${_uri}update",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse);

    Map<String, dynamic> resultData = jsonDecode(httpResponse);
    return resultData;
  }
}