import 'dart:convert';
import 'dart:io';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:http/http.dart' as http;
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:shared_preferences/shared_preferences.dart';


  HttpClient getHttpClient() {
  bool trustSelfSigned = true;
  HttpClient httpClient = new HttpClient()
  ..connectionTimeout = const Duration(seconds: 10)
  ..badCertificateCallback = ((X509Certificate cert, String host, int port) => trustSelfSigned);

  return httpClient;
  }

Future<String> getRequest( String uri  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var response = await http.get('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {"Content-Type": "application/json","Accept": "application/json"}
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> getRequestWithAccessToken( String uri, String accessToken  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var response = await http.get('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       }
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }
  return reply;
}

Future<String> postRequest( String uri, Map jsonMap ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.postUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    // request.headers.set('Authorization', 'Bearer ${accessToken}');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();
    if( Constant.ENABLE_LOGS)
      print(response.statusCode);
    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var body = json.encode(jsonMap);
  //   var response = await http.post('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //       "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json"
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<HTTPResponse> postRequestWithAccessToken( String uri, Map jsonMap, String accessToken ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  HTTPResponse httpResponse = new HTTPResponse();

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.postUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer $accessToken');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();
    if( Constant.ENABLE_LOGS)
      print(response.statusCode);

    httpResponse.statusCode = response.statusCode;
    if (response.statusCode == 200 || response.statusCode == 201)
      httpResponse.data = await response.transform(utf8.decoder).join();
    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   //encode Map to JSON
  //   var body = json.encode(jsonMap);
  //   var response = await http.post('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return httpResponse;
}

Future<String> putRequestWithAccessToken( String uri, Map jsonMap, String accessToken ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.putUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   //encode Map to JSON
  //   var body = json.encode(jsonMap);
  //   var response = await http.put('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> deleteRequestWithAccessToken( String uri, String accessToken  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.deleteUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var response = await http.delete('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       }
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }
  return reply;
}

Future<String> testConnection( String uri ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply = Constant.CONNECTION_REFUSED;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if ( response.statusCode == 200 || response.statusCode == 201 )
      reply = Constant.CONNECTION_OK;
    else if( response.statusCode == 401 )
      reply = Constant.CONNECTION_DENIED;
    else
      reply = Constant.CONNECTION_BAD_REQUEST;

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }
  return reply;
}

Future<String> getRequestHTML( String uri ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'text/html;charset=UTF-8');
    request.headers.set('Accept', 'text/html');
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  return reply;
}