import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/database/dao/notification_dao.dart';
import 'package:qq_delivery/delivery/database/entity/notification.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/notification_fetch_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'hrm_base_client.dart';


class NotificationService{

  UserService _userService = new UserService( );

  Future<NotificationFetchResponse> fetch(String deviceGUID,
      int lastCampaignId, int first, int max) async{
    NotificationFetchResponse fetchResponse;

    Map<String, dynamic> data = {
      "deviceGUID": deviceGUID,
      "lastCampaignId": lastCampaignId,
      "first": first,
      "max": max
    };

    String uri = "/v3/notification/list";
    HTTPResponse httpResponse = await postRequestWithAccessToken(uri, data, await _userService.getToken());

    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    fetchResponse = new NotificationFetchResponse();
    if( !fetchResponse.fromJson(httpResponse.data)){
      fetchResponse = null;
    }

    return fetchResponse;
  }

  Future<bool> fetchAndUpdate(  )async{
    bool updated = false;
    bool hasNext = true;
    NotificationDao notificationDao = new NotificationDao();
    await notificationDao.open();

    try{
      while( hasNext ){
        int lastId = await getCampaignLastId( );
        NotificationFetchResponse fetchResponse = await fetch( '', lastId, 0, 20);
        if( fetchResponse != null ){
          hasNext = fetchResponse.hasNext;
          for( int i=0; i < fetchResponse.notifications.length; i++){
            Notification notification = fetchResponse.notifications[i];
            await notificationDao.insert(notification);
            lastId = notification.campaignId;
          }
          if( !updated ){
            updated = fetchResponse.notifications.isNotEmpty;
          }
        }else{
          hasNext = false;
        }

        await setCampaignLastId(lastId);
      }
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    await notificationDao.close();

    return updated;
  }

}