import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/request/employee_clearance_list_request.dart';
import 'package:qq_delivery/delivery/model/request/person_list_request.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_clearance_list_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/os_balance_response.dart';
import 'package:qq_delivery/delivery/model/response/person_active_person_by_role_response.dart';
import 'package:qq_delivery/delivery/model/response/person_list_response.dart';
import 'hrm_base_client.dart';
import 'user_service.dart';



class PersonService{

  String _uri = "/v3/person/";

  Future<ActiveDeliverymanResponse> activeDeliveryman() async{
    ActiveDeliverymanResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String jsonString = await getRequestWithAccessToken(
        "${_uri}active_list", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new ActiveDeliverymanResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<PersonActiveByRoleResponse> activePersonByRole(String roleName) async{
    PersonActiveByRoleResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String jsonString = await getRequestWithAccessToken(
        "${_uri}active_person_role_min_list?roleName=$roleName", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new PersonActiveByRoleResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<PersonListResponse> listByRole(PersonListRequest request) async{
    PersonListResponse response;
    UserService _userService = UserService();

    String accessToken = await _userService.getToken();
    String jsonString = await getRequestWithAccessToken(
        "${_uri}list_by_role?${request.toString()}", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new PersonListResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request , [String username]) async{

    String jsonString = await postRequest("${_uri}create?username=${username??''}", request);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<Map<String, dynamic>> createByAdmin(Map<String, dynamic> request , [String token]) async{

    String accessToken = token;
    if( token == null ){
      UserService _userService = UserService();
      accessToken = await _userService.getToken( );
    }
    HTTPResponse httpResponse  = await postRequestWithAccessToken(
        "${_uri}create_by_admin",
        request,
        accessToken);

    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> updateKeyLatLon(Map<String, dynamic> request, [String token]) async{
    String accessToken = token;
    if( token == null ){
      UserService _userService = UserService();
      accessToken = await _userService.getToken( );
    }
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}update_key_lat_lon",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> generateVerificationCode(String phoneNumber) async{

    String jsonString = await getRequest(
        "${_uri}generate_verification_code/$phoneNumber");

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<Map<String, dynamic>> verifyAccount(String tokenNumber) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    String jsonString = await getRequestWithAccessToken(
        "${_uri}verify_account/$tokenNumber", accessToken);

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<Map<String, dynamic>> updateAvatar(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    String jsonString = await putRequestWithAccessToken(
        "${_uri}update",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<Map<String, dynamic>> generatePassword(String phoneNumber) async{

    String jsonString = await getRequest(
        "${_uri}generate_password/$phoneNumber");

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    return resultData;
  }

  Future<OSBalanceResponse> osBalance([String osGuid]) async{
    OSBalanceResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String jsonString = await getRequestWithAccessToken(
        "${_uri}os_balance?osGuid=${osGuid??''}", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);
    response = new OSBalanceResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<EmployeeClearanceListResponse> listEmployeeClearance(EmpployeeClearanceListRequest request ) async{
    EmployeeClearanceListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "/v1/employee_collected/list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new EmployeeClearanceListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }
}