
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_order_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_order_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/purchase_sales_summary_response.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';


class POSOrderService{
  String _uri = "/v1/posorder/";

  Future<POSOrderListResponse> list( Map<String, dynamic> request ) async{
    POSOrderListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}list",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new POSOrderListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<POSOrderDetailResponse> detail( String guid ) async{
    POSOrderDetailResponse response;

    Map<String,dynamic> request = {'guid': guid,};

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}singleorder",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new POSOrderDetailResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<Map<String, dynamic>> orderCancel(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}cancel",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<PurchaseSalesSummaryResponse> purchaseAndSalesSummary( Map<String, dynamic> request ) async{
    PurchaseSalesSummaryResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}purchase_sales_summary",
        request, accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new PurchaseSalesSummaryResponse();
    response.fromJson(httpResponse.data);
    return response;
  }
}