
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_product_list_response.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';


class POSProductService{
  String _uri = "/v1/posproduct/";

  Future<POSProductListResponse> list( Map<String, dynamic> request ) async{
    POSProductListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}productlist",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new POSProductListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}add",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<POSProductDetailItem> getByGuid( Map<String, dynamic> request ) async{
    POSProductDetailItem response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}getproduct",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new POSProductDetailItem();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<String> addPhotoAndCategory( Map<String, dynamic> request ) async{
    String reply;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}addcatimageproduct",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);
    reply = httpResponse.data;
    return reply;
  }

  Future<String> deleteProductImage( String imageGuid ) async{
    String reply;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    reply = await deleteRequestWithAccessToken(
        "${_uri}product_image_delete/$imageGuid",
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(reply);
    return reply;
  }
}