
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/purchase_sales_summary_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/v3/pos_order_detail_responsev3.dart';
import 'package:qq_delivery/delivery/model/response/pos/v3/pos_order_list_responsev3.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';


class POSOrderServiceV3{
  String _uri = "/v3/posorder/";

  Future<POSOrderListResponseV3> list( Map<String, dynamic> request ) async{
    POSOrderListResponseV3 response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}list",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new POSOrderListResponseV3();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<POSOrderDetailResponseV3> detail( String guid ) async{
    POSOrderDetailResponseV3 response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    String httpResponse = await getRequestWithAccessToken(
        "${_uri}detail/$guid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse);

    response = new POSOrderDetailResponseV3();
    response.fromJson(httpResponse);
    return response;
  }

  Future<Map<String, dynamic>> orderCancel(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}cancel",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<PurchaseSalesSummaryResponse> purchaseAndSalesSummary( Map<String, dynamic> request ) async{
    PurchaseSalesSummaryResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken();
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_uri}purchase_sales_summary",
        request, accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new PurchaseSalesSummaryResponse();
    response.fromJson(httpResponse.data);
    return response;
  }
}