import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/database/dao/notification_dao.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/token.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'firebase_service.dart';
import 'hrm_base_client.dart';
import 'person_service.dart';
import '/hrm/config/constant.dart' as config;
import '/hrm/service/user_service.dart' as hrmUserService;


class UserService{

  String _uri = "/v3/user/";

  Future<String> getToken() async{
    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    if( DateTime.now().isBefore(DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)))){
    // if(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
      accessToken = sharedPrefs.getString(Constant.CURRENT_USER_TOKEN_VALUE);
      if( accessToken != null )
        return accessToken;
    }else{
      accessToken = await refreshToken( );
    }
    return accessToken;
  }

  Future<String> refreshToken( ) async{

    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    String accessToken = sharedPrefs.getString(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE);

    Map<String, dynamic> data = {
      'accessToken': accessToken
    };

    String response = await postRequest( "${_uri}refresh_token", data);
    Token token = new Token( );
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(Constant.CURRENT_USER_TOKEN_VALUE, accessToken);
      sharedPrefs.setString(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE, token.refreshToken);
      sharedPrefs.setInt(Constant.CURRENT_USER_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));
    }else{
      accessToken = null;
    }

    if(Constant.ENABLE_LOGS)
      print('************************User token value=${accessToken}');
    if(Constant.ENABLE_LOGS)
      print('************User token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED))}');

    return accessToken;
  }

  Future<String> login(String username, String password, bool retry) async{

    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    if( sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED) != null && !retry){
      if(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
        accessToken = sharedPrefs.getString(Constant.CURRENT_USER_TOKEN_VALUE);
        if( accessToken != null)
          return accessToken;
      }
    }

    Map<String, dynamic> data = {
      'username': username,
      'password': password
    };

    String response = await postRequest(_uri, data);
    Token token = new Token( );
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(Constant.CURRENT_USER_TOKEN_VALUE, accessToken);
      sharedPrefs.setString(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE, token.refreshToken);
      sharedPrefs.setInt(Constant.CURRENT_USER_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));

      await getUserInformation( );
      await authority( accessToken );

      if(Constant.ENABLE_LOGS)
        print('************************User token value=${accessToken}');
      if(Constant.ENABLE_LOGS)
        print('************User token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED))}');

    }
    return accessToken;
  }

  Future<User> getUserInformation( [bool save=true] ) async{

    String userAccessToken = await getToken();
    if ( userAccessToken != null ){
      String userInformation = await getRequestWithAccessToken("${_uri}brief", userAccessToken);
      User user = new User();
      user.fromJson(userInformation, save);

      return user;
    }
    return null;
  }

  Future<AuthorityResponse> authority(String userAccessToken) async{

    String response = await getRequestWithAccessToken("${_uri}role_check", userAccessToken);
    if( Constant.ENABLE_LOGS)
      print(response);
    AuthorityResponse authorityResponse = new AuthorityResponse();
    await authorityResponse.fromJson(response, true);

    return authorityResponse;
  }

  Future<void> logout() async{

    try{
      PersonService personService = new PersonService();
      await personService.updateKeyLatLon(
          {
            'firebaseKey': null,
            'latitude': null,
            'longitude': null,
            'lastFirebaseKey': await getFirebaseToken( ),
            'logout': true
          }, await getToken()
      );

      try{
        NotificationDao notificationDao = new NotificationDao();
        await notificationDao.open();
        await notificationDao.delete();
      }catch(ex){}

    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    try{
      Map<String, dynamic> data = {
        'accessToken': await getToken( )
      };
      await postRequest( "${_uri}revoke_token", data);
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }

    await setSettingsTempLogin(false);
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA);
    await sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
    await sharedPrefs.remove(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE);
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
    await sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
    await sharedPrefs.remove(Constant.DEVICE_GUID);
    await sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
    await sharedPrefs.remove(Constant.AUTHENTICATION_2FA);

    try{
      if( Constant.ENABLE_HR_MODULE ){
        hrmUserService.UserService userService = new hrmUserService.UserService();
        await userService.logout();
      }
    }catch(e){}

  }

  Future<bool> userUpdate( Map<String, dynamic> userData, String accessToken ) async{
    bool result = false;
    if( Constant.ENABLE_LOGS)
      print('User data: ${json.encode(userData)}');

    try{
      String uri = "/v1/recogser/user/update";
      HTTPResponse httpResponse = await postRequestWithAccessToken( uri, userData, accessToken );
      if( Constant.ENABLE_LOGS)
        print( httpResponse.data );

      Map<String, dynamic> registerResponse = jsonDecode(httpResponse.data);
      result = registerResponse['success'];

    }catch(e){}
    return result;
  }

  Future<Map<String, dynamic>> changePassword( Map<String, dynamic> userData ) async{
    Map<String, dynamic> response;

    try{
      String accessToken = await getToken( );
      String jsonString = await putRequestWithAccessToken( '${_uri}change_password', userData, accessToken );
      if( Constant.ENABLE_LOGS)
        print( jsonString );

      response = jsonDecode(jsonString);

    }catch(e){}
    return response;
  }

  Future<Map<String, dynamic>> changeStatus( Map<String, dynamic> userData ) async{
    Map<String, dynamic> response;
    try{
      String accessToken = await getToken();
      String jsonString = await putRequestWithAccessToken( '${_uri}change_status', userData, accessToken );
      if( Constant.ENABLE_LOGS)
        print( jsonString );

      response = jsonDecode(jsonString);

    }catch(e){}
    return response;
  }

  Future<bool> isAvailable( String username )async{
    bool response = false;

    try{
      String uri = "${_uri}available/$username";
      String jsonString = await getRequest( uri );
      if( Constant.ENABLE_LOGS)
        print(jsonString);
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      response = resultData["success"];
    }catch(e){}

    return response;
  }


  Future<String> hrAuthorization( ) async{

    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    String currentToken = await getToken();
    String response = await getRequestWithAccessToken( '${_uri}hrm_authorization', currentToken );
    Token token = new Token( );
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(config.Constant.CURRENT_USER_TOKEN_VALUE, accessToken);
      sharedPrefs.setString(config.Constant.CURRENT_USER_REFRESH_TOKEN_VALUE, token.refreshToken);
      sharedPrefs.setInt(config.Constant.CURRENT_USER_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));

      if(config.Constant.ENABLE_LOGS)
        print('************************ HR User token value=${accessToken}');
      if(config.Constant.ENABLE_LOGS)
        print('************ HR User token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(config.Constant.CURRENT_USER_TOKEN_EXPIRED))}');

    }
    return accessToken;
  }

}