import 'dart:convert';

import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/chat_group_member_list_response.dart';
import 'package:qq_delivery/delivery/model/response/chat_latest_list_response.dart';
import 'package:qq_delivery/delivery/model/response/chat_search_list_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import '../hrm_base_client.dart';
import '../user_service.dart';


class ChatService{
  String _url = "/v3/chatsession/";

  Future<ChatSearchListResponse> searchPersonOrGroup( String type, String keyword, int first, int max) async{
    ChatSearchListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    String jsonString = await getRequestWithAccessToken(
        "${_url}search?type=$type&keyword=$keyword&first=$first&max=$max", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new ChatSearchListResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<ChatLatestListResponse> latestList(Map<String, dynamic> request ) async{
    ChatLatestListResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}latestlist?first=${request['first']}&max=${request['max']}",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new ChatLatestListResponse();
    response.fromJson(httpResponse.data);
    return response;

  }

  Future<ChatLatestListResponse> individualRecord(Map<String, dynamic> request ) async{
    ChatLatestListResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}individualrecord",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new ChatLatestListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<ChatLatestListResponse> groupRecord(Map<String, dynamic> request ) async{
    ChatLatestListResponse response;
    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}grouprecord",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new ChatLatestListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<Map<String, dynamic>> sendMessage(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}sendmessage",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> addMember(Map<String, dynamic> request ) async{

    UserService _userService = UserService( );
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}addmember",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<ChatGroupMemberListResponse> groupMemberList(Map<String, dynamic> request, int first, int max) async{
    ChatGroupMemberListResponse response;

    UserService _userService = UserService();
    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}groupmember?first=$first&max=$max",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new ChatGroupMemberListResponse();
    response.fromJson( httpResponse.data );
    return response;
  }
}