import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_completed_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_count_by_status_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/active_way_count_by_town_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_count_by_status_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/overdue_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';

import '../hrm_base_client.dart';
import '../user_service.dart';


class DeliveryContentServiceV2{

  String _url = "/v3/deliver/";
  UserService _userService = new UserService();

  Future<DeliveryCountByStatusResponse> countByStatus(String accessToken,
      DeliveryCountByStatusRequest request ) async{
    DeliveryCountByStatusResponse statusResponse;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}count_by_status",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    statusResponse = new DeliveryCountByStatusResponse();
    statusResponse.statusCode = httpResponse.statusCode;
    statusResponse.fromJson(httpResponse.data);
    return statusResponse;
  }

  Future<DeliveryListResponse> list(String accessToken,
      DeliveryListRequest request ) async{
    DeliveryListResponse response;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new DeliveryListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }
  Future<DeliveryListResponse> invoices(String accessToken,
      DeliveryListRequest request ) async{
    DeliveryListResponse response;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}invoice_list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new DeliveryListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<EmployeeListForDeliveryResponse> availableAllEmployees([String accessToken]) async{
    EmployeeListForDeliveryResponse response;

    accessToken = await _userService.getToken();
    String jsonString = await getRequestWithAccessToken(
        "${_url}available_all_employees", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new EmployeeListForDeliveryResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<EmployeeListForDeliveryResponse> availableEmployeesByZone(String contentGuid, String accessToken) async{
    EmployeeListForDeliveryResponse response;

    String jsonString = await getRequestWithAccessToken(
        "${_url}available_employees/$contentGuid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new EmployeeListForDeliveryResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<DeliveryDetailResponse> detail( String trackingId ) async{
    DeliveryDetailResponse response;
    String jsonString = await getRequest( "${_url}detail/$trackingId" );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new DeliveryDetailResponse();
    if(response.fromJson(jsonString))
      return response;
    else return null;
  }

  Future<Map<String, dynamic>> process(String accessToken,
      DeliveryAcceptRequest request, String uri ) async{
    String guid;

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> assign(String accessToken,
      DeliveryAssignRequest request, String uri ) async{

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }else{
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> complete(String accessToken,
      DeliveryCompletedRequest request, String uri ) async{

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> multipleAction(String accessToken,
      Map<String, dynamic> request ) async{

    String jsonString = await putRequestWithAccessToken(
        "${_url}multiple_action",
        request, accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }else{
    //   guid = resultData["data"];
    // }
    return resultData;
  }


  Future<Map<String, dynamic>> contactUpdate(String accessToken, DeliveryContactItem request ) async{

    String jsonString = await putRequestWithAccessToken(
        "${_url}update_contact",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }

  Future<Map<String, dynamic>> update(Map<String, dynamic> request ) async{
    Map<String, dynamic> response;
    String accessToken = await _userService.getToken( );
    String jsonString = await putRequestWithAccessToken(
        "${_url}update",
        request,
        accessToken );

    if( jsonString != null ){
      response = jsonDecode(jsonString);
    }

    return response;
  }

  Future<Map<String, dynamic>> availableDeliveryDates( ) async{
    Map<String, dynamic> response;

    String accessToken = await _userService.getToken( );
    String jsonString = await getRequestWithAccessToken(
        "${_url}available_delivery_dates", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString != null ){
      response = jsonDecode(jsonString);
    }

    return response;
  }


  Future<OverdueResponse> overdueCount( [String personGuid]) async{
    OverdueResponse response = new OverdueResponse(overdueDeliver: 0, overduePickup: 0);

    String accessToken = await _userService.getToken( );
    String jsonString = await getRequestWithAccessToken(
        "${_url}overdue_count?$personGuid", accessToken );

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString != null ){
      response.fromJson(jsonString);
    }
    return response;
  }

  Future<ActiveWayCountByTownResponse> activeWayCountByTown(int first, int max, String orderBy, bool asc, String accessToken) async{
    ActiveWayCountByTownResponse response;

    String jsonString = await getRequestWithAccessToken(
        "${_url}active_count_by_town?first=$first&max=$max&orderBy=$orderBy&asc=$asc", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new ActiveWayCountByTownResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<String> wayBillToPrint( String trackingId, [String size="4x6"] ) async{
    String htmlString = await getRequestHTML( "${_url}way_bill_print/$trackingId/$size" );
    if( Constant.ENABLE_LOGS)
      print(htmlString);
    return htmlString;
  }

}