


import 'dart:convert';

import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_count_by_status_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/delivery_count_by_status_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/pickup_detail_response.dart';

import '../hrm_base_client.dart';
import '../user_service.dart';

class PickupServiceV2{

  String _url = "/v3/pickup/";
  UserService _userService = new UserService();


  Future<DeliveryCountByStatusResponse> countByStatus(String accessToken,
      DeliveryCountByStatusRequest request ) async{
    DeliveryCountByStatusResponse statusResponse;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}count_by_status",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    statusResponse = new DeliveryCountByStatusResponse();
    statusResponse.statusCode = httpResponse.statusCode;
    statusResponse.fromJson(httpResponse.data);
    return statusResponse;
  }

  Future<DeliveryListResponse> list(String accessToken,
      DeliveryListRequest request ) async{
    DeliveryListResponse response;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new DeliveryListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

Future<DeliveryListResponse> invoice(String accessToken,
    DeliveryListRequest request, bool invoice, int refunded) async{
DeliveryListResponse response;

HTTPResponse httpResponse = await postRequestWithAccessToken(
"${_url}list?invoice=$invoice&refunded=$refunded",
request.toMap(),
accessToken );
if( Constant.ENABLE_LOGS)
print(httpResponse.data);

response = new DeliveryListResponse();
response.fromJson(httpResponse.data);
return response;
}

  Future<PickupDetailResponse> detail( String trackingId ) async{
    PickupDetailResponse response;
    String jsonString = await getRequest( "${_url}detail/$trackingId" );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new PickupDetailResponse();
    response.fromJson(jsonString);
    return response;

    // if( response.fromJson(jsonString)  )
    //   return response;
    // else return null;
  }

  Future<Map<String, dynamic>> create(Map<String, dynamic> request ) async{

    String accessToken = await _userService.getToken( );
    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}create",
        request,
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    Map<String, dynamic> resultData = jsonDecode(httpResponse.data);
    return resultData;
  }
  Future<EmployeeListForDeliveryResponse> availableEmployeesByZone(String contentGuid, String accessToken) async{
    EmployeeListForDeliveryResponse response;

    String jsonString = await getRequestWithAccessToken(
        "${_url}available_employees/$contentGuid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new EmployeeListForDeliveryResponse();
    response.fromJson(jsonString);
    return response;
  }


  Future<Map<String, dynamic>> process(String accessToken,
      DeliveryAcceptRequest request, String uri ) async{
    String guid;

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> assign(String accessToken,
      DeliveryAssignRequest request, String uri ) async{

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }else{
    //   guid = resultData["data"];
    // }
    return resultData;
  }
}