


import 'dart:convert';

import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_completed_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_count_by_status_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/delivery_count_by_status_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/http_response.dart';
import 'package:qq_delivery/delivery/model/response/route_list_by_tracking_id_response.dart';

import '../hrm_base_client.dart';
import '../user_service.dart';

class WarehouseStationRouteService{

  String _url = "/v3/warehouse_route/";
  UserService _userService = new UserService();

  Future<DeliveryCountByStatusResponse> inboundCountByStatus(String accessToken,
      DeliveryCountByStatusRequest request ) async{
    DeliveryCountByStatusResponse statusResponse;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}inbound_count_by_status",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    statusResponse = new DeliveryCountByStatusResponse();
    statusResponse.statusCode = httpResponse.statusCode;
    statusResponse.fromJson(httpResponse.data);
    return statusResponse;
  }

  Future<DeliveryCountByStatusResponse> outboundCountByStatus(String accessToken,
      DeliveryCountByStatusRequest request ) async{
    DeliveryCountByStatusResponse statusResponse;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}outbound_count_by_status",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    statusResponse = new DeliveryCountByStatusResponse();
    statusResponse.statusCode = httpResponse.statusCode;
    statusResponse.fromJson(httpResponse.data);
    return statusResponse;
  }

  Future<DeliveryCountByStatusResponse> countByStatusAndDeliveryman(String accessToken,
      DeliveryCountByStatusRequest request ) async{
    DeliveryCountByStatusResponse statusResponse;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}count_by_status_and_employee",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    statusResponse = new DeliveryCountByStatusResponse();
    statusResponse.statusCode = httpResponse.statusCode;
    statusResponse.fromJson(httpResponse.data);
    return statusResponse;
  }

  Future<DeliveryListResponse> list(String accessToken,
      DeliveryListRequest request ) async{
    DeliveryListResponse response;

    HTTPResponse httpResponse = await postRequestWithAccessToken(
        "${_url}list",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(httpResponse.data);

    response = new DeliveryListResponse();
    response.fromJson(httpResponse.data);
    return response;
  }

  Future<DeliveryListItem> detail( String guid, accessToken ) async{
    DeliveryListItem response;
    String jsonString = await getRequestWithAccessToken( "${_url}detail/$guid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new DeliveryListItem();
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    if ( resultData["success"] ) {
      Map<String, dynamic> data = resultData["data"];
      response.fromMap(data);
    }
    return response;
  }

  Future<EmployeeListForDeliveryResponse> availableEmployeesByZone(String contentGuid, String accessToken) async{
    EmployeeListForDeliveryResponse response;

    String jsonString = await getRequestWithAccessToken(
        "${_url}available_employees/$contentGuid", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new EmployeeListForDeliveryResponse();
    response.fromJson(jsonString);
    return response;
  }

  Future<Map<String, dynamic>> process(String accessToken,
      DeliveryAcceptRequest request, String uri ) async{
    String guid;

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> assign(String accessToken,
      DeliveryAssignRequest request, String uri ) async{

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }else{
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> complete(String accessToken,
      DeliveryCompletedRequest request, String uri ) async{

    String jsonString = await putRequestWithAccessToken(
        "$_url$uri",
        request.toMap(),
        accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    if( jsonString == null) return null;
    Map<String, dynamic> resultData = jsonDecode(jsonString);
    // if( resultData["success"]){
    //   guid = resultData["data"];
    // }
    return resultData;
  }

  Future<Map<String, dynamic>> routeProcessByQR( String trackingId, accessToken, [bool auto=false] ) async{
    Map<String, dynamic> resultData;
    String jsonString = await getRequestWithAccessToken( "${_url}route_process_by_qr/$trackingId?$auto", accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);
    resultData = jsonDecode(jsonString);
    return resultData;
  }
  Future<RouteListByTrackingIdResponse> routeListByDelivery( String trackingId, [bool admin=false] ) async{
    RouteListByTrackingIdResponse response;
    String jsonString = await getRequest( "${_url}routes_by_delivery_tracking_id/$trackingId?admin=$admin" );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    response = new RouteListByTrackingIdResponse();
    response.fromJson(jsonString);
    return response;
  }

}