
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:location/location.dart';


class MapSelection extends StatefulWidget {
  MapSelection({@required this.startLocation, @required this.title, this.selected});

  final LatLng startLocation;
  final title;
  final Function(LatLng) selected;

  @override
  _MapSelectionState createState() => _MapSelectionState();
}

class _MapSelectionState extends State<MapSelection> {

  MapType _mapType = MapType.normal;
  bool _rotateGesturesEnabled = true;
  bool _scrollGesturesEnabled = true;
  bool _tiltGesturesEnabled = true;
  bool _zoomGesturesEnabled = true;
  bool _indoorViewEnabled = true;
  bool _myLocationEnabled = true;
  bool _myLocationButtonEnabled = true;
  bool _isMapCreated = false;
  bool _isMoving = false;
  bool _compassEnabled = true;
  bool _mapToolbarEnabled = true;

  GoogleMapController _controller;
  CameraPosition _kGooglePlex;
  Map<MarkerId, Marker> markers = <MarkerId, Marker>{};
  MarkerId selectedMarker;
  BitmapDescriptor _markerIcon;
  LatLng _selectedPosition;


  @override
  void initState() {
    super.initState();
    _kGooglePlex = CameraPosition(
      target: widget.startLocation==null ? LatLng(16.813538,96.171205) : LatLng(widget.startLocation.latitude, widget.startLocation.longitude),
      zoom: 14.4746,
    );
    _selectedPosition = widget.startLocation==null ? LatLng(16.813538,96.171205) : LatLng(widget.startLocation.latitude, widget.startLocation.longitude);
  }

  Future<void> _createMarkerImageFromAsset(BuildContext context) async {
    if (_markerIcon == null) {
      final ImageConfiguration imageConfiguration =
      createLocalImageConfiguration(context);
      BitmapDescriptor.fromAssetImage(
          imageConfiguration, 'assets/images/map_marker.png')
          .then(_updateBitmap);
    }
  }
  void _updateBitmap(BitmapDescriptor bitmap) {
    setState(() {
      _markerIcon = bitmap;
    });
  }
  void _add( ) {

      final MarkerId markerId = MarkerId('1');
      final Marker marker = Marker(
        markerId: markerId,
        position: _selectedPosition,
        //infoWindow: InfoWindow(title: employee['username'], snippet: employee['phone']),
        //onTap: () { _onMarkerTapped(markerId); },
        //icon: _markerIcon,//BitmapDescriptor.fromBytes(base64Decode(employee['coverphoto'].substring(22)))
      );

      setState(() {
        markers[markerId] = marker;
      });


  }

  void _onMapCreated(GoogleMapController controller) {
    this._controller = controller;
    _add();
  }

  void _onMarkerTapped(MarkerId markerId) {
    final Marker tappedMarker = markers[markerId];
    if (tappedMarker != null) {
      setState(() {
        if (markers.containsKey(selectedMarker)) {
          final Marker resetOld = markers[selectedMarker]
              .copyWith(iconParam: BitmapDescriptor.defaultMarker);
          markers[selectedMarker] = resetOld;
        }
        selectedMarker = markerId;
        final Marker newMarker = tappedMarker.copyWith(
          iconParam: BitmapDescriptor.defaultMarkerWithHue(
            BitmapDescriptor.hueGreen,
          ),
        );
        markers[markerId] = newMarker;
      });
    }
  }


  @override
  void dispose() {
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {
    _createMarkerImageFromAsset(context);
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
          title: Text(widget.title),
        ),
        body: Center(
          child:Container(
            //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
            child: GoogleMap(
              mapType: _mapType,
              initialCameraPosition: _kGooglePlex,
              onMapCreated: _onMapCreated,
              compassEnabled: _compassEnabled,
              mapToolbarEnabled: _mapToolbarEnabled,
              rotateGesturesEnabled: _rotateGesturesEnabled,
              scrollGesturesEnabled: _scrollGesturesEnabled,
              tiltGesturesEnabled: _tiltGesturesEnabled,
              zoomGesturesEnabled: _zoomGesturesEnabled,
              indoorViewEnabled: _indoorViewEnabled,
              myLocationEnabled: _myLocationEnabled,
              myLocationButtonEnabled: _myLocationButtonEnabled,
              markers: Set<Marker>.of(markers.values),
              onTap: (LatLng pos) {
              setState(() {
                _selectedPosition = pos;
                widget.selected(pos);
                _add( );
                //print('${pos.latitude}, ${pos.longitude}');
              });
            },
            ),
          )
        ),
      )
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}