import 'dart:async';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/response/person_active_person_by_role_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/state_service.dart';
import 'package:qq_delivery/delivery/service/street_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';



class AddressBookCreate extends StatefulWidget {
  final String title;
  final AddressBookItem item;
  final Function onChange;

  AddressBookCreate({Key key, @required this.title, @required this.item,
  this.onChange}) : super(key: key);

  @override
  _AddressBookCreateState createState() => _AddressBookCreateState();
}

class _AddressBookCreateState extends State<AddressBookCreate> {

  AddressBookService _addressBookService = new AddressBookService();
  SharedPreferences _sharedPrefs;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  TownshipService _townshipService = new TownshipService();
  StateService _stateService = new StateService();
  StreetService _streetService = new StreetService();
  GeneralAddressResponse _addressResponseState;
  GeneralAddressItem _addressItemState;
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;
  GeneralAddressResponse _addressResponseStreet;
  GeneralAddressItem _addressItemStreet;

  PersonService _personService = new PersonService();
  PersonActiveByRoleResponse _activeByRoleResponseOS;
  GeneralPersonItem _personItemOS;

  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerState = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final TextEditingController _controllerStreet = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerPrimaryPhone = new TextEditingController();
  final TextEditingController _controllerOtherPhones = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerComment = new TextEditingController();
  String _lang = Constant.LANGUAGE_CODE_MY;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();

    if( _authorityResponse.authorities!=null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ")
        && widget.item.guid == null) {
      _activeByRoleResponseOS = await _personService.activePersonByRole( "OS" );
    }else{
      widget.item.personGuid = _currentUser.guid;
    }

    _addressResponseState = await _stateService.list();

    if( widget.item.guid != null ){
      // select on state list
      if( _addressResponseState!=null && _addressResponseState.items!=null){
        _addressResponseState.items.forEach((element) {
          if( widget.item.stateGuid == element.guid ){
            _controllerState.text = element.name;
            _addressItemState = element;
            return;
          }
        });

        // select on township
        if( _addressItemState != null){
          await _loadTownships( widget.item.townshipGuid );
        }

        // select on street
        if( _addressItemTownship != null){
          await _loadStreets( widget.item.street );
        }
      }
    }
    setState(() {_processing = false;});
  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && widget.item.guid == null? _oSSelection(context): Container(),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblName),
                ),
                validator: (val) => val.isEmpty ? 'Name required!' : null,
                onSaved: (val) => widget.item.name = val,
                initialValue: widget.item.name,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '67, 2nd Floor',
                  labelText: tr(LocaleKeys.lblHouseNo),
                ),
                validator: (val) => val.isEmpty ? 'House No. (or) Unit, floor required!' : null,
                keyboardType: TextInputType.text,
                onSaved: (val) => widget.item.address = val,
                initialValue: widget.item.address,
              ),
            ),
            _stateSelection( context ),
            _townshipSelection( context ),
            _streetSelection(context),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                decoration: InputDecoration(
                  hintText: '09xxxxxxxxx',
                  labelText: tr(LocaleKeys.lblMobilePhone),
                ),
                validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                keyboardType: TextInputType.phone,
                onSaved: (val) => widget.item.primaryPhone = val,
                initialValue: widget.item.primaryPhone,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: "Use \",\" for multiple numbers",
                  labelText: tr(LocaleKeys.lblOtherPhones),
                ),
                // validator: (val) => val.isEmpty ? 'Other phone required!' : null,
                onSaved: (val) => widget.item.otherPhones = val,
                initialValue: widget.item.otherPhones,
                keyboardType: TextInputType.phone,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: 'name@domain.com',
                  labelText: tr(LocaleKeys.lblEmailAddress),
                ),
                validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                keyboardType: TextInputType.emailAddress,
                onSaved: (val) => widget.item.email = val,
                initialValue: widget.item.email,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblRemark),
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                onSaved: (val) => widget.item.comment = val,
                initialValue: widget.item.comment,
                keyboardType: TextInputType.text,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                          // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        widget.item.guid == null?'Save' :'Update',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                          // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }

  Widget _oSSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<GeneralPersonItem>(
          selectedItem: _personItemOS,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (GeneralPersonItem i, GeneralPersonItem s) => i?.personGuid==s?.personGuid,
          label: tr(LocaleKeys.lblOS),
          onFind: (String filter) => _oSSuggestion(filter),
          onChanged: (GeneralPersonItem data) {
            _personItemOS = data;
          },
          dropdownBuilder: (BuildContext context, GeneralPersonItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle, color: Colors.blueGrey[600]),
                  title: Text(
                    tr(LocaleKeys.lblSelectOS),
                    style: TextStyle(color: Colors.black),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0),
                  title: Text(
                    item.name,
                    style: TextStyle(color: Colors.black),
                  ),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                    width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, GeneralPersonItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text('${item.name}'),
                  // subtitle: Text('${item.townshipName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<GeneralPersonItem>> _oSSuggestion(String query) async{
    List<GeneralPersonItem> matches = List();
    if( _activeByRoleResponseOS==null || _activeByRoleResponseOS.items == null)
      return matches;
    matches.addAll(_activeByRoleResponseOS.items);

    matches.retainWhere((s) {
      GeneralPersonItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _stateSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 0),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type state/division name',
              labelText: tr(LocaleKeys.lblState),
            ),
            controller: _controllerState,
          ),
          suggestionsCallback: (pattern) async {
            return await _stateSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerState.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemState = suggestion;
            _loadTownships();
          },
          validator: (val) => val.isEmpty ? 'State/Division required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _stateSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseState==null || _addressResponseState.items == null)
      return matches;
    matches.addAll(_addressResponseState.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadTownships([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemState != null) {
      _addressResponseTownship = await _townshipService.list(_addressItemState.guid);
      if( _addressResponseTownship !=null && defaultSelected!=null){
        _addressResponseTownship.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemTownship = element;
            _controllerTownship.text = _addressItemTownship.name;
            return;
          }
        });
      }
    }else{
      _addressResponseTownship.items = [];
      _addressItemTownship = null;
      _controllerTownship.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type township name',
              labelText: tr(LocaleKeys.lblTownship),
            ),
            controller: _controllerTownship,
            enabled: _addressItemState!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemTownship = suggestion;
            _loadStreets();
          },
          validator: (val) => val.isEmpty ? 'Township (or) Town required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadStreets([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemTownship != null) {
      _addressResponseStreet = await _streetService.list(_addressItemTownship.guid);
      if( _addressResponseStreet !=null && defaultSelected!=null){
        _addressResponseStreet.items.forEach((element) {
          if( element.name == defaultSelected || element.nameMM == defaultSelected){
            _addressItemStreet = element;
            _controllerStreet.text = _addressItemStreet.name;
            return;
          }
        });
      }
      if( defaultSelected!=null )
      _controllerStreet.text = defaultSelected;
    }else{
      _addressResponseStreet.items = [];
      _addressItemStreet = null;
      _controllerStreet.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _streetSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type street name',
              labelText: tr(LocaleKeys.lblStreet),
            ),
            controller: _controllerStreet,
            enabled: _addressItemTownship!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _streetSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerStreet.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemStreet = suggestion;
          },
          noItemsFoundBuilder: (BuildContext context) => new Container( height: 0, ),
          validator: (val) => val.isEmpty ? 'Street name required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _streetSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseStreet==null || _addressResponseStreet.items == null)
      return matches;
    matches.addAll(_addressResponseStreet.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    if( _activeByRoleResponseOS != null && _personItemOS==null && widget.item.guid == null){
      showMessage('Please select the OS');
      return;
    }

    if( _controllerState.text.trim().isEmpty){
      showMessage('State required!');
      return;
    }

    if( _controllerTownship.text.trim().isEmpty){
      showMessage('Township required!');
      return;
    }

    if( _controllerStreet.text.trim().isEmpty){
      showMessage('Street required!');
      return;
    }

    setState(() {_processing = true;});

    Map<String,dynamic> addressBookCreateItem;
    // new record
    if( widget.item.guid == null ){
      addressBookCreateItem={
        'ownerGuid':_personItemOS==null? _currentUser.guid : _personItemOS.personGuid,
        'name': widget.item.name,
        'address': widget.item.address,
        'street': _controllerStreet.text,
        'streetGuid':_addressItemStreet!=null? _addressItemStreet.guid : null,
        'township': _controllerTownship.text,
        'townshipGuid': _addressItemTownship!=null? _addressItemTownship.guid : null,
        'primaryPhone': widget.item.primaryPhone,
        'otherPhones': widget.item.otherPhones,
        'email': widget.item.email,
        'comment': widget.item.comment,
        'latitude': _addressItemStreet!=null? _addressItemStreet.latitude : null,
        'longitude': _addressItemStreet!=null? _addressItemStreet.longitude : null,
        'addressType': widget.item.addressType,
        'primaryContact': false
      };
    }else{
      // update record
      addressBookCreateItem={
        'guid': widget.item.guid,
        'name': widget.item.name,
        'address': widget.item.address,
        'street': _controllerStreet.text,
        'streetGuid':_addressItemStreet!=null? _addressItemStreet.guid : null,
        'township': _controllerTownship.text,
        'townshipGuid': _addressItemTownship!=null? _addressItemTownship.guid : null,
        'primaryPhone': widget.item.primaryPhone,
        'otherPhones': widget.item.otherPhones,
        'email': widget.item.email,
        'comment': widget.item.comment,
        'latitude': _addressItemStreet!=null? _addressItemStreet.latitude : null,
        'longitude': _addressItemStreet!=null? _addressItemStreet.longitude : null,
        'primaryContact': widget.item.primaryContact
      };
    }

    Map<String, dynamic> result;
    result = widget.item.guid == null?
      await _addressBookService.create(addressBookCreateItem)
    : await _addressBookService.update(addressBookCreateItem);

    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange();
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }


    setState(() {_processing = false;});
  }


  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{8,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}
