import 'dart:async';

import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'address_book_create.dart';


class AddressBookList extends StatefulWidget {
  AddressBookList({Key key}) : super(key: key);
  @override
  _AddressBookListState createState() => _AddressBookListState();
}

class _AddressBookListState extends State<AddressBookList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = 'OS';
  bool _tapChange = false;
  String _osAddressLabel = "OS Address";


  @override
  void initState() {
    super.initState();
    _getLoadData();
  }

  void _getLoadData() async{
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      setState(() {
        if( _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("EMPLOYEE_READ")){
          _osAddressLabel = "My Address";
          _title= 'My Addresses';
        }else{
          _title= tr(LocaleKeys.lblOS);
          _osAddressLabel = tr(LocaleKeys.lblOS);
        }
      });
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.supervised_user_circle_sharp),
                label: '$_osAddressLabel',
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.people_alt_sharp),
                label: tr(LocaleKeys.lblCustomer),
              ),
            ],
            currentIndex: _selectedIndex,
            // selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Colors.white,
            onTap: _onItemTapped,
          ),
          floatingActionButton: FloatingActionButton.extended(
            onPressed: (){
              _addNew();
            },
            label: Text(
              'Add New',
              style: TextStyle(color: Colors.white),
            ),
            icon: Icon(Icons.add, color: Colors.white),
            backgroundColor: Theme.of(context).primaryColor,
          ),
        )
    );
  }

  Widget _createBody(){

    if( _selectedIndex == 0 && !_tapChange){
      return AddressBookLoader(
        addressType: 'OS',
        currentUser: _currentUser,
        authorityResponse: _authorityResponse,
        userService: _userService,
        sharedPrefs: _sharedPrefs,
      );
    }else if( _selectedIndex == 1 && !_tapChange){
      return AddressBookLoader(
        addressType: 'Customer',
        currentUser: _currentUser,
        authorityResponse: _authorityResponse,
        userService: _userService,
        sharedPrefs: _sharedPrefs,
      );
    }else{
      return new Container();
    }
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        if( _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ")){
          _title= Constant.ADDRESS_TYPE_OS;
        }else{
          _title= 'My Addresses';
        }
      }else if( _selectedIndex == 1){
        _title= Constant.ADDRESS_TYPE_CUSTOMER;
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  void _addNew(){
    AddressBookItem item = new AddressBookItem(
        addressType: _selectedIndex == 0? Constant.ADDRESS_TYPE_OS: Constant.ADDRESS_TYPE_CUSTOMER,
        name: '',
        address: '',
        street: '',
        townshipName: '',
        primaryPhone: '',
        otherPhones: '',
        email: '',
        comment: '',
    );
    String currentTitle = 'Add new ${item.addressType} address';
    if( _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("EMPLOYEE_READ")
    && item.addressType == Constant.ADDRESS_TYPE_OS){
      currentTitle = 'Add new address';
    }

    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => AddressBookCreate(
          title: currentTitle,
          item: item,
          onChange: (){
            _tapChange = true;
            new ThreadTimer(new Duration(milliseconds: 500), () {
              setState(() {
                _tapChange = false;
              });
            });
          },
        ),
      ),
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}


class AddressBookLoader extends StatefulWidget {
  AddressBookLoader({Key key, @required this.addressType, this.currentUser,
  this.authorityResponse, this.userService, this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final AuthorityResponse authorityResponse;
  final UserService userService;
  final SharedPreferences sharedPrefs;
  final String addressType;
  @override
  _AddressBookLoaderState createState() => _AddressBookLoaderState();
}

class _AddressBookLoaderState extends State<AddressBookLoader>{

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  AddressBookService _addressBookService = new AddressBookService();
  AddressBookListResponse _addressBookListResponse;
  List<AddressBookItem> _items = [];
  int _page = 0;
  String _keyword = '';
  final _searchQuery = new TextEditingController();
  Timer _debounce;



  @override
  void initState() {
    super.initState();
    _searchQuery.addListener(_onSearchChanged);
  }
  @override
  void deactivate() {
    super.deactivate( );
  }

  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(seconds: 3), () {
      _keyword = _searchQuery.text.trim();
      if( (_keyword.length ==0 || _keyword.length >=3) && !_refreshController.isRefresh){
        _refreshController.requestRefresh();
      }
    });
  }

  Future<void> _getLoadData( bool refresh ) async{
    _addressBookListResponse = await _addressBookService.list(
        new AddressBookRequest(
            addressType: widget.addressType,
            osGuid: widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_READ") && !widget.authorityResponse.authorities.contains("MANAGER_READ")? widget.currentUser.guid : null,
            first: _page,
            max: Constant.ROW_PER_PAGE,
            keyword: _keyword
        )
    );

    if( _addressBookListResponse!=null && _addressBookListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _addressBookListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => i==0? _topWidget(c) : AddressBookListItem(
        post: _items[i-1],
        authorityResponse: widget.authorityResponse,
        addressBookService: _addressBookService,
        isEnd: i == _items.length,
        currentUser: widget.currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: widget.sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length + 1,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

  Widget _topWidget(BuildContext context){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, bottom: 8.0, left: 0.0, right: 0.0),
      child: Container(
        padding: EdgeInsets.only(top: 8, left: 4, bottom: 0, right: 4),
        child: TextField(
          textAlign: TextAlign.center,
          keyboardType: TextInputType.text,
          decoration: InputDecoration(
            hintText: tr(LocaleKeys.btnSearch),
            hintStyle: TextStyle(fontSize: 14),
            border: OutlineInputBorder(
              borderRadius: BorderRadius.circular(16),
              borderSide: BorderSide(
                width: 1,
                style: BorderStyle.none,
              ),
            ),
            filled: true,
            contentPadding: EdgeInsets.all(8),
            fillColor: Theme.of(context).backgroundColor,
            suffixIcon: _searchQuery.text.length>0? IconButton(
              icon: Icon(Icons.close_sharp,
                // color: Theme.of(context).primaryColorDark
              ),
              onPressed: (){
                setState(() {
                  _searchQuery.text = '';
                  _keyword = '';
                });
                _refreshController.requestRefresh();
              },
            ): Icon(Icons.search_sharp),
          ),
          controller: _searchQuery,
        ),
      ),
    );
  }
}


class AddressBookListItem extends StatefulWidget {

  AddressBookListItem({Key key, @required this.post,
    @required this.authorityResponse, @required this.addressBookService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final AddressBookItem post;
  final AuthorityResponse authorityResponse;
  final AddressBookService addressBookService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _AddressBookItemState createState() => _AddressBookItemState();

}

class _AddressBookItemState extends State<AddressBookListItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Wrap(
                  children: <Widget>[
                  ListTile(
                    title: Text(widget.post.name,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                        '${widget.post.address}, ${widget.post.street}, ${widget.post.townshipName}',
                        // style: TextStyle(color: Colors.black),
                    ),
                    leading: widget.post.avatar!=null && widget.post.avatar.length>10? ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Theme.of(context).backgroundColor, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.avatar}', width: 48, height: 48, ),
                            ),
                          ),
                        )
                    )
                    :ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Theme.of(context).primaryColor, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: Icon( Icons.person,
                                // color: Colors.white,
                              ),
                            ),
                          ),
                        )
                    ),
                    trailing: Icon( Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600],
                    ),
                    onTap: (){
                      String currentTitle = 'Update ${widget.post.addressType} address';
                      if( widget.authorityResponse.authorities!=null && !widget.authorityResponse.authorities.contains("EMPLOYEE_READ")
                          && widget.post.addressType == Constant.ADDRESS_TYPE_OS){
                        currentTitle = 'Update address';
                      }
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => AddressBookCreate(
                            title: currentTitle,
                            item: widget.post,
                            onChange: (){
                              widget.onChange();
                            },
                          ),
                        ),
                      );
                    },
                  ),
                      // _createActionButtons( context, widget.post ),
                      // _showConfirm( context, widget.post ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
