import 'package:provider/provider.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery/delivery_map.dart';
import 'package:qq_delivery/delivery/view/login.dart';
import 'package:qq_delivery/delivery/view/pos/pos_product_list.dart';
import 'package:qq_delivery/delivery/view/pos/pos_purchase_list.dart';
import 'package:qq_delivery/delivery/view/profile/profile_home.dart';
import 'package:qq_delivery/delivery/view/user/user_list.dart';
import 'package:qq_delivery/delivery/view/utils/my_webview.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:location/location.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';
import '../../theme.dart';
import 'address_book/address_book_list.dart';
import 'package:easy_localization/easy_localization.dart';
import 'delivery/delivery_list_home.dart';
import 'delivery/price_list.dart';
import 'delivery2/delivery_create_v2.dart';
import 'delivery2/pickup_list_home.dart';
import 'financial/employee_clearance_list.dart';
import 'pos/pos_category_list.dart';
import 'pos/pos_order_list.dart';
import 'pos/pos_product_order_list.dart';
import 'pos/pos_product_purchase_list.dart';
import 'pos/pos_supplier_list.dart';
import 'pos/purchase_sales_summary.dart';
import 'reports/active_way_count_by_town_list.dart';
import 'user/person_bank_list.dart';
import 'utils/qr_scan_page.dart';
import '/hrm/service/user_service.dart' as hrm;


class AppDrawer extends StatefulWidget {
  AppDrawer({Key key, this.currentUser, this.authorityResponse,
  this.sharedPrefs, this.startLocation}) : super(key: key);
  final User currentUser;
  final AuthorityResponse authorityResponse;
  final SharedPreferences sharedPrefs;
  final LocationData startLocation;

  @override
  _AppDrawerState createState() => _AppDrawerState();
}

class _AppDrawerState extends State<AppDrawer>{

  @override
  Widget build(BuildContext context) {
    var _themeProvider=Provider.of<ThemeChanger>(context);
    return Drawer(
      child: ListView(
        children: <Widget>[
          _createHeader( context),
          ListTile(
            title: Text('${widget.currentUser.name}', style: Theme.of(context).textTheme.headline6,),
            leading: Icon(Icons.edit,
                // color: Theme.of(context).primaryColor
            ),
            onTap: () async{
              Navigator.pop(context);
              String exit = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) => ProfileHome(),
                  )
              );
              if( exit != null && exit=='exit'){
                UserService userService = new UserService();
                await userService.logout();
                Navigator.pushReplacement(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => LoginUser(
                        title: 'Login',
                      ),
                    )
                );
              }
            },
          ),
          Divider(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName!=Constant.ROLE_NAME_EMPLOYEE && widget.authorityResponse.authorities.contains("USER_WRITE")?
          _createDrawerItem(
              icon: Icons.local_shipping_sharp,
              text: tr(LocaleKeys.lblDeliCreateTitle),
              onTap: () {
                Navigator.pop(context);
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => DeliveryCreateV2( ),
                  ),
                );
              }
          ) : Container(),

          _createDrawerItem(
              icon: Icons.assignment,
              text: tr(LocaleKeys.lblMenuPickupWays),
              onTap: () {
                Navigator.pop(context);
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => PickupListHome(
                      selectedIndex: 1,
                      title: tr(LocaleKeys.lblToPickup),
                        onChange: (){},
                    ),
                  ),
                );
              }
          ),

          _createDrawerItem(
              icon: Icons.assignment,
              text: tr(LocaleKeys.lblMenuDeliverWays),
              onTap: () {
                Navigator.pop(context);
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => DeliveryListHome(
                      selectedIndex: 1,
                      title: tr(widget.authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                      onChange: (){},
                    ),
                  ),
                );
              }
          ),

          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("EMPLOYEE_READ")?
          _createDrawerItem(
              icon: Icons.format_list_numbered_sharp,
              text: tr(LocaleKeys.lblActiveWaysByTownship),
              onTap: (){
                try{
                  Navigator.pop(context);
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => ActiveWayCountByTownList(),
                      )
                  );
                }catch(e){}
              }
          ):Container(),
          // _createDrawerItem(
          //     icon: Icons.assignment,
          //     text: tr(LocaleKeys.lblInvoices),
          //     onTap: () {
          //       Navigator.pop(context);
          //       Navigator.push(
          //         context,
          //         MaterialPageRoute(
          //           builder: (context) => DeliveryInvoice( ),
          //         ),
          //       );
          //     }
          // ),
          // widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_READ")?
          // _createDrawerItem(
          //     icon: Icons.monetization_on_sharp,
          //     text: tr(LocaleKeys.lblOSFinancialStatement),
          //     onTap: (){
          //       try{
          //         Navigator.pop(context);
          //         Navigator.push(
          //             context,
          //             MaterialPageRoute(
          //               builder: (BuildContext context) => OSDepositRefundList(),
          //             )
          //         );
          //       }catch(e){}
          //     }
          // ):Container(),
          Divider(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("EMPLOYEE_READ")?
          _createDrawerItem(
              icon: Icons.monetization_on_outlined,
              text: tr(LocaleKeys.lblEmployeeClearance),
              onTap: (){
                try{
                  Navigator.pop(context);
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => EmployeeClearanceList(),
                      )
                  );
                }catch(e){}
              }
          ):Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_READ")?
          Divider() : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_READ")?
          _createDrawerItem(
              icon: Icons.person_pin_circle_sharp,
              text: tr(LocaleKeys.lblTrackingMap),
              onTap: () async{
                try{
                  DeliveryContentServiceV2 service = new DeliveryContentServiceV2();
                  EmployeeListForDeliveryResponse response = await service.availableAllEmployees();
                  if( response.items != null ){
                    Navigator.pop(context);
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => DeliveryMap(
                            startLocation: widget.startLocation,
                            items: response.items,
                            title: "Employees' location",
                          ),
                        )
                    );
                  }
                }catch(e){}
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_WRITE")?
          Divider() : Container(),
          widget.authorityResponse.authorities!=null &&
              (widget.authorityResponse.authorities.contains("OS_WRITE")
                  || widget.authorityResponse.authorities.contains("EMPLOYEE_WRITE"))?
          _createDrawerItem(
              icon: Icons.menu_book,
              text: tr(LocaleKeys.lblAddressBook),
              onTap: (){
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => AddressBookList(),
                    )
                );
              }
          ) : Container(),

          widget.authorityResponse.authorities!=null &&
                  widget.authorityResponse.authorities.contains("MANAGER_WRITE")?
          Divider() : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_WRITE")?
          ListTile(title: Text('User accounts'),) : Container(),

          // widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_WRITE")?
          // Container(
          //   margin: EdgeInsets.only(left: 16),
          //   child: _createDrawerItem(
          //       icon: Icons.people_alt_sharp,
          //       text: 'Customers',
          //       onTap: (){
          //         if(widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_WRITE")) {
          //           Navigator.pop(context);
          //           Navigator.push(
          //               context,
          //               MaterialPageRoute(
          //                 builder: (BuildContext context) =>
          //                     UserList(
          //                       selectedIndex: 0,
          //                     ),
          //               )
          //           );
          //         }else{
          //           Navigator.pop(context);
          //           Navigator.push(
          //               context,
          //               MaterialPageRoute(
          //                 builder: (BuildContext context) =>
          //                     CustomerList(),
          //               )
          //           );
          //         }
          //
          //       }
          //   ),
          // ) : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_WRITE")?
          Container(
            margin: EdgeInsets.only(left: 16),
            child: _createDrawerItem(
              icon: Icons.perm_contact_cal_sharp,
              text: tr(LocaleKeys.lblEmployee),
              onTap: (){
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => UserList(
                        selectedIndex: 0,
                      ),
                    )
                );
              }
            ),
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_WRITE")?
          Container(
            margin: EdgeInsets.only(left: 16),
            child: _createDrawerItem(
                icon: Icons.supervised_user_circle_sharp,
                text: tr(LocaleKeys.lblOS),
                onTap: (){
                  Navigator.pop(context);
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => UserList(
                          selectedIndex: 1,
                        ),
                      )
                  );
                }
            ),
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_READ")?
          Divider() : Container(),
          // _createDrawerItem(
          //     icon: Icons.monetization_on,
          //     text: 'Accounting',
          //     onTap: (){}),
          // _createDrawerItem(
          //     icon: Icons.attach_money_sharp,
          //     text: 'Employee clearance',
          //     onTap: (){}),
          // Divider(),

          _createDrawerItem(
              icon: Icons.qr_code_scanner_sharp,
              text: tr(LocaleKeys.lblQRScanner),
              onTap: () async{
                Navigator.pop(context);
                await Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => QRScanPage( ),
                    )
                );
              }
          ),




          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          Divider() : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.business_sharp,
              text: 'Bank accounts',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          PersonBankAccountList(),
                    )
                );
              }
          ) : Container(),

          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          Divider() : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          ListTile(title: Text('POS management'), trailing: Icon(Icons.point_of_sale_sharp),) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.view_list,
              text: 'Products',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSProductList(),
                    )
                );
                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // String url = "https://testing.quickdoor2door.com/access_request/pos/$token";
                // if (await canLaunch('$url')) {
                // await launch('$url');
                // } else {
                // throw 'Could not launch $url';
                // }

                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'POS - Products',
                //             url: 'https://testing.quickdoor2door.com/access_request/pos_productlist/$token',
                //           ),
                //     )
                // );
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) => POSProductOrderList( ),
                //     )
                // );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.supervisor_account_sharp,
              text: 'Suppliers',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSSupplierList( ),
                    )
                );
                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'POS - Suppliers',
                //             url: 'https://testing.quickdoor2door.com/access_request/pos_supplierlist/$token',
                //           ),
                //     )
                // );
              }
          ) : Container(),
          // widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          // _createDrawerItem(
          //     icon: Icons.account_tree_sharp,
          //     text: 'Categories',
          //     onTap: () async{
          //       Navigator.pop(context);
          //       Navigator.push(
          //           context,
          //           MaterialPageRoute(
          //             builder: (BuildContext context) =>
          //                 POSCategoryList( ),
          //           )
          //       );
          //     }
          // ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.input,
              text: 'Purchase',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSProductPurchaseList( ),
                    )
                );
                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'POS - Purchase',
                //             url: 'https://testing.quickdoor2door.com/access_request/pos_purchase/$token',
                //           ),
                //     )
                // );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.list_alt_sharp,
              text: 'Purchase report',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSPurchaseList(),
                    )
                );
                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'POS - Purchase Report',
                //             url: 'https://testing.quickdoor2door.com/access_request/pos_report_purchaseorder/$token',
                //           ),
                //     )
                // );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.shopping_cart_sharp,
              text: 'Sale',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSProductOrderList( ),
                    )
                );
                // UserService userService = new UserService();
                // String token = await userService.getToken();
                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'POS - Customer Order',
                //             url: 'https://testing.quickdoor2door.com/access_request/pos_order/$token',
                //           ),
                //     )
                // );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.list_alt_sharp,
              text: 'Sales report',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSOrderList(status: 'Accept',),
                    )
                );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.list_alt_sharp,
              text: 'Sales cancel report',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          POSOrderList(status: 'Cancel',),
                    )
                );
              }
          ) : Container(),
          widget.authorityResponse.authorities!=null && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
          _createDrawerItem(
              icon: Icons.monetization_on_outlined,
              text: 'Profit summary',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          PurchaseSalesSummary(),
                    )
                );
              }
          ) : Container(),

          Divider(),
          _createDrawerItem(
              icon: Icons.playlist_add_check_sharp,
              text: tr(LocaleKeys.lblPriceList),
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          PriceList( ),
                    )
                );
              }
          ),
          Divider(),
          ListTile(
            title: Text('Language'),
            subtitle: _dropdownWidget(),
            leading: Icon(Icons.language,
              // color: Colors.blueGrey[600],
            ),
          ),
          Constant.THEME_CHANGABLE?Divider():Container(),
          Constant.THEME_CHANGABLE?ListTile(
            title: Text('Theme'),
            subtitle: _themeChooserWidget( _themeProvider ),
            leading: Icon(Icons.wb_sunny_sharp,
              // color: Colors.blueGrey[600],
            ),
          ):Container(),
          _createDrawerItem(
              icon: Icons.privacy_tip_sharp,
              text: 'Terms and conditions',
              onTap: () async{
                Navigator.pop(context);
                Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          MyWebView(
                            title: 'Terms and conditions',
                            url: 'https://www.quickdoor2door.com/toc.html',
                          ),
                    )
                );
              }
          ),
          _createDrawerItem(
              icon: Icons.help_outline_sharp,
              text: 'User guide',
              onTap: () async{
                Navigator.pop(context);

                String uri = widget.authorityResponse.authorities!=null
                    && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
                'https://v3.quickdoor2door.com/os_user_guid.pdf' :
                'https://v3.quickdoor2door.com/employee_user_guid.pdf';
                if (await canLaunch( '$uri' )) {
                  await launch( '$uri' );
                } else {
                  throw 'Could not launch $uri';
                }

                // Navigator.push(
                //     context,
                //     MaterialPageRoute(
                //       builder: (BuildContext context) =>
                //           MyWebView(
                //             title: 'User guide',
                //             url: widget.authorityResponse.authorities!=null
                //                 && widget.currentUser.roleName==Constant.ROLE_NAME_OS?
                //             'https://v3.quickdoor2door.com/os_user_guid.pdf' : 'https://v3.quickdoor2door.com/employee_user_guid.pdf',
                //           ),
                //     )
                // );
              }
          ),
          _createDrawerItem(
              icon: Icons.power_settings_new_sharp,
              text: 'Logout',
              onTap: () async{
                UserService userService = new UserService();
                await userService.logout();
                hrm.UserService hrmUserService = new hrm.UserService();
                await hrmUserService.logout();
                Navigator.pop(context);
                Navigator.pushReplacement(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) => LoginUser(
                        title: 'Login',
                      ),
                    )
                );
              }
          ),

          // ListTile(
          //   title: Text('Version: 1.0.0.1'),
          //   onTap: () {},
          // ),
        ],
      ),
    );
  }

  @override
  void initState() {
    super.initState();
    _loadDefaultLanguage();
    _loadDefaultTheme();
  }

  void _loadDefaultLanguage() async{
    String lang = await getLanguage();
    if( lang == 'en')
      selectedLanguage = languages[1];
    else
      selectedLanguage = languages[0];
    setState(() { });
  }

  void _loadDefaultTheme() async{
    String name = await getTheme();
    // if( name == Constant.THEME_NAME_BLUE)
    //   _selectedTheme = themeNames[0];
    // else if( name == Constant.THEME_NAME_RED)
    //   _selectedTheme = themeNames[1];
    // else if( name == Constant.THEME_NAME_GOLD)
    //   _selectedTheme = themeNames[2];
    // else if( name == Constant.THEME_NAME_GREEN)
    //   _selectedTheme = themeNames[3];
    // else
      if( name == Constant.THEME_NAME_ORANGE)
      _selectedTheme = themeNames[0];
    else if( name == Constant.THEME_NAME_DARK_MODE)
      _selectedTheme = themeNames[1];
    setState(() { });
  }

  Widget _createHeader(BuildContext context) {
    return DrawerHeader(
        margin: EdgeInsets.zero,
        padding: EdgeInsets.all(16),
        decoration: BoxDecoration(
            shape: BoxShape.circle,
            image: DecorationImage(
                fit: BoxFit.scaleDown,
                image: widget.currentUser.avatar!=null && widget.currentUser.avatar.length>10?
                new NetworkImage( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.currentUser.avatar}',)
                    : AssetImage('assets/images/account.png')
            ),
        ),
        /*child: Stack(children: <Widget>[
          Positioned(
              bottom: 0.1,
              left: 16.0,
              child: RaisedButton.icon(
                elevation: 0.0,
                icon: Icon(Icons.edit, color: Theme.of(context).primaryColor),
                color: Colors.transparent,
                onPressed:() async{
                  Navigator.pop(context);
                  String exit = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => ProfileHome(),
                      )
                  );
                  if( exit != null && exit=='exit'){
                    UserService userService = new UserService();
                    await userService.logout();
                    Navigator.pushReplacement(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => LoginUser(
                            title: 'Login',
                          ),
                        )
                    );
                  }
                },
                label: Text(
                  '${widget.currentUser.name}',
                  style: TextStyle(
                    fontWeight: FontWeight.bold,
                    fontSize: 20,
                    color: Theme.of(context).primaryColor,
                  ),
                ),
              ),
          ),
        ])*/
    );
  }

  Widget _createDrawerItem({IconData icon, String text, GestureTapCallback onTap}) {
    return ListTile(
      title: Row(
        children: <Widget>[
          Icon(icon,
            // color: Colors.blueGrey[600],
          ),
          Padding(
            padding: EdgeInsets.only(left: 8.0),
            child: Text(text),
          )
        ],
      ),
      onTap: onTap,
    );
  }


  Language selectedLanguage;
  List<Language> languages = <Language> [new Language("မြန်မာ", SvgPicture.asset( 'assets/images/myanmar.svg', semanticsLabel: 'My',height: 32, width: 32, )),
    Language('English', SvgPicture.asset( 'assets/images/uk.svg', semanticsLabel: 'En',height: 32, width: 32, ))];

  Widget _dropdownWidget() {
    return new DropdownButtonHideUnderline(
      child: DropdownButton<Language>(
        //hint:  Text("Select item"),
        value: selectedLanguage,
        onChanged: (Language value){
          setState(() {
            selectedLanguage = value;
            if( value.name != 'English'){
              context.locale = Locale('my','');
              setLanguage( 'my' );
            }else{
              context.locale = Locale('en','');
              setLanguage( 'en' );
            }
          });
        },
        items: languages.map((Language lang) {
          return  DropdownMenuItem<Language>(
            value: lang,
            child: Row(
              children: <Widget>[
                lang.icon,
                SizedBox(width: 15,),
                Text(
                  lang.name,
//                  style:  TextStyle(color: Colors.white),
                ),
              ],
            ),
          );
        }).toList(),
      ),
    );
  }


  ThemeItem _selectedTheme;
  List<ThemeItem> themeNames = <ThemeItem> [
    // new ThemeItem(Constant.THEME_NAME_BLUE,'Branded Blue'),
    // new ThemeItem(Constant.THEME_NAME_RED,'Branded Red'),
    // new ThemeItem(Constant.THEME_NAME_GOLD,'Gold'),
    // new ThemeItem(Constant.THEME_NAME_GREEN,'Green'),
    new ThemeItem(Constant.THEME_NAME_ORANGE,'Branded Orange'),
    new ThemeItem(Constant.THEME_NAME_DARK_MODE, 'Dark Mode')];

  Widget _themeChooserWidget(var _themeProvider) {
    return new DropdownButtonHideUnderline(
      child: DropdownButton<ThemeItem>(
        value: _selectedTheme,
        onChanged: (ThemeItem value){
          _themeProvider.setTheme(getDefaultTheme( value.name ));
          setState(() {
            _selectedTheme = value;
            setTheme(value.name);
          });
        },
        items: themeNames.map((ThemeItem theme) {
          return  DropdownMenuItem<ThemeItem>(
            value: theme,
            child: Text(theme.label),
          );
        }).toList(),
      ),
    );
  }
}
class ThemeItem {
  const ThemeItem(this.name, this.label);
  final String name;
  final String label;
}

class Language {
  const Language(this.name,this.icon);
  final String name;
  final SvgPicture icon;
}