
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:isolate';
import 'dart:ui';
import 'package:easy_localization/easy_localization.dart';
// import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:flutter_downloader/flutter_downloader.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:flutter_uploader/flutter_uploader.dart';
import 'package:image_picker/image_picker.dart';
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/chat_latest_list_response.dart';
import 'package:qq_delivery/delivery/model/response/chat_search_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/chat_list_item.dart';
import 'package:qq_delivery/delivery/model/shared/message_item.dart';
import 'package:qq_delivery/delivery/service/v2/chat_service.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:qq_delivery/delivery/view/utils/voice_recorder.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:image/image.dart' as imglib;
import 'package:flutter_sound/flutter_sound.dart' as sound;
import 'package:flutter_sound_platform_interface/flutter_sound_recorder_platform_interface.dart' as sound;
import 'package:url_launcher/url_launcher.dart';
import 'chat_group_member.dart';
import 'chat_search.dart';


class ChatConversationList extends StatefulWidget {
  ChatConversationList({Key key, this.sharedPrefs, this.sessionGuid,
    this.sessionType, this.currentUser, this.title, this.groupGuid}) : super(key: key);
  final SharedPreferences sharedPrefs;
  final String sessionGuid;
  final String sessionType;
  final User currentUser;
  final String title;
  final String groupGuid;

  @override
  _CommentState createState() => _CommentState();
}

class _CommentState extends State<ChatConversationList>{

  ChatLatestListResponse _chatLatestListResponse;
  ChatService _chatService = new ChatService();
  List<ChatListItem> _items = [];
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  int _page = 0;
  bool _processing = false;
  bool _hasText = false;
  final TextEditingController _controller_comment = new TextEditingController();
  FlutterUploader uploader = FlutterUploader();
  StreamSubscription _progressSubscription;
  StreamSubscription _resultSubscription;
  Map<String, UploadItem> _tasks = {};
  String _mediaFileGuid;
  bool _recording = false;
  bool isStopped = false;
  int _totalRecords = 0;

  @override
  void initState() {
    super.initState();

    ////// START OF FILE UPLOAD /////////////////////////////////
    _progressSubscription = uploader.progress.listen((progress) {
      final task = _tasks[progress.tag];
      print("progress: ${progress.progress} , tag: ${progress.tag}");
      if (task == null) return;
      if (task.isCompleted()) return;
      setState(() {
        _tasks[progress.tag] =
            task.copyWith(progress: progress.progress, status: progress.status);
      });
    });

    _resultSubscription = uploader.result.listen((result) {
      // print("id: ${result.taskId}, status: ${result.status}, response: ${result.response}, statusCode: ${result.statusCode}, tag: ${result.tag}, headers: ${result.headers}");
      Map<String, dynamic> resultData = jsonDecode(result.response);
      if( resultData['success']){
        _mediaFileGuid = resultData['data'];
        _sendMediaMessage( _tasks[result.tag] );
      }
      final task = _tasks[result.tag];
      if (task == null) return;

      setState(() {
        _tasks[result.tag] = task.copyWith(status: result.status);
      });
    }, onError: (ex, stacktrace) {
      print("exception: $ex");
      print("stacktrace: $stacktrace" ?? "no stacktrace");
      final exp = ex as UploadException;
      final task = _tasks[exp.tag];
      if (task == null) return;

      setState(() {
        _tasks[exp.tag] = task.copyWith(status: exp.status);
      });
    });
    // END OF FILE UPLOAD //////////////////////

    Timer.periodic(Duration(seconds: 10), (timer) {
      if (isStopped ) {
        timer.cancel();
      }
      try{
        print( _page );
        if( _page == 0 )
          _getLoadData( true );
      }catch(e){print(e.toString());}

    });
  }
  @override
  void deactivate() {
    isStopped=true;
    super.deactivate( );
  }
  @override
  void dispose() {
    isStopped=true;
    super.dispose();
  }
  _getLoadData( bool refresh ) async{
    Map<String, dynamic> request = {
      'first': _page,
      'max':Constant.ROW_PER_PAGE,
      'sessionGuid': widget.sessionGuid
    };

    if( widget.sessionType == 'Individual')
      _chatLatestListResponse = await _chatService.individualRecord(request);
    else if( widget.sessionType == 'Group')
      _chatLatestListResponse = await _chatService.groupRecord(request);

    if( _chatLatestListResponse!=null && _chatLatestListResponse.items!=null){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _chatLatestListResponse.items.forEach((element) {
        _items.add(element);
      });
      _totalRecords = _chatLatestListResponse.totalRecords;
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            backgroundColor: Theme.of(context).backgroundColor,
            key: _scaffoldKey,
            appBar: AppBar(
              title: Text( widget.title ),
              actions: <Widget>[
                widget.sessionType=='Group'?IconButton(
                  icon: const Icon(Icons.view_list_sharp),
                  tooltip: 'Search',
                  onPressed: () async{
                    SearchListItem searchListItem = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => ChatGroupMember(
                          title: 'Member list',
                          groupGuid: widget.groupGuid,
                        ),
                      ),
                    );

                    if( searchListItem != null && searchListItem.type =='person' ) {
                      setState(() {
                        _processing = true;
                      });
                      Map<String, dynamic> request = {
                        'memberGuid':searchListItem.guid,
                        'sessionGuid': widget.sessionGuid
                      };

                      Map<String, dynamic> result = await _chatService.addMember( request );
                      if( result['success'] ){

                        MessageItem commentItem = new MessageItem(
                            userGuid:null,
                            messageContent: '${searchListItem.name} has been added by ${widget.currentUser.name}',
                            attachFile: null,
                            sessionType: widget.sessionType,
                            sessionGuid: widget.sessionGuid,
                            mediaType: 'Text'
                        );
                        await _chatService.sendMessage(commentItem.toMap());

                        _refreshController.requestRefresh();
                      }

                      setState(() {
                        _processing = false;
                      });
                    }

                  },
                ) : Container(),
                widget.sessionType=='Group'?IconButton(
                  icon: const Icon(Icons.person_add),
                  tooltip: 'Add more people',
                  onPressed: () async{
                    SearchListItem searchListItem = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => ChatSearch(title: 'Add more people', ),
                      ),
                    );

                    if( searchListItem != null && searchListItem.type =='person' ) {
                      setState(() {
                        _processing = true;
                      });
                      Map<String, dynamic> request = {
                        'memberGuid':searchListItem.guid,
                        'sessionGuid': widget.sessionGuid
                      };

                      Map<String, dynamic> result = await _chatService.addMember( request );
                      if( result['success'] ){

                        MessageItem commentItem = new MessageItem(
                            userGuid:null,
                            messageContent: '${searchListItem.name} has been added by ${widget.currentUser.name}',
                            attachFile: null,
                            sessionType: widget.sessionType,
                            sessionGuid: widget.sessionGuid,
                            mediaType: 'Text'
                        );
                        await _chatService.sendMessage(commentItem.toMap());

                        _refreshController.requestRefresh();
                      }

                      setState(() {
                        _processing = false;
                      });
                    }
                  },
                ) : Container(),
              ],
              bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            ),
            body:
            SingleChildScrollView(
                child: Container(
                  height: MediaQuery.of(context).size.height,
                  child:
                  Column(
                      children:<Widget>[
                        Container(
                          height: MediaQuery.of(context).size.height * 0.78,
                          child: SmartRefresher(
                            controller: _refreshController,
                            enablePullUp: true,
                            child: buildContent(context),
                            physics: BouncingScrollPhysics(),
                            header: WaterDropHeader(),
                            footer: ClassicFooter(
                              loadStyle: LoadStyle.ShowWhenLoading,
                              completeDuration: Duration(milliseconds: 500),
                            ),

                            // scroll down
                            onRefresh: () async {
                              _page = 0;
                              await _getLoadData( true);
                              if (mounted) setState(() {});
                              _refreshController.refreshCompleted();
                            },
                            // scroll up
                            onLoading: () async {
                              ++_page;
                              await _getLoadData( false );
                              if (mounted) setState(() {});
                              _refreshController.loadComplete();
                            },
                          ),
                        ),
                        !_recording? Align(
                            alignment: Alignment.bottomCenter,
                            child: Container(
                                margin: EdgeInsets.only(top: 8, bottom: 0),
                                child: Wrap(
                                    children: <Widget>[
                                      Container(
                                          child: _tasks.isNotEmpty && _tasks["image upload 1"].status!=null && _tasks["image upload 1"].status == UploadTaskStatus.running
                                              ? LinearProgressIndicator(value: _tasks["image upload 1"].progress.toDouble() / 100)
                                              : Container()
                                      ),
                                      Container(
                                        width: MediaQuery.of(context).size.width * (_hasText?0.85 : 0.55),
                                        child:TextFormField(
                                          textAlign: TextAlign.left,
                                          keyboardType: TextInputType.multiline,
                                          minLines: 1,
                                          maxLines: 5,
                                          decoration: InputDecoration(
                                            hintText: 'Message here',
                                            // hintStyle: TextStyle(fontSize: 14),
                                            border: OutlineInputBorder(
                                              borderRadius: BorderRadius.circular(8),
                                              borderSide: BorderSide(
                                                width: 1,
                                                style: BorderStyle.none,
                                              ),
                                            ),
                                            filled: true,
                                            contentPadding: EdgeInsets.all(4),
                                            fillColor: Theme.of(context).backgroundColor,
                                          ),
                                          controller: _controller_comment,
                                          onChanged: (value){
                                            setState(() {
                                              if( value.isEmpty)
                                                _hasText = false;
                                              else
                                                _hasText = true;
                                            });
                                          },
                                        ),
                                      ),
                                      _hasText?IconButton(
                                        icon: const Icon(Icons.send_sharp,),
                                        tooltip: 'Send message',
                                        color: Theme.of(context).primaryColor,
                                        onPressed:_tasks.isNotEmpty && _tasks["image upload 1"].status!=null
                                            && _tasks["image upload 1"].status == UploadTaskStatus.running?null: () async{
                                          if( _controller_comment.text.trim().isNotEmpty ) {
                                            setState(() {
                                              _processing = true;
                                            });
                                            MessageItem commentItem = new MessageItem(
                                                userGuid:null,
                                                messageContent: _controller_comment.text,
                                                attachFile: null,
                                                sessionType: widget.sessionType,
                                                sessionGuid: widget.sessionGuid,
                                                mediaType: ChatListItem.MEDIA_TYPE_TEXT
                                            );
                                            await _chatService.sendMessage(commentItem.toMap());
                                            _controller_comment.text = "";
                                            setState(() {
                                              _processing = false;
                                              _hasText = false;
                                            });
                                            _refreshController.requestRefresh();
                                          }
                                        },
                                      ):SizedBox(),

                                      !_hasText?IconButton(
                                        icon: const Icon(Icons.keyboard_voice_sharp),
                                        tooltip: 'Voice message',
                                        color: Theme.of(context).primaryColor,
                                        onPressed:_tasks.isNotEmpty && _tasks["image upload 1"].status!=null
                                            && _tasks["image upload 1"].status == UploadTaskStatus.running?null:  () async{
                                          setState(() {
                                            _recording = true;
                                          });
                                        },
                                      ):SizedBox(),

                                      !_hasText?IconButton(
                                        icon: const Icon(Icons.add_to_photos_rounded),
                                        tooltip: 'Attach photo',
                                        color: Theme.of(context).primaryColor,
                                        onPressed:_tasks.isNotEmpty && _tasks["image upload 1"].status!=null
                                            && _tasks["image upload 1"].status == UploadTaskStatus.running?null:  () async{
                                          String image = await _getImage(false);
                                          if( image != null && image.length>100){
                                            setState(() {
                                              _processing = true;
                                            });
                                            MessageItem commentItem = new MessageItem(
                                                userGuid:null,
                                                messageContent: _controller_comment.text,
                                                attachFile: image,
                                                sessionType: widget.sessionType,
                                                sessionGuid: widget.sessionGuid,
                                                mediaType: ChatListItem.MEDIA_TYPE_IMAGE
                                            );
                                            await _chatService.sendMessage(commentItem.toMap());
                                            _controller_comment.text = "";
                                            setState(() {
                                              _processing = false;
                                            });
                                            _refreshController.requestRefresh();
                                          }
                                        },
                                      ):SizedBox(),
                                      !_hasText?IconButton(
                                        icon: const Icon(Icons.camera_alt_sharp),
                                        tooltip: 'Take photo',
                                        color: Theme.of(context).primaryColor,
                                        onPressed:_tasks.isNotEmpty && _tasks["image upload 1"].status!=null
                                            && _tasks["image upload 1"].status == UploadTaskStatus.running?null:  () async{
                                          String image = await _getImage(true);
                                          if( image != null && image.length>100){
                                            setState(() {
                                              _processing = true;
                                            });
                                            MessageItem commentItem = new MessageItem(
                                                userGuid:null,
                                                messageContent: _controller_comment.text,
                                                attachFile: image,
                                                sessionType: widget.sessionType,
                                                sessionGuid: widget.sessionGuid,
                                                mediaType: ChatListItem.MEDIA_TYPE_IMAGE
                                            );
                                            await _chatService.sendMessage(commentItem.toMap());
                                            _controller_comment.text = "";
                                            setState(() {
                                              _processing = false;
                                            });
                                            _refreshController.requestRefresh();
                                          }
                                        },
                                      ):SizedBox(),
                                      !_hasText?IconButton(
                                        icon: const Icon(Icons.attach_file_sharp),
                                        tooltip: 'Attach file',
                                        color: Theme.of(context).primaryColor,
                                        onPressed:_tasks.isNotEmpty && _tasks["image upload 1"].status!=null
                                            && _tasks["image upload 1"].status == UploadTaskStatus.running?null:  () async{
                                          _tasks.clear();
                                          await _getFile();
                                        },
                                      ):SizedBox(),
                                    ]
                                )
                            )
                        ) : VoiceRecorder(
                          onChange: (recordFile) async{
                            await _getRecording( recordFile );
                            setState(() {
                              _recording = false;
                            });
                          },
                        ),
                      ]
                  ),
                )
            )
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      shrinkWrap: false,
      reverse: true,
      padding: EdgeInsets.only(left: 4, right: 4),
      itemBuilder: (c, i) => CommentListItem(
        post: _items[i],
        sharedPrefs: widget.sharedPrefs,
        currentUser: widget.currentUser,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.1,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Future<String> _getImage( bool isCamera ) async {
    String selectedImage;
    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/gelivery_tmp';
        await Directory(tmpDir).create(recursive: true);

        String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
        File rotatedFile = await FlutterImageCompress.compressAndGetFile(
          image.path,
          photoPath2,
          quality: 72,
          rotate: 0,
        );
        imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
        String cropFilePath = '$tmpDir/cropped.jpg';
        File(cropFilePath)
          ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));

        File smallFile  =File(cropFilePath);
        String encodedString = encode( smallFile );
        smallFile.deleteSync(recursive: false);
        rotatedFile.deleteSync(recursive: false);
        setState(() {
          selectedImage = encodedString;
        });

      }
    }catch(e){}
    return selectedImage;
  }

  Future<void> _getFile() async{
    _mediaFileGuid = null;
    try{

      // FilePickerResult result = await FilePicker.platform.pickFiles();
      final picker = ImagePicker();
      final result = await picker.getImage(source: ImageSource.gallery);

      if(result != null) {
        File file = File( result.path);
        // File file = File(result.files.single.path);
        final String filename = basename(file.path);
        final String savedDir = dirname(file.path);
        final tag = "image upload ${_tasks.length + 1}";
        final taskId = await uploader.enqueue(
            url: '${getBaseURL(widget.sharedPrefs)}/v1/files/upload',
            files: [FileItem(filename: filename, savedDir: savedDir, fieldname:"file")],
            method: UploadMethod.POST,
            // headers: {"apikey": "api_123456", "userkey": "userkey_123456"},
            // data: {"name": "john"},
            showNotification: false,
            tag: tag);

        setState(() {
          _tasks.putIfAbsent(
              tag,
                  () => UploadItem(
                id: taskId,
                tag: tag,
                type: ChatListItem.MEDIA_TYPE_FILE,
                status: UploadTaskStatus.enqueued,
              ));
        });
      }
    }catch(e){}
  }

  Future<void> _getRecording( String recordingPath ) async{
    try{
      if( recordingPath == 'Stop') return;
      File recordFile  =File(recordingPath);
      print(recordingPath );
      final String filename = basename(recordFile.path);
      final String savedDir = dirname(recordFile.path);
      final tag = "image upload ${_tasks.length + 1}";
      final taskId = await uploader.enqueue(
          url: '${getBaseURL(widget.sharedPrefs)}/v1/files/upload',
          files: [FileItem(filename: filename, savedDir: savedDir, fieldname:"file")],
          method: UploadMethod.POST,
          // headers: {"apikey": "api_123456", "userkey": "userkey_123456"},
          // data: {"name": "john"},
          showNotification: false,
          tag: tag);

      setState(() {
        _tasks.putIfAbsent(
            tag,
                () => UploadItem(
                id: taskId,
                tag: tag,
                type: ChatListItem.MEDIA_TYPE_VOICE,
                status: UploadTaskStatus.enqueued,
                filePath: recordingPath
            ));
      });
    }catch(e){}
  }

  Future cancelUpload(String id) async {
    await uploader.cancel(taskId: id);
  }

  void _sendMediaMessage(UploadItem task) async{
    setState(() {
      _processing = true;
    });
    MessageItem commentItem = new MessageItem(
        userGuid:null,
        messageContent: _controller_comment.text,
        attachFile: _mediaFileGuid,
        sessionType: widget.sessionType,
        sessionGuid: widget.sessionGuid,
        mediaType: task.type
    );
    await _chatService.sendMessage(commentItem.toMap());
    _controller_comment.text = "";
    setState(() {
      _processing = false;
    });
    _refreshController.requestRefresh();

    try{
      if( task.type == ChatListItem.MEDIA_TYPE_VOICE ){
        File recordFile  =File(task.filePath);
        recordFile.deleteSync(recursive: false);
      }
    }catch(e){}
  }
}

class UploadItem {
  final String id;
  final String tag;
  final String type;
  final int progress;
  final UploadTaskStatus status;
  final String filePath;

  UploadItem({
    this.id,
    this.tag,
    this.type,
    this.progress = 0,
    this.status = UploadTaskStatus.undefined,
    this.filePath
  });

  UploadItem copyWith({UploadTaskStatus status, int progress}) => UploadItem(
      id: this.id,
      tag: this.tag,
      type: this.type,
      status: status ?? this.status,
      progress: progress ?? this.progress);

  bool isCompleted() =>
      this.status == UploadTaskStatus.canceled ||
          this.status == UploadTaskStatus.complete ||
          this.status == UploadTaskStatus.failed;
}

// enum MediaType { Image, Voice, File }

class CommentListItem extends StatefulWidget {

  CommentListItem({Key key, @required this.post, this.sharedPrefs,
    this.currentUser}) : super(key: key);
  final ChatListItem post;
  final SharedPreferences sharedPrefs;
  final User currentUser;

  @override
  _CommentListItemState createState() => _CommentListItemState();

}

class _CommentListItemState extends State<CommentListItem> {

  List<_TaskInfo> _tasks;
  List<_ItemHolder> _items;
  bool _isLoading;
  bool _permissionReady;
  String _localPath;
  ReceivePort _port = ReceivePort();
  double _progressVal = 0;

  @override
  void initState() {
    super.initState();

    if( widget.post.mediaType == ChatListItem.MEDIA_TYPE_FILE ) {
      _bindBackgroundIsolate();
      FlutterDownloader.registerCallback(downloadCallback);
      _isLoading = true;
      _permissionReady = false;
    }
  }
  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              widget.currentUser.guid==widget.post.senderGuid?
              Card(
                // color: Colors.white,
                elevation: 0,
                child: widget.post.attachFile!=null && widget.post.attachFile.isNotEmpty && widget.post.attachFile.length>10?
                ListTile(
                  // title: Text(
                  //   widget.post.sender,
                  //   textAlign: TextAlign.end,
                  // ),
                  title: Text(
                    new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                        ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                    '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                        : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}',
                    textAlign: TextAlign.end,
                  ),
                  subtitle: _mediaFile( context ),
                ) :
                ListTile(
                  // title: Text(
                  //   widget.post.sender,
                  //   textAlign: TextAlign.end,
                  // ),
                  subtitle: Text(widget.post.messageContent, textAlign: TextAlign.end,),
                  title: Text(
                    new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                        ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                    '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                        : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}',
                    textAlign: TextAlign.end,
                  ),
                ),
              )
                  :
              Card(
                // color: Colors.white,
                elevation: 0,
                child: widget.post.attachFile!=null && widget.post.attachFile.isNotEmpty && widget.post.attachFile.length>10?
                ListTile(
                  title: Text(
                    widget.post.sender + ', ' + (new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                        ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                    '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                        : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}'),
                  ),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 42,
                            height: 42,
                            child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.senderPhoto}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                  // trailing: Text(
                  //   new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                  //       ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                  //   '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                  //       : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}',
                  //   textAlign: TextAlign.center,
                  // ),
                  subtitle: _mediaFile( context ),
                ) :
                ListTile(
                  title: Text(
                    widget.post.sender + ', ' + (new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                        ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                    '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                        : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}'),
                    textAlign: TextAlign.start,
                  ),
                  subtitle: Text(widget.post.messageContent, textAlign: TextAlign.start,),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 42,
                            height: 42,
                            child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.senderPhoto}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                  // trailing: Text(
                  //   new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                  //       ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                  //   '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                  //       : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}',
                  //   textAlign: TextAlign.center,
                  // ),
                ),
              ),
            ]
        )
    );
  }

  Widget _mediaFile( BuildContext context ){
    if( widget.post.mediaType == ChatListItem.MEDIA_TYPE_IMAGE ){
      return GestureDetector(
        child: ClipRRect(
          borderRadius: BorderRadius.circular(24.0),
          child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.attachFile}?size=300', fit: BoxFit.cover, height: 250,),
        ),
        onTap: () async{
          if(widget.post.attachFile!=null){
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (context) => FullScreenWrapper(
                  imageProvider: NetworkImage( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.attachFile}', ),
                ),
              ),
            );
          }
        },
      );
    }else if( widget.post.mediaType ==ChatListItem.MEDIA_TYPE_FILE ){
      return ListTile(
        title: Text('Attachment', textAlign: widget.currentUser.guid==widget.post.senderGuid? TextAlign.end : TextAlign.start,),
        trailing: Icon(Icons.attach_file_sharp),
        // subtitle: LinearProgressIndicator(value: _progressVal / 100),
        onTap: (){
          // _prepare( );
          makeOpenFile( '${getBaseURL(widget.sharedPrefs)}/v1/files/download/${widget.post.attachFile}');
        },
      );
    }else if( widget.post.mediaType ==ChatListItem.MEDIA_TYPE_VOICE ){
      return ListTile(
        title: Text('Voice message', textAlign: widget.currentUser.guid==widget.post.senderGuid? TextAlign.end : TextAlign.start,),
        trailing: Icon(_mPlayer.isPlaying? Icons.adjust_sharp : Icons.play_circle_outline_sharp),
        onTap: (){
          if( _mPlayer.isPlaying)
            _stopPlayer();
          else
            _playVoiceMessage();
        },
      );
    }
  }

  @override
  void dispose() {
    if( widget.post.mediaType == ChatListItem.MEDIA_TYPE_FILE
        || widget.post.mediaType == ChatListItem.MEDIA_TYPE_VOICE) {
      _unbindBackgroundIsolate();
    }

    _stopPlayer();
    _mPlayer.closeAudioSession();
    _mPlayer = null;

    super.dispose();
  }

  void _bindBackgroundIsolate() {
    bool isSuccess = IsolateNameServer.registerPortWithName(
        _port.sendPort, 'downloader_send_port');
    if (!isSuccess) {
      _unbindBackgroundIsolate();
      _bindBackgroundIsolate();
      return;
    }
    _port.listen((dynamic data) {
      if (Constant.ENABLE_LOGS) {
        print('UI Isolate Callback: $data');
      }
      String id = data[0];
      DownloadTaskStatus status = data[1];
      int progress = data[2];

      if (_tasks != null && _tasks.isNotEmpty) {
        final task = _tasks.firstWhere((task) => task.taskId == id);
        if (task != null) {
          setState(() {
            task.status = status;
            task.progress = progress;
            _progressVal = progress.toDouble();
            if( status == DownloadTaskStatus.complete ){
              _openDownloadedFile( task );
            }
          });
        }
      }
    });
  }

  void _unbindBackgroundIsolate() {
    IsolateNameServer.removePortNameMapping('downloader_send_port');
  }

  static void downloadCallback(String id, DownloadTaskStatus status, int progress) {
    if (Constant.ENABLE_LOGS) {
      print(
          'Background Isolate Callback: task ($id) is in status ($status) and process ($progress)');
    }
    final SendPort send =
    IsolateNameServer.lookupPortByName('downloader_send_port');
    send.send([id, status, progress]);
  }

  void makeOpenFile( String url ) async{
    if (await canLaunch('$url')) {
      await launch('$url');
    } else {
      throw 'Could not launch $url';
    }
  }

  Future<Null> _prepare( ) async {
    final tasks = await FlutterDownloader.loadTasks();

    int count = 0;
    _tasks = [];
    _items = [];
    _TaskInfo _taskInfo = _TaskInfo(
        name: widget.post.attachFile,
        link: '${getBaseURL(widget.sharedPrefs)}/v1/files/download/${widget.post.attachFile}'
    );

    _tasks.add( _taskInfo );

    _items.add(_ItemHolder(name: 'Documents'));
    for (int i = count; i < _tasks.length; i++) {
      _items.add(_ItemHolder(name: _tasks[i].name, task: _tasks[i]));
      count++;
    }

    tasks?.forEach((task) {
      for (_TaskInfo info in _tasks) {
        if (info.link == task.url) {
          info.taskId = task.taskId;
          info.status = task.status;
          info.progress = task.progress;
        }
      }
    });

    _permissionReady = await _checkPermission();

    if( _permissionReady) {
      _localPath =
          (await _findLocalPath()) + Platform.pathSeparator + 'Downloads';
      final savedDir = Directory(_localPath);
      bool hasExisted = await savedDir.exists();
      if (!hasExisted) {
        savedDir.create();
      }
      setState(() {
        _isLoading = false;
      });

      _requestDownload( _taskInfo );
    }

  }

  Future<String> _findLocalPath() async {
    final directory = Platform.isAndroid? await getExternalStorageDirectory()
        : await getApplicationDocumentsDirectory();
    return directory.path;
  }

  Future<bool> _checkPermission() async {
    if (Platform.isAndroid) {
      final status = await Permission.storage.status;
      if (status != PermissionStatus.granted) {
        final result = await Permission.storage.request();
        if (result == PermissionStatus.granted) {
          return true;
        }
      } else {return true;}
    } else {return true;}
    return false;
  }

  void _requestDownload(_TaskInfo task) async {
    task.taskId = await FlutterDownloader.enqueue(
        url: task.link,
        // headers: {"auth": "test_for_sql_encoding"},
        savedDir: _localPath,
        showNotification: true,
        openFileFromNotification: true);
  }

  Future<bool> _openDownloadedFile(_TaskInfo task) {
    return FlutterDownloader.open(taskId: task.taskId);
  }


  sound.FlutterSoundPlayer _mPlayer = sound.FlutterSoundPlayer();
  Future<void> _playVoiceMessage(  ) async{
    if( !_mPlayer.isOpen() ){
      _mPlayer.openAudioSession().then((value) {
        setState(() {});
      });
    }
    if( _mPlayer.isOpen() ){
      _play();
    }
  }

  void _play() async {
    setState(() {});
    await _mPlayer.startPlayer(
        fromURI: '${getBaseURL(widget.sharedPrefs)}/v1/files/download/${widget.post.attachFile}',
        codec: sound.Codec.aacMP4,
        whenFinished: () {
          setState(() {});
        });
    setState(() {});
  }

  Future<void> _stopPlayer() async {
    if (_mPlayer != null) {
      await _mPlayer.stopPlayer().then((value) => {
        setState(() {})
      });
    }
  }

}
class _TaskInfo {
  final String name;
  final String link;

  String taskId;
  int progress = 0;
  DownloadTaskStatus status = DownloadTaskStatus.undefined;

  _TaskInfo({this.name, this.link});
}

class _ItemHolder {
  final String name;
  final _TaskInfo task;

  _ItemHolder({this.name, this.task});
}