
import 'dart:io';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/chat_latest_list_response.dart';
import 'package:qq_delivery/delivery/model/response/chat_search_list_response.dart';
import 'package:qq_delivery/delivery/model/response/comment_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/chat_list_item.dart';
import 'package:qq_delivery/delivery/model/shared/comment_item.dart';
import 'package:qq_delivery/delivery/model/shared/message_item.dart';
import 'package:qq_delivery/delivery/service/v2/chat_service.dart';
import 'package:qq_delivery/delivery/service/v2/comment_service.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:image/image.dart' as imglib;
import 'package:threading/threading.dart';

import 'chat_conversation_list.dart';
import 'chat_search.dart';


class ChatLatestList extends StatefulWidget {
  ChatLatestList({Key key, this.sharedPrefs, this.currentUser}) : super(key: key);
  final SharedPreferences sharedPrefs;
  final User currentUser;

  @override
  _CommentState createState() => _CommentState();
}

class _CommentState extends State<ChatLatestList>{

  ChatLatestListResponse _chatLatestListResponse;
  ChatService _chatService = new ChatService();
  List<ChatListItem> _items = [];
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  int _page = 0;
  bool _processing = false;
  final TextEditingController _controller_comment = new TextEditingController();


  @override
  void initState() {
    super.initState();
  }
  _getLoadData( bool refresh ) async{

    Map<String, dynamic> request = {
      'first': _page,
      'max':Constant.ROW_PER_PAGE
    };
    _chatLatestListResponse = await _chatService.latestList(request);

    if( _chatLatestListResponse!=null && _chatLatestListResponse.items!=null){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _chatLatestListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('Chat'),
            actions: <Widget>[
              IconButton(
                icon: const Icon(Icons.search),
                tooltip: 'Search',
                onPressed: () async{
                  SearchListItem searchListItem = await Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => ChatSearch(title: 'Search a person and chat', ),
                    ),
                  );

                  if( searchListItem != null && searchListItem.type =='person' ) {
                    setState(() {
                      _processing = true;
                    });
                    Map<String, dynamic> request = {
                      'sessionType':'Individual',
                      'memberGuid': searchListItem.guid
                    };

                    Map<String, dynamic> result = await _chatService.create( request );
                    if( result['success'] ){
                      // creating default message
                      MessageItem commentItem = new MessageItem(
                          userGuid:null,
                          messageContent: 'Say hello!',
                          attachFile: null,
                          sessionType: 'Individual',
                          sessionGuid: result['data'],
                          mediaType: 'Text'
                      );
                      await _chatService.sendMessage(commentItem.toMap());

                      // opening chat window
                      await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => ChatConversationList(
                            sharedPrefs: widget.sharedPrefs,
                            sessionGuid: result['data'],
                            sessionType: 'Individual',
                            currentUser: widget.currentUser,
                            title: searchListItem.name,
                          ),
                        ),
                      );
                      _refreshController.requestRefresh();
                    }else{
                      showMessage("Session creating failed!", context);
                      setState(() {
                        _processing = false;
                      });
                    }

                    setState(() {
                      _processing = false;
                    });
                  }else if( searchListItem != null && searchListItem.type =='group' ) {
                    setState(() {
                      _processing = true;
                    });
                    Map<String, dynamic> request = {
                      'memberGuid':widget.currentUser.guid,
                      'sessionGuid': searchListItem.sessionGuid
                    };

                    Map<String, dynamic> result = await _chatService.addMember( request );
                    if( result['success'] ){

                      MessageItem commentItem = new MessageItem(
                          userGuid:null,
                          messageContent: '${widget.currentUser.name} is joining this group',
                          attachFile: null,
                          sessionType: 'Group',
                          sessionGuid: searchListItem.sessionGuid,
                          mediaType: 'Text'
                      );
                      await _chatService.sendMessage(commentItem.toMap());

                      _refreshController.requestRefresh();
                    }

                    setState(() {
                      _processing = false;
                    });
                  }
                },
              ),
              IconButton(
                icon: const Icon(Icons.group_add_sharp),
                tooltip: 'Add group',
                onPressed: () async{

                  String sessionGuid = await Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (context) => CreateGroup(currentUser: widget.currentUser,),
                    ),
                  );

                  if( sessionGuid != null && sessionGuid.isNotEmpty && sessionGuid.length>10){
                    // creating default message
                    MessageItem commentItem = new MessageItem(
                        userGuid:null,
                        messageContent: widget.currentUser.name +' created the group!',
                        attachFile: null,
                        sessionType: 'Group',
                        sessionGuid: sessionGuid,
                        mediaType: 'Text'
                    );
                    await _chatService.sendMessage( commentItem.toMap() );

                    // opening chat window
                    await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => ChatConversationList(
                          sharedPrefs: widget.sharedPrefs,
                          sessionGuid: sessionGuid,
                          sessionType: 'Group',
                          currentUser: widget.currentUser,
                          title: 'New group',
                        ),
                      ),
                    );
                    _refreshController.requestRefresh();

                  }
                },
              ),
            ],
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      shrinkWrap: true,
      reverse: false,
      padding: EdgeInsets.only(left: 4, right: 4),
      itemBuilder: (c, i) => CommentListItem(
        post: _items[i],
        sharedPrefs: widget.sharedPrefs,
        currentUser: widget.currentUser,
        onChange: (){
          _refreshController.requestRefresh();
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.1,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

  class CommentListItem extends StatefulWidget {

    CommentListItem({Key key, @required this.post, this.sharedPrefs,
    this.currentUser, this.onChange}) : super(key: key);
    final ChatListItem post;
    final SharedPreferences sharedPrefs;
    final User currentUser;
    final Function onChange;

    @override
    _CommentListItemState createState() => _CommentListItemState();

  }

class _CommentListItemState extends State<CommentListItem> {
  @override
  void initState() {
    super.initState();
  }
  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 0,
                  child: ListTile(
                          title: Text(
                            widget.post.sessionType=='Group'? widget.post.owner
                                : (widget.currentUser.guid==widget.post.senderGuid? widget.post.receiver: widget.post.sender),
                          ),
                          subtitle: Text(widget.post.messageContent),
                          leading: _getAvatar(),
                          trailing: Text(
                            new DateFormat('dd-MM-yyyy').format(widget.post.createOn.toLocal())
                                ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                            '${new DateFormat('hh:mm a').format(widget.post.createOn.toLocal())}'
                                : '${new DateFormat('dd/MM/yy').format(widget.post.createOn.toLocal())}',
                            textAlign: TextAlign.center,
                          ),
                    onTap: () async{
                      await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => ChatConversationList(
                            sharedPrefs: widget.sharedPrefs,
                            sessionGuid: widget.post.sessionGuid,
                            sessionType: widget.post.sessionType,
                            currentUser: widget.currentUser,
                            groupGuid: widget.post.ownerGuid,
                            title: widget.post.sessionType=='Group'? widget.post.owner
                                : (widget.currentUser.guid==widget.post.senderGuid? widget.post.receiver: widget.post.sender),
                          ),
                        ),
                      );
                      widget.onChange();
                    },
                  ),
              ),
            ]
        )
    );
  }

  Widget _getAvatar(){
    if( widget.post.sessionType=='Group'){
      if( widget.post.ownerPhoto !=null && widget.post.ownerPhoto.isNotEmpty && widget.post.ownerPhoto.length>10){
        return ClipOval(
            child: Material(
              elevation: 1,
              color: Colors.transparent, // button color
              child: InkWell(
                // splashColor: Colors.red, // inkwell color
                child: SizedBox(
                  width: 42,
                  height: 42,
                  child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.post.ownerPhoto}', width: 42, height: 42, ),
                ),
              ),
            )
        );
      }else{
        return Icon(Icons.group_rounded);
      }
    }else{
      if( (widget.post.senderPhoto !=null && widget.post.senderPhoto.isNotEmpty && widget.post.senderPhoto.length>10)
      || (widget.post.receiverPhoto !=null && widget.post.receiverPhoto.isNotEmpty && widget.post.receiverPhoto.length>10)){

        if( widget.currentUser.guid==widget.post.senderGuid){
          return ClipOval(
              child: Material(
                elevation: 1,
                color: Colors.transparent, // button color
                child: InkWell(
                  // splashColor: Colors.red, // inkwell color
                  child: SizedBox(
                    width: 42,
                    height: 42,
                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.post.receiverPhoto}', width: 42, height: 42, ),
                  ),
                ),
              )
          );
        }else{
          return ClipOval(
              child: Material(
                elevation: 1,
                color: Colors.transparent, // button color
                child: InkWell(
                  // splashColor: Colors.red, // inkwell color
                  child: SizedBox(
                    width: 42,
                    height: 42,
                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.post.senderPhoto}', width: 42, height: 42, ),
                  ),
                ),
              )
          );
        }
      }else{
        return ClipOval(
            child: Material(
              elevation: 1,
              child: InkWell(
                // splashColor: Theme.of(context).backgroundColor,
                child: new Container(
                  // width: 60.0,
                  // height: 60.0,
                  padding: const EdgeInsets.all(12.0),
                  decoration: new BoxDecoration(
                    shape: BoxShape.circle,
                    // borderRadius: new BorderRadius.circular(30.0),
                    // color: Colors.red,
                  ),
                  child: new Text(
                    widget.post.owner[0],
                    // style: Theme.of(context).textTheme.headline5,
                  ),
                ),
              ),
            )
        );
      }

    }
  }
}


class CreateGroup extends StatefulWidget {
  CreateGroup({Key key, this.currentUser}) : super(key: key);
  final User currentUser;

  @override
  _CreateGroupState createState() => _CreateGroupState( );
}

class _CreateGroupState extends State<CreateGroup> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  String _code;
  bool _codeSent = false;
  int _resendCount = 0;
  IconData _topIcon = Icons.group_rounded;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            backgroundColor: Theme.of(context).primaryColor,
            key: _scaffoldKey,
            appBar: AppBar(
//            backgroundColor: Colors.transparent,
              title: Text('Groups'),
              bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            ),
            body: Stack(
                fit: StackFit.passthrough,
                children: [
                  // _backgroundPanel( true ),
                  Center(
                      child: SingleChildScrollView(
                          child: Column(
                              children: <Widget>[
                                Container(
                                    padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                    child:Card(
                                      // color: Color.fromRGBO(250, 250, 250, .4),
                                      elevation: 1,
                                      child: Column(
                                          children: <Widget>[
                                            new Container(
                                              margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                              child: Text('Create a group',
                                                  style: TextStyle(
                                                    // color: Theme.of(context).primaryColor,
                                                      fontWeight: FontWeight.bold,
                                                      fontSize: 18
                                                  )),
                                            ),
                                            Container(
                                              padding: const EdgeInsets.all(5.0),
                                              child: ClipOval(
                                                  child: Material(
                                                    elevation: 8,
                                                    color: Colors.white, // button color
                                                    child: InkWell(
                                                      // splashColor: Colors.red, // inkwell color
                                                      child: SizedBox(width: 150,
                                                        height: 150,
                                                        child: Icon( _topIcon, size: 100,
                                                          // color: Theme.of(context).primaryColor,
                                                        ),
                                                      ),
                                                    ),
                                                  )
                                              ),
                                            ),
                                            new Container(
                                              margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                              child: formData(context),
                                            ),
                                          ]
                                      ),
                                    )
                                ),
                              ]
                          )
                      )
                  )
                ]
            )
        )
    );
  }

  Form formData(BuildContext context ){
    return new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.done),
                  hintText: 'e.g. Customer support',
                  labelText: 'Group name',
                ),
                validator: (val) => val.isEmpty ? 'Code required' : null,
                onSaved: (val) => _code = val,
                keyboardType: TextInputType.text,
                // maxLength: 128,
                autofocus: true,
              ),
              // Container(
              //   margin: const EdgeInsets.only(left: 20.0, right: 5.0, top: 10),
              //   child:Text(
              //     "Code already sent to your mobile phone via SMS. If you didn't receive yet, press again on \"Resend\" button.",
              //     style: new TextStyle(fontSize: 12.0,
              //       // color: Colors.black87
              //     ),
              //   ),
              // ),
              new Wrap(
                  children: [
                    Container(
                      padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                      child: RaisedButton.icon(
                        onPressed:_processing?null: () {_submitForm(context); },
                        elevation: 1,
                        icon: Icon(Icons.done,
                          color: Colors.white
                        ),
                        label: Text(
                          'Create',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                  ]
              ),
            ])
    );
  }

  void _submitForm( BuildContext context ) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Group name required!', context);
      return;
    }
    form.save();

    if( _processing ) return;
    setState(() {
      _processing = true;
    });

    Map<String, dynamic> request = {
      'sessionType':'Group',
      'groupName': _code,
      'memberGuid':widget.currentUser.guid
    };

    ChatService _chatService = new ChatService();
    Map<String, dynamic> result = await _chatService.create( request );
      setState(() {
        _processing = false;
      });

    if( result['success'] ){
      setState(() {
        _topIcon = Icons.done;
        showMessage("$_code has been created.", context);
        new ThreadTimer(new Duration(milliseconds: 3000), () {
          Navigator.pop( context, result['data'] );
        });
      });
    }else{
      showMessage("Group creating failed!", context);
      setState(() {
        _processing = false;
      });
    }

  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}