import 'dart:async';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';



class DeliveryContactUpdate extends StatefulWidget {
  DeliveryContactUpdate({
    Key key, this.onChange,
    this.contactItem,
    this.deliveryContentService,
    this.accessToken
  }) : super(key: key);
  final Function onChange;
  final DeliveryContactItem contactItem;
  final DeliveryContentServiceV2 deliveryContentService;
  final String accessToken;

  @override
  _DeliveryContactUpdateState createState() => _DeliveryContactUpdateState();
}

class _DeliveryContactUpdateState extends State<DeliveryContactUpdate> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  DateTime _date;
  // DeliveryContactItem _contactItem;
  DeliveryContentServiceV2 _deliveryContentService;
  String _accessToken;

  final TextEditingController _controller_name = new TextEditingController();
  final TextEditingController _controller_date = new TextEditingController();
  final TextEditingController _controller_address = new TextEditingController();
  final TextEditingController _controller_street = new TextEditingController();
  final TextEditingController _controller_primaryPhone = new TextEditingController();
  final TextEditingController _controller_otherPhone = new TextEditingController();
  final TextEditingController _controller_email = new TextEditingController();
  final TextEditingController _controller_remark = new TextEditingController();

  @override
  void initState() {
    super.initState();

    // _contactItem = widget.contactItem;
    _deliveryContentService = widget.deliveryContentService;
    _accessToken = widget.accessToken;

    _controller_name.text = widget.contactItem.name;
    _controller_date.text = new DateFormat('yyyy-MM-dd').format(widget.contactItem.date);
    _date = widget.contactItem.date;
    _controller_address.text = widget.contactItem.address;
    _controller_street.text = widget.contactItem.street;
    _controller_primaryPhone.text = widget.contactItem.primaryPhone;
    _controller_otherPhone.text = widget.contactItem.otherPhones;
    _controller_email.text = widget.contactItem.email;
    _controller_remark.text = widget.contactItem.comment;
  }

  @override
  Widget build(BuildContext context){
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(
                widget.contactItem.contactType==Constant.CONTACT_TYPE_PICKUP?'Edit pickup information' : 'Edit drop off information',
                style: TextStyle(color: Colors.white)
            ),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
            body: ListView(
              children: <Widget>[
                Container(
                    margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
                    child: Card(
                        elevation: 1,
                        color: Colors.white,
                        child: new Form(
                            key: _formKey,
                            autovalidateMode: AutovalidateMode.onUserInteraction,
                            child: new Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisSize: MainAxisSize.min,
                              children: <Widget>[
                                ListTile(
                                  title: Text(
                                    widget.contactItem.contactType==Constant.CONTACT_TYPE_PICKUP?'Edit pickup information' : 'Edit drop off information',
                                    style: Theme.of(context).textTheme.headline6,
                                  ),
                                  // subtitle: Text("Complete your delivery process or try again later", style: TextStyle(color: Colors.white)),
                                  // trailing: Icon(Icons.done_all, color: Colors.white),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Name',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    validator: (val) => val.isEmpty ? 'Name required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_name,
                                    keyboardType: TextInputType.text,
                                  ),
                                ),
                                Container(
                                  margin: const EdgeInsets.all(16),
                                  child:new Row(children: <Widget>[
                                    new Expanded(
                                        child: new TextFormField(
                                          // style: TextStyle(
                                          //   color: Colors.white,
                                          // ),
                                          decoration: new InputDecoration(
                                            hintText: 'yyyy-mm-dd',
                                            labelText: 'Date',
                                            // hintStyle: TextStyle( color: Colors.white60),
                                            // enabledBorder: UnderlineInputBorder(
                                            //   borderSide: BorderSide(color: Colors.white60),
                                            // ),
                                            // focusedBorder: UnderlineInputBorder(
                                            //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                            // ),
                                            // labelStyle: TextStyle(
                                            //     color: Colors.white
                                            // ),
                                          ),
                                          controller: _controller_date,
                                          keyboardType: TextInputType.datetime,
                                        )
                                    ),
                                    new IconButton(
                                      icon: new Icon(Icons.calendar_today, color: Colors.blueGrey[600]),
                                      tooltip: 'Choose date',
                                      onPressed: (() {
                                        _chooseDate(context, _controller_date.text);
                                      }),
                                    )
                                  ]),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Address',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    validator: (val) => val.isEmpty ? 'Address required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_address,
                                    keyboardType: TextInputType.multiline,
                                    maxLines: null,
                                    minLines: 2,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Street',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    validator: (val) => val.isEmpty ? 'Street required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_street,
                                    keyboardType: TextInputType.text,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      // labelText: 'Township (Not editable)',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    // validator: (val) => val.isEmpty ? 'Street required!' : null,
                                    // onSaved: (val) => _name = val,
                                    initialValue: widget.contactItem.township,
                                    // controller: _controller_street,
                                    // keyboardType: TextInputType.number,
                                    enabled: false,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Primary phone',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    validator: (val) => val.isEmpty ? 'Primary phone required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_primaryPhone,
                                    keyboardType: TextInputType.phone,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Other phones',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    // validator: (val) => val.isEmpty ? 'Other phone required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_otherPhone,
                                    keyboardType: TextInputType.phone,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Email address',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    // validator: (val) => val.isEmpty ? 'Other phone required!' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_email,
                                    keyboardType: TextInputType.emailAddress,
                                  ),
                                ),
                                Container(
                                  padding: EdgeInsets.all(16),
                                  child: new TextFormField(
                                    // style: TextStyle(
                                    //   color: Colors.white,
                                    // ),
                                    decoration: const InputDecoration(
                                      // icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Remark',
                                      // hintStyle: TextStyle( color: Colors.white60),
                                      // enabledBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60),
                                      // ),
                                      // focusedBorder: UnderlineInputBorder(
                                      //   borderSide: BorderSide(color: Colors.white60, width: 2),
                                      // ),
                                      // labelStyle: TextStyle(
                                      //     color: Colors.white
                                      // ),
                                    ),
                                    // validator: (val) => val.isEmpty ? 'Remark required' : null,
                                    // onSaved: (val) => _name = val,
                                    // initialValue: '',
                                    controller: _controller_remark,
                                    keyboardType: TextInputType.multiline,
                                    maxLines: null,
                                    minLines: 2,
                                  ),
                                ),
                                ButtonBar(
                                  children: <Widget>[
                                    RaisedButton.icon(
                                      elevation: 0.0,
                                      icon: Icon(Icons.done_all, color: Colors.white),
                                      color: Theme.of(context).primaryColor,
                                      onPressed: _processing==null? null : ()async{
                                        await _update();
                                      },
                                      label: Text(
                                        'Update',
                                        style: TextStyle(
                                          fontSize: 16,
                                          fontWeight: FontWeight.w400,
                                          color: Colors.white,
                                        ),
                                      ),
                                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                    ),
                                    RaisedButton.icon(
                                      elevation: 0.0,
                                      icon: Icon(Icons.close, color: Colors.white),
                                      color: Colors.blueGrey[600],
                                      onPressed: _processing? null : ()async{
                                        Navigator.pop(context);
                                      },
                                      label: Text(
                                        "Cancel",
                                        style: TextStyle(
                                          fontSize: 16,
                                          fontWeight: FontWeight.w400,
                                          color: Colors.white,
                                        ),
                                      ),
                                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                    ),
                                  ],
                                )
                              ],
                            )
                        )
                    )
                )
              ],
            )
        )
    );

  }

  Future _chooseDate(BuildContext context, String initialDateString) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
        _controller_date.text = new DateFormat('yyyy-MM-dd').format(result);
        _date = result;
    });
  }

  Future<void> _update() async{
    if(!_formKey.currentState.validate())
      return null;

    setState(() {
      _processing = true;
    });

    widget.contactItem.name = _controller_name.text;
    widget.contactItem.date = _date;
    widget.contactItem.address = _controller_address.text;
    widget.contactItem.street = _controller_street.text;
    widget.contactItem.primaryPhone = _controller_primaryPhone.text;
    widget.contactItem.otherPhones = _controller_otherPhone.text;
    widget.contactItem.email = _controller_email.text;
    widget.contactItem.comment = _controller_remark.text;

    Map<String, dynamic> resultData = await _deliveryContentService.contactUpdate( _accessToken, widget.contactItem );

    if( resultData != null) {
      if (resultData["success"]) {
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
        widget.onChange();
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }
  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  // void showMessage( String title ){
  //   final snackBar = SnackBar(
  //     content: Text(title),
  //     duration: const Duration(seconds: 3),
  //     // action: SnackBarAction(
  //     //   label: 'OK',
  //     //   onPressed: () {
  //     //     // Some code to undo the change.
  //     //   },
  //     // ),
  //   );
  //   Scaffold.of(context).showSnackBar(snackBar);
  //   setState(() {});
  // }

  void showMessage(String message, [MaterialColor color = Colors.orange]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar( content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  Future<bool> _onWillPop() async {
    return true;
  }
}