import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:device_info/device_info.dart';
import 'package:flutter/services.dart';
import 'package:flutter_html_to_pdf/flutter_html_to_pdf.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:image_picker/image_picker.dart';
import 'package:location/location.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_completed_request.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/comment_list_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/route_list_by_tracking_id_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/comment_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/warehouse_station_route_service.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:qq_delivery/delivery/view/delivery2/comment.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:qq_delivery/delivery/view/utils/printer_discover.dart';
import 'package:qq_delivery/delivery/view/utils/remote_printer_discover.dart';
import 'package:qq_delivery/delivery/view/utils/signature_pad.dart';
import 'package:qq_delivery/delivery/view/utils/sunmi_print.dart';
import 'package:share_plus/share_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'delivery_contact_update.dart';
import 'delivery_map.dart';
import 'package:image/image.dart' as imglib;



class DeliveryDetail extends StatefulWidget {
  DeliveryDetail({Key key, this.onChange, this.wayId}) : super(key: key);
  final Function onChange;
  final String wayId;
  @override
  _DeliveryDetailState createState() => _DeliveryDetailState();

}

class _DeliveryDetailState extends State<DeliveryDetail> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  String _accessToken;

  User _currentUser = new User();
  UserService _userService = new UserService();
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  DeliveryDetailResponse _detailResponse;
  DeliveryContactItem _contactItemOS;
  DeliveryContactItem _contactItemCustomer;
  DeliveryContactItem _contactItemPickupPerson;
  DeliveryContactItem _contactItemDropOffPerson;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  bool _pickupEditable = false;
  bool _dropOffEditable = false;
  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  ActiveDeliverymanResponse _deliverymanResponse;
  DeliveryForDeliveryItem _employeeSearchItemAll;
  PersonService _personService = new PersonService();
  String _employeeGuid;
  bool _transfer = false;
  bool _firstLoad = true;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  bool _requiredToRefund = false;
  double _refundAmount;
  String _signature;
  String _assigneeSign;
  int _totalComments = 0;
  RouteListByTrackingIdResponse _routeListByTrackingIdResponse;
  Polyline _polyline;
  PolylineId _polylineId;
  bool _fullCollected = true;
  bool _showReason = false;
  String _reasonOfReduceAmount= "Transfer";
  String _nonCashTransferTo= "Office";

  @override
  void initState() {
    super.initState();
    // _loadData();
  }

  _loadData() async{
    setState(() {
      _processing = true;
    });

    if( _firstLoad ) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
    }

    _detailResponse = await _deliveryContentService.detail(widget.wayId);
    _detailResponse.contacts.forEach((element) {
      if( element.contactType== Constant.CONTACT_TYPE_PICKUP){
        _contactItemOS = element;
      }else if( element.contactType== Constant.CONTACT_TYPE_DROP_OFF){
        _contactItemCustomer = element;
      }else if( element.contactType== Constant.CONTACT_TYPE_PICKUP_EMPLOYEE){
        _contactItemPickupPerson = element;
      }else if( element.contactType== Constant.CONTACT_TYPE_DROP_OFF_EMPLOYEE){
        _contactItemDropOffPerson = element;
      }
    });
    if(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains('MANAGER_WRITE') &&
        ( _detailResponse.status == Constant.STATUS_PICKUP_CREATED
        || _detailResponse.status == Constant.STATUS_PICKUP_SCHEDULED
        || _detailResponse.status == Constant.STATUS_RETRY) ){
      _pickupEditable = true;
    }

    if(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains('MANAGER_WRITE') &&
        ( _detailResponse.status == Constant.STATUS_CREATED
        || _detailResponse.status == Constant.STATUS_ASSIGNED
        || _detailResponse.status == Constant.STATUS_PICKUP_CREATED
        || _detailResponse.status == Constant.STATUS_PICKUP_SCHEDULED
        || _detailResponse.status == Constant.STATUS_PICKUP_ON_WAY
        || _detailResponse.status == Constant.STATUS_RETRY ) ){
      _dropOffEditable = true;
    }

    if(_authorityResponse.authorities!=null ) {
      _accessToken = await _userService.getToken();
      _employeeSearchResponse =
      await _deliveryContentService.availableEmployeesByZone(
          _detailResponse.guid, _accessToken);
    }
    if(_authorityResponse.authorities !=null
        && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))){
      _deliverymanResponse = await _personService.activeDeliveryman();
    }

    if( _detailResponse != null ){
      double itemPrice = 0;
      double deliCharge = 0;
      double deliChargeForShow = 0;
      double promo = 0;
      if( _detailResponse.items[0].priceOption == 'Unpaid')
        itemPrice = _detailResponse.items[0].price??0;

      if( _detailResponse.items[0].chargeOption == 'Paid') {
        deliCharge = _detailResponse.items[0].charges ?? 0;
      }
      deliChargeForShow = _detailResponse.items[0].forShowCharges ?? 0;
      double chargeDiff = deliChargeForShow - _detailResponse.items[0].charges ?? 0;

      promo = _detailResponse.promotion??0;
      _refundAmount = (itemPrice + promo + chargeDiff) - deliCharge;
    }

    CommentService _commentService = new CommentService();
    CommentListResponse _commentListResponse = await _commentService.list(_accessToken, 'deliver', _detailResponse.guid, 0, 1000000);
    _totalComments = _commentListResponse.totalRecords;

    try{
      WarehouseStationRouteService warehouseStationRouteService = new WarehouseStationRouteService();
      _routeListByTrackingIdResponse = await warehouseStationRouteService.routeListByDelivery(
          widget.wayId,
          _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')
      );

      if( _routeListByTrackingIdResponse !=null && _routeListByTrackingIdResponse.items!=null ){
        RouteListItem routeListItem = _routeListByTrackingIdResponse.items.last;
        if( routeListItem.carRoutePoints.isNotEmpty ){
          _polylineId = PolylineId("last route");
          _polyline = Polyline(
            polylineId: _polylineId,
            consumeTapEvents: true,
            color: Colors.orange,
            width: 12,
            points: routeListItem.carRoutePoints,
            // onTap: () {
            //   _onPolylineTapped(polylineId);
            // },
          );
          print('Total size=${routeListItem.carRoutePoints.length}');
        }
      }
    }catch(e){}

    if( _firstLoad ) {
      if( _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
          && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _detailResponse !=null
          && _detailResponse.status==Constant.STATUS_ON_WAY && _contactItemDropOffPerson.personGuid==_currentUser.guid){
        _initLocationService( );
      }
    }

    _firstLoad = false;
    setState(() {_processing=false;});

  }

  Future<void> generatePDFDocument( BuildContext context ) async {

    if( Constant.SHARE_MODE =='print') {
      setState(() {
        _processing = true;
      });
      var htmlContent = await _deliveryContentService.wayBillToPrint(
          widget.wayId, "4x6");
      setState(() {
        _processing = false;
      });

      Directory appDocDir = await getApplicationDocumentsDirectory();
      var targetPath = appDocDir.path;
      var targetFileName = "${widget.wayId}";

      var generatedPdfFile = await FlutterHtmlToPdf.convertFromHtmlContent(
          htmlContent, targetPath, targetFileName);
      String generatedPdfFilePath = generatedPdfFile.path;

      final box = context.findRenderObject() as RenderBox;
      await Share.shareFiles([generatedPdfFilePath],
          text: '${widget.wayId}',
          sharePositionOrigin: box.localToGlobal(Offset.zero) & box.size);
    }else{
      final box = context.findRenderObject() as RenderBox;
      await Share.share("${Constant.UI_DOMAIN_NAME_SHARE}${widget.wayId}",
          subject: 'Tracking link',
          sharePositionOrigin: box.localToGlobal(Offset.zero) & box.size);
    }
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(widget.wayId, style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.print,
                    // color: Colors.white,
                  ),
                  onPressed: () async{
                    if( _authorityResponse.authorities!=null
                        && (_authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                            || _contactItemOS.personGuid==_currentUser.guid)){

                      if( _detailResponse !=null && _contactItemOS!=null
                          && _contactItemCustomer !=null ){

                        if( Constant.SHARE_MODE =='remotePrint'){

                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) =>
                                    RemotePrinterDiscover(
                                      detailResponse: _detailResponse,
                                      contactItemOS: _contactItemOS,
                                      contactItemCustomer: _contactItemCustomer,
                                      sharedPrefs: _sharedPrefs,
                                    ),
                              )
                          );

                        }else {
                          if (Platform.isAndroid) {
                            DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
                            AndroidDeviceInfo androidInfo = await deviceInfo
                                .androidInfo;
                            if (androidInfo.manufacturer.toLowerCase() ==
                                'sunmi' || androidInfo.brand.toLowerCase() ==
                                'sunmi') {
                              SunmiPrint sunmiPrint = new SunmiPrint();
                              sunmiPrint.printSlip(
                                  _detailResponse, _contactItemOS,
                                  _contactItemCustomer, _sharedPrefs, context);
                            } else {
                              Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (BuildContext context) =>
                                        PrinterDiscover(
                                          detailResponse: _detailResponse,
                                          contactItemOS: _contactItemOS,
                                          contactItemCustomer: _contactItemCustomer,
                                          sharedPrefs: _sharedPrefs,
                                        ),
                                  )
                              );
                            }
                          } else {
                            Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (BuildContext context) =>
                                      PrinterDiscover(
                                        detailResponse: _detailResponse,
                                        contactItemOS: _contactItemOS,
                                        contactItemCustomer: _contactItemCustomer,
                                        sharedPrefs: _sharedPrefs,
                                      ),
                                )
                            );
                          }
                        }
                      }
                    }
                  },
                  tooltip: 'Print slip',
                ),
              ),
              Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.share_sharp,
                    // color: Colors.white,
                  ),
                  onPressed: () async{
                    generatePDFDocument(context);
                  },
                  tooltip: 'Print way bill',
                ),
              ),
            ],
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: false,
            child: ListView(
              children: <Widget>[
                _detailResponse!=null && _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_detailResponse.status==Constant.STATUS_PICKUP_CREATED || _detailResponse.status==Constant.STATUS_CREATED || _detailResponse.status==Constant.STATUS_RETRY)?
                  _deliveryAssign( context ) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_detailResponse.status==Constant.STATUS_PICKUP_SCHEDULED || _detailResponse.status==Constant.STATUS_ASSIGNED)?
                  _deliveryManagerTransfer( context ) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_detailResponse.status==Constant.STATUS_PICKUP_ON_WAY || _detailResponse.status==Constant.STATUS_ON_WAY)?
                  _deliveryManagerCancel( context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                    && ( (_detailResponse.status== Constant.STATUS_ASSIGNED && _contactItemDropOffPerson.personGuid==_currentUser.guid)
                        || (_detailResponse.status==Constant.STATUS_PICKUP_SCHEDULED && _contactItemPickupPerson.personGuid==_currentUser.guid) )?
                  _deliveryEmployeeAccept(context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _detailResponse !=null
                    && _detailResponse.status== Constant.STATUS_PICKUP_ON_WAY &&  _contactItemPickupPerson.personGuid==_currentUser.guid?
                    _deliveryEmployeePickupComplete(context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _detailResponse !=null
                && _detailResponse.status==Constant.STATUS_ON_WAY && _contactItemDropOffPerson.personGuid==_currentUser.guid
                && _detailResponse.totalCharges > 0?
                _cashOnDelivery(context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _detailResponse !=null
                    && _detailResponse.status==Constant.STATUS_ON_WAY && _contactItemDropOffPerson.personGuid==_currentUser.guid?
                _deliveryEmployeeComplete(context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('OS_WRITE')
                    && !_authorityResponse.authorities.contains('MANAGER_WRITE') && _detailResponse !=null
                    && (_detailResponse.status== Constant.STATUS_PICKUP_CREATED || _detailResponse.status== Constant.STATUS_PICKUP_SCHEDULED)
                    && _contactItemOS!=null && _currentUser.guid==_contactItemOS.personGuid?
                    _deliveryManagerCancel( context) : Container(),

                // _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('MANAGER_WRITE')
                //     && _detailResponse.status== Constant.STATUS_RETURN?
                // _deliveryReturnedCompleted( context) : Container( ),

                _detailResponse!=null
                    && _detailResponse.status != Constant.STATUS_COMPLETED
                    && _detailResponse.status != Constant.STATUS_CANCELED
                    && _detailResponse.status != Constant.STATUS_RETURN
                    && _contactItemDropOffPerson !=null? _deliveryTrackingMap(context): Container(),

                _deliverySummary( context ),

                _detailResponse!=null && _authorityResponse.authorities!=null?
                _deliveryPickup( context) : Container( ),

                _detailResponse!=null && _authorityResponse.authorities!=null?
                _deliveryDropOff( context) : Container( ),

                _detailResponse!=null && _detailResponse.items.length>0? _deliveryItemView(context, _detailResponse.items[0]): Container(),
                _detailResponse!=null && _detailResponse.items.length>1? _deliveryItemView(context, _detailResponse.items[1]): Container(),
                _detailResponse!=null && _detailResponse.items.length>2? _deliveryItemView(context, _detailResponse.items[2]): Container(),
                _detailResponse!=null && _detailResponse.items.length>3? _deliveryItemView(context, _detailResponse.items[3]): Container(),
                _detailResponse!=null && _detailResponse.items.length>4? _deliveryItemView(context, _detailResponse.items[4]): Container(),

                _detailResponse!=null && _routeListByTrackingIdResponse !=null && _routeListByTrackingIdResponse.currentStationContact !=null && _authorityResponse.authorities==null?
                _officeContact( context) : Container( ),

                _detailResponse!=null && _authorityResponse.authorities!=null?
                _comment(context) : Container(),
              ]
          ),
            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            // scroll down
            onRefresh: () {
              _loadData();
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
          ),
        )
    );
  }

  Widget _cashOnDelivery( BuildContext context ){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 16),
        child: Card(
            elevation: 1,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  subtitle: Text(
                    '${tr(LocaleKeys.lblCashToCollect)}',
                    // style: Theme.of(context).textTheme.headline6,
                  ),
                  title: Text('${NumberFormat('#,###').format(_detailResponse.totalCharges)} Ks'
                    // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  leading: Icon(Icons.attach_money_rounded,
                    // color: Colors.blueGrey[600]
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryTrackingMap( BuildContext context ){
    LatLng startLocation= Constant.OFFICE_LOCATION;
    LatLng officeLocation= Constant.OFFICE_LOCATION;
    bool startRoute = false;

    if( _routeListByTrackingIdResponse!=null
        && _routeListByTrackingIdResponse.currentStationContact!=null
    && _routeListByTrackingIdResponse.currentStationContact.latitude !=null
    && _routeListByTrackingIdResponse.currentStationContact.longitude !=null)
      officeLocation= LatLng( _routeListByTrackingIdResponse.currentStationContact.latitude, _routeListByTrackingIdResponse.currentStationContact.longitude);

    List<Map<String,dynamic>> items=[];
    String mapTitle = "Current way location";
    if( _detailResponse != null ){
      if( _detailResponse.status == Constant.STATUS_COMPLETED
      || _detailResponse.status == Constant.STATUS_CANCELED
      || _detailResponse.status == Constant.STATUS_RETURN){
        items = null;
        mapTitle = "No tracking available";
      } else if(_detailResponse.status == Constant.STATUS_PICKUP_CREATED
      || _detailResponse.status == Constant.STATUS_PICKUP_SCHEDULED ){
        startLocation= officeLocation;
        mapTitle = "Processing to pickup";
        if( _contactItemOS.latitude != null && _contactItemOS.longitude !=null) {
          items.add({
            'latitude': _contactItemOS.latitude,
            'longitude': _contactItemOS.longitude,
            'guid': _contactItemOS.guid,
            'name': _contactItemOS.name,
            'phone': "${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
            'employee': false
          });
        }
        items.add({
          'latitude': officeLocation.latitude,
          'longitude': officeLocation.longitude,
          'guid': 'quickq-delivery',
          'name': _detailResponse.status == Constant.STATUS_PICKUP_CREATED?'Office' : _contactItemPickupPerson.name,
          'phone': _detailResponse.status == Constant.STATUS_PICKUP_CREATED?Constant.OFFICE_ADDRESS : _contactItemPickupPerson.primaryPhone,
          'employee': true
        });
      }else if(_detailResponse.status == Constant.STATUS_PICKUP_ON_WAY ){
        startLocation= officeLocation;
        mapTitle = "Pickup on way";
        if( _contactItemOS.latitude != null && _contactItemOS.longitude !=null) {
          items.add({
            'latitude': _contactItemOS.latitude,
            'longitude': _contactItemOS.longitude,
            'guid': _contactItemOS.guid,
            'name': _contactItemOS.name,
            'phone': "${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
            'employee': false
          });
        }
        if( _contactItemPickupPerson.latitude != null && _contactItemPickupPerson.longitude !=null) {
          startLocation= LatLng(_contactItemPickupPerson.latitude, _contactItemPickupPerson.longitude);
          items.add({
            'latitude': _contactItemPickupPerson.latitude,
            'longitude': _contactItemPickupPerson.longitude,
            'guid': _contactItemPickupPerson.guid,
            'name': _contactItemPickupPerson.name,
            'phone': "${_contactItemPickupPerson.primaryPhone}",
            'employee': true
          });
        }
      } else if(_detailResponse.status == Constant.STATUS_CREATED
          || _detailResponse.status == Constant.STATUS_ASSIGNED ){
        startLocation= officeLocation;
        mapTitle = "Processing to deliver";
        if( _contactItemCustomer.latitude != null && _contactItemCustomer.longitude !=null) {
          items.add({
            'latitude': _contactItemCustomer.latitude,
            'longitude': _contactItemCustomer.longitude,
            'guid': _contactItemCustomer.guid,
            'name': _contactItemCustomer.name,
            'phone': "${_contactItemCustomer.address??''}, ${_contactItemCustomer.street??''}, ${_contactItemCustomer.township??''}",
            'employee': false
          });
        }
        items.add({
          'latitude': officeLocation.latitude,
          'longitude': officeLocation.longitude,
          'guid': 'quickq-delivery',
          'name': _detailResponse.status == Constant.STATUS_CREATED? 'Office' : _contactItemDropOffPerson.name,
          'phone': _detailResponse.status == Constant.STATUS_CREATED? Constant.OFFICE_ADDRESS : _contactItemDropOffPerson.primaryPhone,
          'employee': true
        });
      } else if(_detailResponse.status == Constant.STATUS_ON_WAY ){
        startLocation= officeLocation;
        mapTitle = "Deliver on way";
        if( _contactItemCustomer.latitude != null && _contactItemCustomer.longitude !=null) {
          items.add({
            'latitude': _contactItemCustomer.latitude,
            'longitude': _contactItemCustomer.longitude,
            'guid': _contactItemCustomer.guid,
            'name': _contactItemCustomer.name,
            'phone': "${_contactItemCustomer.address??''}, ${_contactItemCustomer.street??''}, ${_contactItemCustomer.township??''}",
            'employee': false
          });
          if( _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') )
            startRoute = true;
        }
        if( _contactItemDropOffPerson.latitude != null && _contactItemDropOffPerson.longitude !=null) {
          startLocation= LatLng(_contactItemDropOffPerson.latitude, _contactItemDropOffPerson.longitude);
          items.add({
            'latitude': _contactItemDropOffPerson.latitude,
            'longitude': _contactItemDropOffPerson.longitude,
            'guid': _contactItemDropOffPerson.guid,
            'name': _contactItemDropOffPerson.name,
            'phone': "${_contactItemDropOffPerson.primaryPhone}",
            'employee': true
          });
        }
      }
    }
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${_authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                    && !_authorityResponse.authorities.contains('MANAGER_READ')?tr(LocaleKeys.lblDeliveryGPSNavigationToCustomer):tr(LocaleKeys.lblLiveTracking)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                  onTap: (){
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => TrackingMap(
                            startLocation: startLocation,
                            items: items,
                            title: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')? tr(LocaleKeys.lblDeliveryGPSNavigationToCustomer) : mapTitle,
                            isEmployee: false,
                            polyline: _polyline,
                            polylineId: _polylineId,
                            contactItemCustomer: _contactItemCustomer,
                            navigate: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE'),
                            startRoute: startRoute,
                          ),
                        )
                    );
                  },
                ),
                Container(
                  height: 250,
                  width: MediaQuery.of(context).size.width,
                  child: TrackingMap(
                    startLocation: startLocation,
                    items: items,
                    title: null,
                    polyline: _polyline,
                    polylineId: _polylineId,
                  ),
                ),
              ],
            ),
        )
    ) : new Container();
  }

  Widget _deliverySummary( BuildContext context ){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblStatus)} ${_createStatus(_detailResponse.status)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('Last updated: '+ new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.updatedOn.toLocal()),
                      // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  trailing: Icon(Icons.local_shipping_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    'Created date: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.createdOn.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                      '${tr(LocaleKeys.lblWayID)}: ${_detailResponse.trackingId}',
                      style: TextStyle(
                        // color: Colors.black,
                          fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                      _detailResponse.status==Constant.STATUS_COMPLETED? "${tr(LocaleKeys.lblCashCollected)}: ${new NumberFormat("#,###").format(_detailResponse.totalCharges)} Ks" : "${tr(LocaleKeys.lblCashToCollect)}: ${new NumberFormat("#,###").format(_detailResponse.totalCharges)} Ks",
                      style: TextStyle(
                          // color: Colors.black,
                          fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblCashOnDelivery)} : ${_detailResponse.deliveryChargeOption=='Paid' && _detailResponse.itemPriceOption=='Paid'? 'No': 'Yes'}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),

                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                      '${tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethod)}: ${_getDeliveryMethod(_detailResponse.deliveryMethods)}',
                      // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //       '${tr(LocaleKeys.lblItemPrice)}: ${_detailResponse.itemPriceOption=='Unpaid'?tr(LocaleKeys.lblToCollectItemPriceFromRecipient):tr(LocaleKeys.lblItemPriceAlreadyPaidByRecipient)}',
                //       // style: TextStyle(color: Colors.black),
                //   ),
                // ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //       '${tr(LocaleKeys.lblStatus)} ${_createStatus(_detailResponse.status)}',
                //       // style: TextStyle(color: Colors.black),
                //   ),
                // ),
                _authorityResponse.authorities!=null
                    && (_authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                    || _contactItemOS.personGuid==_currentUser.guid)?
                Container(
                  padding: const EdgeInsets.only(bottom: 16),
                  child: ListTile(
                    title: Text(tr(LocaleKeys.lblPromotion),style: Theme.of(context).textTheme.headline6),
                    subtitle: Text('${new NumberFormat("#,###").format(_detailResponse.promotion)} Ks'),
                    leading: Icon(Icons.redeem_sharp,
                      // color: Theme.of(context).primaryColor,
                    ),
                  ),
                ) : Container(),
                _routeListByTrackingIdResponse!=null && _routeListByTrackingIdResponse.items!=null? Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 0),
                  child: Stepper(
                    steps: _stepList(  ),
                    currentStep: _routeListByTrackingIdResponse.active+1,
                    controlsBuilder: (BuildContext context,ControlsDetails details) => Container(),
                    type: StepperType.vertical,
                    physics: ClampingScrollPhysics(),
                  ),
                ) : Container(),
                _contactItemDropOffPerson !=null && _contactItemDropOffPerson.signature!=null?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 5.0, bottom: 32),
                  child: ListTile(
                    leading: _contactItemDropOffPerson.signature!=null?ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemDropOffPerson.signature}', width: 48, height: 48, ),
                            ),
                          ),
                        )
                    ): Icon(Icons.photo, color: Theme.of(context).primaryColor),
                    title: Text(
                      tr(LocaleKeys.lblDeliveryDetailPOD),
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.black
                    ),
                    onTap: () async{
                      if(_contactItemDropOffPerson.signature!=null){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => FullScreenWrapper(
                              imageProvider: NetworkImage( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemDropOffPerson.signature}', ),
                            ),
                          ),
                        );
                      }
                    },
                  ),
                ) : Container(),
                _contactItemCustomer !=null && _contactItemCustomer.signature!=null?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 5.0, bottom: 32),
                  child: ListTile(
                    leading: _contactItemCustomer.signature!=null?ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemCustomer.signature}', width: 48, height: 48, ),
                            ),
                          ),
                        )
                    ): Icon(Icons.photo, color: Theme.of(context).primaryColor),
                    title: Text(
                      tr(LocaleKeys.lblRecipientSignature),
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.black
                    ),
                    onTap: () async{
                      if(_contactItemCustomer.signature!=null){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => FullScreenWrapper(
                              imageProvider: NetworkImage( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemCustomer.signature}', ),
                            ),
                          ),
                        );
                      }
                    },
                  ),
                ) : Container(),
              ],
            )
        )
    ) : new Container();
  }

  String _getDeliveryMethod(String method){
    if( method=='Regular'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodRegular);
    }else if( method=='24hr'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin4hrs);
    }else if( method=='Same Day'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodSameDay);
    }else if( method=='Urgent'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodUrgent);
    }else if( method=='Within4hrs'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin4hrs);
    }else if( method=='Within1hr'){
      return tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin1hr);
    }else
      return method;
  }

  List _stepList(  ){
    List<Step> steps=[];
    Step start  = Step(
      isActive: true,
      state: _detailResponse.pickupOn!=null? StepState.complete : StepState.indexed,
      title: Text('Pickup'),
        subtitle: Text( _detailResponse.pickupOn !=null? new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.pickupOn.toLocal()) : ''),
        content: Container()
    );
    steps.add(start);

    int count = 0;
    _routeListByTrackingIdResponse.items.forEach((element) {
      Step route  = Step(
        isActive: count <= _routeListByTrackingIdResponse.active,
        state: count < _routeListByTrackingIdResponse.active? StepState.complete : _detailResponse.status=='Completed'? StepState.complete:StepState.indexed,
        title: Text('${element.deliverStationName}'),
        subtitle: Text( element.doneOn !=null? new DateFormat('dd/MM/yyyy hh:mm a').format(element.doneOn.toLocal()) : ''),
        content: Container()
      );
      steps.add(route);
      ++count;
    });

    return steps;
  }

  Widget _deliveryPickup( BuildContext context ){
    return _detailResponse!=null && _contactItemOS!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  leading: _contactItemPickupPerson!=null && _contactItemPickupPerson.avatar!=null?ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemPickupPerson.avatar}', width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ): Icon(Icons.person,
                      // color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    '${tr(LocaleKeys.lblPickupAlreadyInformation)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(
                      !_pickupEditable? "${_contactItemPickupPerson!=null? 'Pickup by: ' + _contactItemPickupPerson.name+ (_contactItemPickupPerson.primaryPhone!=null? '('+ _contactItemPickupPerson.primaryPhone +')' :'') : ''}"
                      :  "${_contactItemPickupPerson!=null? _contactItemPickupPerson.name+ (_contactItemPickupPerson.primaryPhone!=null? '('+ _contactItemPickupPerson.primaryPhone +')' :'')+' will pickup soon' : ''}",
                      // style: TextStyle(color: Colors.black)
                  ),
                  trailing: _pickupEditable? Icon(Icons.edit,
                      // color: Colors.black
                  ) : Icon(Icons.done,
                      // color: Colors.blueGrey[600]
                  ),
                  // onTap: !_pickupEditable? null : () async{
                  //   bool result = await Navigator.push(
                  //       context,
                  //       MaterialPageRoute(
                  //         builder: (BuildContext context) => DeliveryContactUpdate(
                  //           contactItem: _contactItemOS,
                  //           deliveryContentService: _deliveryContentService,
                  //           accessToken: _accessToken,
                  //           onChange: (){
                  //             widget.onChange();
                  //             setState(() {});
                  //           },
                  //         ),
                  //       )
                  //   );
                  // },
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    _detailResponse.pickupOn!=null? '${tr(LocaleKeys.lblDeliPickupDoneDate)}: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.pickupOn.toLocal())}'
                        :'${tr(LocaleKeys.lblDeliCreatePickupDate)}: ${new DateFormat('dd/MM/yyyy').format(_contactItemOS.date.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${_contactItemOS.name}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderMobile)}: ${_contactItemOS.primaryPhone}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderOtherPhones)}: ${_contactItemOS.otherPhones}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                _contactItemOS.comment!=null && _contactItemOS.comment.isNotEmpty?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    "${tr(LocaleKeys.lblRemark)}: ${_contactItemOS.comment??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ) : Container(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                  child: Text(
                    _contactItemOS.email!=null? '${tr(LocaleKeys.lblEmailAddress)}: ${_contactItemOS.email}' : '${tr(LocaleKeys.lblEmailAddress)}:',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryDropOff( BuildContext context ){
    return _detailResponse!=null && _contactItemCustomer!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  leading: _contactItemDropOffPerson!=null && _contactItemDropOffPerson.avatar!=null?ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemDropOffPerson.avatar}', width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ): Icon(Icons.person,
                      // color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    '${tr(LocaleKeys.lblDeliveryInformation)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(
                      !_dropOffEditable? "${_contactItemDropOffPerson!=null? 'Drop-off by: ' + _contactItemDropOffPerson.name+ (_contactItemDropOffPerson.primaryPhone!=null? '('+ _contactItemDropOffPerson.primaryPhone +')' :'') : ''}"
                          :  "${_contactItemDropOffPerson!=null? _contactItemDropOffPerson.name+ (_contactItemDropOffPerson.primaryPhone!=null? '('+ _contactItemDropOffPerson.primaryPhone +')' :'')+' will deliver soon' : ''}",
                      // style: TextStyle(color: Colors.black)
                  ),
                  trailing: _dropOffEditable? Icon(Icons.edit,
                      // color: Colors.black
                  ) : ( _detailResponse.status==Constant.STATUS_COMPLETED? Icon(Icons.done,
                      // color: Colors.blueGrey[600]
                  ) : Icon(Icons.get_app_sharp,
                      // color: Colors.white
                  )),
                  // onTap: !_dropOffEditable? null : () async{
                  //   bool result = await Navigator.push(
                  //       context,
                  //       MaterialPageRoute(
                  //         builder: (BuildContext context) => DeliveryContactUpdate(
                  //           contactItem: _contactItemCustomer,
                  //           deliveryContentService: _deliveryContentService,
                  //           accessToken: _accessToken,
                  //           onChange: (){
                  //             widget.onChange();
                  //             setState(() {});
                  //           },
                  //         ),
                  //       )
                  //   );
                  // },
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    _detailResponse.doneOn!=null? '${tr(LocaleKeys.lblDeliCreateDropOffDate)}: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.doneOn.toLocal())}'
                        :'${tr(LocaleKeys.lblDeliCreateDropOffDate)}: ${_contactItemCustomer.date!=null? new DateFormat('dd/MM/yyyy').format(_contactItemCustomer.date.toLocal()) : "--/--/----"}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientName)}: ${_contactItemCustomer.name}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientMobile)}: ${_contactItemCustomer.primaryPhone}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientOtherPhones)}: ${_contactItemCustomer.otherPhones}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${_contactItemCustomer.address??''}, ${_contactItemCustomer.street??''}, ${_contactItemCustomer.township??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                _contactItemCustomer.comment!=null && _contactItemCustomer.comment.isNotEmpty?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    "${tr(LocaleKeys.lblRemark)}: ${_contactItemCustomer.comment??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ) : Container(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                  child: Text(
                    _contactItemCustomer.email!=null? '${tr(LocaleKeys.lblEmailAddress)}: ${_contactItemCustomer.email}' : '${tr(LocaleKeys.lblEmailAddress)}:',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                //   child: Text(
                //     _contactItemCustomer.comment!=null? 'Remark: ${_contactItemCustomer.comment}' : 'Remark:',
                //     style: TextStyle(color: Colors.black),
                //   ),
                // ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _officeContact( BuildContext context ){
    return _routeListByTrackingIdResponse!=null && _routeListByTrackingIdResponse.currentStationContact!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    'Inquiry',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('Office contact'),
                  trailing: Icon(Icons.phone_in_talk_sharp,
                    // color: Colors.white
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)} ${_routeListByTrackingIdResponse.currentStationContact.address??''}, ${_routeListByTrackingIdResponse.currentStationContact.street??''}, ${_routeListByTrackingIdResponse.currentStationContact.townshipName??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblMobilePhone)} ${_routeListByTrackingIdResponse.currentStationContact.primaryPhone}, ${_routeListByTrackingIdResponse.currentStationContact.otherPhones??''}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                  child: Text(
                    _routeListByTrackingIdResponse.currentStationContact.email!=null? '${tr(LocaleKeys.lblEmailAddress)}  ${_routeListByTrackingIdResponse.currentStationContact.email}' : '${tr(LocaleKeys.lblEmailAddress)} ',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryItemView( BuildContext context, DeliveryItem item ){
    return _detailResponse!=null && item!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  leading: item.image!=null?ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.image}', width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ): Icon(Icons.photo, color: Theme.of(context).primaryColor),
                  title: Text(
                    tr(LocaleKeys.lblItemDetail),
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.black
                  ),
                  onTap: () async{
                    if(item.image!=null){
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => FullScreenWrapper(
                            imageProvider: NetworkImage( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.image}', ),
                          ),
                        ),
                      );
                    }
                  },
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblItemPrice)}: ${new NumberFormat("#,###").format(item.price)} Ks (${item.priceOption=='Paid'?'Already paid' :'Paid by recipient'})',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblDeliCharges)}: ${new NumberFormat("#,###").format(item.forShowCharges)} Ks (${item.chargeOption=='Paid'?'Already paid' :'Paid by recipient'})',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //     'Charge option: ${item.chargeOption}',
                //     style: TextStyle(color: Colors.black),
                //   ),
                // ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    "${tr(LocaleKeys.lblFragile)}: ${item.fragile? 'Yes':'No'}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblWeight)}: ${item.weight.toStringAsFixed(1)}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblWidth)}: ${item.width}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblHeight)}: ${item.height}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                  child: Text(
                    '${tr(LocaleKeys.lblLength)}: ${item.length}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //     "Insurance: ${item.insurance? 'Yes':'No'}",
                //     style: TextStyle(color: Colors.black),
                //   ),
                // ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                //   child: Text(
                //     'Price option: ${item.priceOption}',
                //     style: TextStyle(color: Colors.black),
                //   ),
                // ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _comment( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child:
          ListTile(
            title: Text(
              _totalComments==0?'Write a comment':'View $_totalComments more comments',
              style: Theme.of(context).textTheme.headline6,
            ),
            trailing: Icon(Icons.comment_bank,
                // color: Colors.blueGrey[600]
            ),
            onTap: (){
              Navigator.push(
                  context,
                  MaterialPageRoute(
                      builder: (BuildContext context) => Comment(
                        personGuid: _currentUser.guid,
                        type: 'deliver',
                        referenceGuid: _detailResponse.guid,
                        sharedPrefs: _sharedPrefs,
                        accessToken: _accessToken,
                      )
                  )
              );
            },
          ),
        )
    );
  }

  Widget _deliveryAssign( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
              key: _formKey,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              child: new Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblAssignToEmployee)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text("Assign to your deliveryman to ${_detailResponse.status==Constant.STATUS_PICKUP_CREATED?'pickup':'deliver'}",
                        // style: TextStyle(color: Colors.blueGrey[600])
                    ),
                    trailing: Icon(Icons.assignment_ind,
                        // color: Colors.blueGrey[600]
                    ),
                  ),
                  _deliverySelection(context),
                  _remarkTextField(),
                  ButtonBar(
                    children: <Widget>[
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.person_pin_circle_sharp,
                            // color: Colors.white
                        ),
                        color: Colors.green[700],
                        onPressed: _employeeGuid==null? null : ()async{
                          await _assign();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblAcceptAndAssign)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.cancel_schedule_send_sharp,
                            // color: Colors.white
                        ),
                        color: Colors.red[700],
                        onPressed: _processing? null : ()async{
                          await _cancel();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblCancelDelivery)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ],
                  )
                ],
              )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerTransfer( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblChangeEmployee)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Change your current assigned deliveryman with another",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.wifi_protected_setup_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _deliverySelection(context),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerCancel( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblCancelTheDelivery)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Cancel here if there have any issue",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.cancel_schedule_send_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryReturnedCompleted( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblReturnItemHandover)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text('${tr(LocaleKeys.lblReturnHandoverToMerchant)}',
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.flip_camera_android_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.flip_camera_android_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await returned();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblHandover)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_carNumber = new TextEditingController();
  Widget _deliveryEmployeeAccept( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        _detailResponse!=null && _detailResponse.status==Constant.STATUS_PICKUP_SCHEDULED?'${tr(LocaleKeys.lblAcceptToPickup)}':'${tr(LocaleKeys.lblAcceptToDeliver)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("${tr(LocaleKeys.lblYouCanAcceptOrTransferToAnother)}",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.assignment_ind,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    CheckboxListTile(
                        value: _transfer,
                        onChanged: ( value) {setState(() { _transfer = value; });},
                      title: Text('${tr(LocaleKeys.lblTransferToAnother)}',
                        // style: TextStyle(color: Colors.black),
                      ),
                    ),
                    _transfer?_employeeSelection(context) : Container(),
                    // !_transfer?Container(
                    //   padding: EdgeInsets.all(16),
                    //   child: new TextFormField(
                    //     decoration: InputDecoration(
                    //       hintText: '၃ဆ/၁၂၃၄',
                    //       labelText: '${tr(LocaleKeys.lblCarNumberToPickupDeliver)}',
                    //     ),
                    //     validator: (val) => val.isEmpty ? 'Car number required!' : null,
                    //     controller: _controller_carNumber,
                    //     keyboardType: TextInputType.text,
                    //   ),
                    // ) : Container(),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        _transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ) : Container(),
                        !_transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _processing? null : ()async{
                            await _accept();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.btnOK)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ): Container(),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _reject();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnCancel)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryEmployeePickupComplete( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblPickupComplete)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Complete your pickup process or try again later",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.done,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _refundAmount!=null && _refundAmount>0 ? Container(
                      padding: EdgeInsets.only(top:8, bottom: 0, left:8, right: 8),
                      child: CheckboxListTile(
                        value: _requiredToRefund,
                        onChanged: ( value) {setState(() { _requiredToRefund = value; });},
                        title: Text(
                          '${tr(LocaleKeys.lblRequiredToRefund)}',
                          // style: TextStyle(color: Colors.black),
                        ),
                      ),
                    ) : Container(),
                    _requiredToRefund? ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblCashToRefund)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text('${new NumberFormat("#,###").format(_refundAmount)} Ks',
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      leading: Icon(Icons.monetization_on_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ) : Container(),
                    _remarkTextField(),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 16.0, right: 16.0, top: 8),
                      width: double.infinity,
                      child: RaisedButton(
                        onPressed: () async{
                          String result = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => SignaturePad(
                                  title: tr(LocaleKeys.lblSenderSignature),
                                ),
                              )
                          );
                        },
                        color: Theme.of(context).primaryColorDark,
                        elevation: 1,
                        child: Text(
                          tr(LocaleKeys.lblSenderSignature),
                          // style: TextStyle(color: Colors.white),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Theme.of(context).primaryColor,
                          onPressed: _processing==null? null : ()async{
                            await _pickupCompleted();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.btnPickupCompleted)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.refresh_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _retry();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnPickupTryAgainLater)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_labourCost = new TextEditingController();
  final TextEditingController _controller_transportCost = new TextEditingController();
  final TextEditingController _controller_extraCost = new TextEditingController();
  final TextEditingController _controller_finalCashCollected= new TextEditingController();
  final TextEditingController _controller_nonCashCollected= new TextEditingController();
  final TextEditingController _controller_receivedOptionName= new TextEditingController();
  Widget _deliveryEmployeeComplete( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblDeliveryDetailPOD)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: Icon(Icons.approval,
                        // color: Colors.blueGrey[600]
                      ),
                    ),
                    _assigneeSign !=null && _assigneeSign.isNotEmpty? GestureDetector(
                      child: Container(
                        height: 100,
                        alignment: Alignment.center,
                        child: _assigneeSign==null ? Icon(Icons.photo, color: Colors.grey[100], size: 50,) : decode( _assigneeSign ),
                        decoration: _assigneeSign!=null ?BoxDecoration(
                            border: Border.all(color: Colors.green[50], width: 3)
                        ) : null,
                      ),
                      onTap: ()async{
                        if(_assigneeSign !=null && _assigneeSign.isNotEmpty){
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (context) => FullScreenWrapper(
                                imageProvider: MemoryImage(base64Decode(_assigneeSign)),// MemoryImage(base64Decode(widget.item.image.substring(22))) //NetworkImage( post.image ),
                              ),
                            ),
                          );
                        }
                      },
                      onLongPress: (){
                        setState(() {
                          _assigneeSign = null;
                        });
                      },
                    ) : Container(),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 8.0, right: 16.0, top: 8),
                      width: double.infinity,
                      child: RaisedButton.icon(
                        onPressed: () async{
                          await _getImage( true );
                        },
                        // color: Theme.of(context).primaryColorDark,
                        elevation: 1,
                        icon: Icon(Icons.photo_camera,
                            color: Colors.white
                        ),
                        label: Text(
                          tr(LocaleKeys.btnCamera),
                          style: TextStyle(color: Colors.white),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 8.0, right: 16.0, top: 0),
                      width: double.infinity,
                      child: RaisedButton.icon(
                        onPressed: () async{
                          String result = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => SignaturePad(
                                  title: tr(LocaleKeys.lblRecipientSignature),
                                ),
                              )
                          );
                          if( result !=null && result.length>100 ){
                            _signature = result;
                          }
                        },
                        // color: Theme.of(context).primaryColorDark,
                        elevation: 1,
                        icon: Icon(Icons.edit,
                            color: Colors.white
                        ),
                        label: Text(
                          tr(LocaleKeys.lblRecipientSignature),
                          style: TextStyle(color: Colors.white),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblDeliveryComplete)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Complete your delivery process or try again later",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.done_all,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    // Container(
                    //   padding: EdgeInsets.only(left: 16, right: 16),
                    //   child: new TextFormField(
                    //     decoration: InputDecoration(
                    //       hintText: '',
                    //       labelText: '${tr(LocaleKeys.lblLabourCost)}',
                    //     ),
                    //     validator: (val) => val.isEmpty ? null : (double.parse(val)>=0 ? null : 'Invalid data'),
                    //     controller: _controller_labourCost,
                    //     keyboardType: TextInputType.number,
                    //   ),
                    // ),
                    Container(
                      padding: EdgeInsets.only(left: 16, right: 16),
                      child: new TextFormField(
                        decoration: InputDecoration(
                          hintText: '',
                          labelText: '${tr(LocaleKeys.lblTransportationCost)}',
                        ),
                        validator: (val) => val.isEmpty ? null : (double.parse(val)>=0 ? null : 'Invalid data'),
                        controller: _controller_transportCost,
                        keyboardType: TextInputType.number,
                      ),
                    ),
                    // _contactItemCustomer!=null && _contactItemCustomer.addressType!='Customer'? Container(
                    //   padding: EdgeInsets.only(left: 16, right: 16),
                    //   child: new TextFormField(
                    //     decoration: InputDecoration(
                    //       hintText: '',
                    //       labelText: '${tr(LocaleKeys.lblExtraCost)}',
                    //     ),
                    //     validator: (val) => val.isEmpty ? null : (double.parse(val)>=0 ? null : 'Invalid data'),
                    //     controller: _controller_extraCost,
                    //     keyboardType: TextInputType.number,
                    //   ),
                    // ) : Container( ),
                    // _detailResponse.totalCharges>0? Container(
                    //   margin: const EdgeInsets.only(top: 0.0, left: 16.0, right: 16.0),
                    //   child: new CheckboxListTile(
                    //     value: _fullCollected,
                    //     onChanged: ( value){
                    //       setState(() { _fullCollected = value; });
                    //     },
                    //     title: Text(
                    //       tr(LocaleKeys.lblCODReceivedWithCash),
                    //     ),
                    //     subtitle: Text(tr(LocaleKeys.lblCODReceivedWithCashNot)),
                    //   ),
                    // ) : Container(),
                    !_fullCollected?Container(
                      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                      child: new TextFormField(
                        decoration: InputDecoration(
                          hintText: '',
                          labelText: '${tr(LocaleKeys.lblCODAmountReceivedWithCash)}',
                        ),
                        validator: (val) => val.isEmpty ? 'Required' : null,
                        controller: _controller_finalCashCollected,
                        keyboardType: TextInputType.number,
                        onChanged: (val){
                          setState(() {
                            if( double.parse(val.trim()) < _detailResponse.totalCharges){
                              _showReason = true;
                            }else{
                              _showReason = false;
                            }
                            _controller_nonCashCollected.text = ( _detailResponse.totalCharges - double.parse(val) ).toStringAsFixed(0);
                          });
                        },
                        // maxLines: null,
                        // minLines: 1,
                      ),
                    ): Container(),
                    !_fullCollected && _showReason? Container(
                        margin: const EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0),
                        child:new Column(
                          mainAxisSize: MainAxisSize.min,
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            new Text(
                              tr(LocaleKeys.lblCODTransferWithEMoney),
                              style: new TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold, color: Theme.of(context).primaryColor),
                            ),
                            // new RadioListTile(
                            //   value: 'Transfer',
                            //   groupValue: _reasonOfReduceAmount,
                            //   onChanged: ( value){
                            //     setState(() { _reasonOfReduceAmount = value; });
                            //   },
                            //   title: Text(
                            //     tr(LocaleKeys.lblCODTransferWithEMoney),
                            //     style: new TextStyle(
                            //         fontSize: 16.0,
                            //         color: Theme.of(context).primaryColor
                            //     ),
                            //   ),
                            // ),
                            // new RadioListTile(
                            //   value: 'DeliveryFeeReduce',
                            //   groupValue: _reasonOfReduceAmount,
                            //   onChanged: ( value){
                            //     setState(() {
                            //       _reasonOfReduceAmount = value;
                            //     });
                            //   },
                            //   title: Text(
                            //     tr(LocaleKeys.lblCODReduceDeliveryCharges),
                            //     style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                            //   ),
                            // ),
                            // new RadioListTile(
                            //   value: 'ReduceFromMerchant',
                            //   groupValue: _reasonOfReduceAmount,
                            //   onChanged: ( value){
                            //     setState(() {
                            //       _reasonOfReduceAmount = value;
                            //     });
                            //   },
                            //   title: Text(
                            //     tr(LocaleKeys.lblCODReduceFromMerchant),
                            //     style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                            //   ),
                            // ),
                          ],
                        ),
                      ): Container(),
                    !_fullCollected && _reasonOfReduceAmount=='Transfer' && _showReason?Container(
                      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                      child: new TextFormField(
                        decoration: InputDecoration(
                          hintText: '',
                          labelText: '${tr(LocaleKeys.lblCODTransferAmount)}',
                        ),
                        validator: (val) => val.isEmpty ? 'Required' : ( double.parse(val.trim()) + double.parse(_controller_finalCashCollected.text) > _detailResponse.totalCharges? 'Invalid amount' : null),
                        controller: _controller_nonCashCollected,
                        keyboardType: TextInputType.number,
                        // onChanged: (val){
                        //   setState(() {
                        //     if( double.parse(val.trim()) < _detailResponse.totalCharges){
                        //       _showReason = true;
                        //     }else{
                        //       _showReason = false;
                        //     }
                        //   });
                        // },
                        // maxLines: null,
                        // minLines: 1,
                      ),
                    ): Container(),
                    !_fullCollected && _reasonOfReduceAmount=='Transfer' && _showReason?Container(
                      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                      child: new TextFormField(
                        decoration: InputDecoration(
                          hintText: '${tr(LocaleKeys.lblCODTransferServiceNameHint)}',
                          labelText: '${tr(LocaleKeys.lblCODTransferServiceName)}',
                        ),
                        validator: (val) => val.isEmpty ? 'Required' : null,
                        controller: _controller_receivedOptionName,
                        keyboardType: TextInputType.text,
                        onChanged: (val){ },
                        // maxLines: null,
                        // minLines: 1,
                      ),
                    ): Container(),
                    !_fullCollected && _reasonOfReduceAmount=='Transfer' && _showReason? Container(
                      margin: const EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0),
                      child:new Column(
                        mainAxisSize: MainAxisSize.min,
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          new Text(
                            tr(LocaleKeys.lblPlacementOfTransfer),
                            style: new TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold, color: Theme.of(context).primaryColor),
                          ),
                          new RadioListTile(
                            value: 'Office',
                            groupValue: _nonCashTransferTo,
                            onChanged: ( value){
                              setState(() { _nonCashTransferTo = value; });
                            },
                            title: Text(
                              tr(LocaleKeys.lblPlacementOfTransferToOffice),
                              style: new TextStyle(
                                  fontSize: 16.0,
                                  color: Theme.of(context).primaryColor
                              ),
                            ),
                          ),
                          new RadioListTile(
                            value: 'Merchant',
                            groupValue: _nonCashTransferTo,
                            onChanged: ( value){
                              setState(() {
                                _nonCashTransferTo = value;
                              });
                            },
                            title: Text(
                              tr(LocaleKeys.lblPlacementOfTransferToMerchant),
                              style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                            ),
                          ),
                        ],
                      ),
                    ): Container(),
                    _remarkTextField(),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 16.0, right: 16.0, top: 8),
                      width: double.infinity,
                      child: RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.done_all,
                          color: Colors.white
                        ),
                        // color: Theme.of(context).primaryColor,
                        onPressed: _processing==null? null : ()async{
                          await _onCompleted();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.btnDeliverComplete)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.refresh_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _retry();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnDeliverTryAgain)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.backspace_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _onReturn();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnDeliverReturn)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    ),
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_remark = new TextEditingController();
  Widget _remarkTextField(){
    return Container(
      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: new TextFormField(
        decoration: InputDecoration(
          hintText: '',
          labelText: '${tr(LocaleKeys.lblRemark)}*',
        ),
        validator: (val) => val.isEmpty ? 'Remark required' : null,
        controller: _controller_remark,
        keyboardType: TextInputType.text,
        // maxLines: null,
        // minLines: 1,
      ),
    );
  }

  Future<void> _assign() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAssignRequest request = new DeliveryAssignRequest(
      contentGuid: _detailResponse.guid,
      personGuid: _employeeGuid,
      comment: _controller_remark.text,
      type: _detailResponse.status == Constant.STATUS_RETRY? 'Re-assign' : 'Initial',
    );

    Map<String, dynamic> resultData = await _deliveryContentService.assign(_accessToken, request, "assign");
    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }
    setState(() {
      _processing = false;
    });
  }

  Future<void> _accept() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
      contentGuid: _detailResponse.guid,
      personGuid: null,
      comment: _controller_remark.text,
      carNumber: _controller_carNumber.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "accept");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _reject() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "reject");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _cancel() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "cancel");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> returned() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "returned");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _doTransfer() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: _employeeGuid,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "transfer");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _retry() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "retry");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _pickupCompleted() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text,
        refundInAdvanced: _requiredToRefund,
        refundAmount: _requiredToRefund? _refundAmount : null,
        senderSign: _signature,
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "pickup_completed");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _onReturn() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _deliveryContentService.process(_accessToken, request, "return");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _onCompleted() async{

    if( !_fullCollected && _controller_finalCashCollected.text.trim().isEmpty){
      showMessage( "Received cash money required!" );
      return;
    }

    if( _reasonOfReduceAmount=='Transfer' && !_fullCollected
        && _controller_nonCashCollected.text.trim().isEmpty){
      showMessage( "Transfer amount required!" );
      return;
    }

    if( _reasonOfReduceAmount=='Transfer' && !_fullCollected
        && _controller_receivedOptionName.text.trim().isEmpty){
      showMessage( "Financial institute name required!" );
      return;
    }

    if( _controller_remark.text.isEmpty ){
      showMessage( "Remark required!" );
      return;
    }

    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }


    bool fullCollected = _fullCollected;
    double finalCashCollected = 0.0;
    double nonCashCollected = 0.0;
    String receivedOption = "Cash";//Cash, Bank, E-money
    String receivedOptionName = "Cash";//Cash, KPay, WavePay, AYA Bank, CB Bank, etc.
    String nonCashTransferTo = "Office";//Office, Merchant
    String reasonOfReduceAmount = "DeliveryFeeReduce"; // DeliveryFeeReduce, ReduceFromMerchant
    if( fullCollected ){
      finalCashCollected = null;
      nonCashCollected = null;
      receivedOption = "Cash";
      receivedOptionName = null;
      nonCashTransferTo = null;
      reasonOfReduceAmount = null;
    }else{
      finalCashCollected = double.parse( _controller_finalCashCollected.text.trim() );
      if( _reasonOfReduceAmount=='Transfer' ) {
        nonCashCollected =
            double.parse(_controller_nonCashCollected.text.trim());
        receivedOptionName = _controller_receivedOptionName.text.trim();
        nonCashTransferTo = _nonCashTransferTo;
        receivedOption = "E-money";
      }else {
        nonCashCollected = _detailResponse.totalCharges - finalCashCollected;
        receivedOptionName = null;
        nonCashTransferTo = null;
      }
      reasonOfReduceAmount = _reasonOfReduceAmount=='Transfer'? null : _reasonOfReduceAmount;
    }

    setState(() {
      _processing = true;
    });

    DeliveryCompletedRequest request = new DeliveryCompletedRequest(
        contentGuid: _detailResponse.guid,
        labourCost: _controller_labourCost.text!=null&& _controller_labourCost.text.isNotEmpty? double.parse(_controller_labourCost.text) : 0,
        transportCost: _controller_transportCost.text!=null&& _controller_transportCost.text.isNotEmpty? double.parse(_controller_transportCost.text) : 0,
        extraCost: _controller_extraCost.text!=null&& _controller_extraCost.text.isNotEmpty? double.parse(_controller_extraCost.text) : 0,
        customerSign: _signature,
        assigneeSign: _assigneeSign,
        comment: _controller_remark.text,
        latitude: _startLocation !=null? _startLocation.latitude : null,
        longitude: _startLocation !=null? _startLocation.longitude : null,
        fullCollected: fullCollected,
        finalCashCollected: finalCashCollected,
        nonCashCollected: nonCashCollected,
        receivedOption: receivedOption,
        receivedOptionName: receivedOptionName,
        nonCashTransferTo: nonCashTransferTo,
        reasonOfReduceAmount: reasonOfReduceAmount
    );
print( request.toMap() );
    Map<String, dynamic> resultData = await _deliveryContentService.complete(_accessToken, request, "completed");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 16, left: 16, right: 16),
        child: DropdownSearch<EmployeeForDeliveryItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeForDeliveryItem data) {
            setState(() {
              _employeeSearchItem = data;
              if( _employeeSearchItem != null){
                _employeeGuid = _employeeSearchItem.guid;
              }else{
                _employeeGuid = null;
              }
            });
          },
          dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle,
                      // color: Colors.black
                  ),
                  title: Text(tr(LocaleKeys.lblSelectEmployees),
                    // style: TextStyle(color: Colors.black),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name,
                  // style: TextStyle(color: Colors.black),
                ),

                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                  // leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  // color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.zoneName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = List();
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _deliverySelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && (_authorityResponse.authorities.contains("MANAGER_READ") || _authorityResponse.authorities.contains("OPERATOR_READ"))?
      Container(
          margin: EdgeInsets.only(top: 16, left: 16, right: 16, bottom: 16),
          child: DropdownSearch<DeliveryForDeliveryItem>(
            selectedItem: _employeeSearchItemAll,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (DeliveryForDeliveryItem i, DeliveryForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _deliverymanSuggestion(filter),
            onChanged: (DeliveryForDeliveryItem data) {
              _employeeSearchItemAll = data;
              if( _employeeSearchItemAll != null){
                _employeeGuid = _employeeSearchItemAll.guid;
              }else{
                _employeeGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, DeliveryForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ),
                  ),
                ),
            popupItemBuilder: (BuildContext context, DeliveryForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text(item.personId),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }

  Future<List<DeliveryForDeliveryItem>> _deliverymanSuggestion(String query) async{
    List<DeliveryForDeliveryItem> matches = [];
    if( _deliverymanResponse.items == null)
      return matches;
    matches.addAll(_deliverymanResponse.items);

    matches.retainWhere((s) {
      DeliveryForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else if( employeeSearchItem.personId.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else
        return false;
    }
    );
    return matches;
  }


  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm", style: TextStyle(color: Colors.black),),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

  String _createStatus(String status ){
    String title = '';
    if( _authorityResponse.authorities!=null
        && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')){
        if( status == Constant.STATUS_CREATED ){
        title = tr(LocaleKeys.lblItemListDropOffQueue);
        }else if( status == Constant.STATUS_ASSIGNED ){
        title = tr(LocaleKeys.lblItemListToDropOff);
        }else if( status == Constant.STATUS_ON_WAY ){
        title = tr(LocaleKeys.lblItemListDropOffOnWay);
        }else if( status == Constant.STATUS_RETURN ){
        title = tr(LocaleKeys.lblItemListReturn);
        }else if( status == Constant.STATUS_RETRY ){
        title = tr(LocaleKeys.lblItemListRetry);
        }else if( status == Constant.STATUS_CANCELED ){
        title = tr(LocaleKeys.lblItemListCancelled);
        }else if( status == Constant.STATUS_COMPLETED ){
        title = tr(LocaleKeys.lblItemListCompleted);
        }else if( status == Constant.STATUS_PICKUP_CREATED ){
        title = tr(LocaleKeys.lblItemListPickupQueue);
        }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
        title = tr(LocaleKeys.lblItemListToPickup);
        }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
        title = tr(LocaleKeys.lblItemListPickupOnWay);
        }
    }else{
      if( _detailResponse.pickupStatus =='Completed' ){
        if( _detailResponse.status == 'Created')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerInTransit);
        else if( _detailResponse.status == 'Assigned')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerInTransit);
        else if( _detailResponse.status == 'OnWay')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerOnTheWay);
        else if( _detailResponse.status == 'Completed')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerDelivered);
        else title = _detailResponse.status;
      }else if( _detailResponse.pickupStatus =='PickupCreated' || _detailResponse.pickupStatus =='PickupScheduled' || _detailResponse.pickupStatus =='PickupOnWay' ){
        title = tr(LocaleKeys.lblDeliveryStatusForCustomerPickup);
      }else if( _detailResponse.pickupStatus =='Canceled' ){
        title = 'Canceled';
      }else{
        title = _detailResponse.pickupStatus;
      }
    }
    return title;
  }

  void showMessage(String message, [MaterialColor color = Colors.green]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(content: new Text(message)));
  }

  Future<void> _getImage( bool isCamera) async {
    _formKey.currentState.save();

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/gelivery_tmp';
        await Directory(tmpDir).create(recursive: true);

        String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
        File rotatedFile = await FlutterImageCompress.compressAndGetFile(
          image.path,
          photoPath2,
          quality: 72,
          rotate: 0,
        );
        imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
        String cropFilePath = '$tmpDir/cropped.jpg';
        File(cropFilePath)
          ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));

        File smallFile  =File(cropFilePath);
        String encodedString = encode( smallFile );
        smallFile.deleteSync(recursive: false);
        rotatedFile.deleteSync(recursive: false);
        setState(() {
          _assigneeSign = encodedString;
        });
        showMessage("To remove the photo, Long Press on that photo.");
      }
    }catch(e){}
  }


  Location _locationService  = new Location();
  bool _permission = false;
  LocationData _currentLocation;
  LocationData _startLocation;
  String error;
  _initLocationService() async {
    LocationData location;
    try {

      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
            // print('Lat: ${_startLocation.latitude}, Lon:${_startLocation.longitude}');
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }
    _startLocation = location;
  }

}


class InvoiceDetailPreProcess extends StatefulWidget {

  InvoiceDetailPreProcess({Key key, @required this.detailResponse,
    @required this.contactItemOS, @required this.contactItemCustomer,
    @required this.currentUser, @required this.item,
  @required this.refundAmount}) : super(key: key);
  final DeliveryContactItem contactItemOS;
  final DeliveryContactItem contactItemCustomer;
  final DeliveryDetailResponse detailResponse;
  final DeliveryItem item;
  final double refundAmount;
  // final DeliveryListItem post;
  final User currentUser;

  @override
  _InvoiceDetailPreProcessState createState() => _InvoiceDetailPreProcessState();
}

class _InvoiceDetailPreProcessState extends State<InvoiceDetailPreProcess> {

  @override
  Widget build( BuildContext context ) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.detailResponse.trackingId, style: TextStyle(color: Colors.white)),
      ),
      body: SingleChildScrollView(
        child: _deliverySummary( context ),
      ),
    );
  }

  Widget _deliverySummary( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblInvoiceDetail)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text('Invoice date: '+ new DateFormat('dd/MM/yyyy').format(widget.detailResponse.updatedOn.toLocal()),
                        style: TextStyle(color: Colors.blueGrey[600])
                    ),
                    // trailing: IconButton(
                    //   icon: SvgPicture.asset( 'assets/images/pdf.svg', semanticsLabel: 'PDF',height: 24, width: 23, ),
                    //   onPressed: (){},
                    // )
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    'Order date: ${new DateFormat('dd/MM/yyyy').format(widget.detailResponse.createdOn.toLocal())}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    'Delivery date: ${new DateFormat('dd/MM/yyyy').format(widget.contactItemCustomer.date.toLocal())}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblFrom)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${widget.contactItemOS.name}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${widget.contactItemOS.address??''}, ${widget.contactItemOS.street??''}, ${widget.contactItemOS.township??''}",
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderMobile)}: ${widget.contactItemOS.primaryPhone}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),

                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblTo)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${widget.contactItemCustomer.name}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${widget.contactItemCustomer.address??''}, ${widget.contactItemCustomer.street??''}, ${widget.contactItemCustomer.township??''}",
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientMobile)}: ${widget.contactItemCustomer.primaryPhone}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),

                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblChargesDetails)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblDeliCharges)}: ${new NumberFormat("#,###").format(widget.item.charges??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblLabourCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.labourCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblTransportationCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.transportCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblExtraCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.extraCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSubTotal)}: ${new NumberFormat("#,###").format(widget.item.charges)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblPromotion)}: -${new NumberFormat("#,###").format(widget.detailResponse.promotion)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblTotal)}: ${new NumberFormat("#,###").format(widget.item.charges - widget.detailResponse.promotion)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblCashToCollect)}: ${new NumberFormat("#,###").format(widget.detailResponse.totalCharges)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblCashToRefund)}: ${new NumberFormat("#,###").format(widget.refundAmount)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: Text(
                    '${tr(LocaleKeys.lblRefundDate)}: ${widget.detailResponse.refundDate!=null? new DateFormat('dd/MM/yyyy hh:mm a').format(widget.detailResponse.refundDate.toLocal()): "Not yet!"}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    );
  }

  // double _deliveryCharges( DeliveryListItem post ){
  //   double deliCharge = post.totalDeliveryCharges!=null? post.totalDeliveryCharges : 0;
  //   double promo = post.promotionAmount!=null? post.promotionAmount : 0;
  //
  //   return (deliCharge - promo);
  // }
}