import 'dart:async';
import 'package:badges/badges.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_search_filter.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'delivery_list_main.dart';



class DeliveryListHome extends StatefulWidget {
  DeliveryListHome({Key key, this.onChange, this.selectedIndex, this.title}) : super(key: key);
  final Function onChange;
  final int selectedIndex;
  final String title;
  @override
  _DeliveryListHomeState createState() => _DeliveryListHomeState();
}

class _DeliveryListHomeState extends State<DeliveryListHome> {

  User _currentUser = new User();
  UserService _userService = new UserService();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  String _employeeGuid;
  String _oSGuid;
  String _customerGuid;

  String _accessToken;
  bool _search = false;
  bool _tapChange = false;
  DateTime _startDate;
  DateTime _endDate;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblQueue);
  int _pendingLeaveCount = 0;
  bool _processing = false;
  bool _multipleSelected = false;
  bool _showMultipleSelect = true;
  List<String> _selectedWays=[];
  String _keyword = '';

  @override
  void initState() {
    super.initState();
    _title = widget.title;
    _loadData();
  }

  _loadData() async{
        _selectedIndex = widget.selectedIndex;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
        _accessToken = await _userService.getToken();

        if( _selectedIndex == 1){
          _showMultipleSelect = _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ");
        }else if( _selectedIndex == 2){
          _showMultipleSelect = true;
        }else{
          _showMultipleSelect = false;
        }

        setState(() {});
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : _createSelectedIndicator(),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:() async{
                    setState(() {
                      _reset();
                      _search = !_search;
                    });
                    Map<String, dynamic> result = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => PickupSearchFilter(
                            title: _title,
                            authorityResponse: _authorityResponse,
                            currentUser: _currentUser,
                            sharedPrefs: _sharedPrefs,
                            accessToken: _accessToken,
                          ),
                        )
                    );
                    if( result != null ){
                      _keyword = result['keyword'];
                      _startDate = result['startDate'];
                      _endDate = result['endDate'];
                      _oSGuid = result['osGuid'];
                      _employeeGuid = result['employeeGuid'];
                    }
                    setState(() {
                      _search = !_search;
                    });
                  },
                  icon: Icon(Icons.search, color: Colors.white,),
                  label: Text(tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              ),
              !_search && _showMultipleSelect? FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _multipleSelected = !_multipleSelected;
                    });
                    if( !_multipleSelected){
                      setState(() {_tapChange = true;});
                      new ThreadTimer(new Duration(milliseconds: 300), () {
                        setState(() {_tapChange = false;});
                      });
                    }
                  },
                  icon: _multipleSelected? Icon(Icons.check_box_outlined, color: Colors.white,)
                      : Icon(Icons.check_box_outline_blank_outlined, color: Colors.white,),
                  label: Text(
                    tr(LocaleKeys.btnHomeActionBarSelect),
                    style:  TextStyle(color: Colors.white),
                  )
              ) : Container(),
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.list),
                label: tr(LocaleKeys.lblHomeAdminToAssign),
              ),
              BottomNavigationBarItem(
                icon: _authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('SUPERVISOR_WRITE')
                    && _pendingLeaveCount>0? Badge(
                    position: BadgePosition.topEnd(top: -20, end: -20),
                    badgeContent: Text('$_pendingLeaveCount',style: TextStyle(color: Colors.white),),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: Icon(Icons.help)
                ) : Icon(Icons.download_sharp),
                label: tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblHomeAdminToAccept:LocaleKeys.lblToDropOff),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.local_shipping_sharp),
                label: tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.done_all),
                label: tr(LocaleKeys.lblCompleted),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.cancel_schedule_send_sharp),
                label: tr(LocaleKeys.lblCancel),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.refresh),
                label: tr(LocaleKeys.lblRetry),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.backspace_sharp),
                label: tr(LocaleKeys.lblReturn),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Theme.of(context).primaryColor,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  Widget _createBody(){

    List<String> status;
    if( _selectedIndex == 0 && !_search && !_tapChange){
      status = [Constant.STATUS_CREATED];
    }else if( _selectedIndex == 1 && !_search && !_tapChange){
      status = [Constant.STATUS_ASSIGNED];
    }else if( _selectedIndex == 2 && !_search && !_tapChange){
      status = [Constant.STATUS_ON_WAY];
    }else if( _selectedIndex == 3 && !_search && !_tapChange){
      status = [Constant.STATUS_COMPLETED];
    }else if( _selectedIndex == 4 && !_search && !_tapChange){
      status = [Constant.STATUS_CANCELED];
    }else if( _selectedIndex == 5 && !_search && !_tapChange){
      status = [Constant.STATUS_RETRY];
    }else if( _selectedIndex == 6 && !_search && !_tapChange){
      status = [Constant.STATUS_RETURN];
    }else{
      return new Container();
    }

    return DeliveryListMain(
      keyword: _keyword,
      startDate: _startDate,
      endDate: _endDate,
      status: status,
      osGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _oSGuid,
      customerGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("USER_READ") && !_authorityResponse.authorities.contains("MANAGER_READ") && !_authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("EMPLOYEE_READ")? _currentUser.guid : _customerGuid,
      employeeGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _employeeGuid,
      branchGuid: null,
      multipleSelected: _multipleSelected,
      selectedIndex: _selectedIndex,
      onChange: (){
        widget.onChange();
      },
      onSelected: ( String guid, bool isSelected ){
        if( isSelected ){
          setState(() {
            _selectedWays.add(guid);
          });
        }else{
          setState(() {
            _selectedWays.remove(guid);
          });
        }
      },
    );
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = tr(LocaleKeys.lblDropOffQueue);
        _showMultipleSelect = false;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else if( _selectedIndex == 1){
        _title = tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblHomeAdminToAccept :LocaleKeys.lblToDropOff);
        _showMultipleSelect = _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ");
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else if( _selectedIndex == 2){
        _title = tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")? LocaleKeys.lblToDropOff : LocaleKeys.lblDropOffOnWay);
        _showMultipleSelect = true;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else if( _selectedIndex == 3){
        _title = tr(LocaleKeys.lblCompleted);
        _showMultipleSelect = false;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else if( _selectedIndex == 4){
        _title = tr(LocaleKeys.lblCancel);
        _showMultipleSelect = false;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else if( _selectedIndex == 5){
        _title = tr(LocaleKeys.lblRetry);
        _showMultipleSelect = false;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }else{
        _title = tr(LocaleKeys.lblReturn);
        _showMultipleSelect = false;
        _multipleSelected = false;
        _selectedWays.clear();
        _reset();
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  _reset(){
    _keyword = '';
    _employeeGuid = null;
    _startDate = null;
    _endDate = null;
  }


  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  //PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,))
  PreferredSize _createSelectedIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, (_selectedIndex==1 || _selectedIndex==2) && _multipleSelected? 56.0 : 1.0),
      child: (_selectedIndex==1 || _selectedIndex==2) && _multipleSelected?SizedBox(
          height: 56.0,
          child: Card(
            margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
            elevation: 0,
            child: Wrap(
                children: <Widget>[
                  _processing?LinearProgressIndicator(backgroundColor: Colors.white,) : Container(),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: [
                      Text('${_selectedWays.length}${_selectedWays.length>0? tr(LocaleKeys.btnHomeActionBarSelectedWayCountMessageMultiple): tr(LocaleKeys.btnHomeActionBarSelectedWayCountMessageSingle)}'),
                      _selectedIndex==1?ButtonBar(
                        children: <Widget>[
                          RaisedButton.icon(
                            elevation: 0.0,
                            icon: Icon(Icons.done,
                              // color: Colors.white
                            ),
                            color: Colors.green[700],
                            onPressed: _processing? null : ()async{
                              if( _selectedWays.isEmpty){
                                showMessage("Please select the ways to accept");
                                return;
                              }
                              bool  result = await _showConfirm(context, 'Confirm to deliver','You have selected ${_selectedWays.length} ways to deliver, are you sure?');
                              if ( ! result ){
                                return;
                              }
                              _multipleAction('accept');
                            },
                            label: Text(
                              '${tr(LocaleKeys.btnOK)}',
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w400,
                                // color: Colors.white,
                              ),
                            ),
                            shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                          ),
                          RaisedButton.icon(
                            elevation: 0.0,
                            icon: Icon(Icons.cancel,
                              // color: Colors.white
                            ),
                            color: Colors.red[700],
                            onPressed: _processing? null : ()async{
                              if( _selectedWays.isEmpty){
                                showMessage("Please select the ways to reject");
                                return;
                              }
                              bool  result = await _showConfirm(context, 'Confirm to reject','You have selected ${_selectedWays.length} ways that you will NOT deliver, are you sure?');
                              if ( ! result ){
                                return;
                              }
                              _multipleAction('reject');
                            },
                            label: Text(
                              "${tr(LocaleKeys.btnCancel)}",
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w400,
                                // color: Colors.white,
                              ),
                            ),
                            shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                          ),
                        ],
                      ) : Container(),
                      _selectedIndex==2?ButtonBar(
                        children: <Widget>[
                          RaisedButton.icon(
                            elevation: 0.0,
                            icon: Icon(Icons.done_all,
                              // color: Colors.white
                            ),
                            // color: Colors.red[700],
                            onPressed: _processing? null : ()async{
                              if( _selectedWays.isEmpty){
                                showMessage("Please select the ways to complete");
                                return;
                              }
                              bool  result = await _showConfirm(context, 'Confirm to complete','You have selected ${_selectedWays.length} ways that you have completed and delivered, are you sure?');
                              if ( ! result ){
                                return;
                              }
                              _multipleAction('complete');
                            },
                            label: Text(
                              "${tr(LocaleKeys.btnDeliverComplete)}",
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w400,
                                // color: Colors.white,
                              ),
                            ),
                            shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(8.0)),
                          ),
                          RaisedButton.icon(
                            elevation: 0.0,
                            icon: Icon(Icons.refresh_sharp,
                              // color: Colors.white
                            ),
                            color: Colors.red[700],
                            onPressed: _processing? null : ()async{
                              if( _selectedWays.isEmpty){
                                showMessage("Please select the ways to date change");
                                return;
                              }
                              bool  result = await _showConfirm(context, 'Confirm to date change','You have selected ${_selectedWays.length} ways which are required to deliver again later, are you sure?');
                              if ( ! result ){
                                return;
                              }
                              _multipleAction('retry');
                            },
                            label: Text(
                              "${tr(LocaleKeys.btnDeliverTryAgain)}",
                              style: TextStyle(
                                fontSize: 16,
                                fontWeight: FontWeight.w400,
                                // color: Colors.white,
                              ),
                            ),
                            shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(8.0)),
                          ),
                        ],
                      ) : Container(),
                    ],
                  ),
                ]
            ),
          ),
      ) : SizedBox(height: 1.0,)
  );
  Future<bool> _showConfirm( BuildContext context, String title, String message )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( title, style: TextStyle(color: Colors.black),),
          content: new Text( message ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new TextButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new TextButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<void> _multipleAction( String action ) async{
    setState(() {
      _processing = true;
    });
    Map<String, dynamic> request = {
        'guid': _selectedWays,
        'personGuid': null,
        'action': action,
        'comment': null
    };

    DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
    Map<String, dynamic> resultData = await _deliveryContentService.multipleAction( _accessToken, request );
    if( resultData != null) {
      if (resultData["success"]) {
        showMessage(resultData["message"]);
        _tapChange = true;
        Timer(Duration(seconds: 2), () {
          setState(() {
            _tapChange = false;
            _multipleSelected = false;
            _selectedWays.clear();
            widget.onChange();
          });
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }
    setState(() {
      _processing = false;
    });
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}
