
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_list_main.dart';
import 'package:threading/threading.dart';
import 'delivery_list_main.dart';



class DeliveryListHomeCustomer extends StatefulWidget {
  DeliveryListHomeCustomer({Key key, this.onChange, this.selectedIndex, this.title}) : super(key: key);
  final Function onChange;
  final int selectedIndex;
  final String title;
  @override
  _DeliveryListHomeState createState() => _DeliveryListHomeState();
}

class _DeliveryListHomeState extends State<DeliveryListHomeCustomer> {

  User _currentUser = new User();
  UserService _userService = new UserService();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  bool _search = false;
  bool _tapChange = false;
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_keyword = new TextEditingController();
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblQueue);
  bool _processing = false;

  @override
  void initState() {
    super.initState();
    _title = widget.title;
    _loadData();
  }

  _loadData() async{
          if( (widget.selectedIndex == 3 && _title== tr(LocaleKeys.lblPickupCancel)) || (widget.selectedIndex==2 && _title== tr(LocaleKeys.lblPickupCompleted)) )
            _selectedIndex = 0;
          else
          _selectedIndex = widget.selectedIndex;

        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom: _search?_createSearchIndicator() : ( _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)) ),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _reset();
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel,
                    color: Colors.white,
                  )
                      : Icon(Icons.search,
                    color: Colors.white,
                  ),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.local_shipping_sharp),
                label: tr(LocaleKeys.lblDeliver),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.local_shipping_sharp),
                label: tr(LocaleKeys.lblReceive),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.done_all),
                label: tr(LocaleKeys.lblCompleted),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.cancel_schedule_send_sharp),
                label: tr(LocaleKeys.lblCancel),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.refresh),
                label: tr(LocaleKeys.lblRetry),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.backspace_sharp),
                label: tr(LocaleKeys.lblReturn),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Theme.of(context).primaryColor,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  Widget _createBody(){

    List<String> status;
    if( _selectedIndex == 0 && !_search && !_tapChange){
      status = [ Constant.STATUS_PICKUP_CREATED,
        Constant.STATUS_PICKUP_SCHEDULED,
        Constant.STATUS_PICKUP_ON_WAY,
        Constant.STATUS_COMPLETED, Constant.STATUS_CANCELED ];
    }else if( _selectedIndex == 1 && !_search && !_tapChange){
      status = [Constant.STATUS_CREATED,
        Constant.STATUS_ASSIGNED,
        Constant.STATUS_ON_WAY];
    }else if( _selectedIndex == 2 && !_search && !_tapChange){
      status = [Constant.STATUS_COMPLETED];
    }else if( _selectedIndex == 3 && !_search && !_tapChange){
      status = [Constant.STATUS_CANCELED];
    }else if( _selectedIndex == 4 && !_search && !_tapChange){
      status = [Constant.STATUS_RETRY];
    }else if( _selectedIndex == 5 && !_search && !_tapChange){
      status = [Constant.STATUS_RETURN];
    }else{
      return new Container();
    }

    if( _selectedIndex == 0 ){
      return PickupListMain(
        keyword: _controller_keyword.text,
        startDate: _startDate,
        endDate: _endDate,
        status: status,
        osGuid: _selectedIndex==0? _currentUser.guid : null,
        customerGuid: _selectedIndex==1? _currentUser.guid : null,
        employeeGuid: null,
        zoneGuid: null,
        branchGuid: null,
        onChange: (){
          widget.onChange();
        },
      );
    }

    return DeliveryListMain(
      keyword: _controller_keyword.text,
      startDate: _startDate,
      endDate: _endDate,
      status: status,
      osGuid: _selectedIndex==0? _currentUser.guid : null,
      customerGuid: _selectedIndex==1? _currentUser.guid : null,
      employeeGuid: null,
      zoneGuid: null,
      branchGuid: null,
      multipleSelected: false,
      selectedIndex: _selectedIndex,
      onChange: (){
        widget.onChange();
      },
      onSelected: (){},
    );

  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = tr(LocaleKeys.lblDeliver);
        _reset();
      }else if( _selectedIndex == 1){
        _title = tr(LocaleKeys.lblReceive);
        _reset();
      }else if( _selectedIndex == 2){
        _title = tr(LocaleKeys.lblCompleted);
        _reset();
      }else if( _selectedIndex == 3){
        _title = tr(LocaleKeys.lblCancel);
        _reset();
      }else if( _selectedIndex == 4){
        _title = tr(LocaleKeys.lblRetry);
        _reset();
      }else{
        _title = tr(LocaleKeys.lblReturn);
        _reset();
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  _reset(){
    _controller_keyword.text = '';
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 610.0),
      child: SizedBox(
          height: 610.0,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 0,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child:new Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new TextFormField(
                            decoration: new InputDecoration(
                              icon: const Icon(Icons.keyboard),
                              hintText: '${tr(LocaleKeys.lblExample)} ${tr(LocaleKeys.lblWayID)}',
                              labelText: tr(LocaleKeys.lblKeyword),
                            ),
                            controller: _controller_keyword,
                            keyboardType: TextInputType.text,
                            autofocus: true,
                          ),
                        ),
                        // _customerSelection(context),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon,
              // color: Colors.white
          ),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _reset();
                _search = !_search;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                if( _controller_start.text.isEmpty || _controller_end.text.isEmpty){
                  _startDate = null;
                  _endDate = null;
                }
                _search = !_search;
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              // color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }
  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}
