
import 'dart:async';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:location/location.dart';


class DeliveryMap extends StatefulWidget {
  DeliveryMap({this.startLocation, this.items, this.title});

  final LocationData startLocation;
  List<EmployeeForDeliveryItem> items;
  final title;
  @override
  _DeliveryMapState createState( ) => _DeliveryMapState( );
}

class _DeliveryMapState extends State<DeliveryMap> {

  MapType _mapType = MapType.normal;
  bool _rotateGesturesEnabled = true;
  bool _scrollGesturesEnabled = true;
  bool _tiltGesturesEnabled = true;
  bool _zoomGesturesEnabled = true;
  bool _indoorViewEnabled = true;
  bool _myLocationEnabled = true;
  bool _myLocationButtonEnabled = true;
  bool _isMapCreated = false;
  bool _isMoving = false;
  bool _compassEnabled = true;
  bool _mapToolbarEnabled = true;

  GoogleMapController _controller;
  CameraPosition _kGooglePlex;
  Map<MarkerId, Marker> markers = <MarkerId, Marker>{};
  MarkerId selectedMarker;
  BitmapDescriptor _markerIcon;


  @override
  void initState() {
    super.initState();
    _kGooglePlex = CameraPosition(
      target: widget.startLocation==null ? LatLng(16.813538,96.171205) : LatLng(widget.startLocation.latitude, widget.startLocation.longitude),
      zoom: 14.4746);
  }

  Future<void> _createMarkerImageFromAsset(BuildContext context) async {
    if (_markerIcon == null) {
      final ImageConfiguration imageConfiguration =
      createLocalImageConfiguration(context, size: Size(24,24));
      BitmapDescriptor.fromAssetImage(
          imageConfiguration, 'assets/images/pin.png')
          .then(_updateBitmap);
    }
  }
  void _updateBitmap(BitmapDescriptor bitmap) {
    setState(() {
      _markerIcon = bitmap;
    });
  }
  void _add( ) {
    for( int i=0; i < widget.items.length; i++){
      EmployeeForDeliveryItem employee = widget.items[i];

      if( employee.latitude != null && employee.longitude != null) {
        final MarkerId markerId = MarkerId('${employee.guid}');
        final Marker marker = Marker(
          markerId: markerId,
          position: LatLng(
            employee.latitude,
            employee.longitude,
          ),
          infoWindow: InfoWindow(title: employee.name, snippet: employee.phone),
          onTap: () {
            _onMarkerTapped(markerId);
          },
          icon: _markerIcon,//BitmapDescriptor.fromBytes(base64Decode(employee['coverphoto'].substring(22)))
        );

        setState(() {
          markers[markerId] = marker;
        });
      }
    }
  }

  void _onMapCreated(GoogleMapController controller) {
    this._controller = controller;
    if( widget.items != null)
      _add( );
  }

  void _onMarkerTapped(MarkerId markerId) {
    final Marker tappedMarker = markers[markerId];
    if (tappedMarker != null) {
      setState(() {
        if (markers.containsKey(selectedMarker)) {
          final Marker resetOld = markers[selectedMarker]
              .copyWith(iconParam: BitmapDescriptor.defaultMarker);
          markers[selectedMarker] = resetOld;
        }
        selectedMarker = markerId;
        final Marker newMarker = tappedMarker.copyWith(
          iconParam: BitmapDescriptor.defaultMarkerWithHue(
            BitmapDescriptor.hueOrange,
          ),
        );
        markers[markerId] = newMarker;
      });
    }
  }


  @override
  void dispose() {
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {
    _createMarkerImageFromAsset(context);
    return ( widget.title == null ? Container(
      //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
      child: GoogleMap(
        mapType: _mapType,
        initialCameraPosition: _kGooglePlex,
        onMapCreated: _onMapCreated,
        compassEnabled: _compassEnabled,
        mapToolbarEnabled: _mapToolbarEnabled,
        rotateGesturesEnabled: _rotateGesturesEnabled,
        scrollGesturesEnabled: _scrollGesturesEnabled,
        tiltGesturesEnabled: _tiltGesturesEnabled,
        zoomGesturesEnabled: _zoomGesturesEnabled,
        indoorViewEnabled: _indoorViewEnabled,
        myLocationEnabled: _myLocationEnabled,
        myLocationButtonEnabled: _myLocationButtonEnabled,
        markers: Set<Marker>.of(markers.values),
      ),
    )
      :
      Scaffold(
          appBar: AppBar(
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
      ),
        body: Center(
          child:Container(
            //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
            child: GoogleMap(
              mapType: _mapType,
              initialCameraPosition: _kGooglePlex,
              onMapCreated: _onMapCreated,
              compassEnabled: _compassEnabled,
              mapToolbarEnabled: _mapToolbarEnabled,
              rotateGesturesEnabled: _rotateGesturesEnabled,
              scrollGesturesEnabled: _scrollGesturesEnabled,
              tiltGesturesEnabled: _tiltGesturesEnabled,
              zoomGesturesEnabled: _zoomGesturesEnabled,
              indoorViewEnabled: _indoorViewEnabled,
              myLocationEnabled: _myLocationEnabled,
              myLocationButtonEnabled: _myLocationButtonEnabled,
              markers: Set<Marker>.of(markers.values),
            ),
          )
        ),
      )
    );
  }
}