import 'dart:convert';
import 'package:easy_localization/easy_localization.dart';
import 'package:latlong/latlong.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'delivery_detail.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart' as google;



class GoingOutDeliverList extends StatefulWidget {
  GoingOutDeliverList({Key key, this.keyword, this.employeeGuid}) : super(key: key);
  final String keyword;
  final String employeeGuid;

  @override
  _GoingOutDeliverListState createState() => _GoingOutDeliverListState();
}

class _GoingOutDeliverListState extends State<GoingOutDeliverList>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DeliveryListResponse _deliveryListResponse;
  String _accessToken;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  DeliveryListRequest _listRequest = new DeliveryListRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  bool _isFirst = true;
  UserService userService = UserService();

  @override
  void initState() {
    super.initState();

    _listRequest = new DeliveryListRequest(
        keyword: widget.keyword,
        employeeGuid: widget.employeeGuid,
        status:[Constant.STATUS_ON_WAY],
        first: _page,
        max: Constant.ROW_PER_PAGE,
        orderBy: 'customer.name_mm'
    );

  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      _accessToken = await userService.getToken();
      _listRequest.first = _page;
      _listRequest.max = Constant.ROW_PER_PAGE;

      _deliveryListResponse = await _deliveryContentService.list(_accessToken, _listRequest);
    if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: _deliveryContentService,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final DeliveryContentServiceV2 deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  int _selectedStep = 0;
  StepState _pickupStepState = StepState.indexed;
  StepState _dropOffStepState = StepState.indexed;
  bool _dropOffInactive = false;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    _stepState(post.status);

    return Container(
        padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child: new Wrap(
            children: <Widget>[
              ListTile(
                title: //Text('Way ID: ${post.trackingId}', style: TextStyle(color: Colors.white),),
                _createText(post.status, post.trackingId, context, post),
                subtitle: Text(tr(LocaleKeys.lblDeliCreateDropOffDate)+ ' ' + (post.dropOffDate!=null?new DateFormat('dd/MM/yyyy').format(post.dropOffDate.toLocal()): '../../....'),
                  // style: TextStyle(color: Colors.blueGrey[600]),
                ),
                onTap: post.customerName==null && post.customerAddress==null? null : () async {
                  bool result = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => DeliveryDetail(
                          wayId: post.trackingId,
                          onChange: (){
                            widget.onChange();
                          },
                        ),
                      )
                  );
                  if( result!=null && result ){
                    widget.onChange();
                  }
                },
                trailing: post.customerName==null && post.customerAddress==null?Icon(Icons.hourglass_top_sharp,
                  // color: Colors.yellow[700],
                ):Icon(Icons.arrow_forward_sharp,
                  // color: Colors.blueGrey[600],
                ),
              ),

              Stepper(
                steps: [
                  // Step(
                  //   isActive: true,
                  //   state: _pickupStepState,
                  //   title: Text('${post.osName}'),
                  //   subtitle: Text('${tr(LocaleKeys.lblSenderMobile)} ${post.osPrimaryPhone}'),
                  //   content: GestureDetector(
                  //     child: Text(
                  //       '${tr(LocaleKeys.lblDeliCreatePickup)} ${post.osAddress}',
                  //     ),
                  //     onTap: () async {
                  //       makeCall(post.osPrimaryPhone);
                  //     },
                  //   ),
                  // ),
                  Step(
                    isActive: _dropOffInactive,
                    state: _dropOffStepState,
                    title: Text('${post.customerName}'),
                    subtitle: Text('${tr(LocaleKeys.lblRecipientMobile)} ${post.customerPrimaryPhone}'),
                    content: GestureDetector(
                        child: Text('${tr(LocaleKeys.lblDeliCreateDropOff)} ${post.customerAddress} ${post.customerAddressType=="Highway Gate"? ", "+post.customerComment:""}'),
                      onTap: () async {
                        makeCall(post.customerPrimaryPhone);
                      }
                    ),
                  ),
                ],
                currentStep: 0,
                controlsBuilder: (BuildContext context,ControlsDetails details) => Container(),
                type: StepperType.vertical,
                physics: ClampingScrollPhysics(),
                onStepTapped: (step){setState(() {
                  _selectedStep = step;
                });},
              ),
              ListTile(
                title: LinearProgressIndicator(
                    backgroundColor: Colors.blueGrey[200],
                    value: double.parse(_completedPercent(post)[0]),
                    valueColor: AlwaysStoppedAnimation<Color>(Theme.of(context).primaryColor),
                ),
                leading: IconButton(icon: Icon(Icons.location_on_sharp,
                  // color: Colors.blueGrey[600],
                ),
                onPressed: (){
                  if( post.osLongitude != null && post.osLatitude!=null){
                    List<Map<String,dynamic>> items=[
                      {
                        'latitude':post.osLatitude,
                        'longitude':post.osLongitude,
                        'guid':post.osGUID,
                        'name':post.osName,
                        'phone':post.osAddress,
                        'employee':false
                      }
                    ];
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => TrackingMap(
                            startLocation: google.LatLng(post.osLatitude,post.osLongitude),
                            items: items,
                            title: tr(LocaleKeys.lblDeliCreatePickup),
                            isEmployee: false,
                            navigate: false,
                            startRoute: false,
                          ),
                        )
                    );
                  }
                },
              ),
                trailing: IconButton(icon: Icon(Icons.location_on_sharp,
                  // color: Colors.blueGrey[600],
                ),
                  onPressed: (){
                    if( post.customerLongitude != null && post.customerLatitude!=null){
                      List<Map<String,dynamic>> items=[
                        {
                          'latitude':post.customerLatitude,
                          'longitude':post.customerLongitude,
                          'guid':post.customerGUID,
                          'name':post.customerName,
                          'phone':post.customerAddress,
                          'employee':false
                        }
                      ];
                      Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) => TrackingMap(
                              startLocation: google.LatLng(post.customerLatitude,post.customerLongitude),
                              items: items,
                              title: tr(LocaleKeys.lblDeliCreateDropOff),
                              isEmployee: false,
                              navigate: false,
                              startRoute: false,
                            ),
                          )
                      );
                    }
                  },
                ),
                subtitle: Text('${_completedPercent(post)[1]}/${_getTotalDistance(post.osLatitude, post.osLongitude, post.customerLatitude, post.customerLongitude)??'Unknown'}'),
                // trailing: Icon(_expand? Icons.keyboard_arrow_up_sharp : Icons.keyboard_arrow_down_sharp, color: Colors.white,),
              ),
              ListTile(
                title: Text(
                  '${tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethod)} > ${post.deliveryMethods=='Regular'?tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodRegular):tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethod24hr)}',
                ),
                // subtitle: Text('${post.deliveryMethods}'),
              ),
              widget.authorityResponse.authorities!=null
                  && (widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)
                  && post.refundInAdvanced!=null
                  && post.refundInAdvanced
                  &&(post.totalItemPrice+post.promotionAmount)>0?ListTile(
                    title: Text(
                      tr(LocaleKeys.lblRequiredToRefund),
                      style: TextStyle(color: Colors.red[600]),
                    ),
                    subtitle: Text('${NumberFormat('#,###').format(post.totalItemPrice+post.promotionAmount)} Ks'),
              ) : Container(),
            ],
          ),
        )
    );
  }

  Icon _createStatusIcon(String status ){

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      return Icon(Icons.get_app_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY){
      return Icon(Icons.upload_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_COMPLETED ){
      return Icon(Icons.done_all, color: Colors.white);
    }else if( status == Constant.STATUS_CANCELED ){
      return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_RETRY ){
      return Icon(Icons.refresh, color: Colors.white);
    }else if( status == Constant.STATUS_RETURN ){
      return Icon(Icons.backspace_sharp, color: Colors.white);
    }else
      return Icon(Icons.view_list, color: Colors.white);

    // if( status == Constant.STATUS_CREATED || status == Constant.STATUS_PICKUP_CREATED){
    //   return Icon(Icons.view_list, color: Colors.white);
    // }else if( status == Constant.STATUS_ASSIGNED || status == Constant.STATUS_PICKUP_SCHEDULED){
    //   return Icon(Icons.person_pin_circle, color: Colors.white);
    // }else if( status == Constant.STATUS_ON_WAY || status == Constant.STATUS_PICKUP_ON_WAY){
    //   return Icon(Icons.local_shipping_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_COMPLETED ){
    //   return Icon(Icons.done_all, color: Colors.white);
    // }else if( status == Constant.STATUS_CANCELED ){
    //   return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_RETRY ){
    //   return Icon(Icons.refresh, color: Colors.white);
    // }else if( status == Constant.STATUS_RETURN ){
    //   return Icon(Icons.backspace_sharp, color: Colors.white);
    // }else
    //   return Icon(Icons.view_list, color: Colors.white);

  }

  Text _createText(String status, String tracking, BuildContext context, DeliveryListItem post ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }

    if( post.customerName==null && post.customerAddress==null ){
      statusColor = Colors.yellow[700];
      title =  tr(LocaleKeys.lblDeliParcelWaiting);
    }

    return Text(
      '$title\n${tr(LocaleKeys.lblWayID)} $tracking',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }

  StepState _stepState(String status){
    if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.indexed;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CANCELED ){
      _pickupStepState = StepState.error;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.indexed;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( status == Constant.STATUS_COMPLETED){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( status == Constant.STATUS_RETURN){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.error;
      _selectedStep = 1;
    }
    return _pickupStepState;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  String _getTotalDistance(double lat1, double lon1, double lat2, double lon2 ){
    if( lat1 == null || lon1==null || lat2==null || lon2==null)
      return null;
    final Distance distance = Distance();
    return '${distance.as(LengthUnit.Kilometer, LatLng( lat1, lon1 ), LatLng( lat2, lon2 ))}km';
  }

  List<String> _completedPercent( DeliveryListItem post ){
    List<String> values = [];
    final Distance distance = Distance();
    double totalDistance;
    double travelDistance;
    double _completed = 0.0;

    if( post.osLatitude != null && post.osLongitude!=null && post.customerLatitude!=null && post.customerLongitude!=null)
      totalDistance = distance.as(LengthUnit.Kilometer, LatLng( post.osLatitude, post.osLongitude), LatLng( post.customerLatitude, post.customerLongitude));

    if( post.status == Constant.STATUS_PICKUP_CREATED || post.status == Constant.STATUS_PICKUP_SCHEDULED){
      values.add('0.0');
      values.add('0.0km');
      return values;
    }else if( post.status == Constant.STATUS_PICKUP_ON_WAY){
      travelDistance = totalDistance!=null? totalDistance/4 : 0;
      values.add('0.25');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else if( post.status == Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY){
      travelDistance = totalDistance!=null? totalDistance/2 : 0;
      values.add('0.5');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else if( post.status == Constant.STATUS_ON_WAY){
      if( post.dropOffPersonLatitude != null && post.dropOffPersonLongitude!=null && post.customerLatitude!=null && post.customerLongitude!=null)
        travelDistance = distance.as(LengthUnit.Kilometer, LatLng( post.dropOffPersonLatitude, post.dropOffPersonLongitude), LatLng( post.customerLatitude, post.customerLongitude));
      if( totalDistance!=null && totalDistance>0 && travelDistance!=null && travelDistance>0){
        _completed = travelDistance / totalDistance;
      }
      if( _completed>0.5){
        values.add('$_completed');
        values.add( travelDistance.toStringAsFixed(2)+'km' );
        return values;
      }else{
        travelDistance = totalDistance!=null? totalDistance/2 : 0;
        values.add('0.5');
        values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
        return values;
      }
    }else if( post.status == Constant.STATUS_COMPLETED){
      travelDistance = totalDistance!=null? totalDistance : 0;
      values.add('1.0');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else{
      values.add('0.0');
      values.add( 'Unknown' );
      return values;
    }
  }
}