import 'package:badges/badges.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/price_list_response.dart';
import 'package:qq_delivery/delivery/service/price_method_service.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';


class PriceList extends StatefulWidget {
  PriceList({Key key}) : super(key: key);
  @override
  _PriceListState createState() => _PriceListState();
}

class _PriceListState extends State<PriceList>{

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  List<PriceListItem> _priceListItems = [];
  PriceMethodService _priceMethodService = new PriceMethodService();
  String _lang = 'my';


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData() async{
    _lang = await getLanguage( );
    PriceListResponse response = await _priceMethodService.priceList(_lang);
    if( _priceListItems.isNotEmpty)
      _priceListItems.removeRange(0, _priceListItems.length);

    response.items.forEach((element) {
      _priceListItems.add(element);
    });
    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => PriceListItemView(
        post: _priceListItems[i],
        index: i,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _priceListItems.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Text('Price list', style: TextStyle(color: Colors.white)),
        ),
        body: SmartRefresher(
          controller: _refreshController,
          enablePullUp: false,
          child: buildContent(context),
          physics: BouncingScrollPhysics(),
          header: CustomRefreshIndicator(),
          // scroll down
          onRefresh: () async {
            await _getLoadData( );
            if (mounted) setState(() {});
            _refreshController.refreshCompleted();
          },
        )
    );
  }

}

class PriceListItemView extends StatefulWidget {

  PriceListItemView({Key key, @required this.post, this.index}) : super(key: key);
  final PriceListItem post;
  final int index;

  @override
  _PriceListItemViewState createState() => _PriceListItemViewState();

}

class _PriceListItemViewState extends State<PriceListItemView> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Card(
            elevation: 0,
            child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment:CrossAxisAlignment.start,
                children: <Widget>[
                  ListTile(
                    title: Text(
                      '${widget.post.name}',
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                    subtitle: Text(
                      "",
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                    trailing: Text(
                      '${NumberFormat('#,###').format(widget.post.basedPrice)}Ks',
                      // style: TextStyle(decoration: TextDecoration.lineThrough),
                    ),
                    leading: Icon(Icons.menu,
                      // color: Colors.blueGrey[600],
                    ),
                    onTap: () {
                      setState(() {
                        widget.post.showAll = !widget.post.showAll;
                      });
                    },
                  ),
                  widget.post.showAll? ListTile(
                    title: Text(
                      'From',
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                    subtitle: Text(
                      "${widget.post.townships}",
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                  ) : Container(),
                  widget.post.showAll? ListTile(
                    title: Text(
                      'To',
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                    subtitle: Text(
                      "${widget.post.differentDestination?widget.post.destinationTownships:widget.post.townships}",
                      // style: TextStyle(color: Theme.of(context).primaryColor,),
                    ),
                  ) : Container(),
                ]
            )
        )
    );
  }
}