
import 'dart:io';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/comment_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/comment_item.dart';
import 'package:qq_delivery/delivery/service/v2/comment_service.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:image/image.dart' as imglib;


class Comment extends StatefulWidget {
  Comment({Key key, this.personGuid, this.type, this.referenceGuid,
    this.sharedPrefs, this.accessToken}) : super(key: key);
  final String personGuid;
  final String type;
  final String referenceGuid;
  final SharedPreferences sharedPrefs;
  final String accessToken;

  @override
  _CommentState createState() => _CommentState();
}

class _CommentState extends State<Comment>{

  CommentListResponse _commentListResponse;
  CommentService _commentService = new CommentService();
  List<CommentItem> _items = [];
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  int _page = 0;
  bool _processing = false;
  final TextEditingController _controller_comment = new TextEditingController();


  @override
  void initState() {
    super.initState();
  }
  _getLoadData( bool refresh ) async{

    _commentListResponse = await _commentService.list(widget.accessToken, widget.type, widget.referenceGuid, _page, Constant.ROW_PER_PAGE);
    print(_commentListResponse);
    if( _commentListResponse!=null && _commentListResponse.items!=null){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _commentListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: TextFormField(
              textAlign: TextAlign.left,
              keyboardType: TextInputType.multiline,
              // minLines: 1,
              // maxLines: 5,
              decoration: InputDecoration(
                hintText: 'Write a comment',
                // hintStyle: TextStyle(fontSize: 14),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(8),
                  borderSide: BorderSide(
                    width: 1,
                    style: BorderStyle.none,
                  ),
                ),
                filled: true,
                contentPadding: EdgeInsets.all(4),
                fillColor: Theme.of(context).backgroundColor,
              ),
              controller: _controller_comment,
            ),
            actions: <Widget>[
              IconButton(
                icon: const Icon(Icons.send_sharp),
                tooltip: 'Send message',
                onPressed: () async{
                  if( _controller_comment.text.trim().isNotEmpty ) {
                    setState(() {
                      _processing = true;
                    });
                    CommentItem commentItem = new CommentItem();
                    commentItem.type = widget.type;
                    commentItem.referenceGuid = widget.referenceGuid;
                    commentItem.message = _controller_comment.text;
                    await _commentService.create(commentItem.toMap());
                    _controller_comment.text = "";
                    setState(() {
                      _processing = false;
                    });
                    _refreshController.requestRefresh();
                  }

                },
              ),
              IconButton(
                icon: const Icon(Icons.add_to_photos_rounded),
                tooltip: 'Attach photo',
                onPressed: () async{
                  String image = await _getImage(false);
                  if( image != null && image.length>100){
                    setState(() {
                      _processing = true;
                    });
                    CommentItem commentItem = new CommentItem();
                    commentItem.type = widget.type;
                    commentItem.referenceGuid = widget.referenceGuid;
                    commentItem.image = image;
                    await _commentService.create(commentItem.toMap());
                    setState(() {
                      _processing = false;
                    });
                    _refreshController.requestRefresh();

                  }
                },
              ),
            ],
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      shrinkWrap: true,
      reverse: false,
      padding: EdgeInsets.only(left: 4, right: 4),
      itemBuilder: (c, i) => CommentListItem(
        post: _items[i],
        sharedPrefs: widget.sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.1,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Future<String> _getImage( bool isCamera ) async {
    String selectedImage;
    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/gelivery_tmp';
        await Directory(tmpDir).create(recursive: true);

        String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
        File rotatedFile = await FlutterImageCompress.compressAndGetFile(
          image.path,
          photoPath2,
          quality: 72,
          rotate: 0,
        );
        imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
        String cropFilePath = '$tmpDir/cropped.jpg';
        File(cropFilePath)
          ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));

        File smallFile  =File(cropFilePath);
        String encodedString = encode( smallFile );
        smallFile.deleteSync(recursive: false);
        rotatedFile.deleteSync(recursive: false);
        setState(() {
          selectedImage = encodedString;
        });

      }
    }catch(e){}
    return selectedImage;
  }

}

  class CommentListItem extends StatefulWidget {

    CommentListItem({Key key, @required this.post, this.sharedPrefs}) : super(key: key);
    final CommentItem post;
    final SharedPreferences sharedPrefs;

    @override
    _CommentListItemState createState() => _CommentListItemState();

  }

class _CommentListItemState extends State<CommentListItem> {
  @override
  void initState() {
    super.initState();
  }
  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 0,
                  child: widget.post.image==null?ListTile(
                          title: Text(
                            widget.post.personName,
                          ),
                          subtitle: Text(widget.post.message),
                          leading: ClipOval(
                              child: Material(
                                elevation: 1,
                                color: Colors.transparent, // button color
                                child: InkWell(
                                  // splashColor: Colors.red, // inkwell color
                                  child: SizedBox(
                                    width: 42,
                                    height: 42,
                                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.personAvatar}', width: 42, height: 42, ),
                                  ),
                                ),
                              )
                          ),
                          trailing: Text(
                            new DateFormat('dd-MM-yyyy').format(widget.post.createdOn.toLocal())
                                ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                            '${new DateFormat('hh:mm a').format(widget.post.createdOn.toLocal())}'
                                : '${new DateFormat('dd/MM/yy').format(widget.post.createdOn.toLocal())}',
                            textAlign: TextAlign.center,
                          ),
                        ) : ListTile(
                          title: Text(
                            widget.post.personName,
                          ),
                          leading: ClipOval(
                              child: Material(
                                elevation: 1,
                                color: Colors.transparent, // button color
                                child: InkWell(
                                  // splashColor: Colors.red, // inkwell color
                                  child: SizedBox(
                                    width: 42,
                                    height: 42,
                                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.personAvatar}', width: 42, height: 42, ),
                                  ),
                                ),
                              )
                          ),
                          trailing: Text(
                            new DateFormat('dd-MM-yyyy').format(widget.post.createdOn.toLocal())
                                ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                            '${new DateFormat('hh:mm a').format(widget.post.createdOn.toLocal())}'
                                : '${new DateFormat('dd/MM/yy').format(widget.post.createdOn.toLocal())}',
                            textAlign: TextAlign.center,
                          ),
                          subtitle: GestureDetector(
                            child: ClipRRect(
                              borderRadius: BorderRadius.circular(24.0),
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.image}?size=300', fit: BoxFit.cover, height: 250,),
                            ),
                            onTap: () async{
                              if(widget.post.image!=null){
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) => FullScreenWrapper(
                                      imageProvider: NetworkImage( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.image}', ),
                                    ),
                                  ),
                                );
                              }
                            },
                          ),
                        ),

              ),
            ]
        )
    );
  }
}
