import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:qq_delivery/delivery/view/widget/address_widget.dart';
import 'package:shared_preferences/shared_preferences.dart';


class DeliveryAddress extends StatefulWidget {
  DeliveryAddress({Key key, @required this.title, @required this.addressType,
    this.ownerGuid, this.wayType}) : super(key: key);
  final String title;
  final String addressType;
  final String ownerGuid;
  final String wayType;
  @override
  _DeliveryAddressState createState() => _DeliveryAddressState();
}

class _DeliveryAddressState extends State<DeliveryAddress> {

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  bool _first = true;

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  AddressBookService _addressBookService = new AddressBookService();
  AddressBookListResponse _addressBookListResponse;
  List<AddressBookItem> _items = [];
  int _page = 0;

  final TextEditingController _controllerStreet = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  List<bool> _selectionsDropOff = List.generate(4, (_)=> false);
  int _selectedIndex = 0;
  String _keyword = '';
  final _searchQuery = new TextEditingController();
  Timer _debounce;


  @override
  void initState() {
    super.initState();
    _selectionsDropOff[0]=true;
    _searchQuery.addListener(_onSearchChanged);
  }
  @override
  void deactivate() {
    super.deactivate( );
  }
  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(seconds: 3), () {
      _keyword = _searchQuery.text.trim();
      if( (_keyword.length ==0 || _keyword.length >3) && !_refreshController.isRefresh){
        _refreshController.requestRefresh();
      }
    });
  }

  Future<void> _getLoadData( bool refresh ) async{
    if( _first) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      // _first = false;
    }
    // if( _currentUser.guid==null) return;

    if( _selectedIndex == 0 ) {
      if( _first && widget.addressType =='Customer' ) {
      }else{
        _addressBookListResponse = await _addressBookService.listAll(
            new AddressBookRequest(
                addressType: widget.addressType,
                osGuid: widget.ownerGuid,
                first: _page,
                max: Constant.ROW_PER_PAGE,
                keyword: _keyword
            )
        );
      }
    }else if( _selectedIndex == 1){
      if(widget.wayType == null ) {
        _addressBookListResponse = await _addressBookService.listPublic(
            new AddressBookRequest(
                addressType: "Highway Gate",
                osGuid: null,
                first: _page,
                max: Constant.ROW_PER_PAGE,
                keyword: _keyword
            )
        );
      }else{
        if(widget.wayType.toLowerCase() == 'pickup' ) {
          _addressBookListResponse = await _addressBookService.listPOIForPickup(
              new AddressBookRequest(
                  addressType: "Highway Gate",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else if(widget.wayType.toLowerCase() == 'deliver' ) {
          _addressBookListResponse = await _addressBookService.listPOIForDelivery(
              new AddressBookRequest(
                  addressType: "Highway Gate",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else{
          _addressBookListResponse = await _addressBookService.listPublic(
              new AddressBookRequest(
                  addressType: "Highway Gate",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }
      }
    }else if( _selectedIndex == 2){
      if(widget.wayType == null ) {
        _addressBookListResponse = await _addressBookService.listPublic(
            new AddressBookRequest(
                addressType: "Post Office",
                osGuid: null,
                first: _page,
                max: Constant.ROW_PER_PAGE,
                keyword: _keyword
            )
        );
      }else{
        if(widget.wayType.toLowerCase() == 'pickup' ) {
          _addressBookListResponse = await _addressBookService.listPOIForPickup(
              new AddressBookRequest(
                  addressType: "Post Office",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else if(widget.wayType.toLowerCase() == 'deliver' ) {
          _addressBookListResponse = await _addressBookService.listPOIForDelivery(
              new AddressBookRequest(
                  addressType: "Post Office",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else{
          _addressBookListResponse = await _addressBookService.listPublic(
              new AddressBookRequest(
                  addressType: "Post Office",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }
      }
    }else if( _selectedIndex == 3){
      if(widget.wayType == null ) {
        _addressBookListResponse = await _addressBookService.listPublic(
            new AddressBookRequest(
                addressType: "POI",
                osGuid: null,
                first: _page,
                max: Constant.ROW_PER_PAGE,
                keyword: _keyword
            )
        );
      }else{
        if(widget.wayType.toLowerCase() == 'pickup' ) {
          _addressBookListResponse = await _addressBookService.listPOIForPickup(
              new AddressBookRequest(
                  addressType: "POI",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else if(widget.wayType.toLowerCase() == 'deliver' ) {
          _addressBookListResponse = await _addressBookService.listPOIForDelivery(
              new AddressBookRequest(
                  addressType: "POI",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }else{
          _addressBookListResponse = await _addressBookService.listPublic(
              new AddressBookRequest(
                  addressType: "POI",
                  osGuid: null,
                  first: _page,
                  max: Constant.ROW_PER_PAGE,
                  keyword: _keyword
              )
          );
        }
      }
    }

    if( refresh && _items.isNotEmpty)
      _items.removeRange(0, _items.length);
    if( _addressBookListResponse!=null && _addressBookListResponse.items!=null){
      _addressBookListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if( _first ) {
      _first = false;
      if( _addressBookListResponse == null && widget.addressType !='Customer'){
        setState(() {
          _selectedIndex = 1;
          for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
            if (buttonIndex == _selectedIndex) {
              _selectionsDropOff[buttonIndex] = true;
            } else {
              _selectionsDropOff[buttonIndex] = false;
            }
          }
        });

        Timer(const Duration(milliseconds: 700), () {
          _refreshController.requestRefresh();
        });
      }
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
        ),
        body: SmartRefresher(
          controller: _refreshController,
          enablePullUp: true,
          child: buildContent(context),

          physics: BouncingScrollPhysics(),
          header: CustomRefreshIndicator(),
          footer: ClassicFooter(
            loadStyle: LoadStyle.ShowWhenLoading,
            completeDuration: Duration(milliseconds: 500),
          ),
          // scroll down
          onRefresh: () async {
            _page = 0;
            await _getLoadData( true);
            if (mounted) setState(() {});
            _refreshController.refreshCompleted();
          },
          // scroll up
          onLoading: () async {
            ++_page;
            await _getLoadData( false );
            if (mounted) setState(() {});
            _refreshController.loadComplete();
          },
        )
    );
  }

  Widget _topWidget(BuildContext context){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, bottom: 8.0, left: 0.0, right: 0.0),
      child: Card(
        // color: Colors.white,
        elevation: 0,
        child: new Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment:CrossAxisAlignment.start,
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top: 16, left: 16, bottom: 0, right: 16),
              child: TextField(
                textAlign: TextAlign.center,
                keyboardType: TextInputType.text,
                decoration: InputDecoration(
                    hintText: 'Phone or name',
                    hintStyle: TextStyle(fontSize: 14),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(16),
                      borderSide: BorderSide(
                        width: 1,
                        style: BorderStyle.none,
                      ),
                    ),
                    filled: true,
                    contentPadding: EdgeInsets.all(8),
                    fillColor: Theme.of(context).backgroundColor,
                    suffixIcon: _searchQuery.text.length>0? IconButton(
                      icon: Icon(Icons.close_sharp,
                          // color: Theme.of(context).primaryColorDark
                      ),
                      onPressed: (){
                        setState(() {
                          _searchQuery.text = '';
                          _keyword = '';
                        });
                        _refreshController.requestRefresh();
                      },
                    ): Icon(Icons.search_sharp),
                ),
                controller: _searchQuery,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 8.0, left: 16, right: 16),
              child: RaisedButton.icon(
                color: Theme.of(context).backgroundColor,
                // textColor: Colors.white,
                disabledColor: Colors.grey,
                disabledTextColor: Colors.black,
                padding: EdgeInsets.all(8.0),
                // splashColor: Colors.blueAccent,
                onPressed: () async{
                  AddressBookItem addressBookItem = await Navigator.push(
                    context, MaterialPageRoute(
                      builder: (context) => AddressWidget(
                        controllerStreet: _controllerStreet,
                        controllerAddress: _controllerAddress,
                        embedded: false,
                        title: tr(LocaleKeys.btnNewAddress),
                        onChange: ( GeneralAddressItem addressItemState,
                            GeneralAddressItem addressItemTownship,
                            GeneralAddressItem addressItemStreet ) {},
                      ),
                    ),
                  );

                  if( addressBookItem != null) {
                    addressBookItem.addressType = widget.addressType;
                    Navigator.pop(context, addressBookItem);
                  }
                },
                icon: Icon(Icons.add_location_sharp,
                  // color: Theme.of(context).primaryColor,
                ),
                label: Text(
                  tr(LocaleKeys.btnNewAddress),
                    style: Theme.of(context).textTheme.headline6,
                ),
                shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(32.0)),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: ToggleButtons(
                children: <Widget>[
                  Container(
                    padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                    child: Text(
                      tr(LocaleKeys.lblAddressBook),
                      style: TextStyle(fontSize: 12.0, fontFamily: 'Roboto'),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  Container(
                    padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                    child: Text(
                      tr(LocaleKeys.lblHighwayGates),
                      style: TextStyle(fontSize: 12.0, fontFamily: 'Roboto'),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  Container(
                    padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                    child: Text(
                      tr(LocaleKeys.lblPostOffices),
                      style: TextStyle(fontSize: 12.0, fontFamily: 'Roboto'),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  Container(
                    padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                    child: Text(
                      tr(LocaleKeys.lblPOI),
                      style: TextStyle(fontSize: 12.0, fontFamily: 'Roboto'),
                      textAlign: TextAlign.center,
                    ),
                  ),
                ],
                onPressed: (int index) {
                  setState(() {
                    _selectedIndex = index;
                    for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
                      if (buttonIndex == index) {
                        _selectionsDropOff[buttonIndex] = true;
                      } else {
                        _selectionsDropOff[buttonIndex] = false;
                      }
                    }
                  });
                  _refreshController.requestRefresh();
                },
                isSelected: _selectionsDropOff,
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.builder(
      reverse: false,
      padding: EdgeInsets.only(left: 8, right: 8),
      itemBuilder: (c, i) => i==0? _topWidget(c) : AddressBookListItem(
        post: _items[i-1],
        authorityResponse: _authorityResponse,
        addressBookService: _addressBookService,
        isEnd: i == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      itemCount: _items.length + 1,
    );
  }
}

class AddressBookListItem extends StatefulWidget {
  AddressBookListItem({Key key, @required this.post,
    @required this.authorityResponse, @required this.addressBookService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final AddressBookItem post;
  final AuthorityResponse authorityResponse;
  final AddressBookService addressBookService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _AddressBookItemState createState() => _AddressBookItemState();

}

class _AddressBookItemState extends State<AddressBookListItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 0,
                  child: Wrap(
                      children: <Widget>[
                        ListTile(
                          title: Text(widget.post.name,
                            style: Theme.of(context).textTheme.headline6,
                          ),
                          subtitle: Text(
                            '${widget.post.address}, ${widget.post.street}, ${widget.post.townshipName}',
                            // style: TextStyle(color: Colors.black),
                          ),
                          leading: Icon(Icons.location_on_sharp,
                              // color: Colors.blueGrey[600]
                          ),
                          trailing: Icon( Icons.radio_button_off_sharp,
                            // color: Colors.blueGrey[600],
                          ),
                          onTap: (){
                            Navigator.pop( context, widget.post );
                          },
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
