import 'dart:async';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/service/price_method_service.dart';
import 'package:qq_delivery/delivery/service/promotion_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'delivery_address.dart';
import 'delivery_item.dart';


class DeliveryCreate extends StatefulWidget {
  DeliveryCreate({Key key, this.onChange, this.addressBookItemSender,
  this.addressBookItemRecipient}) : super(key: key);
  final Function onChange;
  final AddressBookItem addressBookItemSender;
  final AddressBookItem addressBookItemRecipient;
  @override
  _DeliveryCreateState createState() => _DeliveryCreateState();
}

class _DeliveryCreateState extends State<DeliveryCreate> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;

  final TextEditingController _controller_promoCode = new TextEditingController();
  final TextEditingController _controller_senderName = new TextEditingController();
  final TextEditingController _controller_senderAddress = new TextEditingController();
  final TextEditingController _controller_senderPrimaryPhone = new TextEditingController();
  final TextEditingController _controller_senderOtherPhones = new TextEditingController();
  final TextEditingController _controller_senderComment = new TextEditingController();
  DateTime _senderDate = new DateTime.now();
  final TextEditingController _controller_recipientAddress = new TextEditingController();
  final TextEditingController _controller_recipientName = new TextEditingController();
  final TextEditingController _controller_recipientPrimaryPhone = new TextEditingController();
  final TextEditingController _controller_recipientOtherPhones = new TextEditingController();
  final TextEditingController _controller_recipientDate = new TextEditingController();
  final TextEditingController _controller_edit_deliCharges = new TextEditingController();
  final TextEditingController _controller_edit_charges_by_admin = new TextEditingController();
  DateTime _recipientDate = new DateTime.now();
  List<bool> _selections = List.generate(6, (_)=> false);
  List<DateTime> _pickupDateLabels = <DateTime>[DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now()];
  List<DateTime> _dropOffDateLabels = <DateTime>[DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now()];
  List<bool> _selectionsDropOff = List.generate(6, (_)=> false);
  DeliveryItem _deliveryItem = new DeliveryItem(
    name: 'Parcel',
    weight: 2,
    width: 2,
    height: 2,
    length: 2,
    priceOption: "Unpaid",
    price: 0,
    chargeOption: "Unpaid",
    charges: 0,
    forShowCharges: 0,
    fragile: false,
    insurance: false,
  );
  User _currentUser = new User();
  bool _isFirst = true;
  bool _requiredToRefund = false;
  bool _allow_cash_on_advanced = false;

  AddressBookItem _addressBookItemSender;
  AddressBookItem _addressBookItemRecipient;
  bool _itemDetailAlreadyCheck = false;
  bool _itemDetailError = false;
  double _promoAmount = 0;
  bool _hasPromoCode = false;
  bool _editDeliCharges = false;
  String _lang = Constant.LANGUAGE_CODE_MY;
  double _deliveryCharges;
  bool _editChargesByAdmin = false;
  AuthorityResponse _authorityResponse = new AuthorityResponse();


  @override
  void initState() {
    super.initState();
    setState( () {
      _selections[0]=true;
      _selectionsDropOff[0]=true;
      _recipientDate = new DateTime( _senderDate.year, _senderDate.month, _senderDate.day + 1 );

      int dayPlus = (_senderDate.toLocal().hour>14? 1 : 0);
      for( int i=0; i< _pickupDateLabels.length; i++){
        _pickupDateLabels[i] = new DateTime( _senderDate.year, _senderDate.month, _senderDate.day + i + dayPlus );
        _dropOffDateLabels[i] = new DateTime( _recipientDate.year, _recipientDate.month, _recipientDate.day + i + dayPlus );
      }
      _senderDate = _pickupDateLabels[0];
      _recipientDate = _dropOffDateLabels[0];

      _addressBookItemRecipient = widget.addressBookItemRecipient;
      _addressBookItemSender = widget.addressBookItemSender;
      if( widget.addressBookItemRecipient != null){
        _checkPrice( );
      }
    });

    _loadCurrentUser( );
  }

  void _loadCurrentUser( ) async{

    if( _isFirst ){
      _lang = await getLanguage( );
      _isFirst = false;
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      await _availableDates();
      if(_authorityResponse.authorities !=null
          && _authorityResponse.authorities.contains("MANAGER_WRITE")){
        setState(() {
          _allow_cash_on_advanced = true;
        });
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          key: _scaffoldKey,
          title: Text(tr(LocaleKeys.lblDeliCreateTitle), style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Center(
            child: SingleChildScrollView(
              child: Container(
                margin: const EdgeInsets.only(top: 16.0, bottom: 16.0, left: 8.0, right: 8.0),
                child: Card(
                  color: Colors.white,
                  elevation: 8,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment:CrossAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        margin: EdgeInsets.only(top: 0, bottom: 32),
                        child: new Image.asset("assets/images/logo_rectangle.jpg"),
                      ),
                      Container(
                        padding: EdgeInsets.all(16.0),
                        child: TextField(
                          textAlign: TextAlign.center,
                          keyboardType: TextInputType.text,
                          decoration: InputDecoration(
                            hintText: tr(LocaleKeys.lblDeliCreatePickup),// 'Pickup from',
                            hintStyle: TextStyle(fontSize: 14),
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(16),
                              borderSide: BorderSide(
                                  width: 1,
                                  style: BorderStyle.solid,
                                  color: Theme.of(context).primaryColor
                              ),
                            ),
                            filled: true,
                            contentPadding: EdgeInsets.all(8),
                            fillColor: Theme.of(context).backgroundColor,
                            prefixIcon: Icon(Icons.upload_sharp,),
                            suffixIcon : IconButton(
                              icon: Icon(Icons.add_location_sharp, color: Theme.of(context).primaryColorDark),
                              // onPressed: (){print('Icon press');},
                            ),
                          ),
                          onTap: () async{
                            FocusScope.of(context).requestFocus(new FocusNode());
                            AddressBookItem addressBookItem = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => DeliveryAddress(
                                  title: tr(LocaleKeys.lblDeliCreatePickup),
                                  addressType: 'OS',
                                ),
                              ),
                            );
                            if( addressBookItem != null) {
                              _addressBookItemSender = addressBookItem;
                              _controller_senderAddress.text = _addressBookItemSender.address
                                  +(_addressBookItemSender.street!=null && _addressBookItemSender.street.trim().isNotEmpty? ','+_addressBookItemSender.street.trim() : '')
                                  +(_addressBookItemSender.townshipName!=null && _addressBookItemSender.townshipName.trim().isNotEmpty? ','+_addressBookItemSender.townshipName.trim() : '');
                              _controller_senderPrimaryPhone.text = addressBookItem.primaryPhone??'';
                              _controller_senderOtherPhones.text = addressBookItem.otherPhones??'';
                              _controller_senderName.text = addressBookItem.name??'';
                              if(_controller_senderPrimaryPhone.text.trim().isEmpty){
                                _controller_senderPrimaryPhone.text = _currentUser.phone;
                              }

                              if( _addressBookItemRecipient != null && _addressBookItemRecipient.townshipGuid!=null)
                                _checkPrice();
                            }
                          },
                          controller: _controller_senderAddress,
                        ),
                      ),
                      Container(
                        padding: EdgeInsets.all(16.0),
                        child: TextField(
                          textAlign: TextAlign.center,
                          keyboardType: TextInputType.text,
                          decoration: InputDecoration(
                            hintText: tr(LocaleKeys.lblDeliCreateDropOff),//Deliver to
                            hintStyle: TextStyle(fontSize: 14),
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(16),
                              borderSide: BorderSide(
                                  width: 1,
                                  style: BorderStyle.solid,
                                  color: Theme.of(context).primaryColor
                              ),
                            ),
                            filled: true,
                            contentPadding: EdgeInsets.all(8),
                            fillColor: Theme.of(context).backgroundColor,
                            prefixIcon: Icon(Icons.download_sharp,),
                            suffixIcon : IconButton(
                              icon: Icon(Icons.add_location_sharp, color: Theme.of(context).primaryColorDark),
                              onPressed: (){print('Icon press');},
                            ),
                          ),
                          onTap: ()async{
                            FocusScope.of(context).requestFocus(new FocusNode());
                            AddressBookItem addressBookItem = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => DeliveryAddress(
                                  title: tr(LocaleKeys.lblDeliCreateDropOff),
                                  addressType: 'Customer',
                                ),
                              ),
                            );

                            if( addressBookItem != null) {
                              _addressBookItemRecipient = addressBookItem;
                              _controller_recipientAddress.text = _addressBookItemRecipient.address
                                  +(_addressBookItemRecipient.street!=null && _addressBookItemRecipient.street.trim().isNotEmpty? ','+_addressBookItemRecipient.street.trim() : '')
                                  +(_addressBookItemRecipient.townshipName!=null && _addressBookItemRecipient.townshipName.trim().isNotEmpty? ','+_addressBookItemRecipient.townshipName.trim() : '');
                              _controller_recipientPrimaryPhone.text = addressBookItem.primaryPhone??'';
                              _controller_recipientOtherPhones.text = addressBookItem.otherPhones??'';
                              if(addressBookItem.addressType != 'Post Office'
                                  && addressBookItem.addressType != 'Highway Gate'
                                  && addressBookItem.addressType != 'POI')
                              _controller_recipientName.text = addressBookItem.name??'';

                              if( _addressBookItemSender != null && _addressBookItemSender.townshipGuid!=null)
                              _checkPrice();
                            }
                          },
                          controller: _controller_recipientAddress,
                        ),
                      ),
                      Container(
                        margin: EdgeInsets.only(top: 0, bottom: 32),
                        child: new SafeArea(
                            top: false,
                            bottom: false,
                            child: new Form(
                              key: _formKey,
                              autovalidateMode: AutovalidateMode.onUserInteraction,
                              child: new Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Card(
                                    // color: Colors.white,
                                    elevation: 0,
                                    child: new Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisSize: MainAxisSize.min,
                                    children: <Widget>[
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                        child: Text('${tr(LocaleKeys.lblDeliCreatePickupDate)} ${new DateFormat('dd/MM/yyyy').format(_senderDate.toLocal())}'),
                                      ),
                                      Container(
                                        padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                        child: ToggleButtons(
                                          children: <Widget>[
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[0]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[1]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[2]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[3]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[4]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: FlatButton(
                                                onPressed: () {
                                                  setState(() {
                                                    for (int buttonIndex = 0; buttonIndex < _selections.length; buttonIndex++) {
                                                      if (buttonIndex == _selections.length-1) {
                                                        _selections[buttonIndex] = true;
                                                      } else {
                                                        _selections[buttonIndex] = false;
                                                      }
                                                    }
                                                  });
                                                  _chooseDate(context, new DateFormat('yyyy-MM-dd').format(_senderDate.toLocal()), true);
                                                  },
                                                // color: Theme.of(context).backgroundColor,
                                                child: Text(
                                                  tr(LocaleKeys.lblAntherDay),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              )
                                            ),
                                          ],
                                          onPressed: (int index) {
                                            setState(() {
                                              _senderDate = _pickupDateLabels[index];
                                              for (int buttonIndex = 0; buttonIndex < _selections.length; buttonIndex++) {
                                                if (buttonIndex == index) {
                                                  _selections[buttonIndex] = true;
                                                } else {
                                                  _selections[buttonIndex] = false;
                                                }
                                              }
                                            });
                                          },
                                          isSelected: _selections,
                                        ),
                                      ),
                                      _currentUser.roleName!=Constant.ROLE_NAME_OS? Container(
                                        padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblSenderName),
                                              labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          validator: (val) => val.isEmpty ? 'sender name required!' : null,
                                          controller: _controller_senderName,
                                          keyboardType: TextInputType.text,
                                        ),
                                      ) : Container(),
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                            hintText: '09xxxxxxxxx',
                                            labelText: tr(LocaleKeys.lblSenderMobile),
                                            labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                                          controller: _controller_senderPrimaryPhone,
                                          keyboardType: TextInputType.phone,
                                        ),
                                      ),
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                            hintText: "Use \",\" for multiple numbers",
                                            labelText: tr(LocaleKeys.lblSenderOtherPhones),
                                            labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          controller: _controller_senderOtherPhones,
                                          keyboardType: TextInputType.text,
                                        ),
                                      ),
                                    ]
                                    )
                                  ),
                                  Card(
                                      // color: Theme.of(context).backgroundColor,
                                      elevation: 0,
                                      child: new Column(
                                      crossAxisAlignment: CrossAxisAlignment.start,
                                      mainAxisSize: MainAxisSize.min,
                                      children: <Widget>[
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                          child: Text('${tr(LocaleKeys.lblDeliCreateDropOffDate)} ${new DateFormat('dd/MM/yyyy').format(_recipientDate.toLocal())}'),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                          child: ToggleButtons(
                                            children: <Widget>[
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                    _dayToLabel(_dropOffDateLabels[0]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[1]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[2]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[3]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[4]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                  padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                  child: FlatButton(
                                                    onPressed: () {
                                                      setState(() {
                                                        for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
                                                          if (buttonIndex == _selections.length-1) {
                                                            _selectionsDropOff[buttonIndex] = true;
                                                          } else {
                                                            _selectionsDropOff[buttonIndex] = false;
                                                          }
                                                        }
                                                      });
                                                      _chooseDate(context, new DateFormat('yyyy-MM-dd').format(_recipientDate.toLocal()), false);
                                                    },
                                                    // color: Theme.of(context).backgroundColor,
                                                    child: Text(
                                                      tr(LocaleKeys.lblAntherDay),
                                                      style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                      textAlign: TextAlign.center,
                                                    ),
                                                  )
                                              ),
                                            ],
                                            onPressed: (int index) {
                                              setState(() {
                                                _recipientDate = _dropOffDateLabels[index];
                                                for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
                                                  if (buttonIndex == index) {
                                                    _selectionsDropOff[buttonIndex] = true;
                                                  } else {
                                                    _selectionsDropOff[buttonIndex] = false;
                                                  }
                                                }
                                              });
                                            },
                                            isSelected: _selectionsDropOff,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblRecipientName),
                                                labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            validator: (val) => val.isEmpty ? 'Name required!' : null,
                                            controller: _controller_recipientName,
                                            keyboardType: TextInputType.text,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '09xxxxxxxxx',
                                              labelText: tr(LocaleKeys.lblRecipientMobile),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                                            controller: _controller_recipientPrimaryPhone,
                                            keyboardType: TextInputType.phone,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: "Use \",\" for multiple numbers",
                                              labelText: tr(LocaleKeys.lblRecipientOtherPhones),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            // initialValue: '',
                                            controller: _controller_recipientOtherPhones,
                                            keyboardType: TextInputType.text,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblRemark),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            controller: _controller_senderComment,
                                            keyboardType: TextInputType.text,
                                            // maxLines: null,
                                            // minLines: 1,
                                          ),
                                        ),
                                      ]
                                    )
                                  ),
                                  Container(
                                    padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                    child: Card(
                                    color: _itemDetailError? Colors.red[100] : (_itemDetailAlreadyCheck? Colors.green[50] : Theme.of(context).backgroundColor),
                                    elevation: 1,
                                    child: ListTile(
                                        title: Text(tr(LocaleKeys.lblItemDetail)),
                                        subtitle: Text(_itemDetailAlreadyCheck? '${_deliveryItem.weight}kg${_deliveryItem.price>0?", "+NumberFormat('#,###').format(_deliveryItem.price)+" Ks": ""}' : ''),
                                        trailing: Icon(Icons.edit_sharp),
                                        leading: Icon(Icons.work_sharp),
                                        onTap: _addressBookItemRecipient==null || _addressBookItemSender==null? null : () async{
                                          bool isAdd = await Navigator.push(
                                              context,
                                              MaterialPageRoute(
                                                builder: (BuildContext context) => NewDeliveryItem(
                                                  title: tr(LocaleKeys.lblItemDetail),
                                                  item: _deliveryItem,
                                                  lang: _lang,
                                                ),
                                              )
                                          );
                                          if( isAdd !=null && isAdd ) {
                                            setState(() {
                                              _itemDetailAlreadyCheck = true;
                                              _itemDetailError = false;
                                            });
                                            _checkPrice();
                                          }
                                        },
                                      ),
                                    ),
                                  ),
                                  _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")?
                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[50],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliCharges)),
                                        subtitle: Text('${_deliveryCharges !=null? NumberFormat('#,###').format(_deliveryItem.charges)+' Ks' : ''}'),
                                        trailing: Icon(!_editChargesByAdmin?Icons.edit_sharp : Icons.clear),
                                        leading: Icon(Icons.monetization_on_sharp),
                                        onTap: _deliveryCharges==null? null: (){
                                          setState(() {
                                            _editChargesByAdmin = !_editChargesByAdmin;
                                          });
                                        },
                                      ),
                                    ),
                                  )
                                  :Container(
                                    padding: EdgeInsets.only(top:32, bottom: 0, left:16, right: 16),
                                    child: Card(
                                    color: Colors.green[50],
                                    elevation: 1,
                                    child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliCharges)),
                                        subtitle: Text('${_deliveryCharges!=null? NumberFormat('#,###').format(_deliveryItem.charges)+' Ks' : ''}'),
                                        leading: Icon(Icons.monetization_on_sharp),
                                      ),
                                    ),
                                  ),
                                  _editChargesByAdmin?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblDeliCharges),
                                            ),
                                            controller: _controller_edit_charges_by_admin,
                                            keyboardType: TextInputType.number,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done, color: Colors.white),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: () {
                                          setState(() {
                                            _editChargesByAdmin = !_editChargesByAdmin;
                                            if( _controller_edit_charges_by_admin.text.trim().isNotEmpty) {
                                              _deliveryItem.charges =
                                                  double.parse(
                                                      _controller_edit_charges_by_admin
                                                          .text);
                                              _deliveryCharges = _deliveryItem.charges;
                                            }
                                          });
                                        },
                                        label: Text(
                                          tr(LocaleKeys.btnOK),
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ): Container(),

                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[50],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliChargesToShowOnRecipient)),
                                        subtitle: Text('${_deliveryItem.forShowCharges>0? NumberFormat('#,###').format(_deliveryItem.forShowCharges)+' Ks' : ''}'),
                                        trailing: Icon(!_editDeliCharges?Icons.edit_sharp : Icons.clear),
                                        leading: Icon(Icons.monetization_on_outlined),
                                        onTap: _deliveryCharges==null? null: (){
                                          setState(() {
                                            _editDeliCharges = !_editDeliCharges;
                                          });
                                        },
                                      ),
                                    ),
                                  ),
                                  _editDeliCharges?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblDeliChargesToShowOnRecipient),
                                            ),
                                            controller: _controller_edit_deliCharges,
                                            keyboardType: TextInputType.number,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done, color: Colors.white),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: () {
                                          setState(() {
                                            _editDeliCharges = !_editDeliCharges;
                                            if( _controller_edit_deliCharges.text.trim().isNotEmpty)
                                              _deliveryItem.forShowCharges = double.parse(_controller_edit_deliCharges.text);
                                          });
                                        },
                                        label: Text(
                                          tr(LocaleKeys.btnOK),
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ) : Container(),

                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[50],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblPromotion)),
                                        subtitle: Text('${_promoAmount>0? NumberFormat('#,###').format(_promoAmount)+' Ks' : ''}'),
                                        trailing: Icon(!_hasPromoCode?Icons.edit_sharp : Icons.clear),
                                        leading: Icon(Icons.redeem_sharp),
                                        onTap: (){
                                          setState(() {
                                            _hasPromoCode = !_hasPromoCode;
                                          });
                                        },
                                      ),
                                    ),
                                  ),
                                  _hasPromoCode?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: 'Promotion code',
                                            ),
                                            controller: _controller_promoCode,
                                            keyboardType: TextInputType.text,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done, color: Colors.white),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: ()async {
                                          await _checkPromoCode();
                                        },
                                        label: Text(
                                          'Check code',
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ) : Container(),
                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[50],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblTotalCharges)),
                                        subtitle: Text('${_deliveryCharges !=null? NumberFormat('#,###').format(_deliveryItem.charges - _promoAmount)+' Ks' : ''}'),
                                        leading: Icon(Icons.monetization_on_sharp),
                                      ),
                                    ),
                                  ),
                                  _deliveryItem.price>0 && _allow_cash_on_advanced? Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: CheckboxListTile(
                                      value: _requiredToRefund,
                                      onChanged: ( value) {setState(() { _requiredToRefund = value; });},
                                      title: Text(
                                        '${tr(LocaleKeys.lblRequiredToRefund)}',
                                        style: TextStyle(color: Colors.black),
                                      ),
                                    ),
                                  ) : Container(),
                                ],

                              ),
                            )
                        ),
                      ),
                      Container(
                        padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 0),
                        width: double.infinity,
                        child: RaisedButton(
                          onPressed: _addressBookItemRecipient==null || _addressBookItemSender==null || _deliveryCharges==null || _processing? null :  () { _submitForm(); },
                          color: Theme.of(context).primaryColorDark,
                          elevation: 1,
                          child: Text(
                            tr(LocaleKeys.btnOK),
                            style: Theme.of(context).textTheme.headline5,
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(32.0)),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            )
        )
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool os) async {
    var now = os? _senderDate : _recipientDate;
    var initialDate = convertToDate(initialDateString) ?? now;
    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")?
        new DateTime( _pickupDateLabels[0].year, _pickupDateLabels[0].month-1, _pickupDateLabels[0].day )
            : (os? _pickupDateLabels[0] : _dropOffDateLabels[0]),
        lastDate: new DateTime( _pickupDateLabels[0].year, _pickupDateLabels[0].month+1, _pickupDateLabels[0].day )
    );

    if (result == null) return;
    setState(() {
      if( os ) {
        _senderDate = result;
      }else {
        _recipientDate = result;
      }
    });
  }
  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }
  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }
  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }
  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }
  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }
  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  Future<bool> _onWillPop() async {
    return true;
  }
  String _dayToLabel(DateTime date){
    String day = new DateFormat('EEEE').format( date.toLocal() ).toLowerCase();
    if( day == 'sunday')
      return tr(LocaleKeys.lblSUN);
    else if( day == 'monday')
      return tr(LocaleKeys.lblMON);
    else if( day == 'tuesday')
      return tr(LocaleKeys.lblTUE);
    else if( day == 'wednesday')
      return tr(LocaleKeys.lblWED);
    else if( day == 'thursday')
      return tr(LocaleKeys.lblTHU);
    else if( day == 'friday')
      return tr(LocaleKeys.lblFRI);
    else if( day == 'saturday')
      return tr(LocaleKeys.lblSAT);
  }

  Future<void> _checkPrice( ) async{
    setState(() {
      _processing = true;
    });
    try{
      String senderGuid = _addressBookItemSender.personGuid;

      if(_addressBookItemSender.addressType == 'Post Office'
          || _addressBookItemSender.addressType == 'Highway Gate'
          || _addressBookItemSender.addressType == 'POI'){
        senderGuid = _currentUser.guid;
      }

      PriceMethodService priceMethodService = new PriceMethodService( );
      Map<String, dynamic> data = await priceMethodService.calculatePrice(
          _deliveryItem,
          _addressBookItemRecipient.townshipGuid,
          _addressBookItemSender.townshipGuid,
          _lang,
          _addressBookItemRecipient.guid??'',
          _addressBookItemSender.guid??'',
          senderGuid
      );
      if( data['success']){
        Map<String, dynamic> prices = data["data"];
        _deliveryItem.charges = prices['totalPrice'];
        _promoAmount = prices['totalDiscount'];
        _deliveryItem.forShowCharges = _deliveryItem.charges;
        _controller_edit_deliCharges.text = _deliveryItem.charges.toStringAsFixed(0);
        _deliveryCharges = _deliveryItem.charges;
        _allow_cash_on_advanced = data["refundInAdvanced"]??false;
        _controller_edit_charges_by_admin.text = _deliveryItem.charges.toStringAsFixed(0);
      }else{
        _deliveryItem.charges = 0;
        _promoAmount = 0;
        _deliveryItem.forShowCharges = _deliveryItem.charges;
        _controller_edit_deliCharges.text = _deliveryItem.charges.toStringAsFixed(0);
        _controller_edit_charges_by_admin.text = _deliveryItem.charges.toStringAsFixed(0);
        _deliveryCharges = null;
        showMessage(data['data']);
      }
    }catch(e){print(e);}
    setState(() {
      _processing = false;
    });

    if(_authorityResponse.authorities !=null
        && _authorityResponse.authorities.contains("MANAGER_WRITE")){
      setState(() {
        _allow_cash_on_advanced = true;
      });
    }
  }

  Future<void> _checkPromoCode( ) async{
    try{

      if( _controller_promoCode.text.length<6 || _controller_promoCode.text.length>10) {
        showMessage('Invalid promotion code!');
        return;
      }
      PromotionService promotionService = new PromotionService();
      Map<String, dynamic> data = await promotionService.checkPromoCode( _controller_promoCode.text);

      if( data['success'] ){
        _promoAmount = data['data'];
        showMessage('Congratulation! this promotion code is valid and usable.');
      }else{
        _promoAmount = 0;
        showMessage(data['data']);
      }


    }catch(e){print(e);}
  }

  void _submitForm( ) async{

    if( _processing ) return;

    final FormState form = _formKey.currentState;
    if (!form.validate() ) {
      if(!_itemDetailAlreadyCheck){
        setState(() {
          _itemDetailError = true;
        });
      }
      showMessage('Form is not valid! Please revise.');
    } else {
      form.save( );

      if(!_itemDetailAlreadyCheck){
        setState(() {
          _itemDetailError = true;
        });
        showMessage(tr(LocaleKeys.lblRequiredParcelDetailToCheck));
        return;
      }

      if( _senderDate.isAfter(_recipientDate)){
        showMessage(tr(LocaleKeys.lblWrongDate));
        return;
      }

      bool  result = await _showConfirm(context);
      if ( !result ){
        return;
      }
      setState(() {
        _processing = true;
      });

      String senderGuid = _addressBookItemSender.personGuid;

      if(_addressBookItemSender.addressType == 'Post Office'
      || _addressBookItemSender.addressType == 'Highway Gate'
          || _addressBookItemSender.addressType == 'POI'){
        senderGuid = _currentUser.guid;
      }
      DeliveryContactItem os = new DeliveryContactItem(
          personGuid: senderGuid,
          // name: _addressBookItemSender.name,
          name: _currentUser.roleName==Constant.ROLE_NAME_OS?_currentUser.name : _controller_senderName.text,
          date: _senderDate,
          address: _addressBookItemSender.address,
          street: _addressBookItemSender.street,
          ward: null,
          township: _addressBookItemSender.townshipName,
          townshipGuid: _addressBookItemSender.townshipGuid,
          primaryPhone: _controller_senderPrimaryPhone.text,
          otherPhones: _controller_senderOtherPhones.text,
          email: null,
          comment: _controller_senderComment.text,
          latitude: _addressBookItemSender.latitude,
          longitude: _addressBookItemSender.longitude,
          contactType: Constant.CONTACT_TYPE_PICKUP,
          guid: _addressBookItemSender.guid,
          addressType: _addressBookItemSender.addressType
      );

      String recipientGuid;
      if( _addressBookItemRecipient !=null ){
        if(_addressBookItemSender.addressType != 'Post Office'
            && _addressBookItemSender.addressType != 'Highway Gate'
            && _addressBookItemSender.addressType != 'POI'
                && _addressBookItemRecipient.name == _controller_recipientName.text){
          recipientGuid = _addressBookItemRecipient.personGuid;
        }
      }
      DeliveryContactItem customer = new DeliveryContactItem(
          personGuid: recipientGuid,
          // personGuid: _addressBookItemRecipient!=null && _addressBookItemRecipient.name==_controller_recipientName.text? _addressBookItemRecipient.personGuid : null,
          name: _controller_recipientName.text,
          date: _recipientDate,
          address: _addressBookItemRecipient.address,
          street: _addressBookItemRecipient.street,
          ward: null,
          township: _addressBookItemRecipient.townshipName,
          townshipGuid: _addressBookItemRecipient.townshipGuid,
          primaryPhone: _controller_recipientPrimaryPhone.text,
          otherPhones: _controller_recipientOtherPhones.text,
          email: null,
          comment: _controller_senderComment.text,
          latitude: _addressBookItemRecipient.latitude,
          longitude: _addressBookItemRecipient.longitude,
          contactType: Constant.CONTACT_TYPE_DROP_OFF,
          guid: _addressBookItemRecipient.guid,
          addressType: _addressBookItemRecipient.addressType
      );

      List<Map<String, dynamic>> items = [_deliveryItem.toMap()];
      Map<String, dynamic> request = {
        'extraPaidBy': 'Customer',
        'labourRequired': false,
        'specialCase': false,
        'promoCode': _controller_promoCode.text.isNotEmpty?_controller_promoCode.text : null,
        'branchGuid': null,
        'contacts': [os.toMap(), customer.toMap()],
        'items': items,
        'refundInAdvanced': _requiredToRefund,
        'refundAmount': null,
        'lang': _lang,
        'promoAmount':_promoAmount
      };

      DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
      Map<String, dynamic> response = await _deliveryContentService.create(request );
      if( response['success'] ){
        showMessage(response["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
        widget.onChange();
      }else{
        showMessage(response["data"]);
        setState(() {
          _processing = false;
        });
      }
    }
  }

  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( tr(LocaleKeys.lblActionConfirm), style: TextStyle(color: Colors.black),),
          content: new Text(tr(LocaleKeys.lblAreYouSureToDeliverYourParcel)),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnYes)),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnCancel)),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<void> _availableDates( ) async{
    DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
    Map<String, dynamic> resultData = await _deliveryContentService.availableDeliveryDates( );
    if( resultData["success"] ){
      Map<String, dynamic> data = resultData["data"];
      final pickupDates = data["pickupDates"] as List;
      final dropOffDates = data["dropOffDates"] as List;
      _allow_cash_on_advanced = data["refundInAdvanced"]??false;

      for( int i=0; i< pickupDates.length; i++){
        _pickupDateLabels[i] = pickupDates[i]!=null? DateTime.parse(pickupDates[i]) : null;
        _dropOffDateLabels[i] = dropOffDates[i]!=null? DateTime.parse(dropOffDates[i]) : null;
      }

      setState((){
      _senderDate = _pickupDateLabels[0];
      _recipientDate = _dropOffDateLabels[0];
      });
    }
  }
}