import 'dart:async';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/person_list_request.dart';
import 'package:qq_delivery/delivery/model/request/pickup_create_request.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/person_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/promotion_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'delivery_address.dart';


class DeliveryCreateV2 extends StatefulWidget {
  DeliveryCreateV2({Key key, this.onChange, this.addressBookItemSender}) : super(key: key);
  final Function onChange;
  final AddressBookItem addressBookItemSender;
  @override
  _DeliveryCreateV2State createState() => _DeliveryCreateV2State();
}

class _DeliveryCreateV2State extends State<DeliveryCreateV2> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  ActiveDeliverymanResponse _deliverymanResponse;
  DeliveryForDeliveryItem _employeeSearchItem;

  final TextEditingController _controller_promoCode = new TextEditingController();
  final TextEditingController _controller_senderName = new TextEditingController();
  final TextEditingController _controller_senderAddress = new TextEditingController();
  final TextEditingController _controller_senderPrimaryPhone = new TextEditingController();
  final TextEditingController _controller_senderOtherPhones = new TextEditingController();
  final TextEditingController _controller_senderComment = new TextEditingController();
  final TextEditingController _controller_totalWays = new TextEditingController();
  final TextEditingController _controller_cashToCollect = new TextEditingController();
  final TextEditingController _controller_cashAdvance = new TextEditingController();

  DateTime _senderDate = new DateTime.now();

  List<bool> _selections = List.generate(6, (_)=> false);
  List<DateTime> _pickupDateLabels = <DateTime>[DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now()];

  User _currentUser = new User();
  bool _isFirst = true;
  bool _requiredToRefund = false;
  bool _allow_cash_on_advanced = false;
  bool _cashOnDelivery = true;
  bool _carRequired = false;
  bool _autoAssignDeliveryman = true;
  bool _approved = true;
  List<bool> _refundOptionSelections = List.generate(4, (_)=> false);
  List<String> _refundOptions = <String>['Cash', 'Bank', 'E-money','Any'];
  String _refundOption = "Any";

  AddressBookItem _addressBookItemSender;
  double _promoAmount = 0;
  bool _hasPromoCode = false;
  String _lang = Constant.LANGUAGE_CODE_MY;
  AuthorityResponse _authorityResponse = new AuthorityResponse();

  PersonListResponse _personListResponse;
  PersonListItem _addressBookItemOS;
  // GeneralPersonItem _personItemOS;
  PersonService _personService = new PersonService();
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();
    setState( () {
      _selections[0]=true;
      _refundOptionSelections[3] = true;

      int dayPlus = (_senderDate.toLocal().hour>14? 1 : 0);
      for( int i=0; i< _pickupDateLabels.length; i++){
        _pickupDateLabels[i] = new DateTime( _senderDate.year, _senderDate.month, _senderDate.day + i + dayPlus );
      }
      _senderDate = _pickupDateLabels[0];
      _addressBookItemSender = widget.addressBookItemSender;
    });

    _loadCurrentUser( );
  }

  void _loadCurrentUser( ) async{

    if( _isFirst ){
      _lang = await getLanguage( );
      _isFirst = false;
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      await _availableDates();
      if(_authorityResponse.authorities !=null
          && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))){
        // setState(() {
        //   _allow_cash_on_advanced = true;
        // });

        _personListResponse = await _personService.listByRole(
            new PersonListRequest(
              first: 0,
              max: 100000,
              orderBy: "firstName",
              asc: true,
              keyword: '',
              status: 'Active',
              roleName: 'OS',
              osGuid: '',
            )
        );
        _deliverymanResponse = await _personService.activeDeliveryman();

      }
      _sharedPrefs = await SharedPreferences.getInstance();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          key: _scaffoldKey,
          title: Text(tr(LocaleKeys.lblDeliCreateTitle), style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Center(
            child: SingleChildScrollView(
              child: Container(
                margin: const EdgeInsets.only(top: 16.0, bottom: 16.0, left: 8.0, right: 8.0),
                child: Card(
                  // color: Colors.white,
                  elevation: 8,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment:CrossAxisAlignment.center,
                    children: <Widget>[
                      // Container(
                      //   margin: EdgeInsets.only(top: 0, bottom: 32),
                      //   child: new Image.asset("assets/images/landing_background_large.jpg"),
                      // ),
                      _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")? _merchantSelection(context): Container(),
                      Container(
                        padding: EdgeInsets.all(16.0),
                        child: TextField(
                          textAlign: TextAlign.center,
                          keyboardType: TextInputType.text,
                          decoration: InputDecoration(
                            hintText: tr(LocaleKeys.lblDeliCreatePickup),// 'Pickup from',
                            hintStyle: TextStyle(fontSize: 14),
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(16),
                              borderSide: BorderSide(
                                  width: 1,
                                  style: BorderStyle.solid,
                                  // color: Theme.of(context).primaryColor
                              ),
                            ),
                            filled: true,
                            contentPadding: EdgeInsets.all(8),
                            // fillColor: Theme.of(context).backgroundColor,
                            prefixIcon: Icon(Icons.upload_sharp,),
                            suffixIcon : IconButton(
                              icon: Icon(Icons.add_location_sharp,
                                  // color: Theme.of(context).primaryColorDark
                              ),
                              // onPressed: (){print('Icon press');},
                            ),
                          ),
                          onTap: () async{
                            if( _addressBookItemOS==null
                                && _authorityResponse.authorities!=null
                                && _authorityResponse.authorities.contains("MANAGER_WRITE")){
                              showMessage('Please select the merchant and try again');
                              return;
                            }
                            FocusScope.of(context).requestFocus(new FocusNode());
                            AddressBookItem addressBookItem = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => DeliveryAddress(
                                  title: tr(LocaleKeys.lblDeliCreatePickup),
                                  addressType: 'OS',
                                  wayType: 'pickup',
                                  ownerGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                                      ?_addressBookItemOS.guid: _currentUser.guid,
                                ),
                              ),
                            );
                            if( addressBookItem != null) {
                              _addressBookItemSender = addressBookItem;
                              _controller_senderAddress.text = _addressBookItemSender.address
                                  +(_addressBookItemSender.street!=null && _addressBookItemSender.street.trim().isNotEmpty? ','+_addressBookItemSender.street.trim() : '')
                                  +(_addressBookItemSender.townshipName!=null && _addressBookItemSender.townshipName.trim().isNotEmpty? ','+_addressBookItemSender.townshipName.trim() : '');

                              // merchant self register and POI
                              if( (_currentUser.roleName == Constant.ROLE_NAME_OS || _currentUser.roleName == Constant.ROLE_NAME_CUSTOMER)
                              && (_addressBookItemSender.addressType == Constant.ADDRESS_TYPE_HIGHWAY
                                  || _addressBookItemSender.addressType == Constant.ADDRESS_TYPE_POST_OFFICE
                                  || _addressBookItemSender.addressType == Constant.ADDRESS_TYPE_POI) ){
                                _controller_senderPrimaryPhone.text = _currentUser.phone;
                                _controller_senderOtherPhones.text = '';
                                _controller_senderName.text = _currentUser.name;

                              }else{
                                // admin register and POI
                                if( _addressBookItemOS != null
                                    && (_addressBookItemSender.addressType == Constant.ADDRESS_TYPE_HIGHWAY
                                        || _addressBookItemSender.addressType == Constant.ADDRESS_TYPE_POST_OFFICE
                                        || _addressBookItemSender.addressType == Constant.ADDRESS_TYPE_POI) ){
                                  _controller_senderPrimaryPhone.text = _addressBookItemOS.mobilePhone;
                                  _controller_senderOtherPhones.text = _addressBookItemOS.otherPhones;
                                  _controller_senderName.text = _addressBookItemOS.name;

                                }else{
                                  // Existing contact
                                  _controller_senderPrimaryPhone.text = addressBookItem.primaryPhone;
                                  _controller_senderOtherPhones.text = addressBookItem.otherPhones??'';
                                  _controller_senderName.text = addressBookItem.name??_currentUser.name;
                                  if(_controller_senderPrimaryPhone.text.trim().isEmpty){
                                    _controller_senderPrimaryPhone.text = _currentUser.phone;
                                  }

                                }
                              }
                            }
                          },
                          controller: _controller_senderAddress,
                        ),
                      ),
                      Container(
                        margin: EdgeInsets.only(top: 0, bottom: 32),
                        child: new SafeArea(
                            top: false,
                            bottom: false,
                            child: new Form(
                              key: _formKey,
                              autovalidateMode: AutovalidateMode.onUserInteraction,
                              child: new Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Card(
                                    // color: Colors.white,
                                    elevation: 0,
                                    child: new Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisSize: MainAxisSize.min,
                                    children: <Widget>[
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                        child: Text('${tr(LocaleKeys.lblDeliCreatePickupDate)} ${new DateFormat('dd/MM/yyyy').format(_senderDate.toLocal())}'),
                                      ),
                                      Container(
                                        padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                        child: ToggleButtons(
                                          children: <Widget>[
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[0]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[1]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[2]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[3]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: Text(
                                                _dayToLabel(_pickupDateLabels[4]),
                                                style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                              child: FlatButton(
                                                onPressed: () {
                                                  setState(() {
                                                    for (int buttonIndex = 0; buttonIndex < _selections.length; buttonIndex++) {
                                                      if (buttonIndex == _selections.length-1) {
                                                        _selections[buttonIndex] = true;
                                                      } else {
                                                        _selections[buttonIndex] = false;
                                                      }
                                                    }
                                                  });
                                                  _chooseDate(context, new DateFormat('yyyy-MM-dd').format(_senderDate.toLocal()) );
                                                  },
                                                // color: Theme.of(context).backgroundColor,
                                                child: Text(
                                                  tr(LocaleKeys.lblAntherDay),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              )
                                            ),
                                          ],
                                          onPressed: (int index) {
                                            setState(() {
                                              _senderDate = _pickupDateLabels[index];
                                              for (int buttonIndex = 0; buttonIndex < _selections.length; buttonIndex++) {
                                                if (buttonIndex == index) {
                                                  _selections[buttonIndex] = true;
                                                } else {
                                                  _selections[buttonIndex] = false;
                                                }
                                              }
                                            });
                                          },
                                          isSelected: _selections,
                                        ),
                                      ),
                                      _currentUser.roleName!=Constant.ROLE_NAME_OS? Container(
                                        padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblSenderName),
                                              labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          validator: (val) => val.isEmpty ? 'sender name required!' : null,
                                          controller: _controller_senderName,
                                          keyboardType: TextInputType.text,
                                        ),
                                      ) : Container(),
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                            hintText: '09xxxxxxxxx',
                                            labelText: tr(LocaleKeys.lblSenderMobile),
                                            labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                                          controller: _controller_senderPrimaryPhone,
                                          keyboardType: TextInputType.phone,
                                        ),
                                      ),
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                            hintText: "Use \",\" for multiple numbers",
                                            labelText: tr(LocaleKeys.lblSenderOtherPhones),
                                            labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          controller: _controller_senderOtherPhones,
                                          keyboardType: TextInputType.text,
                                        ),
                                      ),
                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                              hintText: "",
                                              labelText: tr(LocaleKeys.lblItemCreateTotalWays),
                                              labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          controller: _controller_totalWays,
                                          keyboardType: TextInputType.number,
                                        ),
                                      ),

                                      // Container(
                                      //   padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                      //   child: CheckboxListTile(
                                      //     value: _cashOnDelivery,
                                      //     onChanged: ( value) {setState(() {
                                      //       _cashOnDelivery = value;
                                      //       if( !_cashOnDelivery ) _requiredToRefund = _cashOnDelivery;
                                      //     });},
                                      //     title: Text(
                                      //       '${tr(LocaleKeys.lblCashOnDelivery)}',
                                      //       // style: TextStyle(color: Colors.black),
                                      //     ),
                                      //   ),
                                      // ),
                                      // _cashOnDelivery?Container(
                                      //   padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                      //   child: new TextFormField(
                                      //     decoration: InputDecoration(
                                      //         hintText: '',
                                      //         labelText: tr(LocaleKeys.lblTotalAmountOnCashOnDelivery),
                                      //         labelStyle: TextStyle(fontSize: 14)
                                      //     ),
                                      //     controller: _controller_cashToCollect,
                                      //     keyboardType: TextInputType.number,
                                      //     // maxLines: null,
                                      //     // minLines: 1,
                                      //   ),
                                      // ) : Container(),

                                      _cashOnDelivery?Container(
                                        padding: EdgeInsets.only(top:16, bottom: 0, left:16, right: 16),
                                        child: Text(tr(LocaleKeys.lblPreferredRefundOption)),
                                      ) : Container(),
                                      _cashOnDelivery?Container(
                                        padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                        child: ToggleButtons(
                                          children: <Widget>[
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                                              child: Text(
                                                tr(LocaleKeys.lblPreferredRefundOptionCash),
                                                // style: TextStyle(fontSize: 12.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                                              child: Text(
                                                tr(LocaleKeys.lblPreferredRefundOptionBank),
                                                // style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                                              child: Text(
                                                tr(LocaleKeys.lblPreferredRefundOptionEMoney),
                                                // style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                            Container(
                                              padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                                              child: Text(
                                                tr(LocaleKeys.lblPreferredRefundOptionAny),
                                                // style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                textAlign: TextAlign.center,
                                              ),
                                            ),
                                          ],
                                          onPressed: (int index) {
                                            setState(() {
                                              _refundOption = _refundOptions[index];
                                              for (int buttonIndex = 0; buttonIndex < _refundOptions.length; buttonIndex++) {
                                                if (buttonIndex == index) {
                                                  _refundOptionSelections[buttonIndex] = true;
                                                } else {
                                                  _refundOptionSelections[buttonIndex] = false;
                                                }
                                              }
                                            });
                                          },
                                          isSelected: _refundOptionSelections,
                                        ),
                                      ) : Container(),
                                      _allow_cash_on_advanced && _cashOnDelivery? Container(
                                        padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                        child: CheckboxListTile(
                                          value: _requiredToRefund,
                                          onChanged: ( value) {setState(() { _requiredToRefund = value; });},
                                          title: Text(
                                            '${tr(LocaleKeys.lblRequiredToRefund)}',
                                            // style: TextStyle(color: Colors.black),
                                          ),
                                        ),
                                      ) : Container(),
                                      _requiredToRefund?Container(
                                        padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblPickupCreateCashAdvanceAmount),
                                              labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          controller: _controller_cashAdvance,
                                          keyboardType: TextInputType.number,
                                          // maxLines: null,
                                          // minLines: 1,
                                        ),
                                      ) : Container(),
                                      Container(
                                        padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                        child: CheckboxListTile(
                                          value: _carRequired,
                                          onChanged: ( value) {setState(() { _carRequired = value; });},
                                          title: Text(
                                            '${tr(LocaleKeys.lblPickupDetailCarRequired)}',
                                            // style: TextStyle(color: Colors.black),
                                          ),
                                        ),
                                      ),
                                      _authorityResponse.authorities!=null
                                          && (_authorityResponse.authorities.contains("MANAGER_WRITE")
                                          || _authorityResponse.authorities.contains("OPERATOR_WRITE")) ? Container(
                                        padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                        child: CheckboxListTile(
                                          value: _approved,
                                          onChanged: ( value) {setState(() { _approved = value; });},
                                          title: Text(
                                            tr(LocaleKeys.lblConfirmAssignToDeliveryman),
                                            // style: TextStyle(color: Colors.black),
                                          ),
                                        ),
                                      ) : Container(),
                                      _authorityResponse.authorities!=null
                                          && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))
                                          && _approved? Container(
                                        padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                        child: CheckboxListTile(
                                          value: _autoAssignDeliveryman,
                                          onChanged: ( value) {setState(() { _autoAssignDeliveryman = value; });},
                                          title: Text(
                                            '${tr(LocaleKeys.lblAutoAssignDeliveryman)}',
                                            // style: TextStyle(color: Colors.black),
                                          ),
                                        ),
                                      ) : Container(),
                                      !_autoAssignDeliveryman? _deliverySelection(context): Container(),

                                      Container(
                                        padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                        child: new TextFormField(
                                          decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblRemark),
                                              labelStyle: TextStyle(fontSize: 14)
                                          ),
                                          controller: _controller_senderComment,
                                          keyboardType: TextInputType.text,
                                          // maxLines: null,
                                          // minLines: 1,
                                        ),
                                      ),
                                    ]
                                    )
                                  ),

                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[100],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblPromotion),style: TextStyle(color: Colors.black)),
                                        subtitle: Text('${_promoAmount>0? NumberFormat('#,###').format(_promoAmount)+' Ks' : ''}',style: TextStyle(color: Colors.black)),
                                        trailing: Icon(!_hasPromoCode?Icons.edit_sharp : Icons.clear, color: Colors.black),
                                        leading: Icon(Icons.redeem_sharp, color: Colors.black),
                                        onTap: (){
                                          setState(() {
                                            _hasPromoCode = !_hasPromoCode;
                                          });
                                        },
                                      ),
                                    ),
                                  ),
                                  _hasPromoCode?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: 'Promotion code',
                                            ),
                                            controller: _controller_promoCode,
                                            keyboardType: TextInputType.text,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done, color: Colors.white),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: ()async {
                                          await _checkPromoCode();
                                        },
                                        label: Text(
                                          'Check code',
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ) : Container(),
                                ],
                              ),
                            )
                        ),
                      ),
                      Container(
                        padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 0),
                        width: double.infinity,
                        child: RaisedButton(
                          onPressed: _addressBookItemSender==null || _processing? null :  () { _submitForm(); },
                          // color: Theme.of(context).primaryColorDark,
                          elevation: 1,
                          child: Text(
                            tr(LocaleKeys.btnOK),
                            style: Theme.of(context).textTheme.headline5,
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(32.0)),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            )
        )
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString ) async {
    var now = _senderDate;
    var initialDate = convertToDate(initialDateString) ?? now;
    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")?
        new DateTime( _pickupDateLabels[0].year, _pickupDateLabels[0].month-1, _pickupDateLabels[0].day )
            :_pickupDateLabels[0],
        lastDate: new DateTime( _pickupDateLabels[0].year, _pickupDateLabels[0].month+1, _pickupDateLabels[0].day )
    );

    if (result == null) return;
    setState(() {
        _senderDate = result;
    });
  }
  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }
  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }
  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }
  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }
  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }
  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  Future<bool> _onWillPop() async {
    return true;
  }
  String _dayToLabel(DateTime date){
    String day = new DateFormat('EEEE').format( date.toLocal() ).toLowerCase();
    if( day == 'sunday')
      return tr(LocaleKeys.lblSUN);
    else if( day == 'monday')
      return tr(LocaleKeys.lblMON);
    else if( day == 'tuesday')
      return tr(LocaleKeys.lblTUE);
    else if( day == 'wednesday')
      return tr(LocaleKeys.lblWED);
    else if( day == 'thursday')
      return tr(LocaleKeys.lblTHU);
    else if( day == 'friday')
      return tr(LocaleKeys.lblFRI);
    else if( day == 'saturday')
      return tr(LocaleKeys.lblSAT);
  }

  Future<void> _checkPromoCode( ) async{
    try{

      if( _controller_promoCode.text.length<6 || _controller_promoCode.text.length>10) {
        showMessage('Invalid promotion code!');
        return;
      }
      PromotionService promotionService = new PromotionService();
      Map<String, dynamic> data = await promotionService.checkPromoCode( _controller_promoCode.text);

      if( data['success'] ){
        _promoAmount = data['data'];
        showMessage('Congratulation! this promotion code is valid and usable.');
      }else{
        _promoAmount = 0;
        showMessage(data['data']);
      }


    }catch(e){print(e);}
  }

  void _submitForm( ) async{

    if( _processing ) return;

    final FormState form = _formKey.currentState;
    if (!form.validate() ) {
      showMessage('Form is not valid! Please revise.');
    } else {
      form.save( );

      if(_controller_totalWays.text.isEmpty){
        showMessage(tr(LocaleKeys.lblRequiredTotalWays));
        return;
      }
      if(_controller_totalWays.text.isNotEmpty && int.parse(_controller_totalWays.text.trim()) <=0){
        showMessage(tr(LocaleKeys.lblRequiredTotalWays));
        return;
      }

      if(_controller_cashToCollect.text.isEmpty && _cashOnDelivery){
        // showMessage('Total cash amount required!');
        // return;
        _controller_cashToCollect.text = '0';
      }
      if(_controller_cashToCollect.text.isNotEmpty && double.parse(_controller_cashToCollect.text.trim()) <0 && _cashOnDelivery){
        showMessage( 'Total cash amount required!' );
        return;
      }

      if(_controller_cashAdvance.text.isEmpty && _requiredToRefund){
        showMessage('Cash advance amount required!');
        return;
      }
      if(_controller_cashAdvance.text.isNotEmpty && double.parse(_controller_cashAdvance.text.trim()) <=0 && _requiredToRefund ){
        showMessage( 'Cash advance amount required!' );
        return;
      }

      bool  result = await _showConfirm(context);
      if ( !result ){
        return;
      }
      setState(() {
        _processing = true;
      });

      String senderGuid = (_currentUser.roleName== Constant.ROLE_NAME_OS ||_currentUser.roleName== Constant.ROLE_NAME_CUSTOMER )? _currentUser.guid: _addressBookItemOS.guid;

      // if(_addressBookItemSender.addressType == 'Post Office'
      // || _addressBookItemSender.addressType == 'Highway Gate'
      //     || _addressBookItemSender.addressType == 'POI'){
      //   senderGuid = _currentUser.guid;
      // }
      DeliveryContactItem os = new DeliveryContactItem(
          personGuid: senderGuid,
          // name: _addressBookItemSender.name,
          name: _currentUser.roleName==Constant.ROLE_NAME_OS || _currentUser.roleName==Constant.ROLE_NAME_CUSTOMER?_currentUser.name : _controller_senderName.text,
          date: _senderDate,
          address: _addressBookItemSender.address,
          street: _addressBookItemSender.street,
          ward: null,
          township: _addressBookItemSender.townshipName,
          townshipGuid: _addressBookItemSender.townshipGuid,
          primaryPhone: _controller_senderPrimaryPhone.text,
          otherPhones: _controller_senderOtherPhones.text,
          email: null,
          comment: _controller_senderComment.text,
          latitude: _addressBookItemSender.latitude,
          longitude: _addressBookItemSender.longitude,
          contactType: Constant.CONTACT_TYPE_PICKUP,
          guid: _addressBookItemSender.guid,
          addressType: _addressBookItemSender.addressType
      );
      DeliveryContactItem assignee;
      if( !_autoAssignDeliveryman && _employeeSearchItem!=null ){
        assignee = new DeliveryContactItem();
        assignee.personGuid = _employeeSearchItem.guid;
      }

      PickupCreateRequest pickupCreateRequest = new PickupCreateRequest();
      pickupCreateRequest.promoCode = _controller_promoCode.text.isNotEmpty?_controller_promoCode.text : null;
      pickupCreateRequest.branchGuid = null;
      pickupCreateRequest.pickup = os;
      pickupCreateRequest.totalWays =  int.parse(_controller_totalWays.text);
      pickupCreateRequest.specialCase = false;
      pickupCreateRequest.refundInAdvanced = _requiredToRefund;
      pickupCreateRequest.cashAdvanceAmount = _requiredToRefund?double.parse(_controller_cashAdvance.text.trim()) : null;
      pickupCreateRequest.lang = _lang;
      pickupCreateRequest.remark = _controller_senderComment.text;
      pickupCreateRequest.promoAmount = _promoAmount;
      pickupCreateRequest.carRequired = _carRequired;
      pickupCreateRequest.cashToCollect =_controller_cashToCollect.text.isNotEmpty? double.parse(_controller_cashToCollect.text.trim()) : 0;
      pickupCreateRequest.assignee = assignee;
      pickupCreateRequest.refundOption = _refundOption;
      pickupCreateRequest.approved = _approved;

      // print( pickupCreateRequest.toMap() );

      // setState(() {
      //   _processing = false;
      // });
      // return;

      PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
      Map<String, dynamic> response = await _pickupServiceV2.create( pickupCreateRequest.toMap() );
      if( response['success'] ){
        showMessage(response["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
        widget.onChange();
      }else{
        showMessage(response["data"]);
        setState(() {
          _processing = false;
        });
      }
    }
  }

  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( tr(LocaleKeys.lblActionConfirm), style: TextStyle(color: Colors.black),),
          content: new Text(tr(LocaleKeys.lblAreYouSureToDeliverYourParcel)),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnYes)),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnCancel)),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<void> _availableDates( ) async{
    DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
    Map<String, dynamic> resultData = await _deliveryContentService.availableDeliveryDates( );
    if( resultData["success"] ){
      Map<String, dynamic> data = resultData["data"];
      final pickupDates = data["pickupDates"] as List;
      _allow_cash_on_advanced = data["refundInAdvanced"]??false;

      for( int i=0; i< pickupDates.length; i++){
        _pickupDateLabels[i] = pickupDates[i]!=null? DateTime.parse(pickupDates[i]) : null;
      }

      setState((){
      _senderDate = _pickupDateLabels[0];
      });
    }
  }

  Widget _merchantSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<PersonListItem>(
          selectedItem: _addressBookItemOS,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (PersonListItem i, PersonListItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblOS),
          onFind: (filter) async{ return await _merchantSuggestion(filter); },
          onChanged: (PersonListItem data) {
            _addressBookItemOS = data;
            _controller_senderAddress.text = '';
            _controller_senderName.text ='';
            _controller_senderPrimaryPhone.text = '';
            _controller_senderOtherPhones.text = '';
            _controller_totalWays.text='';
            _controller_cashToCollect.text = '';
          },
          dropdownBuilder: (BuildContext context, PersonListItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle,
                      // color: Colors.blueGrey[600]
                  ),
                  title: Text(
                    tr(LocaleKeys.lblSelectOS),
                    // style: TextStyle(color: Colors.black),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0),
                  title: Text(
                    item.name,
                    // style: TextStyle(color: Colors.black),
                  ),
                  subtitle: Text(item.personId),
                  leading: item.avatar!=null? new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                    width: 42, height: 42, ) : Icon(Icons.account_circle),
                ),
              ),
          popupItemBuilder: (BuildContext context, PersonListItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  // color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text('${item.name}'),
                  subtitle: Text('${item.personId}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: item.avatar!=null? new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ) : Icon(Icons.account_circle),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<PersonListItem>> _merchantSuggestion(String query) async{
    List<PersonListItem> _matches = [];
    if( _personListResponse==null || _personListResponse.items == null)
      return _matches;
    _matches.addAll(_personListResponse.items);

    _matches.retainWhere((s) {
      PersonListItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else if( addressBookItem.personId.toLowerCase().contains(query.toLowerCase()) )
      return true;
      else
        return false;
    }
    );
    return _matches;
  }

  Widget _deliverySelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && (_authorityResponse.authorities.contains("MANAGER_READ") || _authorityResponse.authorities.contains("OPERATOR_READ"))?
      Container(
          margin: EdgeInsets.only(top: 16, left: 16, right: 16, bottom: 16),
          child: DropdownSearch<DeliveryForDeliveryItem>(
            selectedItem: _employeeSearchItem,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (DeliveryForDeliveryItem i, DeliveryForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _deliverymanSuggestion(filter),
            onChanged: (DeliveryForDeliveryItem data) {
              _employeeSearchItem = data;
            },
            dropdownBuilder: (BuildContext context, DeliveryForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: item.avatar!=null? new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ) : Icon(Icons.account_circle),
                  ),
                ),
            popupItemBuilder: (BuildContext context, DeliveryForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text(item.personId),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: item.avatar!=null? new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ) : Icon(Icons.account_circle),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }

  Future<List<DeliveryForDeliveryItem>> _deliverymanSuggestion(String query) async{
    List<DeliveryForDeliveryItem> matches = [];
    if( _deliverymanResponse.items == null)
      return matches;
    matches.addAll(_deliverymanResponse.items);

    matches.retainWhere((s) {
      DeliveryForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else if( employeeSearchItem.personId.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else
        return false;
    }
    );
    return matches;
  }

}