import 'dart:convert';
import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/service/price_method_service.dart';
import 'package:path_provider/path_provider.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:image/image.dart' as imglib;
import 'package:shared_preferences/shared_preferences.dart';


class NewDeliveryItem extends StatefulWidget {
  final String title;
  final DeliveryItem item;
  final String lang;
  final SharedPreferences sharedPrefs;
  final Function onChange;
  NewDeliveryItem({Key key, @required this.title,
    @required this.item, this.lang, this.sharedPrefs, this.onChange}) : super(key: key);

  @override
  _NewDeliveryItemState createState() => _NewDeliveryItemState();
}

class _NewDeliveryItemState extends State<NewDeliveryItem> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  bool _largeScale = false;
  bool _addImage = false;
  bool _cashOnDelivery = true;

  @override
  void initState() {
    super.initState();
    if( widget.item.image != null){
      setState(() {
        _addImage = true;
      });
    }
    // setState(() {
    //   _controller_deli_charges.text = '${widget.item.charges>0? widget.item.charges.toStringAsFixed(0) : ''}';
    // });
  }
  @override
  Widget build(BuildContext context) {
    return SafeArea(
      top: false,
      bottom: false,
      child: new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: new Container(
          margin: EdgeInsets.only(top: 16, bottom: 16),
          child: Card(
            // color: Colors.white,
            elevation: 8,
            child: new Column(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                Container(
                  margin: const EdgeInsets.only(top: 0.0, left: 16.0, right: 16.0),
                  child: new CheckboxListTile(
                    value: widget.item.fragile,
                    onChanged: ( value){
                      setState(() { widget.item.fragile = value; });
                      widget.onChange();
                    },
                    title: Text(
                      tr(LocaleKeys.lblFragile),
                      // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                    ),
                  ),
                ),
                Container(
                  margin: const EdgeInsets.only(top: 8.0, left: 16.0, right: 16.0),
                  child: new CheckboxListTile(
                    value: widget.item.insurance,
                    onChanged: ( value){
                      setState(() { widget.item.insurance = value; });
                    },
                    title: Text(
                      tr(LocaleKeys.lblInsurance),
                      style: new TextStyle(fontSize: 16.0,
                        // color: Colors.black
                      ),
                    ),
                  ),
                ),
                Container(
                  margin: const EdgeInsets.only(top: 0.0, left: 16.0, right: 16.0),
                  child: new CheckboxListTile(
                    value: _largeScale,
                    onChanged: ( value){
                      setState(() { _largeScale = value; });
                    },
                    title: Text(
                      tr(LocaleKeys.lblItIsLargeScaleItem),
                      // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                    ),
                  ),
                ),
                Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 16.0, right: 16.0),
                  child: new TextFormField(
                    decoration: InputDecoration(
                      hintText: '',
                      labelText: tr(LocaleKeys.lblWeight),
                    ),
                    keyboardType: TextInputType.number,
                    validator: (val) => val.isEmpty ? 'Weight[kg] required' : (double.parse(val)>=1 ? null : 'Invalid weight[kg]'),
                    // onSaved: (val) => widget.item.weight = double.parse(val),
                    initialValue: '${widget.item.weight.toStringAsFixed(0)}',
                    onChanged: (val){
                      if( val.isNotEmpty){
                        widget.item.weight = double.parse(val);
                        widget.onChange();
                      }
                    },
                  ),
                ),
                _largeScale? Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 16.0, right: 16.0),
                  child: new TextFormField(
                    decoration: InputDecoration(
                      hintText: '',
                      labelText: tr(LocaleKeys.lblWidth),
                    ),
                    keyboardType: TextInputType.number,
                    validator: (val) => val.isEmpty ? 'Width[ft] required' : (int.parse(val)>=1 ? null : 'Invalid width[ft]'),
                    // onSaved: (val) => widget.item.width = double.parse(val),
                    initialValue: '${widget.item.width.toStringAsFixed(0)}',
                    onChanged: (val){
                      if( val.isNotEmpty){
                        widget.item.width = double.parse(val);
                        widget.onChange();
                      }
                    },
                  ),
                ) : Container(),
                _largeScale?Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 16.0, right: 16.0),
                  child: new TextFormField(
                    decoration: InputDecoration(
                      hintText: '',
                      labelText: tr(LocaleKeys.lblLength),
                    ),
                    keyboardType: TextInputType.number,
                    validator: (val) => val.isEmpty ? 'Length [ft] required' : (int.parse(val)>=1 ? null : 'Invalid length[ft]'),
                    // onSaved: (val) => widget.item.length = double.parse(val),
                    initialValue: '${widget.item.length.toStringAsFixed(0)}',
                    onChanged: (val){
                      if( val.isNotEmpty){
                        widget.item.length = double.parse(val);
                        widget.onChange();
                      }
                    },
                  ),
                ):Container(),
                _largeScale? Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 16.0, right: 16.0),
                  child: new TextFormField(
                    decoration: InputDecoration(
                      hintText: '',
                      labelText: tr(LocaleKeys.lblHeight),
                    ),
                    keyboardType: TextInputType.number,
                    validator: (val) => val.isEmpty ? 'Height[ft] required' : (int.parse(val)>=1 ? null : 'Invalid height[ft]'),
                    // onSaved: (val) => widget.item.height = double.parse(val),
                    initialValue: '${widget.item.height.toStringAsFixed(0)}',
                    onChanged: (val){
                      if( val.isNotEmpty){
                        widget.item.height = double.parse(val);
                        widget.onChange();
                      }
                    },
                  ),
                ):Container(),
                Container(
                  margin: const EdgeInsets.only(top: 0.0, left: 16.0, right: 16.0),
                  child: new CheckboxListTile(
                    value: _cashOnDelivery,
                    onChanged: ( value){
                      if( value ){
                        widget.item.priceOption = 'Unpaid';
                        widget.item.chargeOption = 'Unpaid';
                      }else{
                        widget.item.priceOption = 'Paid';
                        widget.item.chargeOption = 'Paid';
                      }
                      setState(() { _cashOnDelivery = value; });
                    },
                    title: Text(
                      tr(LocaleKeys.lblCashOnDelivery),
                      // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                    ),
                  ),
                ),

                /*Container(
                        margin: const EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0),
                        child:new Column(
                          mainAxisSize: MainAxisSize.min,
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            new Text(
                              tr(LocaleKeys.lblItemPrice),
                              style: new TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold, color: Theme.of(context).primaryColor),
                            ),
                            new RadioListTile(
                              value: 'Unpaid',
                              groupValue: widget.item.priceOption,
                              onChanged: ( value){
                                setState(() { widget.item.priceOption = value; });
                              },
                              title: Text(
                                tr(LocaleKeys.lblToCollectItemPriceFromRecipient),
                                style: new TextStyle(
                                    fontSize: 16.0,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                            new RadioListTile(
                              value: 'Paid',
                              groupValue: widget.item.priceOption,
                              onChanged: ( value){
                                setState(() {
                                  widget.item.priceOption = value;
                                  widget.item.price = 0;
                                });
                              },
                              title: Text(
                                tr(LocaleKeys.lblItemPriceAlreadyPaidByRecipient),
                                style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          ],
                        ),
                      ),*/
                _cashOnDelivery? Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 16.0, right: 16.0),
                  child: new TextFormField(
                    decoration: InputDecoration(
                        hintText: '',
                        labelText: tr(LocaleKeys.lblItemPriceEntry),
                        prefixIcon: Icon(Icons.attach_money)
                    ),
                    keyboardType: TextInputType.number,
                    validator: (val) => (val.isEmpty && widget.item.priceOption=='Paid') ? null : ( (val.isNotEmpty && double.parse(val)>=0) ? null : 'Invalid item price'),
                    // onSaved: (val) => widget.item.price = val.isEmpty?0 : double.parse(val),
                    initialValue: '${widget.item.price>=0? widget.item.price.toStringAsFixed(0) : ''}',
                    onChanged: (val){
                      if( val.isNotEmpty){
                        widget.item.price = val.isEmpty?0 : double.parse(val);
                      }
                    },
                  ),
                ) : Container(),
                /*Container(
                        margin: const EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0),
                        child:new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            new Text(
                              tr(LocaleKeys.lblDeliCharges),
                              style: new TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold, color: Theme.of(context).primaryColor),
                            ),
                            new RadioListTile(
                              value: 'Unpaid',
                              groupValue: widget.item.chargeOption,
                              onChanged: ( value){
                                setState(() { widget.item.chargeOption = value; });
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliChargeToCollectFromRecipient),
                                style: new TextStyle(
                                    fontSize: 16.0,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                            new RadioListTile(
                              value: 'Paid',
                              groupValue: widget.item.chargeOption,
                              onChanged: ( value){
                                setState(() { widget.item.chargeOption = value; });
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliChargeAlreadyPaid),
                                style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          ],
                        ),
                      ),*/
                Container(
                  margin: const EdgeInsets.only(top: 0.0, left: 16.0, right: 16.0),
                  child: new CheckboxListTile(
                    value: _addImage,
                    onChanged: ( value){
                      setState(() { _addImage = value; });
                    },
                    title: Text(
                      tr(LocaleKeys.lblAddItemPhoto),
                      // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                    ),
                  ),
                ),
                _addImage? _attachedImage() : Container(),
                // Container(
                //   margin: const EdgeInsets.only(top: 32.0, bottom: 16.0, left: 16.0, right: 16.0),
                //   child: ButtonBar(
                //       children: <Widget>[
                //         RaisedButton.icon(
                //           elevation: 0.0,
                //           icon: Icon(Icons.done, color: Colors.white),
                //           color: Theme.of(context).primaryColor,
                //           onPressed:_processing?null : ()async {
                //             _submitForm( );
                //           },
                //           shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                //           label: Text(
                //             tr(LocaleKeys.btnOK),
                //             style: TextStyle(
                //               fontSize: 16,
                //               fontWeight: FontWeight.w400,
                //               color: Colors.white,
                //             ),
                //           ),
                //         ),
                //         RaisedButton.icon(
                //           elevation: 1.0,
                //           icon: Icon(Icons.close, color: Colors.white),
                //           color: Colors.red[600],
                //           onPressed:_processing?null : () async { Navigator.pop(context, false);},
                //           shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                //           label: Text(
                //             tr(LocaleKeys.btnCancel),
                //             style: TextStyle(
                //               fontSize: 16,
                //               fontWeight: FontWeight.w400,
                //               color: Colors.white,
                //             ),
                //           ),
                //         ),
                //       ]
                //   ),
                // ),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _attachedImage(){
    return new Column(
      children: <Widget>[
        GestureDetector(
          child: Container(
            margin: const EdgeInsets.only(left: 5.0, right: 5.0),
            width: 100,
            height: 100,
            child: widget.item.image==null ? Icon(Icons.photo, color: Colors.grey[100], size: 100,) : (widget.item.image.length>100? decode( widget.item.image ) : new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.item.image}',) ),
            decoration: widget.item.image!=null ?BoxDecoration(
                border: Border.all(color: Colors.green[50], width: 3)
            ) : null,
          ),
          onTap: ()async{
            if(widget.item.image !=null && widget.item.image.isNotEmpty){
              Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => FullScreenWrapper(
                    imageProvider: widget.item.image.length>100?MemoryImage(base64Decode(widget.item.image)) : NetworkImage( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.item.image}' ),// MemoryImage(base64Decode(widget.item.image.substring(22))) //NetworkImage( post.image ),
                  ),
                ),
              );
            }
          },
        ),
        Container(
          margin: EdgeInsets.only(left: 0.0, right: 0.0, top: 24),
          child: ButtonBar(
              alignment: MainAxisAlignment.center,
              children: <Widget>[
                RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.photo_camera, color: Colors.white),
                  color: Theme.of(context).primaryColor,
                  onPressed: () async {
                    await _getImage( true );
                  },
                  label: Text(
                    tr(LocaleKeys.btnCamera),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
                RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.photo_library, color: Colors.white),
                  color: Theme.of(context).primaryColor,
                  onPressed: () async { await _getImage( false);},
                  label: Text(
                    tr(LocaleKeys.btnBrowsePhoto),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
                widget.item.image==null ? Container() : RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.delete_forever, color: Colors.white),
                  color: Colors.red[600],
                  onPressed: () async {
                    setState(() {
                      widget.item.image = null;
                    });
                  },
                  label: Text(
                    tr(LocaleKeys.btnRemove),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
              ]
          ),
        ),
      ],
    );
  }

  Future<void> _getImage( bool isCamera) async {
    _formKey.currentState.save();

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/gelivery_tmp';
        await Directory(tmpDir).create(recursive: true);

        String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
        File rotatedFile = await FlutterImageCompress.compressAndGetFile(
          image.path,
          photoPath2,
          quality: 72,
          rotate: 0,
        );
        imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
        String cropFilePath = '$tmpDir/cropped.jpg';
        File(cropFilePath)
          ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));

        File smallFile  =File(cropFilePath);
        String encodedString = encode( smallFile );
        smallFile.deleteSync(recursive: false);
        rotatedFile.deleteSync(recursive: false);
        setState(() {
          widget.item.image = encodedString;
        });

      }
    }catch(e){}

    // var image = await ImagePicker.pickImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
    // if( image != null ){
    //   var result = await FlutterImageCompress.compressWithList(
    //     image.readAsBytesSync(),
    //     minWidth: 800,
    //     quality: 80,
    //     rotate: 0,
    //   );
    //
    //   final Directory extDir = await getApplicationDocumentsDirectory();
    //   final String dirPath = '${extDir.path}/Pictures/quick_tmp';
    //   await Directory(dirPath).create(recursive: true);
    //   String cropFilePath = '$dirPath/cropped.jpg';
    //   File(cropFilePath)
    //     ..writeAsBytesSync(imglib.encodeJpg(imglib.decodeImage( result ), quality: 80));
    //   final smallFile = File(cropFilePath);
    //   String encodedString = encode( smallFile );
    //   smallFile.deleteSync(recursive: false);
    //   setState(() {
    //     widget.item.image = encodedString;
    //   });
    // }
  }

  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
    } else {
      form.save();
      Navigator.pop(context, true);
    }
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}