
import 'package:easy_localization/easy_localization.dart';
import 'package:latlong/latlong.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/request/pickup_create_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pickup_detail_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery/delivery_detail.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart' as google;

import 'delivery_way_update.dart';


class DeliveryListForPickup extends StatefulWidget {
  DeliveryListForPickup({Key key, this.onChange, this.trackingId, this.pickupGuid,
  this.merchantName, this.addNew, this.personId}) : super(key: key);
  final Function onChange;
  final String trackingId;
  final String pickupGuid;
  final String merchantName;
  final String personId;
  final bool addNew;

  @override
  _DeliveryListForPickupState createState() => _DeliveryListForPickupState();
}

class _DeliveryListForPickupState extends State<DeliveryListForPickup>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  PickupDetailResponse _deliveryListResponse;
  String _accessToken;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  bool _isFirst = true;
  UserService userService = UserService();
  bool _addNewRequest = false;

  @override
  void initState() {
    super.initState();
    _addNewRequest = widget.addNew;
  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      PickupServiceV2 pickupServiceV2 = new PickupServiceV2();
      _deliveryListResponse = await pickupServiceV2.detail(widget.trackingId);

    if( _deliveryListResponse!=null && _deliveryListResponse.pickupCreateRequest.items !=null ){

      if(_items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.pickupCreateRequest.items.forEach((element) {
        _items.add(element);
      });

      if( _addNewRequest ){
        _addNewRequest = false;

        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => DeliveryWayUpdate(
              onChange: (){
                _refreshController.requestRefresh();
              },
              pickupGuid: _deliveryListResponse.pickupCreateRequest.guid,
              pickupTownshipGuid: _deliveryListResponse.pickupCreateRequest.pickup.townshipGuid,
              senderGuid:_deliveryListResponse.pickupCreateRequest.pickup.personGuid,
              pickupDate: _deliveryListResponse.pickupCreateRequest.pickup.date,
              itemCount: _deliveryListResponse.pickupCreateRequest.items.length??0,
              totalWays: _deliveryListResponse.pickupCreateRequest.totalWays,
              pickupPOIGuid: _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_POST_OFFICE || _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_HIGHWAY || _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_POI?_deliveryListResponse.pickupCreateRequest.pickup.guid:null,
              sharedPrefs: _sharedPrefs,
            ),
          ),
        );
      }
    }

    try{
      if( _deliveryListResponse != null)
        widget.onChange(_deliveryListResponse.pickupCreateRequest.items);
    }catch(e){}

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: _deliveryContentService,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          _refreshController.requestRefresh();
          widget.onChange();
        },
        sharedPrefs: _sharedPrefs,
        pickupGuid: widget.pickupGuid,
        pickupCreateRequest: _deliveryListResponse.pickupCreateRequest,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  Widget build(BuildContext context) {

    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          appBar: AppBar(
            title: Text('${widget.merchantName} [ID: ${widget.personId}]', style: TextStyle(color: Colors.white)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),
            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),
            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              widget.onChange();
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
          floatingActionButton:
          _deliveryListResponse!=null && _deliveryListResponse.pickupCreateRequest.totalWays!=_deliveryListResponse.pickupCreateRequest.items.length?FloatingActionButton.extended(
          onPressed: () async{
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (context) => DeliveryWayUpdate(
                  onChange: (){
                    _refreshController.requestRefresh();
                  },
                  pickupGuid: _deliveryListResponse.pickupCreateRequest.guid,
                  pickupTownshipGuid: _deliveryListResponse.pickupCreateRequest.pickup.townshipGuid,
                  senderGuid:_deliveryListResponse.pickupCreateRequest.pickup.personGuid,
                  pickupDate: _deliveryListResponse.pickupCreateRequest.pickup.date,
                  itemCount: _deliveryListResponse.pickupCreateRequest.items.length??0,
                  totalWays: _deliveryListResponse.pickupCreateRequest.totalWays,
                  pickupPOIGuid: _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_POST_OFFICE || _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_HIGHWAY || _deliveryListResponse.pickupCreateRequest.pickup.addressType==Constant.ADDRESS_TYPE_POI?_deliveryListResponse.pickupCreateRequest.pickup.guid:null,
                  sharedPrefs: _sharedPrefs,
                ),
              ),
            );
          },
          label: Text(
            tr(LocaleKeys.lblItemListPickupAddWays),
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.add_circle_outline, color: Colors.white,),
          backgroundColor: Theme.of(context).primaryColor,
        ) : Container(),
        )
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser,
  this.pickupGuid, this.pickupCreateRequest}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final DeliveryContentServiceV2 deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String pickupGuid;
  final PickupCreateRequest pickupCreateRequest;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  int _selectedStep = 0;
  StepState _pickupStepState = StepState.indexed;
  StepState _dropOffStepState = StepState.indexed;
  bool _dropOffInactive = false;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Column(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 50,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    _stepState(post.status);

    return Container(
        padding: const EdgeInsets.only(left: 4.0, top: 10.0, right: 4.0),
        child: Wrap(
            children: <Widget>[
              Card(
                elevation: 8,
                // color: Colors.white,
                child: new Wrap(
                  children: <Widget>[
                    ListTile(
                      title: //Text('Way ID: ${post.trackingId}', style: TextStyle(color: Colors.white),),
                      _createText(post.status, post.trackingId, context),
                      subtitle: Text('Updated: '+new DateFormat('dd/MM/yyyy hh:mm a').format(post.updatedOn.toLocal()),
                        // style: TextStyle(color: Colors.blueGrey[600]),
                      ),
                      onTap: post.customerAddress==null?null : () async {
                        bool result = await Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) => DeliveryDetail(
                                wayId: post.trackingId,
                                onChange: (){
                                  widget.onChange();
                                },
                              ),
                            )
                        );
                        if( result!=null && result ){
                          widget.onChange();
                        }
                      },
                      trailing: post.customerAddress!=null? Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600],
                      ) : Icon(Icons.warning_amber_rounded,
                        // color: Colors.blueGrey[600],
                      ),
                    ),

                    ListTile(
                      title: Text('${post.customerAddress==null?'Address required':post.customerName} [${post.customerPrimaryPhone??''}]'),
                      subtitle: Text('${post.customerAddress??''}'),
                      trailing: post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY? Icon(Icons.edit,
                        // color: Colors.blueGrey[600],
                      ) : Icon(Icons.done,
                        // color: Colors.blueGrey[600],
                      ),
                      leading: Icon(Icons.location_on_sharp),
                      onTap: post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY? (){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => DeliveryWayUpdate(
                              onChange: (){
                                widget.onChange();
                              },
                              trackingId: post.trackingId,
                              sharedPrefs: widget.sharedPrefs,
                              pickupGuid: widget.pickupGuid,
                            ),
                          ),
                        );
                      }:null,
                    ),
                    post.deliveryItem!=null? ListTile(
                      title: Text(
                        'Total cost: ${new NumberFormat("#,###").format(post.deliveryItem.price + post.deliveryItem.forShowCharges)} Ks',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY?Text(tr(LocaleKeys.lblItemPriceEditable)):Container(),
                      trailing: post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY? Icon(Icons.edit,
                        // color: Colors.blueGrey[600],
                      ) : Icon(Icons.done,
                        // color: Colors.blueGrey[600],
                      ),
                      onTap: (post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY)? (){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => DeliveryWayUpdate(
                              onChange: (){
                                widget.onChange();
                              },
                              trackingId: post.trackingId,
                              sharedPrefs: widget.sharedPrefs,
                              pickupGuid: widget.pickupGuid,
                            ),
                          ),
                        );
                      }:null,
                    ) : ListTile(
                      leading: Icon(Icons.warning_amber_sharp, color: Theme.of(context).primaryColor),
                      title: Text(
                        'Parcel detail required',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY? Icon(Icons.edit, color: Colors.blueGrey[600],) : Icon(Icons.done, color: Colors.blueGrey[600],),
                      onTap: (post.status==Constant.STATUS_ON_WAY || post.status==Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY)? (){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => DeliveryWayUpdate(
                              onChange: (){
                                widget.onChange();
                              },
                              trackingId: post.trackingId,
                              pickupTownshipGuid: post.townshipGuid,
                              pickupGuid: widget.pickupGuid,
                              sharedPrefs: widget.sharedPrefs,
                            ),
                          ),
                        );
                      }:null,
                    ) ,
                    post.deliveryItem!=null && post.deliveryItem.image!=null? Container(
                      child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${post.deliveryItem.image}',),
                    ): Container(),

                  ],
                ),
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )

    );
  }

  Icon _createStatusIcon(String status ){

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      return Icon(Icons.get_app_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY){
      return Icon(Icons.upload_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_COMPLETED ){
      return Icon(Icons.done_all, color: Colors.white);
    }else if( status == Constant.STATUS_CANCELED ){
      return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_RETRY ){
      return Icon(Icons.refresh, color: Colors.white);
    }else if( status == Constant.STATUS_RETURN ){
      return Icon(Icons.backspace_sharp, color: Colors.white);
    }else
      return Icon(Icons.view_list, color: Colors.white);

    // if( status == Constant.STATUS_CREATED || status == Constant.STATUS_PICKUP_CREATED){
    //   return Icon(Icons.view_list, color: Colors.white);
    // }else if( status == Constant.STATUS_ASSIGNED || status == Constant.STATUS_PICKUP_SCHEDULED){
    //   return Icon(Icons.person_pin_circle, color: Colors.white);
    // }else if( status == Constant.STATUS_ON_WAY || status == Constant.STATUS_PICKUP_ON_WAY){
    //   return Icon(Icons.local_shipping_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_COMPLETED ){
    //   return Icon(Icons.done_all, color: Colors.white);
    // }else if( status == Constant.STATUS_CANCELED ){
    //   return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_RETRY ){
    //   return Icon(Icons.refresh, color: Colors.white);
    // }else if( status == Constant.STATUS_RETURN ){
    //   return Icon(Icons.backspace_sharp, color: Colors.white);
    // }else
    //   return Icon(Icons.view_list, color: Colors.white);

  }

  /*Text _createText(String status, String tracking, BuildContext context ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
    || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }

    return Text(
      '${tr(LocaleKeys.lblWayID)} $tracking',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }*/

  Text _createText(String status, String tracking, BuildContext context ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }
    title =  _createStatus( status );
    if( widget.post.customerName==null && widget.post.customerAddress==null ){
      statusColor = Colors.yellow[700];
      title =  tr(LocaleKeys.lblDeliParcelWaiting);
    }

    return Text(
      '$title\n${tr(LocaleKeys.lblWayID)} $tracking',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }

  String _createStatus(String status ){
    String title = '';
    if( widget.authorityResponse.authorities!=null
        && widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')){
      if( status == Constant.STATUS_CREATED ){
        title = tr(LocaleKeys.lblItemListDropOffQueue);
      }else if( status == Constant.STATUS_ASSIGNED ){
        title = tr(LocaleKeys.lblItemListToDropOff);
      }else if( status == Constant.STATUS_ON_WAY ){
        title = tr(LocaleKeys.lblItemListDropOffOnWay);
      }else if( status == Constant.STATUS_RETURN ){
        title = tr(LocaleKeys.lblItemListReturn);
      }else if( status == Constant.STATUS_RETRY ){
        title = tr(LocaleKeys.lblItemListRetry);
      }else if( status == Constant.STATUS_CANCELED ){
        title = tr(LocaleKeys.lblItemListCancelled);
      }else if( status == Constant.STATUS_COMPLETED ){
        title = tr(LocaleKeys.lblItemListCompleted);
      }else if( status == Constant.STATUS_PICKUP_CREATED ){
        title = tr(LocaleKeys.lblItemListPickupQueue);
      }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
        title = tr(LocaleKeys.lblItemListToPickup);
      }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
        title = tr(LocaleKeys.lblItemListPickupOnWay);
      }
    }else{
      if( widget.pickupCreateRequest.status =='Completed' ){
        if( widget.post.status == 'Created')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerInTransit);
        else if( widget.post.status == 'Assigned')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerInTransit);
        else if( widget.post.status == 'OnWay')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerOnTheWay);
        else if( widget.post.status == 'Completed')
          title = tr(LocaleKeys.lblDeliveryStatusForCustomerDelivered);
        else title = widget.post.status;
      }else if( widget.pickupCreateRequest.status =='PickupCreated' || widget.pickupCreateRequest.status =='PickupScheduled' || widget.pickupCreateRequest.status =='PickupOnWay' ){
        title = tr(LocaleKeys.lblDeliveryStatusForCustomerPickup);
      }else if( widget.pickupCreateRequest.status =='Canceled' ){
        title = 'Canceled';
      }else{
        title = widget.pickupCreateRequest.status;
      }
    }
    return title;
  }

  StepState _stepState(String status){
    if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.indexed;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CANCELED ){
      _pickupStepState = StepState.error;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.indexed;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( status == Constant.STATUS_COMPLETED){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( status == Constant.STATUS_RETURN){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.error;
      _selectedStep = 1;
    }
    return _pickupStepState;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}