import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:flutter_uploader/flutter_uploader.dart';
import 'package:image_picker/image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/price_method_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/file_service.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'delivery_address.dart';
import 'delivery_item.dart';
import 'package:image/image.dart' as imglib;
import 'package:path/path.dart';


class DeliveryWayUpdate extends StatefulWidget {
  DeliveryWayUpdate({Key key, this.onChange,
  this.addressBookItemRecipient, this.pickupGuid, this.pickupTownshipGuid,
  this.senderGuid, this.pickupDate, this.trackingId, this.sharedPrefs,
  this.itemCount, this.totalWays, this.pickupPOIGuid}) : super(key: key);
  final Function onChange;
  final AddressBookItem addressBookItemRecipient;
  final String pickupGuid;
  final String pickupTownshipGuid;
  final String pickupPOIGuid;
  final String senderGuid;
  final DateTime pickupDate;
  final String trackingId;
  final SharedPreferences sharedPrefs;
  final int itemCount;
  final int totalWays;

  @override
  _DeliveryWayUpdateState createState() => _DeliveryWayUpdateState();
}

class _DeliveryWayUpdateState extends State<DeliveryWayUpdate> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;

  final TextEditingController _controller_senderComment = new TextEditingController();
  final TextEditingController _controller_recipientAddress = new TextEditingController();
  final TextEditingController _controller_recipientName = new TextEditingController();
  final TextEditingController _controller_recipientPrimaryPhone = new TextEditingController();
  final TextEditingController _controller_recipientOtherPhones = new TextEditingController();
  final TextEditingController _controller_recipientDate = new TextEditingController();
  final TextEditingController _controller_edit_deliCharges = new TextEditingController();
  final TextEditingController _controller_edit_charges_by_admin = new TextEditingController();
  DateTime _recipientDate = new DateTime.now();
  List<DateTime> _dropOffDateLabels = <DateTime>[DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now(),DateTime.now()];
  List<bool> _selectionsDropOff = List.generate(6, (_)=> false);
  DeliveryItem _deliveryItem = new DeliveryItem(
    name: 'Parcel',
    weight: 2,
    width: 2,
    height: 2,
    length: 2,
    priceOption: "Unpaid",
    price: 0,
    chargeOption: "Unpaid",
    charges: 0,
    forShowCharges: 0,
    fragile: false,
    insurance: false,
  );
  User _currentUser = new User();
  bool _isFirst = true;
  bool _requiredToRefund = false;
  bool _allow_cash_on_advanced = false;

  AddressBookItem _addressBookItemRecipient;
  bool _itemDetailAlreadyCheck = true;
  bool _itemDetailError = false;
  double _promoAmount = 0;
  bool _editDeliCharges = false;
  String _lang = Constant.LANGUAGE_CODE_MY;
  double _deliveryCharges;
  bool _editChargesByAdmin = false;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  bool _fullEntry = false;
  bool _hasNext = false;
  DeliveryDetailResponse _detailResponse;
  String _previousTownshipGuid;
  String _pickupTownshipGuid;
  String _pickupPOIGuid;
  String _senderGuid;
  DeliveryContactItem _deliveryContactItem;
  DateTime _pickupDate;
  int _totalAddedCount;
  ActiveDeliverymanResponse _deliverymanResponse;
  DeliveryForDeliveryItem _employeeSearchItem;
  PersonService _personService = new PersonService();
  bool _autoAssignDeliveryman = true;
  bool _approved = true;
  String _deliveryMethods = "Regular";
  Timer _debounce;
  bool _alreadyPaid = false;
  Map<String, UploadItem> _tasks = {};
  int _successCount = 0;
  ScrollController _scrollController;

  @override
  void initState() {
    super.initState();

    _scrollController = new ScrollController(initialScrollOffset: 0.0, keepScrollOffset: true,);
    setState( () {

      _pickupTownshipGuid = widget.pickupTownshipGuid;
      _pickupPOIGuid = widget.pickupPOIGuid;
      _senderGuid = widget.senderGuid;

      _selectionsDropOff[0]=true;
      int dayPlus = (widget.pickupDate!=null && widget.pickupDate.toLocal().hour>14? 1 : 0);
      for( int i=0; i< _dropOffDateLabels.length; i++){
        _dropOffDateLabels[i] = new DateTime( _recipientDate.year, _recipientDate.month, _recipientDate.day + i + dayPlus );
      }
      _recipientDate = _dropOffDateLabels[0];
      _addressBookItemRecipient = widget.addressBookItemRecipient;
      if( widget.addressBookItemRecipient != null){
        _checkPrice( );
      }
      _totalAddedCount = widget.itemCount??0;
    });

    _loadCurrentUser( );
    // _loadTownships();
  }

  void _loadCurrentUser( ) async{
    if( _isFirst ){
      _lang = await getLanguage( );
      _isFirst = false;
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();

      if( widget.trackingId != null ){
        DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
        _detailResponse = await _deliveryContentService.detail(widget.trackingId);
        _deliveryItem = _detailResponse.items[0];
        _deliveryCharges = _deliveryItem.charges??0;
        _approved = _detailResponse.approved;
        _detailResponse.contacts.forEach((element) {
          if( element.contactType== Constant.CONTACT_TYPE_PICKUP){
            _pickupTownshipGuid = element.townshipGuid;
            _pickupDate = element.date;
            _senderGuid = element.personGuid;
            _pickupPOIGuid = element.addressType==Constant.ADDRESS_TYPE_POST_OFFICE || element.addressType==Constant.ADDRESS_TYPE_HIGHWAY || element.addressType==Constant.ADDRESS_TYPE_POI?element.guid:null;
          }else if( element.contactType== Constant.CONTACT_TYPE_DROP_OFF){
            _deliveryContactItem = element;
          }else if( element.contactType== Constant.CONTACT_TYPE_PICKUP_EMPLOYEE){

          }else if( element.contactType== Constant.CONTACT_TYPE_DROP_OFF_EMPLOYEE){

          }
        });
        setState(() {
          _fullEntry = true;
        });
        if( _deliveryContactItem != null ){
          _addressBookItemRecipient = new AddressBookItem(
              personGuid: _deliveryContactItem.personGuid,
              name: _deliveryContactItem.name,
              address: _deliveryContactItem.address,
           street: _deliveryContactItem.street,
               townshipName: _deliveryContactItem.township,
               townshipGuid: _deliveryContactItem.townshipGuid,
               addressType: _deliveryContactItem.addressType,
               primaryPhone:_deliveryContactItem.primaryPhone,
           otherPhones: _deliveryContactItem.otherPhones,
           email: _deliveryContactItem.email,
           latitude: _deliveryContactItem.latitude,
           longitude: _deliveryContactItem.longitude,
           comment: _deliveryContactItem.comment,
               guid: _deliveryContactItem.guid,
               stateGuid: _deliveryContactItem.stateGuid,
          );

          _previousTownshipGuid = _deliveryContactItem.townshipGuid;
          _recipientDate = _deliveryContactItem.date;

          _controller_recipientAddress.text = _addressBookItemRecipient.address
              +(_addressBookItemRecipient.street!=null && _addressBookItemRecipient.street.trim().isNotEmpty? ','+_addressBookItemRecipient.street.trim() : '')
              +(_addressBookItemRecipient.townshipName!=null && _addressBookItemRecipient.townshipName.trim().isNotEmpty? ','+_addressBookItemRecipient.townshipName.trim() : '');
          _controller_recipientPrimaryPhone.text = _addressBookItemRecipient.primaryPhone??'';
          _controller_recipientOtherPhones.text = _addressBookItemRecipient.otherPhones??'';
          if(_addressBookItemRecipient.addressType != 'Post Office'
              && _addressBookItemRecipient.addressType != 'Highway Gate'
              && _addressBookItemRecipient.addressType != 'POI')
            _controller_recipientName.text = _addressBookItemRecipient.name??'';

          setState(() {
            _itemDetailAlreadyCheck = true;
          });
          // _checkPrice();
        }
      }

      await _availableDates();
      for( int i=0; i< _dropOffDateLabels.length; i++){
       if( _dropOffDateLabels[i] == _recipientDate){
         _selectionsDropOff[0]=false;
         _selectionsDropOff[i]=true;
         break;
       }
      }
      if(_authorityResponse.authorities !=null
          && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))){
        // setState(() {
        //   _allow_cash_on_advanced = true;
        // });
        _deliverymanResponse = await _personService.activeDeliveryman();
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          key: _scaffoldKey,
          title: Text(tr(LocaleKeys.lblDeliCreateTitle), style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Center(
            child: SingleChildScrollView(
              child: Container(
                margin: const EdgeInsets.only(top: 8.0, bottom: 24.0, left: 8.0, right: 8.0),
                child: _fullEntry? Card(
                  // color: Colors.white,
                  elevation: 8,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment:CrossAxisAlignment.center,
                    children: <Widget>[
                      widget.totalWays!=null? ListTile(
                        title: LinearProgressIndicator(
                          // backgroundColor: Colors.blueGrey[200],
                          value: ( (_totalAddedCount??0).toDouble() / (widget.totalWays??0).toDouble()).toDouble(),
                          valueColor: AlwaysStoppedAnimation<Color>(Theme.of(context).primaryColor),
                        ),
                        subtitle: Text('${tr(LocaleKeys.lblPickupDetailAddedWayCount)} ${_totalAddedCount??0}/${widget.totalWays??0}'),
                        // trailing: Icon(_expand? Icons.keyboard_arrow_up_sharp : Icons.keyboard_arrow_down_sharp, color: Colors.white,),
                      ): Container(),
                      _deliveryItem!=null && _deliveryItem.image!=null? GestureDetector(
                        child: _deliveryItem.image.length>100? Container(
                          child: decode( _deliveryItem.image ),
                          decoration: _deliveryItem.image!=null ?BoxDecoration(
                              border: Border.all(color: Colors.green[200], width: 3)
                          ) : null,
                        ) : Container(
                          child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_deliveryItem.image}',),
                        ),
                      ) : Container(
                        // margin: EdgeInsets.only(top: 0, bottom: 32),
                        // child: new Image.asset("assets/images/landing_background_large.jpg"),
                      ),
                      _deliveryContactItem==null?Container(
                        margin: EdgeInsets.only(top: 16, bottom: 8),
                        child: CheckboxListTile(
                          value: _fullEntry,
                          onChanged: ( value){
                            setState(() { _fullEntry = value; });
                          },
                          title: Text(
                            tr(LocaleKeys.lblFillDetailInfoForWay),
                            // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                          ),
                        ),
                      ): Container(),
                      // address to deliver
                      Container(
                        padding: EdgeInsets.all(16.0),
                        child: TextField(
                          textAlign: TextAlign.center,
                          keyboardType: TextInputType.text,
                          decoration: InputDecoration(
                            hintText: tr(LocaleKeys.lblDeliCreateDropOff),//Deliver to
                            hintStyle: TextStyle(fontSize: 14),
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(16),
                              borderSide: BorderSide(
                                  width: 1,
                                  style: BorderStyle.solid,
                                  color: Theme.of(context).primaryColor
                              ),
                            ),
                            filled: true,
                            contentPadding: EdgeInsets.all(8),
                            fillColor: Theme.of(context).backgroundColor,
                            prefixIcon: Icon(Icons.download_sharp,),
                            suffixIcon : IconButton(
                              icon: Icon(Icons.add_location_sharp,
                                  // color: Theme.of(context).primaryColorDark
                              ),
                              onPressed: (){print('Icon press');},
                            ),
                          ),
                          onTap: ()async{
                            FocusScope.of(context).requestFocus(new FocusNode());
                            AddressBookItem addressBookItem = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => DeliveryAddress(
                                  title: tr(LocaleKeys.lblDeliCreateDropOff),
                                  addressType: 'Customer',
                                  wayType: 'deliver',
                                ),
                              ),
                            );

                            if( addressBookItem != null) {
                              _addressBookItemRecipient = addressBookItem;
                              _controller_recipientAddress.text = _addressBookItemRecipient.address
                                  +(_addressBookItemRecipient.street!=null && _addressBookItemRecipient.street.trim().isNotEmpty? ','+_addressBookItemRecipient.street.trim() : '')
                                  +(_addressBookItemRecipient.townshipName!=null && _addressBookItemRecipient.townshipName.trim().isNotEmpty? ','+_addressBookItemRecipient.townshipName.trim() : '');
                              _controller_recipientPrimaryPhone.text = addressBookItem.primaryPhone??'';
                              _controller_recipientOtherPhones.text = addressBookItem.otherPhones??'';
                              if(addressBookItem.addressType != Constant.ADDRESS_TYPE_POST_OFFICE
                                  && addressBookItem.addressType != Constant.ADDRESS_TYPE_HIGHWAY
                                  && addressBookItem.addressType != Constant.ADDRESS_TYPE_POI)
                              _controller_recipientName.text = addressBookItem.name??'';
                              _checkPrice();
                            }
                          },
                          controller: _controller_recipientAddress,
                        ),
                      ),
                      Container(
                        margin: EdgeInsets.only(top: 0, bottom: 16),
                        child: new SafeArea(
                            top: false,
                            bottom: false,
                            child: new Form(
                              key: _formKey,
                              autovalidateMode: AutovalidateMode.onUserInteraction,
                              child: new Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[

                                  Card(
                                      // color: Theme.of(context).backgroundColor,
                                      elevation: 0,
                                      child: new Column(
                                      crossAxisAlignment: CrossAxisAlignment.start,
                                      mainAxisSize: MainAxisSize.min,
                                      children: <Widget>[
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                          child: Text('${tr(LocaleKeys.lblDeliCreateDropOffDate)} ${new DateFormat('dd/MM/yyyy').format(_recipientDate.toLocal())}'),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                          child: ToggleButtons(
                                            children: <Widget>[
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                    _dayToLabel(_dropOffDateLabels[0]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[1]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[2]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[3]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                child: Text(
                                                  _dayToLabel(_dropOffDateLabels[4]),
                                                  style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                  textAlign: TextAlign.center,
                                                ),
                                              ),
                                              Container(
                                                  padding: EdgeInsets.only(top:0, bottom: 0, left:4, right: 4),
                                                  child: FlatButton(
                                                    onPressed: () {
                                                      setState(() {
                                                        for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
                                                          if (buttonIndex == _selectionsDropOff.length-1) {
                                                            _selectionsDropOff[buttonIndex] = true;
                                                          } else {
                                                            _selectionsDropOff[buttonIndex] = false;
                                                          }
                                                        }
                                                      });
                                                      _chooseDate(context, new DateFormat('yyyy-MM-dd').format(_recipientDate.toLocal()), false);
                                                    },
                                                    // color: Theme.of(context).backgroundColor,
                                                    child: Text(
                                                      tr(LocaleKeys.lblAntherDay),
                                                      style: TextStyle(fontSize: 10.0, fontFamily: 'Roboto'),
                                                      textAlign: TextAlign.center,
                                                    ),
                                                  )
                                              ),
                                            ],
                                            onPressed: (int index) {
                                              setState(() {
                                                _recipientDate = _dropOffDateLabels[index];
                                                for (int buttonIndex = 0; buttonIndex < _selectionsDropOff.length; buttonIndex++) {
                                                  if (buttonIndex == index) {
                                                    _selectionsDropOff[buttonIndex] = true;
                                                  } else {
                                                    _selectionsDropOff[buttonIndex] = false;
                                                  }
                                                }
                                              });
                                            },
                                            isSelected: _selectionsDropOff,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblRecipientName),
                                                labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            validator: (val) => val.isEmpty ? 'Name required!' : null,
                                            controller: _controller_recipientName,
                                            keyboardType: TextInputType.text,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '09xxxxxxxxx',
                                              labelText: tr(LocaleKeys.lblRecipientMobile),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                                            controller: _controller_recipientPrimaryPhone,
                                            keyboardType: TextInputType.phone,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: "Use \",\" for multiple numbers",
                                              labelText: tr(LocaleKeys.lblRecipientOtherPhones),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            // initialValue: '',
                                            controller: _controller_recipientOtherPhones,
                                            keyboardType: TextInputType.text,
                                          ),
                                        ),
                                        Container(
                                          padding: EdgeInsets.only(top:0, bottom: 16, left:16, right: 16),
                                          child: new TextFormField(
                                            decoration: InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblRemark),
                                              labelStyle: TextStyle(fontSize: 14)
                                            ),
                                            controller: _controller_senderComment,
                                            keyboardType: TextInputType.text,
                                            // maxLines: null,
                                            // minLines: 1,
                                          ),
                                        ),
                                      ]
                                    )
                                  ),
                                  // Container(
                                  //   padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                  //   child: Card(
                                  //   color: _itemDetailError? Colors.red[100] : (_itemDetailAlreadyCheck? Colors.green[200] : Theme.of(context).backgroundColor),
                                  //   elevation: 1,
                                  //   child: ListTile(
                                  //       title: Text(tr(LocaleKeys.lblItemDetail)),
                                  //       subtitle: Text(_itemDetailAlreadyCheck? '${_deliveryItem.weight}kg${_deliveryItem.price>0?", "+NumberFormat('#,###').format(_deliveryItem.price)+" Ks": ""}' : ''),
                                  //       trailing: Icon(Icons.edit_sharp),
                                  //       leading: Icon(Icons.work_sharp),
                                  //       onTap: _addressBookItemRecipient==null? null : () async{
                                  //         bool isAdd = await Navigator.push(
                                  //             context,
                                  //             MaterialPageRoute(
                                  //               builder: (BuildContext context) => NewDeliveryItem(
                                  //                 title: tr(LocaleKeys.lblItemDetail),
                                  //                 item: _deliveryItem,
                                  //                 townshipGuid: _addressBookItemRecipient.townshipGuid,
                                  //                 pickupTownshipGuid: _pickupTownshipGuid,
                                  //                 lang: _lang,
                                  //               ),
                                  //             )
                                  //         );
                                  //         if( isAdd !=null && isAdd ) {
                                  //           setState(() {
                                  //             _itemDetailAlreadyCheck = true;
                                  //             _itemDetailError = false;
                                  //           });
                                  //           _checkPrice();
                                  //         }
                                  //       },
                                  //     ),
                                  //   ),
                                  // ),
                                  Container(
                                    padding: EdgeInsets.only(top:0, bottom: 0, left:16, right: 16),
                                    child: NewDeliveryItem(
                                      title: tr(LocaleKeys.lblItemDetail),
                                      item: _deliveryItem,
                                      lang: _lang,
                                      sharedPrefs: widget.sharedPrefs,
                                      onChange: (){
                                        if (_debounce?.isActive ?? false) _debounce.cancel();
                                        _debounce = Timer(const Duration(seconds: 3), () {
                                          _checkPrice();
                                        });
                                      },
                                    ),
                                  ),
                                  _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")?
                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[200],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliCharges)),
                                        subtitle: Text('${_deliveryCharges !=null? NumberFormat('#,###').format(_deliveryItem.charges)+' Ks' : ''}'),
                                        trailing: Icon(!_editChargesByAdmin?Icons.edit_sharp : Icons.clear),
                                        leading: Icon(Icons.monetization_on_sharp),
                                        onTap: _deliveryCharges==null? null: (){
                                          setState(() {
                                            _editChargesByAdmin = !_editChargesByAdmin;
                                          });
                                        },
                                      ),
                                    ),
                                  )
                                  :Container(
                                    padding: EdgeInsets.only(top:32, bottom: 0, left:16, right: 16),
                                    child: Card(
                                    color: Colors.green[200],
                                    elevation: 1,
                                    child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliCharges)),
                                        subtitle: Text('${_deliveryCharges!=null? NumberFormat('#,###').format(_deliveryItem.charges)+' Ks' : ''}'),
                                        leading: Icon(Icons.monetization_on_sharp),
                                      ),
                                    ),
                                  ),
                                  _editChargesByAdmin?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblDeliCharges),
                                            ),
                                            controller: _controller_edit_charges_by_admin,
                                            keyboardType: TextInputType.number,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done,
                                            // color: Colors.white
                                        ),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: () {
                                          setState(() {
                                            _editChargesByAdmin = !_editChargesByAdmin;
                                            if( _controller_edit_charges_by_admin.text.trim().isNotEmpty) {
                                              _deliveryItem.charges =
                                                  double.parse(
                                                      _controller_edit_charges_by_admin
                                                          .text);
                                              _deliveryCharges = _deliveryItem.charges;
                                            }
                                          });
                                        },
                                        label: Text(
                                          tr(LocaleKeys.btnOK),
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            // color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ): Container(),

                                  Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: Card(
                                      color: Colors.green[200],
                                      elevation: 1,
                                      child: ListTile(
                                        title: Text(tr(LocaleKeys.lblDeliChargesToShowOnRecipient)),
                                        subtitle: Text('${_deliveryItem.forShowCharges>0? NumberFormat('#,###').format(_deliveryItem.forShowCharges)+' Ks' : ''}'),
                                        trailing: Icon(!_editDeliCharges?Icons.edit_sharp : Icons.clear),
                                        leading: Icon(Icons.monetization_on_outlined),
                                        onTap: _deliveryCharges==null? null: (){
                                          setState(() {
                                            _editDeliCharges = !_editDeliCharges;
                                          });
                                        },
                                      ),
                                    ),
                                  ),
                                  _editDeliCharges?Container(
                                    margin: const EdgeInsets.only(top:0, bottom: 8, left:16, right: 16),
                                    child:new Row(children: <Widget>[
                                      new Expanded(
                                          child: new TextFormField(
                                            decoration: new InputDecoration(
                                              hintText: '',
                                              labelText: tr(LocaleKeys.lblDeliChargesToShowOnRecipient),
                                            ),
                                            controller: _controller_edit_deliCharges,
                                            keyboardType: TextInputType.number,
                                          )
                                      ),
                                      RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.done,
                                            // color: Colors.white
                                        ),
                                        color: Theme.of(context).primaryColor,
                                        onPressed: () {
                                          setState(() {
                                            _editDeliCharges = !_editDeliCharges;
                                            if( _controller_edit_deliCharges.text.trim().isNotEmpty)
                                              _deliveryItem.forShowCharges = double.parse(_controller_edit_deliCharges.text);
                                          });
                                        },
                                        label: Text(
                                          tr(LocaleKeys.btnOK),
                                          style: TextStyle(
                                            fontSize: 16,
                                            fontWeight: FontWeight.w400,
                                            // color: Colors.white,
                                          ),
                                        ),
                                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                      ),
                                    ]),
                                  ) : Container(),
                                  _deliveryItem.price>0 && _allow_cash_on_advanced? Container(
                                    padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                    child: CheckboxListTile(
                                      value: _requiredToRefund,
                                      onChanged: ( value) {setState(() { _requiredToRefund = value; });},
                                      title: Text(
                                        '${tr(LocaleKeys.lblRequiredToRefund)}',
                                        // style: TextStyle(color: Colors.black),
                                      ),
                                    ),
                                  ) : Container(),
                                ],

                              ),
                            )
                        ),
                      ),
                      Container(
                        margin: const EdgeInsets.only(top: 16.0, left: 16.0, right: 16.0),
                        child:new Column(
                          mainAxisSize: MainAxisSize.min,
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            new Text(
                              tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethod),
                              style: new TextStyle(fontSize: 16.0, fontWeight: FontWeight.bold, color: Theme.of(context).primaryColor),
                            ),
                            new RadioListTile(
                              value: 'Regular',
                              groupValue: _deliveryMethods,
                              onChanged: ( value){
                                setState(() { _deliveryMethods = value; });
                                _checkPrice();
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodRegular),
                                style: new TextStyle(
                                    fontSize: 16.0,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                            new RadioListTile(
                              value: 'Same Day',
                              groupValue: _deliveryMethods,
                              onChanged: ( value){
                                setState(() {
                                  _deliveryMethods = value;
                                });
                                _checkPrice();
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodSameDay),
                                style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                              ),
                            ),
                            new RadioListTile(
                              value: 'Urgent',
                              groupValue: _deliveryMethods,
                              onChanged: ( value){
                                setState(() {
                                  _deliveryMethods = value;
                                });
                                _checkPrice();
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodUrgent),
                                style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                              ),
                            ),
                            new RadioListTile(
                              value: '24hr',
                              groupValue: _deliveryMethods,
                              onChanged: ( value){
                                setState(() {
                                  _deliveryMethods = value;
                                });
                                _checkPrice();
                              },
                              title: Text(
                                tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin4hrs),
                                style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                              ),
                            ),
                            // new RadioListTile(
                            //   value: 'Within4hrs',
                            //   groupValue: _deliveryMethods,
                            //   onChanged: ( value){
                            //     setState(() {
                            //       _deliveryMethods = value;
                            //     });
                            //     _checkPrice();
                            //   },
                            //   title: Text(
                            //     tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin4hrs),
                            //     style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                            //   ),
                            // ),
                            // new RadioListTile(
                            //   value: 'Within1hr',
                            //   groupValue: _deliveryMethods,
                            //   onChanged: ( value){
                            //     setState(() {
                            //       _deliveryMethods = value;
                            //     });
                            //     _checkPrice();
                            //   },
                            //   title: Text(
                            //     tr(LocaleKeys.lblDeliveryWayUpdateDeliveryMethodWithin1hr),
                            //     style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                            //   ),
                            // ),
                          ],
                        ),
                      ),

                      _authorityResponse.authorities!=null &&
                          (_authorityResponse.authorities.contains("MANAGER_WRITE")
                              || _authorityResponse.authorities.contains("OPERATOR_WRITE"))? Container(
                                padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                                child: CheckboxListTile(
                                  value: _approved,
                                  onChanged: ( value) {setState(() { _approved = value; });},
                                  title: Text(
                                    tr(LocaleKeys.lblConfirmAssignToDeliveryman),
                                    // style: TextStyle(color: Colors.black),
                                  ),
                                ),
                              ) : Container(),

                      _authorityResponse.authorities!=null &&
                          (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))
                          && _approved? Container(
                        padding: EdgeInsets.only(top:0, bottom: 16, left:8, right: 8),
                        child: CheckboxListTile(
                          value: _autoAssignDeliveryman,
                          onChanged: ( value) {setState(() { _autoAssignDeliveryman = value; });},
                          title: Text(
                            '${tr(LocaleKeys.lblAutoAssignDeliveryman)}',
                            // style: TextStyle(color: Colors.black),
                          ),
                        ),
                      ) : Container(),
                      !_autoAssignDeliveryman? _deliverySelection(context): Container(),

                      // _authorityResponse.authorities!=null &&
                      //     (_authorityResponse.authorities.contains("MANAGER_WRITE")
                      //         || _authorityResponse.authorities.contains("OPERATOR_WRITE"))? Container(
                      //           padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
                      //           child: CheckboxListTile(
                      //             value: _alreadyPaid,
                      //             onChanged: ( value) {setState(() { _alreadyPaid = value; });},
                      //             title: Text(
                      //               tr(LocaleKeys.lblDeliveryChargesAlreadyPaid),
                      //               // style: TextStyle(color: Colors.black),
                      //             ),
                      //           ),
                      //         ) : Container(),

                      Container(
                        padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 0),
                        width: double.infinity,
                        child: RaisedButton(
                          onPressed: _addressBookItemRecipient==null || _deliveryCharges==null || _processing? null :  () { _submitForm( context ); },
                          // color: Theme.of(context).primaryColorDark,
                          elevation: 1,
                          child: Text(
                            tr(LocaleKeys.btnOK),
                            style: Theme.of(context).textTheme.headline5,
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(32.0)),
                        ),
                      ),
                    ],
                  ),
                ) : Card(
                  // color: Colors.white,
                  elevation: 8,
                  child: Form(
                    key: _formKey,
                    autovalidateMode: AutovalidateMode.onUserInteraction,
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment:CrossAxisAlignment.center,
                      children: <Widget>[
                        widget.totalWays!=null? ListTile(
                          title: LinearProgressIndicator(
                            backgroundColor: Colors.blueGrey[200],
                            value: ( (_totalAddedCount??0).toDouble() / (widget.totalWays??0).toDouble()).toDouble(),
                            valueColor: AlwaysStoppedAnimation<Color>(Theme.of(context).primaryColor),
                          ),
                          subtitle: Text('${tr(LocaleKeys.lblPickupDetailAddedWayCount)} ${_totalAddedCount??0}/${widget.totalWays??0}'),
                          // trailing: Icon(_expand? Icons.keyboard_arrow_up_sharp : Icons.keyboard_arrow_down_sharp, color: Colors.white,),
                        ): Container(),
                        // _townshipSelection( context ),
                        // GestureDetector(
                        //   child: Container(
                        //     child: _deliveryItem.image==null ? Icon(Icons.photo, color: Colors.grey[100], size: 400,) : decode( _deliveryItem.image ),
                        //     decoration: _deliveryItem.image!=null ?BoxDecoration(
                        //         border: Border.all(color: Colors.green[50], width: 3)
                        //     ) : null,
                        //   ),
                        //   onTap: ()async{
                        //     if(_deliveryItem.image !=null && _deliveryItem.image.isNotEmpty){
                        //       Navigator.push(
                        //         context,
                        //         MaterialPageRoute(
                        //           builder: (context) => FullScreenWrapper(
                        //             imageProvider: MemoryImage(base64Decode(_deliveryItem.image)),// MemoryImage(base64Decode(widget.item.image.substring(22))) //NetworkImage( post.image ),
                        //           ),
                        //         ),
                        //       );
                        //     }
                        //   },
                        // ),
                        Container(
                          height: 400,
                          child: GridView.builder(
                              itemCount: _images.length,
                              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                                  crossAxisCount: 3
                              ),
                              controller: _scrollController,
                              itemBuilder: (BuildContext context, int index) {
                                return GridTile(
                                  child: GestureDetector(
                                    child:Image.file(File(_images[index].path), fit: BoxFit.cover),
                                    onTap: ()async{
                                        Navigator.push(
                                          context,
                                          MaterialPageRoute(
                                            builder: (context) => FullScreenWrapper(
                                              imageProvider: FileImage(File(_images[index].path))
                                            ),
                                          ),
                                        );
                                    },
                                  ),
                                  header: Align(
                                    alignment: Alignment.topRight,
                                    child: Material(
                                      elevation: 12,
                                      color: Colors.transparent,
                                      child: InkWell(
                                        child: new Container(
                                          padding: const EdgeInsets.all(8.0),
                                          decoration: new BoxDecoration(
                                            shape: BoxShape.circle,
                                            // borderRadius: new BorderRadius.circular(30.0),
                                            color: Theme.of(context).primaryColor,
                                          ),
                                          child: new Text(
                                            '${index + 1}',
                                            style: Theme.of(context).textTheme.headline5,
                                          ),
                                        ),
                                      ),
                                    ),
                                  ),
                                  footer: _imageStatus(context, index),
                                );
                              }
                          ),
                        ),
                        _images.isNotEmpty && _successCount == _images.length? Container() :_attachedImage( context ),
                        _images.isNotEmpty && _successCount == _images.length? Container() :Container(
                            margin: EdgeInsets.only(left: 0, right: 0, top: 16, bottom: 8),
                            child:CheckboxListTile(
                              value: _fullEntry,
                              onChanged: ( value){
                                setState(() { _fullEntry = value; });
                              },
                              title: Text(
                                tr(LocaleKeys.lblFillDetailInfoForWay),
                                // style: new TextStyle(fontSize: 16.0, color: Colors.black),
                              ),
                            )
                        ),
                        Container(
                          margin: const EdgeInsets.only(left: 8.0, right: 8.0, top: 8, bottom: 24),
                          child: ButtonBar(
                              alignment: MainAxisAlignment.center,
                              children: <Widget>[
                                _images.isNotEmpty && _successCount != _images.length && _totalAddedCount+_images.length<=widget.totalWays??0? RaisedButton.icon(
                                  elevation: 0.0,
                                  icon: Icon(Icons.done_all,
                                      // color: Colors.white
                                  ),
                                  color: Theme.of(context).primaryColor,
                                  onPressed: () async {
                                    _uploadMultipleImages( context, 0 );
                                  },
                                  label: Text(
                                    'Finish',
                                    style: TextStyle(
                                      fontSize: 16,
                                      fontWeight: FontWeight.w400,
                                      // color: Colors.white,
                                    ),
                                  ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                ): Container(),
                                _images.isNotEmpty !=null && _successCount == _images.length? RaisedButton.icon(
                                  elevation: 0.0,
                                  icon: Icon(Icons.close,
                                      // color: Colors.white
                                  ),
                                  color: Theme.of(context).primaryColor,
                                  onPressed: () async {
                                    if( _successCount == _images.length ){
                                      if( _successCount > 0)
                                        widget.onChange();
                                      Navigator.pop(this.context, true);
                                    }
                                  },
                                  label: Text(
                                    'Close',
                                    style: TextStyle(
                                      fontSize: 16,
                                      fontWeight: FontWeight.w400,
                                      // color: Colors.white,
                                    ),
                                  ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                ) : Container(),
                              ]
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            )
        )
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool os) async {
    var now =  _recipientDate;
    var initialDate = convertToDate(initialDateString) ?? now;
    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")?
        new DateTime( _dropOffDateLabels[0].year, _dropOffDateLabels[0].month-1, _dropOffDateLabels[0].day )
            : _dropOffDateLabels[0],
        lastDate: new DateTime( _dropOffDateLabels[0].year, _dropOffDateLabels[0].month+1, _dropOffDateLabels[0].day )
    );

    if (result == null) return;
    setState(() {
        _recipientDate = result;
    });
  }
  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }
  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }
  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }
  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }
  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }
  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  Future<bool> _onWillPop() async {
    return true;
  }
  String _dayToLabel(DateTime date){
    String day = new DateFormat('EEEE').format( date.toLocal() ).toLowerCase();
    if( day == 'sunday')
      return tr(LocaleKeys.lblSUN);
    else if( day == 'monday')
      return tr(LocaleKeys.lblMON);
    else if( day == 'tuesday')
      return tr(LocaleKeys.lblTUE);
    else if( day == 'wednesday')
      return tr(LocaleKeys.lblWED);
    else if( day == 'thursday')
      return tr(LocaleKeys.lblTHU);
    else if( day == 'friday')
      return tr(LocaleKeys.lblFRI);
    else if( day == 'saturday')
      return tr(LocaleKeys.lblSAT);
  }

  Future<void> _checkPrice( ) async{
    setState(() {
      _processing = true;
    });
    try{

      PriceMethodService priceMethodService = new PriceMethodService( );
      Map<String, dynamic> data = await priceMethodService.calculatePrice(
          _deliveryItem,
          _addressBookItemRecipient.townshipGuid,
          _pickupTownshipGuid,
          _lang,
          _addressBookItemRecipient.addressType==Constant.ADDRESS_TYPE_POST_OFFICE || _addressBookItemRecipient.addressType==Constant.ADDRESS_TYPE_HIGHWAY || _addressBookItemRecipient.addressType==Constant.ADDRESS_TYPE_POI?_addressBookItemRecipient.guid:null,
          _pickupPOIGuid,
          _senderGuid,
          _deliveryMethods
      );
      if( data['success']){
        Map<String, dynamic> prices = data["data"];
        _deliveryItem.charges = prices['totalPrice'];
        _promoAmount = prices['totalDiscount'];
        _deliveryItem.forShowCharges = _deliveryItem.charges;
        _controller_edit_deliCharges.text = _deliveryItem.charges.toStringAsFixed(0);
        _deliveryCharges = _deliveryItem.charges;
        _allow_cash_on_advanced = data["refundInAdvanced"]??false;
        _controller_edit_charges_by_admin.text = _deliveryItem.charges.toStringAsFixed(0);
      }else{
        _deliveryItem.charges = 0;
        _promoAmount = 0;
        _deliveryItem.forShowCharges = _deliveryItem.charges;
        _controller_edit_deliCharges.text = _deliveryItem.charges.toStringAsFixed(0);
        _controller_edit_charges_by_admin.text = _deliveryItem.charges.toStringAsFixed(0);
        _deliveryCharges = null;
        showMessage(data['data']);
      }
    }catch(e){print(e);}
    setState(() {
      _processing = false;
    });

    if(_authorityResponse.authorities !=null
        && _authorityResponse.authorities.contains("MANAGER_WRITE")){
      // setState(() {
      //   _allow_cash_on_advanced = true;
      // });
    }
  }

  void _submitForm( BuildContext context ) async{

    if( _processing ) return;

    final FormState form = _formKey.currentState;
    if (!form.validate() ) {
      if(!_itemDetailAlreadyCheck){
        setState(() {
          _itemDetailError = true;
        });
      }
      showMessage('Form is not valid! Please revise.');
    } else {
      form.save( );

      if(!_itemDetailAlreadyCheck){
        setState(() {
          _itemDetailError = true;
        });
        showMessage(tr(LocaleKeys.lblRequiredParcelDetailToCheck));
        return;
      }

      if( widget.pickupDate!=null && widget.pickupDate.isAfter(_recipientDate)){
        showMessage(tr(LocaleKeys.lblWrongDate));
        return;
      }

      bool  result = await _showConfirm( context );
      if ( !result ){
        return;
      }
      setState(() {
        _processing = true;
      });

      String recipientGuid;
      if( _addressBookItemRecipient !=null ){
        if(_addressBookItemRecipient.name == _controller_recipientName.text){
          recipientGuid = _addressBookItemRecipient.personGuid;
        }
      }
      DeliveryContactItem customer = new DeliveryContactItem(
          personGuid: recipientGuid,
          name: _controller_recipientName.text,
          date: _recipientDate,
          address: _addressBookItemRecipient.address,
          street: _addressBookItemRecipient.street,
          ward: null,
          township: _addressBookItemRecipient.townshipName,
          townshipGuid: _addressBookItemRecipient.townshipGuid,
          primaryPhone: _controller_recipientPrimaryPhone.text,
          otherPhones: _controller_recipientOtherPhones.text,
          email: null,
          comment: _controller_senderComment.text,
          latitude: _addressBookItemRecipient.latitude,
          longitude: _addressBookItemRecipient.longitude,
          contactType: Constant.CONTACT_TYPE_DROP_OFF,
          guid: _addressBookItemRecipient.guid,
          addressType: _addressBookItemRecipient.addressType
      );

      DeliveryContactItem assignee;
      if( !_autoAssignDeliveryman && _employeeSearchItem!=null ){
        assignee = new DeliveryContactItem();
        assignee.personGuid = _employeeSearchItem.guid;
      }

      List<Map<String, dynamic>> items = [_deliveryItem.toMap()];
      Map<String, dynamic> request = {
        'extraPaidBy': 'Customer',
        'labourRequired': false,
        'specialCase': false,
        'branchGuid': null,
        'contact': customer!=null? customer.toMap() :null,
        'item': items[0],
        'refundInAdvanced': _requiredToRefund,
        'refundAmount': null,
        'lang': _lang,
        'promoAmount':_promoAmount,
        'guid': _detailResponse!=null? _detailResponse.guid : null,
        'townshipGuid':_previousTownshipGuid==null?_addressBookItemRecipient.townshipGuid : _previousTownshipGuid,
        'pickupGuid': widget.pickupGuid,
        'assignee': assignee!=null? assignee.toMap() : null,
        'approved': _approved,
        'transitRoutes':null,
        'deliveryMethods': _deliveryMethods,
        'chargesAlreadyClear': _alreadyPaid
      };

      DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
      Map<String, dynamic> response = widget.trackingId!=null?  await _deliveryContentService.update(request ) : await _deliveryContentService.create(request );
      if( response['success'] ){
        showMessage(response["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
        widget.onChange();
      }else{
        showMessage(response["data"]);
        setState(() {
          _processing = false;
        });
      }
    }
  }

  void _quickSave( BuildContext context ) async{
    if( _processing ) return;

    if( _deliveryItem.image ==null){
      setState(() {
        _itemDetailError = true;
      });
      showMessage('Photo must be selected');
      return;
    }
      bool  result = await _showConfirm( context );
      if ( !result ){
        return;
      }

      setState(() {
        _processing = true;
      });

      Map<String, dynamic> request = {
        'extraPaidBy': 'Customer',
        'labourRequired': false,
        'specialCase': false,
        'branchGuid': null,
        'contact': null,
        'lang': _lang,
        // 'townshipGuid':_previousTownshipGuid==null?_addressItemTownship.townshipGuid : _previousTownshipGuid,
        'pickupGuid': widget.pickupGuid,
        'item': _deliveryItem.toMap(),
        'guid':null,
        'approved':false
      };

      DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
      Map<String, dynamic> response = await _deliveryContentService.create(request );
      if( response['success'] ){

        showMessage(response["message"]);
        setState(() {_totalAddedCount = _totalAddedCount +1;});
        if( _totalAddedCount == widget.totalWays){
          _hasNext = false;
        }
        if( _hasNext ){
          _deliveryItem = new DeliveryItem(
            name: 'Parcel', weight: 2, width: 2, height: 2, length: 2,
            priceOption: "Unpaid", price: 0,
            chargeOption: "Unpaid", charges: 0,
            forShowCharges: 0, fragile: false, insurance: false,
            image: null
          );
          _hasNext = false;
        }else{
          Timer(Duration(seconds: 2), () {Navigator.pop(context, true);});
        }
        widget.onChange();
      }else{
        showMessage(response["data"]);
        print(response);
      }

    setState(() {
      _processing = false;
    });
  }

  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( tr(LocaleKeys.lblActionConfirm), style: TextStyle(color: Colors.black),),
          content: new Text(tr(LocaleKeys.lblAreYouSureToDeliverYourParcel)),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnYes)),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnCancel)),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<void> _availableDates( ) async{
    DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
    Map<String, dynamic> resultData = await _deliveryContentService.availableDeliveryDates( );
    if( resultData["success"] ){
      Map<String, dynamic> data = resultData["data"];
      final pickupDates = data["pickupDates"] as List;
      final dropOffDates = data["dropOffDates"] as List;
      _allow_cash_on_advanced = false;//data["refundInAdvanced"]??false;

      for( int i=0; i< pickupDates.length; i++){
        _dropOffDateLabels[i] = dropOffDates[i]!=null? DateTime.parse(dropOffDates[i]) : null;
      }

      setState((){
      _recipientDate = _dropOffDateLabels[0];
      });
    }
  }


  Widget _attachedImage( BuildContext context ){
    return new Column(
      children: <Widget>[
        Container(
          margin: EdgeInsets.only(left: 0.0, right: 0.0, top: 8),
          child: ButtonBar(
              alignment: MainAxisAlignment.center,
              children: <Widget>[
                RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.photo_camera,
                      // color: Colors.white
                  ),
                  color: Theme.of(context).primaryColor,
                  onPressed: () async {
                    await _getImage( true );
                  },
                  label: Text(
                    tr(LocaleKeys.btnCamera),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      // color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
                RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.photo_library,
                      // color: Colors.white
                  ),
                  color: Theme.of(context).primaryColor,
                  onPressed: () async { await _getImage( false);},
                  label: Text(
                    tr(LocaleKeys.btnBrowsePhoto),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      // color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
                _deliveryItem.image==null ? Container() : RaisedButton.icon(
                  elevation: 0.0,
                  icon: Icon(Icons.delete_forever,
                      // color: Colors.white
                  ),
                  color: Colors.red[600],
                  onPressed: () async {
                    setState(() {
                      _deliveryItem.image = null;
                    });
                  },
                  label: Text(
                    tr(LocaleKeys.btnRemove),
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w400,
                      // color: Colors.white,
                    ),
                  ),
                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                ),
              ]
          ),
        ),
      ],
    );
  }

  List<dynamic> _images = [];
  Future<void> _getImage( bool isCamera) async {
    _formKey.currentState.save();
    if( _processing ) return;

    try{
      final picker = ImagePicker();
      if( isCamera ) {
        final image = await picker.getImage(
            source: isCamera ? ImageSource.camera : ImageSource.gallery, maxWidth: 1080, imageQuality: 72);
        if( image !=null )
          _images.add( image );
      }else{
        if( Platform.isIOS ){
          final image = await picker.getImage(
              source: isCamera ? ImageSource.camera : ImageSource.gallery, maxWidth: 1080, imageQuality: 72);
          if( image !=null )
            _images.add( image );
        }else {
          List<XFile> images = await picker.pickMultiImage(
              maxWidth: 1080, imageQuality: 72);
          if( images !=null ){
            _images.addAll(images);
          }
        }
      }
      setState(() { });

      // if( image != null ) {
      //   final Directory extDir = await getApplicationDocumentsDirectory();
      //   final String tmpDir = '${extDir.path}/gelivery_tmp';
      //   await Directory(tmpDir).create(recursive: true);
      //
      //   String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
      //   File rotatedFile = await FlutterImageCompress.compressAndGetFile(
      //     image.path,
      //     photoPath2,
      //     quality: 72,
      //     rotate: 0,
      //   );
      //   imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
      //   String cropFilePath = '$tmpDir/cropped.jpg';
      //   File(cropFilePath)
      //     ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));
      //
      //   File smallFile  =File(cropFilePath);
      //   String encodedString = encode( smallFile );
      //   smallFile.deleteSync(recursive: false);
      //   rotatedFile.deleteSync(recursive: false);
      //   setState(() {
      //     _deliveryItem.image = encodedString;
      //   });
      //
      // }
    }catch(e){}

    // var image = await ImagePicker.pickImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
    //
    // if( image != null ){
    //   var result = await FlutterImageCompress.compressWithList(
    //     image.readAsBytesSync(),
    //     minWidth: 800,
    //     quality: 80,
    //     rotate: 0,
    //   );
    //
    //   final Directory extDir = await getApplicationDocumentsDirectory();
    //   final String dirPath = '${extDir.path}/Pictures/quick_tmp';
    //   await Directory(dirPath).create(recursive: true);
    //   String cropFilePath = '$dirPath/cropped.jpg';
    //   File(cropFilePath)
    //     ..writeAsBytesSync(imglib.encodeJpg(imglib.decodeImage( result ), quality: 80));
    //   final smallFile = File(cropFilePath);
    //   String encodedString = encode( smallFile );
    //   smallFile.deleteSync(recursive: false);
    //   setState(() {
    //     _deliveryItem.image = encodedString;
    //   });
    // }
  }

  void _uploadMultipleImages( BuildContext context, int index ) async{
    if( _processing ) return;

    if( _images.isEmpty){
      setState(() {_itemDetailError = true;});
      showMessage('Photo(s) must be selected');
      return;
    }
    if( context !=null && index==0 ) {
      bool result = await _showConfirm(context);
      if (!result) {
        return;
      }
    }
    setState(() {_processing = true;});

    UserService _userService = UserService();
    String accessToken = await _userService.getToken();


    // _images.forEach((image) async{

      final task = _tasks["Photo $index"];
      if ( task ==null || (task !=null && task.status !=UploadTaskStatus.complete) ) {
        final image = _images[index];
        final tag = "Photo $index";
        if( Constant.ENABLE_LOGS)
          print( "Tag name = Photo $index" );
        // final taskId = await uploader.enqueue(
        //     url: '${getBaseURL(
        //         widget.sharedPrefs)}/v3/deliver/create_with_image/${widget
        //         .pickupGuid}',
        //     files: [
        //       FileItem(
        //           filename: filename, savedDir: savedDir, fieldname: "file")
        //     ],
        //     method: UploadMethod.POST,
        //     headers: {"Authorization": "Bearer $accessToken"},
        //     // data: {"name": "john"},
        //     showNotification: false,
        //     tag: tag);

        setState(() {
          _tasks.putIfAbsent(
              tag,
                  () =>
                  UploadItem(
                      id: '${index + 1}',
                      tag: tag,
                      type: 'Image',
                      status: UploadTaskStatus.enqueued,
                      filePath: image.path
                  ));
        });

        try{
          final smallFile = File(image.path);
          String response = await FileService.fileUploadMultipart('${getBaseURL(
              widget.sharedPrefs)}/v3/deliver/create_with_image/${widget
              .pickupGuid}',
              accessToken,
              file: smallFile,
              onUploadProgress: (int sentBytes, int totalBytes){
                double __progressValue = Util.remap(sentBytes.toDouble(), 0, totalBytes.toDouble(), 0, 1);
                __progressValue = double.parse(__progressValue.toStringAsFixed(2));
                final task = _tasks[tag];
                if( Constant.ENABLE_LOGS)
                  print("progress: $__progressValue , tag: $tag");
                if (task == null) return;
                if (task.isCompleted()) return;
                UploadTaskStatus status = __progressValue< 1? UploadTaskStatus.running : UploadTaskStatus.complete;
                setState(() {
                  _tasks[tag] = task.copyWith(progress: (__progressValue * 100).toInt(), status: status);
                  _processing = true;
                });
              }
          );

          Map<String, dynamic> resultData = jsonDecode(response);
          if( resultData['success']){
            final task = _tasks[tag];
            if (task == null) return;

            _tasks[tag] = task.copyWith(status: UploadTaskStatus.complete);
            ++_successCount;

            if( _successCount < _images.length)
              _scrollController.animateTo( ( (_successCount-1) *30 ).toDouble(), duration: Duration(milliseconds: 100), curve: Curves.ease);

              setState(() {
                ++_totalAddedCount;
                _processing = false;
                if( _successCount == _images.length)
                  _scrollController.animateTo(
                    _scrollController.position.maxScrollExtent,
                    duration: const Duration(milliseconds: 500), curve: Curves.ease,);
              });

            if( _successCount < _images.length){
              _uploadMultipleImages( null, _successCount );
            }
            try{
              smallFile.deleteSync(recursive: false);
            }catch(e){}
          }else{
            final task = _tasks[tag];
            if (task == null) return;
            setState(() {
              _tasks[tag] = task.copyWith(status: UploadTaskStatus.failed);
              _processing = false;
            });
          }

        }catch(e){
          final task = _tasks[tag];
          if (task == null) return;
          setState(() {
            _tasks[tag] = task.copyWith(status: UploadTaskStatus.failed);
            _processing = false;
          });
        }
      } else{
        if( index < _images.length -1 ){
          setState(() {_processing = false;});
          _uploadMultipleImages( context, ++index );
        }
      }
    // });
  }

  Widget _imageStatus( BuildContext context, int index ){

    final task = _tasks["Photo $index"];
    if( task == null ) {
      return Align(
        alignment: Alignment.bottomRight,
        child: TextButton(
          style: TextButton.styleFrom(
            backgroundColor: Theme
                .of(context)
                .primaryColor,
            shape: CircleBorder(),
          ),
          child: Icon(Icons.delete_forever_outlined, color: Colors.white),
          onPressed: () {
            setState(() {
              _images.removeAt(index);
            });
          },
        ),
      );

    }else{
      if( task.status == UploadTaskStatus.enqueued){
        return Align(
          alignment: Alignment.bottomRight,
          child: Material(
            elevation: 12,
            color: Colors.grey[500],
            child: Icon(Icons.hourglass_top_sharp, color: Colors.white, size: 32,),
          )
        );
      } else if( task.status == UploadTaskStatus.complete){
        return Align(
          alignment: Alignment.bottomRight,
          child: Material(
            elevation: 12,
            color: Colors.green[700],
            child: Icon(Icons.done, color: Colors.white, size: 32,),
          )
        );
      } else if( task.status == UploadTaskStatus.running){
        return Align(
          alignment: Alignment.bottomRight,
          child: CircularProgressIndicator(
            value: task.progress /100,
            semanticsLabel: 'Upload progress',
            backgroundColor: Colors.white,
          ),
        );
      } else {
        return Align(
          alignment: Alignment.bottomRight,
          child: Material(
            elevation: 12,
            color: Colors.yellow[800],
            child: Icon(Icons.warning_amber_sharp, color: Colors.white, size: 32,),
          )
        );
      }
    }
  }

  TownshipService _townshipService = new TownshipService();
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;
  final TextEditingController _controllerTownship = new TextEditingController();
  Future<void> _loadTownships() async{
    setState(() {_processing = true;});
    _addressResponseTownship = await _townshipService.list( );

    setState(() {_processing = false;});
  }
  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type township name',
              labelText: tr(LocaleKeys.lblTownship),
            ),
            controller: _controllerTownship,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemTownship = suggestion;
          },
          validator: (val) => val.isEmpty ? 'Township (or) Town required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Container _deliverySelection( BuildContext context){
    return
      _authorityResponse.authorities!=null
          && (_authorityResponse.authorities.contains("MANAGER_READ") || _authorityResponse.authorities.contains("OPERATOR_READ"))?
      Container(
          margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 32),
          child: DropdownSearch<DeliveryForDeliveryItem>(
            selectedItem: _employeeSearchItem,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (DeliveryForDeliveryItem i, DeliveryForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _employeeSuggestion(filter),
            onChanged: (DeliveryForDeliveryItem data) {
              _employeeSearchItem = data;
            },
            dropdownBuilder: (BuildContext context, DeliveryForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: item.avatar !=null? Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ) : Icon(Icons.account_circle),
                  ),
                ),
            popupItemBuilder: (BuildContext context, DeliveryForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    // color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text('${item.personId}'),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: item.avatar !=null? Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ) : Icon(Icons.account_circle),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }

  Future<List<DeliveryForDeliveryItem>> _employeeSuggestion(String query) async{
    List<DeliveryForDeliveryItem> matches = List();
    if( _deliverymanResponse.items == null)
      return matches;
    matches.addAll(_deliverymanResponse.items);

    matches.retainWhere((s) {
      DeliveryForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( employeeSearchItem.personId.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }
}
class UploadItem {
  final String id;
  final String tag;
  final String type;
  final int progress;
  final UploadTaskStatus status;
  final String filePath;

  UploadItem({
    this.id,
    this.tag,
    this.type,
    this.progress = 0,
    this.status = UploadTaskStatus.undefined,
    this.filePath
  });

  UploadItem copyWith({UploadTaskStatus status, int progress}) => UploadItem(
      id: this.id,
      tag: this.tag,
      type: this.type,
      status: status ?? this.status,
      progress: progress ?? this.progress);

  bool isCompleted() =>
      this.status == UploadTaskStatus.canceled ||
          this.status == UploadTaskStatus.complete ||
          this.status == UploadTaskStatus.failed;
}

class Util {
  static double remap(double value, double originalMinValue, double originalMaxValue,
      double translatedMinValue, double translatedMaxValue) {
    if (originalMaxValue - originalMinValue == 0) return 0;

    return (value - originalMinValue) /
        (originalMaxValue - originalMinValue) *
        (translatedMaxValue - translatedMinValue) +
        translatedMinValue;
  }
}