import 'package:badges/badges.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/view/delivery/going_out_deliver_list.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'going_out_pickup_list.dart';


class DeliverymanGoingOut extends StatefulWidget {
  DeliverymanGoingOut({Key key, this.selectedIndex, this.title, this.township}) : super(key: key);
  final String township;
  final int selectedIndex;
  final String title;
  @override
  _DeliverymanGoingOutState createState() => _DeliverymanGoingOutState();
}

class _DeliverymanGoingOutState extends State<DeliverymanGoingOut> {

  User _currentUser = new User();
  UserService _userService = new UserService();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  String _employeeGuid;

  String _accessToken;
  bool _tapChange = false;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblQueue);
  int _pendingLeaveCount = 0;
  bool _processing = false;
  bool _isAllTowns = false;
  String _keyword = "";
  int _pickupWaysCount = 0;
  int _deliverWaysCount = 0;

  @override
  void initState() {
    super.initState();
    _keyword = widget.township;
    _loadData();
  }

  _loadData() async{
        _selectedIndex = widget.selectedIndex;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
        _accessToken = await _userService.getToken();
        _employeeGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : null;
        setState(() {_title = widget.title;});
        await _loadCount();
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom: _createSearchIndicator(),
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Badge(
                    position: BadgePosition.topEnd(top: -16, end: -16),
                    badgeContent: Text('$_pickupWaysCount',style: TextStyle(color: Colors.white),),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: Icon(Icons.download_sharp),
                ),
                label: tr(LocaleKeys.lblToPickingUp),
              ),
              BottomNavigationBarItem(
                icon: Badge(
                  position: BadgePosition.topEnd(top: -16, end: -16),
                  badgeContent: Text('$_deliverWaysCount',style: TextStyle(color: Colors.white),),
                  toAnimate: true,
                  animationType: BadgeAnimationType.slide,
                  child: Icon(Icons.upload_sharp),
                ),
                label: tr(LocaleKeys.lblToDropOff),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            // unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Theme.of(context).primaryColor,
            onTap: _onItemTapped,
          ),
        )
    );
  }
  Widget _createBody(){

    if( _selectedIndex == 0 && !_tapChange){
      return GoingOutPickupList(
        keyword: _keyword,
        employeeGuid: _employeeGuid,
      );
    }else if( _selectedIndex == 1 && !_tapChange){
      return GoingOutDeliverList(
        keyword: _keyword,
        employeeGuid: _employeeGuid,
      );
    }else{
      return new Container();
    }
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = tr(LocaleKeys.lblToPickingUp);
      }else if( _selectedIndex == 1){
        _title = tr(LocaleKeys.lblToDropOff);
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 60.0),
      child: SizedBox(
          height: 60.0,
          child: Container(
              child:SwitchListTile(
              value: _isAllTowns,
              onChanged: (value) async{
                setState(() {
                  _isAllTowns = value;
                  _keyword = _isAllTowns ?"": widget.township;
                  _tapChange = true;
                });
                new ThreadTimer(new Duration(milliseconds: 500), () {
                  setState(() {
                    _tapChange = false;
                  });
                });
                await _loadCount();
              },
              title: Text(tr(LocaleKeys.lblHomeAllLocations)),
              // secondary: Icon(Icons.playlist_add_check_sharp),
              selected: _isAllTowns,
              activeTrackColor: Colors.white,
              activeColor: Colors.white,
              inactiveTrackColor: Colors.grey[600],
              inactiveThumbColor: Colors.grey[500],
            ),
            // color: Colors.white,
          ),
      )
  );

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  Future<void> _loadCount() async{
    try{
      PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
      DeliveryListRequest _listRequest = new DeliveryListRequest(
          keyword: _keyword,
          employeeGuid: _employeeGuid,
          status: [Constant.STATUS_PICKUP_ON_WAY],
          first: 0,
          max: 1,
          orderBy: 'os.name_mm'
      );
      DeliveryListResponse _deliveryListResponse = await _pickupServiceV2.list(_accessToken, _listRequest);
      if( _deliveryListResponse !=null ){
        setState(() {
          _pickupWaysCount = _deliveryListResponse.totalRecords;
        });
      }

      DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
      _listRequest.status = [Constant.STATUS_ON_WAY];
      _listRequest.orderBy = 'customer.name_mm';
      _deliveryListResponse = await _deliveryContentService.list(_accessToken, _listRequest);
      if( _deliveryListResponse !=null ){
        setState(() {
          _deliverWaysCount = _deliveryListResponse.totalRecords;
        });
      }

    }catch(e){}
  }
}
