
import 'package:easy_localization/easy_localization.dart';
import 'package:latlong/latlong.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_detail.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'delivery_list_for_pickup.dart';



class GoingOutPickupList extends StatefulWidget {
  GoingOutPickupList({Key key, this.keyword, this.employeeGuid,}) : super(key: key);
  final String keyword;
  final String employeeGuid;

  @override
  _GoingOutPickupListState createState() => _GoingOutPickupListState();
}

class _GoingOutPickupListState extends State<GoingOutPickupList>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DeliveryListResponse _deliveryListResponse;
  String _accessToken;
  PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
  DeliveryListRequest _listRequest = new DeliveryListRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  bool _isFirst = true;
  UserService userService = UserService();

  @override
  void initState() {
    super.initState();

    _listRequest = new DeliveryListRequest(
        keyword: widget.keyword,
        employeeGuid: widget.employeeGuid,
        status: [Constant.STATUS_PICKUP_ON_WAY],
        first: _page,
        max: Constant.ROW_PER_PAGE,
        orderBy: 'os.name_mm'
    );
  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      _accessToken = await userService.getToken();
      _listRequest.first = _page;
      _listRequest.max = Constant.ROW_PER_PAGE;

      _deliveryListResponse = await _pickupServiceV2.list(_accessToken, _listRequest);
    if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: null,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final DeliveryContentServiceV2 deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  int _selectedStep = 0;
  StepState _pickupStepState = StepState.indexed;
  StepState _dropOffStepState = StepState.indexed;
  bool _dropOffInactive = false;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    _stepState(post.status);

    return Container(
        padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child: new Wrap(
            children: <Widget>[
              ListTile(
                title: //Text('Way ID: ${post.trackingId}', style: TextStyle(color: Colors.white),),
                _createText(post.status, post.totalWays, context),
                subtitle: Text( tr(LocaleKeys.lblDeliCreatePickupDate)+ ' '+ (post.pickupDate!=null? new DateFormat('dd/MM/yyyy').format(post.pickupDate.toLocal()):'../../....') ,
                  // style: TextStyle(color: Colors.blueGrey[600]),
                ),
                onTap: () async {
                  bool result = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => PickupDetail(
                          wayId: post.trackingId,
                          onChange: (){
                            widget.onChange();
                          },
                        ),
                      )
                  );
                  if( result!=null && result ){
                    widget.onChange();
                  }
                },
                trailing: Icon(Icons.arrow_forward_sharp,
                  // color: Colors.blueGrey[600],
                ),
              ),

              Stepper(
                steps: [
                  Step(
                    isActive: true,
                    state: _pickupStepState,
                    title: Text('${post.osName} [ID:${post.osPersonId}]'),
                    subtitle: Text('${tr(LocaleKeys.lblSenderMobile)} ${post.osPrimaryPhone}'),
                    content: GestureDetector(
                      child: Text(
                        '${tr(LocaleKeys.lblDeliCreatePickup)} ${post.osAddress}',
                      ),
                      onTap: () async {
                        makeCall(post.osPrimaryPhone);
                      },
                    ),
                  ),
                ],
                currentStep: _selectedStep,
                // controlsBuilder: (BuildContext context,
                //     {VoidCallback onStepContinue, VoidCallback onStepCancel}) =>
                //     Container(),
                type: StepperType.vertical,
                physics: ClampingScrollPhysics(),
                onStepTapped: (step){setState(() {
                  _selectedStep = step;
                });},
              ),
            ],
          ),
        )
    );
  }

  Text _createText(String status, int totalWays, BuildContext context ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListPickupCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
    || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }


    title =  tr(LocaleKeys.lblToPickingUp);
    title = totalWays.toString() + tr(LocaleKeys.lblItemListTotalWays) + title;
    return Text(
      '$title',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }

  StepState _stepState(String status){
    if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.indexed;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CANCELED ){
      _pickupStepState = StepState.error;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.indexed;
      _dropOffInactive = true;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_COMPLETED){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_RETURN){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;
    }
    return _pickupStepState;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

}