

import 'dart:async';
import 'package:flutter/services.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:location/location.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_completed_request.dart';
import 'package:qq_delivery/delivery/model/request/pickup_create_request.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/comment_list_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/pickup_detail_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/comment_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:qq_delivery/delivery/view/utils/signature_pad.dart';
import 'package:qq_delivery/delivery/view/utils/sunmi_print.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'comment.dart';



class PickupDetail extends StatefulWidget {
  PickupDetail({Key key, this.onChange, this.wayId}) : super(key: key);
  final Function onChange;
  final String wayId;
  @override
  _PickupDetailState createState() => _PickupDetailState();

}
class _PickupDetailState extends State<PickupDetail> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  String _accessToken;

  User _currentUser = new User();
  UserService _userService = new UserService();
  // DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
  PickupDetailResponse _deliveryListResponse;
  PickupCreateRequest _pickupCreateRequest;
  DeliveryContactItem _contactItemOS;
  DeliveryContactItem _contactItemCustomer;
  DeliveryContactItem _contactItemPickupPerson;
  DeliveryContactItem _contactItemDropOffPerson;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  bool _pickupEditable = false;
  bool _dropOffEditable = false;
  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  ActiveDeliverymanResponse _deliverymanResponse;
  DeliveryForDeliveryItem _employeeSearchItemAll;
  PersonService _personService = new PersonService();
  String _employeeGuid;
  bool _transfer = false;
  bool _firstLoad = true;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  bool _requiredToRefund = false;
  double _refundAmount;
  String _signature;
  int _totalComments = 0;

  @override
  void initState() {
    super.initState();
    // _loadData();
  }

  _loadData() async{
    setState(() {
      _processing = true;
    });

    if( _firstLoad ) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      // _firstLoad = false;
    }

    _deliveryListResponse = await _pickupServiceV2.detail(widget.wayId);
    _pickupCreateRequest = _deliveryListResponse.pickupCreateRequest;
    _contactItemOS = _pickupCreateRequest.pickup;
    _contactItemPickupPerson = _pickupCreateRequest.assignee;
    _controller_actualWays.text = _pickupCreateRequest.totalWays.toString();

    if(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains('MANAGER_WRITE') &&
        ( _pickupCreateRequest.status == Constant.STATUS_PICKUP_CREATED
        || _pickupCreateRequest.status == Constant.STATUS_PICKUP_SCHEDULED
        || _pickupCreateRequest.status == Constant.STATUS_RETRY) ){
      _pickupEditable = true;
    }

    if(_authorityResponse.authorities!=null ) {
      _accessToken = await _userService.getToken();
      _employeeSearchResponse =
      await _pickupServiceV2.availableEmployeesByZone(
          _pickupCreateRequest.guid, _accessToken);
    }
    if(_authorityResponse.authorities !=null
        && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))){
      _deliverymanResponse = await _personService.activeDeliveryman();
    }

    CommentService _commentService = new CommentService();
    CommentListResponse _commentListResponse = await _commentService.list(_accessToken, 'pickup', _pickupCreateRequest.guid, 0, 1000000);
    _totalComments = _commentListResponse.totalRecords;
    setState(() {_processing=false;});

    if( _firstLoad ) {
      _firstLoad = false;
      if( _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
          && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _pickupCreateRequest !=null
          && _pickupCreateRequest.status==Constant.STATUS_PICKUP_ON_WAY
          && _contactItemPickupPerson !=null && _contactItemPickupPerson.personGuid==_currentUser.guid){
        _initLocationService( );
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(widget.wayId, style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            /*actions: <Widget>[
              Platform.isAndroid?Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.print, color: Colors.white,
                  ),
                  onPressed: () {
                    if( _authorityResponse.authorities!=null
                        && (_authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                            || _contactItemOS.personGuid==_currentUser.guid)){

                      // if( _detailResponse !=null && _contactItemOS!=null
                      //     && _contactItemCustomer !=null ){
                      //   // Navigator.push(
                      //   //     context,
                      //   //     MaterialPageRoute(
                      //   //       builder: (BuildContext context) => InvoiceDetailPreProcess(
                      //   //         detailResponse: _detailResponse,
                      //   //         contactItemOS: _contactItemOS,
                      //   //         contactItemCustomer: _contactItemCustomer,
                      //   //         currentUser: _currentUser,
                      //   //         refundAmount: _refundAmount,
                      //   //         item: _detailResponse.items[0],
                      //   //       ),
                      //   //     )
                      //   // );
                      //   SunmiPrint sunmiPrint = new SunmiPrint();
                      //   sunmiPrint.printSlip(_detailResponse, _contactItemOS, _contactItemCustomer, _sharedPrefs, context);
                      // }
                    }
                  },
                  tooltip: 'Search',
                ),
              ) : Container(),
            ],*/
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: false,
            child: ListView(
              children: <Widget>[
                _pickupCreateRequest!=null
                    && _pickupCreateRequest.refundInAdvanced
                    && _authorityResponse.authorities !=null
                    // && _authorityResponse.authorities.contains("EMPLOYEE_READ")
                    && (_pickupCreateRequest.status==Constant.STATUS_PICKUP_SCHEDULED || _pickupCreateRequest.status==Constant.STATUS_PICKUP_CREATED || _pickupCreateRequest.status==Constant.STATUS_PICKUP_ON_WAY)?
                _cashAdvance( context ) : Container(),

                _pickupCreateRequest!=null
                    && _pickupCreateRequest.preRefundRemind
                    && _authorityResponse.authorities !=null
                    && _authorityResponse.authorities.contains("EMPLOYEE_READ")
                    && (_pickupCreateRequest.status==Constant.STATUS_PICKUP_SCHEDULED || _pickupCreateRequest.status==Constant.STATUS_PICKUP_CREATED || _pickupCreateRequest.status==Constant.STATUS_PICKUP_ON_WAY)?
                _refundRemind( context ) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_pickupCreateRequest.status==Constant.STATUS_PICKUP_CREATED || _pickupCreateRequest.status==Constant.STATUS_CREATED || _pickupCreateRequest.status==Constant.STATUS_RETRY)?
                  _deliveryAssign( context ) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_pickupCreateRequest.status==Constant.STATUS_PICKUP_SCHEDULED || _pickupCreateRequest.status==Constant.STATUS_ASSIGNED)?
                  _deliveryManagerTransfer( context ) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && (_pickupCreateRequest.status==Constant.STATUS_PICKUP_ON_WAY || _pickupCreateRequest.status==Constant.STATUS_ON_WAY)?
                  _deliveryManagerCancel( context) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _pickupCreateRequest !=null
                    && ( (_pickupCreateRequest.status== Constant.STATUS_ASSIGNED && _contactItemDropOffPerson.personGuid==_currentUser.guid)
                        || (_pickupCreateRequest.status==Constant.STATUS_PICKUP_SCHEDULED && _contactItemPickupPerson.personGuid==_currentUser.guid) )?
                  _deliveryEmployeeAccept(context) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _pickupCreateRequest !=null
                    && _pickupCreateRequest.status== Constant.STATUS_PICKUP_ON_WAY &&  _contactItemPickupPerson.personGuid==_currentUser.guid?
                    _deliveryEmployeePickupComplete(context) : Container(),

                _pickupCreateRequest!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('OS_WRITE')
                    && !_authorityResponse.authorities.contains('MANAGER_WRITE') && _pickupCreateRequest !=null
                    && (_pickupCreateRequest.status== Constant.STATUS_PICKUP_CREATED || _pickupCreateRequest.status== Constant.STATUS_PICKUP_SCHEDULED)
                    && _contactItemOS!=null && _currentUser.guid==_contactItemOS.personGuid?
                    _deliveryManagerCancel( context) : Container(),

                _pickupCreateRequest!=null
                    && _pickupCreateRequest.status != Constant.STATUS_COMPLETED
                    && _pickupCreateRequest.status != Constant.STATUS_CANCELED
                    && _pickupCreateRequest.status != Constant.STATUS_RETURN? _deliveryTrackingMap(context): Container(),
                _deliverySummary( context ),
                _deliveryPickup(context),
                _pickupCreateRequest !=null? _comment(context) : Container(),
              ]
          ),
            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            // scroll down
            onRefresh: () {
              _loadData();
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
          ),
        )
    );
  }

  Widget _cashAdvance( BuildContext context ){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 16),
        child: Card(
            elevation: 1,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblRequiredToRefund)}',
                    // style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('${NumberFormat('#,###').format(_pickupCreateRequest.cashAdvanceAmount)} Ks'
                    // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  leading: Icon(Icons.attach_money_rounded,
                    // color: Colors.blueGrey[600]
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _refundRemind( BuildContext context ){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 16),
        child: Card(
            elevation: 1,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblRefundInformation)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('${_pickupCreateRequest.preRefundMessage}'
                    // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  leading: Icon(Icons.attach_money_rounded,
                    // color: Colors.blueGrey[600]
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryTrackingMap( BuildContext context ){
    LatLng startLocation= Constant.OFFICE_LOCATION;
    LatLng officeLocation= Constant.OFFICE_LOCATION;

    List<Map<String,dynamic>> items=[];
    String mapTitle = "Current way location";
    if( _pickupCreateRequest != null ){
      if( _pickupCreateRequest.status == Constant.STATUS_COMPLETED
      || _pickupCreateRequest.status == Constant.STATUS_CANCELED
      || _pickupCreateRequest.status == Constant.STATUS_RETURN){
        items = null;
        mapTitle = "No tracking available";
      } else if(_pickupCreateRequest.status == Constant.STATUS_PICKUP_CREATED
      || _pickupCreateRequest.status == Constant.STATUS_PICKUP_SCHEDULED ){
        startLocation= officeLocation;
        mapTitle = "Processing to pickup";
        if( _contactItemOS.latitude != null && _contactItemOS.longitude !=null) {
          items.add({
            'latitude': _contactItemOS.latitude,
            'longitude': _contactItemOS.longitude,
            'guid': _contactItemOS.guid,
            'name': _contactItemOS.name,
            'phone': "${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
            'employee': false
          });
        }
        items.add({
          'latitude': officeLocation.latitude,
          'longitude': officeLocation.longitude,
          'guid': 'quickq-delivery',
          'name': _pickupCreateRequest.status == Constant.STATUS_PICKUP_CREATED?'Office' : _contactItemPickupPerson.name,
          'phone': _pickupCreateRequest.status == Constant.STATUS_PICKUP_CREATED?Constant.OFFICE_ADDRESS : _contactItemPickupPerson.primaryPhone,
          'employee': true
        });
      }else if(_pickupCreateRequest.status == Constant.STATUS_PICKUP_ON_WAY ){
        startLocation= officeLocation;
        mapTitle = "Pickup on way";
        if( _contactItemOS.latitude != null && _contactItemOS.longitude !=null) {
          items.add({
            'latitude': _contactItemOS.latitude,
            'longitude': _contactItemOS.longitude,
            'guid': _contactItemOS.guid,
            'name': _contactItemOS.name,
            'phone': "${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
            'employee': false
          });
        }
        if( _contactItemPickupPerson.latitude != null && _contactItemPickupPerson.longitude !=null) {
          startLocation= LatLng(_contactItemPickupPerson.latitude, _contactItemPickupPerson.longitude);
          items.add({
            'latitude': _contactItemPickupPerson.latitude,
            'longitude': _contactItemPickupPerson.longitude,
            'guid': _contactItemPickupPerson.guid,
            'name': _contactItemPickupPerson.name,
            'phone': "${_contactItemPickupPerson.primaryPhone}",
            'employee': true
          });
        }
      } else if(_pickupCreateRequest.status == Constant.STATUS_CREATED
          || _pickupCreateRequest.status == Constant.STATUS_ASSIGNED ){
        startLocation= officeLocation;
        mapTitle = "Processing to deliver";
        if( _contactItemCustomer.latitude != null && _contactItemCustomer.longitude !=null) {
          items.add({
            'latitude': _contactItemCustomer.latitude,
            'longitude': _contactItemCustomer.longitude,
            'guid': _contactItemCustomer.guid,
            'name': _contactItemCustomer.name,
            'phone': "${_contactItemCustomer.address??''}, ${_contactItemCustomer.street??''}, ${_contactItemCustomer.township??''}",
            'employee': false
          });
        }
        items.add({
          'latitude': officeLocation.latitude,
          'longitude': officeLocation.latitude,
          'guid': 'quickq-delivery',
          'name': _pickupCreateRequest.status == Constant.STATUS_CREATED? 'Office' : _contactItemDropOffPerson.name,
          'phone': _pickupCreateRequest.status == Constant.STATUS_CREATED? Constant.OFFICE_ADDRESS : _contactItemDropOffPerson.primaryPhone,
          'employee': true
        });
      } else if(_pickupCreateRequest.status == Constant.STATUS_ON_WAY ){
        startLocation= officeLocation;
        mapTitle = "Deliver on way";
        if( _contactItemCustomer.latitude != null && _contactItemCustomer.longitude !=null) {
          items.add({
            'latitude': _contactItemCustomer.latitude,
            'longitude': _contactItemCustomer.longitude,
            'guid': _contactItemCustomer.guid,
            'name': _contactItemCustomer.name,
            'phone': "${_contactItemCustomer.address??''}, ${_contactItemCustomer.street??''}, ${_contactItemCustomer.township??''}",
            'employee': false
          });
        }
        if( _contactItemDropOffPerson.latitude != null && _contactItemDropOffPerson.longitude !=null) {
          startLocation= LatLng(_contactItemDropOffPerson.latitude, _contactItemDropOffPerson.longitude);
          items.add({
            'latitude': _contactItemDropOffPerson.latitude,
            'longitude': _contactItemDropOffPerson.longitude,
            'guid': _contactItemDropOffPerson.guid,
            'name': _contactItemDropOffPerson.name,
            'phone': "${_contactItemDropOffPerson.primaryPhone}",
            'employee': true
          });
        }
      }
    }
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 16),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${_authorityResponse.authorities!=null
                        && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                        && !_authorityResponse.authorities.contains('MANAGER_READ')?tr(LocaleKeys.lblDeliveryGPSNavigationToCustomer):tr(LocaleKeys.lblLiveTracking)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                  onTap: (){
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => TrackingMap(
                            startLocation: startLocation,
                            items: items,
                            title: mapTitle,
                            isEmployee: false,
                            navigate: _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE'),
                            startRoute: false,
                            contactItemCustomer: _contactItemOS,
                          ),
                        )
                    );
                  },
                ),
                Container(
                  height: 250,
                  width: MediaQuery.of(context).size.width,
                  child: TrackingMap(
                    startLocation: startLocation,
                    items: items,
                    title: null
                  ),
                ),
              ],
            ),
        )
    ) : new Container();
  }

  Widget _deliverySummary( BuildContext context ){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 16),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblSummary)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('Last updated: '+ new DateFormat('dd/MM/yyyy hh:mm a').format(_pickupCreateRequest.updatedOn.toLocal()),
                      // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  trailing: Icon(Icons.local_shipping_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    'Created date: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_pickupCreateRequest.createdOn.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                      '${tr(LocaleKeys.lblWayID)}: ${_pickupCreateRequest.trackingId}',
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //     _pickupCreateRequest.status==Constant.STATUS_COMPLETED? "${tr(LocaleKeys.lblCashCollected)}: ${new NumberFormat("#,###").format(_pickupCreateRequest.totalCharges)} Ks" : "${tr(LocaleKeys.lblCashToCollect)}: ${new NumberFormat("#,###").format(_pickupCreateRequest.totalCharges)} Ks",
                //       style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                //   ),
                // ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //       '${tr(LocaleKeys.lblDeliCharges)}: ${_pickupCreateRequest.deliveryChargeOption=='Unpaid'?tr(LocaleKeys.lblDeliChargeToCollectFromRecipient): tr(LocaleKeys.lblDeliChargeAlreadyPaid)}',
                //       style: TextStyle(color: Colors.black),
                //   ),
                // ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                //   child: Text(
                //       '${tr(LocaleKeys.lblItemPrice)}: ${_pickupCreateRequest.itemPriceOption=='Unpaid'?tr(LocaleKeys.lblToCollectItemPriceFromRecipient):tr(LocaleKeys.lblItemPriceAlreadyPaidByRecipient)}',
                //       style: TextStyle(color: Colors.black),
                //   ),
                // ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 8.0, right: 8.0, bottom: 16),
                  child: Text(
                      '${tr(LocaleKeys.lblStatus)} ${_createStatus(_pickupCreateRequest.status)}',
                      // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                //   child: Text(
                //       "Message: ${_pickupCreateRequest.message==null?'':_pickupCreateRequest.message}",
                //       style: TextStyle(color: Theme.of(context).primaryColorDark, fontWeight: FontWeight.w500, fontSize: 18),
                //   ),
                // ),
                // _authorityResponse.authorities!=null
                //     && (_authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                //     || _contactItemOS.personGuid==_currentUser.guid)?
                // Container(
                //   padding: const EdgeInsets.only(bottom: 16),
                //   child: ListTile(
                //     title: Text(tr(LocaleKeys.lblPromotion),style: Theme.of(context).textTheme.headline6),
                //     subtitle: Text('${new NumberFormat("#,###").format(_detailResponse.promotion)} Ks'),
                //     leading: Icon(Icons.redeem_sharp, color: Theme.of(context).primaryColor,),
                //   ),
                // ) : Container(),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryPickup( BuildContext context ){
    return _pickupCreateRequest!=null && _contactItemOS!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  leading: _contactItemPickupPerson!=null && _contactItemPickupPerson.avatar!=null?ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_contactItemPickupPerson.avatar}', width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ): Icon(Icons.person,
                      // color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    '${tr(LocaleKeys.lblPickupInformation)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(
                      !_pickupEditable? "${_contactItemPickupPerson!=null? 'Pickup by: ' + _contactItemPickupPerson.name+ (_contactItemPickupPerson.primaryPhone!=null? '('+ _contactItemPickupPerson.primaryPhone +')' :'') : ''}"
                      :  "${_contactItemPickupPerson!=null? _contactItemPickupPerson.name+ (_contactItemPickupPerson.primaryPhone!=null? '('+ _contactItemPickupPerson.primaryPhone +')' :'')+' will pickup soon' : ''}",
                      // style: TextStyle(color: Colors.black)
                  ),
                  trailing: _pickupEditable? Icon(Icons.edit,
                      // color: Colors.black
                  ) : Icon(Icons.done,
                      // color: Colors.blueGrey[600]
                  ),
                  onTap: !_pickupEditable? null : () async{
                    // bool result = await Navigator.push(
                    //     context,
                    //     MaterialPageRoute(
                    //       builder: (BuildContext context) => DeliveryContactUpdate(
                    //         contactItem: _contactItemOS,
                    //         deliveryContentService: _deliveryContentService,
                    //         accessToken: _accessToken,
                    //         onChange: (){
                    //           widget.onChange();
                    //           setState(() {});
                    //         },
                    //       ),
                    //     )
                    // );
                  },
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    _pickupCreateRequest.pickupOn!=null? '${tr(LocaleKeys.lblDeliCreatePickupDate)}: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_pickupCreateRequest.pickupOn.toLocal())}'
                        :'${tr(LocaleKeys.lblDeliCreatePickupDate)}: ${new DateFormat('dd/MM/yyyy').format(_contactItemOS.date.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${_contactItemOS.name}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderMobile)}: ${_contactItemOS.primaryPhone}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderOtherPhones)}: ${_contactItemOS.otherPhones}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${_contactItemOS.address??''}, ${_contactItemOS.street??''}, ${_contactItemOS.township??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                _contactItemOS.comment!=null && _contactItemOS.comment.isNotEmpty?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0),
                  child: Text(
                    "${tr(LocaleKeys.lblRemark)}: ${_contactItemOS.comment??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ) : Container(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0, bottom: 16),
                  child: Text(
                    _contactItemOS.email!=null? '${tr(LocaleKeys.lblEmailAddress)}: ${_contactItemOS.email}' : '${tr(LocaleKeys.lblEmailAddress)}:',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                // Container(
                //   padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 16),
                //   child: Text(
                //     _contactItemOS.comment!=null? '${tr(LocaleKeys.lblRemark)}: ${_contactItemOS.comment}' : '${tr(LocaleKeys.lblRemark)}:',
                //     style: TextStyle(color: Colors.black),
                //   ),
                // ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _comment( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child:
          ListTile(
            title: Text(
              _totalComments==0?'Write a comment':'View $_totalComments more comments',
              style: Theme.of(context).textTheme.headline6,
            ),
            trailing: Icon(Icons.comment_bank,
                // color: Colors.blueGrey[600]
            ),
            onTap: (){
              Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) => Comment(
                      personGuid: _currentUser.guid,
                      type: 'pickup',
                      referenceGuid: _pickupCreateRequest.guid,
                      sharedPrefs: _sharedPrefs,
                      accessToken: _accessToken,
                    )
                  )
              );
            },
          ),
        )
    );
  }

  Widget _deliveryAssign( BuildContext context){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
              key: _formKey,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              child: new Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblAssignToEmployee)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text("Assign to your deliveryman to ${_pickupCreateRequest.status==Constant.STATUS_PICKUP_CREATED?'pickup':'drop off'}",
                        // style: TextStyle(color: Colors.blueGrey[600])
                    ),
                    trailing: Icon(Icons.assignment_ind,
                        // color: Colors.blueGrey[600]
                    ),
                  ),
                  _deliverySelection(context),
                  _remarkTextField(),
                  ButtonBar(
                    children: <Widget>[
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.person_pin_circle_sharp,
                            // color: Colors.white
                        ),
                        color: Colors.green[700],
                        onPressed: _employeeGuid==null? null : ()async{
                          await _assign();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblAcceptAndAssign)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.cancel_schedule_send_sharp, color: Colors.white),
                        color: Colors.red[700],
                        onPressed: _processing? null : ()async{
                          await _cancel();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblCancelDelivery)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ],
                  )
                ],
              )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerTransfer( BuildContext context){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblChangeEmployee)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Change your current assigned deliveryman with another",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.wifi_protected_setup_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _deliverySelection(context),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp, color: Colors.white),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerCancel( BuildContext context){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblCancelTheDelivery)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Cancel here if there have any issue",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.cancel_schedule_send_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_carNumber = new TextEditingController();
  Widget _deliveryEmployeeAccept( BuildContext context){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        _pickupCreateRequest!=null && _pickupCreateRequest.status==Constant.STATUS_PICKUP_SCHEDULED?'${tr(LocaleKeys.lblAcceptToPickup)}':'${tr(LocaleKeys.lblAcceptToDeliver)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("${tr(LocaleKeys.lblYouCanAcceptOrTransferToAnother)}",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.assignment_ind,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    CheckboxListTile(
                        value: _transfer,
                        onChanged: ( value) {setState(() { _transfer = value; });},
                      title: Text('${tr(LocaleKeys.lblTransferToAnother)}',
                        // style: TextStyle(color: Colors.black),
                      ),
                    ),
                    _transfer?_employeeSelection(context) : Container(),
                    // !_transfer?Container(
                    //   padding: EdgeInsets.all(16),
                    //   child: new TextFormField(
                    //     decoration: InputDecoration(
                    //       hintText: '၃ဆ/၁၂၃၄',
                    //       labelText: '${tr(LocaleKeys.lblCarNumberToPickupDeliver)}',
                    //     ),
                    //     validator: (val) => val.isEmpty ? 'Car number required!' : null,
                    //     controller: _controller_carNumber,
                    //     keyboardType: TextInputType.text,
                    //   ),
                    // ) : Container(),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        _transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ) : Container(),
                        !_transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _processing? null : ()async{
                            await _accept();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.btnOK)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ): Container(),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _reject();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnCancel)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_actualWays = new TextEditingController();
  Widget _deliveryEmployeePickupComplete( BuildContext context){
    return _pickupCreateRequest!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblPickupComplete)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Complete your pickup process or try again later",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.done,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _refundAmount!=null && _refundAmount>0 ? Container(
                      padding: EdgeInsets.only(top:8, bottom: 0, left:8, right: 8),
                      child: CheckboxListTile(
                        value: _requiredToRefund,
                        onChanged: ( value) {setState(() { _requiredToRefund = value; });},
                        title: Text(
                          '${tr(LocaleKeys.lblRequiredToRefund)}',
                          // style: TextStyle(color: Colors.black),
                        ),
                      ),
                    ) : Container(),
                    _requiredToRefund? ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblCashToRefund)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text('${new NumberFormat("#,###").format(_refundAmount)} Ks',
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      leading: Icon(Icons.monetization_on_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ) : Container(),
                      Container(
                        padding: EdgeInsets.all(16),
                        child: new TextFormField(
                          decoration: InputDecoration(
                            hintText: '',
                            labelText: '${tr(LocaleKeys.lblTotalWay)}',
                          ),
                          validator: (val) => val.isEmpty ? 'Total ways required!' : null,
                          controller: _controller_actualWays,
                          keyboardType: TextInputType.number,
                        ),
                      ),
                    _remarkTextField(),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 16.0, right: 16.0, top: 8),
                      width: double.infinity,
                      child: RaisedButton(
                        onPressed: () async{
                          String result = await Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => SignaturePad(
                                  title: tr(LocaleKeys.lblSenderSignature),
                                ),
                              )
                          );
                          if( result !=null && result.length>10)
                            _signature = result;
                        },
                        color: Theme.of(context).primaryColorDark,
                        elevation: 1,
                        child: Text(
                          tr(LocaleKeys.lblSenderSignature),
                          style: TextStyle(color: Colors.white),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Theme.of(context).primaryColor,
                          onPressed: _processing==null? null : ()async{
                            await _pickupCompleted();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.btnPickupCompleted)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.refresh_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _retry();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnPickupTryAgainLater)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_remark = new TextEditingController();
  Widget _remarkTextField(){
    return Container(
      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: new TextFormField(
        decoration: InputDecoration(
          hintText: '',
          labelText: '${tr(LocaleKeys.lblRemark)}*',
        ),
        validator: (val) => val.isEmpty ? 'Remark required' : null,
        controller: _controller_remark,
        keyboardType: TextInputType.text,
        // maxLines: null,
        // minLines: 1,
      ),
    );
  }

  Future<void> _assign() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAssignRequest request = new DeliveryAssignRequest(
      contentGuid: _pickupCreateRequest.guid,
      personGuid: _employeeGuid,
      comment: _controller_remark.text,
      type: _pickupCreateRequest.status == Constant.STATUS_RETRY? 'Re-assign' : 'Initial',
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.assign(_accessToken, request, "assign");
    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }
    setState(() {
      _processing = false;
    });
  }

  Future<void> _accept() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
      contentGuid: _pickupCreateRequest.guid,
      personGuid: null,
      comment: _controller_remark.text,
      carNumber: _controller_carNumber.text
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "accept");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _reject() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _pickupCreateRequest.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "reject");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _cancel() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _pickupCreateRequest.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "cancel");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _doTransfer() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _pickupCreateRequest.guid,
        personGuid: _employeeGuid,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "transfer");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _retry() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _pickupCreateRequest.guid,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "retry");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _pickupCompleted() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _pickupCreateRequest.guid,
        personGuid: null,
        comment: _controller_remark.text,
        refundInAdvanced: _requiredToRefund,
        refundAmount: _requiredToRefund? _refundAmount : null,
        senderSign: _signature,
        actualWays: int.parse(_controller_actualWays.text),
        latitude: _startLocation !=null? _startLocation.latitude : null,
        longitude: _startLocation !=null? _startLocation.longitude : null,
    );

    Map<String, dynamic> resultData = await _pickupServiceV2.process(_accessToken, request, "pickup_completed");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 16, left: 16, right: 16),
        child: DropdownSearch<EmployeeForDeliveryItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeForDeliveryItem data) {
            setState(() {
              _employeeSearchItem = data;
              if( _employeeSearchItem != null){
                _employeeGuid = _employeeSearchItem.guid;
              }else{
                _employeeGuid = null;
              }
            });
          },
          dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle,
                      // color: Colors.black
                  ),
                  title: Text(tr(LocaleKeys.lblSelectEmployees),
                    // style: TextStyle(color: Colors.black),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name,
                  // style: TextStyle(color: Colors.black),
                ),

                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                  // leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  // color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.zoneName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }

  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = [];
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _deliverySelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && (_authorityResponse.authorities.contains("MANAGER_READ") || _authorityResponse.authorities.contains("OPERATOR_READ"))?
      Container(
          margin: EdgeInsets.only(top: 16, left: 16, right: 16, bottom: 16),
          child: DropdownSearch<DeliveryForDeliveryItem>(
            selectedItem: _employeeSearchItemAll,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (DeliveryForDeliveryItem i, DeliveryForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _deliverymanSuggestion(filter),
            onChanged: (DeliveryForDeliveryItem data) {
              _employeeSearchItemAll = data;
              if( _employeeSearchItemAll != null){
                _employeeGuid = _employeeSearchItemAll.guid;
              }else{
                _employeeGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, DeliveryForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ),
                  ),
                ),
            popupItemBuilder: (BuildContext context, DeliveryForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text(item.personId),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }

  Future<List<DeliveryForDeliveryItem>> _deliverymanSuggestion(String query) async{
    List<DeliveryForDeliveryItem> matches = [];
    if( _deliverymanResponse.items == null)
      return matches;
    matches.addAll(_deliverymanResponse.items);

    matches.retainWhere((s) {
      DeliveryForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else if( employeeSearchItem.personId.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else
        return false;
    }
    );
    return matches;
  }

  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm", style: TextStyle(color: Colors.black),),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

  String _createStatus(String status ){
    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
    }
    return title;
  }

  void showMessage(String message, [MaterialColor color = Colors.green]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(content: new Text(message)));
  }


  Location _locationService  = new Location();
  bool _permission = false;
  LocationData _currentLocation;
  LocationData _startLocation;
  String error;
  _initLocationService() async {
    LocationData location;
    try {

      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
            // print('Lat: ${_startLocation.latitude}, Lon:${_startLocation.longitude}');
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }
    _startLocation = location;
  }
}

class InvoiceDetailPreProcess extends StatefulWidget {

  InvoiceDetailPreProcess({Key key, @required this.detailResponse,
    @required this.contactItemOS, @required this.contactItemCustomer,
    @required this.currentUser, @required this.item,
  @required this.refundAmount}) : super(key: key);
  final DeliveryContactItem contactItemOS;
  final DeliveryContactItem contactItemCustomer;
  final DeliveryDetailResponse detailResponse;
  final DeliveryItem item;
  final double refundAmount;
  // final DeliveryListItem post;
  final User currentUser;

  @override
  _InvoiceDetailPreProcessState createState() => _InvoiceDetailPreProcessState();
}

class _InvoiceDetailPreProcessState extends State<InvoiceDetailPreProcess> {

  @override
  Widget build( BuildContext context ) {
    return Scaffold(
      appBar: AppBar(
        title: Text(widget.detailResponse.trackingId, style: TextStyle(color: Colors.white)),
      ),
      body: SingleChildScrollView(
        child: _deliverySummary( context ),
      ),
    );
  }

  Widget _deliverySummary( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblInvoiceDetail)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text('Invoice date: '+ new DateFormat('dd/MM/yyyy').format(widget.detailResponse.updatedOn.toLocal()),
                        style: TextStyle(color: Colors.blueGrey[600])
                    ),
                    // trailing: IconButton(
                    //   icon: SvgPicture.asset( 'assets/images/pdf.svg', semanticsLabel: 'PDF',height: 24, width: 23, ),
                    //   onPressed: (){},
                    // )
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    'Order date: ${new DateFormat('dd/MM/yyyy').format(widget.detailResponse.createdOn.toLocal())}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    'Delivery date: ${new DateFormat('dd/MM/yyyy').format(widget.contactItemCustomer.date.toLocal())}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblFrom)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${widget.contactItemOS.name}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${widget.contactItemOS.address??''}, ${widget.contactItemOS.street??''}, ${widget.contactItemOS.township??''}",
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderMobile)}: ${widget.contactItemOS.primaryPhone}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),

                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblTo)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${widget.contactItemCustomer.name}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${widget.contactItemCustomer.address??''}, ${widget.contactItemCustomer.street??''}, ${widget.contactItemCustomer.township??''}",
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientMobile)}: ${widget.contactItemCustomer.primaryPhone}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),

                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblChargesDetails)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblDeliCharges)}: ${new NumberFormat("#,###").format(widget.item.charges??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblLabourCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.labourCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblTransportationCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.transportCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblExtraCost)}: ${new NumberFormat("#,###").format(widget.detailResponse.extraCost??0)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSubTotal)}: ${new NumberFormat("#,###").format(widget.item.charges)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblPromotion)}: -${new NumberFormat("#,###").format(widget.detailResponse.promotion)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblTotal)}: ${new NumberFormat("#,###").format(widget.item.charges - widget.detailResponse.promotion)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblCashToCollect)}: ${new NumberFormat("#,###").format(widget.detailResponse.totalCharges)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Divider(),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblCashToRefund)}: ${new NumberFormat("#,###").format(widget.refundAmount)} Ks',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: Text(
                    '${tr(LocaleKeys.lblRefundDate)}: ${widget.detailResponse.refundDate!=null? new DateFormat('dd/MM/yyyy hh:mm a').format(widget.detailResponse.refundDate.toLocal()): "Not yet!"}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    );
  }

  // double _deliveryCharges( DeliveryListItem post ){
  //   double deliCharge = post.totalDeliveryCharges!=null? post.totalDeliveryCharges : 0;
  //   double promo = post.promotionAmount!=null? post.promotionAmount : 0;
  //
  //   return (deliCharge - promo);
  // }
}