import 'package:badges/badges.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/request/person_list_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/person_list_response.dart';
import 'package:qq_delivery/delivery/model/response/zone_list_response.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/zone_service.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_list_main.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_search_filter.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';


class PickupListHome extends StatefulWidget {
  PickupListHome({Key key, this.onChange, this.selectedIndex, this.title}) : super(key: key);
  final Function onChange;
  final int selectedIndex;
  final String title;
  @override
  _PickupListHomeState createState() => _PickupListHomeState();
}

class _PickupListHomeState extends State<PickupListHome> {

  User _currentUser = new User();
  UserService _userService = new UserService();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  String _employeeGuid;
  String _oSGuid;
  String _customerGuid;
  String _keyword = '';

  String _accessToken;
  bool _search = false;
  bool _tapChange = false;
  DateTime _startDate;
  DateTime _endDate;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblQueue);
  int _pendingLeaveCount = 0;
  bool _processing = false;

  @override
  void initState() {
    super.initState();
    _loadData();
  }

  _loadData() async{
        _selectedIndex = widget.selectedIndex;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
        _accessToken = await _userService.getToken();
        setState(() {_title = widget.title;});
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:() async{
                    setState(() {
                      _reset();
                      _search = !_search;
                    });
                    Map<String, dynamic> result = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => PickupSearchFilter(
                            title: _title,
                            authorityResponse: _authorityResponse,
                            currentUser: _currentUser,
                            sharedPrefs: _sharedPrefs,
                            accessToken: _accessToken,
                          ),
                        )
                    );
                    if( result != null ){
                      _keyword = result['keyword'];
                      _startDate = result['startDate'];
                      _endDate = result['endDate'];
                      _oSGuid = result['osGuid'];
                      _employeeGuid = result['employeeGuid'];
                    }
                    setState(() {
                      _search = !_search;
                    });
                  },
                  icon: Icon(Icons.search, color: Colors.white,),
                  label: Text(tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              ),
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.list),
                label: tr(LocaleKeys.lblHomeAdminToAssign),
              ),
              BottomNavigationBarItem(
                icon: _authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('SUPERVISOR_WRITE')
                    && _pendingLeaveCount>0? Badge(
                    position: BadgePosition.topEnd(top: -20, end: -20),
                    badgeContent: Text('$_pendingLeaveCount',style: TextStyle(color: Colors.white),),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: Icon(Icons.help)
                ) : Icon(Icons.upload_sharp),
                label: tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblHomeAdminToAccept:LocaleKeys.lblToPickingUp),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.local_shipping_sharp),
                label: tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblToPickingUp:LocaleKeys.lblPickupOnWay),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.done_outline_sharp),
                label: tr(LocaleKeys.lblPickupCompleted),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.cancel_schedule_send_sharp),
                label: tr(LocaleKeys.lblPickupCancel),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Theme.of(context).primaryColor,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  Widget _createBody(){

    List<String> status;
    if( _selectedIndex == 0 && !_search && !_tapChange){
      status = [Constant.STATUS_PICKUP_CREATED];
    }else if( _selectedIndex == 1 && !_search && !_tapChange){
      status = [Constant.STATUS_PICKUP_SCHEDULED];
    }else if( _selectedIndex == 2 && !_search && !_tapChange){
      status = [Constant.STATUS_PICKUP_ON_WAY];
    }else if( _selectedIndex == 3 && !_search && !_tapChange){
      status = [Constant.STATUS_COMPLETED];
    }else if( _selectedIndex == 4 && !_search && !_tapChange){
      status = [Constant.STATUS_CANCELED];
    }else{
      return new Container();
    }

    return PickupListMain(
      keyword: _keyword,
      startDate: _startDate,
      endDate: _endDate,
      status: status,
      osGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _oSGuid,
      customerGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("USER_READ") && !_authorityResponse.authorities.contains("MANAGER_READ") && !_authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("EMPLOYEE_READ")? _currentUser.guid : _customerGuid,
      employeeGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _employeeGuid,
      branchGuid: null,
      onChange: (){
        widget.onChange();
      },
    );
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = tr(LocaleKeys.lblPickupQueue);
        _reset();
      }else if( _selectedIndex == 1){
        _title = tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblHomeAdminToAccept:LocaleKeys.lblToPickingUp);
        _reset();
      }else if( _selectedIndex == 2){
        _title = tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ") && _authorityResponse.authorities.contains("EMPLOYEE_READ")?LocaleKeys.lblToPickingUp:LocaleKeys.lblPickupOnWay);
        _reset();
      }else if( _selectedIndex == 3){
        _title = tr(LocaleKeys.lblPickupCompleted);
        _reset();
      }else if( _selectedIndex == 4){
        _title = tr(LocaleKeys.lblPickupCancel);
        _reset();
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  _reset(){
    _keyword = '';
    _employeeGuid = null;
    _startDate = null;
    _endDate = null;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}
