
import 'package:easy_localization/easy_localization.dart';
import 'package:latlong/latlong.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery2/pickup_detail.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';
import 'delivery_list_for_pickup.dart';



class PickupListMain extends StatefulWidget {
  PickupListMain({Key key, this.keyword, this.startDate, this.endDate, this.osGuid,
    this.customerGuid, this.employeeGuid, this.zoneGuid,
    this.branchGuid, this.status, this.onChange}) : super(key: key);
  final String keyword;
  final DateTime startDate;
  final DateTime endDate;
  final String osGuid;
  final String customerGuid;
  final String employeeGuid;
  final String zoneGuid;
  final String branchGuid;
  final List<String> status;
  final Function onChange;

  @override
  _PickupListMainState createState() => _PickupListMainState();
}

class _PickupListMainState extends State<PickupListMain>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DeliveryListResponse _deliveryListResponse;
  String _accessToken;
  PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
  DeliveryListRequest _listRequest = new DeliveryListRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  bool _isFirst = true;
  UserService userService = UserService();

  @override
  void initState() {
    super.initState();

    String orderBy = 'id';
    if( widget.status[0] == Constant.STATUS_PICKUP_CREATED)
      orderBy = 'id';
    if( widget.status[0] == Constant.STATUS_PICKUP_SCHEDULED)
      orderBy = 'id';
    if( widget.status[0] == Constant.STATUS_PICKUP_ON_WAY)
      orderBy = 'os.name_mm';
    if( widget.status[0] == Constant.STATUS_COMPLETED)
      orderBy = 'updated_on';

    _listRequest = new DeliveryListRequest(
        keyword: widget.keyword,
        startDate: widget.startDate,
        endDate: widget.endDate,
        osGuid: widget.osGuid,
        customerGuid: widget.customerGuid,
        employeeGuid: widget.employeeGuid,
        zoneGuid: widget.zoneGuid,
        branchGuid: widget.branchGuid,
        status: widget.status,
        first: _page,
        max: Constant.ROW_PER_PAGE,
        orderBy: orderBy,
        dateParam: 'month'
    );
  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      _accessToken = await userService.getToken();
      _listRequest.first = _page;
      _listRequest.max = Constant.ROW_PER_PAGE;
      _listRequest.osGuid = _authorityResponse.authorities !=null
          && _authorityResponse.authorities.contains("OS_READ")
          && !_authorityResponse.authorities.contains("MANAGER_READ")
          && _listRequest.customerGuid==null? _currentUser.guid : _listRequest.osGuid;
      // _listRequest.customerGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("USER_READ") && !_authorityResponse.authorities.contains("MANAGER_READ") && !_authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("EMPLOYEE_READ")? _currentUser.guid : _listRequest.customerGuid;
      _listRequest.employeeGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _listRequest.employeeGuid;

      _deliveryListResponse = await _pickupServiceV2.list(_accessToken, _listRequest);
    if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    try{
      if( _deliveryListResponse != null)
        widget.onChange(_deliveryListResponse.totalRecords);
    }catch(e){}

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: null,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        currentPage: '${ i+1 } of ${_deliveryListResponse.totalRecords}',
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
          widget.onChange();
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        widget.onChange();
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser,
  this.currentPage}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final DeliveryContentServiceV2 deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String currentPage;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  int _selectedStep = 0;
  StepState _pickupStepState = StepState.indexed;
  StepState _dropOffStepState = StepState.indexed;
  bool _dropOffInactive = false;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    _stepState(post.status);

    return Container(
        padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child: new Wrap(
            children: <Widget>[
              ListTile(
                title: //Text('Way ID: ${post.trackingId}', style: TextStyle(color: Colors.white),),
                _createText(post.status, post.totalWays, context),
                subtitle: Text( tr(LocaleKeys.lblDeliCreatePickupDate)+ ' '+ (post.pickupDate!=null? new DateFormat('dd/MM/yyyy').format(post.pickupDate.toLocal()):'../../....') ,
                  // style: TextStyle(color: Colors.blueGrey[600]),
                ),
                onTap: () async {
                  bool result = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => PickupDetail(
                          wayId: post.trackingId,
                          onChange: (){
                            widget.onChange();
                          },
                        ),
                      )
                  );
                  if( result!=null && result ){
                    widget.onChange();
                  }
                },
                trailing: Icon(Icons.arrow_forward_sharp,
                  // color: Colors.blueGrey[600],
                ),
              ),

              Stepper(
                steps: [
                  Step(
                    isActive: true,
                    state: _pickupStepState,
                    title: Text('${post.osName} [ID:${post.osPersonId}]'),
                    subtitle: Text('${tr(LocaleKeys.lblSenderMobile)} ${post.osPrimaryPhone}'),
                    content: GestureDetector(
                      child: Text(
                        '${tr(LocaleKeys.lblDeliCreatePickup)} ${post.osAddress}',
                      ),
                      onTap: () async {
                        makeCall(post.osPrimaryPhone);
                      },
                    ),
                  ),
                ],
                currentStep: _selectedStep,
                controlsBuilder: (BuildContext context,ControlsDetails details) => Container(),
                type: StepperType.vertical,
                physics: ClampingScrollPhysics(),
                onStepTapped: (step){setState(() {
                  _selectedStep = step;
                });},
              ),
              post.addressCount ==null || (post.addressCount!=null && post.itemCount<post.totalWays)? ListTile(
                title: Text(tr(LocaleKeys.lblItemListAddressRequired)),
                trailing: ClipOval(
                    child: Material(
                      elevation: 1,
                      child: InkWell(
                        // splashColor: Theme.of(context).backgroundColor,
                        child: new Container(
                          // width: 60.0,
                          // height: 60.0,
                          padding: const EdgeInsets.all(12.0),
                          decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            // borderRadius: new BorderRadius.circular(30.0),
                            color: Colors.yellow[800],
                          ),
                          child: new Text(
                            '${post.addressCount!=null? (post.totalWays-post.addressCount) : post.totalWays }',
                            style: Theme.of(context).textTheme.headline5,
                          ),
                        ),
                      ),
                    )
                ),
                onTap: () {
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => DeliveryListForPickup(
                          trackingId: post.trackingId,
                          pickupGuid: post.contentGUID,
                          merchantName: post.osName,
                          personId: post.osPersonId,
                          onChange: (){
                            widget.onChange();
                          },
                          addNew: false,
                        ),
                      )
                  );
                },
              ) : Container(),
              post.itemCount ==null || (post.itemCount!=null && post.itemCount<post.totalWays)? ListTile(
                title: Text(tr(LocaleKeys.lblItemListItemRequired)),
                trailing: ClipOval(
                    child: Material(
                      elevation: 1,
                      child: InkWell(
                        // splashColor: Theme.of(context).backgroundColor,
                        child: new Container(
                          // width: 60.0,
                          // height: 60.0,
                          padding: const EdgeInsets.all(12.0),
                          decoration: new BoxDecoration(
                            shape: BoxShape.circle,
                            // borderRadius: new BorderRadius.circular(30.0),
                            color: Colors.yellow[800],
                          ),
                          child: new Text(
                            '${post.itemCount!=null? (post.totalWays-post.itemCount) : post.totalWays }',
                            style: Theme.of(context).textTheme.headline5,
                          ),
                        ),
                      ),
                    )
                ),
                onTap: () {
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => DeliveryListForPickup(
                          trackingId: post.trackingId,
                          pickupGuid: post.contentGUID,
                          merchantName: post.osName,
                          personId: post.osPersonId,
                          onChange: (){
                            widget.onChange();
                          },
                          addNew: false,
                        ),
                      )
                  );
                },
              ) : Container(),
              ListTile(
                title: LinearProgressIndicator(
                    backgroundColor: Colors.blueGrey[200],
                    value: ((post.completedCount??0).toDouble() / post.totalWays.toDouble()).toDouble(),
                    valueColor: AlwaysStoppedAnimation<Color>(Theme.of(context).primaryColor),
                ),
                leading: IconButton(icon: Icon(Icons.thumb_up, color: Colors.blueGrey[200],),
              ),
                subtitle: Text('${tr(LocaleKeys.lblDeliveryProgress)} ${post.completedCount??0}/${post.totalWays}'),
                // trailing: Icon(_expand? Icons.keyboard_arrow_up_sharp : Icons.keyboard_arrow_down_sharp, color: Colors.white,),
              ),
              widget.authorityResponse.authorities!=null
                  && (widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)
                  && post.refundInAdvanced!=null
                  && post.refundInAdvanced
                  && post.cashAdvanceAmount>0?ListTile(
                title: Text(
                  tr(LocaleKeys.lblRequiredToRefund),
                  style: TextStyle(color: Colors.red[600]),
                ),
                subtitle: Text('${NumberFormat('#,###').format(post.cashAdvanceAmount)} Ks'),
              ) : Container(),
              widget.authorityResponse.authorities!=null
                  && (widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)
                  && post.highestMessage!=null?ListTile(
                    title: Text(
                      tr(LocaleKeys.lblRefundInformation),
                      style: TextStyle(color: Colors.red[600]),
                    ),
                    subtitle: Text('${post.highestMessage}'),
              ) : Container(),
              /*
              widget.authorityResponse.authorities!=null
                  && (
                  widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)
                  && post.totalWays!=post.completedCount && post.totalWays!=post.itemCount? Container(
                padding: EdgeInsets.all(8),
                child: ElevatedButton.icon(
                  onPressed: () {
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => DeliveryListForPickup(
                            trackingId: post.trackingId,
                            pickupGuid: post.contentGUID,
                            merchantName: post.osName,
                            onChange: (){
                              widget.onChange();
                            },
                            addNew: true,
                          ),
                        )
                    );
                  },
                  label: Text(tr(LocaleKeys.lblItemListPickupAddWays), style: TextStyle(color: Colors.white)),
                  icon: Icon(Icons.add, color: Colors.white,),
                  style: ElevatedButton.styleFrom(
                    shape: new RoundedRectangleBorder(
                      borderRadius: new BorderRadius.circular(15.0),
                    ),
                  ),
                ),
              ): Container(),

              widget.authorityResponse.authorities!=null
                  && (widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)
                  && post.totalWays!=post.completedCount? Container(
                padding: EdgeInsets.all(8),
                child: ElevatedButton.icon(
                  label: Text(tr(LocaleKeys.lblItemListPickupEditWays), style: TextStyle(color: Colors.white),),
                  icon: Icon(Icons.edit, color: Colors.white),
                  style: ElevatedButton.styleFrom(
                  shape: new RoundedRectangleBorder(
                    borderRadius: new BorderRadius.circular(15.0),
                  ),
                ),
                  onPressed: () {
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => DeliveryListForPickup(
                            trackingId: post.trackingId,
                            pickupGuid: post.contentGUID,
                            merchantName: post.osName,
                            onChange: (){
                              widget.onChange();
                            },
                            addNew: false,
                          ),
                        )
                    );
                  },
                ),
              ) : Container(),
              */

              widget.authorityResponse.authorities!=null
                  && (widget.authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                  || post.osGUID==widget.currentUser.guid)? Container(
                padding: EdgeInsets.all(8),
                width: double.infinity,
                child: ElevatedButton.icon(
                  label: Text(tr(LocaleKeys.lblPickupListWaysList), style: TextStyle(color: Colors.white),),
                  icon: Icon(Icons.format_list_numbered_sharp, color: Colors.white),
                  style: ElevatedButton.styleFrom(
                    shape: new RoundedRectangleBorder(
                      borderRadius: new BorderRadius.circular(15.0),
                    ),
                  ),
                  onPressed: () {
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => DeliveryListForPickup(
                            trackingId: post.trackingId,
                            pickupGuid: post.contentGUID,
                            merchantName: post.osName,
                            personId: post.osPersonId,
                            onChange: (){
                              widget.onChange();
                            },
                            addNew: false,
                          ),
                        )
                    );
                  },
                ),
              ) : Container(),

              widget.authorityResponse.authorities!=null
                  // && post.osGUID==widget.currentUser.guid
                  && post.totalWays==post.itemCount
                  && post.totalWays==post.addressCount
                  && post.refunded==false? Container(
                      padding: EdgeInsets.all(8),
                      width: double.infinity,
                      child: ElevatedButton.icon(
                        label: Text(tr(LocaleKeys.lblDeliveryWayReceipt), style: TextStyle(color: Colors.white),),
                        icon: Icon(Icons.receipt_long_sharp, color: Colors.white),
                        style: ElevatedButton.styleFrom(
                          shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(15.0),
                          ),
                        ),
                        onPressed: ()async{
                          String uri = '/v1/forwarder/deliver_invoice_pdf/${post.trackingId}?at=${widget.accessToken}&docType=receipt';
                          if (await canLaunch( '${getBaseURL(widget.sharedPrefs)}$uri' )) {
                            await launch( '${getBaseURL(widget.sharedPrefs)}$uri' );
                          } else {
                            throw 'Could not launch $uri';
                          }
                        },
                      ),
                  ) : Container(),

              widget.authorityResponse.authorities!=null
                  // && post.osGUID==widget.currentUser.guid
                  && post.totalWays==post.itemCount
                  && post.totalWays==post.addressCount
                  && post.refunded==true? Container(
                    padding: EdgeInsets.all(8),
                    width: double.infinity,
                      child: ElevatedButton.icon(
                        label: Text(tr(LocaleKeys.lblDeliveryWayInvoice), style: TextStyle(color: Colors.white),),
                        icon: Icon(Icons.money, color: Colors.white),
                        style: ElevatedButton.styleFrom(
                          shape: new RoundedRectangleBorder(
                            borderRadius: new BorderRadius.circular(15.0),
                          ),
                        ),
                        onPressed: ()async{
                          String uri = '/v1/forwarder/deliver_invoice_pdf/${post.trackingId}?at=${widget.accessToken}&docType=invoice';
                          if (await canLaunch( '${getBaseURL(widget.sharedPrefs)}$uri' )) {
                            await launch( '${getBaseURL(widget.sharedPrefs)}$uri' );
                          } else {
                            throw 'Could not launch $uri';
                          }
                        },
                      ),
                    ) : Container(),
              Container(
                padding: EdgeInsets.only(right: 4.0),
                child: Align(
                  alignment: Alignment.bottomRight,
                  child: ElevatedButton(
                    onPressed: ()async{ },
                    child: Text(
                      '${widget.currentPage}',
                      style: TextStyle(
                        // fontWeight: FontWeight.w400,
                        // color: Theme.of(context).primaryColor,
                      ),
                    ),
                    style: ElevatedButton.styleFrom(
                      shape: new RoundedRectangleBorder(
                        borderRadius: new BorderRadius.circular(32.0),
                      ),
                    ),
                  ),
                ),
              ),
            ],
          ),
        )
    );
  }

  Text _createText(String status, int totalWays, BuildContext context ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListPickupCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
    || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }

    title = totalWays.toString() + tr(LocaleKeys.lblItemListTotalWays) + title;
    return Text(
      '$title',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }

  StepState _stepState(String status){
    if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.indexed;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CANCELED ){
      _pickupStepState = StepState.error;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.indexed;
      _dropOffInactive = true;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_COMPLETED){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 0;

    }else if( status == Constant.STATUS_RETURN){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;
    }
    return _pickupStepState;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  String _getTotalDistance(double lat1, double lon1, double lat2, double lon2 ){
    if( lat1 == null || lon1==null || lat2==null || lon2==null)
      return null;
    final Distance distance = Distance();
    return '${distance.as(LengthUnit.Kilometer, LatLng( lat1, lon1 ), LatLng( lat2, lon2 ))}km';
  }

  List<String> _completedPercent( DeliveryListItem post ){

    double completed = (post.completedCount??0 / post.totalWays) * 100;

    List<String> values = [];
    final Distance distance = Distance();
    double totalDistance;
    double travelDistance;
    double _completed = 0.0;

    if( post.osLatitude != null && post.osLongitude!=null && post.customerLatitude!=null && post.customerLongitude!=null)
      totalDistance = distance.as(LengthUnit.Kilometer, LatLng( post.osLatitude, post.osLongitude), LatLng( post.customerLatitude, post.customerLongitude));

    if( post.status == Constant.STATUS_PICKUP_CREATED || post.status == Constant.STATUS_PICKUP_SCHEDULED){
      values.add('0.0');
      values.add('0.0km');
      return values;
    }else if( post.status == Constant.STATUS_PICKUP_ON_WAY){
      travelDistance = totalDistance!=null? totalDistance/4 : 0;
      values.add('0.25');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else if( post.status == Constant.STATUS_CREATED || post.status==Constant.STATUS_ASSIGNED || post.status==Constant.STATUS_RETRY){
      travelDistance = totalDistance!=null? totalDistance/2 : 0;
      values.add('0.5');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else if( post.status == Constant.STATUS_ON_WAY){
      if( post.dropOffPersonLatitude != null && post.dropOffPersonLongitude!=null && post.customerLatitude!=null && post.customerLongitude!=null)
        travelDistance = distance.as(LengthUnit.Kilometer, LatLng( post.dropOffPersonLatitude, post.dropOffPersonLongitude), LatLng( post.customerLatitude, post.customerLongitude));
      if( totalDistance!=null && totalDistance>0 && travelDistance!=null && travelDistance>0){
        _completed = travelDistance / totalDistance;
      }
      if( _completed>0.5){
        values.add('$_completed');
        values.add( travelDistance.toStringAsFixed(2)+'km' );
        return values;
      }else{
        travelDistance = totalDistance!=null? totalDistance/2 : 0;
        values.add('0.5');
        values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
        return values;
      }
    }else if( post.status == Constant.STATUS_COMPLETED){
      travelDistance = totalDistance!=null? totalDistance : 0;
      values.add('1.0');
      values.add( travelDistance>0? travelDistance.toStringAsFixed(2)+'km' :'Unknown' );
      return values;
    }else{
      values.add('0.0');
      values.add( 'Unknown' );
      return values;
    }
  }
}