import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/request/person_list_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/person_list_response.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:shared_preferences/shared_preferences.dart';


class PickupSearchFilter extends StatefulWidget {
  PickupSearchFilter({Key key, this.title,
    this.authorityResponse, this.currentUser, this.sharedPrefs,
    this.accessToken }) : super(key: key);
  final String title;
  final AuthorityResponse authorityResponse;
  final User currentUser;
  final SharedPreferences sharedPrefs;
  final String accessToken;
  @override
  _PickupSearchFilterState createState() => _PickupSearchFilterState();
}

class _PickupSearchFilterState extends State<PickupSearchFilter> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_osDate = new TextEditingController();
  final TextEditingController _controller_customerDate = new TextEditingController();
  final TextEditingController _controller_keyword = new TextEditingController();
  DateTime _osDate;
  DateTime _customerDate;
  bool _clearOnly = true;
  bool _onHandOnly = true;

  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  String _employeeGuid;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();

  // AddressBookListResponse _addressBookListResponseOS;
  PersonListItem _addressBookItemOS;
  String _oSGuid;
  PersonListResponse _personListResponse;

  AddressBookListResponse _addressBookListResponseCustomer;
  AddressBookItem _addressBookItemCustomer;
  String _customerGuid;
  AddressBookService _addressBookService = new AddressBookService();


  @override
  void initState() {
    super.initState();
    _loadData();
  }

  _loadData() async{
    _employeeSearchResponse = await _deliveryContentService.availableAllEmployees( widget.accessToken );

    PersonService _personService = new PersonService();
    _personListResponse = await _personService.listByRole(
        new PersonListRequest(
          first: 0,
          max: 100000,
          orderBy: "firstName",
          asc: true,
          keyword: '',
          status: 'Active',
          roleName: 'OS',
          osGuid: '',
        )
    );
    // _addressBookListResponseCustomer = await _addressBookService.list(
    //     new AddressBookRequest(
    //         addressType: 'Customer',
    //         osGuid: widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_READ") && !widget.authorityResponse.authorities.contains("MANAGER_READ")? widget.currentUser.guid : null
    //     )
    // );
    setState(() { });
  }


  @override
  Widget build(BuildContext context) {

    return new WillPopScope(
        onWillPop: _onWillPop,
        child:  Scaffold(
            backgroundColor: Theme.of(context).backgroundColor,
            appBar: AppBar(
              title: Text(widget.title, style: TextStyle(color: Colors.white)),
              actions: <Widget>[
                FlatButton.icon(
                    onPressed:() {
                      setState(() {
                        Map<String, dynamic> result = {
                          'keyword': _controller_keyword.text,
                          'startDate': _osDate,
                          'endDate': _customerDate,
                          'osGuid': _oSGuid,
                          'employeeGuid': _employeeGuid
                        };
                        Navigator.pop(context, result);
                      });
                    },
                    icon: Icon(Icons.cancel_outlined, color: Theme.of(context).primaryColor,),
                    label: Text( tr(LocaleKeys.btnClose),
                      style:  TextStyle(color: Theme.of(context).primaryColor),
                    )
                )
              ],
            ),

            // AppBar(
            //   key: _scaffoldKey,
            //   title: Text(widget.title, style: TextStyle(color: Colors.white)),
            // ),
            body: SingleChildScrollView(
              child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: Container(
                    margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 8.0, right: 8.0),
                    child: Card(
                      color: Colors.white,
                      elevation: 0,
                      child: Column(
                          children: <Widget>[
                            Container(
                              margin: const EdgeInsets.only(top: 5.0, bottom: 0.0, left: 5.0, right: 5.0),
                              child:new TextFormField(
                                decoration: new InputDecoration(
                                  icon: Icon(Icons.keyboard, color: Theme.of(context).primaryColor),
                                  hintText: '${tr(LocaleKeys.lblExample)} ${tr(LocaleKeys.lblWayID)}',
                                  labelText: tr(LocaleKeys.lblKeyword),
                                  labelStyle: TextStyle(color: Theme.of(context).primaryColor),
                                ),
                                controller: _controller_keyword,
                                keyboardType: TextInputType.text,
                                autofocus: true,
                              ),
                            ),
                            widget.authorityResponse != null && ( widget.authorityResponse.authorities.contains("MANAGER_READ") || widget.authorityResponse.authorities.contains("EMPLOYEE_READ") )? _oSSelection(context) : Container(),
                            widget.authorityResponse != null && widget.authorityResponse.authorities.contains("MANAGER_READ")? _employeeSelection(context) : Container(),
                            // _customerSelection(context),
                            Container(
                              margin: const EdgeInsets.only(top:8, bottom: 8, left:8, right: 8),
                              child:new Row(children: <Widget>[
                                new Expanded(
                                    child: new TextFormField(
                                      decoration: new InputDecoration(
                                        hintText: 'yyyy-mm-dd',
                                        labelText: '${tr(LocaleKeys.lblStartDate)}',
                                        labelStyle: TextStyle(color: Theme.of(context).primaryColor),
                                      ),
                                      controller: _controller_osDate,
                                      keyboardType: TextInputType.datetime,
                                      // validator: (val) => isValidDob(val) ? null : 'Not a valid date',
                                      onSaved: (val) => _osDate = convertToDate(val),
                                    )
                                ),
                                new IconButton(
                                  icon: new Icon(Icons.calendar_today, color: Theme.of(context).primaryColor),
                                  tooltip: 'Choose date',
                                  onPressed: (() {
                                    _chooseDate(context, _controller_osDate.text, true);
                                  }),
                                )
                              ]),
                            ),
                            Container(
                              margin: const EdgeInsets.only(top:8, bottom: 8, left:8, right: 8),
                              child:new Row(children: <Widget>[
                                new Expanded(
                                    child: new TextFormField(
                                      decoration: new InputDecoration(
                                        hintText: 'yyyy-mm-dd',
                                        labelText: '${tr(LocaleKeys.lblEndDate)}',
                                        labelStyle: TextStyle(color: Theme.of(context).primaryColor),
                                      ),
                                      controller: _controller_customerDate,
                                      keyboardType: TextInputType.datetime,
                                      // validator: (val) => isValidDob(val) ? null : 'Not a valid date',
                                      onSaved: (val) => _customerDate = convertToDate(val),
                                    )
                                ),
                                new IconButton(
                                  icon: new Icon(Icons.calendar_today, color: Theme.of(context).primaryColor),
                                  tooltip: 'Choose date',
                                  onPressed: (() {
                                    _chooseDate(context, _controller_customerDate.text, false);
                                  }),
                                )
                              ]),
                            ),
                            Container(
                              margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                children: [
                                  Column(
                                    mainAxisSize: MainAxisSize.min,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      OutlinedButton(
                                        onPressed: (){
                                          Map<String, dynamic> result = {
                                            'keyword': _controller_keyword.text,
                                            'startDate': _osDate,
                                            'endDate': _customerDate,
                                            'osGuid': _oSGuid,
                                            'employeeGuid': _employeeGuid
                                          };
                                          Navigator.pop(context, result);
                                        },
                                        child: Text( tr(LocaleKeys.btnSearch).toUpperCase(), ),
                                        style: OutlinedButton.styleFrom(
                                            side: BorderSide(width: 2.0, color: Theme.of(context).primaryColor),
                                            shape: StadiumBorder(),
                                            minimumSize: Size(180,32)
                                        ),
                                      ),
                                    ],
                                  ),
                                  Column(
                                    mainAxisSize: MainAxisSize.min,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      OutlinedButton(
                                        onPressed: (){
                                          Map<String, dynamic> result = {
                                            'keyword': '',
                                            'startDate': null,
                                            'endDate': null,
                                            'osGuid': null,
                                            'employeeGuid': null
                                          };
                                          Navigator.pop(context, result);
                                        },
                                        child: Text( tr(LocaleKeys.btnReset).toUpperCase() ),
                                        style: OutlinedButton.styleFrom(
                                            side: BorderSide(width: 2.0, color: Theme.of(context).primaryColor),
                                            shape: StadiumBorder(),
                                            minimumSize: Size(180,32)
                                        ),
                                      ),
                                    ],
                                  ),
                                ],
                              ),
                            ),


                          ]
                      ),
                    )
                ),
              ),
            )
        )
    );
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 16, left: 8, right: 8),
        child: DropdownSearch<EmployeeForDeliveryItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeForDeliveryItem data) {
            _employeeSearchItem = data;
            if( _employeeSearchItem != null){
              _employeeGuid = _employeeSearchItem.guid;
            }else{
              _employeeGuid = null;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle, color: Theme.of(context).primaryColor),
                  title: Text(tr(LocaleKeys.lblAllEmployees),  style: TextStyle(color: Theme.of(context).primaryColor)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: item.avatar !=null? new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                    width: 42, height: 42, ) : Icon(Icons.account_circle, color: Theme.of(context).primaryColor),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.zoneName}'),
                  leading: item.avatar ==null? Icon(Icons.account_circle, color: Theme.of(context).primaryColor) : ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = List();
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _oSSelection( BuildContext context){
    return
      widget.authorityResponse.authorities!=null
          && (widget.authorityResponse.authorities.contains("EMPLOYEE_READ")
          || widget.authorityResponse.authorities.contains("OS_READ"))?
      Container(
          margin: EdgeInsets.only(top: 16, left: 8, right: 8),
          child: DropdownSearch<PersonListItem>(
            selectedItem: _addressBookItemOS,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (PersonListItem i, PersonListItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblOS),
            onFind: (String filter) => _oSSuggestion(filter),
            onChanged: (PersonListItem data) {
              _addressBookItemOS = data;
              if( _addressBookItemOS != null){
                _oSGuid = _addressBookItemOS.guid;
              }else{
                _oSGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, PersonListItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle, color: Theme.of(context).primaryColor),
                    title: Text(tr(LocaleKeys.lblAllOS), style: TextStyle(color: Theme.of(context).primaryColor),),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: item.avatar !=null? Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ) : Icon(Icons.account_circle, color: Theme.of(context).primaryColor),
                  ),
                ),
            popupItemBuilder: (BuildContext context, PersonListItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    // color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text('${item.name} [${item.personId}]'),
                    subtitle: Text(item.address),
                    leading: item.avatar ==null? Icon(Icons.account_circle, color: Theme.of(context).primaryColor) : ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }
  Future<List<PersonListItem>> _oSSuggestion(String query) async{
    List<PersonListItem> matches = List();
    if( _personListResponse.items == null)
      return matches;
    matches.addAll(_personListResponse.items);

    matches.retainWhere((s) {
      PersonListItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if (addressBookItem.personId.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _customerSelection( BuildContext context){
    return
      widget.authorityResponse.authorities!=null
          && (widget.authorityResponse.authorities.contains("EMPLOYEE_READ")
          || widget.authorityResponse.authorities.contains("OS_READ"))?
      Container(
          margin: EdgeInsets.only(top: 5, left: 10, right: 10),
          child: DropdownSearch<AddressBookItem>(
            selectedItem: _addressBookItemCustomer,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (AddressBookItem i, AddressBookItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblCustomer),
            onFind: (String filter) => _customerSuggestion(filter),
            onChanged: (AddressBookItem data) {
              _addressBookItemCustomer = data;
              if( _addressBookItemCustomer != null){
                _customerGuid = _addressBookItemCustomer.personGuid;
              }else{
                _customerGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, AddressBookItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllCustomers)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    subtitle: Text(item.personGuid),
                    leading: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ),
                  ),
                ),
            popupItemBuilder: (BuildContext context, AddressBookItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    // color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text('${item.name} [${item.primaryPhone}]'),
                    subtitle: Text(item.townshipName),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }
  Future<List<AddressBookItem>> _customerSuggestion(String query) async{
    List<AddressBookItem> matches = List();
    if( _addressBookListResponseCustomer.items == null)
      return matches;
    matches.addAll(_addressBookListResponseCustomer.items);

    matches.retainWhere((s) {
      AddressBookItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool os) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(1900),
        lastDate: new DateTime(2030)//new DateTime.now()
    );

    if (result == null) return;

    setState(() {
      if( os ) {
        _controller_osDate.text = new DateFormat.yMd().format(result);
        _osDate = result;
      }else {
        _controller_customerDate.text = new DateFormat.yMd().format(result);
        _customerDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Future<bool> _onWillPop() async {
    return true;
  }

}