
import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:location/location.dart';
import 'package:qq_delivery/delivery/model/response/route_list_by_navigation_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/service/v2/route_navigation_service.dart';

class TrackingMap extends StatefulWidget {
  TrackingMap({this.startLocation, this.items, this.title, this.isEmployee,
   this.polyline, this.polylineId, this.contactItemCustomer, this.navigate,
  this.startRoute});

  final LatLng startLocation;
  List<Map<String,dynamic>> items;
  final title;
  final bool isEmployee;
  final Polyline polyline;
  final PolylineId polylineId;
  final DeliveryContactItem contactItemCustomer;
  final bool navigate;
  final bool startRoute;
  @override
  _TrackingMapState createState( ) => _TrackingMapState( );
}

class _TrackingMapState extends State<TrackingMap> {

  MapType _mapType = MapType.normal;
  bool _rotateGesturesEnabled = true;
  bool _scrollGesturesEnabled = true;
  bool _tiltGesturesEnabled = true;
  bool _zoomGesturesEnabled = true;
  bool _indoorViewEnabled = true;
  bool _myLocationEnabled = true;
  bool _myLocationButtonEnabled = true;
  bool _isMapCreated = false;
  bool _isMoving = false;
  bool _compassEnabled = true;
  bool _mapToolbarEnabled = true;

  GoogleMapController _controller;
  CameraPosition _kGooglePlex;
  Map<MarkerId, Marker> markers = <MarkerId, Marker>{};
  MarkerId selectedMarker;
  BitmapDescriptor _markerIcon;
  Map<PolylineId, Polyline> polylines = <PolylineId, Polyline>{};
  List<bool> _routeOptionSelections = List.generate(3, (_)=> false);
  bool _processing = false;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();
    _kGooglePlex = CameraPosition(
      target: widget.startLocation==null ? LatLng(16.813538,96.171205) : widget.startLocation,
      zoom: 13,
    );
  }

  Future<void> _createMarkerImageFromAsset(BuildContext context) async {
    if (_markerIcon == null) {
      final ImageConfiguration imageConfiguration =
      createLocalImageConfiguration(context, size: Size(24,24));
      BitmapDescriptor.fromAssetImage(
          imageConfiguration, 'assets/images/pin.png')
          .then(_updateBitmap);
    }
  }
  void _updateBitmap(BitmapDescriptor bitmap) {
    setState(() {
      _markerIcon = bitmap;
    });
  }
  void _add( ) {
    for( int i=0; i < widget.items.length; i++){
      Map employee = widget.items[i];
      if( employee['latitude'] != null && employee['longitude'] != null) {
        final MarkerId markerId = MarkerId('${employee['guid']}');
        final Marker marker = Marker(
          markerId: markerId,
          position: LatLng(
            employee['latitude'],
            employee['longitude'],
          ),
          infoWindow: InfoWindow(title: employee['name'], snippet: employee['phone']),
          onTap: () {
            _onMarkerTapped(markerId);
          },
          icon: employee['employee']?_markerIcon : BitmapDescriptor.defaultMarker,
          //BitmapDescriptor.fromBytes(base64Decode(employee['coverphoto'].substring(22)))
        );

        setState(() {
          markers[markerId] = marker;
        });
      }
    }
  }

  void _onMapCreated(GoogleMapController controller) {
    this._controller = controller;
    if( widget.items != null)
      _add( );
    if( widget.polyline !=null ){
      polylines[widget.polylineId] = widget.polyline;
    }
    if( widget.navigate !=null && widget.navigate )
      _initLocationService( );

    if( widget.startRoute !=null && widget.startRoute ){
      _navigate('bike');
    }
  }

  void _onMarkerTapped(MarkerId markerId) {
    final Marker tappedMarker = markers[markerId];
    if (tappedMarker != null) {
      setState(() {
        if (markers.containsKey(selectedMarker)) {
          final Marker resetOld = markers[selectedMarker]
              .copyWith(iconParam: BitmapDescriptor.defaultMarker);
          markers[selectedMarker] = resetOld;
        }
        selectedMarker = markerId;
        final Marker newMarker = tappedMarker.copyWith(
          iconParam: BitmapDescriptor.defaultMarkerWithHue(
            BitmapDescriptor.hueOrange,
          ),
        );
        markers[markerId] = newMarker;
      });
    }
  }


  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _createMarkerImageFromAsset(context);
    return ( widget.title == null ? Container(
      //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
      child: GoogleMap(
        mapType: _mapType,
        initialCameraPosition: _kGooglePlex,
        onMapCreated: _onMapCreated,
        compassEnabled: _compassEnabled,
        mapToolbarEnabled: _mapToolbarEnabled,
        rotateGesturesEnabled: _rotateGesturesEnabled,
        scrollGesturesEnabled: _scrollGesturesEnabled,
        tiltGesturesEnabled: _tiltGesturesEnabled,
        zoomGesturesEnabled: _zoomGesturesEnabled,
        indoorViewEnabled: _indoorViewEnabled,
        myLocationEnabled: _myLocationEnabled,
        myLocationButtonEnabled: _myLocationButtonEnabled,
        markers: Set<Marker>.of(markers.values),
        polylines: Set<Polyline>.of(polylines.values),
      ),
    )
        :
    Scaffold(
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: widget.navigate? _createSearchIndicator() : ( _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)) ),
      ),
      body: Center(
          child:Container(
            //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
            child: GoogleMap(
              mapType: _mapType,
              initialCameraPosition: _kGooglePlex,
              onMapCreated: _onMapCreated,
              compassEnabled: _compassEnabled,
              mapToolbarEnabled: _mapToolbarEnabled,
              rotateGesturesEnabled: _rotateGesturesEnabled,
              scrollGesturesEnabled: _scrollGesturesEnabled,
              tiltGesturesEnabled: _tiltGesturesEnabled,
              zoomGesturesEnabled: _zoomGesturesEnabled,
              indoorViewEnabled: _indoorViewEnabled,
              myLocationEnabled: _myLocationEnabled,
              myLocationButtonEnabled: _myLocationButtonEnabled,
              markers: Set<Marker>.of(markers.values),
              polylines: Set<Polyline>.of(polylines.values),
            ),
          )
      ),
    )
    );
  }

  Future<void> _navigate( String profile ) async{

    if( widget.contactItemCustomer ==null || (widget.contactItemCustomer !=null
        && (widget.contactItemCustomer.latitude ==null
        || widget.contactItemCustomer.longitude ==null) ) ) {
      showMessage("Customer location not available");
      return;
    }
    if( _startLocation == null ) {
      showMessage("Please turn GPS Location service on and then try again!");
      return;
    }
    if( _processing ) return;
    setState(() { _processing = true; });

    if( _startLocation !=null && widget.contactItemCustomer.latitude !=null
        && widget.contactItemCustomer.longitude !=null ){
      RouteNavigationService navigationService = new RouteNavigationService();
      RouteListByNavigationResponse response = await navigationService.navigate(
          _startLocation.latitude, _startLocation.longitude,
          widget.contactItemCustomer.latitude, widget.contactItemCustomer.longitude, profile);

      if( response !=null ){
        // removing old routes
        polylines.clear();

        if( profile == 'car') {
          if (response.carRoutePoints.isNotEmpty) {
            PolylineId polylineId = PolylineId("last route");
            Polyline polyline = Polyline(
              polylineId: polylineId,
              consumeTapEvents: true,
              color: Colors.orange,
              width: 15,
              points: response.carRoutePoints,
            );
            polylines[polylineId] = polyline;
            showMessage( 'Car driving distance: ${new NumberFormat("#,###.###").format(response.carDistance/1000)} kilometers' );
          }
        }else if( profile == 'bike') {
          if (response.bikeRoutePoints.isNotEmpty) {
            PolylineId polylineId = PolylineId("last route");
            Polyline polyline = Polyline(
              polylineId: polylineId,
              consumeTapEvents: true,
              color: Colors.orange,
              width: 15,
              points: response.bikeRoutePoints,
            );
            polylines[polylineId] = polyline;
            showMessage( 'Bike riding distance: ${new NumberFormat("#,###.###").format(response.bikeDistance/1000)} kilometers' );
          }

        }else if( profile == 'foot') {
          if (response.footRoutePoints.isNotEmpty) {
            PolylineId polylineId = PolylineId("last route");
            Polyline polyline = Polyline(
              polylineId: polylineId,
              consumeTapEvents: true,
              color: Colors.orange,
              width: 15,
              points: response.footRoutePoints,
            );
            polylines[polylineId] = polyline;
            showMessage( 'Walking distance: ${new NumberFormat("#,###.###").format(response.footDistance/1000)} kilometers' );
          }
        }

      }
    }

    setState(() { _processing = false; });
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 52.0),
      child: SizedBox(
          height: 52.0,
          child: Container(
            padding: EdgeInsets.only(top:0, bottom: 0, left:0, right: 0),
            child: Card(
                child: ToggleButtons(
                  children: <Widget>[
                    Container(
                      padding: EdgeInsets.only(top:0, bottom: 0, left:0, right: 0),
                      child: Icon(Icons.directions_car ),
                    ),
                    Container(
                      padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                      child: Icon(Icons.directions_bike_sharp ),
                    ),
                    Container(
                      padding: EdgeInsets.only(top:0, bottom: 0, left:8, right: 8),
                      child: Icon(Icons.directions_walk_sharp ),
                    ),
                  ],
                  onPressed: (int index) {
                    setState(() {
                      for (int buttonIndex = 0; buttonIndex < _routeOptionSelections.length; buttonIndex++) {
                        if (buttonIndex == index) {
                          _routeOptionSelections[buttonIndex] = true;
                        } else {
                          _routeOptionSelections[buttonIndex] = false;
                        }
                      }
                    });

                    if( index ==0 )
                      _navigate('car');
                    else if( index ==1 )
                      _navigate('bike');
                    else if( index ==2 )
                      _navigate('foot');
                  },
                  isSelected: _routeOptionSelections,
                ),
            )
          ),
      )
  );

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
  void showMessage(String message, [MaterialColor color = Colors.green]) {
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(new SnackBar(content: new Text(message)));
  }

  Location _locationService  = new Location();
  bool _permission = false;
  LocationData _currentLocation;
  LocationData _startLocation;
  String error;
  _initLocationService() async {
    LocationData location;
    try {

      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
            // print('Lat: ${_startLocation.latitude}, Lon:${_startLocation.longitude}');
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }
    _startLocation = location;
  }

}