import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';



class DeliveryInvoice extends StatefulWidget {
  DeliveryInvoice({Key key}) : super(key: key);

  @override
  _DeliveryInvoiceState createState() => _DeliveryInvoiceState();
}

class _DeliveryInvoiceState extends State<DeliveryInvoice>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DeliveryListResponse _deliveryListResponse;
  String _accessToken;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  DeliveryListRequest _listRequest = new DeliveryListRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  String _keyword = '';
  bool _isFirst = true;
  UserService userService = UserService();


  @override
  void initState() {
    super.initState();
    _listRequest = new DeliveryListRequest(
        keyword: _keyword,
        startDate: null,
        endDate: null,
        osGuid: null,
        customerGuid: null,
        employeeGuid: null,
        zoneGuid: null,
        branchGuid: null,
        status: [Constant.STATUS_COMPLETED,Constant.STATUS_RETURN],
        first: _page,
        max: Constant.ROW_PER_PAGE
    );
  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      _accessToken = await userService.getToken();
      _listRequest.first = _page;
      _listRequest.max = Constant.ROW_PER_PAGE;
      _listRequest.osGuid = _authorityResponse.authorities !=null
          && _authorityResponse.authorities.contains("OS_READ")
          && !_authorityResponse.authorities.contains("MANAGER_READ")
          && _listRequest.customerGuid==null? _currentUser.guid : _listRequest.osGuid;
      // _listRequest.customerGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("USER_READ") && !_authorityResponse.authorities.contains("MANAGER_READ") && !_authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("EMPLOYEE_READ")? _currentUser.guid : _listRequest.customerGuid;
      _listRequest.employeeGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _listRequest.employeeGuid;

      _deliveryListResponse = await _deliveryContentService.invoices(_accessToken, _listRequest);
    if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: _deliveryContentService,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return  Scaffold(
        appBar: AppBar(
          title: Text(tr(LocaleKeys.lblInvoices), style: TextStyle(color: Colors.white)),
          actions: <Widget>[
            Container(
              child: IconButton(
                highlightColor: Colors.red,
                icon: Icon(Icons.download_sharp, color: Colors.white,),
                onPressed: (){
                  openBrowser(
                      Constant.BASE_URL_PRIMARY
                      +'/v1/forwarder/invoice/$_accessToken'
                      +'?osg='+ _listRequest.osGuid??''
                      +'&eg='+ _listRequest.employeeGuid??''
                  );
                },
                tooltip: 'Download',
              ),
            ),
          ],
        ),
        body: SmartRefresher(
          controller: _refreshController,
          enablePullUp: true,
          child: buildContent(context),
          physics: BouncingScrollPhysics(),
          header: CustomRefreshIndicator(),
          footer: ClassicFooter(
            loadStyle: LoadStyle.ShowWhenLoading,
            completeDuration: Duration(milliseconds: 500),
          ),

          // scroll down
          onRefresh: () async {
            _page = 0;
            await _getLoadData( true);
            if (mounted) setState(() {});
            _refreshController.refreshCompleted();

            /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
          },
          // scroll up
          onLoading: () async {
            ++_page;
            await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
            if (mounted) setState(() {});
            _refreshController.loadComplete();
          },
        ),
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final DeliveryContentServiceV2 deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    return Container(
        padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
        child: Card(
          elevation: 8,
          color: Colors.white,
          child: new Wrap(
            children: <Widget>[
              ListTile(
                title: Text(
                  'Invoice No.: ${post.trackingId}',
                  style: TextStyle(fontWeight: FontWeight.bold,
                      color: post.status==Constant.STATUS_COMPLETED?Theme.of(context).primaryColor: Colors.red),
                ),
                subtitle: Text('Inv date: ${new DateFormat('dd/MM/yyyy hh:mm a').format(post.updatedOn.toLocal())} ${ post.status==Constant.STATUS_COMPLETED?tr(LocaleKeys.lblItemListCompleted):tr(LocaleKeys.lblItemListReturn)}',
                  style: TextStyle(color: Colors.blueGrey[600]),
                ),
                onTap: () {
                  Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => InvoiceDetail(
                          post: post,
                          currentUser: widget.currentUser,
                        ),
                      )
                  );
                },
                trailing: Icon(Icons.double_arrow_sharp, color: Colors.blueGrey[600],),
              ),
              ListTile(
                title: Text('${post.customerName}'),
                subtitle: Text('${post.customerAddress}'),
              ),
              ListTile(
                title: Text('${post.dueAmount==0? tr(LocaleKeys.lblAccountClear) : tr(LocaleKeys.lblDueAmount)}'),
                subtitle: Text('${new NumberFormat("#,###").format(post.dueAmount*-1)} Ks'),
              ),
            ],
          ),
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class InvoiceDetail extends StatefulWidget {

  InvoiceDetail({Key key, @required this.post, @required this.currentUser}) : super(key: key);
  final DeliveryListItem post;
  final User currentUser;

  @override
  _InvoiceDetailState createState() => _InvoiceDetailState();
}

class _InvoiceDetailState extends State<InvoiceDetail> {


  @override
  Widget build( BuildContext context ) {
    return Scaffold(
        appBar: AppBar(
          title: Text(widget.post.trackingId, style: TextStyle(color: Colors.white)),
        ),
      body: SingleChildScrollView(
          child: _deliverySummary( context ),
      ),
    );
  }

  Widget _deliverySummary( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblInvoiceDetail)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('Invoice date: '+ new DateFormat('dd/MM/yyyy hh:mm a').format(widget.post.updatedOn.toLocal()),
                      style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  trailing: IconButton(
                    icon: SvgPicture.asset( 'assets/images/pdf.svg', semanticsLabel: 'PDF',height: 24, width: 23, ),
                    onPressed: (){},
                  )
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    'Delivery order date: ${new DateFormat('dd/MM/yyyy hh:mm a').format(widget.post.createdOn.toLocal())}',
                    style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    '${tr(LocaleKeys.lblWayID)}: ${widget.post.trackingId}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblItemPrice)}: ${widget.post.totalItemPrice!=null? new NumberFormat("#,###").format(widget.post.totalItemPrice) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblDeliveryChargesShownOnRecipient)}: ${widget.post.forShowCharges!=null? new NumberFormat("#,###").format(widget.post.forShowCharges) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: Text(
                    "${tr(LocaleKeys.lblTotalCollectedAmount)}: ${widget.post.totalCharges!=null? new NumberFormat("#,###").format(widget.post.totalCharges) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),


                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblDeliCharges)}: ${widget.post.totalDeliveryCharges!=null? new NumberFormat("#,###").format(widget.post.totalDeliveryCharges) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0),
                  child: Text(
                    "${tr(LocaleKeys.lblPromotion)}: -${widget.post.promotionAmount!=null? new NumberFormat("#,###").format(widget.post.promotionAmount) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: Text(
                    "${tr(LocaleKeys.lblTotalDeliveryChargesAfterPromo)}: ${new NumberFormat("#,###").format(_deliveryCharges(widget.post))} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),

                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: Text(
                    "${tr(LocaleKeys.lblDueAmount)}: ${widget.post.dueAmount!=null? new NumberFormat("#,###").format(widget.post.dueAmount*-1) : 0} Ks",
                    style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                  ),
                ),

                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 0),
                  child: ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblSender)} ${widget.post.osName}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text('${tr(LocaleKeys.lblAddress)} ${widget.post.osAddress}',
                        style: TextStyle(color: Colors.blueGrey[600])
                    ),
                  ),
                ),

                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 16.0, bottom: 32),
                  child: ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblRecipient)} ${widget.post.customerName}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text('${tr(LocaleKeys.lblAddress)} ${widget.post.customerAddress}',
                        style: TextStyle(color: Colors.blueGrey[600])
                    ),
                  ),
                ),
              ],
            )
        )
    );
  }

  double _deliveryCharges( DeliveryListItem post ){
    double deliCharge = post.totalDeliveryCharges!=null? post.totalDeliveryCharges : 0;
    double promo = post.promotionAmount!=null? post.promotionAmount : 0;

    return (deliCharge - promo);
  }
}