import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:shared_preferences/shared_preferences.dart';


class EmployeeClearanceFilter extends StatefulWidget {
  EmployeeClearanceFilter({Key key, this.title,
  this.authorityResponse, this.currentUser, this.sharedPrefs,
  this.status}) : super(key: key);
  final String title;
  final AuthorityResponse authorityResponse;
  final User currentUser;
  final SharedPreferences sharedPrefs;
  final String status;
  @override
  _EmployeeClearanceFilterState createState() => _EmployeeClearanceFilterState();
}

class _EmployeeClearanceFilterState extends State<EmployeeClearanceFilter> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_osDate = new TextEditingController();
  final TextEditingController _controller_customerDate = new TextEditingController();
  DateTime _osDate;
  DateTime _customerDate;
  String _status ='On hand';


  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  String _employeeGuid;
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();

  @override
  void initState() {
    super.initState();
    _status = widget.status;
    _getLoadData();
  }

  void _getLoadData( ) async{
    if( widget.authorityResponse != null && widget.authorityResponse.authorities.contains("MANAGER_READ")) {
      UserService userService = UserService();
      String _accessToken = await userService.getToken();
      _employeeSearchResponse =
      await _deliveryContentService.availableAllEmployees(_accessToken);
      if (mounted) setState(() {});
    }
  }


  @override
  Widget build(BuildContext context) {

    return new WillPopScope(
      onWillPop: _onWillPop,
      child:  Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          key: _scaffoldKey,
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
        ),
        body: SingleChildScrollView(
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: Container(
            margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 8.0, right: 8.0),
            child: Card(
              color: Colors.white,
              elevation: 0,
              child: Column(
                  children: <Widget>[
                    Container(
                      padding: EdgeInsets.only(top:8, bottom: 0, left:8, right: 8),
                      child: RadioListTile(
                        value: 'On hand',
                        groupValue: _status,
                        onChanged: ( value){
                          setState(() {
                            _status = value;
                          });
                        },
                        title: Text(
                          tr(LocaleKeys.lblOnHand),
                          style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                        ),
                      ),
                    ),
                    Container(
                      padding: EdgeInsets.only(top:8, bottom: 0, left:8, right: 8),
                      child: RadioListTile(
                        value: 'Clear',
                        groupValue: _status,
                        onChanged: ( value){
                          setState(() {
                            _status = value;
                          });
                        },
                        title: Text(
                          tr(LocaleKeys.lblClear),
                          style: new TextStyle(fontSize: 16.0, color: Theme.of(context).primaryColor),
                        ),
                      ),
                    ),
                    Container(
                      margin: const EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                      child:new Row(children: <Widget>[
                        new Expanded(
                            child: new TextFormField(
                              decoration: new InputDecoration(
                                hintText: 'yyyy-mm-dd',
                                labelText: '${tr(LocaleKeys.lblStartDate)}',
                              ),
                              controller: _controller_osDate,
                              keyboardType: TextInputType.datetime,
                              // validator: (val) => isValidDob(val) ? null : 'Not a valid date',
                              onSaved: (val) => _osDate = convertToDate(val),
                            )
                        ),
                        new IconButton(
                          icon: new Icon(Icons.calendar_today, color: Colors.blueGrey[600]),
                          tooltip: 'Choose date',
                          onPressed: (() {
                            _chooseDate(context, _controller_osDate.text, true);
                          }),
                        )
                      ]),
                    ),
                    Container(
                      margin: const EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                      child:new Row(children: <Widget>[
                        new Expanded(
                            child: new TextFormField(
                              decoration: new InputDecoration(
                                hintText: 'yyyy-mm-dd',
                                labelText: '${tr(LocaleKeys.lblEndDate)}',
                              ),
                              controller: _controller_customerDate,
                              keyboardType: TextInputType.datetime,
                              // validator: (val) => isValidDob(val) ? null : 'Not a valid date',
                              onSaved: (val) => _customerDate = convertToDate(val),
                            )
                        ),
                        new IconButton(
                          icon: new Icon(Icons.calendar_today, color: Colors.blueGrey[600]),
                          tooltip: 'Choose date',
                          onPressed: (() {
                            _chooseDate(context, _controller_customerDate.text, false);
                          }),
                        )
                      ]),
                    ),
                    widget.authorityResponse != null && widget.authorityResponse.authorities.contains("MANAGER_READ")? _employeeSelection(context) : Container(),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 16),
                      width: double.infinity,
                      child: RaisedButton(
                        onPressed: () {
                          Map<String, dynamic> result = {
                            'startDate': _osDate,
                            'endDate': _customerDate,
                            'employeeGuid': _employeeGuid,
                            'status': _status,
                          };
                          Navigator.pop(context, result);
                        },
                        color: Theme.of(context).primaryColorDark,
                        elevation: 1,
                        child: Text(
                          tr(LocaleKeys.btnOK),
                          style: TextStyle(color: Colors.white),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ),
                  ]
              ),
            )
          ),
        ),
      )
      )
    );
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
          margin: EdgeInsets.only(top: 5, left: 10, right: 10),
          child: DropdownSearch<EmployeeForDeliveryItem>(
            selectedItem: _employeeSearchItem,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _employeeSuggestion(filter),
            onChanged: (EmployeeForDeliveryItem data) {
              _employeeSearchItem = data;
              if( _employeeSearchItem != null){
                _employeeGuid = _employeeSearchItem.guid;
              }else{
                _employeeGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ),
                  ),
                ),
            popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text('${item.zoneName}'),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      );
  }
  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = List();
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool os) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(1900),
        lastDate: new DateTime(2030)//new DateTime.now()
    );

    if (result == null) return;

    setState(() {
      if( os ) {
        _controller_osDate.text = new DateFormat.yMd().format(result);
        _osDate = result;
      }else {
        _controller_customerDate.text = new DateFormat.yMd().format(result);
        _customerDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}