import 'dart:async';
import 'dart:convert';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/employee_clearance_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_clearance_list_response.dart';
import 'package:qq_delivery/delivery/service/employee_collect_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'employee_clearance_filter.dart';


class EmployeeClearanceList extends StatefulWidget {
  EmployeeClearanceList({Key key}) : super(key: key);

  @override
  _EmployeeClearanceListState createState() => _EmployeeClearanceListState();
}

class _EmployeeClearanceListState extends State<EmployeeClearanceList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  EmpployeeClearanceListRequest request;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  EmployeeClearanceListResponse _employeeClearanceListResponse;
  EmployeeCollectedService _employeeCollectedService = new EmployeeCollectedService();
  List<EmployeeCollectedItem> _items = [];
  int _page = 0;
  bool _first = true;
  double _totalAmount = 0.0;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
      request = new EmpployeeClearanceListRequest(
          startDate: null,
          endDate: null,
          employeeGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : '',
          branchGuid: null,
          zoneGuid: null,
          status: 'On hand',
          first: _page,
          max: Constant.ROW_PER_PAGE,
          orderBy:'id',
          asc: false
      );
    }
    setState(() {});

    request.first = _page;
    request.max = Constant.ROW_PER_PAGE;

    if( Constant.ENABLE_LOGS)
      print('Request param= ${request.toMap()}');
    _employeeClearanceListResponse = await _employeeCollectedService.listEmployeeClearance(request);
    if( _employeeClearanceListResponse !=null && _employeeClearanceListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _employeeClearanceListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(tr(LocaleKeys.lblEmployeeClearance), style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : _createSelectedIndicator(),
            actions: <Widget>[
              Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.search_sharp, color: Colors.white,),
                  onPressed: () async{
                    Map<String, dynamic> result = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => EmployeeClearanceFilter(
                            title: 'Filter',
                            currentUser: _currentUser,
                            authorityResponse: _authorityResponse,
                            sharedPrefs: _sharedPrefs,
                            status: request.status,
                          ),
                        )
                    );
                    if( result != null ){
                      request.startDate = result['startDate']??null;
                      request.endDate = result['endDate']??null;
                      request.status = result['status']??'On hand';
                      request.employeeGuid = result['employeeGuid']??'';
                      _refreshController.requestRefresh();
                    }
                  },
                  tooltip: 'Filter',
                ),
              ),
              // Container(
              //   child: IconButton(
              //     highlightColor: Colors.red,
              //     icon: Icon(Icons.download_sharp, color: Colors.white,),
              //     onPressed: () async{
              //       String _accessToken = await _userService.getToken();
              //       String uri = _selectedIndex==0?'deposit':'refund';
              //       // openBrowser(
              //       //     Constant.BASE_URL_PRIMARY
              //       //         +'/v1/forwarder/$uri/$_accessToken'
              //       //         +'?osg='+ request.osGuid??''
              //       //         +'&bg='+ request.branchGuid??''
              //       //         +'&type='+ request.type??'all'
              //       //         +'&sd='+ (request.startDate!=null? new DateFormat('yyyy-MM-dd').format(request.startDate.toLocal()): '')
              //       //         +'&ed='+ (request.endDate!=null? new DateFormat('yyyy-MM-dd').format(request.endDate.toLocal()): '')
              //       // );
              //     },
              //     tooltip: 'Download',
              //   ),
              // ),
            ],
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }

  PreferredSize _createSelectedIndicator() => PreferredSize(
      preferredSize: Size(double.infinity,  56.0 ),
      child: SizedBox(
        height: 56.0,
        child: Card(shape: BeveledRectangleBorder(
            borderRadius: BorderRadius.only(topLeft: Radius.circular(0), topRight: Radius.circular(0), bottomRight: Radius.circular(4), bottomLeft: Radius.circular(4)),
          ),
          margin: EdgeInsets.only(left: 0, right: 0, bottom: 1, top: 0),
          elevation: 8,
          child: Wrap(
              children: <Widget>[
                ListTile(title: Text('${tr(LocaleKeys.lblTotalCollectedAmount)}',
                  textAlign: TextAlign.center,
                  style: Theme.of(context).textTheme.headline6),
                  trailing: Text('${ new NumberFormat("#,###").format(_employeeClearanceListResponse !=null? _employeeClearanceListResponse.totalCollected : 0)} Ks',
                      textAlign: TextAlign.center,
                      style: Theme.of(context).textTheme.headline6),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        child: InkWell(
                          child: new Container(
                            padding: const EdgeInsets.all(8.0),
                            decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              color: Theme.of(context).primaryColor,
                            ),
                            child: new Text(
                              '${_employeeClearanceListResponse !=null? _employeeClearanceListResponse.totalWays : 0}',
                              style: Theme.of(context).textTheme.headline5,
                            ),
                          ),
                        ),
                      )
                  ),
                ),
              ]
          ),
        ),
      )
  );

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => EmployeeClearanceListItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class EmployeeClearanceListItem extends StatefulWidget {

  EmployeeClearanceListItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final EmployeeCollectedItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _EmployeeClearanceListItemState createState() => _EmployeeClearanceListItemState();

}

class _EmployeeClearanceListItemState extends State<EmployeeClearanceListItem> {
  final TextEditingController _controller_remark = new TextEditingController();
  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.employeeName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      new DateFormat('dd/MM/yyyy').format(widget.post.wayDate.toLocal()),
                        // style: TextStyle(color: Colors.black),
                    ),
                    trailing: Text(
                      '${new NumberFormat("#,###").format(widget.post.collectedAmount)} Ks',
                      style: widget.post.status=='Clear'? Theme.of(context).textTheme.headline6
                          : TextStyle(color: Colors.red, fontSize: 18.0, fontStyle: FontStyle.normal),
                    ),
                    leading: Icon(widget.post.status=='Clear'?Icons.done_sharp : Icons.monetization_on,
                      // color: Colors.blueGrey[600],
                    ),
                    onTap: (){},
                  ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text(
                        "${tr(LocaleKeys.lblStatus)} ${widget.post.status=='Clear'?tr(LocaleKeys.lblClear) : tr(LocaleKeys.lblOnHand)}",
                        style: TextStyle(fontWeight: FontWeight.bold),
                      ),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 0),
                      child: Text(
                        "${tr(LocaleKeys.lblRemark)} ${widget.post.description??''}",
                        style: TextStyle(fontWeight: FontWeight.bold),
                      ),
                    ),
                    !widget.post.isClearNow && widget.post.status!='Clear'
                        && widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("MANAGER_READ")? ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Theme.of(context).primaryColorDark,
                          onPressed: ()async{
                            setState(() {
                              widget.post.isClearNow = true;
                            });
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblClearNow)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    ) : Container(),
                    widget.post.isClearNow? Container(
                        child: new Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          mainAxisSize: MainAxisSize.min,
                          children: <Widget>[
                            Container(
                              padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                              child: new TextFormField(
                                decoration: InputDecoration(
                                  hintText: '',
                                  labelText: '${tr(LocaleKeys.lblRemark)}*',
                                ),
                                validator: (val) => val.isEmpty ? 'Remark required' : null,
                                controller: _controller_remark,
                                keyboardType: TextInputType.text,
                                // maxLines: null,
                                // minLines: 1,
                              ),
                            ),
                            ButtonBar(
                              children: <Widget>[
                                RaisedButton.icon(
                                  elevation: 0.0,
                                  icon: Icon(Icons.done_all,
                                      // color: Colors.white
                                  ),
                                  color: Theme.of(context).primaryColorDark,
                                  onPressed: _processing? null : ()async{
                                    clear();
                                  },
                                  label: Text(
                                    '${tr(LocaleKeys.btnOK)}',
                                    style: TextStyle(
                                      fontSize: 16,
                                      fontWeight: FontWeight.w400,
                                      // color: Colors.white,
                                    ),
                                  ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                ),
                                RaisedButton.icon(
                                  elevation: 0.0,
                                  icon: Icon(Icons.cancel,
                                      // color: Colors.white
                                  ),
                                  color: Colors.red[700],
                                  onPressed: _processing? null : ()async{
                                    setState(() {
                                      widget.post.isClearNow = false;
                                    });
                                  },
                                  label: Text(
                                    '${tr(LocaleKeys.btnCancel)}',
                                    style: TextStyle(
                                      fontSize: 16,
                                      fontWeight: FontWeight.w400,
                                      // color: Colors.white,
                                    ),
                                  ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                ),
                              ],
                            )
                          ],
                        )
                    ) : Container(),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Future<void> clear( ) async{
    try{
      EmployeeCollectedService collectedService = new EmployeeCollectedService();
      setState(() {
        _processing = true;
      });
      Map<String, dynamic> request = {
        "guid": widget.post.guid,
        "description": _controller_remark.text,
        "approverGuid": widget.currentUser.guid
      };

      String response = await collectedService.clear(request);
      if( response == null) {
        showMessage('Error, processing failed!');
      };
      Map<String, dynamic> resultData = jsonDecode(response);
      if( resultData["success"]){
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        widget.onChange();
      }else{
        showMessage(resultData["message"]);
      }

    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    setState(() {
      _processing = false;
    });
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
