import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/os_deposit_refund_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/os_deposit_refund_list_response.dart';
import 'package:qq_delivery/delivery/service/deposit_service.dart';
import 'package:qq_delivery/delivery/service/refund_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'deposit_refund_filter.dart';


class OSDepositRefundList extends StatefulWidget {
  OSDepositRefundList({Key key}) : super(key: key);

  @override
  _OSDepositRefundListState createState() => _OSDepositRefundListState();
}

class _OSDepositRefundListState extends State<OSDepositRefundList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  bool _tapChange = false;
  OSDepositRefundListRequest request;


  @override
  void initState() {
    super.initState();
    _getLoadData();
  }

  void _getLoadData() async{
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();

      request = new OSDepositRefundListRequest(
          startDate: null,
          endDate: null,
          osGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("OS_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : '',
          branchGuid: null,
          type: 'all',
          first: 0,
          max: Constant.ROW_PER_PAGE,
          orderBy:'id',
          asc: false
      );
      setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.filter_alt_sharp, color: Colors.white,),
                  onPressed: () async{
                    Map<String, dynamic> result = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => DepositRefundFilter(
                            title: 'Filter',
                            type: _selectedIndex==0?'deposit' :'refund',
                            currentUser: _currentUser,
                            authorityResponse: _authorityResponse,
                            sharedPrefs: _sharedPrefs,
                          ),
                        )
                    );
                    if( result != null ){
                      String type = 'all';
                      if( result['entryOnly'] && !result['systemOnly']){
                        type = 'entry';
                      }else if( !result['entryOnly'] && result['systemOnly']){
                        type = 'generated';
                      }
                      request.startDate = result['startDate']??null;
                      request.endDate = result['endDate']??null;
                      request.type = type;
                      request.osGuid = result['osGuid']??'';
                      _onItemTapped(_selectedIndex);
                    }
                  },
                  tooltip: 'Filter',
                ),
              ),
              Container(
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.download_sharp, color: Colors.white,),
                  onPressed: () async{
                    String _accessToken = await _userService.getToken();
                    String uri = _selectedIndex==0?'deposit':'refund';
                    openBrowser(
                        Constant.BASE_URL_PRIMARY
                            +'/v1/forwarder/$uri/$_accessToken'
                            +'?osg='+ request.osGuid??''
                            +'&bg='+ request.branchGuid??''
                            +'&type='+ request.type??'all'
                            +'&sd='+ (request.startDate!=null? new DateFormat('yyyy-MM-dd').format(request.startDate.toLocal()): '')
                            +'&ed='+ (request.endDate!=null? new DateFormat('yyyy-MM-dd').format(request.endDate.toLocal()): '')
                    );
                  },
                  tooltip: 'Download',
                ),
              ),
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.work_rounded),
                label: tr(LocaleKeys.lblDeposit),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.monetization_on_sharp),
                label: tr(LocaleKeys.lblRefund),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            backgroundColor: Colors.white,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  Widget _createBody(){

    if( _selectedIndex == 0 && !_tapChange){
      return DepositRefundLoader(
        type: 'deposit',
        currentUser: _currentUser,
        authorityResponse: _authorityResponse,
        userService: _userService,
        sharedPrefs: _sharedPrefs,
        request: request,
      );
    }else if( _selectedIndex == 1 && !_tapChange){
      return DepositRefundLoader(
        type: 'refund',
        currentUser: _currentUser,
        authorityResponse: _authorityResponse,
        userService: _userService,
        sharedPrefs: _sharedPrefs,
        request: request,
      );
    }else{
      return new Container();
    }
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title= tr(LocaleKeys.lblDeposit);
      }else if( _selectedIndex == 1){
        _title= tr(LocaleKeys.lblRefund);
      }
    });

    new Timer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}


class DepositRefundLoader extends StatefulWidget {
  DepositRefundLoader({Key key, @required this.type, this.currentUser,
  this.authorityResponse, this.userService, this.sharedPrefs,
  this.request}) : super(key: key);
  final User currentUser;
  final AuthorityResponse authorityResponse;
  final UserService userService;
  final SharedPreferences sharedPrefs;
  final String type;
  final OSDepositRefundListRequest request;
  @override
  _DepositRefundLoaderState createState() => _DepositRefundLoaderState();
}

class _DepositRefundLoaderState extends State<DepositRefundLoader>{

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DepositService _depositService = new DepositService();
  RefundService _refundService = new RefundService();
  OSDepositRefundListResponse _depositRefundListResponse;
  List<DepositRefundItem> _items = [];
  int _page = 0;


  Future<void> _getLoadData( bool refresh ) async{

    widget.request.first = _page;
    widget.request.max = Constant.ROW_PER_PAGE;
    // OSDepositRefundListRequest request = new OSDepositRefundListRequest(
    //     startDate: DateTime(2020,07,1),
    //     endDate: DateTime(2021,02,1),
    //     osGuid: widget.authorityResponse.authorities!=null && widget.authorityResponse.authorities.contains("OS_READ") && !widget.authorityResponse.authorities.contains("MANAGER_READ")? widget.currentUser.guid : '',
    //     branchGuid: null,
    //     type: 'all',
    //     first: _page,
    //     max: Constant.ROW_PER_PAGE,
    //     orderBy:'id',
    //     asc: false
    // );
print('Request param= ${widget.request.toMap()}');
    if( widget.type == 'deposit'){
      _depositRefundListResponse = await _depositService.list( widget.request );
    }else{
      _depositRefundListResponse = await _refundService.list( widget.request );
    }

    if( _depositRefundListResponse !=null && _depositRefundListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _depositRefundListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => DepositRefundListItem(
        post: _items[i],
        authorityResponse: widget.authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: widget.currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: widget.sharedPrefs,
        type: widget.type,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }
}


class DepositRefundListItem extends StatefulWidget {

  DepositRefundListItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser, this.type}) : super(key: key);
  final DepositRefundItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String type;

  @override
  _DepositRefundListItemState createState() => _DepositRefundListItemState();

}

class _DepositRefundListItemState extends State<DepositRefundListItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.ownerName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      new DateFormat('dd/MM/yyyy hh:mm a').format(widget.post.updatedOn.toLocal()),
                        style: TextStyle(color: Colors.black),
                    ),
                    trailing: Text(
                      '${new NumberFormat("#,###").format(widget.post.amount)} Ks',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    onTap: (){},
                  ),
                  Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 16.0, bottom: 8),
                      child: Text(
                        "${widget.type=='deposit'? tr(LocaleKeys.lblPaidWith) : tr(LocaleKeys.lblRefundWith)}: ${widget.post.methods??''}",
                        style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                      ),
                  ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 16.0, bottom: 8),
                      child: Text(
                        "${tr(LocaleKeys.lblRemark)} ${widget.post.description}",
                        style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                      ),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 16.0, bottom: 16),
                      child: Text(
                        "${tr(LocaleKeys.lblApprovedBy)} ${widget.post.systemGenerated? tr(LocaleKeys.lblApprovedByName) : widget.post.approverName}",
                        style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold),
                      ),
                    ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
