import 'dart:async';
import 'dart:io';
import 'package:badges/badges.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/database/dao/notification_dao.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_count_by_status_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/request/employee_clearance_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/cod_balance_list_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_count_by_status_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_clearance_list_response.dart';
import 'package:qq_delivery/delivery/model/response/financial_center_merchant_list_response.dart';
import 'package:qq_delivery/delivery/model/response/mem_promotion_response.dart';
import 'package:qq_delivery/delivery/model/response/overdue_response.dart';
import 'package:qq_delivery/delivery/service/employee_collect_service.dart';
import 'package:qq_delivery/delivery/service/firebase_service.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/member_service.dart';
import 'package:qq_delivery/delivery/service/notification_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/settings_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/financial_center_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/warehouse_station_route_service.dart';
import 'package:qq_delivery/delivery/utils/advertising.dart';
import 'package:qq_delivery/delivery/view/user_validate.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:location/location.dart';
import 'package:qq_delivery/delivery/view/utils/connection_broken.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'app_drawer.dart';
import 'chat/chat_latest_list.dart';
import 'delivery/delivery_detail.dart';
import 'delivery/delivery_list_home.dart';
import 'delivery/delivery_list_home_customer.dart';
import 'delivery2/delivery_create_v2.dart';
import 'delivery2/pickup_list_home.dart';
import 'financial/employee_clearance_list.dart';
import 'login.dart';
import 'notification/notification_list_home.dart';
import 'package:flutter_gauge/flutter_gauge.dart';
import 'reports/active_way_count_by_town_list.dart';
import 'transit/transit_list_home.dart';
import 'utils/auto_completed_qr_scan_page.dart';
import 'utils/custom_refresh_indicator.dart';
import 'utils/qr_scan_page.dart';


class Home extends StatefulWidget {
  Home({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<Home> {

  FirebaseMessage _firebaseMessage = FirebaseMessage( );
  User _currentUser = new User();
  UserService userService = new UserService( );
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  NotificationDao notificationDao = new NotificationDao();
  SharedPreferences _sharedPrefs;
  int _notificationCount = 0;

  int _pickupCount = 0;
  int _pickupScheduleCount = 0;
  int _pickupOnWayCount = 0;
  int _pickupCompletedCount = 0;
  int _pickupCanceledCount = 0;

  int _dropOffCount = 0;
  int _dropOffScheduleCount = 0;
  int _dropOffOnWayCount = 0;
  int _completedCount = 0;
  int _canceledCount = 0;
  int _retryCount = 0;
  int _returnCount = 0;
  int _totalWays = 0;
  double _completedPercent = 90;
  bool _statusCountLoaded = false;
  bool isStopped = false;
  bool _notificationRefreshing = false;
  bool _isFirstOpen = true;
  double _osBalance = 0.0;
  double _promoBalance = 0.0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  Timer _timerNotificationRefresh;
  Timer _timerEmployeeTrack;
  OverdueResponse _overdueResponse;
  final _searchQuery = new TextEditingController();
  Timer _debounce;
  bool _processing = false;
  bool _processingError = false;
  DeliveryContentServiceV2 _deliveryContentService;

  int _inboundCount = 0;
  int _inboundScheduleCount = 0;
  int _inboundOnWayCount = 0;
  int _inboundRequestCount = 0;
  int _inboundBucketCount = 0;
  int _totalInboundCount = 0;

  int _outboundCount = 0;
  int _outboundScheduleCount = 0;
  int _outboundOnWayCount = 0;
  int _outboundRequestCount = 0;
  int _outboundBucketCount = 0;
  int _totalOutboundCount = 0;

  int _transitCount = 0;
  int _transitScheduleCount = 0;
  int _transitOnWayCount = 0;
  int _transitRequestCount = 0;
  int _takeAwayRequestCount = 0;
  int _transitBucketCount = 0;
  int _totalTransitCount = 0;
  bool _posPrivilege = false;
  double _totalCollected = 0.0;
  EmployeeCollectedService _employeeCollectedService = new EmployeeCollectedService();


  @override
  void initState() {
    super.initState();
    _timerNotificationRefresh = Timer.periodic(Duration(seconds: 3), (timer) {
      if (isStopped ) {timer.cancel();}
      try{
        _refreshNotifications();
      }catch(e){}
    });
    _searchQuery.addListener(_onSearchChanged);
  }

  @override
  void deactivate() {
    isStopped=true;
    super.deactivate( );
  }

  @override
  void dispose() {
    isStopped=true;
    if (_timerNotificationRefresh?.isActive ?? false) _timerNotificationRefresh.cancel();
    try{
      if (_timerEmployeeTrack?.isActive ?? false) _timerEmployeeTrack.cancel();
    }catch(e){}
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(milliseconds: 700), () {
      String searchText = _searchQuery.text.trim();
      if( searchText.length >=6 && !_processing){
        _search();
      }
    });
  }

  void _search() async{
    if( _processing ) return;
    setState(() {
      _processing = true;
      _processingError = false;
    });
    _debounce.cancel();

    if( _deliveryContentService == null)
      _deliveryContentService = new DeliveryContentServiceV2( );

    DeliveryDetailResponse _detailResponse = await _deliveryContentService.detail(_searchQuery.text.trim());
    if( _detailResponse == null){
      _processingError = true;
    }else{
      _processingError = false;
      FocusScope.of(context).unfocus();
      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => DeliveryDetail(
              wayId: _searchQuery.text.trim(),
              onChange: (){},
            ),
          )
      );
    }
    setState(() {
      _processing = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    double ratio = MediaQuery.of(context).size.width / MediaQuery.of(context).size.height;
    double gaugeHeight = 280;
    if( ratio >= 0.74 && ratio < 0.76)
      gaugeHeight = 470;
    else if( ratio <= 0.75 && ratio >0.74999)
      gaugeHeight = 400;
    else if( ratio < 0.74 && ratio >0.68)
      gaugeHeight = 400;
    else if( ratio > 0.6 && ratio <=0.68)
      gaugeHeight = 400;
    else if( ratio > 1.3)
      gaugeHeight = 400;

    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
          actions: <Widget>[
            Container(
                padding: EdgeInsets.all(16),
                child: Badge(
                  badgeContent: Text('$_notificationCount'),
                  toAnimate: true,
                  animationType: BadgeAnimationType.slide,
                  child: IconButton(
                    highlightColor: Colors.red,
                    icon: Icon(Icons.notifications_active, color: Colors.white,),
                    onPressed: () async{
                      _gotoNotifications();
                    },
                    tooltip: 'Notifications',
                  ),
                )
            ),
          ],
        ),
        body: SmartRefresher(
          controller: _refreshController,
          enablePullUp: false,
          child: _createBody( gaugeHeight ),
          physics: BouncingScrollPhysics(),
          header: CustomRefreshIndicator(),
          // scroll down
          onRefresh: () async {
            if( _isFirstOpen ){
              _loadProfile();
            }else{
              await _loadStatusCount();
              if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
                  && !_authorityResponse.authorities.contains("MANAGER_READ")){
                _updateOSBalance();
              }
            }
            if (mounted) setState(() {});
            _refreshController.refreshCompleted();
          },
        ),
        drawer: AppDrawer(
          currentUser: _currentUser,
          authorityResponse: _authorityResponse,
          sharedPrefs: _sharedPrefs,
          startLocation: _startLocation,
        ),
        floatingActionButton:
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("USER_WRITE") && _currentUser.roleName!=Constant.ROLE_NAME_EMPLOYEE ?
        FloatingActionButton.extended(
          onPressed: () async{
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (context) => DeliveryCreateV2(
                  onChange: (){
                    _loadStatusCount();
                  },
                ),
              ),
            );
            // Navigator.push(
            //   context,
            //   MaterialPageRoute(
            //     builder: (context) => ChatLatestList(
            //       sharedPrefs: _sharedPrefs,
            //       currentUser: _currentUser,
            //     ),
            //   ),
            // );
          },
          label: Text(
            tr(LocaleKeys.lblDeliCreateTitle),
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.add_circle_outline, color: Colors.white,),
          backgroundColor: Theme.of(context).primaryColor,
        ) : FloatingActionButton.extended(
          onPressed: () async{
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (context) => ActiveWayCountByTownList(),
              ),
            );
          },
          label: Text(
            tr(LocaleKeys.lblHomeGoingOut),
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.logout, color: Colors.white,),
          backgroundColor: Theme.of(context).primaryColor,
        ),
        bottomNavigationBar: _navigationBar()
    );
  }

  void _gotoList( BuildContext context, int index, String title, bool isPickup ) {
    if (_authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("OS_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ")) {
      Navigator.push(
        context,
        MaterialPageRoute(
          builder: (context) =>
              DeliveryListHomeCustomer(
                selectedIndex: index>2? index-1 : index,
                title: title,
                onChange: () {
                  _loadStatusCount();
                },
              ),
        ),
      );
    }else {
      if( isPickup ){
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) =>
                PickupListHome(
                  selectedIndex: index,
                  title: title,
                  onChange: () {
                    _loadStatusCount();
                  },
                ),
          ),
        );
      }else{
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) =>
                DeliveryListHome(
                  selectedIndex: index,
                  title: title,
                  onChange: () {
                    _loadStatusCount();
                  },
                ),
          ),
        );
      }
    }
  }

  void _loadProfile( )async{

    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    setState(() { });

    _loadStatusCount();
    try {
      if( await getFirebaseToken( ) == null)
        await _firebaseMessage.initMessaging( );
    }catch(e){ }

    // if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("EMPLOYEE_READ")){
    _initLocationService();
    // }
    try{
      if( !_currentUser.verified ){
        Timer(Duration(seconds: 4), () async{
          _showVerification( );
        });
      }
    }catch(e){print(e);}
    try{
      if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("EMPLOYEE_READ")
          && !_authorityResponse.authorities.contains("MANAGER_READ")){
        _timerEmployeeTrack = Timer.periodic(Duration(minutes: 1), (timer) {
          if (isStopped ) {timer.cancel();}
          try{
            _updateUserInfo();
          }catch(e){}
        });
      }else{
        Timer(Duration(seconds: 2), () async{
          _updateUserInfo();
        });
      }
    }catch(e){}

    if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ") ){
      _updateOSBalance();
    }
  }

  Future<void>  _loadStatusCount( ) async{
    try{
      String accessToken = await userService.getToken();
      DeliveryContentServiceV2 contentService = new DeliveryContentServiceV2();
      PickupServiceV2 pickupServiceV2 = new PickupServiceV2();
      DeliveryCountByStatusRequest request = new DeliveryCountByStatusRequest(filter: Constant.SHOW_CREATED_WAYS_TO_DELIVERYMAN? 'all' : null );
      // request.branchGuid = _currentUser.branchGuid;
      // extracting delivery info
      DeliveryCountByStatusResponse _response = await contentService.countByStatus(accessToken, request);

      if( _response.statusCode == Constant.HTTP_OK || _response.statusCode == Constant.HTTP_CREATE ){

        // setState(() {_statusCountLoaded = false;});

        _totalWays = 0;
        _response.items.forEach((element) {
          if( element.status == Constant.STATUS_CREATED){
            _dropOffCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_ASSIGNED){
            _dropOffScheduleCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_ON_WAY){
            _dropOffOnWayCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_COMPLETED){
            _completedCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_CANCELED){
            _canceledCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_RETRY){
            _retryCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_RETURN){
            _returnCount = element.total;
            _totalWays += element.total;
          }
        });

        // extracting pickup info
        _response = await pickupServiceV2.countByStatus(accessToken, request);
        _response.items.forEach((element) {
          if( element.status == Constant.STATUS_PICKUP_CREATED){
            _pickupCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_PICKUP_SCHEDULED){
            _pickupScheduleCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_PICKUP_ON_WAY){
            _pickupOnWayCount = element.total;
            // _totalWays += element.total;
          }

          else if( element.status == Constant.STATUS_COMPLETED){
            _pickupCompletedCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_CANCELED){
            _pickupCanceledCount = element.total;
            // _totalWays += element.total;
          }
        });

        if( _currentUser.roleName =='Employee')
          _overdueResponse = await contentService.overdueCount(_currentUser.guid);
        else
          _overdueResponse = await contentService.overdueCount( );

        setState(() {
          _completedPercent = _completedCount>0? ( (_completedCount/_totalWays) *100 ).toDouble() : 0.0;
          _statusCountLoaded = true;
        });

        // loading inbound/outbound counts
        if( _authorityResponse.authorities != null
            && _authorityResponse.authorities.contains("MANAGER_READ"))
          _loadInboundOutboundCount();

        // loading transit counts
        if( _authorityResponse.authorities != null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")
            && !_authorityResponse.authorities.contains("MANAGER_READ"))
          _loadTransitCount();

      }else if( _response.statusCode == Constant.HTTP_DENIED ){
        await userService.logout( );
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => LoginUser(
                title: tr(LocaleKeys.login),
              ),
            )
        );
      }else{
        String status = await testConnection("/v1/state/list");
        if( status == Constant.CONNECTION_REFUSED){
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => ConnectionBroken(),
              )
          );
        }
        // else if( status == Constant.CONNECTION_OK ){
        //   await userService.logout( );
        //   Navigator.pushReplacement(
        //       context,
        //       MaterialPageRoute(
        //         builder: (BuildContext context) => LoginUser(
        //           title: tr(LocaleKeys.login),
        //         ),
        //       )
        //   );
        // }
      }
    }catch(e){}
  }

  Future<void> _updateUserInfo() async{
    try{
      PersonService personService = new PersonService();
      await personService.updateKeyLatLon(
          {
            'firebaseKey':await getFirebaseToken( ),
            'latitude': _startLocation!=null? _startLocation.latitude : null,
            'longitude': _startLocation!=null? _startLocation.longitude : null,
            'platform': Platform.isAndroid? 'Android' : 'iOS'
          }
      );
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  Future<void> _updateOSBalance() async{
    try{
      FinancialCeterServiceV2 financialCeterServiceV2 = new FinancialCeterServiceV2();
      DeliveryListRequest _listRequest = new DeliveryListRequest();
      _listRequest = new DeliveryListRequest(
          keyword: '',
          startDate: null,
          endDate: null,
          osGuid: _currentUser.guid,
          customerGuid: null,
          employeeGuid: null,
          zoneGuid: null,
          branchGuid: null,
          status: null,
          first: 0,
          max: 1000000,
          orderBy: 'payable.payableAmount',
          asc: false
      );

      UserService _userService = UserService();
      String accessToken = await _userService.getToken();
      FinancialCenterMerchantListResponse merchantListResponse = await financialCeterServiceV2.merchantList(accessToken, _listRequest );

      double toRefund = 0;
      if( merchantListResponse !=null && merchantListResponse.items !=null ){
        merchantListResponse.items.forEach((element) {
          if( element.balanceAmount != null){
            //toRefund += (contentItem.payableAmount? contentItem.payableAmount :0) - ((contentItem.receivableAmount? contentItem.receivableAmount :0) + (contentItem.advanceAmount? contentItem.advanceAmount :0));
            toRefund += element.balanceAmount;
          }
        });
        _osBalance  = toRefund;
      }
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    await _updatePOSSettings();
  }

  Future<void> _updatePOSSettings( ) async{
    try{
      SettingsService settingsService = new SettingsService( );
      bool result = await settingsService.localSetting( _currentUser.guid );
      setState(() {
        _posPrivilege = result;
      });
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  Future<void> _updatePromoBalance( ) async{
    try{
      MemberService memberService = new MemberService( );
      MemberPromotionResponse balanceResponse = await memberService.promoBalance( _currentUser.guid );
      if( balanceResponse != null && balanceResponse.items!=null){
        if( balanceResponse.items.isNotEmpty){
          MemberPromotionItem balanceItem = balanceResponse.items[0];
          _promoBalance = balanceItem.balance;
        }
      }
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  Future<void> _showVerification( )async {

    String result = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => UserValidation(
            phoneNumber: _currentUser.phone,
          ),
        )
    );
    if( result!=null && result=='Verified' ){
      await userService.getUserInformation();
      await _currentUser.fromSharedPreferences( );
    }else{
      await userService.logout();
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => LoginUser(
              title: tr(LocaleKeys.login),
            ),
          )
      );
    }
  }

  void _refreshNotifications( )async{
    if( _notificationRefreshing ) return;
    _notificationRefreshing = true;

    if( _isFirstOpen ) {
      NotificationService notificationService = new NotificationService();
      bool hasUpdate = await notificationService.fetchAndUpdate( );
      _isFirstOpen = false;
    }

    await notificationDao.open( );
    int count = await notificationDao.unreadCount();
    await notificationDao.close();

    if( count != _notificationCount && mounted){
      _notificationCount = count;
      _loadStatusCount( );
    }

    _notificationRefreshing = false;
    if( mounted )
      setState(() {});
  }

  void _gotoNotifications() async{
    await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => NotificationListHome( ),
        )
    );
  }

  Location _locationService  = new Location();
  bool _permission = false;
  LocationData _currentLocation;
  LocationData _startLocation;
  String error;
  _initLocationService() async {
    LocationData location;
    try {

      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
            // print('Lat: ${_startLocation.latitude}, Lon:${_startLocation.longitude}');
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }
    _startLocation = location;
  }


  int _selectedIndex = 0;
  BottomNavigationBar _navigationBar(){

    if( _authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ")){
      return BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.download_sharp),
            label: tr(LocaleKeys.lblMenuPickupWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.upload_sharp),
            label: tr(LocaleKeys.lblMenuDeliverWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.transfer_within_a_station_sharp),
            label: tr(LocaleKeys.lblStationInOut),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.chat),
            label: 'Chat',
          ),
        ],
        currentIndex: _selectedIndex,
        selectedItemColor: Theme.of( context ).primaryColor,
        unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        // backgroundColor: Theme.of(context).primaryColor,
        onTap: _onItemTappedDeliveryman,
      );
    }else if( _authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("MANAGER_READ")){
      return BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.download_sharp),
            label: tr(LocaleKeys.lblMenuPickupWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.upload_sharp),
            label: tr(LocaleKeys.lblMenuDeliverWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.input),
            label: tr(LocaleKeys.mnuInbound),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.send_outlined),
            label: tr(LocaleKeys.mnuOutbound),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.chat),
            label: 'Chat',
          ),
        ],
        currentIndex: _selectedIndex,
        selectedItemColor: Theme.of( context ).primaryColor,
        unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        // backgroundColor: Theme.of(context).primaryColor,
        onTap: _onItemTappedManager,
      );
    }else{
      // merchant
      return BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.home_sharp),
            label: tr(LocaleKeys.mnuHome),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.chat),
            label: 'Chat',
          ),
        ],
        currentIndex: _selectedIndex,
        selectedItemColor: Theme.of( context ).primaryColor,
        unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        // backgroundColor: Theme.of(context).primaryColor,
        onTap: _onItemTappedOther,
      );
    }
  }

  void _onItemTappedManager(int index) {
    setState(() {
      _selectedIndex = index;
      if( _selectedIndex == 4){
        _selectedIndex = 0;
        Navigator.push(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => ChatLatestList(
                currentUser: _currentUser,
                sharedPrefs: _sharedPrefs,
              ),
            )
        );
      }
    });
  }
  void _onItemTappedDeliveryman(int index) {
    setState(() {
      _selectedIndex = index;
      if( _selectedIndex == 3){
        _selectedIndex = 0;
        Navigator.push(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => ChatLatestList(
                currentUser: _currentUser,
                sharedPrefs: _sharedPrefs,
              ),
            )
        );
      }
    });
  }
  void _onItemTappedOther(int index) {
    setState(() {
      _selectedIndex = index;
      if( _selectedIndex == 1){
        _selectedIndex = 0;
        Navigator.push(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => ChatLatestList(
                currentUser: _currentUser,
                sharedPrefs: _sharedPrefs,
              ),
            )
        );
      }
    });
  }

  Widget _createBody( double gaugeHeight ){
    if( _authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("MANAGER_READ") ){
      if( _selectedIndex == 0 ){
        return _bodyAdminPickup( );
      }else if( _selectedIndex == 1){
        return _bodyAdminDeliver( gaugeHeight );
      }else if( _selectedIndex == 2){
        return _bodyInbound();
      }else if( _selectedIndex == 3){
        return _bodyOutbound();
      }else if( _selectedIndex == 4){
        return Container();
      }

    }else if( _authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ")){
      if( _selectedIndex == 0 ){
        return _bodyDeliverymanPickup( );
      }else if( _selectedIndex == 1){
        return _bodyDeliverymanDeliver( gaugeHeight );
      }else if( _selectedIndex == 2){
        return _bodyTransit();
      }else if( _selectedIndex == 3){
        return Container();
      }
    }else {
      if( _selectedIndex == 0 ){
        return _bodyOther( gaugeHeight );
      }else if( _selectedIndex == 1){
        return Container();
      }
    }
  }

  Widget _searchBar() {
    return Column(
        children: <Widget>[
          Container(
            padding: EdgeInsets.only(top: 16, left: 16, bottom: 0, right: 16),
            child: TextField(
              textAlign: TextAlign.center,
              keyboardType: TextInputType.text,
              decoration: InputDecoration(
                hintText: tr(LocaleKeys.lblEnterWayIDToTrack),
                // fillColor: Colors.white,
                hintStyle: TextStyle(fontSize: 12),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(8),
                  borderSide: BorderSide(
                    width: 1,
                    style: BorderStyle.none,
                  ),
                ),
                filled: true,
                contentPadding: EdgeInsets.all(8),
                // fillColor: Theme.of(context).backgroundColor,
                suffixIcon: _searchQuery.text.length>0? IconButton(
                  icon: Icon(Icons.close_sharp,
                    // color: Theme.of(context).primaryColor
                  ),
                  onPressed: (){
                    setState(() {
                      _searchQuery.text = '';
                      _processingError = false;
                    });
                  },
                ): IconButton(
                  icon: Icon(Icons.qr_code_sharp),
                  onPressed: () async{
                    await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => QRScanPage( ),
                        )
                    );
                  },
                ),
              ),
              controller: _searchQuery,
              style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
            ),
          ),
          _processingError?Container(
            child: Text('Invalid wayID', style: TextStyle(color: Colors.red),),
          ):Container(),
        ]
    );
  }

  Widget _bodyAdminPickup( ){
    return ListView(
      children: <Widget>[
        _searchBar(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("MANAGER_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_ind_sharp, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.green[800], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done, size: 32, color: Colors.white ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCompletedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.red, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        Advertising(),
/*
        _currentUser.roleName == Constant.ROLE_NAME_MANAGER && _totalCollected>0? Wrap(
          children: <Widget>[
            Container(
              // width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                // height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Theme.of(context).primaryColor,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_totalCollected)} Ks',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblEmployeeToTransfer),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                      onTap: () {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) => EmployeeClearanceList(),
                            )
                        );
                      },
                    )
                )
            ),
          ],
        ) : Container(),

        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 32),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 32),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.download_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
          ],
        ),

        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
          ],
        ),

        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_pickupCompletedCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '${tr(LocaleKeys.lblPickupCompleted)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      // subtitle: Text("Total return ways", style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.done_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            )
        ),
        _overdueResponse!=null? Wrap(
          children: <Widget>[
            Container(
              //width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 8),
                child: Card(
                    elevation: 0,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overduePickup}${tr(LocaleKeys.lblHomeWays)}',
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomePickupOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      leading: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () { /*TODO goto list view and filter with date*/},
                    )
                )
            ),
          ],
        ) : Container(),
*/
      ],
    );
  }

  Widget _bodyAdminDeliver( double gaugeHeight ){
    return ListView(
      children: <Widget>[
        _searchBar(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("MANAGER_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_ind_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 1, tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        // Cancel
        Wrap(
          children: <Widget>[
            _authorityResponse.authorities!=null
                && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                      },
                    )
                )
            ): Container(),
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.green[800], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_completedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Completed
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.yellow[700], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.red, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                  leading:ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.red, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    '$_returnCount',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblReturn),
                    // style: TextStyle(color: Colors.black)
                  ),
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                  },
                )
            )
        ),
        Advertising()
        /*
        _currentUser.roleName == Constant.ROLE_NAME_MANAGER && _totalCollected>0? Wrap(
          children: <Widget>[
            Container(
              // width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                // height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Theme.of(context).primaryColor,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_totalCollected)} Ks',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblEmployeeToTransfer),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                      onTap: () {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) => EmployeeClearanceList(),
                            )
                        );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 32),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 32),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.upload_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                      },
                    )
                )
            ),
          ],
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),

          ],
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_return_sharp,
                                    size: 32,
                                    color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_returnCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblReturn),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                      },
                    )
                )
            ),
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_completedCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '${tr(LocaleKeys.lblCompleted)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      // subtitle: Text("Total return ways", style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.done_all_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                      },
                    ))
            )
        ),
        _overdueResponse!=null? Wrap(
          children: <Widget>[
            Container(
              // width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 64, top: 8),
                child: Card(
                    elevation: 0,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overdueDeliver}${tr(LocaleKeys.lblHomeWays)}',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeDeliverOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      leading: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () {/*TODO goto list view and filter with date*/ },
                    )
                )
            ),
          ],
        ) : Container(),
        */
      ],
    );
  }

  Widget _bodyDeliverymanPickup( ){
    return ListView(
      children: <Widget>[
        _searchBar(),
        Wrap(
          children: <Widget>[
            Container(
              // width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                // height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Theme.of(context).primaryColor,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_totalCollected)} Ks',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblEmployeeToTransfer),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                      onTap: () {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) => EmployeeClearanceList(),
                            )
                        );
                      },
                    )
                )
            ),
          ],
        ),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("MANAGER_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_ind_sharp, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.green[800], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done, size: 32, color: Colors.white ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCompletedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.red, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        Advertising(),

    /*
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.upload_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblToPickup),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
          ],
        ),
        // Cancel
        _overdueResponse!=null?  Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 0,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overduePickup}${tr(LocaleKeys.lblHomeWays)}',
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomePickupOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      leading: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () { /*TODO goto list view and filter with date*/},
                    )
                )
            ),
          ],
        ) : Container(),
        // Completed
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child: ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_pickupCompletedCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '${tr(LocaleKeys.lblPickupCompleted)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: Icon(Icons.done_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            )
        ),
    */
      ],
    );
  }

  Widget _bodyDeliverymanDeliver( double gaugeHeight ){
    return ListView(
      children: <Widget>[
        _searchBar(),
        Wrap(
          children: <Widget>[
            Container(
              // width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                // height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Theme.of(context).primaryColor,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_totalCollected)} Ks',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblEmployeeToTransfer),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                      onTap: () {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) => EmployeeClearanceList(),
                            )
                        );
                      },
                    )
                )
            ),
          ],
        ),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("MANAGER_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Colors.white
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_ind_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 1, tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        // Cancel
        Wrap(
          children: <Widget>[
            _authorityResponse.authorities!=null
                && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).primaryColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                      },
                    )
                )
            ): Container(),
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.green[800], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_completedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Completed
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.yellow[700], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.red, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                  leading:ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.red, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    '$_returnCount',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblReturn),
                    // style: TextStyle(color: Colors.black)
                  ),
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                  },
                )
            )
        ),
        Advertising()

        /*
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.download_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Cancel
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),
          ],
        ),
        _overdueResponse!=null? Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_return_sharp,
                                    size: 32,
                                    color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_returnCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblReturn),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 0,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overdueDeliver}${tr(LocaleKeys.lblHomeWays)}',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeDeliverOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      leading: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () {/*TODO goto list view and filter with date*/ },
                    )
                )
            ),
          ],
        ) : Container(),
        Constant.SHOW_CREATED_WAYS_TO_DELIVERYMAN? Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 16, top: 16),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  leading:ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Theme.of(context).backgroundColor, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: Icon(Icons.view_list_rounded, size: 32,
                                color: Theme.of(context).primaryColor
                            ),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    '$_dropOffCount',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                    // style: TextStyle(color: Colors.black)
                  ),
                  onTap: () {
                    _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                  },
                )
            )
        ) : Container(),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 72, top: 8),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_completedCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '${tr(LocaleKeys.lblCompleted)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      // subtitle: Text("Total return ways", style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.done_all_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                      },
                    ))
            )
        ),
        */
      ],
    );
  }

  Widget _bodyOther( double gaugeHeight ){
    return ListView(
      children: <Widget>[
        _searchBar(),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_osBalance)} Ks',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblMainBalance),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                    )
                )
            ),
            /*Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_promoBalance)} Ks',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPromotionBalance),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.redeem,
                        // color: Colors.blueGrey[600],
                      ),
                      // onTap: () {
                      //   // _gotoList( context, 0, tr(LocaleKeys.lblQueue) );
                      // },
                    )
                )
            ),*/
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
            child: Card(
                elevation: 8,
                child: ListTile(
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        child: InkWell(
                          child: new Container(
                            // width: 60.0,
                            // height: 60.0,
                            padding: const EdgeInsets.all(12.0),
                            decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              color: Colors.yellow[700],
                            ),
                            child: new Text(
                              '${_pickupCount + _pickupScheduleCount + _pickupOnWayCount + _dropOffCount + _dropOffScheduleCount + _dropOffOnWayCount}',
                              style: Theme.of(context).textTheme.headline5,
                            ),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    tr(LocaleKeys.lblTrackTheWays),
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblPickupDeliveryOnWay),
                    // style: TextStyle(color: Colors.black)
                  ),// Active ways
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 0, tr(LocaleKeys.lblDeliver), false );
                  },
                )
            )
        ),
        // Cancel
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.green[800], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_completedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Completed
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.yellow[700], // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.9999,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.red, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.arrow_forward_sharp,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                  leading:ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.red, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    '$_returnCount',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblReturn),
                    // style: TextStyle(color: Colors.black)
                  ),
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                  },
                )
            )
        ),
        Advertising()

/*
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/cancel.svg', semanticsLabel: 'Cancel',height: 48 ),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        // color: Theme.of(context).backgroundColor,
                        child: InkWell(
                          // splashColor: Theme.of(context).backgroundColor,
                          child: new Container(
                            // width: 60.0,
                            // height: 60.0,
                            padding: const EdgeInsets.all(12.0),
                            decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              // borderRadius: new BorderRadius.circular(30.0),
                              color: Colors.yellow[700],
                            ),
                            child: new Text(
                              '${_pickupCount + _pickupScheduleCount + _pickupOnWayCount + _dropOffCount + _dropOffScheduleCount + _dropOffOnWayCount}',
                              style: Theme.of(context).textTheme.headline5,
                            ),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    tr(LocaleKeys.lblTrackTheWays),
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblPickupDeliveryOnWay),
                    // style: TextStyle(color: Colors.black)
                  ),// Active ways
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 0, tr(LocaleKeys.lblDeliver), false );
                  },
                )
            )
        ),
        // Deliver completed gauge
        Container(
            height: gaugeHeight,
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
            child: Card(
              elevation: 8,
              // color: Colors.white,
              child: Wrap(
                children: <Widget>[
                  ListTile(
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          child: InkWell(
                            // splashColor: Theme.of(context).backgroundColor,
                            child: new Container(
                              // width: 60.0,
                              // height: 60.0,
                              padding: const EdgeInsets.all(12.0),
                              decoration: new BoxDecoration(
                                shape: BoxShape.circle,
                                // borderRadius: new BorderRadius.circular(30.0),
                                color: Theme.of(context).primaryColor,
                              ),
                              child: new Text(
                                '$_completedCount',
                                style: Theme.of(context).textTheme.headline5,
                              ),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      '${tr(LocaleKeys.lblCompleted)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    // subtitle: Text("Total successfully completed", style: TextStyle(color: Colors.black)),
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                    },
                  ),
                  _statusCountLoaded? Container(
                      margin: EdgeInsets.only(top: 8),
                      child:FlutterGauge(
                        width: 200,
                        index:_completedPercent,
                        fontFamily: "Iran",
                        counterStyle: TextStyle(
                          color: Colors.black,
                          fontSize: 32,),
                        numberInAndOut: NumberInAndOut.outside,
                        counterAlign: CounterAlign.center,
                        secondsMarker: SecondsMarker.secondsAndMinute,
                        hand: Hand.short,
                        handColor: Theme.of(context).primaryColor,
                        circleColor: Theme.of(context).backgroundColor,
                        activeColor: Theme.of(context).primaryColorDark,
                      )
                  ) : Container(),
                ],
              ),
            )
        ),
        // Completed
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_outline_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCompletedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.assignment_return_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_returnCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblReturn),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Cancel
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 64, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
          ],
        ),
        */
      ],
    );
  }

  Widget _bodyOld( double gaugeHeight ){
    return ListView(
      children: <Widget>[
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("OS_READ")
            && !_authorityResponse.authorities.contains("MANAGER_READ")? Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_osBalance)} Ks',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblMainBalance),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.monetization_on_outlined,
                        // color: Colors.blueGrey[600],
                      ),
                      // onTap: () {
                      //   try{
                      //     Navigator.push(
                      //         context,
                      //         MaterialPageRoute(
                      //           builder: (BuildContext context) => OSDepositRefundList(),
                      //         )
                      //     );
                      //   }catch(e){}
                      // },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${new NumberFormat("#,###").format(_promoBalance)} Ks',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPromotionBalance),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.redeem,
                        // color: Colors.blueGrey[600],
                      ),
                      // onTap: () {
                      //   // _gotoList( context, 0, tr(LocaleKeys.lblQueue) );
                      // },
                    )
                )
            ),
          ],
        ) : Container(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("OS_READ")
            && !_authorityResponse.authorities.contains("MANAGER_READ")?Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/cancel.svg', semanticsLabel: 'Cancel',height: 48 ),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        // color: Theme.of(context).backgroundColor,
                        child: InkWell(
                          // splashColor: Theme.of(context).backgroundColor,
                          child: new Container(
                            // width: 60.0,
                            // height: 60.0,
                            padding: const EdgeInsets.all(12.0),
                            decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              // borderRadius: new BorderRadius.circular(30.0),
                              color: Colors.yellow[700],
                            ),
                            child: new Text(
                              '${_pickupCount + _pickupScheduleCount + _pickupOnWayCount + _dropOffCount + _dropOffScheduleCount + _dropOffOnWayCount}',
                              style: Theme.of(context).textTheme.headline5,
                            ),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    tr(LocaleKeys.lblTrackTheWays),
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(tr(LocaleKeys.lblPickupDeliveryOnWay),
                    // style: TextStyle(color: Colors.black)
                  ),// Active ways
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 0, tr(LocaleKeys.lblDeliver), false );
                  },
                )
            )
        )
            : Container(),

        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")
            && _overdueResponse!=null? Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 1,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overduePickup}${tr(LocaleKeys.lblHomeWays)}',
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomePickupOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      trailing: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () { /*TODO goto list view and filter with date*/},
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 1,
                    // color: Colors.white,
                    child: ListTile(
                      title: Text(
                        '${_overdueResponse.overdueDeliver}${tr(LocaleKeys.lblHomeWays)}',
                        // style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeDeliverOverdue),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      trailing: Icon(Icons.warning,
                        // color: Theme.of(context).primaryColor
                      ),
                      // onTap: () {/*TODO goto list view and filter with date*/ },
                    )
                )
            ),
          ],
        ) : Container(),

        // Deliver completed gauge
        Container(
            height: gaugeHeight,
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
            child: Card(
              elevation: 8,
              // color: Colors.white,
              child: Wrap(
                children: <Widget>[
                  ListTile(
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          child: InkWell(
                            // splashColor: Theme.of(context).backgroundColor,
                            child: new Container(
                              // width: 60.0,
                              // height: 60.0,
                              padding: const EdgeInsets.all(12.0),
                              decoration: new BoxDecoration(
                                shape: BoxShape.circle,
                                // borderRadius: new BorderRadius.circular(30.0),
                                color: Theme.of(context).primaryColor,
                              ),
                              child: new Text(
                                '$_completedCount',
                                style: Theme.of(context).textTheme.headline5,
                              ),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      '${tr(LocaleKeys.lblCompleted)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    // subtitle: Text("Total successfully completed", style: TextStyle(color: Colors.black)),
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                    },
                  ),
                  _statusCountLoaded? Container(
                      margin: EdgeInsets.only(top: 8),
                      child:FlutterGauge(
                        width: 200,
                        index:_completedPercent,
                        fontFamily: "Iran",
                        counterStyle: TextStyle(
                          color: Colors.black,
                          fontSize: 32,),
                        numberInAndOut: NumberInAndOut.outside,
                        counterAlign: CounterAlign.center,
                        secondsMarker: SecondsMarker.secondsAndMinute,
                        hand: Hand.short,
                        handColor: Theme.of(context).primaryColor,
                        circleColor: Theme.of(context).backgroundColor,
                        activeColor: Theme.of(context).primaryColorDark,
                      )
                  ) : Container(),
                ],
              ),
            )
        ),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("MANAGER_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.upload_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.download_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblSchedule)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 1, tr(_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        _authorityResponse.authorities!=null
            && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.local_shipping_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 2, tr(!_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                      },
                    )
                )
            ),
          ],
        ) : Container(),
        // Cancel
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCanceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_canceledCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                      },
                    )
                )
            ),
          ],
        ),
        // Completed
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_outline_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_pickupCompletedCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                        // style: TextStyle(color: Colors.black)
                      ),// if Sender = လာယူနေ
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.refresh, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_retryCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                        // style: TextStyle(color: Colors.black)
                      ),
                      // subtitle: Text("${tr(LocaleKeys.lblOnWay)}", style: TextStyle(color: Colors.black)),
                      // trailing: Icon(Icons.arrow_forward_sharp, color: Colors.blueGrey[600],),
                      onTap: () {
                        _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                      },
                    )
                )
            ),
          ],
        ),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 64, top: 8),
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: ListTile(
                  // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        child: InkWell(
                          // splashColor: Theme.of(context).backgroundColor,
                          child: new Container(
                            // width: 60.0,
                            // height: 60.0,
                            padding: const EdgeInsets.all(12.0),
                            decoration: new BoxDecoration(
                              shape: BoxShape.circle,
                              // borderRadius: new BorderRadius.circular(30.0),
                              color: Colors.red,
                            ),
                            child: new Text(
                              '$_returnCount',
                              style: Theme.of(context).textTheme.headline5,
                            ),
                          ),
                        ),
                      )
                  ),
                  title: Text(
                    '${tr(LocaleKeys.lblReturn)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  // subtitle: Text("Total return ways", style: TextStyle(color: Colors.black)),
                  trailing: Icon(Icons.arrow_forward_sharp,
                    // color: Colors.blueGrey[600]
                  ),
                  onTap: () {
                    _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                  },
                )
            )
        ),
        // Advertising()
      ],
    );
  }

  Widget _bodyInbound( ){
    return ListView(
      children: <Widget>[
        // _searchBar(),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 32),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.transparent,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_inboundRequestCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(tr(LocaleKeys.lblInboundRequest),
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: Icon(Icons.input,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoTransit( context, 3, tr(LocaleKeys.lblInboundRequest), 'inbound');
                      },
                    )
                )
            )
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_inboundCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblInboundToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 0, tr(LocaleKeys.lblInboundToAssign), 'inbound');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.download_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_inboundScheduleCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblInboundScheduled),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 1, tr(LocaleKeys.lblInboundScheduled), 'inbound');
                      },
                    )
                )
            ),
          ],
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.directions_bike_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_inboundOnWayCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblInboundOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 2, tr(LocaleKeys.lblInboundOnWay), 'inbound');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.shopping_bag_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_inboundBucketCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text( tr(LocaleKeys.lblInboundBucket),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 4, tr(LocaleKeys.lblInboundBucket), 'inbound');
                      },
                    )
                )
            ),
          ],
        ),
        Center(
          child: Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 16),
              child: Card(
                  elevation: 1,
                  // color: Colors.white,
                  child: Center(
                      child:ListTile(
                        title: Text(tr(LocaleKeys.lblInboundGrantByQR),
                          style: Theme.of(context).textTheme.headline6,
                          textAlign: TextAlign.center,
                        ),
                        subtitle: Icon(Icons.qr_code_sharp, size: 150,
                          // color: Colors.blueGrey[600]
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp),
                        onTap: () {
                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => AutoCompletedQRScanPage( ),
                              )
                          );
                        },
                      )
                  )
              )
          ),
        )
      ],
    );
  }

  Widget _bodyOutbound( ){
    return ListView(
      children: <Widget>[
        // _searchBar(),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 32),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_outboundRequestCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(tr(LocaleKeys.lblOutboundRequest),
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: Icon(Icons.send_outlined,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoTransit( context, 3, tr(LocaleKeys.lblOutboundRequest), 'outbound');
                      },
                    )
                )
            )
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.view_list_rounded, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_outboundCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundToAssign),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 0, tr(LocaleKeys.lblOutboundToAssign), 'outbound');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.upload_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_outboundScheduleCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundScheduled),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 1, tr(LocaleKeys.lblOutboundScheduled), 'outbound');
                      },
                    )
                )
            ),
          ],
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.directions_bike_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_outboundOnWayCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 2, tr(LocaleKeys.lblOutboundOnWay), 'outbound');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_outboundBucketCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundBucket),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 4, tr(LocaleKeys.lblOutboundBucket), 'outbound');
                      },
                    )
                )
            ),
          ],
        ),
        Center(
          child: Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 16),
              child: Card(
                  elevation: 1,
                  // color: Colors.white,
                  child: Center(
                      child:ListTile(
                        title: Text(tr(LocaleKeys.lblOutboundGrantByQR),
                          style: Theme.of(context).textTheme.headline6,
                          textAlign: TextAlign.center,
                        ),
                        subtitle: Icon(Icons.qr_code_sharp, size: 150,
                          // color: Colors.blueGrey[600]
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp),
                        onTap: () {
                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => AutoCompletedQRScanPage( ),
                              )
                          );
                        },
                      )
                  )
              )
          ),
        )
      ],
    );
  }

  Widget _bodyTransit( ){
    return ListView(
      children: <Widget>[
        // _searchBar(),
        Container(
            margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 32),
            height: 96.0,
            child: Card(
                elevation: 8,
                // color: Colors.white,
                child: Center(
                    child:ListTile(
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            child: InkWell(
                              // splashColor: Theme.of(context).backgroundColor,
                              child: new Container(
                                // width: 60.0,
                                // height: 60.0,
                                padding: const EdgeInsets.all(16.0),
                                decoration: new BoxDecoration(
                                  shape: BoxShape.circle,
                                  // borderRadius: new BorderRadius.circular(30.0),
                                  color: Theme.of(context).primaryColor,
                                ),
                                child: new Text(
                                  '$_takeAwayRequestCount',
                                  style: Theme.of(context).textTheme.headline5,
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(tr(LocaleKeys.lblOutboundRequested),
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      trailing: Icon(Icons.send_outlined,
                        // color: Colors.blueGrey[600]
                      ),
                      onTap: () {
                        _gotoTransit( context, 3, tr(LocaleKeys.lblOutboundRequested), 'deliveryman');
                      },
                    )
                )
            )
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.playlist_add_check, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '${_transitScheduleCount}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblHomeAdminToAccept),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 1, tr(LocaleKeys.lblHomeAdminToAccept), 'deliveryman');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.directions_bike_sharp, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_transitOnWayCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundOnWay),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 2, tr(LocaleKeys.lblOutboundOnWay), 'deliveryman');
                      },
                    )
                )
            ),
          ],
        ),
        Wrap(
          children: <Widget>[
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.input, size: 32,
                                    color: Theme.of(context).primaryColor
                                ),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_transitRequestCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblInboundRequested),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 2, tr(LocaleKeys.lblInboundRequested), 'deliveryman');
                      },
                    )
                )
            ),
            Container(
                width: MediaQuery.of(context).size.width * 0.46,
                margin: EdgeInsets.only(left: 8, right: 8, bottom: 8, top: 16),
                height: 96.0,
                child: Card(
                    elevation: 8,
                    // color: Colors.white,
                    child: ListTile(
                      leading:ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Theme.of(context).backgroundColor, // button color
                            child: InkWell(
                              // splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: Icon(Icons.done_sharp, size: 32, color: Theme.of(context).primaryColor),
                              ),
                            ),
                          )
                      ),
                      title: Text(
                        '$_transitBucketCount',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text(tr(LocaleKeys.lblOutboundBucket),
                        // style: TextStyle(color: Colors.black)
                      ),
                      onTap: () {
                        _gotoTransit( context, 4, tr(LocaleKeys.lblOutboundBucket), 'deliveryman');
                      },
                    )
                )
            ),
          ],
        ),
        Center(
          child: Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 32, top: 16),
              child: Card(
                  elevation: 1,
                  // color: Colors.white,
                  child: Center(
                      child:ListTile(
                        title: Text(tr(LocaleKeys.lblOutboundTakeAwayByQR),
                          style: Theme.of(context).textTheme.headline6,
                          textAlign: TextAlign.center,
                        ),
                        subtitle: Icon(Icons.qr_code_sharp, size: 150,
                          // color: Colors.blueGrey[600]
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp),
                        onTap: () {
                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => AutoCompletedQRScanPage( ),
                              )
                          );
                        },
                      )
                  )
              )
          ),
        )
      ],
    );
  }

  Future<void>  _loadInboundOutboundCount( ) async{
    try{
      String accessToken = await userService.getToken();
      WarehouseStationRouteService contentService = new WarehouseStationRouteService();
      DeliveryCountByStatusRequest request = new DeliveryCountByStatusRequest();
      // request.branchGuid = _currentUser.branchGuid;

      // extracting inbound info
      DeliveryCountByStatusResponse _responseInbound = await contentService.inboundCountByStatus(accessToken, request);
      if( _responseInbound.statusCode == Constant.HTTP_OK || _responseInbound.statusCode == Constant.HTTP_CREATE ){

        _totalInboundCount = 0;
        _responseInbound.items.forEach((element) {
          if( element.status == Constant.STATUS_CREATED){
            _inboundCount = element.total;
            _totalInboundCount += element.total;
          }else if( element.status == Constant.STATUS_ASSIGNED){
            _inboundScheduleCount = element.total;
            _totalInboundCount += element.total;
          }else if( element.status == Constant.STATUS_ON_WAY){
            _inboundOnWayCount = element.total;
            _totalInboundCount += element.total;
          }else if( element.status == Constant.STATUS_COMPLETED){
            _inboundRequestCount = element.total;
            _totalInboundCount += element.total;
          }else if( element.status == Constant.STATUS_BUCKET){
            _inboundBucketCount = element.total;
            _totalInboundCount += element.total;
          }
        });
      }

      // extracting outbound info
      DeliveryCountByStatusResponse _responseOutbound = await contentService.outboundCountByStatus(accessToken, request);
      if( _responseOutbound.statusCode == Constant.HTTP_OK || _responseOutbound.statusCode == Constant.HTTP_CREATE ){

        _totalOutboundCount = 0;
        _outboundBucketCount = 0;
        _responseOutbound.items.forEach((element) {
          if( element.status == Constant.STATUS_CREATED){
            _outboundCount = element.total;
            _totalOutboundCount += element.total;
          }else if( element.status == Constant.STATUS_ASSIGNED){
            _outboundScheduleCount = element.total;
            _totalOutboundCount += element.total;
          }else if( element.status == Constant.STATUS_ON_WAY){
            _outboundOnWayCount = element.total;
            _totalOutboundCount += element.total;
          }else if( element.status == Constant.BUCKET_STATUS_REQUEST){
            _outboundRequestCount = element.total;
            _totalOutboundCount += element.total;
          }else if( element.status == Constant.STATUS_BUCKET || element.status == Constant.STATUS_COMPLETED){
            _outboundBucketCount += element.total;
            _totalOutboundCount += element.total;
          }
        });

        setState(() {
          _statusCountLoaded = true;
        });
      }


    }catch(e){}

    _getDepartmentCollected( );
  }

  Future<void>  _loadTransitCount( ) async{
    try{
      String accessToken = await userService.getToken();
      WarehouseStationRouteService contentService = new WarehouseStationRouteService();
      DeliveryCountByStatusRequest request = new DeliveryCountByStatusRequest();
      // request.branchGuid = _currentUser.branchGuid;

      // extracting inbound info
      DeliveryCountByStatusResponse _responseInbound = await contentService.countByStatusAndDeliveryman(accessToken, request);
      if( _responseInbound.statusCode == Constant.HTTP_OK || _responseInbound.statusCode == Constant.HTTP_CREATE ){

        _totalTransitCount = 0;
        _responseInbound.items.forEach((element) {
          if( element.status == Constant.STATUS_CREATED){
            _transitCount = element.total;
            _totalTransitCount += element.total;
          }else if( element.status == Constant.STATUS_ASSIGNED){
            _transitScheduleCount = element.total;
            _totalTransitCount += element.total;
          }else if( element.status == Constant.STATUS_ON_WAY){
            _transitOnWayCount = element.total;
            _totalTransitCount += element.total;
          }else if( element.status == Constant.STATUS_COMPLETED){
            _transitRequestCount = element.total;
            _totalTransitCount += element.total;
          }else if( element.status == Constant.STATUS_BUCKET){
            _transitBucketCount = element.total;
            _totalTransitCount += element.total;
          }else if( element.status == Constant.BUCKET_STATUS_REQUEST){
            _takeAwayRequestCount = element.total;
          }
        });

        setState(() {
          _statusCountLoaded = true;
        });
      }

    }catch(e){}
    _getCollected( );
  }

  Future<void> _getCollected( ) async{
    EmpployeeClearanceListRequest request = new EmpployeeClearanceListRequest(
      startDate: null,
      endDate: null,
      employeeGuid: null,
      branchGuid: null,
      zoneGuid: null,
      status: 'On hand',
      first: 0,
      max: Constant.ROW_PER_PAGE,
      orderBy:'id',
      asc: false,
    );

    EmployeeClearanceListResponse _employeeClearanceListResponse = await _employeeCollectedService.listEmployeeClearance(request);
    if( _employeeClearanceListResponse !=null && _employeeClearanceListResponse.items!=null ){
      setState(() {_totalCollected = _employeeClearanceListResponse.totalCollected + _employeeClearanceListResponse.collectedPrepaid;});
    }
  }

  Future<void> _getDepartmentCollected( ) async{
    if( _currentUser.roleName != Constant.ROLE_NAME_MANAGER)
      return;

    try{
      EmpployeeClearanceListRequest request = new EmpployeeClearanceListRequest(
          startDate: null,
          endDate: null,
          employeeGuid: null,
          branchGuid: null,
          zoneGuid: null,
          status: 'On hand',
          first: 0,
          max: Constant.ROW_PER_PAGE,
          orderBy:'id',
          asc: false,
          wayType: 'branch'
      );

      CODBalanceListResponse _codBalancelistResponse = await _employeeCollectedService.fetchCashOnDelivery(request);
      if( _codBalancelistResponse !=null && _codBalancelistResponse.items!=null && _codBalancelistResponse.items.isNotEmpty){
        setState(() {_totalCollected = _codBalancelistResponse.items.first.onHand;});
      }
    }catch(e){}
  }

  void _gotoTransit( BuildContext context, int index, String title, String wayType ) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) =>
            TransitListHome(
              selectedIndex: index,
              title: title,
              onChange: () {
                _loadStatusCount();
              },
              wayType: wayType,
            ),
      ),
    );
  }


}