import 'dart:async';
import 'dart:io';
import 'package:badges/badges.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/database/dao/notification_dao.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_count_by_status_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_count_by_status_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/mem_promotion_response.dart';
import 'package:qq_delivery/delivery/model/response/overdue_response.dart';
import 'package:qq_delivery/delivery/service/firebase_service.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/service/member_service.dart';
import 'package:qq_delivery/delivery/service/notification_service.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/pickup_service_v2.dart';
import 'package:qq_delivery/delivery/view/user_validate.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:location/location.dart';
import 'package:qq_delivery/delivery/view/utils/connection_broken.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'app_drawer.dart';
import 'chat/chat_latest_list.dart';
import 'delivery/delivery_list_home.dart';
import 'delivery/delivery_list_home_customer.dart';
import 'delivery2/delivery_create_v2.dart';
import 'delivery2/pickup_list_home.dart';
import 'login.dart';
import 'notification/notification_list_home.dart';
import 'reports/active_way_count_by_town_list.dart';
import 'utils/advertising.dart';
import 'utils/custom_refresh_indicator.dart';


class Home extends StatefulWidget {
  Home({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<Home> {

  FirebaseMessage _firebaseMessage = FirebaseMessage( );
  User _currentUser = new User();
  UserService userService = new UserService( );
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  NotificationDao notificationDao = new NotificationDao();
  SharedPreferences _sharedPrefs;
  int _notificationCount = 0;

  int _pickupCount = 0;
  int _pickupScheduleCount = 0;
  int _pickupOnWayCount = 0;
  int _pickupCompletedCount = 0;
  int _pickupCanceledCount = 0;

  int _dropOffCount = 0;
  int _dropOffScheduleCount = 0;
  int _dropOffOnWayCount = 0;
  int _completedCount = 0;
  int _canceledCount = 0;
  int _retryCount = 0;
  int _returnCount = 0;
  int _totalWays = 0;
  double _completedPercent = 90;
  bool _statusCountLoaded = false;
  bool isStopped = false;
  bool _notificationRefreshing = false;
  bool _isFirstOpen = true;
  double _osBalance = 0.0;
  double _promoBalance = 0.0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  Timer _timerNotificationRefresh;
  Timer _timerEmployeeTrack;
  OverdueResponse _overdueResponse;
  int _selectedIndex = 0;

  @override
  void initState() {
    super.initState();
    _timerNotificationRefresh = Timer.periodic(Duration(seconds: 3), (timer) {
      if (isStopped ) {timer.cancel();}
      try{
        _refreshNotifications();
      }catch(e){}
    });
  }

  @override
  void deactivate() {
    isStopped=true;
    super.deactivate( );
  }

  @override
  void dispose() {
    isStopped=true;
    if (_timerNotificationRefresh?.isActive ?? false) _timerNotificationRefresh.cancel();
    try{
      if (_timerEmployeeTrack?.isActive ?? false) _timerEmployeeTrack.cancel();
    }catch(e){}
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        actions: <Widget>[
          Container(
            padding: EdgeInsets.all(16),
              child: Badge(
                badgeContent: Text('$_notificationCount'),
                toAnimate: true,
                animationType: BadgeAnimationType.slide,
                child: IconButton(
                  highlightColor: Colors.red,
                  icon: Icon(Icons.notifications_active, color: Colors.white,),
                  onPressed: () async{
                    _gotoNotifications();
                  },
                  tooltip: 'Notifications',
                ),
              )
          ),
        ],
      ),
      body: _createBody(),
      drawer: AppDrawer(
        currentUser: _currentUser,
        authorityResponse: _authorityResponse,
        sharedPrefs: _sharedPrefs,
        startLocation: _startLocation,
      ),
      floatingActionButton:
      _authorityResponse.authorities!=null
          && _authorityResponse.authorities.contains("USER_WRITE") && _currentUser.roleName!=Constant.ROLE_NAME_EMPLOYEE ?
      FloatingActionButton.extended(
        onPressed: () async{
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => DeliveryCreateV2(
                onChange: (){
                  _loadStatusCount();
                },
              ),
            ),
          );
        },
        label: Text(
          tr(LocaleKeys.lblDeliCreateTitle),
          style: TextStyle(color: Colors.white),
        ),
        icon: Icon(Icons.add_circle_outline, color: Colors.white,),
        backgroundColor: Theme.of(context).primaryColor,
      ) : FloatingActionButton.extended(
        onPressed: () async{
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => ActiveWayCountByTownList(),
            ),
          );
        },
        label: Text(
          tr(LocaleKeys.lblHomeGoingOut),
          style: TextStyle(color: Colors.white),
        ),
        icon: Icon(Icons.logout, color: Colors.white,),
        backgroundColor: Theme.of(context).primaryColor,
      ),
      bottomNavigationBar: _authorityResponse.authorities!=null
          && _authorityResponse.authorities.contains("EMPLOYEE_READ")? BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.download_sharp),
            label: tr(LocaleKeys.lblMenuPickupWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.upload_sharp),
            label: tr(LocaleKeys.lblMenuDeliverWays),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.chat),
            label: 'Chat',
          ),
        ],
        currentIndex: _selectedIndex,
        // selectedItemColor: Theme.of( context ).primaryColor,
        // unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        // backgroundColor: Colors.white,
        onTap: _onItemTapped,
      ) : BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.home),
            label: 'Home',
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.chat),
            label: 'Chat',
          ),
        ],
        currentIndex: _selectedIndex,
        // selectedItemColor: Theme.of( context ).primaryColor,
        // unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        // backgroundColor: Colors.white,
        onTap: _onItemTapped,
      ),
    );
  }

  void _gotoList( BuildContext context, int index, String title, bool isPickup ) {
    if (_authorityResponse.authorities != null
        && _authorityResponse.authorities.contains("OS_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ")) {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) =>
                  DeliveryListHomeCustomer(
                    selectedIndex: index>2? index-1 : index,
                    title: title,
                    onChange: () {
                      _loadStatusCount();
                    },
                  ),
            ),
          );
    }else {
      if( isPickup ){
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) =>
                PickupListHome(
                  selectedIndex: index,
                  title: title,
                  onChange: () {
                    _loadStatusCount();
                  },
                ),
          ),
        );
      }else{
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) =>
                DeliveryListHome(
                  selectedIndex: index,
                  title: title,
                  onChange: () {
                    _loadStatusCount();
                  },
                ),
          ),
        );
      }
    }
  }

  void _loadProfile( )async{

    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    setState(() { });

    _loadStatusCount();
    try {
      if( await getFirebaseToken( ) == null)
        await _firebaseMessage.initMessaging( );
    }catch(e){ }

    // if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("EMPLOYEE_READ")){
      _initLocationService();
    // }
    try{
      if( !_currentUser.verified ){
        Timer(Duration(seconds: 4), () async{
          _showVerification( );
        });
      }
    }catch(e){print(e);}
    try{
      if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("EMPLOYEE_READ")
          && !_authorityResponse.authorities.contains("MANAGER_READ")){
        _timerEmployeeTrack = Timer.periodic(Duration(minutes: 1), (timer) {
          if (isStopped ) {timer.cancel();}
          try{
            _updateUserInfo();
          }catch(e){}
        });
      }else{
        Timer(Duration(seconds: 2), () async{
          _updateUserInfo();
        });
      }
    }catch(e){}

    if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
        && !_authorityResponse.authorities.contains("MANAGER_READ")){
      _updateOSBalance();
    }
  }

  Future<void>  _loadStatusCount( ) async{
    try{
      String accessToken = await userService.getToken();
      DeliveryContentServiceV2 contentService = new DeliveryContentServiceV2();
      PickupServiceV2 pickupServiceV2 = new PickupServiceV2();
      DeliveryCountByStatusRequest request = new DeliveryCountByStatusRequest();
      request.branchGuid = _currentUser.branchGuid;
      // extracting delivery info
      DeliveryCountByStatusResponse _response = await contentService.countByStatus(accessToken, request);

      if( _response.statusCode == Constant.HTTP_OK || _response.statusCode == Constant.HTTP_CREATE ){

        // setState(() {_statusCountLoaded = false;});

        _totalWays = 0;
        _response.items.forEach((element) {
          if( element.status == Constant.STATUS_CREATED){
            _dropOffCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_ASSIGNED){
            _dropOffScheduleCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_ON_WAY){
            _dropOffOnWayCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_COMPLETED){
            _completedCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_CANCELED){
            _canceledCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_RETRY){
            _retryCount = element.total;
            _totalWays += element.total;
          }else if( element.status == Constant.STATUS_RETURN){
            _returnCount = element.total;
            _totalWays += element.total;
          }
        });

        // extracting pickup info
        _response = await pickupServiceV2.countByStatus(accessToken, request);
        _response.items.forEach((element) {
          if( element.status == Constant.STATUS_PICKUP_CREATED){
            _pickupCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_PICKUP_SCHEDULED){
            _pickupScheduleCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_PICKUP_ON_WAY){
            _pickupOnWayCount = element.total;
            // _totalWays += element.total;
          }

          else if( element.status == Constant.STATUS_COMPLETED){
            _pickupCompletedCount = element.total;
            // _totalWays += element.total;
          }else if( element.status == Constant.STATUS_CANCELED){
            _pickupCanceledCount = element.total;
            // _totalWays += element.total;
          }
        });

        if( _currentUser.roleName =='Employee')
          _overdueResponse = await contentService.overdueCount(_currentUser.guid);
        else
          _overdueResponse = await contentService.overdueCount( );

        setState(() {
          _completedPercent = _completedCount>0? ( (_completedCount/_totalWays) *100 ).toDouble() : 0.0;
          _statusCountLoaded = true;
        });
      }else if( _response.statusCode == Constant.HTTP_DENIED ){
        await userService.logout( );
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => LoginUser(
                title: tr(LocaleKeys.login),
              ),
            )
        );
      }else{
        String status = await testConnection("/v1/state/list");
        if( status == Constant.CONNECTION_REFUSED){
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => ConnectionBroken(),
              )
          );
        }
        // else if( status == Constant.CONNECTION_OK ){
        //   await userService.logout( );
        //   Navigator.pushReplacement(
        //       context,
        //       MaterialPageRoute(
        //         builder: (BuildContext context) => LoginUser(
        //           title: tr(LocaleKeys.login),
        //         ),
        //       )
        //   );
        // }
      }
    }catch(e){}
  }

  Future<void> _updateUserInfo() async{
    try{
      PersonService personService = new PersonService();
      await personService.updateKeyLatLon(
          {
            'firebaseKey':await getFirebaseToken( ),
            'latitude': _startLocation!=null? _startLocation.latitude : null,
            'longitude': _startLocation!=null? _startLocation.longitude : null,
            'platform': Platform.isAndroid? 'Android' : 'iOS'
          }
          );
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  Future<void> _updateOSBalance() async{
    try{
      PickupServiceV2 _pickupServiceV2 = new PickupServiceV2();
      DeliveryListRequest _listRequest = new DeliveryListRequest();
      _listRequest = new DeliveryListRequest(
          keyword: '',
          startDate: null,
          endDate: null,
          osGuid: _currentUser.guid,
          customerGuid: null,
          employeeGuid: null,
          zoneGuid: null,
          branchGuid: null,
          status: null,
          first: 0,
          max: 1000000,
          orderBy: 'id',
          asc: false
      );

      UserService _userService = UserService();
      String accessToken = await _userService.getToken();
      DeliveryListResponse _deliveryListResponse = await _pickupServiceV2.invoice(accessToken, _listRequest, true, 0 );

      double toRefund = 0;
      if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){
        _deliveryListResponse.items.forEach((element) {
          if( element.dueAmount != null){
            toRefund += (element.dueAmount * -1);
          }
        });
        _osBalance  = toRefund;
      }
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    // await _updatePromoBalance();
  }

  Future<void> _updatePromoBalance( ) async{
    try{
      MemberService memberService = new MemberService( );
      MemberPromotionResponse balanceResponse = await memberService.promoBalance( _currentUser.guid );
      if( balanceResponse != null && balanceResponse.items!=null){
        if( balanceResponse.items.isNotEmpty){
          MemberPromotionItem balanceItem = balanceResponse.items[0];
          _promoBalance = balanceItem.balance;
        }
      }
    }catch(e) {
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  Future<void> _showVerification( )async {

    String result = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => UserValidation(
            phoneNumber: _currentUser.phone,
          ),
        )
    );
    if( result!=null && result=='Verified' ){
      await userService.getUserInformation();
      await _currentUser.fromSharedPreferences( );
    }else{
      await userService.logout();
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => LoginUser(
              title: tr(LocaleKeys.login),
            ),
          )
      );
    }
  }

  void _refreshNotifications( )async{
    if( _notificationRefreshing ) return;
    _notificationRefreshing = true;

    if( _isFirstOpen ) {
      NotificationService notificationService = new NotificationService();
      bool hasUpdate = await notificationService.fetchAndUpdate( );
      _isFirstOpen = false;
    }

    await notificationDao.open( );
    int count = await notificationDao.unreadCount();
    await notificationDao.close();

    if( count != _notificationCount && mounted){
      _notificationCount = count;
      _loadStatusCount( );
    }

    _notificationRefreshing = false;
    if( mounted )
      setState(() {});
  }

  void _gotoNotifications() async{
    await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => NotificationListHome( ),
        )
    );
  }

  Location _locationService  = new Location();
  bool _permission = false;
  LocationData _currentLocation;
  LocationData _startLocation;
  String error;
  _initLocationService() async {
    LocationData location;
    try {

      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
            // print('Lat: ${_startLocation.latitude}, Lon:${_startLocation.longitude}');
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }
    _startLocation = location;
  }

  void _onItemTapped(int index) {
    setState(() {
      _selectedIndex = index;
    });
    if( index == 2 &&  _authorityResponse.authorities!=null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ")){
      Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => ChatLatestList(
              currentUser: _currentUser,
              sharedPrefs: _sharedPrefs,
            ),
          )
      );
      setState(() {
        _selectedIndex = 0;
      });
    }
    if( index == 1 &&  _authorityResponse.authorities!=null
        && !_authorityResponse.authorities.contains("EMPLOYEE_READ")){
      Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => ChatLatestList(
              currentUser: _currentUser,
              sharedPrefs: _sharedPrefs,
            ),
          )
      );
      setState(() {
        _selectedIndex = 0;
      });
    }
  }

  Widget _createBody(){
    if( _authorityResponse.authorities!=null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ") ){
      if( _selectedIndex == 0 ){
        return _pickupPanel();
      }else if( _selectedIndex == 1 ){
        return _deliverPanel();
      }else{
        return new Container();
      }
    }else{
      if( _selectedIndex == 0 ) {
        return _merchantPanel();
      }else{
        return new Container();
      }
    }
  }

  Widget _pickupPanel(){
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: false,
      child: ListView(
        children: <Widget>[

          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("MANAGER_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.view_list_rounded, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),

          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.assignment_ind_sharp, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.local_shipping_sharp, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.green[800], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.done, size: 32, color: Colors.white ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCompletedCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                          // style: TextStyle(color: Colors.black)
                        ),// if Sender = လာယူနေ
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.red, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCanceledCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          Advertising()
        ],
      ),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      // scroll down
      onRefresh: () async {
        if( _isFirstOpen ){
          _loadProfile();
        }else{
          await _loadStatusCount();
          if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
              && !_authorityResponse.authorities.contains("MANAGER_READ")){
            _updateOSBalance();
          }
        }
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
    );
  }

  Widget _deliverPanel(){
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: false,
      child: ListView(
        children: <Widget>[
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("MANAGER_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.view_list_rounded, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.assignment_ind_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 1, tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          // Cancel
          Wrap(
            children: <Widget>[
              _authorityResponse.authorities!=null
                  && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.local_shipping_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                        },
                      )
                  )
              ): Container(),
              Container(
                  width: MediaQuery.of(context).size.width * 0.999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.green[800], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_completedCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblCompleted),
                          // style: TextStyle(color: Colors.black)
                        ),// if Sender = လာယူနေ
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                        },
                      )
                  )
              ),
            ],
          ),
          // Completed
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.yellow[700], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.refresh, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_retryCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.red, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_canceledCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                        },
                      )
                  )
              ),
            ],
          ),
          Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
              child: Card(
                  elevation: 8,
                  // color: Colors.white,
                  child: ListTile(
                    // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                    leading:ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.red, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      '$_returnCount',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(tr(LocaleKeys.lblReturn),
                      // style: TextStyle(color: Colors.black)
                    ),
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                    },
                  )
              )
          ),
          Advertising()
        ],
      ),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      // scroll down
      onRefresh: () async {
        if( _isFirstOpen ){
          _loadProfile();
        }else{
          await _loadStatusCount();
          if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
              && !_authorityResponse.authorities.contains("MANAGER_READ")){
            _updateOSBalance();
          }
        }
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
    );
  }

  Widget _merchantPanel(){
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: false,
      child: ListView(
        children: <Widget>[
          Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
              child: Card(
                  elevation: 8,
                  child: ListTile(
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          child: InkWell(
                            child: new Container(
                              // width: 60.0,
                              // height: 60.0,
                              padding: const EdgeInsets.all(12.0),
                              decoration: new BoxDecoration(
                                shape: BoxShape.circle,
                                color: Colors.yellow[700],
                              ),
                              child: new Text(
                                '${_pickupCount + _pickupScheduleCount + _pickupOnWayCount + _dropOffCount + _dropOffScheduleCount + _dropOffOnWayCount}',
                                style: Theme.of(context).textTheme.headline5,
                              ),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      tr(LocaleKeys.lblTrackTheWays),
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(tr(LocaleKeys.lblPickupDeliveryOnWay),
                      // style: TextStyle(color: Colors.black)
                    ),// Active ways
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 0, tr(LocaleKeys.lblDeliver), false );
                    },
                  )
              )
          ),
          // Cancel
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.green[800], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_completedCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblCompleted),
                          // style: TextStyle(color: Colors.black)
                        ),// if Sender = လာယူနေ
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                        },
                      )
                  )
              ),
            ],
          ),
          // Completed
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.yellow[700], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.refresh, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_retryCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.red, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_canceledCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                        },
                      )
                  )
              ),
            ],
          ),
          Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
              child: Card(
                  elevation: 8,
                  // color: Colors.white,
                  child: ListTile(
                    // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                    leading:ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.red, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      '$_returnCount',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(tr(LocaleKeys.lblReturn),
                      // style: TextStyle(color: Colors.black)
                    ),
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                    },
                  )
              )
          ),
          Advertising()
        ],
      ),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      // scroll down
      onRefresh: () async {
        if( _isFirstOpen ){
          _loadProfile();
        }else{
          await _loadStatusCount();
          if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
              && !_authorityResponse.authorities.contains("MANAGER_READ")){
            _updateOSBalance();
          }
        }
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
    );
  }

  Widget _backupPanel(){
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: false,
      child: ListView(
        children: <Widget>[
          // _authorityResponse.authorities!=null
          //     && _authorityResponse.authorities.contains("OS_READ")
          //     && !_authorityResponse.authorities.contains("MANAGER_READ")? Wrap(
          //   children: <Widget>[
          //     Container(
          //         width: MediaQuery.of(context).size.width * 0.46,
          //         margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
          //         child: Card(
          //             elevation: 8,
          //             // color: Colors.white,
          //             child: ListTile(
          //               title: Text(
          //                 '${new NumberFormat("#,###").format(_osBalance)} Ks',
          //               ),
          //               subtitle: Text(tr(LocaleKeys.lblMainBalance),
          //               ),
          //               trailing: Icon(Icons.monetization_on_outlined,
          //               ),
          //             )
          //         )
          //     ),
          //     Container(
          //         width: MediaQuery.of(context).size.width * 0.46,
          //         margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
          //         child: Card(
          //             elevation: 8,
          //             // color: Colors.white,
          //             child: ListTile(
          //               title: Text(
          //                 '${new NumberFormat("#,###").format(_promoBalance)} Ks',
          //                 // style: Theme.of(context).textTheme.headline6,
          //               ),
          //               subtitle: Text(tr(LocaleKeys.lblPromotionBalance),
          //               ),
          //               trailing: Icon(Icons.redeem,
          //               ),
          //             )
          //         )
          //     ),
          //   ],
          // ) : Container(),

          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("OS_READ")
              && !_authorityResponse.authorities.contains("MANAGER_READ")?Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
              child: Card(
                  elevation: 8,
                  child: ListTile(
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          child: InkWell(
                            child: new Container(
                              // width: 60.0,
                              // height: 60.0,
                              padding: const EdgeInsets.all(12.0),
                              decoration: new BoxDecoration(
                                shape: BoxShape.circle,
                                color: Colors.yellow[700],
                              ),
                              child: new Text(
                                '${_pickupCount + _pickupScheduleCount + _pickupOnWayCount + _dropOffCount + _dropOffScheduleCount + _dropOffOnWayCount}',
                                style: Theme.of(context).textTheme.headline5,
                              ),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      tr(LocaleKeys.lblTrackTheWays),
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(tr(LocaleKeys.lblPickupDeliveryOnWay),
                      // style: TextStyle(color: Colors.black)
                    ),// Active ways
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 0, tr(LocaleKeys.lblDeliver), false );
                    },
                  )
              )
          )
              : Container(),

          // _authorityResponse.authorities!=null
          //     && _authorityResponse.authorities.contains("EMPLOYEE_READ")
          //     && _overdueResponse!=null? Wrap(
          //   children: <Widget>[
          //     Container(
          //         width: MediaQuery.of(context).size.width * 0.46,
          //         margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
          //         child: Card(
          //             elevation: 1,
          //             // color: Colors.white,
          //             child: ListTile(
          //               title: Text(
          //                 '${_overdueResponse.overduePickup}${tr(LocaleKeys.lblHomeWays)}',
          //               ),
          //               subtitle: Text(tr(LocaleKeys.lblHomePickupOverdue),
          //                   // style: TextStyle(color: Colors.black)
          //               ),
          //               trailing: Icon(Icons.warning,
          //                   // color: Theme.of(context).primaryColor
          //               ),
          //               // onTap: () { /*TODO goto list view and filter with date*/},
          //             )
          //         )
          //     ),
          //     Container(
          //         width: MediaQuery.of(context).size.width * 0.46,
          //         margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
          //         child: Card(
          //             elevation: 1,
          //             // color: Colors.white,
          //             child: ListTile(
          //               title: Text(
          //                 '${_overdueResponse.overdueDeliver}${tr(LocaleKeys.lblHomeWays)}',
          //                 // style: Theme.of(context).textTheme.headline6,
          //               ),
          //               subtitle: Text(tr(LocaleKeys.lblHomeDeliverOverdue),
          //                   // style: TextStyle(color: Colors.black)
          //               ),
          //               // subtitle: Text(tr(LocaleKeys.lblQueue), style: TextStyle(color: Colors.black)),
          //               trailing: Icon(Icons.warning,
          //                   // color: Theme.of(context).primaryColor
          //               ),
          //               // onTap: () {/*TODO goto list view and filter with date*/ },
          //             )
          //         )
          //     ),
          //   ],
          // ) : Container(),

          // Deliver completed gauge
          // Container(
          //     height: gaugeHeight,
          //     margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
          //     child: Card(
          //       elevation: 8,
          //       // color: Colors.white,
          //       child: Wrap(
          //         children: <Widget>[
          //           ListTile(
          //             leading: ClipOval(
          //                 child: Material(
          //                   elevation: 1,
          //                   child: InkWell(
          //                     // splashColor: Theme.of(context).backgroundColor,
          //                     child: new Container(
          //                       // width: 60.0,
          //                       // height: 60.0,
          //                       padding: const EdgeInsets.all(12.0),
          //                       decoration: new BoxDecoration(
          //                         shape: BoxShape.circle,
          //                         // borderRadius: new BorderRadius.circular(30.0),
          //                         color: Theme.of(context).primaryColor,
          //                       ),
          //                       child: new Text(
          //                         '$_completedCount',
          //                         style: Theme.of(context).textTheme.headline5,
          //                       ),
          //                     ),
          //                   ),
          //                 )
          //             ),
          //             title: Text(
          //               '${tr(LocaleKeys.lblCompleted)}',
          //               style: Theme.of(context).textTheme.headline6,
          //             ),
          //             // subtitle: Text("Total successfully completed", style: TextStyle(color: Colors.black)),
          //             trailing: Icon(Icons.arrow_forward_sharp,
          //                 // color: Colors.blueGrey[600]
          //             ),
          //             onTap: () {
          //               _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
          //             },
          //           ),
          //           _statusCountLoaded? Container(
          //               margin: EdgeInsets.only(top: 8),
          //               child:FlutterGauge(
          //                 width: 200,
          //                 index:_completedPercent,
          //                 fontFamily: "Iran",
          //                 counterStyle: TextStyle(
          //                   color: Colors.black,
          //                   fontSize: 32,),
          //                 numberInAndOut: NumberInAndOut.outside,
          //                 counterAlign: CounterAlign.center,
          //                 secondsMarker: SecondsMarker.secondsAndMinute,
          //                 hand: Hand.short,
          //                 handColor: Theme.of(context).primaryColor,
          //                 circleColor: Theme.of(context).backgroundColor,
          //                 activeColor: Theme.of(context).primaryColorDark,
          //               )
          //           ) : Container(),
          //         ],
          //       ),
          //     )
          // ),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("MANAGER_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.view_list_rounded, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblHomeAdminPickupToAssign),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminPickupToAssign), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),

          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.assignment_ind_sharp, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup: LocaleKeys.lblHomeAdminPickupToAccept),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 1, tr(LocaleKeys.lblToPickup), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.local_shipping_sharp, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToPickup:LocaleKeys.lblPickupOnWay), true );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.green[800], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.done, size: 32, color: Colors.white ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCompletedCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblPickupCompleted),
                          // style: TextStyle(color: Colors.black)
                        ),// if Sender = လာယူနေ
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 3, tr(LocaleKeys.lblPickupCompleted), true );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.red, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_pickupCanceledCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblPickupCancel),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 4, tr(LocaleKeys.lblPickupCancel), true );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("MANAGER_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/queue.svg', semanticsLabel: 'Queue',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.view_list_rounded, size: 32,
                                      color: Colors.white
                                  ),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblHomeAdminDeliverAssign),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 0, tr(LocaleKeys.lblHomeAdminDeliverAssign), false );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          _authorityResponse.authorities!=null
              && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/schedule.svg', semanticsLabel: 'Schedule',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.assignment_ind_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffScheduleCount',//'${_pickupScheduleCount + _dropOffScheduleCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 1, tr(_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblHomeAdminDeliveryToAccept), false );
                        },
                      )
                  )
              ),
            ],
          ) : Container(),
          // Cancel
          Wrap(
            children: <Widget>[
              _authorityResponse.authorities!=null
                  && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Theme.of(context).primaryColor, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.local_shipping_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_dropOffOnWayCount',//'${_pickupOnWayCount + _dropOffOnWayCount}',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 2, tr(_authorityResponse.authorities!=null && !_authorityResponse.authorities.contains("MANAGER_READ")?LocaleKeys.lblToDropOff:LocaleKeys.lblDropOffOnWay), false );
                        },
                      )
                  )
              ): Container(),
              Container(
                  width: MediaQuery.of(context).size.width * 0.999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.green[800], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.done_all_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_completedCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblCompleted),
                          // style: TextStyle(color: Colors.black)
                        ),// if Sender = လာယူနေ
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 3, tr(LocaleKeys.lblCompleted), false );
                        },
                      )
                  )
              ),
            ],
          ),
          // Completed
          Wrap(
            children: <Widget>[
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.yellow[700], // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.refresh, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_retryCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverRetry),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 5, tr(LocaleKeys.lblDeliverRetry), false );
                        },
                      )
                  )
              ),
              Container(
                  width: MediaQuery.of(context).size.width * 0.9999,
                  margin: EdgeInsets.only(left: 8, right: 8, bottom: 0, top: 8),
                  child: Card(
                      elevation: 8,
                      // color: Colors.white,
                      child: ListTile(
                        // leading: SvgPicture.asset( 'assets/images/on_way.svg', semanticsLabel: 'On way',height: 48 ),
                        leading:ClipOval(
                            child: Material(
                              elevation: 1,
                              color: Colors.red, // button color
                              child: InkWell(
                                // splashColor: Colors.red, // inkwell color
                                child: SizedBox(
                                  width: 48,
                                  height: 48,
                                  child: Icon(Icons.cancel_schedule_send_sharp, size: 32, color: Colors.white),
                                ),
                              ),
                            )
                        ),
                        title: Text(
                          '$_canceledCount',
                          style: Theme.of(context).textTheme.headline6,
                        ),
                        subtitle: Text(tr(LocaleKeys.lblDeliverCancel),
                          // style: TextStyle(color: Colors.black)
                        ),
                        trailing: Icon(Icons.arrow_forward_sharp,
                          // color: Colors.blueGrey[600]
                        ),
                        onTap: () {
                          _gotoList( context, 4, tr(LocaleKeys.lblDeliverCancel), false );
                        },
                      )
                  )
              ),
            ],
          ),
          Container(
              margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 8),
              child: Card(
                  elevation: 8,
                  // color: Colors.white,
                  child: ListTile(
                    // leading: SvgPicture.asset( 'assets/images/return.svg', semanticsLabel: 'Return',height: 48 ),
                    leading:ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.red, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: Icon(Icons.backspace_sharp, size: 32, color: Colors.white),
                            ),
                          ),
                        )
                    ),
                    title: Text(
                      '$_returnCount',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(tr(LocaleKeys.lblReturn),
                      // style: TextStyle(color: Colors.black)
                    ),
                    trailing: Icon(Icons.arrow_forward_sharp,
                      // color: Colors.blueGrey[600]
                    ),
                    onTap: () {
                      _gotoList( context, 6, tr(LocaleKeys.lblReturn), false );
                    },
                  )
              )
          ),
          Advertising()
        ],
      ),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      // scroll down
      onRefresh: () async {
        if( _isFirstOpen ){
          _loadProfile();
        }else{
          await _loadStatusCount();
          if( _authorityResponse.authorities != null && _authorityResponse.authorities.contains("OS_READ")
              && !_authorityResponse.authorities.contains("MANAGER_READ")){
            _updateOSBalance();
          }
        }
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
    );
  }
}