import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:launch_review/launch_review.dart';
import 'package:provider/provider.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/service/settings_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:shared_preferences/shared_preferences.dart';
import '../../theme.dart';
import 'app_drawer.dart';
import 'delivery/delivery_detail.dart';
import 'delivery2/delivery_address.dart';
import 'home.dart';
import 'login.dart';
import 'user/user_register.dart';


class Landing extends StatefulWidget {
  Landing({Key key}) : super(key: key);
  @override
  _LandingState createState() => _LandingState();
}

class _LandingState extends State<Landing> {

  AddressBookItem _addressBookItemSender;
  AddressBookItem _addressBookItemRecipient;
  final TextEditingController _controller_senderAddress = new TextEditingController();
  final TextEditingController _controller_recipientAddress = new TextEditingController();

  final _searchQuery = new TextEditingController();
  Timer _debounce;
  bool _processing = false;
  bool _processingError = false;
  DeliveryContentServiceV2 _deliveryContentService;
  // ThemeData _selectedTheme;
  var _themeProvider;

  @override
  void initState() {
    super.initState();
    work( );
    _loadDefaultLanguage();
    _searchQuery.addListener(_onSearchChanged);
  }
  @override
  void deactivate() {
    super.deactivate( );
  }
  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(milliseconds: 700), () {
      String searchText = _searchQuery.text.trim();
      if( searchText.length >=7 && !_processing){
        _search();
      }
    });
  }

  void _search() async{
    if( _processing ) return;
    setState(() {
      _processing = true;
      _processingError = false;
    });
    _debounce.cancel();

    if( _deliveryContentService == null)
      _deliveryContentService = new DeliveryContentServiceV2( );

    DeliveryDetailResponse _detailResponse = await _deliveryContentService.detail(_searchQuery.text.trim());
    if( _detailResponse == null){
      _processingError = true;
    }else{
      _processingError = false;
      FocusScope.of(context).unfocus();
      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => DeliveryDetail(
              wayId: _searchQuery.text.trim(),
              onChange: (){},
            ),
          )
      );
    }
    setState(() {
      _processing = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    _themeProvider = Provider.of<ThemeChanger>(context);
    return Scaffold(
        // backgroundColor: Theme.of(context).backgroundColor,
        body: _createBody(context)
    );
  }

  Widget _createBody(BuildContext context){
    return Center(
        child: SingleChildScrollView(
          child: Container(
            margin: const EdgeInsets.only(top: 16.0, bottom: 0.0, left: 8.0, right: 8.0),
            child: Card(
              // color: Colors.white,
              elevation: 8,
              child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment:CrossAxisAlignment.center,
                  children: <Widget>[
                    Container(
                      padding: EdgeInsets.only(top: 32, left: 16, bottom: 0, right: 16),
                      child: TextField(
                        textAlign: TextAlign.center,
                        keyboardType: TextInputType.text,
                        decoration: InputDecoration(
                            hintText: tr(LocaleKeys.lblEnterWayIDToTrack),
                            hintStyle: TextStyle(fontSize: 14),
                            border: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(16),
                              borderSide: BorderSide(
                                width: 1,
                                style: BorderStyle.none,
                              ),
                            ),
                            filled: true,
                            contentPadding: EdgeInsets.all(8),
                            // fillColor: Theme.of(context).backgroundColor,
                            suffixIcon: _searchQuery.text.length>0? IconButton(
                              icon: Icon(Icons.close_sharp,
                                  // color: Theme.of(context).primaryColor
                              ),
                              onPressed: (){
                                setState(() {
                                  _searchQuery.text = '';
                                  _processingError = false;
                                });
                              },
                            ): Icon(Icons.search_sharp),
                          // prefixIcon: SizedBox(width: 16, height: 16, child: CircularProgressIndicator())
                        ),
                        controller: _searchQuery,
                      ),
                    ),
                    _processingError?Container(
                      child: Text('Invalid wayID', style: TextStyle(color: Colors.red),),
                    ):Container(),
                    _processing?Container(
                      padding: EdgeInsets.only(left: 32, right: 32),
                        child: PreferredSize(
                            preferredSize: Size(double.infinity, 2.0),
                            child: SizedBox(
                                height: 2.0,
                                child: LinearProgressIndicator(
                                    // backgroundColor: Theme.of(context).backgroundColor
                                )
                            )
                        )
                    ) : Container(),
                    Container(
                      margin: EdgeInsets.only(top: 32, bottom: 32),
                      child: new Image.asset("assets/images/logo_rectangle.jpg"),
                    ),
                    Container(
                      padding: EdgeInsets.all(16.0),
                      child: TextField(
                        textAlign: TextAlign.center,
                        keyboardType: TextInputType.text,
                        decoration: InputDecoration(
                          hintText: tr(LocaleKeys.lblDeliCreatePickup),
                          hintStyle: TextStyle(fontSize: 14),
                          enabledBorder: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(16),
                            borderSide: BorderSide(
                                width: 1,
                                style: BorderStyle.solid,
                                // color: Theme.of(context).primaryColor
                            ),
                          ),
                          filled: true,
                          contentPadding: EdgeInsets.all(8),
                          // fillColor: Theme.of(context).backgroundColor,
                          prefixIcon: Icon(Icons.upload_sharp,),
                          suffixIcon : IconButton(
                            icon: Icon(Icons.add_location_sharp,
                                // color: Theme.of(context).primaryColor
                            ),
                            onPressed: (){print('Icon press');},
                          ),
                        ),
                        onTap: () async{
                          FocusScope.of(context).requestFocus(new FocusNode());
                          AddressBookItem addressBookItem = await Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (context) => DeliveryAddress(
                                title: tr(LocaleKeys.lblDeliCreatePickup),
                                addressType: 'OS',
                              ),
                            ),
                          );
                          if( addressBookItem != null) {
                            _addressBookItemSender = addressBookItem;
                            _controller_senderAddress.text = _addressBookItemSender.address+','+_addressBookItemSender.street+','+_addressBookItemSender.townshipName;
                          }
                        },
                        controller: _controller_senderAddress,
                      ),
                    ),
                    Container(
                      padding: EdgeInsets.all(16.0),
                      child: TextField(
                        textAlign: TextAlign.center,
                        keyboardType: TextInputType.text,
                        decoration: InputDecoration(
                          hintText: tr(LocaleKeys.lblDeliCreateDropOff),
                          hintStyle: TextStyle(fontSize: 14),
                          enabledBorder: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(16),
                            borderSide: BorderSide(
                                width: 1,
                                style: BorderStyle.solid,
                                // color: Theme.of(context).primaryColor
                            ),
                          ),
                          filled: true,
                          contentPadding: EdgeInsets.all(8),
                          // fillColor: Theme.of(context).backgroundColor,
                          prefixIcon: Icon(Icons.download_sharp,),
                          suffixIcon : IconButton(
                            icon: Icon(Icons.add_location_sharp,
                                // color: Theme.of(context).primaryColor
                            ),
                            onPressed: (){print('Icon press');},
                          ),
                        ),
                        onTap: () async{
                          FocusScope.of(context).requestFocus(new FocusNode());
                          AddressBookItem addressBookItem = await Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (context) => DeliveryAddress(
                                title: tr(LocaleKeys.lblDeliCreateDropOff),
                                addressType: 'Customer',
                              ),
                            ),
                          );

                          if( addressBookItem != null) {
                            _addressBookItemRecipient = addressBookItem;
                            _controller_recipientAddress.text = _addressBookItemRecipient.address+','+_addressBookItemRecipient.street+','+_addressBookItemRecipient.townshipName;
                          }
                        },
                        controller: _controller_recipientAddress,
                      ),
                    ),
                    Container(
                      height: 95.0,
                      padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 16),
                      width: double.infinity,
                      child: RaisedButton(
                        onPressed: () {
                            Navigator.pushReplacement(
                              context,
                              MaterialPageRoute(
                              builder: (BuildContext context) => LoginUser(
                                  title: tr(LocaleKeys.login),
                                ),
                              )
                            );
                        },
                        color: Theme.of(context).primaryColor,
                        elevation: 1,
                        child: Text(
                          tr(LocaleKeys.btnContinue),
                          style: Theme.of(context).textTheme.headline5,
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(32.0)),
                      ),
                    ),
                    Container(
                      margin: const EdgeInsets.only(top: 48.0, bottom: 0.0, left: 16, right: 16),
                      child: new Wrap(
                          direction: Axis.horizontal,
                          crossAxisAlignment: WrapCrossAlignment.center,
                          spacing: 16,
                          runSpacing: 16,
                          children: <Widget>[
                            FlatButton(
                              color: Colors.transparent,
                              // textColor: Colors.white,
                              disabledColor: Colors.grey,
                              disabledTextColor: Colors.black,
                              padding: EdgeInsets.all(8.0),
                              // splashColor: Colors.blueAccent,
                              onPressed: () {
                                Navigator.push(
                                    context,
                                    MaterialPageRoute(
                                      builder: (BuildContext context) => LoginUser(
                                        title: tr(LocaleKeys.login),
                                      ),
                                    )
                                );
                              },
                              child: Text(
                                  tr(LocaleKeys.login),
                                  style: TextStyle(
                                      decoration: TextDecoration.underline,
                                      // color: Theme.of(context).primaryColor
                                  )
                              ),
                            ),
                            FlatButton(
                              color: Colors.transparent,
                              // textColor: Colors.white,
                              disabledColor: Colors.grey,
                              disabledTextColor: Colors.black,
                              padding: EdgeInsets.all(8.0),
                              // splashColor: Colors.blueAccent,
                              onPressed: () {
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) => UserRegister( ),
                                  ),
                                );
                              },
                              child: Text(
                                  tr(LocaleKeys.lblCreateAccount),
                                  style: TextStyle(
                                      decoration: TextDecoration.underline,
                                      // color: Theme.of(context).primaryColor
                                  )
                              ),
                            ),
                          ]
                      ),
                    ),
                    new Wrap(
                        direction: Axis.horizontal,
                        crossAxisAlignment: WrapCrossAlignment.center,
                        spacing: 16,
                        runSpacing: 16,
                      children: <Widget>[
                        Container(
                          child: Column(
                            mainAxisSize: MainAxisSize.min,
                            crossAxisAlignment:CrossAxisAlignment.center,
                            children: <Widget>[
                              Container(
                                padding: const EdgeInsets.only(top: 0, bottom: 4, left: 16, right: 16),
                                child: Text(
                                  '©${new DateFormat('yyyy').format( DateTime.now())} ${Constant.APP_NAME}',
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    // color: Theme.of(context).primaryColor,
                                  ),
                                ),
                              ),
                              Container(
                                padding: const EdgeInsets.only(top: 4, bottom: 16, left: 16, right: 16),
                                child: Text(
                                  'Hotline: ${Constant.CONTACT_PHONE}',
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    // color: Theme.of(context).primaryColor,
                                  ),
                                ),
                              ),
                            ],
                          ),
                          padding: EdgeInsets.only(top: 32),
                        ),
                        _dropdownWidget(),
                        ]
                      )
                    ]
              ),
            ),
          ),
        )
    );
  }

  Language selectedLanguage;
  List<Language> languages = <Language> [
    new Language("မြန်မာ", SvgPicture.asset( 'assets/images/myanmar.svg', semanticsLabel: 'My',height: 32, width: 32, )),
    // Language("ေဇာ္ဂ်ီ", SvgPicture.asset( 'assets/images/myanmar.svg', semanticsLabel: 'MM',height: 32, width: 32, )),
    Language('English', SvgPicture.asset( 'assets/images/uk.svg', semanticsLabel: 'En',height: 32, width: 32, ))
  ];

  Widget _dropdownWidget() {
    return new DropdownButtonHideUnderline(
      child: DropdownButton<Language>(
        value: selectedLanguage,
        onChanged: (Language value){
          setState(() {
            selectedLanguage = value;
            if( value.name != 'English'){
              context.locale = Locale( Constant.LANGUAGE_CODE_MY,'');
              setLanguage( Constant.LANGUAGE_CODE_MY );
            }else{
              context.locale = Locale( Constant.LANGUAGE_CODE_EN,'');
              setLanguage( Constant.LANGUAGE_CODE_EN );
            }
          });
        },
        items: languages.map((Language lang) {
          return  DropdownMenuItem<Language>(
            value: lang,
            child: Row(
              children: <Widget>[
                lang.icon,
                SizedBox(width: 12,),
                Text(
                  lang.name,
                 // style:  TextStyle(color: Theme.of(context).primaryColor),
                ),
              ],
            ),
          );
        }).toList(),
      ),
    );
  }

  void _loadDefaultLanguage() async{
    String lang = await getLanguage();
    if( lang == Constant.LANGUAGE_CODE_EN){
      context.locale = Locale( Constant.LANGUAGE_CODE_EN,'');
      selectedLanguage = languages[1];
    }
    else {
      context.locale = Locale( Constant.LANGUAGE_CODE_MY,'');
      selectedLanguage = languages[0];
    }
    setState(() { });
    if( Constant.THEME_CHANGABLE)
      _loadDefaultTheme();
  }

  Future work() async {

    try{
      SettingsService settingsService = SettingsService();
      Map<String, dynamic> resultData = await settingsService.codeVersion();
      if( resultData["success"] ){
        Map<String, dynamic> data = resultData["data"];
        int codeVersion = data["codeVersion"]??9;
        bool isCritical = data["critical"]??true;

        if( Constant.CODE_VERSION < codeVersion ){
          bool  result = await _showConfirm( context, isCritical );
          if( isCritical || result ){
            LaunchReview.launch(
                iOSAppId: 'id1549905897',
                androidAppId: 'com.kcc.deliveryservice'
            );
            return;
          }
        }
      }
    }catch(e){
      if(Constant.ENABLE_LOGS)
        print(e);
    }

      final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
      if( await getSettingsTempLogin() ){
        await setSettingsTempLogin(false);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA);
        await sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
        await sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
        await sharedPrefs.remove(Constant.DEVICE_GUID);
        await sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
        await sharedPrefs.remove(Constant.AUTHENTICATION_2FA);
      }
      if( sharedPrefs.getString(Constant.CURRENT_USER_DATA) != null ){
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => Home(
                title: '${Constant.TESTING_MODE?Constant.APP_NAME+' test mode':Constant.APP_NAME}',
              ),
            )
        );
      }
  }

  Future<bool> _showConfirm( BuildContext context, bool isCritical ) async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( tr(LocaleKeys.lblSoftwareUpdateTitle), style: TextStyle(color: Colors.black),),
          content: new Text(tr(LocaleKeys.lblSoftwareUpdateDescription)),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text(tr(LocaleKeys.btnUpgrade)),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            !isCritical? new FlatButton(
              child: new Text(tr(LocaleKeys.btnLater)),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ) : Container(),
          ],
        );
      },
    );
    return result;
  }

  void _loadDefaultTheme() async{
    String name = await getTheme();
    _themeProvider.setTheme( getDefaultTheme( name ) );
    setState(() { });
  }
}