import 'package:easy_localization/easy_localization.dart';
import 'package:provider/provider.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/view/home.dart';
import 'package:qq_delivery/delivery/view/user/forgot_password.dart';
import 'package:qq_delivery/delivery/view/user/user_register.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import '/hrm/view/login/pre_login.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import '../../theme.dart';
import 'utils/setup_private_connection.dart';
import '/hrm/config/constant.dart' as config;
import '/hrm/view/home/home.dart' as hrHome;


//https://codingwithjoe.com/building-forms-with-flutter/
//https://codingwithjoe.com/building-forms-with-flutter/
class LoginUser extends StatefulWidget {
  LoginUser({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _LoginUserUserPageState createState() => _LoginUserUserPageState( );
}

class _LoginUserUserPageState extends State<LoginUser> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool _processing = false;
  String _name;
  String _password;
  String _message = '';
  Color _messageColor = Colors.teal;
  Color progressColor;

  @override
  void initState() {
    super.initState();
    work( );
  }

  Future work() async {
    new ThreadTimer(new Duration(milliseconds: 1), () async{
      final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
      if( await getSettingsTempLogin() ){
        await setSettingsTempLogin(false);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA);
        await sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
        await sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
        await sharedPrefs.remove(Constant.DEVICE_GUID);
        await sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
        await sharedPrefs.remove(Constant.AUTHENTICATION_2FA);
      }

      if( sharedPrefs.getString(Constant.CURRENT_USER_DATA) != null ){

        User _currentUser = new User();
        await _currentUser.fromSharedPreferences();
        if( _currentUser.roleName =='OS' || _currentUser.roleName =='Customer' ){
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => Home(
                  title: '${Constant.APP_NAME}',
                ),
              )
          );
        }else{
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => ModuleSelector(),
              )
          );
        }

      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Stack(
          fit: StackFit.passthrough,
          children: [
            Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          Container(
                              padding: EdgeInsets.only(left: 16.0, right: 16.0),
                              child:Card(
                                // color: Colors.white,
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      Container(
                                        padding: EdgeInsets.all(16.0),
                                        child: GestureDetector(
                                          child: new Image.asset("assets/images/logo_rectangle.png", width: 180, height: 180,),
                                          onDoubleTap: (){
                                            _changeBaseUrl();
                                          },
                                        )
                                      ),
                                      formData(),
                                    ]
                                ),
                              )
                          ),
                          Container(
                            margin: const EdgeInsets.all(10.0),
                            child:Divider(thickness: 1,
                              // color: Theme.of(context).primaryColor,
                            )
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                            child: new Wrap(
                                direction: Axis.horizontal,
                                crossAxisAlignment: WrapCrossAlignment.end,
                                spacing: 45,
                                runSpacing: 45,
                                children: [
                                  FlatButton(
                                    color: Colors.transparent,
                                    // textColor: Colors.white,
                                    disabledColor: Colors.grey,
                                    disabledTextColor: Colors.black,
                                    padding: EdgeInsets.all(8.0),
                                    // splashColor: Colors.blueAccent,
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) => UserRegister( ),
                                        ),
                                      );
                                    },
                                    child: Text(
                                        tr(LocaleKeys.lblCreateAccount),
                                        style: TextStyle(
                                            decoration: TextDecoration.underline,
                                            // color: Theme.of(context).primaryColor
                                        )
                                    ),
                                  ),FlatButton(
                                    color: Colors.transparent,
                                    // textColor: Colors.white,
                                    disabledColor: Colors.grey,
                                    disabledTextColor: Colors.black,
                                    padding: EdgeInsets.all(8.0),
                                    // splashColor: Colors.blueAccent,
                                    onPressed: () {
                                      Navigator.push(
                                          context,
                                          MaterialPageRoute(
                                            builder: (BuildContext context) => ForgotPassword(),
                                          )
                                      );
                                    },
                                    child: Text(
                                        tr(LocaleKeys.lblGetPassword),
                                        style: TextStyle(
                                            decoration: TextDecoration.underline,
                                            // color: Theme.of(context).primaryColor
                                        )
                                    ),
                                  ),
//                          GestureDetector(
//                              child: Text("Forgot password?", style: TextStyle(decoration: TextDecoration.underline, color: Colors.blue)),
//                              onTap: () { }
//                          )
                                ]
                            ),
                          ),
                        ]
                    )
                )
            )
          ],
        )
    );
  }

  Form formData( ){

    return new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
        child:
        Container(
          padding: EdgeInsets.only(left: 16, right: 16),
        child: new Column(
        children: <Widget>[
          new TextFormField(
            decoration: InputDecoration(
              icon: Icon(Icons.phone),
              hintText: '',
              labelText: tr(LocaleKeys.lblUsername),
            ),
            validator: (val) => val.isEmpty ? 'Username (or) Phone number is required' : null,
            onSaved: (val) => _name = val,
            initialValue: '',
          ),

          new TextFormField(
            obscureText: true,
            decoration: InputDecoration(
              icon: const Icon(Icons.star),
              hintText: '',
              labelText: tr(LocaleKeys.lblPassword),
            ),
            validator: (val) => val.isEmpty ? 'Password is required' : null,
            onSaved: (val) => _password = val,
            initialValue: '',
          ),
          Container(
            margin: const EdgeInsets.only( bottom: 8.0, top: 8),
            child: Text(
              _message,
              softWrap: true,
              style: TextStyle(
                fontWeight: FontWeight.w400,
                color: _messageColor,
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.only(left: 0.0, bottom: 32.0, right: 0.0, top: 8),
            width: double.infinity,
            child: RaisedButton(
              onPressed: () { _submitForm();},
              color: Theme.of(context).primaryColor,
              elevation: 1,
              child: Text(
                tr(LocaleKeys.btnLoginMain),
                style: TextStyle(color: Colors.white),
              ),
              shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
            ),
          ),
        ])
        )
        );
  }

  void _submitForm() async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
    } else {
      form.save();

      setState(() {
        _processing = true;
      });

      UserService userService = new UserService();
      String accessToken = await userService.login( _name, _password, true );
      if( accessToken == null){
        _messageColor = Colors.red[600];
        _message = 'Invalid login, please try again!';
        _processing = false;
      }else{

        User _currentUser = new User();
        await _currentUser.fromSharedPreferences();
        if( _currentUser.roleName =='OS' || _currentUser.roleName =='Customer' ){
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => Home(
                  title: '${Constant.APP_NAME}',
                ),
              )
          );
        }else{
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => ModuleSelector(),
              )
          );
        }

        // Navigator.pushReplacement(
        //     context,
        //     MaterialPageRoute(
        //       builder: (BuildContext context) => Home(
        //             title: '${Constant.APP_NAME}',
        //           ),
        //     )
        // );
      }

      setState(() {
        _processing = false;
      });
    }
  }

  void showMessage(String message, [MaterialColor color = Colors.red]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: color, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white, )
      )
  );

  int _doubleTapCount = 0;
  Future<void> _changeBaseUrl() async{
    ++_doubleTapCount;
    if( _doubleTapCount == 1 ) {
      new ThreadTimer(new Duration(milliseconds: 1000 * 5), () {
        _doubleTapCount = 0;
      });
    }

    if( _doubleTapCount >= 3){
      _doubleTapCount = 0;
      SharedPreferences _sharedPrefs = await SharedPreferences.getInstance();
      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => SetupPrivateConnection(sharedPrefs: _sharedPrefs),
          )
      );
    }
  }
}

class Welcome extends StatefulWidget {
  Welcome({Key key}) : super(key: key);
  @override
  _WelcomeState createState() => _WelcomeState();
}

class _WelcomeState extends State<Welcome> {

  var _themeProvider;
  @override
  void initState() {
    super.initState();
    work();
    _loadDefaultLanguage();
  }
  Future work() async {
    new ThreadTimer(new Duration(milliseconds: 3000), () async{

      final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
      if( await getSettingsTempLogin() ){
        await setSettingsTempLogin(false);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA);
        await sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_REFRESH_TOKEN_VALUE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
        await sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
        await sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
        await sharedPrefs.remove(Constant.DEVICE_GUID);
        await sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
        await sharedPrefs.remove(Constant.AUTHENTICATION_2FA);
      }

      if( sharedPrefs.getString(Constant.CURRENT_USER_DATA) != null ){
        User _currentUser = new User();
        await _currentUser.fromSharedPreferences();
        if( _currentUser.roleName =='OS' || _currentUser.roleName =='Customer' ){
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => Home(
                  title: '${Constant.APP_NAME}',
                ),
              )
          );
        }else{
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => ModuleSelector(),
              )
          );
        }
      }else{
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => LoginUser(
                title: tr(LocaleKeys.login),
              ),
            )
        );
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    _themeProvider = Provider.of<ThemeChanger>(context);
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        body: _createBody(context)
    );
  }

  Widget _createBody(BuildContext context){
        return Stack(
          fit: StackFit.passthrough,
          children: [
            // _welcomePanel( true ),
        Center(
        child: SingleChildScrollView(
              child: Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 0.0, right: 0.0),
                  child: Card(
                    color: Colors.transparent,
                    elevation: 0,
                    child: new Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Container(
                          padding: const EdgeInsets.all(10.0),
                          child: ListTile(
                            title: Text(
                              'Welcome!',
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontSize: 52
                              ),
                            ),
                          ),
                        ),
                        Container(
                          margin: EdgeInsets.only(top:35.0),
                          child: new Image.asset("assets/images/logo_rectangle.png", width: 250, height: 250,),
                        ),
                        Container(
                          margin: EdgeInsets.only(left:10.0, top: 10),
                          child: ListTile(
                            title: Text(
                              Constant.APP_NAME,
                              style: TextStyle(
                                  color: Theme.of(context).primaryColor,
                                fontSize: 32,
                                fontWeight: FontWeight.bold
                              ),
                            ),
                          ),
                        ),
                        Container(
                          margin: EdgeInsets.only(left: (MediaQuery.of(context).size.width *.2), top: 0, bottom: 0, right: 0),
                          child: Text(
                            'Delivery Service',
                            style: TextStyle(
                                color: Theme.of(context).primaryColor,
                                fontSize: 32,
                                fontWeight: FontWeight.bold
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
            )
        )
          ],
        );
  }

  void _loadDefaultLanguage() async{
    String lang = await getLanguage();
    if( lang == Constant.LANGUAGE_CODE_EN){
      context.locale = Locale( Constant.LANGUAGE_CODE_EN,'');
    }
    else {
      context.locale = Locale( Constant.LANGUAGE_CODE_MY,'');
    }
    setState(() { });
    if( Constant.THEME_CHANGABLE)
      _loadDefaultTheme();
  }
  void _loadDefaultTheme() async{
    String name = await getTheme();
    _themeProvider.setTheme( getDefaultTheme( name ) );
    setState(() { });
  }
}


class ModuleSelector extends StatefulWidget {
  ModuleSelector({Key key}) : super(key: key);
  @override
  _ModuleSelectorState createState() => _ModuleSelectorState();
}

class _ModuleSelectorState extends State<ModuleSelector> {

  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {

    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          title: Text('Collections', style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Center(
            child: SingleChildScrollView(
              child: Container(
                margin: const EdgeInsets.only(top: 0.0, bottom: 0.0, left: 0.0, right: 0.0),
                child: Card(
                  color: Colors.transparent,
                  elevation: 0,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Container(
                        padding: const EdgeInsets.all(10.0),
                        child: ListTile(
                          leading: Icon(Icons.local_shipping_sharp),
                          title: Text('Delivery Management',),
                          subtitle: Text('Manage your delivery process here!'),
                          trailing: Icon(Icons.arrow_forward_sharp),
                          onTap: (){
                            Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (BuildContext context) => Home(
                                    title: '${Constant.APP_NAME}',
                                  ),
                                )
                            );
                          },
                        ),
                      ),
                      Container(
                        padding: const EdgeInsets.all(10.0),
                        child: ListTile(
                          title: Text('HR Management',),
                          leading: Icon(Icons.group_rounded),
                          subtitle: Text('Manage your HR process such as Attendance, Leave, Payroll, etc.'),
                          trailing: Icon(Icons.arrow_forward_sharp),
                          onTap: () async{
                            SharedPreferences _sharedPrefs = await SharedPreferences.getInstance();
                            if( _sharedPrefs.getString(config.Constant.CURRENT_USER_DATA) != null ){
                              Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: ( BuildContext context ) => hrHome.Home(
                                      title: '${config.Constant.APP_NAME}',
                                    ),
                                  )
                              );
                            }else{
                              Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: ( BuildContext context ) => LoginSelection(
                                      title: '${config.Constant.APP_NAME}',
                                    ),
                                  )
                              );
                            }
                          },
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            )
        )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white, )
      )
  );
}