import 'dart:async';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_category_list_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_category_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class POSCategoryList extends StatefulWidget {
  POSCategoryList({Key key}) : super(key: key);

  @override
  _POSCategoryListState createState() => _POSCategoryListState();
}

class _POSCategoryListState extends State<POSCategoryList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSCategoryListResponse _categoryListResponse;
  POSCategoryService _categoryService = new POSCategoryService();
  List<PosCategoryListItem> _items = [];
  Map<String, dynamic> _orders = {};
  int _page = 0;
  bool _first = true;
  int _itemCount = 0;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});
    Map<String,dynamic> request = {
      'categoryName': '',
      'displayName': '',
      'keyword': '',
      'first': _page,
      'max': Constant.ROW_PER_PAGE,
      'orderBy': 'name',
      'asc': true
    };

    _categoryListResponse = await _categoryService.list(request);
    if( _categoryListResponse !=null && _categoryListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _categoryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('POS - Product category list', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                bool response = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => CategoryCreate(
                      onChange: () {},
                      title: 'Add new category',
                      item: new PosCategoryListItem(
                          categoryId: 0,
                          categoryName:'',
                          displayName: '',
                          guid: ''
                      ),
                    ),
                  ),
                );
                if( response ){
                  _refreshController.requestRefresh();
                }
              },
              label: Text(
                'New category',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_circle_outline_sharp, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSCategoryItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () {
          _refreshController.requestRefresh();
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSCategoryItem extends StatefulWidget {

  POSCategoryItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final PosCategoryListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSCategoryItemState createState() => _POSCategoryItemState();

}

class _POSCategoryItemState extends State<POSCategoryItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.categoryName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      widget.post.displayName,
                        // style: TextStyle(color: Colors.black),
                    ),
                    trailing: IconButton(
                      icon: Icon(Icons.edit),
                      onPressed: () async{
                        bool response = await Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => CategoryCreate(
                              onChange: () {},
                              title: 'Edit ${widget.post.categoryName}',
                              item: widget.post,
                            ),
                          ),
                        );
                        if( response ){
                          widget.onChange();
                        }
                      },
                    ),
                  ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class CategoryCreate extends StatefulWidget {
  final String title;
  final Function onChange;
  final PosCategoryListItem item;
  CategoryCreate({Key key, @required this.title, this.onChange, this.item}) : super(key: key);

  @override
  _CategoryCreateState createState() => _CategoryCreateState();
}

class _CategoryCreateState extends State<CategoryCreate> {

  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  POSCategoryService _categoryService = new POSCategoryService();

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    setState(() {_processing = false;});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Category Name',
                ),
                validator: (val) => val.isEmpty ? 'Category name required!' : null,
                onSaved: (val) => widget.item.categoryName = val,
                initialValue: widget.item.categoryName,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Display Name',
                ),
                validator: (val) => val.isEmpty ? 'Display name required!' : null,
                keyboardType: TextInputType.text,
                onSaved: (val) => widget.item.displayName = val,
                initialValue: widget.item.displayName,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                        // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        widget.item.guid == ''?'Save' :'Update',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                        // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }

  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    setState(() {_processing = true;});

    Map<String, dynamic> result;
    result = await _categoryService.create(widget.item.toMap());
    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange();
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }

    setState(() {_processing = false;});
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}
