import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:path/path.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_order_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_order_list_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_order_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';


class POSOrderDetail extends StatefulWidget {
  POSOrderDetail({Key key, @required this.guid, this.showCancel}) : super(key: key);

  final String guid;
  final bool showCancel;
  @override
  _POSOrderDetailState createState() => _POSOrderDetailState();
}

class _POSOrderDetailState extends State<POSOrderDetail>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSOrderDetailResponse _detailResponse;
  POSOrderService _posOrderService = new POSOrderService();
  List<PosOrderItemInfoForm> _items = [];
  int _page = 0;
  bool _first = true;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});

    _detailResponse = await _posOrderService.detail(widget.guid);
    if( _detailResponse !=null && _detailResponse.items!=null ){
      if(_items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _detailResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(widget.showCancel?'POS - Sales':'POS - Sales canceled', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => i==0?_orderDetail(context) : POSOrderItem(
        post: _items[i-1],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length+1,
        currentUser: _currentUser,
        onChange: () async{
          _refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length+1,
    );
  }

  Widget _orderDetail( BuildContext context ){
    return _detailResponse==null? Container() : Container(
        child: Wrap(
            children: <Widget>[
              Card(
                // color: Colors.white,
                  elevation: 0,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        ListTile(
                          title: Text(
                            'Customer: ${_detailResponse.buyerName}',
                          ),
                          subtitle: Text(
                            'Order code: ${_detailResponse.orderCode}',
                          ),
                        ),
                        ListTile(
                          title: Text(
                            'Phone: ${_detailResponse.contactNo}',
                          ),
                          trailing: Text(
                            'Pickup: ${new DateFormat('dd/MM/yyyy').format(_detailResponse.orderDate)}',
                          ),
                          subtitle: Text(
                            _detailResponse.deliveryAddress,
                          ),
                        ),
                        ListTile(
                          trailing: Text(
                            'After discount: ${new NumberFormat("#,###").format(_detailResponse.paidAmount)} Ks',
                          ),
                          title: Text(
                            'Total: ${new NumberFormat("#,###").format(_detailResponse.totalAmount)} Ks',
                          ),
                          subtitle: Text(
                            'Discount: ${new NumberFormat("#,###").format(_detailResponse.discountAmount)} Ks',
                          ),
                        ),
                        Container(
                          padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                          child: Text("Cash to collect: ${_detailResponse.itemPayment=='notpaid'?'Yes':'No'}",),
                        ),
                        ButtonBar(
                          children: <Widget>[
                            RaisedButton.icon(
                              elevation: 0.0,
                              icon: Icon(Icons.receipt_long_sharp,
                              ),
                              color: Colors.green[700],
                              onPressed: ()async{
                                String token = await _userService.getToken();
                                String uri = '/v1/forwarder/pos_order_invoice_pdf/${widget.guid}?at=$token';
                                  if (await canLaunch( '${getBaseURL(_sharedPrefs)}$uri' )) {
                                    await launch( '${getBaseURL(_sharedPrefs)}$uri' );
                                  } else {
                                    throw 'Could not launch $url';
                                  }
                              },
                              label: Text(
                                'Invoice',
                                style: TextStyle(
                                  fontSize: 16,
                                  fontWeight: FontWeight.w400,
                                  // color: Colors.white,
                                ),
                              ),
                              shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                            ),
                            widget.showCancel? RaisedButton.icon(
                              elevation: 0.0,
                              icon: Icon(Icons.cancel_schedule_send_sharp, color: Colors.white),
                              color: Colors.red[700],
                              onPressed: _processing? null : ()async{

                                bool  result = await _showConfirm(context);
                                if ( ! result ){
                                  return;
                                }

                                Map<String, dynamic> response = await _posOrderService.orderCancel({'guid':widget.guid});
                                if( response['success']){
                                  showMessage( response['message']);
                                  Timer(Duration(seconds: 2), () {
                                    Navigator.pop(context, true);
                                  });
                                }
                              },
                              label: Text(
                                'Order cancel',
                                style: TextStyle(
                                  fontSize: 16,
                                  fontWeight: FontWeight.w400,
                                  // color: Colors.white,
                                ),
                              ),
                              shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                            ) : Container(),
                          ],
                        )
                        // Container(
                        //   padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                        //   child: Text("Total amount: ${new NumberFormat("#,###").format(widget.post.salePrice*widget.post.quantity)} Ks",),
                        // ),
                      ]
                  )
              ),
            ]
        )
    );
  }


  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm", style: TextStyle(color: Colors.black),),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.green]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSOrderItem extends StatefulWidget {

  POSOrderItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final PosOrderItemInfoForm post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSOrderListItemState createState() => _POSOrderListItemState();

}

class _POSOrderListItemState extends State<POSOrderItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.displayName,
                    ),
                    trailing: Text("${widget.post.quantity} pcs",),
                    subtitle: Text(
                      widget.post.productCode,
                    ),
                  ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Unit price: ${new NumberFormat("#,###").format(widget.post.salePrice)} pcs",),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Total amount: ${new NumberFormat("#,###").format(widget.post.salePrice*widget.post.quantity)} Ks",),
                    ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

}
