import 'dart:async';
import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:image_picker/image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_category_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_product_list_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_category_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_product_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:qq_delivery/delivery/utils/photo_zoom.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:image/image.dart' as imglib;


class POSProductList extends StatefulWidget {
  POSProductList({Key key}) : super(key: key);

  @override
  _POSProductListState createState() => _POSProductListState();
}

class _POSProductListState extends State<POSProductList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSProductListResponse _productListResponse;
  POSProductService _productService = new POSProductService();
  List<POSProductListItem> _items = [];
  Map<String, dynamic> _orders = {};
  int _page = 0;
  bool _first = true;
  int _itemCount = 0;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});
    Map<String,dynamic> request = {
      'productName': '',
      'keyword': '',
      'guid': null,
      'first': _page,
      'max': Constant.ROW_PER_PAGE,
      'orderBy': 'display_name',
      'asc': true
    };

    _productListResponse = await _productService.list(request);
    if( _productListResponse !=null && _productListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _productListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('POS - Product list', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                bool response = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => OrderCreate(
                      onChange: () {},
                      title: 'Add new product',
                      item: new POSProductListItem(
                        id: 0,
                        productCode:'',
                        displayName: '',
                        description: '',
                        guid: '',
                        imageGuid: [],
                        categoryGuid: []
                      ),
                    ),
                  ),
                );
                if( response ){
                  _refreshController.requestRefresh();
                }
              },
              label: Text(
                'New product',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_circle_outline_sharp, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSProductItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () {
          _refreshController.requestRefresh();
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSProductItem extends StatefulWidget {

  POSProductItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final POSProductListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSProductItemState createState() => _POSProductItemState();

}

class _POSProductItemState extends State<POSProductItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.displayName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      'Buy: ${new NumberFormat("#,###").format(widget.post.priceBuy)} Ks, Sell: ${new NumberFormat("#,###").format(widget.post.priceSell)} Ks',
                        // style: TextStyle(color: Colors.black),
                    ),
                    leading: IconButton(
                      icon: Icon(Icons.edit),
                      onPressed: () async{
                        bool response = await Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => OrderCreate(
                              onChange: () {},
                              title: 'Edit ${widget.post.displayName}',
                              item: widget.post,
                            ),
                          ),
                        );
                        if( response ){
                          widget.onChange();
                        }
                      },
                    ),
                    trailing: IconButton(
                      icon: Icon(Icons.add_a_photo_sharp),
                      onPressed: (){
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (context) => ProductPhotoList(
                              guid: widget.post.guid,
                            ),
                          ),
                        );
                      },
                    ),
                  ),
                    ListTile(
                      title: Text(
                        'Stock: ${new NumberFormat("#,###").format(widget.post.stockQty)}',
                      ),
                      subtitle: Text("Description: ${widget.post.description}",),
                    ),
                    // Container(
                    //   padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                    //   child: Text("Description: ${widget.post.description}",),
                    // ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class OrderCreate extends StatefulWidget {
  final String title;
  final Function onChange;
  final POSProductListItem item;
  OrderCreate({Key key, @required this.title, this.onChange, this.item}) : super(key: key);

  @override
  _OrderCreateState createState() => _OrderCreateState();
}

class _OrderCreateState extends State<OrderCreate> {

  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  String _lang = Constant.LANGUAGE_CODE_MY;
  POSProductService _productService = new POSProductService();

  POSCategoryListResponse _categoryListResponse;
  POSCategoryService _categoryService = new POSCategoryService();
  PosCategoryListItem categoryItem;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _categoryListResponse = await _categoryService.list({});

    if( widget.item.guid !=null && widget.item.guid.isNotEmpty){
      POSProductDetailItem detailItem = await _productService.getByGuid({'guid':widget.item.guid});
      widget.item.categoryGuid = detailItem.categoryGuid;
      _categoryListResponse.items.forEach((element) {
        detailItem.categoryGuid.forEach((elementExistingString) {
          if( element.guid == elementExistingString && categoryItem==null ){
            categoryItem = element;
            _controllerCategory.text = categoryItem!=null? categoryItem.categoryName : '';
            return;
          }
        });
      });
    }

    setState(() {_processing = false;});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _categorySelection(context),
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Product code',
                ),
                validator: (val) => val.isEmpty ? 'Product code required!' : null,
                onSaved: (val) => widget.item.productCode = val,
                initialValue: widget.item.productCode,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Display name',
                ),
                validator: (val) => val.isEmpty ? 'Display name required!' : null,
                keyboardType: TextInputType.text,
                onSaved: (val) => widget.item.displayName = val,
                initialValue: widget.item.displayName,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Buying price',
                ),
                validator: (val) => val.isEmpty ? 'Buying price required!' : null,
                keyboardType: TextInputType.number,
                onSaved: (val) => widget.item.priceBuy = val.isEmpty? 0 : int.parse(val),
                initialValue: widget.item.priceBuy==null?'':widget.item.priceBuy.toString(),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: "",
                  labelText: 'Selling price',
                ),
                validator: (val) => val.isEmpty ? 'Selling price required!' : null,
                onSaved: (val) => widget.item.priceSell = val.isEmpty? 0 : int.parse(val),
                initialValue: widget.item.priceSell==null?'':widget.item.priceSell.toString(),
                keyboardType: TextInputType.number,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Quantity',
                ),
                validator: (val) => val.isEmpty ? 'Quantity required!' : null,
                keyboardType: TextInputType.number,
                onSaved: (val) => widget.item.stockQty = val.isEmpty? 0 : int.parse(val),
                initialValue: widget.item.stockQty==null? '' :widget.item.stockQty.toString(),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Description',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                onSaved: (val) => widget.item.description = val,
                initialValue: widget.item.description,
                keyboardType: TextInputType.text,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                        // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        widget.item.guid == ''?'Save' :'Update',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                        // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }


  final TextEditingController _controllerCategory = new TextEditingController();
  Widget _categorySelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Select the category',
              labelText: 'Category',
            ),
            controller: _controllerCategory,
          ),
          suggestionsCallback: (pattern) async {
            return await _categorySuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.categoryName }'),
              subtitle: Text('${suggestion.displayName}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            categoryItem = suggestion;
            _controllerCategory.text = categoryItem!=null? categoryItem.categoryName : '';
          },
          validator: (val) => val.isEmpty ? 'Category required!' : null,
        )
    );
  }

  Future<List<PosCategoryListItem>> _categorySuggestion(String query) async{
    List<PosCategoryListItem> matches = List();
    if( _categoryListResponse==null || _categoryListResponse.items == null)
      return matches;
    matches.addAll(_categoryListResponse.items);

    matches.retainWhere((s) {
      PosCategoryListItem addressBookItem = s;
      if( addressBookItem.categoryName.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.displayName.contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    setState(() {_processing = true;});

    if( categoryItem != null)
      widget.item.categoryGuid.add(categoryItem.guid);

    Map<String, dynamic> result;
    result = await _productService.create(widget.item.toMap());

    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange();
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }


    setState(() {_processing = false;});
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}


class ProductPhotoList extends StatefulWidget {
  ProductPhotoList({Key key, @required this.guid}) : super(key: key);

  final String guid;
  @override
  _ProductPhotoListState createState() => _ProductPhotoListState();
}

class _ProductPhotoListState extends State<ProductPhotoList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSProductService _productService = new POSProductService();
  bool _first = true;

  POSProductDetailItem detailItem;
  List<String> _items = [];

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});

    detailItem = await _productService.getByGuid({'guid':widget.guid});
    if( detailItem !=null && detailItem.imageGuid!=null ){
      if(_items.isNotEmpty)
        _items.removeRange(0, _items.length);

      detailItem.imageGuid.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            backgroundColor: Theme.of(context).backgroundColor,
            key: _scaffoldKey,
            appBar: AppBar(
              title: Text(detailItem !=null? detailItem.displayName + ' - Photo list' :'Product - Photo list', style: TextStyle(color: Colors.white)),
              bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            ),
            body: SmartRefresher(
              controller: _refreshController,
              enablePullUp: true,
              child: buildContent(context),

              physics: BouncingScrollPhysics(),
              header: CustomRefreshIndicator(),
              footer: ClassicFooter(
                loadStyle: LoadStyle.ShowWhenLoading,
                completeDuration: Duration(milliseconds: 500),
              ),

              // scroll down
              onRefresh: () async {
                await _getLoadData( true);
                if (mounted) setState(() {});
                _refreshController.refreshCompleted();
              },
              // scroll up
              onLoading: () async {
                await _getLoadData( false );
                if (mounted) setState(() {});
                _refreshController.loadComplete();
              },
            ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                _showMediaSelection( context );
              },
              label: Text(
                'New photo',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_circle_outline_sharp, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ProductPhoto(
        image: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () {_refreshController.requestRefresh();},
        sharedPrefs: _sharedPrefs,
        detailItem: detailItem,
        productService: _productService,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Future<void> _getImage( bool isCamera) async {

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/gelivery_tmp';
        await Directory(tmpDir).create(recursive: true);

        String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
        File rotatedFile = await FlutterImageCompress.compressAndGetFile(
          image.path,
          photoPath2,
          quality: 72,
          rotate: 0,
        );
        imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 600);
        String cropFilePath = '$tmpDir/cropped.jpg';
        File(cropFilePath)
          ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 72));

        File smallFile  =File(cropFilePath);
        String encodedString = encode( smallFile );
        smallFile.deleteSync(recursive: false);
        rotatedFile.deleteSync(recursive: false);
        String result = await _productService.addPhotoAndCategory({
          'productguid': widget.guid,
          'type':'image',
          'guid': encodedString
        });

        _refreshController.requestRefresh();
        // setState(() {
        //   widget.item.image = encodedString;
        // });
      }
    }catch(e){}

    // var image = await ImagePicker.pickImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
    // if( image != null ){
    //   var result = await FlutterImageCompress.compressWithList(
    //     image.readAsBytesSync(),
    //     minWidth: 800,
    //     quality: 80,
    //     rotate: 0,
    //   );
    //
    //   final Directory extDir = await getApplicationDocumentsDirectory();
    //   final String dirPath = '${extDir.path}/Pictures/quick_tmp';
    //   await Directory(dirPath).create(recursive: true);
    //   String cropFilePath = '$dirPath/cropped.jpg';
    //   File(cropFilePath)
    //     ..writeAsBytesSync(imglib.encodeJpg(imglib.decodeImage( result ), quality: 80));
    //   final smallFile = File(cropFilePath);
    //   String encodedString = encode( smallFile );
    //   smallFile.deleteSync(recursive: false);
    //   setState(() {
    //     widget.item.image = encodedString;
    //   });
    // }
  }

  Future<void> _showMediaSelection( BuildContext context )async {
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Select the media source", style: TextStyle(color: Colors.black),),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            RaisedButton.icon(
              elevation: 0.0,
              icon: Icon(Icons.camera_alt_sharp,
              ),
              color: Theme.of(context).primaryColor,
              onPressed: () {
                _getImage( true );
                Navigator.of(context).pop();
              },
              label: Text("Camera"),
              shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
            ),
            RaisedButton.icon(
              elevation: 0.0,
              icon: Icon(Icons.photo_album_sharp,
              ),
              color: Theme.of(context).primaryColor,
              onPressed: () {
                _getImage( false );
                Navigator.of(context).pop();
              },
              label: Text("Photos"),
              shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
            ),
          ],
        );
      },
    );
  }
}

class ProductPhoto extends StatefulWidget {

  ProductPhoto({Key key, @required this.detailItem, this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser, @required this.image,
  this.productService}) : super(key: key);
  final POSProductDetailItem detailItem;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String image;
  final POSProductService productService;

  @override
  _ProductPhotoState createState() => _ProductPhotoState();

}

class _ProductPhotoState extends State<ProductPhoto> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        ListTile(
                          title: GestureDetector(
                            child: ClipRRect(
                              borderRadius: BorderRadius.circular(24.0),
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.image}?size=500', fit: BoxFit.cover, height: 250,),
                            ),
                            onTap: () async{
                              if(widget.image!=null){
                                Navigator.push(
                                  context,
                                  MaterialPageRoute(
                                    builder: (context) => FullScreenWrapper(
                                      imageProvider: NetworkImage( '${getBaseURL(widget.sharedPrefs)}/v1/image/download/${widget.image}', ),
                                    ),
                                  ),
                                );
                              }
                            },
                          ),
                          trailing: IconButton(
                            icon: Icon(Icons.delete_forever_sharp),
                            onPressed: () async{
                              String result = await widget.productService.deleteProductImage(widget.image);
                              widget.onChange();
                            },
                          ),
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

