import 'dart:async';
import 'dart:convert';
import 'package:badges/badges.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/address_book_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_product_list_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_order_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_product_service.dart';
import 'package:qq_delivery/delivery/service/price_method_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/pos/pos_order_list.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class POSProductOrderList extends StatefulWidget {
  POSProductOrderList({Key key}) : super(key: key);

  @override
  _POSProductOrderListState createState() => _POSProductOrderListState();
}

class _POSProductOrderListState extends State<POSProductOrderList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSProductListResponse _productListResponse;
  POSProductService _productService = new POSProductService();
  List<POSProductListItem> _items = [];
  Map<String, dynamic> _orders = {};
  int _page = 0;
  bool _first = true;
  int _itemCount = 0;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});
    Map<String,dynamic> request = {
      'productName': '',
      'keyword': '',
      'guid': null,
      'first': _page,
      'max': Constant.ROW_PER_PAGE,
      'orderBy': 'display_name',
      'asc': true
    };

    _productListResponse = await _productService.list(request);
    if( _productListResponse !=null && _productListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _productListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('POS - Customer order', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              Container(
                  padding: EdgeInsets.all(16),
                  child: Badge(
                    badgeContent: Text('${_orders.length}'),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: IconButton(
                      highlightColor: Colors.red,
                      icon: Icon(Icons.shopping_cart_sharp, color: Colors.white,),
                      onPressed: () async{
                        _checkout( context );
                      },
                      tooltip: 'Cart',
                    ),
                  )
              ),
            ],
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                _checkout( context );
              },
              label: Text(
                'Checkout',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.shopping_bag_sharp, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }


  void _checkout(BuildContext context) async{
    if( _orders.isEmpty){
      _scaffoldKey.currentState.showSnackBar(new SnackBar(
          backgroundColor: Theme.of(context).primaryColor,
          content: new Text('The cart is empty!')
        )
      );
      return;
    }
    bool response = await Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => POSProductOrderConfirmList(
          onChange: ( key ){
            setState(() {
              _orders.remove(key);
            });
          },
          orders: _orders,
        ),
      ),
    );

    if( response ){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) =>
                POSOrderList( ),
          )
      );
    }
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSOrderItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: (key, value) async{
          setState(() {
            _orders.update( key, (v) => value, ifAbsent: () {_itemCount++; return value;});
          });
          //_refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSOrderItem extends StatefulWidget {

  POSOrderItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final POSProductListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSOrderListItemState createState() => _POSOrderListItemState();

}

class _POSOrderListItemState extends State<POSOrderItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.displayName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      'Unit price: ${new NumberFormat("#,###").format(widget.post.priceSell)} Ks',
                        // style: TextStyle(color: Colors.black),
                    ),
                    trailing: IconButton(
                      icon: Icon(Icons.add_shopping_cart_sharp),
                      onPressed: () async{
                        _controller_price.text = widget.post.priceSell.toString();
                        int result = await _showConfirm( context );
                        if( result > 0 ){//if( result > 0 && result <= widget.post.stockQty ){
                          widget.onChange(
                              widget.post.guid,
                              {'guid': widget.post.guid, 'quantity': result, 'price': int.parse(_controller_price.text), 'name':widget.post.displayName}
                          );
                        }else{
                          showMessage('Invalid quantity');
                        }
                      },
                    ),
                    leading: Text(
                      'Stock: ${new NumberFormat("#,###").format(widget.post.stockQty)}',
                    ),
                  ),
                    // Container(
                    //   padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                    //   child: Text("Phone: ${widget.post.contactNo}",),
                    // ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  final TextEditingController _controller_qty = new TextEditingController();
  final TextEditingController _controller_price = new TextEditingController();
  Future<int> _showConfirm( BuildContext context )async {
    _controller_qty.text = '';
    int result = 0;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Number of items", style: TextStyle(color: Colors.black),),
          content: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              new TextField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Selling price',
                ),
                controller: _controller_price,
                keyboardType: TextInputType.number,
              ),
              new TextField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Quantity',
                ),
                controller: _controller_qty,
                keyboardType: TextInputType.number,
              )
            ]
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = int.parse(_controller_qty.text??0);Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = 0; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}


class POSProductOrderConfirmList extends StatefulWidget {
  POSProductOrderConfirmList({Key key, @required this.orders, this.onChange}) : super(key: key);
  final Map<String, dynamic> orders;
  final Function onChange;

  @override
  _POSProductOrderConfirmListState createState() => _POSProductOrderConfirmListState();
}

class _POSProductOrderConfirmListState extends State<POSProductOrderConfirmList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  List<dynamic> _items = [];

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    _items.clear();
    widget.orders.forEach((key, value) {
      _items.add(value);
    });

    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            backgroundColor: Theme.of(context).backgroundColor,
            key: _scaffoldKey,
            appBar: AppBar(
              title: Text('Order confirm', style: TextStyle(color: Colors.white)),
            ),
            body: SmartRefresher(
              controller: _refreshController,
              enablePullUp: true,
              child: buildContent(context),

              physics: BouncingScrollPhysics(),
              header: CustomRefreshIndicator(),
              footer: ClassicFooter(
                loadStyle: LoadStyle.ShowWhenLoading,
                completeDuration: Duration(milliseconds: 500),
              ),

              // scroll down
              onRefresh: () async {
                await _getLoadData( true);
                if (mounted) setState(() {});
                _refreshController.refreshCompleted();
              },
              // scroll up
              onLoading: () async {
                await _getLoadData( false );
                if (mounted) setState(() {});
                _refreshController.loadComplete();
              },
            ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                bool response = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => OrderCreate(
                      orders: _items,
                      title: 'Delivery detail for this order',
                      onChange: (){
                      },
                    ),
                  ),
                );

                if( response ){
                  Navigator.pop(context, true);
                }
              },
              label: Text(
                'Confirm',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.done, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSOrderConfirmItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        index: i,
        onChange: (index, guid) async{
          setState(() {
            _items.removeAt(index);
            widget.onChange(guid);
          });
          //_refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}

class POSOrderConfirmItem extends StatefulWidget {

  POSOrderConfirmItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser, this.index}) : super(key: key);
  final dynamic post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final int index;

  @override
  _POSOrderConfirmItemState createState() => _POSOrderConfirmItemState();

}

class _POSOrderConfirmItemState extends State<POSOrderConfirmItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(//{'guid': widget.post.guid, 'quantity': result, 'price': widget.post.priceSell, 'name':widget.post.displayName}
                // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        ListTile(
                          title: Text(
                            widget.post['name'],
                            style: Theme.of(context).textTheme.headline6,
                          ),
                          trailing: Text(
                            '${new NumberFormat("#,###").format( widget.post['quantity'] * widget.post['price'])} Ks',
                            // style: TextStyle(color: Colors.black),
                          ),
                          leading: IconButton(
                            icon: Icon(Icons.delete_forever_rounded),
                            onPressed: () async{
                              widget.onChange( widget.index, widget.post['guid'] );
                            },
                          ),
                          subtitle: Text(
                            '${new NumberFormat("#,###").format(widget.post['quantity'])} pcs',
                          ),
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  final TextEditingController _controller_qty = new TextEditingController();
  Future<int> _showConfirm( BuildContext context )async {
    _controller_qty.text = '';
    int result = 0;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Number of items", style: TextStyle(color: Colors.black),),
          content: new TextField(
            decoration: InputDecoration(
              hintText: '',
              labelText: 'Quantity',
            ),
            controller: _controller_qty,
            keyboardType: TextInputType.number,
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = int.parse(_controller_qty.text??0);Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = 0; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class OrderCreate extends StatefulWidget {
  final String title;
  final Function onChange;
  final List<dynamic> orders;
  OrderCreate({Key key, @required this.title, this.onChange, this.orders}) : super(key: key);

  @override
  _OrderCreateState createState() => _OrderCreateState();
}

class _OrderCreateState extends State<OrderCreate> {

  SharedPreferences _sharedPrefs;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;

  TownshipService _townshipService = new TownshipService();
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;

  AddressBookService _addressBookService = new AddressBookService();
  AddressBookListResponse _addressBookListResponse;
  List<AddressBookItem> _items = [];
  AddressBookItem addressBookItem;

  AddressBookListResponse _addressBookListResponseDeliver;
  List<AddressBookItem> _itemsDeliver = [];
  AddressBookItem addressBookItemDeliver;


  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerPrimaryPhone = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerTotalAmount = new TextEditingController();
  final TextEditingController _controllerDiscountAmount = new TextEditingController();
  final TextEditingController _controllerAfterDiscountAmount = new TextEditingController();
  final TextEditingController _controllerPickupAddress = new TextEditingController();
  final TextEditingController _controllerDeliverAddress = new TextEditingController();
  final TextEditingController _controller_osDate = new TextEditingController();
  String _lang = Constant.LANGUAGE_CODE_MY;
  int _totalAmount = 0;
  DateTime _senderDate = new DateTime(DateTime.now().year, DateTime.now().month, DateTime.now().day+1);
  bool _itemPriceAlreadyPaid = false;
  bool _deliveryChargesAlreadyPaid = false;
  bool _fragile = false;
  bool _insurance = false;
  double _deliveryCharges;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    await _loadTownships(  );

    // pickup address list
    _addressBookListResponse = await _addressBookService.list(
        new AddressBookRequest(
            addressType: Constant.ROLE_NAME_OS,
            osGuid: _currentUser.guid,
            first: 0,
            max: 1000000,
            keyword: ''
        )
    );
    if( _addressBookListResponse!=null && _addressBookListResponse.items!=null){
      _addressBookListResponse.items.forEach((element) {
        _items.add(element);

        if(element.primaryContact && addressBookItem==null && element.addressType==Constant.ROLE_NAME_OS) {
          addressBookItem = element;
          _controllerPickupAddress.text =  addressBookItem.name +'(' + addressBookItem.townshipName + ')';
        }
      });
    }

    // existing customer list
    _addressBookListResponseDeliver = await _addressBookService.listAll(
        new AddressBookRequest(
            addressType: Constant.ROLE_NAME_CUSTOMER,
            osGuid: _currentUser.guid,
            first: 0,
            max: 1000000,
            keyword: ''
        )
    );
    if( _addressBookListResponseDeliver!=null && _addressBookListResponseDeliver.items!=null){
      _addressBookListResponseDeliver.items.forEach((element) {
        _itemsDeliver.add(element);
      });
    }



    //widget.post['price']
    widget.orders.forEach((element) {
      _totalAmount += element['price'] * element['quantity'];
    });
    _controllerTotalAmount.text = _totalAmount.toString();
    _controllerAfterDiscountAmount.text = _totalAmount.toString();
    setState(() {_processing = false;});

  }

  void _checkByPhone( ) async{
    // existing customer list
    _addressBookListResponseDeliver = await _addressBookService.listAll(
        new AddressBookRequest(
            addressType: Constant.ROLE_NAME_CUSTOMER,
            osGuid: _currentUser.guid,
            first: 0,
            max: 1000000,
            keyword: _controllerPrimaryPhone.text
        )
    );
    if( _addressBookListResponseDeliver!=null && _addressBookListResponseDeliver.items!=null){
      addressBookItemDeliver = _addressBookListResponseDeliver.items.isNotEmpty? _addressBookListResponseDeliver.items.first : null;
      if( addressBookItemDeliver !=null) {
        _controllerDeliverAddress.text =
        '${addressBookItemDeliver.name}${addressBookItemDeliver
            .primaryPhone == '' ? "" : " (" +
            addressBookItemDeliver.primaryPhone + ')'}';
        _controllerName.text = addressBookItemDeliver.name;
        _controllerAddress.text = addressBookItemDeliver.address +', '+ addressBookItemDeliver.street;
        _controllerPrimaryPhone.text =
            addressBookItemDeliver.primaryPhone;
        _controllerEmail.text = addressBookItemDeliver.email;

        _addressResponseTownship.items.forEach((element) {
          if( element.guid== addressBookItemDeliver.townshipGuid){

            _controllerTownship.text = _lang == Constant.LANGUAGE_CODE_MY
                ? element.nameMM
                : element.name;
            _addressItemTownship = element;
            return;

          }
        });
        _checkPrice( );
      }else{
        _controllerDeliverAddress.text = '';
        _controllerName.text = '';
        _controllerAddress.text = '';
        _controllerEmail.text = '';
        _addressItemTownship = null;
        _deliveryCharges = null;
      }
    }

  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _pickupAddressSelection(context),
              Container(
                margin: const EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                child:new Row(children: <Widget>[
                  new Expanded(
                      child: new TextFormField(
                        // style: TextStyle(
                        //   color: Colors.white,
                        // ),
                        decoration: new InputDecoration(
                          hintText: 'Date to pickup',
                          labelText: '${tr(LocaleKeys.lblDeliCreatePickupDate)}',
                        ),
                        controller: _controller_osDate,
                        keyboardType: TextInputType.datetime,
                        validator: (val) => isValidDob(val) ? null : 'Not a valid date',
                        onSaved: (val) => _senderDate = convertToDate(val),
                      )
                  ),
                  new IconButton(
                    icon: new Icon(Icons.calendar_today),
                    tooltip: 'Choose date',
                    onPressed: (() {
                      _chooseDate( context, _controller_osDate.text );
                    }),
                  )
                ]),
              ),
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            // _deliverAddressSelection(context),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                decoration: InputDecoration(
                  hintText: '09xxxxxxxxx',
                  labelText: tr(LocaleKeys.lblMobilePhone),
                ),
                validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                controller: _controllerPrimaryPhone,
                keyboardType: TextInputType.phone,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: RaisedButton.icon(
                elevation: 0.0,
                icon: Icon(Icons.done,
                  // color: Colors.white
                ),
                color: Theme.of(context).primaryColor,
                onPressed: _processing ?null : ()async {
                  _checkByPhone( );
                },
                label: Text(
                  'Check',
                  style: TextStyle(
                    fontSize: 16,
                    fontWeight: FontWeight.w400,
                    // color: Colors.white,
                  ),
                ),
                shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblName),
                ),
                validator: (val) => val.isEmpty ? 'Name required!' : null,
                controller: _controllerName,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: 'No, street, etc',
                  labelText: tr(LocaleKeys.lblAddress),
                ),
                validator: (val) => val.isEmpty ? 'House No. (or) Unit, street required!' : null,
                controller: _controllerAddress,
                keyboardType: TextInputType.text,
              ),
            ),
            _townshipSelection( context ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: 'name@domain.com',
                  labelText: tr(LocaleKeys.lblEmailAddress),
                ),
                validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                controller: _controllerEmail,
                keyboardType: TextInputType.emailAddress,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Total Amount',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                controller: _controllerTotalAmount,
                keyboardType: TextInputType.number,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Discount amount',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                controller: _controllerDiscountAmount,
                keyboardType: TextInputType.number,
                onChanged: (val){
                  if( val.isEmpty ){
                    _controllerAfterDiscountAmount.text = _controllerTotalAmount.text;
                  }else{
                    _controllerAfterDiscountAmount.text = (int.parse( _controllerTotalAmount.text ) - int.parse( _controllerDiscountAmount.text )).toString();
                  }
                },
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'After discount amount',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                controller: _controllerAfterDiscountAmount,
                keyboardType: TextInputType.number,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: CheckboxListTile(
                value: _fragile,
                onChanged: ( value) {setState(() { _fragile = value; });_checkPrice( );},
                title: Text(
                  'Fragile',
                  // style: TextStyle(color: Colors.black),
                ),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: CheckboxListTile(
                value: _insurance,
                onChanged: ( value) {setState(() { _insurance = value; });_checkPrice( );},
                title: Text(
                  'Insurance',
                  // style: TextStyle(color: Colors.black),
                ),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: CheckboxListTile(
                value: _itemPriceAlreadyPaid,
                onChanged: ( value) {setState(() { _itemPriceAlreadyPaid = value; });},
                title: Text(
                  'Item price already paid',
                  // style: TextStyle(color: Colors.black),
                ),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: CheckboxListTile(
                value: _deliveryChargesAlreadyPaid,
                onChanged: ( value) {setState(() { _deliveryChargesAlreadyPaid = value; });},
                title: Text(
                  'Delivery charges already paid',
                  // style: TextStyle(color: Colors.black),
                ),
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: ListTile(
                title: Text('Delivery charges'),
                subtitle: Text(_deliveryCharges !=null? _deliveryCharges.toStringAsFixed(0) : ''),
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                        // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        'Save',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                        // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }

  Future<void> _loadTownships( ) async{
    setState(() {_processing = true;});
    _addressResponseTownship = await _townshipService.list();
    setState(() {_processing = false;});
  }
  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type township name',
              labelText: tr(LocaleKeys.lblTownship),
            ),
            controller: _controllerTownship,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemTownship = suggestion;
            _checkPrice( );
          },
          validator: (val) => val.isEmpty ? 'Township (or) Town required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void _submitForm( ) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    if( _controllerTownship.text.trim().isEmpty){
      showMessage('Township required!');
      return;
    }

    setState(() {_processing = true;});

    Map<String, dynamic> request = {
      'buyerName': _controllerName.text,
      'contactNo': _controllerPrimaryPhone.text,
      'totalAmount': int.parse( _controllerTotalAmount.text.isEmpty? '0' : _controllerTotalAmount.text.trim()),
      'purchaseDetailList': widget.orders,
      'deliveryAddress': _controllerAddress.text,
      'emailAddress': _controllerEmail.text,
      'townShipGuid': _addressItemTownship.guid,
      'itemPayment': (_itemPriceAlreadyPaid? 'alreadypaid' : 'notpaid'),
      'orderCode': 'SN-00' + DateTime.now().millisecondsSinceEpoch.toString(),
      'pickUpDate':_senderDate!=null? new DateFormat('yyyy-MM-dd').format(_senderDate) : null,
      'paidAmount': int.parse(_controllerAfterDiscountAmount.text.isEmpty? '0' : _controllerAfterDiscountAmount.text.trim()),
      'discountType': 'amount',
      'discountAmount': int.parse(_controllerDiscountAmount.text.isEmpty? '0' : _controllerDiscountAmount.text.trim()),
      'pickUpAddress': addressBookItem.address,
      'pickupAddressBookGuid': addressBookItem.guid,
      'itemPriceAlreadyPaid': _itemPriceAlreadyPaid,
      'deliveryChargesAlreadyPaid': _deliveryChargesAlreadyPaid,
      'fragile': _fragile.toString(),
      'insurance': _insurance.toString()
    };

    POSOrderService posOrderService = new POSOrderService();
    Map<String, dynamic> result = await posOrderService.create( request );

    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange( );
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }


    setState(() {_processing = false;});
  }

  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{8,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );


  Widget _pickupAddressSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'OS name or phone',
              labelText: tr(LocaleKeys.lblDeliCreatePickup),
            ),
            controller: _controllerPickupAddress,
          ),
          suggestionsCallback: (pattern) async {
            return await _osAddressSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.name}${suggestion.primaryPhone=='' ? "" : " (" + suggestion.primaryPhone+')'}'),
              subtitle: Text('${suggestion.address}, ${suggestion.street}, ${suggestion.townshipName}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerPickupAddress.text =  suggestion.name +'(' + suggestion.townshipName + ')';
            addressBookItem = suggestion;
            _checkPrice( );
          },
          validator: (val) => val.isEmpty ? 'Pickup address required!' : null,
        )
    );
  }

  Future<List<AddressBookItem>> _osAddressSuggestion(String query) async{
    List<AddressBookItem> matches = List();
    if( _addressBookListResponse==null || _addressBookListResponse.items == null)
      return matches;
    matches.addAll(_addressBookListResponse.items);

    matches.retainWhere((s) {
      AddressBookItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.primaryPhone.contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Widget _deliverAddressSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Exiting customer name or phone',
              labelText: tr(LocaleKeys.lblDeliCreateDropOff),
            ),
            controller: _controllerDeliverAddress,
          ),
          suggestionsCallback: (pattern) async {
            return await _customerAddressSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.name}${suggestion.primaryPhone=='' ? "" : " (" + suggestion.primaryPhone+')'}'),
              subtitle: Text('${suggestion.address}, ${suggestion.street}, ${suggestion.townshipName}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            addressBookItemDeliver = suggestion;

            if( addressBookItemDeliver !=null) {
              _controllerDeliverAddress.text =
              '${addressBookItemDeliver.name}${addressBookItemDeliver
                  .primaryPhone == '' ? "" : " (" +
                  addressBookItemDeliver.primaryPhone + ')'}';
              _controllerName.text = addressBookItemDeliver.name;
              _controllerAddress.text = addressBookItemDeliver.address +', '+ addressBookItemDeliver.street;
              _controllerPrimaryPhone.text =
                  addressBookItemDeliver.primaryPhone;
              _controllerEmail.text = addressBookItemDeliver.email;

              _addressResponseTownship.items.forEach((element) {
                if( element.guid== addressBookItemDeliver.townshipGuid){

                  _controllerTownship.text = _lang == Constant.LANGUAGE_CODE_MY
                      ? element.nameMM
                      : element.name;
                  _addressItemTownship = element;
                  return;

                }
              });
              _checkPrice( );
            }
          },
          // validator: (val) => val.isEmpty ? 'Pickup address required!' : null,
        )
    );
  }

  Future<List<AddressBookItem>> _customerAddressSuggestion(String query) async{
    List<AddressBookItem> matches = List();
    if( _addressBookListResponseDeliver==null || _addressBookListResponseDeliver.items == null)
      return matches;
    matches.addAll(_addressBookListResponseDeliver.items);

    matches.retainWhere((s) {
      AddressBookItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.primaryPhone.contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }
  DateTime convertToDate(String input) {
    try
    {
      var d = new DateFormat.yMd().parseStrict(input);
      //print(d);
      return d;
    } catch (e) {
      return null;
    }
  }

  Future _chooseDate(BuildContext context, String initialDateString ) async {
    var now = _senderDate;
    var initialDate = convertToDate(initialDateString) ?? now;
    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: DateTime.now(),
        lastDate: new DateTime( DateTime.now().year, DateTime.now().month+1, DateTime.now().day )
    );

    if (result == null) return;
    setState(() {
      _senderDate = result;
      _controller_osDate.text = new DateFormat.yMd().format(result);
    });
  }

  Future<void> _checkPrice( ) async{
    setState(() {
      _processing = true;
    });
    try{
      DeliveryItem _deliveryItem = new DeliveryItem(
        name: 'Parcel',
        weight: 2,
        width: 2,
        height: 2,
        length: 2,
        priceOption: "Unpaid",
        price: double.parse( _controllerTotalAmount.text.isEmpty? '0.0' : _controllerTotalAmount.text.trim()),
        chargeOption: "Unpaid",
        charges: 0,
        forShowCharges: 0,
        fragile: _fragile,
        insurance: _insurance,
      );

      PriceMethodService priceMethodService = new PriceMethodService( );
      Map<String, dynamic> data = await priceMethodService.calculatePrice(
          _deliveryItem,
          _addressItemTownship.guid,
          addressBookItem.townshipGuid,
          _lang,
          null,
          null,
          addressBookItem.personGuid
      );
      if( data['success']){
        Map<String, dynamic> prices = data["data"];
        _deliveryCharges = prices['totalPrice'];
      }else{
        _deliveryCharges = null;
        showMessage(data['data']);
      }
    }catch(e){print(e);}
    setState(() {
      _processing = false;
    });

    if(_authorityResponse.authorities !=null
        && _authorityResponse.authorities.contains("MANAGER_WRITE")){
      // setState(() {
      //   _allow_cash_on_advanced = true;
      // });
    }
  }
}
