
import 'dart:async';
import 'package:badges/badges.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_product_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_supplier_list_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_product_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_purchase_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_supplier_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'pos_purchase_list.dart';


class POSProductPurchaseList extends StatefulWidget {
  POSProductPurchaseList({Key key}) : super(key: key);

  @override
  _POSProductPurchaseListState createState() => _POSProductPurchaseListState();
}

class _POSProductPurchaseListState extends State<POSProductPurchaseList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSProductListResponse _productListResponse;
  POSProductService _productService = new POSProductService();
  List<POSProductListItem> _items = [];
  Map<String, dynamic> _orders = {};
  int _page = 0;
  bool _first = true;
  int _itemCount = 0;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});
    Map<String,dynamic> request = {
      'productName': '',
      'keyword': '',
      'guid': null,
      'first': _page,
      'max': Constant.ROW_PER_PAGE,
      'orderBy': 'display_name',
      'asc': true
    };

    _productListResponse = await _productService.list(request);
    if( _productListResponse !=null && _productListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _productListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('POS - Product purchasing', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              Container(
                  padding: EdgeInsets.all(16),
                  child: Badge(
                    badgeContent: Text('${_orders.length}'),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: IconButton(
                      highlightColor: Colors.red,
                      icon: Icon(Icons.shopping_cart_sharp, color: Colors.white,),
                      onPressed: () async{
                        _checkout( context );
                      },
                      tooltip: 'Cart',
                    ),
                  )
              ),
            ],
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                _checkout( context );
              },
              label: Text(
                'Add to stock',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_to_photos, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }


  void _checkout(BuildContext context) async{
    if( _orders.isEmpty){
      _scaffoldKey.currentState.showSnackBar(new SnackBar(
          backgroundColor: Theme.of(context).primaryColor,
          content: new Text('The cart is empty!')
        )
      );
      return;
    }
    bool response = await Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => POSProductPurchaseConfirmList(
          onChange: ( key ){
            setState(() {
              _orders.remove(key);
            });
          },
          orders: _orders,
        ),
      ),
    );

    if( response ){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) =>
                POSPurchaseList( ),
          )
      );
    }
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSOrderItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: (key, value) async{
          setState(() {
            _orders.update( key, (v) => value, ifAbsent: () {_itemCount++; return value;});
          });
          //_refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSOrderItem extends StatefulWidget {

  POSOrderItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final POSProductListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSOrderListItemState createState() => _POSOrderListItemState();

}

class _POSOrderListItemState extends State<POSOrderItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.displayName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      'Buying price: ${new NumberFormat("#,###").format(widget.post.priceBuy)} Ks',
                        // style: TextStyle(color: Colors.black),
                    ),
                    trailing: IconButton(
                      icon: Icon(Icons.add_circle_outline),
                      onPressed: () async{
                        _controller_price.text = widget.post.priceBuy.toString();
                        int result = await _showConfirm( context );
                        if( result > 0 ){
                          widget.onChange(
                              widget.post.guid,
                              {'guid': widget.post.guid, 'quantity': result, 'price': int.parse(_controller_price.text), 'name':widget.post.displayName}
                          );
                        }else{
                          showMessage('Invalid quantity');
                        }
                      },
                    ),
                    leading: Text(
                      'Stock: ${new NumberFormat("#,###").format(widget.post.stockQty)}',
                    ),
                  ),
                    // Container(
                    //   padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                    //   child: Text("Phone: ${widget.post.contactNo}",),
                    // ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  final TextEditingController _controller_qty = new TextEditingController();
  final TextEditingController _controller_price = new TextEditingController();
  Future<int> _showConfirm( BuildContext context )async {
    _controller_qty.text = '';
    int result = 0;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Number of items", style: TextStyle(color: Colors.black),),
          content: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              new TextField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Buying price',
                ),
                controller: _controller_price,
                keyboardType: TextInputType.number,
              ),
              new TextField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Quantity',
                ),
                controller: _controller_qty,
                keyboardType: TextInputType.number,
              )
            ]
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = int.parse(_controller_qty.text??0);Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = 0; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}


class POSProductPurchaseConfirmList extends StatefulWidget {
  POSProductPurchaseConfirmList({Key key, @required this.orders, this.onChange}) : super(key: key);
  final Map<String, dynamic> orders;
  final Function onChange;

  @override
  _POSProductPurchaseConfirmListState createState() => _POSProductPurchaseConfirmListState();
}

class _POSProductPurchaseConfirmListState extends State<POSProductPurchaseConfirmList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  List<dynamic> _items = [];

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    _items.clear();
    widget.orders.forEach((key, value) {
      _items.add(value);
    });

    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            backgroundColor: Theme.of(context).backgroundColor,
            key: _scaffoldKey,
            appBar: AppBar(
              title: Text('Purchase confirm', style: TextStyle(color: Colors.white)),
            ),
            body: SmartRefresher(
              controller: _refreshController,
              enablePullUp: true,
              child: buildContent(context),

              physics: BouncingScrollPhysics(),
              header: CustomRefreshIndicator(),
              footer: ClassicFooter(
                loadStyle: LoadStyle.ShowWhenLoading,
                completeDuration: Duration(milliseconds: 500),
              ),

              // scroll down
              onRefresh: () async {
                await _getLoadData( true);
                if (mounted) setState(() {});
                _refreshController.refreshCompleted();
              },
              // scroll up
              onLoading: () async {
                await _getLoadData( false );
                if (mounted) setState(() {});
                _refreshController.loadComplete();
              },
            ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                bool response = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => PurchaseCreate(
                      orders: _items,
                      title: 'Supplier and total amount',
                      onChange: (){
                      },
                    ),
                  ),
                );

                if( response ){
                  Navigator.pop(context, true);
                }
              },
              label: Text(
                'Confirm',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.done, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSPurchaseConfirmItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        index: i,
        onChange: (index, guid) async{
          setState(() {
            _items.removeAt(index);
            widget.onChange(guid);
          });
          //_refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}

class POSPurchaseConfirmItem extends StatefulWidget {

  POSPurchaseConfirmItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser, this.index}) : super(key: key);
  final dynamic post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final int index;

  @override
  _POSPurchaseConfirmItemState createState() => _POSPurchaseConfirmItemState();

}

class _POSPurchaseConfirmItemState extends State<POSPurchaseConfirmItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(//{'guid': widget.post.guid, 'quantity': result, 'price': widget.post.priceSell, 'name':widget.post.displayName}
                // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        ListTile(
                          title: Text(
                            widget.post['name'],
                            style: Theme.of(context).textTheme.headline6,
                          ),
                          trailing: Text(
                            '${new NumberFormat("#,###").format( widget.post['quantity'] * widget.post['price'])} Ks',
                            // style: TextStyle(color: Colors.black),
                          ),
                          leading: IconButton(
                            icon: Icon(Icons.delete_forever_rounded),
                            onPressed: () async{
                              widget.onChange( widget.index, widget.post['guid'] );
                            },
                          ),
                          subtitle: Text(
                            '${new NumberFormat("#,###").format(widget.post['quantity'])} pcs',
                          ),
                        ),
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  final TextEditingController _controller_qty = new TextEditingController();
  Future<int> _showConfirm( BuildContext context )async {
    _controller_qty.text = '';
    int result = 0;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Number of items", style: TextStyle(color: Colors.black),),
          content: new TextField(
            decoration: InputDecoration(
              hintText: '',
              labelText: 'Quantity',
            ),
            controller: _controller_qty,
            keyboardType: TextInputType.number,
          ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = int.parse(_controller_qty.text??0);Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = 0; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class PurchaseCreate extends StatefulWidget {
  final String title;
  final Function onChange;
  final List<dynamic> orders;
  PurchaseCreate({Key key, @required this.title, this.onChange, this.orders}) : super(key: key);

  @override
  _PurchaseCreateState createState() => _PurchaseCreateState();
}

class _PurchaseCreateState extends State<PurchaseCreate> {

  SharedPreferences _sharedPrefs;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;

  POSSupplierService _supplierService = new POSSupplierService();
  POSSupplierListResponse _supplierListResponse;
  PosSupplierListItem _supplierListItem;


  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerPrimaryPhone = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerTotalAmount = new TextEditingController();
  String _lang = Constant.LANGUAGE_CODE_MY;
  int _totalAmount = 0;


  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    await _loadSuppliers(  );

    //widget.post['price']
    widget.orders.forEach((element) {
      _totalAmount += element['price'] * element['quantity'];
    });
    _controllerTotalAmount.text = _totalAmount.toString();
    setState(() {_processing = false;});

  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            _supplierSelection( context ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Total Amount',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                controller: _controllerTotalAmount,
                keyboardType: TextInputType.number,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                        // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        'Save',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                        // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }

  Future<void> _loadSuppliers( ) async{
    setState(() {_processing = true;});
    _supplierListResponse = await _supplierService.list({
      'keyword':'',
      'first':0,
      'max':1000000,
      'orderBy':'name',
      'asc':true
    });
    setState(() {_processing = false;});
  }
  Widget _supplierSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Supplier name',
              labelText: 'Supplier',
            ),
            controller: _controllerTownship,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.name}'),
              subtitle: Text('${suggestion.phone1}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = suggestion.name;
            _supplierListItem = suggestion;
          },
          validator: (val) => val.isEmpty ? 'Supplier required!' : null,
        )
    );
  }
  Future<List<PosSupplierListItem>> _townshipSuggestion(String query) async{
    List<PosSupplierListItem> matches = List();
    if( _supplierListResponse==null || _supplierListResponse.items == null)
      return matches;
    matches.addAll(_supplierListResponse.items);

    matches.retainWhere((s) {
      PosSupplierListItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.phone1.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void _submitForm( ) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    if( _controllerTownship.text.trim().isEmpty){
      showMessage('Township required!');
      return;
    }

    setState(() {_processing = true;});

    Map<String, dynamic> request = {
      'totalAmount': int.parse( _controllerTotalAmount.text ),
      'purchaseDetailList': widget.orders,
      'supplierGuid': _supplierListItem.guid,
    };

    POSPurchaseService purchaseService = new POSPurchaseService();
    Map<String, dynamic> result = await purchaseService.create( request );

    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange( );
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }


    setState(() {_processing = false;});
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}
