
import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_purchase_list_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_purchase_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class POSPurchaseList extends StatefulWidget {
  POSPurchaseList({Key key}) : super(key: key);

  @override
  _POSPurchaseListState createState() => _POSPurchaseListState();
}

class _POSPurchaseListState extends State<POSPurchaseList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  POSPurchaseListResponse _purchaseListResponse;
  POSPurchaseService _purchaseService = new POSPurchaseService();
  List<POSPurchaseListItem> _items = [];
  int _page = 0;
  bool _first = true;
  bool _search = false;
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});

    Map<String,dynamic> request = {
      'startDate': _startDate!=null? new DateFormat('yyyy-MM-dd').format(_startDate.toLocal()): null,
      'endDate': _endDate!=null? new DateFormat('yyyy-MM-dd').format(_endDate.toLocal()): null,
      'guid': null,
      'first': _page,
      'max': Constant.ROW_PER_PAGE,
      'orderBy': 'purchaselist.purchaseId',
      'asc': false
    };

    _purchaseListResponse = await _purchaseService.list(request);
    if( _purchaseListResponse !=null && _purchaseListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _purchaseListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('POS - Purchase list', style: TextStyle(color: Colors.white)),
            bottom: _search?_createSearchIndicator() : ( _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)) ),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel, color: Colors.white,)
                      : Icon(Icons.search, color: Colors.white,),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => POSPurchaseItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _refreshController.requestRefresh();
          // _page = 0;
          // await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 280.0),
      child: SizedBox(
          height: 280.0,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 0,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child:new Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )
                            ),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _search = !_search;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                if( _controller_start.text.isEmpty || _controller_end.text.isEmpty){
                  _startDate = null;
                  _endDate = null;
                }
                _search = !_search;
                _refreshController.requestRefresh();
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class POSPurchaseItem extends StatefulWidget {

  POSPurchaseItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final POSPurchaseListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _POSPurchaseItemState createState() => _POSPurchaseItemState();

}

class _POSPurchaseItemState extends State<POSPurchaseItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.supplierName,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text(
                      new DateFormat('dd/MM/yyyy').format(widget.post.purchaseDate),
                        // style: TextStyle(color: Colors.black),
                    ),
                    trailing: Text(
                      '${new NumberFormat("#,###").format(widget.post.totalAmount)} Ks',
                    ),
                    leading: Text(
                      '${new NumberFormat("#,###").format(widget.post.totalQuantity)} pcs',
                    ),
                  ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Phone: ${widget.post.contactNo}",),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("${widget.post.totalProduct} item(s) group",),
                    ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
