import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_order_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/purchase_sales_summary_response.dart';
import 'package:qq_delivery/delivery/service/pos/pos_order_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/pos/pos_order_detail.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class PurchaseSalesSummary extends StatefulWidget {
  PurchaseSalesSummary({Key key, this.status}) : super(key: key);

  final String status;
  @override
  _PurchaseSalesSummaryState createState() => _PurchaseSalesSummaryState();
}

class _PurchaseSalesSummaryState extends State<PurchaseSalesSummary>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  PurchaseSalesSummaryResponse _purchaseSalesSummaryResponse;
  POSOrderService _posOrderService = new POSOrderService();
  bool _first = true;
  bool _search = false;
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();


  @override
  void initState() {
    _getLoadData();
    super.initState();
  }

  Future<void> _getLoadData() async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});

    Map<String,dynamic> request = {
      'startDate': _startDate!=null? new DateFormat('yyyy-MM-dd').format(_startDate.toLocal()): null,
      'endDate': _endDate!=null? new DateFormat('yyyy-MM-dd').format(_endDate.toLocal()): null,
    };

    _purchaseSalesSummaryResponse = await _posOrderService.purchaseAndSalesSummary(request);
    if( _purchaseSalesSummaryResponse.startedOn == null ){
      _purchaseSalesSummaryResponse.startedOn = _startDate;
      _purchaseSalesSummaryResponse.endOn = _endDate;
      _purchaseSalesSummaryResponse.purchaseTotalInvoices = 0;
      _purchaseSalesSummaryResponse.purchaseAmount = 0;
      _purchaseSalesSummaryResponse.sellingTotalInvoices = 0;
      _purchaseSalesSummaryResponse.sellingAmount = 0;
      _purchaseSalesSummaryResponse.difference = 0;
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('Profit summary', style: TextStyle(color: Colors.white)),
            bottom: _search?_createSearchIndicator() : ( _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)) ),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel, color: Colors.white,)
                      : Icon(Icons.search, color: Colors.white,),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
            body: Center(
                child: SingleChildScrollView(
                    child: Container(
                      margin: const EdgeInsets.only(left: 16.0, right: 16.0, top: 16, bottom: 16),
                      child: Card(
                        child: _purchaseSalesSummaryResponse!=null? Column(
                          children: <Widget>[
                            Container(
                              padding: const EdgeInsets.all(24.0),
                              child: ClipOval(
                                  child: Material(
                                    elevation: 8,
                                    color: Theme.of(context).backgroundColor,
                                    child: InkWell(
                                      splashColor: Colors.red, // inkwell color
                                      child: SizedBox(width: 150,
                                        height: 150,
                                        child: Icon(Icons.monetization_on, size: 120,),
                                      ),
                                    ),
                                  )
                              ),
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 10.0, right: 10.0, top: 20),
                                child: ListTile(
                                  title: Text( new DateFormat('dd MMM, yyyy').format(_purchaseSalesSummaryResponse.startedOn.toLocal()) + ' ~ ' + new DateFormat('dd MMM, yyyy').format(_purchaseSalesSummaryResponse.endOn.toLocal())),
                                  subtitle: Text('Current date range'),
                                ),
                            ),
                            Container(
                              margin: const EdgeInsets.only(left: 8.0, right: 8.0, top: 16),
                              child: ListTile(
                                title: Text( 'Sales' ),
                                subtitle: Text('Total invoices: ${_purchaseSalesSummaryResponse.sellingTotalInvoices}'),
                                trailing: Text('${new NumberFormat("#,###").format(_purchaseSalesSummaryResponse.sellingAmount)} Ks'),
                              ),
                            ),
                            Container(
                              margin: const EdgeInsets.only(left: 8.0, right: 8.0, top: 16),
                              child: ListTile(
                                title: Text( 'Purchase' ),
                                subtitle: Text('Total invoices: ${_purchaseSalesSummaryResponse.purchaseTotalInvoices}'),
                                trailing: Text('${new NumberFormat("#,###").format(_purchaseSalesSummaryResponse.purchaseAmount)} Ks'),
                              ),
                            ),
                            Container(
                              margin: const EdgeInsets.only(left: 8.0, right: 8.0, top: 16, bottom: 24),
                              child: ListTile(
                                title: Text( 'Profit' ),
                                trailing: Text('${new NumberFormat("#,###").format(_purchaseSalesSummaryResponse.difference)} Ks'),
                              ),
                            ),
                          ]
                        ) : _createProgressIndicator(),
                    )
                    )
                )
            )
        )
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 280.0),
      child: SizedBox(
          height: 280.0,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 0,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child:new Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )
                            ),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _search = !_search;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                if( _controller_start.text.isEmpty || _controller_end.text.isEmpty){
                  _startDate = null;
                  _endDate = null;
                }
                _search = !_search;
                _getLoadData();
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}



class POSOrderItem extends StatefulWidget {

  POSOrderItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser, this.status}) : super(key: key);
  final POSOrderListItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String status;

  @override
  _POSOrderListItemState createState() => _POSOrderListItemState();

}

class _POSOrderListItemState extends State<POSOrderItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.buyerName,
                      style: widget.status=='Accept'?Theme.of(context).textTheme.headline6 : TextStyle(color: Colors.red[800]),
                    ),
                    leading: Text(
                      new DateFormat('dd/MM/yyyy').format(widget.post.orderDate),
                        // style: TextStyle(color: Colors.black),
                    ),
                    subtitle: Text(
                      '${new NumberFormat("#,###").format(widget.post.paidAmount)} Ks',
                    ),
                    // leading: Text(
                    //   '${new NumberFormat("#,###").format(widget.post.totalQuantity)} pcs',
                    // ),
                    trailing: Icon(Icons.arrow_forward_sharp),
                    onTap: () async{
                      bool isCancel = await Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => POSOrderDetail(
                              guid: widget.post.orderGuid,
                              showCancel: widget.status=='Accept',
                          ),
                        ),
                      );

                      if( isCancel ){
                        widget.onChange();
                      }
                    },
                  ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Quantity: ${widget.post.totalQuantity} pcs",),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Phone: ${widget.post.contactNo}",),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 16.0, top: 0.0, right: 8.0, bottom: 4),
                      child: Text("Address: ${widget.post.deliveryAddress}",),
                    ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
