import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'edit_password.dart';
import 'edit_photos.dart';



class ProfileHome extends StatefulWidget {
  ProfileHome({Key key}) : super(key: key);
  @override
  _ProfileHomeState createState() => _ProfileHomeState();
}

class _ProfileHomeState extends State<ProfileHome>{

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  bool _processing = false;
  bool _editPhotos = false;
  bool _editPassword = false;
  SharedPreferences _sharedPrefs;
  bool _terminate = false;
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_description = new TextEditingController();

  @override
  initState() {
    super.initState( );
    _loadProfile();
  }

  void _loadProfile( )async{
    await _currentUser.fromSharedPreferences( );
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          resizeToAvoidBottomInset: true,
          // resizeToAvoidBottomPadding: true,
          appBar: AppBar(
            title: Text('Profile', style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          _editPhotos? _photosEditPanel(context) : _photosPanel(context),
                          _userPanel(context),
                          _editPassword ? _loginEditPanel(context) : _loginPanel(context),
                          Container(
                              margin: const EdgeInsets.only(left: 10.0, right: 10.0, top: 20),
                              child: Card(
                                  child: ListTile(
                                    title: Text('Logout'),
                                    subtitle: Text('Press here to exit your account!'),
                                    trailing: Icon(Icons.exit_to_app,
                                        // color: Theme.of(context).primaryColor
                                    ),
                                    onTap: () async{
                                      _exit( context );
                                    },
                                  )
                              )
                          ),
                          !_terminate? Container(
                              margin: const EdgeInsets.only(left: 10.0, right: 10.0, top: 20, bottom: 32),
                              child: Card(
                                  child: ListTile(
                                    title: Text('Terminate my account'),
                                    subtitle: Text('Your personal information will be deleted and you cannot access again!'),
                                    trailing: Icon(Icons.delete_forever_outlined, color: Colors.red[700]),
                                    onTap: () {
                                      setState(() {
                                        _terminate = true;
                                      });
                                    },
                                  )
                              )
                          ) : Container(),
                          _terminate? Form(
                              key: _formKey,
                              autovalidate: true,
                              child: Container(
                                  padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0, bottom: 32),
                                  child:Card(
                                    // color: Colors.grey[50],
                                      elevation: 1,
                                      child: Container(
                                          padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                                          child:new Column(
                                              crossAxisAlignment: CrossAxisAlignment.start,
                                              children: <Widget>[
                                                new TextFormField(
                                                  decoration: const InputDecoration(
                                                    hintText: '',
                                                    labelText: 'Reasons of account termination',
                                                  ),
                                                  controller: _controller_description,
                                                  validator: (val) => val.isEmpty? 'Required' :  null,
                                                  keyboardType: TextInputType.multiline,
                                                ),
                                                Container(
                                                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                                                  child: Row(
                                                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                                    children: [
                                                      RaisedButton.icon(
                                                        elevation: 2.0,
                                                        icon: Icon(Icons.send, // color: Colors.white
                                                        ),
                                                        color: Theme.of(context).primaryColor,
                                                        onPressed:_processing ? null: () async{
                                                          await _terminateAccount(context);
                                                        },
                                                        label: Text(
                                                          'Send',
                                                          style: TextStyle(fontSize: 16, fontWeight: FontWeight.w400, color: Colors.white,),
                                                        ),
                                                      ),
                                                      RaisedButton.icon(
                                                        elevation: 2.0,
                                                        icon: Icon(Icons.cancel, // color: Colors.white
                                                        ),
                                                        color: Theme.of(context).primaryColor,
                                                        onPressed:_processing ? null: (){
                                                          setState(() {
                                                            _terminate = false;
                                                          });
                                                        },
                                                        label: Text(
                                                          'Cancel',
                                                          style: TextStyle(fontSize: 16, fontWeight: FontWeight.w400, color: Colors.white,),
                                                        ),
                                                      ),
                                                    ],
                                                  ),
                                                )
                                              ]
                                          )
                                      )
                                  )
                              )
                          ): Container(),
                        ]
                    )
                )
          )
        )
    );
  }

  Widget _photosPanel( BuildContext context ){
    return _currentUser!=null? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          // color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                ListTile(
                  trailing: Container(
                      child: RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.photo_library,
                          // color: Theme.of(context).primaryColor,
                        ),
                        // color: Theme.of(context).primaryColor,
                        onPressed:() async{
                          setState(() {
                            _editPhotos = true;
                            _editPassword = false;
                          });
                        },
                        label: Text(
                          'Edit',
                          style: TextStyle(
                            fontWeight: FontWeight.w400,
                            // color: Theme.of(context).primaryColor,
                          ),
                        ),
                      )
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(top:0.0, bottom: 32),
                  child: ClipOval(
                      child: Material(
                        elevation: 2,
                        color: Colors.white, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 150,
                            height: 150,
                            child: _currentUser.avatar!=null && _currentUser.avatar.length>10?
                            new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_currentUser.avatar}', width: 130, height: 130,)
                            : Icon(Icons.account_circle, size: 120,),
                          ),
                        ),
                      )
                  ),
                ),
              ]
          ),
        )
    ) : Container();
  }
  Widget _photosEditPanel(BuildContext context){
    return EditPhotos(
      avatar: _currentUser.avatar,
      sharedPrefs: _sharedPrefs,
      onDispose: (image, isSave) async{
        setState(() {
          _editPhotos = false;
        });
        if( isSave ){
          setState(() {
            _processing = true;
          });

          PersonService personService = new PersonService();
          Map<String, dynamic> response = await personService.updateAvatar(
            {
              "avatar":image,
              "guid":_currentUser.guid
            }
          );
          if( response['success']){
            showMessage(response['message']);
            UserService userService = new UserService();
            userService.getUserInformation();
            await _currentUser.fromSharedPreferences( );
          }else{
            showMessage(response['data']);
          }
          setState(() {
            _processing = false;
          });
        }
      },
      onChanged: (){
        setState(() {
          _processing = true;
        });
      },
      onCompleted: (message){
        if( message != 'OK')
          showMessage(message);
        setState(() {
          _processing = false;
        });
      },
    );
  }

  Widget _userPanel( BuildContext context ){
    return _currentUser.name!=null? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          // color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                _currentUser!=null? new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0),
                    child: ListTile(
                      title: Text('${_currentUser.name??''}'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Roles: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.roleName==Constant.ROLE_NAME_OS?'Merchant': (_currentUser.roleName==Constant.ROLE_NAME_EMPLOYEE?'Deliveryman':_currentUser.roleName) }',
                                          style: TextStyle( fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Phone: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.phone??''}',
                                          style: TextStyle(fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ) : Container(),
              ]
          ),
        )
    ) : Container();
  }

  Widget _loginPanel( BuildContext context ){
    return _currentUser!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          // color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit,
                        // color: Theme.of(context).primaryColor,
                      ),
                      // color: Theme.of(context).primaryColor,
                      onPressed:() {
                        setState(() {
                          _editPassword = true;
                          _editPhotos = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          // color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('Login security'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('username: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.phone??''}',
                                          style: TextStyle(fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Password: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '*** *** ***',
                                          style: TextStyle( fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _loginEditPanel(BuildContext context){
    return _currentUser!= null ? EditPassword(
      onDispose: (password, isSave, message) async{
        if( isSave ){
          if( message != 'OK'){
            showMessage(message);
          }else{
            setState(() {
              _editPassword = false;
            });
            await _changePassword(password);
          }
        }else{
          setState(() {
            _editPassword = false;
          });
        }
      },
    ) : new Container();
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
  Future<bool> _onWillPop() async {
    return true;
  }
  @override
  void deactivate() {
    super.deactivate( );
  }

  Future<void> _changePassword(String password) async{
    if( _processing) return null;
    setState(() {
      _processing = true;
    });
    UserService userService = new UserService();
    Map<String, dynamic> response = await userService.changePassword(
        {
          'guid': _currentUser.guid,
          'newPassword': password
        }
    );
    if( response['success']){
      showMessage(response['message']);
    }else{
      showMessage(response['data']);
    }
    setState((){_processing = false;});
  }

  Future<void> _exit( BuildContext context) async{
    // if( _processing) return;
    setState(() {
      _processing = true;
    });

    bool result = await _showConfirmDialog(context, "Logout confirm", "Are you sure to exit from App?");
    if ( !result ){
      setState(() {
        _processing = false;
      });
      return;
    }

    UserService userService = new UserService();
    await userService.logout( );
    Navigator.pop(context, 'exit');
  }

  Future<void> _terminateAccount( BuildContext context) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) { return;}
    setState(() {_processing = true;});

    bool result = await _showConfirmDialog(context, "Terminate confirm", "Are you sure to terminate your user account?");
    if ( !result ){
      setState(() {_processing = false;});
      return;
    }

    UserService userService = new UserService();
    Map<String, dynamic> response = await userService.changeStatus(
        {
          'guid': _currentUser.guid,
          'newPassword': 'Blocked'
        }
    );
    if( response['success']){
      showMessage('Your user account has been scheduled to terminate soon!');
      await userService.logout( );
      Navigator.pop(context, 'exit');

    }else{
      showMessage(response['data']);
      setState((){_processing = false;});
    }

  }

  Future<bool> _showConfirmDialog( BuildContext context, String title, String message )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text(title),
          content: new Text(message),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

}