import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/employee_clearance_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/active_way_count_by_town_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/view/delivery2/deliveryman_going_out.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class ActiveWayCountByTownList extends StatefulWidget {
  ActiveWayCountByTownList({Key key}) : super(key: key);

  @override
  _ActiveWayCountByTownListState createState() => _ActiveWayCountByTownListState();
}

class _ActiveWayCountByTownListState extends State<ActiveWayCountByTownList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  EmpployeeClearanceListRequest request;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  ActiveWayCountByTownResponse _townResponse;
  List<ActiveWayCountByTownItem> _items = [];
  int _page = 0;
  bool _first = true;


  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});

    DeliveryContentServiceV2 deliveryContentServiceV2 = new DeliveryContentServiceV2();
    _townResponse = await deliveryContentServiceV2.activeWayCountByTown(
        _page, Constant.ROW_PER_PAGE, 'township.town', true, await _userService.getToken());

    if( _townResponse !=null && _townResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _townResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(tr(LocaleKeys.lblActiveWaysByTownship), style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
        )
    );
  }


  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => EmployeeClearanceListItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _refreshController.requestRefresh();
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class EmployeeClearanceListItem extends StatefulWidget {

  EmployeeClearanceListItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final ActiveWayCountByTownItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _EmployeeClearanceListItemState createState() => _EmployeeClearanceListItemState();

}

class _EmployeeClearanceListItemState extends State<EmployeeClearanceListItem> {
  @override
  void initState() {
    super.initState();
  }
  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                  ListTile(
                    title: Text(
                      widget.post.town,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text('${tr(LocaleKeys.lblMenuPickupWays)} ${widget.post.pickupCount.toString()}\n${tr(LocaleKeys.lblMenuDeliverWays)} ${widget.post.deliverCount.toString()}'),
                    trailing: widget.currentUser.roleName==Constant.ROLE_NAME_EMPLOYEE?Icon(Icons.arrow_forward_sharp) : Icon(Icons.location_on_sharp),
                    onTap: widget.currentUser.roleName==Constant.ROLE_NAME_EMPLOYEE?(){
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) =>
                              DeliverymanGoingOut(
                                selectedIndex: 0,
                                title: tr(LocaleKeys.lblToPickingUp),
                                township: widget.post.town,
                              ),
                        ),
                      );
                    }:null,
                  ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 100,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}
