import 'dart:async';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_accept_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_assign_request.dart';
import 'package:qq_delivery/delivery/model/request/delivery_completed_request.dart';
import 'package:qq_delivery/delivery/model/response/active_deliveryman_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/warehouse_station_route_service.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class TransitDetail extends StatefulWidget {
  TransitDetail({Key key, this.onChange, this.wayId, this.wayType}) : super(key: key);
  final Function onChange;
  final String wayId;
  final String wayType;
  @override
  _TransitDetailState createState() => _TransitDetailState();

}
class _TransitDetailState extends State<TransitDetail> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  String _accessToken;

  User _currentUser = new User();
  UserService _userService = new UserService();
  WarehouseStationRouteService _stationRouteService = new WarehouseStationRouteService();
  DeliveryListItem _detailResponse;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  ActiveDeliverymanResponse _deliverymanResponse;
  DeliveryForDeliveryItem _employeeSearchItemAll;
  PersonService _personService = new PersonService();
  String _employeeGuid;
  bool _transfer = false;
  bool _firstLoad = true;
  RefreshController _refreshController = RefreshController(initialRefresh: true);

  @override
  void initState() {
    super.initState();
    // _loadData();
  }

  _loadData() async{
    setState(() {
      _processing = true;
    });

    if( _firstLoad ) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _firstLoad = false;
    }

    _accessToken = await _userService.getToken();
    _detailResponse = await _stationRouteService.detail(widget.wayId, _accessToken);

    if(_authorityResponse.authorities!=null ) {
      _employeeSearchResponse =
      await _stationRouteService.availableEmployeesByZone(
          _detailResponse.contentGUID, _accessToken);
    }
    if(_authorityResponse.authorities !=null
        && (_authorityResponse.authorities.contains("MANAGER_WRITE") || _authorityResponse.authorities.contains("OPERATOR_WRITE"))){
      _deliverymanResponse = await _personService.activeDeliveryman();
    }
    setState(() {_processing=false;});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(widget.wayId, style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: false,
            child: ListView(
              children: <Widget>[
                _detailResponse!=null && _authorityResponse.authorities !=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && ( _detailResponse.status==Constant.STATUS_CREATED || _detailResponse.status==Constant.STATUS_RETRY)?
                  _deliveryAssign( context ) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && _detailResponse.status==Constant.STATUS_ASSIGNED && _detailResponse.inboundBucketStatus !=null
                    && _detailResponse.osAddressType =='Station' && (_detailResponse.customerAddressType =='Station' || _detailResponse.customerAddressType =='OS')?
                _deliveryManagerTransfer( context ) : Container(),


                _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                    && ( (_detailResponse.inboundBucketStatus==Constant.BUCKET_STATUS_REQUEST && widget.wayType=='inbound')
                    || (_detailResponse.outboundBucketStatus==Constant.BUCKET_STATUS_REQUEST && widget.wayType=='outbound') )?
                _onBucketAccept( context ) : Container(),

                // _detailResponse!=null && _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE")
                //     && _detailResponse.status==Constant.STATUS_ON_WAY?
                // _deliveryManagerCancel( context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') && _detailResponse !=null
                    && _detailResponse.status== Constant.STATUS_ASSIGNED && _detailResponse.dropOffPersonGUID==_currentUser.guid
                    && _detailResponse.osAddressType == 'Station'?
                  _deliveryEmployeeAccept(context) : Container(),

                _detailResponse!=null && _authorityResponse.authorities!=null && !_authorityResponse.authorities.contains('MANAGER_WRITE')
                    && _authorityResponse.authorities.contains('EMPLOYEE_WRITE')
                && _detailResponse.status==Constant.STATUS_ON_WAY && _detailResponse.dropOffPersonGUID==_currentUser.guid
                    && _detailResponse.remark != 'Deliver' ?
                  _deliveryEmployeeComplete(context) : Container(),

                _detailResponse!=null
                    && _detailResponse.status != Constant.STATUS_COMPLETED
                    && _detailResponse.status != Constant.STATUS_CANCELED
                    && _detailResponse.status != Constant.STATUS_RETURN? _deliveryTrackingMap(context): Container(),
                _deliverySummary( context ),
                _deliveryPickup(context),
                _deliveryDropOff(context),
              ]
          ),
            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            // scroll down
            onRefresh: () {
              _loadData();
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
          ),
        )
    );
  }

  Widget _deliveryTrackingMap( BuildContext context ){
    LatLng startLocation= Constant.OFFICE_LOCATION;
    LatLng officeLocation= Constant.OFFICE_LOCATION;
    List<Map<String,dynamic>> items=[];
    DeliveryContactItem contactItemCustomer = DeliveryContactItem(
        personGuid: _detailResponse.customerGUID,
      name: _detailResponse.customerName,
      address: _detailResponse.osAddress??'',
      latitude: _detailResponse.customerLatitude,
      longitude: _detailResponse.customerLongitude
    );
    bool startRoute = false;

    String mapTitle = "Current way location";
    if( _detailResponse != null ){
      if( _detailResponse.status == Constant.STATUS_COMPLETED
      || _detailResponse.status == Constant.STATUS_CANCELED
      || _detailResponse.status == Constant.STATUS_RETURN){
        items = null;
        mapTitle = "No tracking available";
      }else if(_detailResponse.status == Constant.STATUS_CREATED
          || _detailResponse.status == Constant.STATUS_ASSIGNED ){


        startLocation= officeLocation;
        mapTitle = "Processing to deliver";
        if( _detailResponse.osLatitude != null && _detailResponse.osLongitude !=null) {
          items.add({
            'latitude': _detailResponse.osLatitude,
            'longitude': _detailResponse.osLongitude,
            'guid': _detailResponse.osGUID,
            'name': _detailResponse.osName,
            'phone': "${_detailResponse.osAddress??''}",
            'employee': false
          });
        }

        if( _detailResponse.customerLatitude != null && _detailResponse.customerLongitude !=null) {
          items.add({
            'latitude': _detailResponse.customerLatitude,
            'longitude': _detailResponse.customerLongitude,
            'guid': _detailResponse.customerGUID,
            'name': _detailResponse.customerName,
            'phone': "${_detailResponse.customerAddress??''}",
            'employee': false
          });
        }

      } else if(_detailResponse.status == Constant.STATUS_ON_WAY ){
        startLocation= officeLocation;
        mapTitle = "Deliver on way";
        if( _detailResponse.customerLatitude != null && _detailResponse.customerLongitude !=null) {
          items.add({
            'latitude': _detailResponse.customerLatitude,
            'longitude': _detailResponse.customerLongitude,
            'guid': _detailResponse.customerGUID,
            'name': _detailResponse.customerName,
            'phone': "${_detailResponse.customerAddress??''}",
            'employee': false
          });
        }
        if( _detailResponse.dropOffPersonLatitude != null && _detailResponse.dropOffPersonLongitude !=null) {
          startLocation= LatLng(_detailResponse.dropOffPersonLatitude, _detailResponse.dropOffPersonLongitude);
          items.add({
            'latitude': _detailResponse.dropOffPersonLatitude,
            'longitude': _detailResponse.dropOffPersonLongitude,
            'guid': _detailResponse.dropOffPersonGUID,
            'name': _detailResponse.dropOffPersonName,
            'phone': "${_detailResponse.dropOffPersonPrimaryPhone}",
            'employee': true
          });
        }

        if( _authorityResponse.authorities !=null && _authorityResponse.authorities.contains('EMPLOYEE_WRITE') )
          startRoute = true;
      }
    }
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblLiveTracking)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  trailing: Icon(Icons.zoom_in_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                  onTap: (){
                    Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) => TrackingMap(
                            startLocation: startLocation,
                            items: items,
                            title: mapTitle,
                            isEmployee: false,
                            navigate: true,
                            startRoute: startRoute,
                            contactItemCustomer: contactItemCustomer,
                          ),
                        )
                    );
                  },
                ),
                Container(
                  height: 250,
                  width: MediaQuery.of(context).size.width,
                  child: TrackingMap(
                    startLocation: startLocation,
                    items: items,
                    title: null
                  ),
                ),
              ],
            ),
        )
    ) : new Container();
  }

  Widget _deliverySummary( BuildContext context ){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  title: Text(
                    '${tr(LocaleKeys.lblSummary)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text('Last updated: '+ new DateFormat('dd/MM/yyyy hh:mm a').format( _detailResponse.updatedOn!=null? _detailResponse.updatedOn.toLocal(): DateTime.now().toLocal() ),
                      // style: TextStyle(color: Colors.blueGrey[600])
                  ),
                  trailing: Icon(Icons.local_shipping_sharp,
                      // color: Colors.blueGrey[600]
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                      '${tr(LocaleKeys.lblWayID)}: ${_detailResponse.trackingId}',
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    'Created date: ${new DateFormat('dd/MM/yyyy hh:mm a').format(_detailResponse.createdOn.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 24),
                  child: Text(
                      '${tr(LocaleKeys.lblStatus)} ${_createStatus(_detailResponse.status)}',
                      // style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryPickup( BuildContext context ){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    _detailResponse.pickupDate!=null? '${tr(LocaleKeys.lblDeliPickupDoneDate)}: ${new DateFormat('dd/MM/yyyy').format(_detailResponse.pickupDate.toLocal())}'
                        :'${tr(LocaleKeys.lblDeliCreatePickupDate)}: ${new DateFormat('dd/MM/yyyy').format(_detailResponse.pickupDate.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblName)}: ${_detailResponse.osName!=null? _detailResponse.osName : _detailResponse.outboundStationName}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderMobile)}: ${_detailResponse.osPrimaryPhone??''}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblSenderOtherPhones)}: ${_detailResponse.osOtherPhones??''}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 24),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${_detailResponse.osAddress??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                _detailResponse.osComment!=null && _detailResponse.osComment.isNotEmpty?Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 4.0, right: 4.0, bottom: 24),
                  child: Text(
                    "${tr(LocaleKeys.lblRemark)}: ${_detailResponse.osComment??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ) : Container(),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryDropOff( BuildContext context ){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                ListTile(
                  leading: _detailResponse!=null && _detailResponse.dropOffPersonAvatar!=null?ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${_detailResponse.dropOffPersonAvatar}', width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ): Icon(Icons.person,
                      // color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    '${tr(LocaleKeys.lblDeliveryInformation)}',
                    style: Theme.of(context).textTheme.headline6,
                  ),
                  subtitle: Text(
                      "${_detailResponse!=null? _detailResponse.dropOffPersonName+ (_detailResponse.dropOffPersonPrimaryPhone!=null? '('+ _detailResponse.dropOffPersonPrimaryPhone +')' :'') : ''}",
                      // style: TextStyle(color: Colors.black)
                  ),
                  trailing: Icon(Icons.done,
                    // color: Colors.blueGrey[600]
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    _detailResponse.dropOffDate!=null? '${tr(LocaleKeys.lblDeliCreateDropOffDate)}: ${new DateFormat('dd/MM/yyyy').format(_detailResponse.dropOffDate.toLocal())}'
                        :'${tr(LocaleKeys.lblDeliCreateDropOffDate)}: ${new DateFormat('dd/MM/yyyy').format(_detailResponse.dropOffDate.toLocal())}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientName)}: ${_detailResponse.customerName !=null? _detailResponse.customerName : _detailResponse.inboundStationName}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientMobile)}: ${_detailResponse.customerPrimaryPhone??''}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0),
                  child: Text(
                    '${tr(LocaleKeys.lblRecipientOtherPhones)}: ${_detailResponse.customerOtherPhones??''}',
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
                Container(
                  padding: const EdgeInsets.only(left: 16.0, top: 5.0, right: 5.0, bottom: 24),
                  child: Text(
                    "${tr(LocaleKeys.lblAddress)}: ${_detailResponse.customerAddress??''}",
                    // style: TextStyle(color: Colors.black),
                  ),
                ),
              ],
            )
        )
    ) : new Container();
  }

  Widget _deliveryAssign( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
              key: _formKey,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              child: new Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  ListTile(
                    title: Text(
                      '${tr(LocaleKeys.lblAssignToEmployee)}',
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle: Text("Assign to your deliveryman to deliver",
                        // style: TextStyle(color: Colors.blueGrey[600])
                    ),
                    trailing: Icon(Icons.assignment_ind,
                        // color: Colors.blueGrey[600]
                    ),
                  ),
                  _deliverySelection(context),
                  _remarkTextField(),
                  ButtonBar(
                    children: <Widget>[
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.person_pin_circle_sharp,
                            // color: Colors.white
                        ),
                        color: Colors.green[700],
                        onPressed: _employeeGuid==null? null : ()async{
                          await _assign();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblAcceptAndAssign)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                      RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.cancel_schedule_send_sharp,
                            // color: Colors.white
                        ),
                        color: Colors.red[700],
                        onPressed: _processing? null : ()async{
                          await _cancel();
                          widget.onChange();
                          setState(() {});
                        },
                        label: Text(
                          '${tr(LocaleKeys.lblCancelDelivery)}',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            // color: Colors.white,
                          ),
                        ),
                        shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                      ),
                    ],
                  )
                ],
              )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerTransfer( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblChangeEmployee)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Change your current assigned deliveryman with another",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.wifi_protected_setup_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _deliverySelection(context),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _onBucketAccept( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        'Accept the request',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text('Accept the ${_detailResponse.totalWays} parcel(s) that will be ' + (_detailResponse.inboundBucketStatus=='Request'? 'store-in' : 'store-out'),
                        // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.done_all,
                        // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done_all,
                            // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _bucketAccept();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            'Accept',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryManagerCancel( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        '${tr(LocaleKeys.lblCancelTheDelivery)}',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Cancel here if there have any issue",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.cancel_schedule_send_sharp,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel_schedule_send_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _cancel();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblCancelDelivery)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryEmployeeAccept( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        'Depart and stock out request',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("${tr(LocaleKeys.lblYouCanAcceptOrTransferToAnother)}",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.assignment_ind,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    CheckboxListTile(
                        value: _transfer,
                        onChanged: ( value) {setState(() { _transfer = value; });},
                      title: Text('${tr(LocaleKeys.lblTransferToAnother)}',
                        // style: TextStyle(color: Colors.black),
                      ),
                    ),
                    _transfer?_employeeSelection(context) : Container(),

                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        _transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.wifi_protected_setup_sharp,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _employeeGuid==null? null : ()async{
                            await _doTransfer();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            '${tr(LocaleKeys.lblTransfer)}',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ) : Container(),
                        !_transfer?RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done,
                              // color: Colors.white
                          ),
                          color: Colors.green[700],
                          onPressed: _processing? null : ()async{
                            await _onBucketRequest( true );
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            'Request',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ): Container(),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.cancel,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed: _processing? null : ()async{
                            await _reject();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            "${tr(LocaleKeys.btnCancel)}",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    )
                  ],
                )
            )
        )
    ) : new Container();
  }

  Widget _deliveryEmployeeComplete( BuildContext context){
    return _detailResponse!=null? Container(
        margin: EdgeInsets.only(left: 8, right: 8, bottom: 4, top: 12),
        child: Card(
            elevation: 8,
            // color: Colors.white,
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    ListTile(
                      title: Text(
                        _detailResponse.customerAddressType=='OS'? 'Handover to merchant' : 'Arrived and stock in request',
                        style: Theme.of(context).textTheme.headline6,
                      ),
                      subtitle: Text("Complete your delivery process",
                          // style: TextStyle(color: Colors.blueGrey[600])
                      ),
                      trailing: Icon(Icons.done_all,
                          // color: Colors.blueGrey[600]
                      ),
                    ),
                    _remarkTextField(),
                    ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.done_all,
                              // color: Colors.white
                          ),
                          color: Theme.of(context).primaryColor,
                          onPressed: _processing==null? null : ()async{
                            await _onCompleted();
                            widget.onChange();
                            setState(() {});
                          },
                          label: Text(
                            _detailResponse.customerAddressType=='OS'? 'Handover' : 'Request',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    ),
                  ],
                )
            )
        )
    ) : new Container();
  }

  final TextEditingController _controller_remark = new TextEditingController();
  Widget _remarkTextField(){
    return Container(
      padding: EdgeInsets.only(left: 16, right: 16, bottom: 16),
      child: new TextFormField(
        decoration: InputDecoration(
          hintText: '',
          labelText: '${tr(LocaleKeys.lblRemark)}*',
        ),
        validator: (val) => val.isEmpty ? 'Remark required' : null,
        controller: _controller_remark,
        keyboardType: TextInputType.text,
        // maxLines: null,
        // minLines: 1,
      ),
    );
  }

  // checked and correct
  Future<void> _assign() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAssignRequest request = new DeliveryAssignRequest(
      contentGuid: _detailResponse.contentGUID,
      personGuid: _employeeGuid,
      comment: _controller_remark.text,
    );

    Map<String, dynamic> resultData = await _stationRouteService.assign(_accessToken, request, "assign");
    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }
    setState(() {
      _processing = false;
    });
  }

  // checked and correct
  Future<void> _onBucketRequest( bool shouldConfirm ) async{

    if( shouldConfirm ){
      bool  result = await _showConfirm(context);
      if ( ! result ){
        return;
      }
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
      contentGuid: _detailResponse.contentGUID,
      comment: _controller_remark.text,
    );

    Map<String, dynamic> resultData = await _stationRouteService.process(_accessToken, request, "bucket_request");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  // checked and correct
  Future<void> _reject() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.contentGUID,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _stationRouteService.process(_accessToken, request, "reject");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  // checked and correct
  Future<void> _cancel() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.contentGUID,
        personGuid: null,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _stationRouteService.process(_accessToken, request, "cancel");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  // checked and correct
  Future<void> _doTransfer() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    if( _employeeGuid == null){
      showMessage( "Please select the deliveryman" );
      return;
    }

    if( _controller_remark.text.isEmpty){
      showMessage( "Remark required!" );
      return;
    }

    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.contentGUID,
        personGuid: _employeeGuid,
        comment: _controller_remark.text
    );

    Map<String, dynamic> resultData = await _stationRouteService.process(_accessToken, request, "transfer");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Future<void> _bucketAccept() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }
    setState(() {
      _processing = true;
    });
    DeliveryAcceptRequest request = new DeliveryAcceptRequest(
        contentGuid: _detailResponse.contentGUID,
        personGuid: null,
        comment: _controller_remark.text,
    );

    Map<String, dynamic> resultData = await _stationRouteService.process(_accessToken, request, "bucket_accept");

    if( resultData != null) {
      if (resultData["success"]) {
        _controller_remark.text = '';
        showMessage(resultData["message"]);
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  // checked and correct
  Future<void> _onCompleted() async{
    bool  result = await _showConfirm(context);
    if ( ! result ){
      return;
    }

    // if( _controller_remark.text.isEmpty){
    //   showMessage( "Remark required!" );
    //   return;
    // }

    setState(() {
      _processing = true;
    });

    DeliveryCompletedRequest request = new DeliveryCompletedRequest(
        contentGuid: _detailResponse.contentGUID,
        comment: _controller_remark.text,
    );

    Map<String, dynamic> resultData = await _stationRouteService.complete(_accessToken, request, "completed");

    if( resultData != null) {
      if (resultData["success"]) {
        // _controller_remark.text = '';
        // showMessage(resultData["message"]);
        // Timer(Duration(seconds: 2), () {
        //   Navigator.pop(context, true);
        // });
        _onBucketRequest( false );
      } else {
        showMessage(resultData["data"]);
      }
    }else{
      showMessage('Error: failed to process!');
    }

    setState(() {
      _processing = false;
    });
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 16, left: 16, right: 16),
        child: DropdownSearch<EmployeeForDeliveryItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeForDeliveryItem data) {
            setState(() {
              _employeeSearchItem = data;
              if( _employeeSearchItem != null){
                _employeeGuid = _employeeSearchItem.guid;
              }else{
                _employeeGuid = null;
              }
            });
          },
          dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle,
                      // color: Colors.black
                  ),
                  title: Text(tr(LocaleKeys.lblSelectEmployees),
                    // style: TextStyle(color: Colors.black),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name,
                  // style: TextStyle(color: Colors.black),
                ),

                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                  // leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  // color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.zoneName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = List();
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _deliverySelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && (_authorityResponse.authorities.contains("MANAGER_READ") || _authorityResponse.authorities.contains("OPERATOR_READ"))?
      Container(
          margin: EdgeInsets.only(top: 16, left: 16, right: 16, bottom: 16),
          child: DropdownSearch<DeliveryForDeliveryItem>(
            selectedItem: _employeeSearchItemAll,
            mode: Mode.DIALOG,
            showSearchBox: true,
            showClearButton: true,
            // autoFocusSearchBox: true,
            showSelectedItems: true,
            compareFn: (DeliveryForDeliveryItem i, DeliveryForDeliveryItem s) => i?.guid==s?.guid,
            label: tr(LocaleKeys.lblEmployee),
            onFind: (String filter) => _deliverymanSuggestion(filter),
            onChanged: (DeliveryForDeliveryItem data) {
              _employeeSearchItemAll = data;
              if( _employeeSearchItemAll != null){
                _employeeGuid = _employeeSearchItemAll.guid;
              }else{
                _employeeGuid = null;
              }
            },
            dropdownBuilder: (BuildContext context, DeliveryForDeliveryItem item ) =>
                Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                    leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                      width: 42, height: 42, ),
                  ),
                ),
            popupItemBuilder: (BuildContext context, DeliveryForDeliveryItem item, bool isSelected) =>
                Container(
                  margin: EdgeInsets.symmetric(horizontal: 8),
                  decoration: !isSelected ? null
                      : BoxDecoration(
                    border: Border.all(color: Theme.of(context).primaryColor),
                    borderRadius: BorderRadius.circular(5),
                    color: Colors.white,
                  ),
                  child: ListTile(
                    selected: isSelected,
                    title: Text(item.name),
                    subtitle: Text(item.personId),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                            ),
                          ),
                        )
                    ),
                  ),
                ),
          )
      ) : Container();
  }

  Future<List<DeliveryForDeliveryItem>> _deliverymanSuggestion(String query) async{
    List<DeliveryForDeliveryItem> matches = [];
    if( _deliverymanResponse.items == null)
      return matches;
    matches.addAll(_deliverymanResponse.items);

    matches.retainWhere((s) {
      DeliveryForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else if( employeeSearchItem.personId.toLowerCase().contains(query.toLowerCase())) {
        return true;
      }else
        return false;
    }
    );
    return matches;
  }

  Future<bool> _showConfirm( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm", style: TextStyle(color: Colors.black),),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

  String _createStatus(String status ){
    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
    }else
      title = status;
    return title;
  }

  void showMessage(String message, [MaterialColor color = Colors.green]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(content: new Text(message)));
  }
}
