
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/employee_list_for_delivery_response.dart';
import 'package:qq_delivery/delivery/model/response/zone_list_response.dart';
import 'package:qq_delivery/delivery/service/address_book_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/zone_service.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'transit_list_main.dart';


class TransitListHome extends StatefulWidget {
  TransitListHome({Key key, this.onChange, this.selectedIndex, this.title,
  this.wayType}) : super(key: key);
  final Function onChange;
  final int selectedIndex;
  final String title;
  final String wayType;
  @override
  _TransitListHomeState createState() => _TransitListHomeState();
}

class _TransitListHomeState extends State<TransitListHome> {

  User _currentUser = new User();
  UserService _userService = new UserService();
  DeliveryContentServiceV2 _deliveryContentService = new DeliveryContentServiceV2();
  AddressBookService _addressBookService = new AddressBookService();
  ZoneService _zoneService = new ZoneService();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  EmployeeListForDeliveryResponse _employeeSearchResponse;
  EmployeeForDeliveryItem _employeeSearchItem;
  String _employeeGuid;

  ZoneListResponse _zoneListResponse;
  ZoneItem _zoneItem;
  String _zoneGuid;


  String _accessToken;
  bool _search = false;
  bool _tapChange = false;
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_keyword = new TextEditingController();
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblQueue);
  int _pendingLeaveCount = 0;
  bool _processing = false;

  @override
  void initState() {
    super.initState();
    _title = widget.title;
    _loadData();
  }

  _loadData() async{
        _selectedIndex = widget.selectedIndex;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
        _accessToken = await _userService.getToken();
        _employeeSearchResponse = await _deliveryContentService.availableAllEmployees( _accessToken);
        _zoneListResponse = await _zoneService.list(_currentUser.branchGuid, _accessToken);

        setState(() {});
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom: _search?_createSearchIndicator() : ( _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)) ),
            actions: <Widget>[
              FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _reset();
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel,
                    color: Colors.white,
                  )
                      : Icon(Icons.search,
                    color: Colors.white,
                  ),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.list),
                label: _tabLabel(0),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.download_sharp),
                label: _tabLabel(1),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.local_shipping_sharp),
                label: _tabLabel(2),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.input),
                label: _tabLabel(3),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.shopping_bag_sharp),
                label: _tabLabel(4),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Theme.of(context).primaryColor,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  Widget _createBody(){

    List<String> status;
    if( _selectedIndex == 0 && !_search && !_tapChange){
      status = [Constant.STATUS_CREATED];
    }else if( _selectedIndex == 1 && !_search && !_tapChange){
      status = [Constant.STATUS_ASSIGNED];
    }else if( _selectedIndex == 2 && !_search && !_tapChange){
      status = [Constant.STATUS_ON_WAY];


    }else if( _selectedIndex == 3 && !_search && !_tapChange
        && ( widget.wayType=='inbound' || widget.wayType=='deliveryman') ){
      status = [Constant.STATUS_COMPLETED];
    }else if( _selectedIndex == 3 && !_search && !_tapChange
        && widget.wayType=='outbound' ){
      status = [Constant.BUCKET_STATUS_REQUEST];


    }else if( _selectedIndex == 4 && !_search && !_tapChange
        && ( widget.wayType=='inbound' || widget.wayType=='deliveryman') ){
      status = [Constant.STATUS_BUCKET];


    }else if( _selectedIndex == 4 && !_search && !_tapChange
        && widget.wayType=='outbound' ){
      status = [Constant.STATUS_BUCKET, Constant.STATUS_COMPLETED];


    }else{
      return new Container();
    }

    return TransitListMain(
      keyword: _controller_keyword.text??'',
      startDate: _startDate,
      endDate: _endDate,
      status: status,
      employeeGuid: _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _employeeGuid,
      zoneGuid: _zoneGuid,
      branchGuid: _currentUser.branchGuid,
      onChange: (){
        widget.onChange();
      },
      wayType: widget.wayType,
    );
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = _tabLabel(_selectedIndex);
        _reset();
      }else if( _selectedIndex == 1){
        _title = _tabLabel(_selectedIndex);
        _reset();
      }else if( _selectedIndex == 2){
        _title = _tabLabel(_selectedIndex);
        _reset();
      }else if( _selectedIndex == 3){
        _title = _tabLabel(_selectedIndex);
        _reset();
      }else if( _selectedIndex == 4){
        _title = _tabLabel(_selectedIndex);
        _reset();
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  String _tabLabel(int index){
    if( index == 0){
      return tr(LocaleKeys.lblOutboundToAssign);
    }else if( index == 1){
      if( widget.wayType =='inbound'){
        return tr(LocaleKeys.lblInboundScheduled);
      }else if( widget.wayType =='outbound'){
        return tr(LocaleKeys.lblOutboundScheduled);
      }else{
        return tr(LocaleKeys.lblHomeAdminToAccept);
      }
    }else if( index == 2){
      if( widget.wayType =='inbound'){
        return tr(LocaleKeys.lblInboundOnWay);
      }else if( widget.wayType =='outbound'){
        return tr(LocaleKeys.lblOutboundOnWay);
      }else{
        return tr(LocaleKeys.lblOutboundOnWay);
      }
    }else if( index == 3){
      if( widget.wayType =='inbound'){
        return tr(LocaleKeys.lblInboundRequest);
      }else if( widget.wayType =='outbound'){
        return tr(LocaleKeys.lblOutboundRequest);
      }else{
        return tr(LocaleKeys.lblOutboundRequested);
      }
    }else if( index == 4){
      if( widget.wayType =='inbound'){
        return tr(LocaleKeys.lblInboundBucket);
      }else if( widget.wayType =='outbound'){
        return tr(LocaleKeys.lblOutboundBucket);
      }else{
        return tr(LocaleKeys.lblOutboundBucket);
      }
    }
  }

  _reset(){
    _employeeSearchItem = null;
    _employeeGuid = null;
    _startDate = null;
    _controller_start.text = '';
    _endDate = null;
    _controller_end.text = '';
  }

  PreferredSize _createSearchIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 620.0),
      child: SizedBox(
          height: 620.0,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 0,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child:new Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        // _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('MANAGER_READ')?_employeeSelection(context) : new Container(),

                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new TextFormField(
                            decoration: new InputDecoration(
                              icon: const Icon(Icons.keyboard),
                              hintText: '${tr(LocaleKeys.lblExample)} ${tr(LocaleKeys.lblWayID)}',
                              labelText: tr(LocaleKeys.lblKeyword),
                            ),
                            controller: _controller_keyword,
                            keyboardType: TextInputType.text,
                            autofocus: true,
                          ),
                        ),
                        _employeeSelection(context),
                        _zoneSelection(context),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )
                            ),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _reset();
                _search = !_search;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                if( _controller_start.text.isEmpty || _controller_end.text.isEmpty){
                  _startDate = null;
                  _endDate = null;
                }
                _search = !_search;
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }


  Widget _employeeSelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_READ")?
      Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<EmployeeForDeliveryItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (EmployeeForDeliveryItem i, EmployeeForDeliveryItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeForDeliveryItem data) {
            _employeeSearchItem = data;
            if( _employeeSearchItem != null){
              _employeeGuid = _employeeSearchItem.guid;
            }else{
              _employeeGuid = null;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeForDeliveryItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                    width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeForDeliveryItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.zoneName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    ) : Container();
  }

  Future<List<EmployeeForDeliveryItem>> _employeeSuggestion(String query) async{
    List<EmployeeForDeliveryItem> matches = List();
    if( _employeeSearchResponse.items == null)
      return matches;
    matches.addAll(_employeeSearchResponse.items);

    matches.retainWhere((s) {
      EmployeeForDeliveryItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _zoneSelection( BuildContext context){
    return
      _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ")?
      Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<ZoneItem>(
          selectedItem: _zoneItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (ZoneItem i, ZoneItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblZone),
          onFind: (String filter) => _zoneSuggestion(filter),
          onChanged: (ZoneItem data) {
            _zoneItem = data;
            if( _zoneItem != null){
              _zoneGuid = _zoneItem.guid;
            }else{
              _zoneGuid = null;
            }
          },
          dropdownBuilder: (BuildContext context, ZoneItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllZone)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, ZoneItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text('${item.name}'),
                ),
              ),
        )
    ) : Container();
  }

  Future<List<ZoneItem>> _zoneSuggestion(String query) async{
    List<ZoneItem> matches = List();
    if( _zoneListResponse.items == null)
      return matches;
    matches.addAll(_zoneListResponse.items);

    matches.retainWhere((s) {
      ZoneItem zoneItem = s;
      if( zoneItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}
