
import 'package:easy_localization/easy_localization.dart';
import 'package:latlong/latlong.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/delivery_list_request.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/service/v2/delivery_content_service_v2.dart';
import 'package:qq_delivery/delivery/service/v2/warehouse_station_route_service.dart';
import 'package:qq_delivery/delivery/view/delivery2/tracking_map.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart' as google;
import 'transit_detail.dart';


class TransitListMain extends StatefulWidget {
  TransitListMain({Key key, this.keyword, this.startDate, this.endDate, this.osGuid,
    this.customerGuid, this.employeeGuid, this.zoneGuid,
    this.branchGuid, this.status, this.onChange, this.wayType}) : super(key: key);
  final String keyword;
  final DateTime startDate;
  final DateTime endDate;
  final String osGuid;
  final String customerGuid;
  final String employeeGuid;
  final String zoneGuid;
  final String branchGuid;
  final List<String> status;
  final Function onChange;
  final String wayType;

  @override
  _TransitListMainState createState() => _TransitListMainState();
}

class _TransitListMainState extends State<TransitListMain>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  DeliveryListResponse _deliveryListResponse;
  String _accessToken;
  WarehouseStationRouteService _stationRouteService = new WarehouseStationRouteService();
  DeliveryListRequest _listRequest = new DeliveryListRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<DeliveryListItem> _items = [];
  SharedPreferences _sharedPrefs;
  bool _isFirst = true;
  UserService userService = UserService();

  @override
  void initState() {
    super.initState();

    String orderBy = 'id';
    // if( widget.status[0] == Constant.STATUS_CREATED)
    //   orderBy = 'id';
    // if( widget.status[0] == Constant.STATUS_ASSIGNED)
    //   orderBy = 'id';
    // if( widget.status[0] == Constant.STATUS_ON_WAY)
    //   orderBy = 'customer.name_mm';
    // if( widget.status[0] == Constant.STATUS_COMPLETED)
    //   orderBy = 'updated_on';
    // if( widget.status[0] == Constant.STATUS_RETRY)
    //   orderBy = 'updated_on';

    _listRequest = new DeliveryListRequest(
        keyword: widget.keyword,
        startDate: widget.startDate,
        endDate: widget.endDate,
        // employeeGuid: widget.employeeGuid,
        zoneGuid: widget.zoneGuid,
        branchGuid: widget.branchGuid,
        status: widget.status,
        first: _page,
        max: Constant.ROW_PER_PAGE,
        orderBy: orderBy,
        dateParam: 'month',
        wayType: widget.wayType
    );
  }

  Future<void> _getLoadData( bool refresh ) async{

      if( _isFirst ){
        _isFirst = false;
        await _currentUser.fromSharedPreferences();
        await _authorityResponse.fromSharedPreferences();
        _sharedPrefs = await SharedPreferences.getInstance();
      }

      _accessToken = await userService.getToken();
      _listRequest.first = _page;
      _listRequest.max = Constant.ROW_PER_PAGE;
      _listRequest.employeeGuid = _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : _listRequest.employeeGuid;
      _deliveryListResponse = await _stationRouteService.list(_accessToken, _listRequest);
    if( _deliveryListResponse!=null && _deliveryListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _deliveryListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    try{
      if( _deliveryListResponse != null)
        widget.onChange(_deliveryListResponse.totalRecords);
    }catch(e){}

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => ResultItem(
        post: _items[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        deliveryContentService: _stationRouteService,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
          widget.onChange();
        },
        wayType: widget.wayType,
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),
      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        widget.onChange();
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class ResultItem extends StatefulWidget {

  ResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.authorityResponse, @required this.deliveryContentService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser,
  this.wayType}) : super(key: key);
  final DeliveryListItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final WarehouseStationRouteService deliveryContentService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String wayType;

  @override
  _ResultItemState createState() => _ResultItemState();

}

class _ResultItemState extends State<ResultItem> {

  int _selectedStep = 0;
  StepState _pickupStepState = StepState.indexed;
  StepState _dropOffStepState = StepState.indexed;
  bool _dropOffInactive = false;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              _deliveryPost( context,widget.post ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  Widget _deliveryPost( BuildContext context, DeliveryListItem post ){
    _stepState(post);

    return Container(
        padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
        child: Card(
          elevation: 8,
          // color: Colors.white,
          child: new Wrap(
            children: <Widget>[
              ListTile(
                title: //Text('Way ID: ${post.trackingId}', style: TextStyle(color: Colors.white),),
                _createText(post.status, post.trackingId, context, post),
                subtitle: Text(tr(LocaleKeys.lblDeliCreateDropOffDate)+ ' ' + (post.dropOffDate!=null?new DateFormat('dd/MM/yyyy').format(post.dropOffDate.toLocal()): '../../....'),
                  // style: TextStyle(color: Colors.blueGrey[600]),
                ),
                onTap: () async {
                  bool result = await Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (BuildContext context) => TransitDetail(
                          wayId: post.contentGUID,
                          onChange: (){
                            widget.onChange();
                          },
                          wayType: widget.wayType,
                        ),
                      )
                  );
                  if( result!=null && result ){
                    widget.onChange();
                  }
                },
                trailing: Icon(Icons.arrow_forward_sharp,
                  // color: Colors.blueGrey[600],
                ),
              ),

              Stepper(
                steps: [
                  Step(
                    isActive: true,
                    state: _pickupStepState,
                    title: Text('${post.osName!=null? post.osName : post.outboundStationName}'),
                    subtitle: Text('${tr(LocaleKeys.lblSenderMobile)} ${post.osPrimaryPhone??''}'),
                    content: GestureDetector(
                      child: Text(
                        '${tr(LocaleKeys.lblDeliCreatePickup)} ${post.osAddress}',
                      ),
                      onTap: () async {
                        makeCall(post.osPrimaryPhone);
                      },
                    ),
                  ),
                  Step(
                    isActive: _dropOffInactive,
                    state: _dropOffStepState,
                    title: Text('${post.customerName !=null? post.customerName : post.inboundStationName}'),
                    subtitle: Text('${tr(LocaleKeys.lblRecipientMobile)} ${post.customerPrimaryPhone??''}'),
                    content: GestureDetector(
                      child: Text('${tr(LocaleKeys.lblDeliCreateDropOff)} ${post.customerAddress}'),
                      onTap: () async {
                        makeCall(post.customerPrimaryPhone);
                      }
                    ),
                  ),
                ],
                currentStep: _selectedStep,
                controlsBuilder: (BuildContext context,ControlsDetails details) => Container(),
                type: StepperType.vertical,
                physics: ClampingScrollPhysics(),
                onStepTapped: (step){setState(() {
                  _selectedStep = step;
                });},
              ),
            ],
          ),
        )
    );
  }

  Icon _createStatusIcon(String status ){

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED
        || status == Constant.STATUS_ON_WAY ){
      return Icon(Icons.get_app_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_PICKUP_CREATED
        || status == Constant.STATUS_PICKUP_SCHEDULED
        || status == Constant.STATUS_PICKUP_ON_WAY){
      return Icon(Icons.upload_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_COMPLETED ){
      return Icon(Icons.done_all, color: Colors.white);
    }else if( status == Constant.STATUS_CANCELED ){
      return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    }else if( status == Constant.STATUS_RETRY ){
      return Icon(Icons.refresh, color: Colors.white);
    }else if( status == Constant.STATUS_RETURN ){
      return Icon(Icons.backspace_sharp, color: Colors.white);
    }else
      return Icon(Icons.view_list, color: Colors.white);

    // if( status == Constant.STATUS_CREATED || status == Constant.STATUS_PICKUP_CREATED){
    //   return Icon(Icons.view_list, color: Colors.white);
    // }else if( status == Constant.STATUS_ASSIGNED || status == Constant.STATUS_PICKUP_SCHEDULED){
    //   return Icon(Icons.person_pin_circle, color: Colors.white);
    // }else if( status == Constant.STATUS_ON_WAY || status == Constant.STATUS_PICKUP_ON_WAY){
    //   return Icon(Icons.local_shipping_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_COMPLETED ){
    //   return Icon(Icons.done_all, color: Colors.white);
    // }else if( status == Constant.STATUS_CANCELED ){
    //   return Icon(Icons.cancel_schedule_send_sharp, color: Colors.white);
    // }else if( status == Constant.STATUS_RETRY ){
    //   return Icon(Icons.refresh, color: Colors.white);
    // }else if( status == Constant.STATUS_RETURN ){
    //   return Icon(Icons.backspace_sharp, color: Colors.white);
    // }else
    //   return Icon(Icons.view_list, color: Colors.white);

  }

  Text _createText(String status, String tracking, BuildContext context, DeliveryListItem post ){

    Color statusColor = Colors.white;
    FontWeight fontWeight = FontWeight.normal;

    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
      statusColor = Colors.green[800];
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
      statusColor = Colors.green[900];
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
      statusColor = Colors.teal;
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
      statusColor = Colors.red[700];
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
      statusColor = Colors.yellow[700];
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
      statusColor = Colors.grey;
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
      statusColor = Colors.deepOrangeAccent;
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
      statusColor = Colors.deepOrange;
    }

    //statusColor = Colors.teal;
    fontWeight = FontWeight.bold;
    statusColor = Colors.black;

    if( status == Constant.STATUS_CREATED
        || status == Constant.STATUS_ASSIGNED){
      statusColor = Theme.of(context).primaryColor;
    }else if( status == Constant.STATUS_PICKUP_CREATED
    || status == Constant.STATUS_PICKUP_SCHEDULED){
      statusColor = Colors.black;
    }else if( status == Constant.STATUS_ON_WAY
        || status == Constant.STATUS_PICKUP_ON_WAY){
      statusColor = Colors.teal;
    }

    if( post.customerName==null && post.customerAddress==null ){
      statusColor = Colors.yellow[700];
      title =  tr(LocaleKeys.lblDeliParcelWaiting);
    }
    if( status == Constant.STATUS_BUCKET ){
      title =  tr(LocaleKeys.lblInboundBucket);
      statusColor = Colors.black;
    }
    return Text(
      '$title\n${tr(LocaleKeys.lblWayID)} $tracking',
      style: TextStyle(
        color: statusColor,
        fontWeight: fontWeight,
      ),
    );
  }

  StepState _stepState(DeliveryListItem post){
    if( post.status == Constant.STATUS_PICKUP_CREATED
        || post.status == Constant.STATUS_PICKUP_SCHEDULED
        || post.status == Constant.STATUS_PICKUP_ON_WAY ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.indexed;
      _selectedStep = 0;

    }else if( post.status == Constant.STATUS_CANCELED ){
      _pickupStepState = StepState.error;
      _dropOffStepState = StepState.error;
      _selectedStep = 0;

    }else if( post.status == Constant.STATUS_CREATED
        || post.status == Constant.STATUS_ASSIGNED
        || post.status == Constant.STATUS_ON_WAY ){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.indexed;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( post.status == Constant.STATUS_COMPLETED){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 1;

    }else if( post.status == Constant.STATUS_RETURN){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.error;
      _selectedStep = 1;
    }

    if( post.customerName==null && post.customerAddress==null ){
      _pickupStepState = StepState.indexed;
      _dropOffStepState = StepState.editing;
      _selectedStep = 0;
    }

    if( post.status == Constant.STATUS_BUCKET){
      _pickupStepState = StepState.complete;
      _dropOffStepState = StepState.complete;
      _dropOffInactive = true;
      _selectedStep = 0;
    }

    return _pickupStepState;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}