import 'dart:async';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/view/user/user_create.dart';
import 'package:qq_delivery/delivery/view/user/user_list.dart';
import 'package:flutter/material.dart';


class CustomerList extends StatefulWidget {
  CustomerList({Key key}) : super(key: key);
  @override
  _CustomerListState createState() => _CustomerListState();
}
class _CustomerListState extends State<CustomerList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  String _title = '${Constant.ROLE_NAME_CUSTOMER} account list';
  bool _tapChange = false;

  @override
  void initState() {
    super.initState();
    _loadProfile();
  }

  void _loadProfile() async{
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: _tapChange? Container() : UserLoader(roleName: Constant.ROLE_NAME_CUSTOMER,),
          floatingActionButton: FloatingActionButton.extended(
            onPressed: (){
              _addNew();
            },
            label: Text(
              'Add New Account',
              style: TextStyle(color: Colors.white),
            ),
            icon: Icon(Icons.add,
                color: Colors.white
            ),
            backgroundColor: Theme.of(context).primaryColor,
          ),
        )
    );
  }

  void _addNew(){
    AddressBookItem item = new AddressBookItem(
      addressType: Constant.ROLE_NAME_CUSTOMER,
      name: '',
      address: '',
      street: '',
      townshipName: '',
      primaryPhone: '',
      otherPhones: '',
      email: '',
      comment: '',
    );
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => UserCreate(
          title: 'Create ${item.addressType} new account',
          item: item,
          onChange: (){
            setState(() {
              _tapChange = true;
            });
            new Timer(new Duration(milliseconds: 500), () {
              setState(() {
                _tapChange = false;
              });
            });
          },
        ),
      ),
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}
