import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:threading/threading.dart';



class ForgotPassword extends StatefulWidget {
  ForgotPassword({Key key}) : super(key: key);

  @override
  _ForgotPasswordState createState() => _ForgotPasswordState( );
}

class _ForgotPasswordState extends State<ForgotPassword> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  String _phoneNumber;
  bool _codeSent = false;
  int _resendCount = 0;
  IconData _topIcon = Icons.lock_open_sharp;

  @override
  void initState() {
    super.initState();

  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Get password', style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: Stack(
            fit: StackFit.passthrough,
            children: [
              // _backgroundPanel( true ),
              Center(
                  child: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  // color: Colors.white,
                                  elevation: 1,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                          margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                          child: Text('Get password',
                                              style: TextStyle(
                                                  // color: Theme.of(context).primaryColor,
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 18
                                              )
                                          ),
                                        ),
                                        Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                // color: Colors.white, // button color
                                                child: InkWell(
                                                  // splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                    height: 150,
                                                    child: Icon( _topIcon, size: 100,
                                                      // color: Theme.of(context).primaryColor,
                                                    ),
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                          child: formData(context),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )
              )
            ]
          )
        )
    );
  }

  Form formData(BuildContext context ){
    return new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.phone),
                  hintText: '09xxxxxxxxx',
                  labelText: 'Mobile phone number',
                ),
                validator: (val) => val.isEmpty ? 'Phone number required' : null,
                onSaved: (val) => _phoneNumber = val,
                keyboardType: TextInputType.number,
                maxLength: 11,
                autofocus: true,
              ),
              Container(
                margin: const EdgeInsets.only(left: 20.0, right: 5.0, top: 10),
                child:Text(
                  "Password will be sent to your mobile phone via SMS. Please change your password after logging in.",
                  style: new TextStyle(fontSize: 12.0,
                      // color: Colors.black87
                  ),
                ),
              ),
               new Wrap(
                children: [
                  Container(
                    padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                    child: RaisedButton.icon(
                      onPressed:_codeSent?null: () { _submitForm(context); },
                      elevation: 1,
                      icon: Icon(Icons.send_sharp,
                          color: Colors.white
                      ),
                      // color: Theme.of(context).primaryColor,
                      label: Text(
                        'Send',
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  ),
                  Container(
                    padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                    child: RaisedButton.icon(
                      onPressed:_processing?null: () { Navigator.pop(context);},
                      elevation: 1,
                      icon: Icon(Icons.person_search_sharp,
                          color: Colors.white
                      ),
                      label: Text(
                        'Login again',
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  ),
                ]
               ),
            ])
    );
  }

  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
          image:
          MediaQuery.of(context).orientation == Orientation.portrait ?
          new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
              : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  void _submitForm( BuildContext context ) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Required mobile phone number!', context);
      return;
    }
    form.save();

    _resend( context );
  }

  void _resend( BuildContext context ) async{
    if( _codeSent || _processing ) return;
    setState(() {
      _codeSent = true;
      _processing = true;
    });

    PersonService validationService = new PersonService();
    Map<String, dynamic> result = await validationService.generatePassword( _phoneNumber );

    if( result['success'] ){
      ++_resendCount;
      if( _resendCount < 3 ) {
        new ThreadTimer(new Duration(milliseconds: 1000 * 10), () {
          setState(() {
            _codeSent = false;
          });
        });
      }
      _topIcon = Icons.done;
      showMessage("Password already sent to your registered mobile.", context);
    }else{
      showMessage(result['data'], context);
    }

    setState(() {
      _processing = false;
    });
  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}