import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/bank_list_response.dart';
import 'package:qq_delivery/delivery/model/response/person_bank_list_response.dart';
import 'package:qq_delivery/delivery/model/response/pos/pos_product_list_response.dart';
import 'package:qq_delivery/delivery/service/bank_service.dart';
import 'package:qq_delivery/delivery/service/pos/pos_product_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';


class PersonBankAccountList extends StatefulWidget {
  PersonBankAccountList({Key key}) : super(key: key);

  @override
  _PersonBankAccountListState createState() => _PersonBankAccountListState();
}

class _PersonBankAccountListState extends State<PersonBankAccountList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = tr(LocaleKeys.lblDeposit);
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  PersonBankListResponse _personBankListResponse;
  BankService _bankService = new BankService();
  List<PersonBankItem> _items = [];
  int _page = 0;
  bool _first = true;

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _first ){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _first = false;
    }
    setState(() {});
    _personBankListResponse = await _bankService.listPersonBank( _page, Constant.ROW_PER_PAGE, 'bank.name', true, '');
    if( _personBankListResponse !=null && _personBankListResponse.items!=null ){
      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _personBankListResponse.items.forEach((element) {
        _items.add(element);
      });
    }
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text('Banking account list', style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: SmartRefresher(
            controller: _refreshController,
            enablePullUp: true,
            child: buildContent(context),

            physics: BouncingScrollPhysics(),
            header: CustomRefreshIndicator(),
            footer: ClassicFooter(
              loadStyle: LoadStyle.ShowWhenLoading,
              completeDuration: Duration(milliseconds: 500),
            ),

            // scroll down
            onRefresh: () async {
              _page = 0;
              await _getLoadData( true);
              if (mounted) setState(() {});
              _refreshController.refreshCompleted();
            },
            // scroll up
            onLoading: () async {
              ++_page;
              await _getLoadData( false );
              if (mounted) setState(() {});
              _refreshController.loadComplete();
            },
          ),
            floatingActionButton: FloatingActionButton.extended(
              onPressed: () async{
                bool response = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => BankAccountCreate(
                      onChange: () {},
                      title: 'Add new account',
                      item: new PersonBankItem(
                        bankGuid: null,
                        accountHolderName:'',
                        accountNumber: '',
                        accountType: 'Saving',
                        description: '',
                        guid: '',
                        active: true,
                      ),
                    ),
                  ),
                );
                if( response ){
                  _refreshController.requestRefresh();
                }
              },
              label: Text(
                'New account',
                // style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_circle_outline_sharp, color: Colors.white,),
              backgroundColor: Theme.of(context).primaryColor,
            )
        )
    );
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => BankAccountItem(
        post: _items[i],
        authorityResponse: _authorityResponse,
        isEnd: i+1 == _items.length,
        currentUser: _currentUser,
        onChange: () {
          _refreshController.requestRefresh();
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}

class BankAccountItem extends StatefulWidget {

  BankAccountItem({Key key, @required this.post, @required this.authorityResponse,
    this.onChange, @required this.sharedPrefs, @required this.isEnd,
    @required this.currentUser}) : super(key: key);
  final PersonBankItem post;
  final AuthorityResponse authorityResponse;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;

  @override
  _BankAccountItemState createState() => _BankAccountItemState();

}

class _BankAccountItemState extends State<BankAccountItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    ListTile(
                      title: Text('Account name: ${widget.post.accountHolderName}',),
                      subtitle: Text('Account number: ${widget.post.accountNumber}',),
                      trailing: IconButton(
                        icon: Icon(Icons.edit),
                        onPressed: () async{
                          bool response = await Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (context) => BankAccountCreate(
                                onChange: () {},
                                title: 'Edit ${widget.post.accountHolderName}',
                                item: widget.post,
                              ),
                            ),
                          );
                          if( response ){
                            widget.onChange();
                          }
                        },
                      ),
                    ),
                    ListTile(
                      title: Text('Bank name: ${widget.post.bankName}',),
                      subtitle: Text('Description: ${widget.post.description}'),
                    ),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}

class BankAccountCreate extends StatefulWidget {
  final String title;
  final Function onChange;
  final PersonBankItem item;
  BankAccountCreate({Key key, @required this.title, this.onChange, this.item}) : super(key: key);

  @override
  _BankAccountCreateState createState() => _BankAccountCreateState();
}

class _BankAccountCreateState extends State<BankAccountCreate> {

  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  String _lang = Constant.LANGUAGE_CODE_MY;
  BankService _bankService = new BankService();
  BankListResponse _bankListResponse;
  BankItem _bankItem;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    await _loadBanks( );

    if( widget.item.bankGuid != null ){
      _bankListResponse.items.forEach((element) {
        if( element.guid == widget.item.bankGuid){
          _bankItem = element;
          _controllerBank.text = _bankItem.name;
          return;
        }
      });
    }

    setState(() {_processing = false;});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            _bankSelection( context ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Account name',
                ),
                validator: (val) => val.isEmpty ? 'Account name required!' : null,
                onSaved: (val) => widget.item.accountHolderName = val,
                initialValue: widget.item.accountHolderName,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Account number',
                ),
                validator: (val) => val.isEmpty ? 'Account number required!' : null,
                keyboardType: TextInputType.text,
                onSaved: (val) => widget.item.accountNumber = val,
                initialValue: widget.item.accountNumber,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: 'Description',
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                onSaved: (val) => widget.item.description = val,
                initialValue: widget.item.description,
                keyboardType: TextInputType.text,
                // maxLines: null,
                // minLines: 1,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                        // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        widget.item.guid == ''?'Save' :'Update',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                        // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),
    );
  }

  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    setState(() {_processing = true;});

    Map<String, dynamic> result;
    result = widget.item.guid!=null && widget.item.guid.trim().isNotEmpty? await _bankService.update(widget.item.toMap()) : await _bankService.create(widget.item.toMap());

    if( result['success'] ){
      showMessage( result['message'] );
      widget.onChange();
      Timer(Duration(seconds: 2), () {
        Navigator.pop(context, true);
      });
    }else{
      showMessage( result['data'] );
    }


    setState(() {_processing = false;});
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  Future<void> _loadBanks( ) async{
    setState(() {_processing = true;});
    _bankListResponse = await _bankService.listBank(0, 100000, 'name', true, '');
    setState(() {_processing = false;});
  }

  final TextEditingController _controllerBank = new TextEditingController();
  Widget _bankSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Bank name (or) service name',
              labelText: 'Select the bank (or) money service',
            ),
            controller: _controllerBank,
          ),
          suggestionsCallback: (pattern) async {
            return await _bankSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text(suggestion.name),
              subtitle: Text(suggestion.address),
            );
          },
          onSuggestionSelected: (suggestion) {
            _bankItem = suggestion;
            _controllerBank.text = _bankItem.name;
            if( _bankItem !=null ){
              widget.item.bankGuid = _bankItem.guid;
            }else{
              widget.item.bankGuid = null;
            }

          },
          validator: (val) => val.isEmpty ? 'Bank (or) money service required!' : null,
        )
    );
  }

  Future<List<BankItem>> _bankSuggestion(String query) async{
    List<BankItem> matches = List();
    if( _bankListResponse==null || _bankListResponse.items == null)
      return matches;
    matches.addAll(_bankListResponse.items);

    matches.retainWhere((s) {
      BankItem bankItem = s;
      if( bankItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

}
