import 'dart:async';
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/response/person_active_person_by_role_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/state_service.dart';
import 'package:qq_delivery/delivery/service/street_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';



class UserCreate extends StatefulWidget {
  final String title;
  final AddressBookItem item;
  final Function onChange;

  UserCreate({Key key, @required this.title, this.onChange,
  this.item}) : super(key: key);

  @override
  _UserCreateState createState() => _UserCreateState();
}

class _UserCreateState extends State<UserCreate> {

  SharedPreferences _sharedPrefs;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  User _currentUser = new User();
  UserService _userService = new UserService();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  TownshipService _townshipService = new TownshipService();
  StateService _stateService = new StateService();
  StreetService _streetService = new StreetService();
  GeneralAddressResponse _addressResponseState;
  GeneralAddressItem _addressItemState;
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;
  GeneralAddressResponse _addressResponseStreet;
  GeneralAddressItem _addressItemStreet;

  PersonService _personService = new PersonService();
  PersonActiveByRoleResponse _activeByRoleResponseOS;
  GeneralPersonItem _personItemOS;

  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerState = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final TextEditingController _controllerStreet = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerPrimaryPhone = new TextEditingController();
  final TextEditingController _controllerOtherPhones = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerComment = new TextEditingController();
  final TextEditingController _controllerPassword = new TextEditingController();
  final TextEditingController _controllerConfirmPassword = new TextEditingController();



  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();

    if( _authorityResponse.authorities!=null
        && _authorityResponse.authorities.contains("EMPLOYEE_READ")
        && widget.item.addressType == Constant.ADDRESS_TYPE_CUSTOMER) {
      _activeByRoleResponseOS = await _personService.activePersonByRole( "OS" );
    }else{
      widget.item.personGuid = _currentUser.guid;
    }

    _addressResponseState = await _stateService.list();

    if( widget.item.guid != null ){
      // select on state list
      if( _addressResponseState!=null && _addressResponseState.items!=null){
        _addressResponseState.items.forEach((element) {
          if( widget.item.stateGuid == element.guid ){
            _controllerState.text = element.name;
            _addressItemState = element;
            return;
          }
        });

        // select on township
        if( _addressItemState != null){
          await _loadTownships( widget.item.townshipGuid );
        }

        // select on township
        if( _addressItemState != null){
          await _loadStreets( widget.item.street );
        }
      }
    }
    setState(() {_processing = false;});
  }


  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Theme.of(context).backgroundColor,
      appBar: AppBar(
        key: _scaffoldKey,
        title: Text(widget.title, style: TextStyle(color: Colors.white)),
        bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
      ),
      body: new SafeArea(
        top: false,
        bottom: false,
        child: new Form(
          key: _formKey,
          autovalidateMode: AutovalidateMode.onUserInteraction,
          child: new ListView(
            padding: const EdgeInsets.symmetric(horizontal: 8.0),
            children: <Widget>[
              _addressContacts( context ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(bottom: 16, top: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            // ListTile(
            //   title: Text(
            //     'Create ${widget.item.addressType} new account',
            //     style: Theme.of(context).textTheme.headline6,
            //   ),
            // ),
            _authorityResponse.authorities!=null && _authorityResponse.authorities.contains("EMPLOYEE_READ") && widget.item.addressType == Constant.ADDRESS_TYPE_CUSTOMER? _oSSelection(context): Container(),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '09xxxxxxxxx',
                  labelText: tr(LocaleKeys.lblLoginMobilePhone),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                // controller: _controllerPrimaryPhone,
                keyboardType: TextInputType.phone,
                onSaved: (val) => widget.item.primaryPhone = val,
                initialValue: widget.item.primaryPhone,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                obscureText: true,
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblPassword),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (val) => val.isEmpty ? 'Password required!' : null,
                // onSaved: (val) => _password = val,
                // initialValue: widget.item.name,
                controller: _controllerPassword,
                keyboardType: TextInputType.visiblePassword,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                obscureText: true,
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblConfirmPassword),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (val) => val.isEmpty ? 'Password required!' : (val==_controllerPassword.text?null : 'Password does not match!'),
                // onSaved: (val) => _password = val,
                // initialValue: widget.item.name,
                controller: _controllerConfirmPassword,
                keyboardType: TextInputType.visiblePassword,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblName),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (val) => val.isEmpty ? 'Nme required!' : null,
                onSaved: (val) => widget.item.name = val,
                initialValue: widget.item.name,
                // controller: _controllerName,
                keyboardType: TextInputType.text,
              ),
            ),
            _stateSelection( context ),
            _townshipSelection( context ),
            _streetSelection(context),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblHouseNo),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (val) => val.isEmpty ? 'Address required!' : null,
                // controller: _controllerAddress,
                keyboardType: TextInputType.text,
                onSaved: (val) => widget.item.address = val,
                initialValue: widget.item.address,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: "Use \",\" for multiple numbers",
                  labelText: tr(LocaleKeys.lblOtherPhones),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                // validator: (val) => val.isEmpty ? 'Other phone required!' : null,
                onSaved: (val) => widget.item.otherPhones = val,
                initialValue: widget.item.otherPhones,
                // controller: _controllerOtherPhones,
                keyboardType: TextInputType.phone,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: 'name@domain.com',
                  labelText: tr(LocaleKeys.lblEmailAddress),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                // controller: _controllerEmail,
                keyboardType: TextInputType.emailAddress,
                onSaved: (val) => widget.item.email = val,
                initialValue: widget.item.email,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 16, left:16, right: 16),
              child: new TextFormField(
                // style: TextStyle(
                //   color: Colors.white,
                // ),
                decoration: InputDecoration(
                  // icon: const Icon(Icons.phone),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblRemark),
                  // hintStyle: TextStyle( color: Colors.white60),
                  // enabledBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60),
                  // ),
                  // focusedBorder: UnderlineInputBorder(
                  //   borderSide: BorderSide(color: Colors.white60, width: 2),
                  // ),
                  // labelStyle: TextStyle(
                  //     color: Colors.white
                  // ),
                ),
                // validator: (val) => val.isEmpty ? 'Remark required' : null,
                onSaved: (val) => widget.item.comment = val,
                initialValue: widget.item.comment,
                // controller: _controllerComment,
                keyboardType: TextInputType.multiline,
                maxLines: null,
                minLines: 1,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                          // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        'Save',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                          // color: Colors.white
                      ),
                      // color: Colors.blueGrey[600],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        'Cancel',
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),

    );
  }

  Widget _oSSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<GeneralPersonItem>(
          selectedItem: _personItemOS,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          showSelectedItems: true,
          compareFn: (GeneralPersonItem i, GeneralPersonItem s) => i?.personGuid==s?.personGuid,
          label: tr(LocaleKeys.lblOS),
          onFind: (String filter) => _oSSuggestion(filter),
          onChanged: (GeneralPersonItem data) {
            _personItemOS = data;
          },
          dropdownBuilder: (BuildContext context, GeneralPersonItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle, color: Colors.blueGrey[600]),
                  title: Text(
                    'Select the OS [Option]',
                    // style: TextStyle(color: Colors.white),
                  ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0),
                  title: Text(
                    item.name,
                    // style: TextStyle(color: Colors.white),
                  ),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}',
                    width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, GeneralPersonItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  // color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text('${item.name}'),
                  // subtitle: Text('${item.townshipName}'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          // splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(_sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${item.avatar}', width: 42, height: 42, ),
                          ),
                        ),
                      )
                  ),
                ),
              ),
        )
    );
  }
  Future<List<GeneralPersonItem>> _oSSuggestion(String query) async{
    List<GeneralPersonItem> matches = List();
    if( _activeByRoleResponseOS==null || _activeByRoleResponseOS.items == null)
      return matches;
    matches.addAll(_activeByRoleResponseOS.items);

    matches.retainWhere((s) {
      GeneralPersonItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Widget _stateSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadField(
          textFieldConfiguration: TextFieldConfiguration(
            // style: TextStyle(color: Colors.white),
            autofocus: false,
            decoration: InputDecoration(
              //border: OutlineInputBorder(),
              hintText: 'Type state/division',
              labelText: tr(LocaleKeys.lblState),
              // hintStyle: TextStyle( color: Colors.white60),
              // enabledBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60),
              // ),
              // focusedBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60, width: 2),
              // ),
              // labelStyle: TextStyle(
              //     color: Colors.white
              // ),
            ),
            controller: _controllerState,
          ),
          suggestionsCallback: (pattern) async {
            return await _stateSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              //leading: Icon(Icons.shopping_cart),
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
              //subtitle: townSubTitle( suggestion ),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerState.text = '${suggestion.name}';
            _addressItemState = suggestion;
            _loadTownships();
          },
        )
    );
  }
  Future<List<GeneralAddressItem>> _stateSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseState==null || _addressResponseState.items == null)
      return matches;
    matches.addAll(_addressResponseState.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadTownships([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemState != null) {
      _addressResponseTownship = await _townshipService.list(_addressItemState.guid);
      if( _addressResponseTownship !=null && defaultSelected!=null){
        _addressResponseTownship.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemTownship = element;
            _controllerTownship.text = _addressItemTownship.name;
            return;
          }
        });
      }

    }else{
      _addressResponseTownship.items = [];
      _addressItemTownship = null;
      _controllerTownship.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadField(
          textFieldConfiguration: TextFieldConfiguration(
            // style: TextStyle(color: Colors.white),
            autofocus: false,
            decoration: InputDecoration(
              //border: OutlineInputBorder(),
              hintText: 'Type Township',
              labelText: tr(LocaleKeys.lblTownship),
              // hintStyle: TextStyle( color: Colors.white60),
              // enabledBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60),
              // ),
              // focusedBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60, width: 2),
              // ),
              // labelStyle: TextStyle(
              //     color: Colors.white
              // ),
            ),
            controller: _controllerTownship,
            enabled: _addressItemState!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              //leading: Icon(Icons.shopping_cart),
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
              //subtitle: townSubTitle( suggestion ),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = '${suggestion.name}';
            _addressItemTownship = suggestion;
            _loadStreets();
          },
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadStreets([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemTownship != null) {
      _addressResponseStreet = await _streetService.list(_addressItemTownship.guid);
      if( _addressResponseStreet !=null && defaultSelected!=null){
        _addressResponseStreet.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemStreet = element;
            _controllerStreet.text = _addressItemStreet.name;
            return;
          }
        });
      }

      if( defaultSelected!=null )
      _controllerStreet.text = defaultSelected;


    }else{
      _addressResponseStreet.items = [];
      _addressItemStreet = null;
      _controllerStreet.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _streetSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadField(
          textFieldConfiguration: TextFieldConfiguration(
            // style: TextStyle(color: Colors.white),
            autofocus: false,
            decoration: InputDecoration(
              //border: OutlineInputBorder(),
              hintText: 'Type Street',
              labelText: tr(LocaleKeys.lblStreet),
              // hintStyle: TextStyle( color: Colors.white60),
              // enabledBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60),
              // ),
              // focusedBorder: UnderlineInputBorder(
              //   borderSide: BorderSide(color: Colors.white60, width: 2),
              // ),
              // labelStyle: TextStyle(
              //     color: Colors.white
              // ),
            ),
            controller: _controllerStreet,
            enabled: _addressItemTownship!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _streetSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              //leading: Icon(Icons.shopping_cart),
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
              //subtitle: townSubTitle( suggestion ),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerStreet.text = '${suggestion.name}';
            _addressItemStreet = suggestion;
          },
        )
    );
  }
  Future<List<GeneralAddressItem>> _streetSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseStreet==null || _addressResponseStreet.items == null)
      return matches;
    matches.addAll(_addressResponseStreet.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    if( _controllerState.text.trim().isEmpty){
      showMessage('State required!');
      return;
    }

    if( _controllerTownship.text.trim().isEmpty){
      showMessage('Township required!');
      return;
    }

    if( _controllerStreet.text.trim().isEmpty){
      showMessage('Street required!');
      return;
    }

    setState(() {_processing = true;});

    Map<String,dynamic> addressBookCreateItem={
      'ownerGuid':_personItemOS==null? (_authorityResponse.authorities!=null && _authorityResponse.authorities.contains("MANAGER_WRITE") ? null: _currentUser.guid) : _personItemOS.personGuid,
      'name': widget.item.name,
      'companyName':null,
      'address': widget.item.address,
      'street': _controllerStreet.text,
      'streetGuid':_addressItemStreet!=null? _addressItemStreet.guid : null,
      'township': _controllerTownship.text,
      'townshipGuid': _addressItemTownship!=null? _addressItemTownship.guid : null,
      'primaryPhone': widget.item.primaryPhone,
      'otherPhones': widget.item.otherPhones,
      'email': widget.item.email,
      'comment': widget.item.comment,
      'latitude': _addressItemStreet!=null? _addressItemStreet.latitude : null,
      'longitude': _addressItemStreet!=null? _addressItemStreet.longitude : null,
      'addressType': widget.item.addressType,
      'guid':null,
      'avatar':null,
      'primaryContact': true
    };

    Map<String, dynamic> personCreateUpdateRequest = {
      'branchGuid':null,
      'name':widget.item.name,
      'avatar': null,
      'guid':null,
      'status':'Active',
      'role':widget.item.addressType,
      'contact': addressBookCreateItem,
      'username': widget.item.primaryPhone,
      'password':_controllerConfirmPassword.text
    };

    print(personCreateUpdateRequest);

    // checking username availability
    bool available = await _userService.isAvailable(widget.item.primaryPhone);
    if( available ){
      Map<String, dynamic> result = await _personService.createByAdmin(personCreateUpdateRequest);
      if( result['success'] ){
        showMessage( result['message'] );
        widget.onChange();
        Timer(Duration(seconds: 2), () {
          Navigator.pop(context, true);
        });
      }else{
        showMessage( result['data'] );
      }
    }else{
      showMessage( 'Account already created with ${widget.item.primaryPhone}' );
    }

    setState(() {_processing = false;});
  }


  bool isValidPhoneNumber(String input) {
    final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
}
