import 'dart:async';

import 'package:easy_localization/easy_localization.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/entity/user.dart';
import 'package:qq_delivery/delivery/model/request/person_list_request.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/authorities_response.dart';
import 'package:qq_delivery/delivery/model/response/person_list_response.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/view/user/user_create.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:qq_delivery/delivery/view/utils/custom_refresh_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';


class UserList extends StatefulWidget {
  UserList({Key key, this.selectedIndex}) : super(key: key);
  final int selectedIndex;
  @override
  _UserListState createState() => _UserListState();
}

class _UserListState extends State<UserList>{

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  int _selectedIndex = 0;
  bool _processing = false;
  String _title = Constant.ROLE_NAME_EMPLOYEE;
  bool _tapChange = false;


  @override
  void initState() {
    super.initState();
    _selectedIndex = widget.selectedIndex??0;
    _title = widget.selectedIndex==0? tr(LocaleKeys.mnuEmployees) : tr(LocaleKeys.lblCustomer);
    _loadProfile();
  }

  void _loadProfile() async{
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title, style: TextStyle(color: Colors.white)),
            bottom:  _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: _createBody( ),
          bottomNavigationBar: BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              // BottomNavigationBarItem(
              //   icon: Icon(Icons.people_alt_sharp),
              //   label: 'Customer list',
              // ),
              BottomNavigationBarItem(
                icon: Icon(Icons.perm_contact_cal_sharp),
                label: tr(LocaleKeys.lblEmployee),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.supervised_user_circle_sharp),
                label: tr(LocaleKeys.lblOS),
              ),
            ],
            currentIndex: _selectedIndex,
            // selectedItemColor: Theme.of( context ).primaryColor,
            // unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            // backgroundColor: Colors.white,
            onTap: _onItemTapped,
          ),
          floatingActionButton: FloatingActionButton.extended(
            onPressed: (){
              _addNew();
            },
            label: Text(
              'Add New Account',
              style: TextStyle(color: Colors.white),
            ),
            icon: Icon(Icons.add,
                color: Colors.white
            ),
            backgroundColor: Theme.of(context).primaryColor,
          ),
        )
    );
  }

  Widget _createBody(){
    // if( _selectedIndex == 0 && !_tapChange){
    //   return UserLoader(
    //     roleName: Constant.ROLE_NAME_CUSTOMER,
    //   );
    // }else
      if( _selectedIndex == 0 && !_tapChange){
      return UserLoader(
        roleName: Constant.ROLE_NAME_EMPLOYEE,
      );
    }else if( _selectedIndex == 1 && !_tapChange){
      return UserLoader(
        roleName: Constant.ROLE_NAME_OS,
      );
    }else{
      return new Container();
    }
  }

  void _onItemTapped(int index) {
    setState(() {
      _tapChange = true;
      _selectedIndex = index;
      // if( _selectedIndex == 0 ){
      //   _title= '${Constant.ROLE_NAME_CUSTOMER} account list';
      // }else
        if( _selectedIndex == 0){
        _title= tr(LocaleKeys.mnuEmployees);
      }else if( _selectedIndex == 1){
        _title= tr(LocaleKeys.lblCustomer);
      }
    });

    new ThreadTimer(new Duration(milliseconds: 500), () {
      setState(() {
        _tapChange = false;
      });
    });
  }

  void _addNew(){
    String addressType = Constant.ROLE_NAME_OS;
    if( _selectedIndex==0)
      addressType = Constant.ROLE_NAME_EMPLOYEE;
    else if( _selectedIndex==1)
      addressType = Constant.ROLE_NAME_OS;

    AddressBookItem item = new AddressBookItem(
        addressType: addressType,
        name: '',
        address: '',
        street: '',
        townshipName: '',
        primaryPhone: '',
        otherPhones: '',
        email: '',
        comment: '',
    );
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => UserCreate(
          title: addressType==Constant.ROLE_NAME_EMPLOYEE? tr(LocaleKeys.mnuEmployees) : tr(LocaleKeys.lblCustomer),
          item: item,
          onChange: (){
            setState(() {
              _tapChange = true;
            });
            new ThreadTimer(new Duration(milliseconds: 500), () {
              setState(() {
                _tapChange = false;
              });
            });
          },
        ),
      ),
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white,)
      )
  );
}


class UserLoader extends StatefulWidget {
  UserLoader({Key key, @required this.roleName}) : super(key: key);

  final String roleName;
  @override
  _UserLoaderState createState() => _UserLoaderState();
}

class _UserLoaderState extends State<UserLoader>{

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;
  PersonService _personService = new PersonService();
  PersonListResponse _personListResponse;
  String _accessToken;
  List<PersonListItem> _items = [];
  bool _isFirst = true;
  int _page = 0;
  String _keyword = '';
  final _searchQuery = new TextEditingController();
  Timer _debounce;


  @override
  void initState() {
    super.initState();
    _searchQuery.addListener(_onSearchChanged);
  }
  @override
  void deactivate() {
    super.deactivate( );
  }
  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    if (_debounce?.isActive ?? false) _debounce.cancel();
    super.dispose();
  }

  _onSearchChanged() {
    setState(() { });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(seconds: 3), () {
      _keyword = _searchQuery.text.trim();
      if( (_keyword.length ==0 || _keyword.length >=3) && !_refreshController.isRefresh){
        _refreshController.requestRefresh();
      }
    });
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _isFirst ) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
      _isFirst = false;
    }

    _accessToken = await _userService.getToken();
    _personListResponse = await _personService.listByRole(
      new PersonListRequest(
        first: _page,
        max: Constant.ROW_PER_PAGE,
        orderBy: widget.roleName==Constant.ROLE_NAME_CUSTOMER && _authorityResponse.authorities!=null
            &&_authorityResponse.authorities.contains("OS_READ")
            && !_authorityResponse.authorities.contains("MANAGER_READ")? "name" : "firstName",
        asc: true,
        keyword: _keyword,
        status: 'Active',
        roleName: widget.roleName,
        osGuid: widget.roleName==Constant.ROLE_NAME_CUSTOMER && _authorityResponse.authorities!=null
            &&_authorityResponse.authorities.contains("OS_READ")
            && !_authorityResponse.authorities.contains("MANAGER_READ")? _currentUser.guid : '',
      )
    );

    if( _personListResponse!=null && _personListResponse.items!=null){

      if( refresh && _items.isNotEmpty)
        _items.removeRange(0, _items.length);

      _personListResponse.items.forEach((element) {
        _items.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => i==0? _topWidget(c) : UserListItem(
        post: _items[i-1],
        authorityResponse: _authorityResponse,
        personService: _personService,
        isEnd: i == _items.length,
        currentUser: _currentUser,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
        roleName: widget.roleName,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length +1,
    );
  }

  @override
  Widget build(BuildContext context) {
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: CustomRefreshIndicator(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

  Widget _topWidget(BuildContext context){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, bottom: 8.0, left: 0.0, right: 0.0),
      child: Container(
        padding: EdgeInsets.only(top: 8, left: 4, bottom: 0, right: 4),
        child: TextField(
          textAlign: TextAlign.center,
          keyboardType: TextInputType.text,
          decoration: InputDecoration(
            hintText: tr(LocaleKeys.btnSearch),
            hintStyle: TextStyle(fontSize: 14),
            border: OutlineInputBorder(
              borderRadius: BorderRadius.circular(16),
              borderSide: BorderSide(
                width: 1,
                style: BorderStyle.none,
              ),
            ),
            filled: true,
            contentPadding: EdgeInsets.all(8),
            fillColor: Theme.of(context).backgroundColor,
            suffixIcon: _searchQuery.text.length>0? IconButton(
              icon: Icon(Icons.close_sharp,
                // color: Theme.of(context).primaryColorDark
              ),
              onPressed: (){
                setState(() {
                  _searchQuery.text = '';
                  _keyword = '';
                });
                _refreshController.requestRefresh();
              },
            ): Icon(Icons.search_sharp),
          ),
          controller: _searchQuery,
        ),
      ),
    );
  }
}

class UserListItem extends StatefulWidget {

  UserListItem({Key key, @required this.post,
    @required this.authorityResponse, @required this.personService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentUser,
  this.roleName}) : super(key: key);
  final PersonListItem post;
  final AuthorityResponse authorityResponse;
  final PersonService personService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final User currentUser;
  final String roleName;

  @override
  _UserListItemState createState() => _UserListItemState();

}

class _UserListItemState extends State<UserListItem> {

  bool _isEdit = false;
  bool _isChangePassword = false;
  bool _processing = false;
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  // color: Colors.white,
                  elevation: 1,
                  child: Wrap(
                  children: <Widget>[
                  ListTile(
                    title: Text(widget.post.name,
                      style: Theme.of(context).textTheme.headline6,
                    ),
                    subtitle:
                    widget.roleName==Constant.ROLE_NAME_EMPLOYEE?
                    Text(
                        'Phone: ${widget.post.mobilePhone}',
                        // style: TextStyle(color: Colors.black),
                    ) :
                    Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text(
                          'Phone: ${widget.post.mobilePhone}',
                          // style: TextStyle(color: Colors.black),
                        ),
                        Text(
                          'Address: ${widget.post.address}',
                          // style: TextStyle(color: Colors.black),
                        ),
                      ],
                    ),
                    leading: widget.post.avatar!=null && widget.post.avatar.length>10? ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            // splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}${Constant.IMAGE_DOWNLOAD_BASE_URL}${widget.post.avatar}', width: 48, height: 48, ),
                            ),
                          ),
                        )
                    )
                        :ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Theme.of(context).primaryColor, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: Icon( Icons.person, color: Colors.white, ),
                            ),
                          ),
                        )
                    ),
                    trailing: Icon( !_isEdit? Icons.keyboard_arrow_down_sharp: Icons.keyboard_arrow_up_sharp,
                      // color: Colors.blueGrey[600],
                    ),
                    onTap: (){
                      setState(() {
                        _isEdit = !_isEdit;
                      });
                    },
                  ),
                    _isEdit && !_isChangePassword?ButtonBar(
                      children: <Widget>[
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.edit,
                              // color: Colors.white
                          ),
                          color: Theme.of(context).primaryColor,
                          onPressed:_processing?null : (){
                            setState(() {
                              _isChangePassword = true;
                            });
                          },
                          label: Text(
                            'Change password',
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                        RaisedButton.icon(
                          elevation: 0.0,
                          icon: Icon(Icons.delete_forever,
                              // color: Colors.white
                          ),
                          color: Colors.red[700],
                          onPressed:_processing?null : ()async{
                            bool result = await _showConfirm(context, "Action confirm", "Are you sure to perform this action?" );
                            if( !result )
                              return;

                            UserService userService = new UserService();
                            Map<String, dynamic> response = await userService.changeStatus(
                                {
                                  'guid': widget.post.guid,
                                  'newPassword': 'Blocked'
                                }
                            );
                            if( response['success']){
                              showMessage(response['message']);
                              widget.onChange();
                            }else{
                              showMessage(response['data']);
                            }
                          },
                          label: Text(
                            "Delete",
                            style: TextStyle(
                              fontSize: 16,
                              fontWeight: FontWeight.w400,
                              // color: Colors.white,
                            ),
                          ),
                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                        ),
                      ],
                    ) : Container(),
                    _isEdit && _isChangePassword? _changePassword(context): Container(),
                  ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _showConfirm( BuildContext context, String title, String body )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text( title, style: TextStyle(color: Colors.black),),
          content: new Text( body ),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_pwd = new TextEditingController();
  final TextEditingController _controller_cpwd = new TextEditingController();

  Widget _changePassword(BuildContext context) {
    return new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0, bottom: 64),
            child:Card(
                color: Theme.of(context).backgroundColor,
                elevation: 1,
                child: Container(
                    padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                    child:new Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          new TextFormField(
                            obscureText: true,
                            decoration: const InputDecoration(
                              icon: const Icon(Icons.star),
                              hintText: '',
                              labelText: 'New password',
                            ),
                            controller: _controller_pwd,
                            validator: (val) => val.isEmpty ? 'Password required' :  (val.length<6? 'Minimum password length is 6' : null),
                          ),
                          new TextFormField(
                            obscureText: true,
                            decoration: const InputDecoration(
                              icon: const Icon(Icons.star),
                              hintText: '',
                              labelText: 'Confirm password',
                            ),
                            controller: _controller_cpwd,
                            validator: (val) => val.isEmpty ? 'Password required' : (val!= _controller_pwd.text ? 'Password does not match!' : null),
                          ),
                          ButtonBar(
                            children: <Widget>[
                              RaisedButton.icon(
                                elevation: 0.0,
                                icon: Icon(Icons.done, color: Colors.white),
                                color: Theme.of(context).primaryColor,
                                onPressed:_processing?null :  ()async{

                                if (!_formKey.currentState.validate() ) {
                                  return;
                                }
                                  UserService userService = new UserService();
                                  Map<String, dynamic> response = await userService.changePassword(
                                      {
                                        'guid': widget.post.guid,
                                        'newPassword': _controller_cpwd.text
                                      }
                                  );
                                  if( response['success']){
                                    showMessage(response['message']);
                                    setState(() {
                                      _isChangePassword = false;
                                      _isEdit = false;
                                    });
                                  }else{
                                    showMessage(response['data']);
                                  }
                                },
                                label: Text(
                                  'Change',
                                  style: TextStyle(
                                    fontSize: 16,
                                    fontWeight: FontWeight.w400,
                                    color: Colors.white,
                                  ),
                                ),
                                shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                              ),
                              RaisedButton.icon(
                                elevation: 0.0,
                                icon: Icon(Icons.close, color: Colors.white),
                                color: Colors.red[700],
                                onPressed:_processing?null : (){
                                  setState(() {
                                    _isChangePassword = false;
                                    _isEdit = false;
                                  });
                                },
                                label: Text(
                                  "Cancel",
                                  style: TextStyle(
                                    fontSize: 16,
                                    fontWeight: FontWeight.w400,
                                    color: Colors.white,
                                  ),
                                ),
                                shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                              ),
                            ],
                          )
                        ]
                    )
                )
            )
        )
    );
  }
}