import 'dart:async';
import 'dart:convert';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/person_service.dart';
import 'package:qq_delivery/delivery/service/state_service.dart';
import 'package:qq_delivery/delivery/service/street_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/view/login.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:flutter/material.dart';


class UserRegister extends StatefulWidget {
  UserRegister({Key key}) : super(key: key);

  @override
  _UserRegisterState createState() => _UserRegisterState();
}

class _UserRegisterState extends State<UserRegister> {

  UserService _userService = new UserService();

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.white;
  bool _processing = false;
  TownshipService _townshipService = new TownshipService();
  StateService _stateService = new StateService();
  StreetService _streetService = new StreetService();
  GeneralAddressResponse _addressResponseState;
  GeneralAddressItem _addressItemState;
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;
  GeneralAddressResponse _addressResponseStreet;
  GeneralAddressItem _addressItemStreet;

  PersonService _personService = new PersonService();

  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerState = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final TextEditingController _controllerStreet = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerPrimaryPhone = new TextEditingController();
  final TextEditingController _controllerOtherPhones = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerComment = new TextEditingController();
  final TextEditingController _controllerPassword = new TextEditingController();
  final TextEditingController _controllerConfirmPassword = new TextEditingController();
  AddressBookItem item = new AddressBookItem(
    addressType: Constant.ADDRESS_TYPE_CUSTOMER,
    name: '',
    address: '',
    street: '',
    townshipName: '',
    primaryPhone: '',
    otherPhones: '',
    email: '',
    comment: '',
  );

  String _lang = Constant.LANGUAGE_CODE_MY;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    setState(() {_processing = true;});
    _lang = await getLanguage( );
    _addressResponseState = await _stateService.list();
    setState(() {_processing = false;});
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).backgroundColor,
          appBar: AppBar(
            key: _scaffoldKey,
            title: Text(tr(LocaleKeys.lblCreateAccount), style: TextStyle(color: Colors.white)),
            bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          ),
          body: new SafeArea(
            top: false,
            bottom: false,
            child: new Form(
              key: _formKey,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              child: new ListView(
                padding: const EdgeInsets.symmetric(horizontal: 8.0),
                children: <Widget>[
                  _addressContacts( context ),
                ],
              ),
            ),
          ),
        )
    );
  }

  Widget _addressContacts( BuildContext context ){
    return Container(
      margin: EdgeInsets.only(top: 16, bottom: 16),
      child: Card(
        // color: Colors.white,
        elevation: 8,
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblYourNameOrBusinessName),
                ),
                validator: (val) => val.isEmpty ? 'Name required!' : null,
                onSaved: (val) => item.name = val,
                initialValue: item.name,
                keyboardType: TextInputType.text,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '67, 2nd Floor',
                  labelText: tr(LocaleKeys.lblHouseNo),
                ),
                validator: (val) => val.isEmpty ? 'House No. (or) Unit, floor required!' : null,
                keyboardType: TextInputType.text,
                onSaved: (val) => item.address = val,
                initialValue: item.address,
              ),
            ),
            _stateSelection( context ),
            _townshipSelection( context ),
            _streetSelection(context),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: 'name@domain.com',
                  labelText: tr(LocaleKeys.lblEmailAddress),
                ),
                validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                // controller: _controllerEmail,
                keyboardType: TextInputType.emailAddress,
                onSaved: (val) => item.email = val,
                initialValue: item.email,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:16, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '09xxxxxxxxx',
                  labelText: tr(LocaleKeys.lblLoginMobilePhone),
                ),
                validator: (val) => isValidPhoneNumber(val) ? null : 'Not a valid phone',
                keyboardType: TextInputType.phone,
                onSaved: (val) => item.primaryPhone = val,
                initialValue: item.primaryPhone,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                obscureText: true,
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblPassword),
                ),
                validator: (val) => val.isEmpty ? 'Password required!' : null,
                controller: _controllerPassword,
                keyboardType: TextInputType.visiblePassword,
              ),
            ),
            Container(
              padding: EdgeInsets.only(top:8, bottom: 0, left:16, right: 16),
              child: new TextFormField(
                obscureText: true,
                decoration: InputDecoration(
                  hintText: '',
                  labelText: tr(LocaleKeys.lblConfirmPassword),
                ),
                validator: (val) => val.isEmpty ? 'Password required!' : (val==_controllerPassword.text?null : 'Password does not match!'),
                controller: _controllerConfirmPassword,
                keyboardType: TextInputType.visiblePassword,
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 5.0, right: 5.0),
              child: ButtonBar(
                  children: <Widget>[
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.done,
                          // color: Colors.white
                      ),
                      color: Theme.of(context).primaryColor,
                      onPressed: _processing ?null : ()async {
                        _submitForm( );
                      },
                      label: Text(
                        tr(LocaleKeys.btnOK),
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                    RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.close,
                          // color: Colors.white
                      ),
                      // color: Colors.blueGrey[700],
                      onPressed: _processing?null : ()async {
                        Navigator.pop(context);
                      },
                      label: Text(
                        tr(LocaleKeys.btnCancel),
                        style: TextStyle(
                          fontSize: 16,
                          fontWeight: FontWeight.w400,
                          // color: Colors.white,
                        ),
                      ),
                      shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                    ),
                  ]
              ),
            ),
          ],

        ),
      ),

    );
  }

  Widget _stateSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 0),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type state/division name',
              labelText: tr(LocaleKeys.lblState),
            ),
            controller: _controllerState,
          ),
          suggestionsCallback: (pattern) async {
            return await _stateSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerState.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemState = suggestion;
            _loadTownships();
          },
          validator: (val) => val.isEmpty ? 'State/Division required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _stateSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseState==null || _addressResponseState.items == null)
      return matches;
    matches.addAll(_addressResponseState.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadTownships([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemState != null) {
      _addressResponseTownship = await _townshipService.list(_addressItemState.guid);
      if( _addressResponseTownship !=null && defaultSelected!=null){
        _addressResponseTownship.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemTownship = element;
            _controllerTownship.text = _addressItemTownship.name;
            return;
          }
        });
      }

    }else{
      _addressResponseTownship.items = [];
      _addressItemTownship = null;
      _controllerTownship.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 0),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type township name',
              labelText: tr(LocaleKeys.lblTownship),
            ),
            controller: _controllerTownship,
            enabled: _addressItemState!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemTownship = suggestion;
            _loadStreets();
          },
          validator: (val) => val.isEmpty ? 'Township (or) Town required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadStreets([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemTownship != null) {
      _addressResponseStreet = await _streetService.list(_addressItemTownship.guid);
      if( _addressResponseStreet !=null && defaultSelected!=null){
        _addressResponseStreet.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemStreet = element;
            _controllerStreet.text = _addressItemStreet.name;
            return;
          }
        });
      }

      if( defaultSelected!=null )
      _controllerStreet.text = defaultSelected;


    }else{
      _addressResponseStreet.items = [];
      _addressItemStreet = null;
      _controllerStreet.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _streetSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 0),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type street name',
              labelText: tr(LocaleKeys.lblStreet),
            ),
            controller: _controllerStreet,
            enabled: _addressItemTownship!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _streetSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerStreet.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemStreet = suggestion;
          },
          noItemsFoundBuilder: (BuildContext context) => new Container( height: 0, ),
          validator: (val) => val.isEmpty ? 'Street name required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _streetSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseStreet==null || _addressResponseStreet.items == null)
      return matches;
    matches.addAll(_addressResponseStreet.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  void _submitForm( ) async{
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Form is not valid! Please review and correct.');
      return;
    }else {
      form.save();
    }

    if( _controllerState.text.trim().isEmpty){
      showMessage('State required!');
      return;
    }

    if( _controllerTownship.text.trim().isEmpty){
      showMessage('Township required!');
      return;
    }

    if( _controllerStreet.text.trim().isEmpty){
      showMessage('Street required!');
      return;
    }

    setState(() {_processing = true;});

    Map<String,dynamic> addressBookCreateItem={
      'ownerGuid': null,
      'name': item.name,
      'companyName':null,
      'address': item.address,
      'street': _controllerStreet.text,
      'streetGuid':_addressItemStreet!=null? _addressItemStreet.guid : null,
      'township': _controllerTownship.text,
      'townshipGuid': _addressItemTownship!=null? _addressItemTownship.guid : null,
      'primaryPhone': item.primaryPhone,
      'otherPhones': item.otherPhones,
      'email': item.email,
      'comment': item.comment,
      'latitude': _addressItemStreet!=null? _addressItemStreet.latitude : null,
      'longitude': _addressItemStreet!=null? _addressItemStreet.longitude : null,
      'addressType': item.addressType,
      'guid':null,
      'avatar':null,
      'primaryContact': true
    };

    Map<String, dynamic> personCreateUpdateRequest = {
      'branchGuid':null,
      'name':item.name,
      'avatar': null,
      'guid':null,
      'status':'Active',
      'role':item.addressType,
      'contact': addressBookCreateItem,
      'username': item.primaryPhone,
      'password':_controllerConfirmPassword.text,
      'selfRegister': true
    };

    if(Constant.ENABLE_LOGS)
      print(json.encode(personCreateUpdateRequest));

    // checking username availability
    bool available = await _userService.isAvailable(item.primaryPhone);
    if( available ){
      Map<String, dynamic> result = await _personService.create(personCreateUpdateRequest);
      if( result['success'] ){
        showMessage( result['message'] );
        await _userService.logout( );
        await setSettingsTempLogin(false);
        Timer(Duration(seconds: 2), () {
          Navigator.pushReplacement(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => LoginUser(
                  title: 'Login',
                ),
              )
          );
        });
      }else{
        showMessage( result['data'] );
      }
    }else{
      showMessage( 'Account already created with ${item.primaryPhone}' );
    }

    setState(() {_processing = false;});
  }


  bool isValidPhoneNumber(String input) {
    // final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    // final RegExp regex = new RegExp(r'(?:[0-9]{9,11})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    // return regex.hasMatch(input);
    return input.length>8;
  }

  bool isValidEmail(String input) {
    if( input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  void showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}
