import 'dart:io';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/model/response/delivery_list_response.dart';
import 'package:qq_delivery/delivery/service/user_service.dart';
import 'package:qq_delivery/delivery/service/v2/warehouse_station_route_service.dart';
import 'package:qq_delivery/delivery/view/delivery/delivery_detail.dart';
import 'package:qq_delivery/delivery/view/transit/transit_detail.dart';
import 'package:qr_code_scanner/qr_code_scanner.dart';


class AutoCompletedQRScanPage extends StatefulWidget {
  @override
  _AutoCompletedQRScanPageState createState() => _AutoCompletedQRScanPageState();
}

class _AutoCompletedQRScanPageState extends State<AutoCompletedQRScanPage> {
  bool resultSent = false;
  Barcode result;
  QRViewController controller;
  final GlobalKey qrKey = GlobalKey(debugLabel: 'QR');
  WarehouseStationRouteService _stationRouteService = new WarehouseStationRouteService();
  bool _processing = false;
  bool _error = false;
  bool _pause = false;
  String _message = "";

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
            title: Text('Auto complete QR Scanner', style: TextStyle(color: Colors.white)),
          ),
          body: SafeArea(
              child:Column(
                  children: <Widget>[
                    Expanded(flex: 4, child: _buildQrView(context)),
                    Expanded(
                      flex: 1,
                      child: FittedBox(
                        fit: BoxFit.contain,
                        child:Column(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: <Widget>[
                              Container(
                                padding: const EdgeInsets.all(5.0),
                                child: Text(_message),
                              ),
                              !_error && _pause ? Container(
                                padding: const EdgeInsets.all(5.0),
                                child: ClipOval(
                                    child: Material(
                                      elevation: 8,
                                      color: Colors.white, // button color
                                      child: InkWell(
                                        // splashColor: Colors.red, // inkwell color
                                        child: SizedBox(width: 50,
                                          height: 50,
                                          child: Icon( Icons.check_circle_outline, size: 30,
                                            color: Colors.green[500],
                                          ),
                                        ),
                                      ),
                                    )
                                ),
                              ) : Container(),
                              _error && _pause ? Container(
                                padding: const EdgeInsets.all(5.0),
                                child: ClipOval(
                                    child: Material(
                                      elevation: 8,
                                      color: Colors.white, // button color
                                      child: InkWell(
                                        // splashColor: Colors.red, // inkwell color
                                        child: SizedBox(width: 50,
                                          height: 50,
                                          child: Icon( Icons.error_outline_sharp, size: 30,
                                            color: Colors.red[600],
                                          ),
                                        ),
                                      ),
                                    )
                                ),
                              ) : Container(),
                              !_error && !_pause ? Container(
                                padding: const EdgeInsets.all(5.0),
                                child: ClipOval(
                                    child: Material(
                                      elevation: 8,
                                      color: Colors.white, // button color
                                      child: InkWell(
                                        // splashColor: Colors.red, // inkwell color
                                        child: SizedBox(width: 50,
                                          height: 50,
                                          child: Icon( Icons.qr_code_scanner_sharp, size: 30,
                                            color: Colors.green[600],
                                          ),
                                        ),
                                      ),
                                    )
                                ),
                              ) : Container(),
                              _pause? RaisedButton(
                                elevation: 2.0,
                                shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                child: Text(_error? "Try again" : "Continue"),
                                onPressed: (){
                                  setState(() {
                                    _error = false;
                                    _pause = false;
                                  });
                                  controller.resumeCamera();
                                },
                              ) : Container(),
                            ]
                        ),
                      ),
                    ),
                  ]
              )
          ),
        )
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  void reassemble() {
    super.reassemble();
    if (Platform.isAndroid) {
      controller.pauseCamera();
    }
    controller.resumeCamera();
  }

  Widget _buildQrView(BuildContext context) {
    // For this example we check how width or tall the device is and change the scanArea and overlay accordingly.
    var scanArea = (MediaQuery.of(context).size.width < 400 ||
        MediaQuery.of(context).size.height < 400)
        ? 150.0
        : 300.0;
    // To ensure the Scanner view is properly sizes after rotation
    // we need to listen for Flutter SizeChanged notification and update controller
    return QRView(
      key: qrKey,
      onQRViewCreated: (controller) => _onQRViewCreated( context, controller ),
      overlay: QrScannerOverlayShape(
          borderColor: Colors.red,
          borderRadius: 10,
          borderLength: 30,
          borderWidth: 10,
          cutOutSize: scanArea),
      onPermissionSet: (ctrl, p) => _onPermissionSet(context, ctrl, p),
    );
  }

  void _onQRViewCreated( BuildContext context, QRViewController controller ) {
    setState(() {
      this.controller = controller;
    });
    controller.scannedDataStream.listen((scanData) {
      result = scanData;
      controller.pauseCamera();
      String qrValue = result.code;
      if( qrValue.contains( Constant.UI_DOMAIN_NAME )){
        String wayId = qrValue.substring(qrValue.lastIndexOf("/")+1);
        _loadData( wayId );
      }else{
        _message = 'Invalid QR Code';
        // ScaffoldMessenger.of(context).showSnackBar(
        //   SnackBar(content: Text('Invalid QR Code')),
        // );
        // controller.dispose();
        // Navigator.pop( context );
        setState(() {
          _error = true;
          _pause = true;
        });
      }
    });
  }

  void _onPermissionSet(BuildContext context, QRViewController ctrl, bool p) {
    if (!p) {
      ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(content: Text('no Permission')),
      );
    }
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  _loadData( String wayId ) async{
    setState(() {
      _processing = true;
    });

    UserService _userService = new UserService();
    String accessToken = await _userService.getToken();
    Map<String, dynamic> resultData = await _stationRouteService.routeProcessByQR( wayId, accessToken );

    if( resultData["success"] ) {
      if( resultData["message"] =='Deliver'){
        String data =  resultData["data"].toString();
        // last miles way
        _message = data;
        // ScaffoldMessenger.of(context).showSnackBar(
        //   SnackBar(content: Text( data )),
        // );
        setState(() {
          _error = false;
          _pause = true;
        });
      }else{
        // transit way
        Map<String, dynamic> data = resultData["data"];
        DeliveryListItem listItem = new DeliveryListItem();
        listItem.fromMap(data);
        if( listItem.contentGUID != null ){
          String wayType = 'unknown';

          if( listItem.status==Constant.STATUS_ASSIGNED
              && listItem.outboundBucketStatus!=null
              && listItem.outboundBucketStatus == Constant.BUCKET_STATUS_REQUEST){
            wayType = 'outbound';
          }else if( listItem.status==Constant.STATUS_COMPLETED
              && listItem.inboundBucketStatus!=null
              && listItem.inboundBucketStatus == Constant.BUCKET_STATUS_REQUEST){
            wayType = 'inbound';
          }

          controller.dispose();
          Navigator.of(context).pushReplacement(
              MaterialPageRoute(
                builder: (BuildContext context) => TransitDetail(
                  wayId: listItem.contentGUID,
                  onChange: (){},
                  wayType: wayType,
                ),
              )
          );

        }else{
          _message = 'Invalid QR Code';
          // ScaffoldMessenger.of(context).showSnackBar(
          //   SnackBar(content: Text('Invalid QR Code')),
          // );
          // controller.dispose();
          // Navigator.pop( context );
          setState(() {
            _error = true;
            _pause = true;
          });
        }
      }
    }else{
      _message = resultData["message"];
      // ScaffoldMessenger.of(context).showSnackBar(
      //   SnackBar(content: Text(resultData["message"])),
      // );
      setState(() {
        _error = true;
        _pause = true;
      });
    }

    setState(() {_processing=false;});
  }
}