import 'dart:async';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/service/hrm_base_client.dart';
import 'package:qq_delivery/delivery/view/home.dart';

class ConnectionBroken extends StatefulWidget {
  ConnectionBroken({Key key}) : super(key: key);
  @override
  _ConnectionBrokenPageState createState() => _ConnectionBrokenPageState( );
}
class _ConnectionBrokenPageState extends State<ConnectionBroken> {

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool _processing = false;
  Color progressColor;
  Color _disableColor = Colors.grey[300];
  IconData _topIcon = Icons.wifi_off_sharp;
  String _title = tr(LocaleKeys.lblConnectionBroken);

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text(_title, style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: Center(
          child: SingleChildScrollView(
            child: Column(
                children: <Widget>[
                  Container(
                      padding: EdgeInsets.only(left: 16.0, right: 16.0),
                      child:Card(
                        color: Colors.white,
                        elevation: 0,
                        child: Column(
                            children: <Widget>[
                              new Container(
                                margin: const EdgeInsets.only(top:16, left: 16.0, right: 16.0, bottom: 16),
                                child: Text(_title,
                                    style: TextStyle(
                                        color: Theme.of(context).primaryColor,
                                        fontWeight: FontWeight.bold,
                                        fontSize: 18
                                    )),
                              ),
                              Container(
                                padding: const EdgeInsets.all(16.0),
                                child: ClipOval(
                                    child: Material(
                                      elevation: 8,
                                      color: Colors.white, // button color
                                      child: InkWell(
                                        splashColor: Colors.red, // inkwell color
                                        child: SizedBox(width: 200,
                                          height: 200,
                                          child: Icon( _topIcon, size: 150, color: _disableColor,),
                                        ),
                                      ),
                                    )
                                ),
                              ),
                              Container(
                                padding: const EdgeInsets.only(left: 32.0, bottom: 32.0, right: 32.0, top: 16),
                                width: double.infinity,
                                child: RaisedButton(
                                  onPressed: () async{
                                    String status = await testConnection("/v1/state/list");
                                    if( status == Constant.CONNECTION_REFUSED){
                                      showMessage(tr(LocaleKeys.lblConnectionBroken));
                                    }else{
                                      setState(() {
                                        _topIcon = Icons.done;
                                        _disableColor = Theme.of(context).primaryColor;
                                        _title = tr(LocaleKeys.lblConnectionReady);
                                      });
                                      showMessage(tr(LocaleKeys.lblConnectionReady), Colors.green);

                                      new Timer(new Duration(milliseconds: 2000), () {
                                        Navigator.pushReplacement(
                                            context,
                                            MaterialPageRoute(
                                              builder: (BuildContext context) => Home(
                                                title: '${Constant.APP_NAME}',
                                              ),
                                            )
                                        );
                                      });
                                    }
                                  },
                                  color: Theme.of(context).primaryColorDark,
                                  elevation: 1,
                                  child: Text(
                                    tr(LocaleKeys.btnTryAgain),
                                    style: TextStyle(color: Colors.white),
                                  ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                                ),
                              ),
                            ]
                        ),
                      )
                  ),
                ]
            )
          )
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.red]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: color, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: Colors.white, )
      )
  );

}
