import 'dart:convert';
import 'dart:typed_data';
import 'package:easy_localization/easy_localization.dart';
import 'package:esc_pos_printer/esc_pos_printer.dart';
import 'package:esc_pos_utils/esc_pos_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:image/image.dart' as imglib;
import 'dart:ui' as ui;

class LanPrinting{

  void printSlip(NetworkPrinter printer, DeliveryDetailResponse detailResponse, DeliveryContactItem contactItemOS,
      DeliveryContactItem contactItemCustomer, SharedPreferences sharedPrefs, String printerIp, BuildContext context, imglib.Image bodyImage) async {

    try{
        // Logo image
        ByteData data = await rootBundle.load('assets/images/logo.png');
        final Uint8List bytes = data.buffer.asUint8List();
        final imglib.Image image = imglib.decodeImage(bytes);
        printer.image(image);
        printer.text( Constant.APP_NAME, styles: PosStyles(bold: true, align: PosAlign.center));
        printer.text('Pickup date: '+ new DateFormat('MMM dd, yyyy').format(contactItemOS.date.toLocal()), styles: PosStyles(bold: true, align: PosAlign.right) );
        printer.hr();
        printer.image( bodyImage );

        printer.hr( );
        printer.text('${detailResponse.trackingId}', styles: PosStyles(align: PosAlign.center));
        // QR
        String qrURL = '${getBaseURL(sharedPrefs)}/v1/image/qr_code/${detailResponse.trackingId}';
        List<int> qrBytes = await networkImageToBytes( qrURL );
        if( qrBytes != null){
          imglib.Image image = imglib.decodeImage( qrBytes );
          imglib.Image thumbnail = imglib.copyResize(image, width: 250);
          printer.image( thumbnail );
        }
        // printer.hr( );
        // printer.emptyLines( 3 );

        printer.feed(1);
        printer.cut();
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }
  }
}