
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/view/delivery/delivery_detail.dart';
import 'package:qr_code_scanner/qr_code_scanner.dart';


class QRScanPage extends StatefulWidget {
  @override
  _ScanPageState createState() => _ScanPageState();
}

class _ScanPageState extends State<QRScanPage> {
  bool resultSent = false;
  Barcode result;
  QRViewController controller;
  final GlobalKey qrKey = GlobalKey(debugLabel: 'QR');

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
            title: Text('QR Scanner', style: TextStyle(color: Colors.white)),
          ),
          body: SafeArea(
              child:_buildQrView(context)
          ),
        )
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  void reassemble() {
    super.reassemble();
    if (Platform.isAndroid) {
      controller.pauseCamera();
    }
    controller.resumeCamera();
  }

  Widget _buildQrView(BuildContext context) {
    // For this example we check how width or tall the device is and change the scanArea and overlay accordingly.
    var scanArea = (MediaQuery.of(context).size.width < 400 ||
        MediaQuery.of(context).size.height < 400)
        ? 150.0
        : 300.0;
    // To ensure the Scanner view is properly sizes after rotation
    // we need to listen for Flutter SizeChanged notification and update controller
    return QRView(
      key: qrKey,
      onQRViewCreated: (controller) => _onQRViewCreated( context, controller ),
      overlay: QrScannerOverlayShape(
          borderColor: Colors.red,
          borderRadius: 10,
          borderLength: 30,
          borderWidth: 10,
          cutOutSize: scanArea),
      onPermissionSet: (ctrl, p) => _onPermissionSet(context, ctrl, p),
    );
  }

  void _onQRViewCreated( BuildContext context, QRViewController controller ) {
    setState(() {
      this.controller = controller;
    });
    controller.scannedDataStream.listen((scanData) {
      result = scanData;
      controller.dispose();
      String qrValue = result.code;
      if( qrValue.contains( Constant.UI_DOMAIN_NAME )){
        String wayId = qrValue.substring(qrValue.lastIndexOf("/")+1);
        print( qrValue );
        print(wayId);
        Navigator.of(context).pushReplacement(
            MaterialPageRoute(
              builder: (BuildContext context) => DeliveryDetail(
                wayId: wayId,
                onChange: (){},
              ),
            )
        );
      }else{
        Navigator.pop( context );
      }

    });
  }

  void _onPermissionSet(BuildContext context, QRViewController ctrl, bool p) {
    if (!p) {
      ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(content: Text('no Permission')),
      );
    }
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}