
import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';
import 'dart:ui' as ui;
import 'package:flutter/material.dart';
import 'package:flutter_signature_pad/flutter_signature_pad.dart';


class SignaturePad extends StatefulWidget {
  SignaturePad({Key key,@required this.title}) : super(key: key);
  final title;

  @override
  _SignaturePadState createState() => _SignaturePadState();
}

class _WatermarkPaint extends CustomPainter {
  final String price;
  final String watermark;

  _WatermarkPaint(this.price, this.watermark);

  @override
  void paint(ui.Canvas canvas, ui.Size size) {
    canvas.drawCircle(Offset(size.width / 2, size.height / 2), 10.8, Paint()..color = Colors.blue);
  }

  @override
  bool shouldRepaint(_WatermarkPaint oldDelegate) {
    return oldDelegate != this;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) || other is _WatermarkPaint && runtimeType == other.runtimeType && price == other.price && watermark == other.watermark;

  @override
  int get hashCode => price.hashCode ^ watermark.hashCode;
}

class _SignaturePadState extends State<SignaturePad> {
  ByteData _img = ByteData(0);
  var color = Colors.red;
  var strokeWidth = 5.0;
  final _sign = GlobalKey<SignatureState>();

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
            title: Text(widget.title, style: TextStyle(color: Colors.white)),
          ),
          body: Column(
            children: <Widget>[
              Expanded(
                child: Container(
                  child: Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Signature(
                      color: color,
                      key: _sign,
                      onSign: () {
                        final sign = _sign.currentState;
                        debugPrint('${sign.points.length} points in the signature');
                      },
                      backgroundPainter: _WatermarkPaint("2.0", "2.0"),
                      strokeWidth: strokeWidth,
                    ),
                  ),
                  color: Colors.black12,
                ),
              ),
              _img.buffer.lengthInBytes == 0 ? Container() : LimitedBox(maxHeight: 200.0, child: Image.memory(_img.buffer.asUint8List())),
              ButtonBar(
                children: <Widget>[
                  RaisedButton.icon(
                    elevation: 0.0,
                    icon: Icon(Icons.done, color: Colors.white),
                    color: Theme.of(context).primaryColor,
                    onPressed: () async {
                      final sign = _sign.currentState;
                      final image = await sign.getData();
                      var data = await image.toByteData(format: ui.ImageByteFormat.png);
                      sign.clear();
                      final encoded = base64.encode(data.buffer.asUint8List());
                      setState(() {
                        _img = data;
                      });
                      Navigator.pop( context, encoded );
                    },
                    label: Text(
                      'OK',
                      style: TextStyle(
                        fontSize: 16,
                        fontWeight: FontWeight.w400,
                        color: Colors.white,
                      ),
                    ),
                    shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                  ),
                  RaisedButton.icon(
                    elevation: 0.0,
                    icon: Icon(Icons.cleaning_services_sharp, color: Colors.white),
                    color: Colors.red[700],
                    onPressed: () {
                      final sign = _sign.currentState;
                      sign.clear();
                      setState(() {
                        _img = ByteData(0);
                      });
                    },
                    label: Text(
                      "Clear",
                      style: TextStyle(
                        fontSize: 16,
                        fontWeight: FontWeight.w400,
                        color: Colors.white,
                      ),
                    ),
                    shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                  ),
                ],
              ),
            ],
          ),
        )
    );
  }
  Future<bool> _onWillPop() async {
    return true;
  }
}
