import 'dart:typed_data';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/request/pickup_create_request.dart';
import 'package:qq_delivery/delivery/model/response/delivery_detail_response.dart';
import 'package:qq_delivery/delivery/model/shared/delivery_contact_item.dart';
import 'package:qq_delivery/delivery/utils/base64image.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/services.dart';
import 'package:flutter_sunmi_printer/flutter_sunmi_printer.dart';
import 'package:image/image.dart' as imglib;
import 'package:shared_preferences/shared_preferences.dart';
import 'dart:ui' as ui;

class SunmiPrint{

  void printSlip(DeliveryDetailResponse detailResponse, DeliveryContactItem contactItemOS,
      DeliveryContactItem contactItemCustomer, SharedPreferences sharedPrefs, BuildContext context) async {

    try{

      // SunmiPrinter.text('Updated: '+ new DateFormat('dd/MM/yyyy hh:mm a').format(detailResponse.updatedOn.toLocal()), styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.text('${tr(LocaleKeys.lblStatus)} ${_createStatus(detailResponse.status)}', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.emptyLines(1);

      if( detailResponse.status==Constant.STATUS_PICKUP_CREATED || detailResponse.status==Constant.STATUS_PICKUP_SCHEDULED || detailResponse.status==Constant.STATUS_PICKUP_ON_WAY ) {
        // OS info
        SunmiPrinter.text(
            'ပစ္စည်းယူရန်', styles: SunmiStyles(align: SunmiAlign.left));
        SunmiPrinter.text('အမည်။ ${contactItemOS.name}',
            styles: SunmiStyles(align: SunmiAlign.left));
        SunmiPrinter.text(
          "ဖုန်းနံပါတ်။ ${contactItemOS.primaryPhone}${contactItemOS
              .otherPhones != null ? contactItemOS.otherPhones : ''}",
          styles: SunmiStyles(align: SunmiAlign.left),
        );
        SunmiPrinter.text(
          "လိပ်စာ။ ${contactItemOS.address + ', ' + contactItemOS.street +
              ", " + contactItemOS.township}",
          styles: SunmiStyles(align: SunmiAlign.left),
        );

        double refundAmount = 0.0;
        if (detailResponse != null) {
          double itemPrice = 0;
          double deliCharge = 0;
          double deliChargeForShow = 0;
          double promo = 0;
          if (detailResponse.items[0].priceOption == 'Unpaid')
            itemPrice = detailResponse.items[0].price ?? 0;

          if (detailResponse.items[0].chargeOption == 'Paid') {
            deliCharge = detailResponse.items[0].charges ?? 0;
          }
          deliChargeForShow = detailResponse.items[0].forShowCharges ?? 0;
          double chargeDiff = deliChargeForShow -
              detailResponse.items[0].charges ?? 0;

          promo = detailResponse.promotion ?? 0;
          refundAmount = (itemPrice + promo + chargeDiff) - deliCharge;
        }

        String label = refundAmount > 0 &&
            detailResponse.refundInAdvanced != null &&
            detailResponse.refundInAdvanced ? tr(
            LocaleKeys.lblRequiredToRefundInPrint) +
            new NumberFormat("#,###").format(refundAmount) + ' Ks' : '';
        if (refundAmount > 0)
          SunmiPrinter.text(
              '$label', styles: SunmiStyles(align: SunmiAlign.left));
        SunmiPrinter.hr();
      }

      // Logo image
      ByteData bytes = await rootBundle.load('assets/images/logo.png');
      final buffer = bytes.buffer;
      final imgData = base64.encode(Uint8List.view(buffer));
      SunmiPrinter.image(imgData);
      SunmiPrinter.text( Constant.APP_NAME, styles: SunmiStyles(align: SunmiAlign.center) );
      SunmiPrinter.text('Pickup date: '+ new DateFormat('MMM dd, yyyy').format(contactItemOS.date.toLocal()), styles: SunmiStyles(align: SunmiAlign.right) );
      SunmiPrinter.hr();

      // Customer info
      SunmiPrinter.text('ပစ္စည်းပို့ရန်', styles: SunmiStyles(align: SunmiAlign.left));
      SunmiPrinter.text('အမည်။ ${contactItemCustomer.name}', styles: SunmiStyles(align: SunmiAlign.left));
      SunmiPrinter.text(
        "ဖုန်းနံပါတ်။ ${contactItemCustomer.primaryPhone}${contactItemCustomer.otherPhones!=null?','+contactItemCustomer.otherPhones:''}",
        styles: SunmiStyles(align: SunmiAlign.left),
      );
      SunmiPrinter.text(
        "လိပ်စာ။ ${contactItemCustomer.address + ', ' + contactItemCustomer.street + ", " + contactItemCustomer.township}",
        styles: SunmiStyles(align: SunmiAlign.left),
      );
      SunmiPrinter.text( '${tr(LocaleKeys.lblCashToCollectInPrint)} ${new NumberFormat("#,###").format(detailResponse.totalCharges)} Ks', styles: SunmiStyles(align: SunmiAlign.left));
      SunmiPrinter.hr( );
      SunmiPrinter.text('${detailResponse.trackingId}', styles: SunmiStyles(align: SunmiAlign.center));
      // QR
      String qrURL = '${getBaseURL(sharedPrefs)}/v1/image/qr_code/${detailResponse.trackingId}';
      List<int> qrBytes = await networkImageToBytes( qrURL );
      if( qrBytes != null){
        imglib.Image image = imglib.decodeImage( qrBytes );
        imglib.Image thumbnail = imglib.copyResize(image, width: 250);
        String base64QR = base64String(imglib.encodeJpg(thumbnail));
        if( base64QR != null ){
          SunmiPrinter.image(base64QR);
        }
      }

      if( contactItemCustomer.signature != null ){
        SunmiPrinter.emptyLines( 1 );
        // Customer sign
        String signURL = '${getBaseURL(sharedPrefs)}/v1/image/download/${contactItemCustomer.signature}';
        List<int> signBytes = await networkImageToBytes( signURL );
        if( signBytes != null){
          imglib.Image image = imglib.decodeImage( signBytes );
          imglib.Image thumbnail = imglib.copyResize(image, width: 250);
          String base64QR = base64String(imglib.encodeJpg(thumbnail));
          if( base64QR != null ){
            SunmiPrinter.image(base64QR);
            SunmiPrinter.text('Customer Signature', styles: SunmiStyles(align: SunmiAlign.center));
          }
        }
      }
      SunmiPrinter.hr( );
      SunmiPrinter.emptyLines( 3 );

    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  void printPickupReceipt(PickupCreateRequest detailResponse, DeliveryContactItem contactItemOS,
      SharedPreferences sharedPrefs, BuildContext context) async {

    try{

      // SunmiPrinter.text('Updated: '+ new DateFormat('dd/MM/yyyy hh:mm a').format(detailResponse.updatedOn.toLocal()), styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.text('${tr(LocaleKeys.lblStatus)} ${_createStatus(detailResponse.status)}', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.emptyLines(1);


      // Logo image
      ByteData bytes = await rootBundle.load('assets/images/logo.png');
      final buffer = bytes.buffer;
      final imgData = base64.encode(Uint8List.view(buffer));
      SunmiPrinter.image(imgData);
      SunmiPrinter.text( Constant.APP_NAME, styles: SunmiStyles(align: SunmiAlign.center) );
      SunmiPrinter.hr();
      SunmiPrinter.text('ပစ္စည်းယူသည့် နေ့စွဲ။ '+ new DateFormat('dd/MM/yyyy').format(contactItemOS.date.toLocal()), styles: SunmiStyles(align: SunmiAlign.left));
      SunmiPrinter.emptyLines(1);

        // OS info
        SunmiPrinter.text('ပစ္စည်းယူရန်', styles: SunmiStyles(align: SunmiAlign.left));
        SunmiPrinter.text('အမည်။ ${contactItemOS.name}', styles: SunmiStyles(align: SunmiAlign.left));
        SunmiPrinter.text("ဖုန်းနံပါတ်။ ${contactItemOS.primaryPhone}${contactItemOS.otherPhones != null ? contactItemOS.otherPhones : ''}", styles: SunmiStyles(align: SunmiAlign.left),);
        SunmiPrinter.text("လိပ်စာ။ ${contactItemOS.address + ', ' + contactItemOS.street + ", " + contactItemOS.township}", styles: SunmiStyles(align: SunmiAlign.left),);

      String label = new NumberFormat("#,###").format(detailResponse.totalWays);
      SunmiPrinter.text('စုစုပေါင်း အထုတ်ရေ။ $label', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.hr();

      // Customer info
      // SunmiPrinter.text('ပစ္စည်းပို့ရန်', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.text('အမည်။ ${contactItemCustomer.name}', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.text(
      //   "ဖုန်းနံပါတ်။ ${contactItemCustomer.primaryPhone}${contactItemCustomer.otherPhones!=null?','+contactItemCustomer.otherPhones:''}",
      //   styles: SunmiStyles(align: SunmiAlign.left),
      // );
      // SunmiPrinter.text(
      //   "လိပ်စာ။ ${contactItemCustomer.address + ', ' + contactItemCustomer.street + ", " + contactItemCustomer.township}",
      //   styles: SunmiStyles(align: SunmiAlign.left),
      // );
      // SunmiPrinter.text( '${tr(LocaleKeys.lblCashToCollectInPrint)} ${new NumberFormat("#,###").format(detailResponse.totalCharges)} Ks', styles: SunmiStyles(align: SunmiAlign.left));
      // SunmiPrinter.hr( );
      // SunmiPrinter.text('${detailResponse.trackingId}', styles: SunmiStyles(align: SunmiAlign.center));
      // QR
      // String qrURL = '${getBaseURL(sharedPrefs)}/v1/image/qr_code/${detailResponse.trackingId}';
      // List<int> qrBytes = await networkImageToBytes( qrURL );
      // if( qrBytes != null){
      //   imglib.Image image = imglib.decodeImage( qrBytes );
      //   imglib.Image thumbnail = imglib.copyResize(image, width: 250);
      //   String base64QR = base64String(imglib.encodeJpg(thumbnail));
      //   if( base64QR != null ){
      //     SunmiPrinter.image(base64QR);
      //   }
      // }

      // if( contactItemCustomer.signature != null ){
      //   SunmiPrinter.emptyLines( 1 );
      //   // Customer sign
      //   String signURL = '${getBaseURL(sharedPrefs)}/v1/image/download/${contactItemCustomer.signature}';
      //   List<int> signBytes = await networkImageToBytes( signURL );
      //   if( signBytes != null){
      //     imglib.Image image = imglib.decodeImage( signBytes );
      //     imglib.Image thumbnail = imglib.copyResize(image, width: 250);
      //     String base64QR = base64String(imglib.encodeJpg(thumbnail));
      //     if( base64QR != null ){
      //       SunmiPrinter.image(base64QR);
      //       SunmiPrinter.text('Customer Signature', styles: SunmiStyles(align: SunmiAlign.center));
      //     }
      //   }
      // }

      SunmiPrinter.hr( );
      SunmiPrinter.emptyLines( 3 );

    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }


  String _createStatus(String status ){
    String title = '';
    if( status == Constant.STATUS_CREATED ){
      title = tr(LocaleKeys.lblItemListDropOffQueue);
    }else if( status == Constant.STATUS_ASSIGNED ){
      title = tr(LocaleKeys.lblItemListToDropOff);
    }else if( status == Constant.STATUS_ON_WAY ){
      title = tr(LocaleKeys.lblItemListDropOffOnWay);
    }else if( status == Constant.STATUS_RETURN ){
      title = tr(LocaleKeys.lblItemListReturn);
    }else if( status == Constant.STATUS_RETRY ){
      title = tr(LocaleKeys.lblItemListRetry);
    }else if( status == Constant.STATUS_CANCELED ){
      title =  tr(LocaleKeys.lblItemListCancelled);
    }else if( status == Constant.STATUS_COMPLETED ){
      title =  tr(LocaleKeys.lblItemListCompleted);
    }else if( status == Constant.STATUS_PICKUP_CREATED ){
      title =  tr(LocaleKeys.lblItemListPickupQueue);
    }else if( status == Constant.STATUS_PICKUP_SCHEDULED ){
      title =  tr(LocaleKeys.lblItemListToPickup);
    }else if( status == Constant.STATUS_PICKUP_ON_WAY ){
      title =  tr(LocaleKeys.lblItemListPickupOnWay);
    }
    return title;
  }
}