
import 'dart:async';
import 'dart:io';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter/material.dart';
import 'package:flutter_sound/flutter_sound.dart';
import 'package:flutter_sound_platform_interface/flutter_sound_recorder_platform_interface.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';


typedef _Fn = void Function();
//const theSource = AudioSource.voiceUpLink;
//const theSource = AudioSource.voiceDownlink;

const theSource = AudioSource.microphone;
class VoiceRecorder extends StatefulWidget {

  VoiceRecorder({Key key,this.onChange}) : super(key: key);
  final Function onChange;

  @override
  _VoiceRecorderState createState() => _VoiceRecorderState();
}

class _VoiceRecorderState extends State<VoiceRecorder> {
  Codec _codec = Codec.aacMP4;
  String _mPath = 'record_file.mp4';
  FlutterSoundRecorder _mRecorder = FlutterSoundRecorder();
  bool _mRecorderIsInited = false;
  bool _save = false;

  @override
  void initState() {
    openTheRecorder().then((value) {
      setState(() {
        _mRecorderIsInited = true;
      });
      if( _mRecorderIsInited )
        record();
    });
    super.initState();
  }

  @override
  void dispose() {
    try{
      _mRecorder.closeAudioSession();
      _mRecorder = null;
    }catch(e){}

    super.dispose();
  }

  Future<void> openTheRecorder() async {

    final Directory extDir = await getApplicationDocumentsDirectory();
    final String tmpDir = '${extDir.path}/gelivery_tmp';
    await Directory(tmpDir).create(recursive: true);
    _mPath = '$tmpDir/${DateTime.now()}-3.mp4';

    if (!kIsWeb) {
      var status = await Permission.microphone.request();
      if (status != PermissionStatus.granted) {
        throw RecordingPermissionException('Microphone permission not granted');
      }
    }
    await _mRecorder.openAudioSession();
    if (!await _mRecorder.isEncoderSupported(_codec) && kIsWeb) {
      _codec = Codec.opusWebM;
      _mPath = 'record_file.webm';
      if (!await _mRecorder.isEncoderSupported(_codec) && kIsWeb) {
        _mRecorderIsInited = true;
        return;
      }
    }
    _mRecorderIsInited = true;
  }

  // ----------------------  Here is the code for recording and playback -------

  void record() {
    // print('Prepare to start');
    _mRecorder
        .startRecorder(
      toFile: _mPath,
      codec: _codec,
      audioSource: theSource,
    )
        .then((value) {
      // print('value after start');
      setState(() {});
    });
  }

  void stopRecorder() async {
    // print('Prepare to stop');
    await _mRecorder.stopRecorder().then((value) {
      setState(() {
      });
      // print(value);
      // print(_save);
      try{
        _mRecorder.closeAudioSession();
        _mRecorder = null;
      }catch(e){}

      if( _save ){
        widget.onChange( _mPath );
      }else{
        try{
          File recordFile  =File(_mPath);
          recordFile.deleteSync(recursive: false);
        }catch(e){e.toString();}
        widget.onChange("Stop");
      }
    });
  }

// ----------------------------- UI --------------------------------------------

  _Fn getRecorderFn() {
    if (!_mRecorderIsInited) {
      return null;
    }
    print('Ready to record');
    return _mRecorder.isStopped ? record : stopRecorder;
  }

  @override
  Widget build(BuildContext context) {
    return Align(
          alignment: Alignment.bottomCenter,
          child: Container(
              margin: EdgeInsets.only(top: 8, bottom: 0),
              child: Wrap(
                  children: <Widget>[
                    ListTile(
                      leading: IconButton(
                        icon: Icon(Icons.clear),
                        tooltip: 'Cancel',
                        onPressed: (){
                          _save = false;
                          stopRecorder();
                        },
                      ),
                      trailing: IconButton(
                        icon: Icon(Icons.send_sharp),
                        color: Theme.of(context).primaryColor,
                        tooltip: 'Send voice',
                        onPressed: !_mRecorderIsInited? null : (){
                          _save = true;
                          stopRecorder();
                        },
                      ),
                      title: Text('Recording'),
                      subtitle: LinearProgressIndicator(),
                    )
                  ]
              )
          )
    );
  }
}