import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:qq_delivery/delivery/config/constant.dart';
import 'package:qq_delivery/delivery/generated/locale_keys.g.dart';
import 'package:qq_delivery/delivery/model/response/address_book_list_response.dart';
import 'package:qq_delivery/delivery/model/response/general_address_response.dart';
import 'package:qq_delivery/delivery/model/shared/general_address_item.dart';
import 'package:qq_delivery/delivery/service/state_service.dart';
import 'package:qq_delivery/delivery/service/street_service.dart';
import 'package:qq_delivery/delivery/service/township_service.dart';


class AddressWidget extends StatefulWidget {
  AddressWidget({Key key, this.stateGuid, this.townshipGuid, this.streetGuid,
    this.street, @required this.controllerStreet, @required this.controllerAddress,
    @required this.onChange, this.embedded, this.title}) : super(key: key);
  final String stateGuid;
  final String townshipGuid;
  final String street;
  final String streetGuid;
  final TextEditingController controllerStreet;
  final TextEditingController controllerAddress;
  final Function onChange;
  final bool embedded;
  final String title;

  @override
  _AddressWidgetState createState() => _AddressWidgetState();
}

class _AddressWidgetState extends State<AddressWidget> {
  bool _processing = false;
  Color _progressColor = Colors.white;
  // User _currentUser = new User();
  // AuthorityResponse _authorityResponse = new AuthorityResponse();

  TownshipService _townshipService = new TownshipService();
  StateService _stateService = new StateService();
  StreetService _streetService = new StreetService();
  GeneralAddressResponse _addressResponseState;
  GeneralAddressItem _addressItemState;
  GeneralAddressResponse _addressResponseTownship;
  GeneralAddressItem _addressItemTownship;
  GeneralAddressResponse _addressResponseStreet;
  GeneralAddressItem _addressItemStreet;

  final TextEditingController _controllerState = new TextEditingController();
  final TextEditingController _controllerTownship = new TextEditingController();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  String _lang = Constant.LANGUAGE_CODE_MY;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  _loadData( ) async{
    _lang = await getLanguage( );
    setState(() {_processing = true;});
    // await _currentUser.fromSharedPreferences();
    // await _authorityResponse.fromSharedPreferences();
    _addressResponseState = await _stateService.list();

    if( widget.stateGuid != null ){
      // select on state list
      if( _addressResponseState!=null && _addressResponseState.items!=null){
        _addressResponseState.items.forEach((element) {
          if( widget.stateGuid == element.guid ){
            _controllerState.text = _lang==Constant.LANGUAGE_CODE_MY? element.nameMM : element.name;
            _addressItemState = element;
            return;
          }
        });
        // select on township
        if( _addressItemState != null){
          await _loadTownships( widget.townshipGuid );
        }

        // select on street
        if( _addressItemTownship != null){
          await _loadStreets( widget.street );
        }
      }
    }
    setState(() {_processing = false;});
  }

  @override
  Widget build(BuildContext context) {
    return widget.embedded!=null && !widget.embedded? Scaffold(
        backgroundColor: Theme.of(context).backgroundColor,
        appBar: AppBar(
          key: _scaffoldKey,
          title: Text(widget.title, style: TextStyle(color: Colors.white)),
          bottom: _processing? _createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
        ),
        body: SingleChildScrollView(
            child: new Form(
                key: _formKey,
                autovalidateMode: AutovalidateMode.onUserInteraction,
                child: Container(
                margin: const EdgeInsets.only(top: 16.0, bottom: 32.0, left: 8.0, right: 8.0),
                  child: Card(
                    // color: Colors.white,
                    elevation: 0,
                    child: Column(
                      children: <Widget>[
                        Container(
                          padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
                          child: new TextFormField(
                            decoration: InputDecoration(
                              hintText: '67, 2nd Floor',
                              labelText: tr(LocaleKeys.lblHouseNo),
                            ),
                            validator: (val) => val.isEmpty ? 'House No. (or) Unit, floor required!' : null,
                            controller: widget.controllerAddress,
                            keyboardType: TextInputType.text,
                          ),
                        ),
                        _stateSelection( context ),
                        _townshipSelection( context ),
                        _streetSelection(context),
                        Container(
                          padding: const EdgeInsets.only(left: 16.0, bottom: 32.0, right: 16.0, top: 16),
                          width: double.infinity,
                          child: RaisedButton(
                            onPressed: () { _submit();},
                            color: Theme.of(context).primaryColorDark,
                            elevation: 1,
                            child: Text(
                              tr(LocaleKeys.btnOK),
                              style: TextStyle(color: Colors.white),
                            ),
                            shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(15.0)),
                          ),
                        ),
                      ],
                    ),
                  )
                )
            )
        ),
    ):
    Scaffold(
      backgroundColor: Colors.white,
      body: Container(
        width: MediaQuery.of(context).size.width,
        child: Column(
          children: <Widget>[
            Container(
              padding: EdgeInsets.only(top:8, bottom: 8, left:16, right: 16),
              child: new TextFormField(
                decoration: InputDecoration(
                  hintText: '67, 2nd Floor',
                  labelText: tr(LocaleKeys.lblHouseNo),
                ),
                validator: (val) => val.isEmpty ? 'Address required!' : null,
                controller: widget.controllerAddress,
                keyboardType: TextInputType.text,
              ),
            ),
            _stateSelection( context ),
            _townshipSelection( context ),
            _streetSelection(context),
          ],
        ),
      ),
    );
  }

  Widget _stateSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type state/division name',
              labelText: tr(LocaleKeys.lblState),
            ),
            controller: _controllerState,
          ),
          suggestionsCallback: (pattern) async {
            return await _stateSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerState.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemState = suggestion;
            widget.onChange(_addressItemState, _addressItemTownship, _addressItemStreet);
            _loadTownships();
          },
          validator: (val) => val.isEmpty ? 'State/Division required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _stateSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseState==null || _addressResponseState.items == null)
      return matches;
    matches.addAll(_addressResponseState.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadTownships([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemState != null) {
      _addressResponseTownship = await _townshipService.list(_addressItemState.guid);
      if( _addressResponseTownship !=null && defaultSelected!=null){
        _addressResponseTownship.items.forEach((element) {
          if( element.guid == defaultSelected){
            _addressItemTownship = element;
            _controllerTownship.text = _addressItemTownship.name;
            return;
          }
        });
      }

    }else{
      _addressResponseTownship.items = [];
      _addressItemTownship = null;
      _controllerTownship.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _townshipSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type township name',
              labelText: tr(LocaleKeys.lblTownship),
            ),
            controller: _controllerTownship,
            enabled: _addressItemState!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _townshipSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            _controllerTownship.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemTownship = suggestion;
            widget.onChange(_addressItemState, _addressItemTownship, _addressItemStreet);
            _loadStreets();
          },
          validator: (val) => val.isEmpty ? 'Township (or) Town required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _townshipSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseTownship==null || _addressResponseTownship.items == null)
      return matches;
    matches.addAll(_addressResponseTownship.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Future<void> _loadStreets([String defaultSelected] ) async{
    setState(() {_processing = true;});
    if( _addressItemTownship != null) {
      _addressResponseStreet = await _streetService.list(_addressItemTownship.guid);
      if( _addressResponseStreet !=null && defaultSelected!=null){
        _addressResponseStreet.items.forEach((element) {
          if( element.name == defaultSelected || element.nameMM == defaultSelected){
            _addressItemStreet = element;
            widget.controllerStreet.text = _addressItemStreet.name;
            return;
          }
        });
      }

      if( defaultSelected!=null )
        widget.controllerStreet.text = defaultSelected;


    }else{
      _addressResponseStreet.items = [];
      _addressItemStreet = null;
      widget.controllerStreet.text = '';
    }
    setState(() {_processing = false;});
  }

  Widget _streetSelection( BuildContext context ){
    return Container(
        margin: EdgeInsets.only(top: 8, left: 16, right: 16, bottom: 8),
        child: TypeAheadFormField(
          textFieldConfiguration: TextFieldConfiguration(
            autofocus: false,
            decoration: InputDecoration(
              hintText: 'Type street name',
              labelText: tr(LocaleKeys.lblStreet),
            ),
            controller: widget.controllerStreet,
            enabled: _addressItemTownship!=null,
          ),
          suggestionsCallback: (pattern) async {
            return await _streetSuggestion(pattern);
          },
          itemBuilder: (context, suggestion) {
            return ListTile(
              title: Text('${suggestion.nameMM}${suggestion.name==null ? "" : " - " + suggestion.name}'),
            );
          },
          onSuggestionSelected: (suggestion) {
            widget.controllerStreet.text = _lang==Constant.LANGUAGE_CODE_MY? suggestion.nameMM : suggestion.name;
            _addressItemStreet = suggestion;
            widget.onChange(_addressItemState, _addressItemTownship, _addressItemStreet);
          },
          noItemsFoundBuilder: (BuildContext context) => new Container( height: 0, ),
          validator: (val) => val.isEmpty ? 'Street name required!' : null,
        )
    );
  }
  Future<List<GeneralAddressItem>> _streetSuggestion(String query) async{
    List<GeneralAddressItem> matches = List();
    if( _addressResponseStreet==null || _addressResponseStreet.items == null)
      return matches;
    matches.addAll(_addressResponseStreet.items);

    matches.retainWhere((s) {
      GeneralAddressItem addressBookItem = s;
      if( addressBookItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else if( addressBookItem.nameMM.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void _submit(){
    final FormState form = _formKey.currentState;
    if (!form.validate() ) {
      _showMessage('Form is not valid! Please review and correct.');
    } else {
      form.save();
      AddressBookItem addressBookItem = new AddressBookItem(
          personGuid: null,
          name: null,
          address: widget.controllerAddress.text,
          street: widget.controllerStreet.text,
          townshipName: _lang==Constant.LANGUAGE_CODE_MY? _addressItemTownship.nameMM : _addressItemTownship.name,
          townshipGuid: _addressItemTownship.guid,
          addressType: null,
          primaryPhone: null,
          otherPhones: null,
          email: null,
          latitude: (widget.controllerStreet.text == _addressItemStreet?.name??'') || (widget.controllerStreet.text == _addressItemStreet?.nameMM??'') ? _addressItemStreet.latitude : null,
          longitude: (widget.controllerStreet.text == _addressItemStreet?.name??'') || (widget.controllerStreet.text == _addressItemStreet?.nameMM??'') ? _addressItemStreet.longitude : null,
          comment: null,
          active: true,
          updatedOn: null,
          guid: null,
          avatar: null,
          stateGuid: _addressItemState.guid,
          primaryContact: null
      );
      Navigator.pop( context, addressBookItem );
    }
  }

  void _showMessage( String title ){
    final snackBar = SnackBar(
      content: Text(title),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () {
          // Some code to undo the change.
        },
      ),
    );
    Scaffold.of(_scaffoldKey.currentContext).showSnackBar(snackBar);
    setState(() {});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  Future<bool> _onWillPop() async {
    return true;
  }
}