import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:encrypt/encrypt.dart';


abstract class Constant{

  static const String APP_NAME = 'HRM';
  static const bool ENABLE_LOGS = false;
  static const int ROW_PER_PAGE = 20;
  static const bool TESTING_MODE = false;

  static const String CURRENT_DEVICE_TOKEN_VALUE = "hrm_cdtv";
  static const String CURRENT_DEVICE_TOKEN_EXPIRED = "hrm_cdte";
  static const String CURRENT_USER_TOKEN_VALUE = "hrm_cutv";
  static const String CURRENT_USER_REFRESH_TOKEN_VALUE = "hrm_curtv";
  static const String CURRENT_USER_TOKEN_EXPIRED = "hrm_cute";
  static const String CURRENT_USER_DATA = "hrm_cud";
  static const String CURRENT_USER_DATA_EXT_EMPLOYEE = "hrm_cudEExt";
  static const String CURRENT_USER_DATA_AUTHORITY = "hrm_cudauth";
  static const String FIREBASE_TOKEN_KEY = "gfmtk";
  static const String DEVICE_GUID = "hrm_udvg";
  static const String CURRENT_KIOSK_DATA = "hrm_ckd";
  static const String KIOSK_SETTINGS_AUTO_ATTENDANCE = "hrm_ksauto";
  static const String KIOSK_SETTINGS_DETECTION_DISTANCE = "hrm_ksdd";
  static const String KIOSK_SETTINGS_TIME_INTERVAL = "hrm_ksti";
  static const String KIOSK_SETTINGS_SCREEN_TIMEOUT = "hrm_kssto";
  static const String KIOSK_SETTINGS_MULTI_FACES = "hrm_ksmf";
  static const String KIOSK_SETTINGS_IN_OUT_MODE = "hrm_ksio";
  static const String SETTINGS_ATTENDANCE_SHARING = "hrm_sasing";
  static const String SETTINGS_PUNCH_IN_REMINDER = "hrm_spir";
  static const String SETTINGS_PUNCH_OUT_REMINDER = "hrm_spor";
  static const String CAMPAIGN_LAST_ID = "hrm_camlid";
  static const String LANGUAGE = "hrm_language";
  static const String CAMERA_SHUTTER_SOUND = "hrm_cssound";
  static const String AUTHENTICATION_2FA = "hrm_a2fa";
  static const String CURRENT_ORG_DATA = "hrm_corg";

  static const String BASE_URL_PRIMARY = "https://api.quickdoor2door.com/facial-recognition-hrm";
  static const String BASE_URL_DEV = "https://dev.quickdoor2door.com/facial-recognition-hrm";
  static const String PLATFORM_ANDROID = "Android";
  static const String PLATFORM_IOS = "iOS";
  static const String SALTED_KEY_32 = "ThJZU422uyDsGkvW2fn8xAPH452nrKeJ";
  static const String BASE_URL_STORAGE = "hrm_bustrge";
  static const String DEFAULT_ORGANIZATION_ROLE_GUID = "3c442251-3ede-45ab-9343-c30a89a97dba";
  static const String DEFAULT_ORGANIZATION_PLAN_PRO = "Pro";
  static const String CONTACT_PHONE = '+959 123 435 456';
  static const String NOTIFICATION_PAYLOAD_DEFAULT = 'notification';
  static const String HRM_URL = "https://hrtest.quickdoor2door.com";

  static const String PLAN_BASIC = 'Basic';
  static const String PLAN_ADVANCED = 'Advanced';
  static const String PLAN_PRO = 'Pro';

  static const String EMPLOYEE_STATUS_ACTIVE = "Active";
  static const String EMPLOYEE_STATUS_TERMINATED = "Terminated";
}

String toUTCShortFormat(DateTime dateTime){
  DateFormat dateFormat = DateFormat("yyyy-MM-dd");
  return dateFormat.format(dateTime.toUtc());
}

String toUTCLongFormat(DateTime dateTime){
  DateFormat dateFormat = DateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
  return dateFormat.format(dateTime.toUtc());
}

Future<bool> getKioskSettingsAutoAttendance( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.KIOSK_SETTINGS_AUTO_ATTENDANCE)?? true;
}
Future<void> setKioskSettingsAutoAttendance( bool autoAttendance) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.KIOSK_SETTINGS_AUTO_ATTENDANCE, autoAttendance);
}


Future<double> getKioskSettingsFaceDetectionDistance( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getDouble( Constant.KIOSK_SETTINGS_DETECTION_DISTANCE )?? 1.0;
}
Future<void> setKioskSettingsFaceDetectionDistance( double distance ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setDouble( Constant.KIOSK_SETTINGS_DETECTION_DISTANCE, distance );
}

Future<int> getKioskSettingsTimeIntervalMinutes( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getInt(Constant.KIOSK_SETTINGS_TIME_INTERVAL)?? 120;
}
Future<void> setKioskSettingsTimeIntervalMinutes( double interval) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setInt(Constant.KIOSK_SETTINGS_TIME_INTERVAL, (interval*60).toInt() );
}

Future<bool> getKioskSettingsScreenAlwaysOn( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.KIOSK_SETTINGS_SCREEN_TIMEOUT)?? true;
}
Future<void> setKioskSettingsScreenAlwaysOn( bool autoAttendance) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.KIOSK_SETTINGS_SCREEN_TIMEOUT, autoAttendance);
}

Future<bool> getKioskSettingsMultipleFaces( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.KIOSK_SETTINGS_MULTI_FACES)?? false;
}
Future<void> setKioskSettingsMultipleFaces( bool autoAttendance) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.KIOSK_SETTINGS_MULTI_FACES, autoAttendance);
}



Future<bool> getKioskSettingsInOutMode( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.KIOSK_SETTINGS_IN_OUT_MODE)?? true;
}
Future<void> setKioskSettingsInOutMode( bool autoAttendance) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.KIOSK_SETTINGS_IN_OUT_MODE, autoAttendance);
}

Future<bool> getSettingsAttendanceSharing( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.SETTINGS_ATTENDANCE_SHARING)?? false;
}
Future<void> setSettingsAttendanceSharing( bool autoAttendance) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.SETTINGS_ATTENDANCE_SHARING, autoAttendance);
}


Future<double> getSettingsPunchInReminder( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getDouble( Constant.SETTINGS_PUNCH_IN_REMINDER )?? 15;
}
Future<void> setSettingsPunchInReminder( double distance ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setDouble( Constant.SETTINGS_PUNCH_IN_REMINDER, distance );
}

Future<double> getSettingsPunchOutReminder( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getDouble( Constant.SETTINGS_PUNCH_OUT_REMINDER )?? 5;
}
Future<void> setSettingsPunchOutReminder( double distance ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setDouble( Constant.SETTINGS_PUNCH_OUT_REMINDER, distance );
}

Future<int> getCampaignLastId( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getInt(Constant.CAMPAIGN_LAST_ID)?? 0;
}
Future<void> setCampaignLastId( int campaignLastId) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setInt(Constant.CAMPAIGN_LAST_ID, campaignLastId );
}

Future<String> getLanguage( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getString(Constant.LANGUAGE)?? 'en';
}
Future<void> setLanguage( String language) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setString(Constant.LANGUAGE, language );
}

Future<bool> getCameraShutterSoundNotice( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.CAMERA_SHUTTER_SOUND)?? true;
}
Future<void> setCameraShutterSoundNotice( bool sound) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.CAMERA_SHUTTER_SOUND, sound);
}

Future<bool> get2FA( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getBool(Constant.AUTHENTICATION_2FA)?? false;
}
Future<void> set2FA( bool a2fa) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setBool( Constant.AUTHENTICATION_2FA, a2fa);
}

void makeCall( String phone ) async{
  if (await canLaunch('tel:$phone')) {
    await launch('tel:$phone');
  } else {
    throw 'Could not launch $phone';
  }
}

void openBrowser( String url ) async{
  if (await canLaunch('$url')) {
    await launch(url);
  } else {
    throw 'Could not launch $url';
  }
}

Future<String> setBaseURL( String url ) async{
  final key = Key.fromUtf8( Constant.SALTED_KEY_32 );
  final iv = IV.fromLength(16);
  final encrypter = Encrypter(AES(key));
  final encrypted = encrypter.encrypt(url, iv: iv);
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  sharedPrefs.setString( Constant.BASE_URL_STORAGE, encrypted.base64);
  return encrypted.base64;
}

String getBaseURL( SharedPreferences sharedPrefs ) {

  if( Constant.TESTING_MODE)
    return Constant.BASE_URL_DEV;

  String encrypted = sharedPrefs.getString(Constant.BASE_URL_STORAGE)?? '';
  if( encrypted == ''){
    return Constant.BASE_URL_PRIMARY;
  }else{
    final key = Key.fromUtf8( Constant.SALTED_KEY_32 );
    final iv = IV.fromLength(16);
    final encrypter = Encrypter(AES(key));
    final decrypted = encrypter.decrypt(Encrypted.fromBase64(encrypted), iv: iv);

    return decrypted;
  }
}