
import '/hrm/config/constant.dart';
import '/hrm/database/entity/attendance.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:sqflite/sqlite_api.dart';


class AttendanceDao{

  Database _db;
  String _tableName = 'attendance';

  Future open() async {
    _db = await openDatabase(
      join(await getDatabasesPath(), 'system2.db'),
      onCreate: (db, version) {
        return db.execute(
          "CREATE TABLE IF NOT EXISTS $_tableName(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `facial` INTEGER NOT NULL, `inTime` INTEGER NOT NULL, `inNote` TEXT, `inLat` DOUBLE, `inLon` DOUBLE, `inFace` TEXT NOT NULL, `outTime` INTEGER NOT NULL, `outNote` TEXT, `outLat` DOUBLE, `outLon` DOUBLE, `outFace` TEXT NOT NULL, `guid` TEXT NOT NULL)",
        );
      },
      version: 1,
    );
    try {
      _db.rawQuery("PRAGMA encoding ='UTF-8'");
      _db.rawQuery( "PRAGMA journal_mode ='delete'");
    }catch (e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

  }

  Future<void> insertFacial(Attendance attendance) async {
    await _db.insert(
      _tableName,
      attendance.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<List<Attendance>> attendances(int limit, int offset) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        limit: limit,
        offset: offset
    );
    return List.generate( maps.length, (i) {
      Attendance attendance = new Attendance();
      return attendance.fromMap(maps[i]);
    });

  }

  Future<Attendance> findByGuid(String guid) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'guid=?',
        whereArgs: [guid]
    );

    if (maps.isNotEmpty) {
      Attendance attendance = new Attendance();
      return attendance.fromMap(maps.first);
    }else
      return null;

  }

  Future<Attendance> findById(int id) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
    if (maps.isNotEmpty) {
      Attendance attendance = new Attendance();
      return attendance.fromMap(maps.first);
    }else
      return null;
  }

  Future<List<Attendance>> findByFacial(int facialId) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'facial=?',
        whereArgs: [facialId]
    );
    return List.generate( maps.length, (i) {
      Attendance attendance = new Attendance();
      return attendance.fromMap(maps[i]);
    });

  }
  Future<int> deleteById( int id ) async{
    return await _db.delete(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
  }

  Future close() async => _db.close();

}