
import '/hrm/database/entity/facial.dart';
import '/hrm/config/constant.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';


class FacialDao{
  Database _db;
  String _tableName = 'userFace';

  Future open() async {
    _db = await openDatabase(
      join(await getDatabasesPath(), 'system.db'),
      onCreate: (db, version) {
        return db.execute(
          "CREATE TABLE IF NOT EXISTS $_tableName(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `name` TEXT NOT NULL, `profile` TEXT NOT NULL, `face1` TEXT, `face2` TEXT, `face3` TEXT, `status` TEXT NOT NULL, `createdOn` INTEGER NOT NULL, `updatedOn` INTEGER NOT NULL, `guid` TEXT NOT NULL)",
        );
      },
      version: 1,
    );
    try {
      _db.rawQuery("PRAGMA encoding ='UTF-8'");
      _db.rawQuery( "PRAGMA journal_mode ='delete'");
    }catch (e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

  }

  Future<void> insertFacial(Facial facial) async {
    await _db.insert(
      _tableName,
      facial.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<List<Facial>> faces(int limit, int offset) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        limit: limit,
        offset: offset
    );
    return List.generate( maps.length, (i) {
      return Facial(
        id: maps[i]['id'],
        name: maps[i]['name'],
        profile: maps[i]['profile'],
        face1: maps[i]['face1'],
        face2: maps[i]['face2'],
        face3: maps[i]['face3'],
        status: maps[i]['status'],
        createdOn: maps[i]['createdOn'],
        updatedOn: maps[i]['updatedOn'],
        guid: maps[i]['guid']
      );
    });

  }

  Future<Facial> findByGuid(String guid) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'guid=?',
        whereArgs: [guid]
    );

    if (maps.isNotEmpty) {
      Facial facial = new Facial();
      return await facial.fromMap(maps.first);
    }else
      return null;

  }

  Future<Facial> findById(int id) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
    if (maps.isNotEmpty) {
      Facial facial = new Facial();
      return await facial.fromMap(maps.first);
    }else
      return null;
  }

  Future<int> deleteById( int id ) async{
    return await _db.delete(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
  }

  Future close() async => _db.close();

}