

import '/hrm/database/entity/facial_template.dart';
import '/hrm/config/constant.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';



class FacialTemplateDao{
  Database _db;
  String _tableName = 'facialTemplate';

  Future open() async {
    _db = await openDatabase(
      join(await getDatabasesPath(), 'systems1.db'),
      onCreate: (db, version) {
        return db.execute(
          "CREATE TABLE IF NOT EXISTS $_tableName(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `facial` INTEGER NOT NULL, `template` BLOB NOT NULL, `createdOn` INTEGER NOT NULL)",
        );
      },
      version: 1,
    );
    try {
      _db.rawQuery("PRAGMA encoding ='UTF-8'");
      _db.rawQuery( "PRAGMA journal_mode ='delete'");
    }catch (e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

  }
  Future<void> insertFacialTemplate(FacialTemplate facialTemplate) async {
    await _db.insert(
      _tableName,
      facialTemplate.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<List<FacialTemplate>> templates() async {
      final List<Map<String, dynamic>> maps = await _db.query('$_tableName');
      return List.generate(maps.length, (i) {
        return FacialTemplate(
          id: maps[i]['id'],
          facial: maps[i]['facial'],
          template: maps[i]['template'],
          createdOn: maps[i]['createdOn'],
        );
      });
    }

  Future<List<FacialTemplate>> templatesByFacialId(int facialId) async {
    final List<Map<String, dynamic>> maps = await _db.query(
        '$_tableName',
      where: 'facial=?',
      whereArgs: [facialId]
    );
    return List.generate(maps.length, (i) {
      return FacialTemplate(
        id: maps[i]['id'],
        facial: maps[i]['facial'],
        template: maps[i]['template'],
        createdOn: maps[i]['createdOn'],
      );
    });
  }

  Future<int> deleteByFacialId(int facialId) async{
    return await _db.delete(
        '$_tableName',
        where: 'facial=?',
        whereArgs: [facialId]
    );
  }

    Future close() async => _db.close();

}