
import '/hrm/database/entity/log.dart';
import '/hrm/config/constant.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:sqflite/sqlite_api.dart';


class LogDao{

  Database _db;
  String _tableName = 'logs';

  Future open() async {
    _db = await openDatabase(
      join(await getDatabasesPath(), 'system3.db'),
      onCreate: (db, version) {
        return db.execute(
          "CREATE TABLE IF NOT EXISTS $_tableName(`id` INTEGER PRIMARY KEY AUTOINCREMENT, `logTime` INTEGER NOT NULL, `note` TEXT, `facialGuid` TEXT NOT NULL)",
        );
      },
      version: 1,
    );
    try {
      _db.rawQuery("PRAGMA encoding ='UTF-8'");
      _db.rawQuery( "PRAGMA journal_mode ='delete'");
    }catch (e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

  }

  Future<void> insertLog(Log log) async {
    await _db.insert(
      _tableName,
      log.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<void> update( Log log, String facialGuid ) async{
    await _db.update(
        _tableName,
        log.toMap(),
        where: 'facialGuid=?',
        whereArgs: [facialGuid],
        conflictAlgorithm: ConflictAlgorithm.replace
    );
  }

  Future<List<Log>> list(int limit, int offset) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        limit: limit,
        offset: offset
    );
    return List.generate( maps.length, (i) {
      Log log = new Log();
      log.fromMap(maps[i]);
      return log;
    });

  }

  Future findByFacialGuid(String guid) async {

    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'facialGuid=?',
        whereArgs: [guid]
    );

    if (maps.isNotEmpty) {
      Log log = new Log();
      log.fromMap(maps.first);
      return log;
    }else
      return null;

  }

  Future<Log> findById(int id) async {
    final List<Map<String, dynamic>> maps = await _db.query(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
    if (maps.isNotEmpty) {
      Log log = new Log();
      return log.fromMap(maps.first);
    }else
      return null;
  }

  Future<int> deleteById( int id ) async{
    return await _db.delete(
        _tableName,
        where: 'id=?',
        whereArgs: [id]
    );
  }
  Future<int> deleteByFacialGuid( String guid ) async{
    return await _db.delete(
        _tableName,
        where: 'facialGuid=?',
        whereArgs: [guid]
    );
  }

  Future close() async => _db.close();

}