

import 'package:equatable/equatable.dart';
import '/hrm/model/response/geofencing_detail_response.dart';


class GeofencingAddRequest extends Equatable{

  static const int RESTRICTED_TYPE_GPS = 1;
  static const int RESTRICTED_TYPE_WIFI = 2;
  static const int WIFI_CONDITION_CONNECTED = 1;
  static const int WIFI_CONDITION_DETECTED = 2;
  static const String STATUS_ACTIVE = "Active";
  static const String STATUS_INACTIVE = "Inactive";
  static const String ACTION_IN = "in";
  static const String ACTION_OUT = "out";
  static const String ACTION_BOTH = "both";

  String organizationGUID;
  String name;
  int restrictedType;
  double latitude;
  double longitude;
  int radius;
  String bssId;
  String ssId;
  String neighbouring;
  int wifiCondition;
  String action;
  List<String> employeeGUIDs;
  List<GeofencingEmployeeItem> employees;

  GeofencingAddRequest({this.organizationGUID, this.name, this.restrictedType,
    this.latitude, this.longitude, this.radius, this.bssId, this.ssId,
    this.neighbouring, this.wifiCondition, this.action, this.employeeGUIDs, this.employees});

  @override
  List<Object> get props => [ organizationGUID, name, restrictedType,
    latitude, longitude, radius, bssId, ssId,
    neighbouring, wifiCondition, action, employeeGUIDs, employees];


  Map<String, dynamic> toMap(){

    if( employees != null){
      employeeGUIDs = new List<String>();
      employees.forEach((element) {
        employeeGUIDs.add(element.employeeGUID);
      });
    }

    Map<String, dynamic> data = {
      'organizationGUID': organizationGUID,
      'name': name,
      'restrictedType': restrictedType,
      'latitude': latitude,
      'longitude':longitude,
      'radius':radius,
      'bssId':bssId,
      'ssId':ssId,
      'neighbouring':neighbouring,
      'wifiCondition':wifiCondition,
      'action':action,
      'employeeGUIDs':employeeGUIDs
    };

    return data;
  }

  void addItem( GeofencingEmployeeItem item ){
    if( employees == null)
      employees = new List<GeofencingEmployeeItem>();

    bool alreadyExist = false;
    employees.forEach((element) {
      if( element.employeeGUID == item.employeeGUID){
        alreadyExist = true;
        return;
      }
    });

    if( !alreadyExist)
      employees.add(item);
  }
}