import 'dart:convert';
import 'package:equatable/equatable.dart';



class AttendanceSearchResponse extends Equatable{

  List<AttendanceSearchItem> attendances;
  int totalRecords;


  AttendanceSearchResponse({this.attendances, this.totalRecords});
  @override
  List<Object> get props => [attendances, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final attendancesData = data["attendances"] as List;
      attendances = new List<AttendanceSearchItem>();
      attendancesData.forEach((element) {
        AttendanceSearchItem attendanceSearchItem = new AttendanceSearchItem();
        attendanceSearchItem.fromMap(element);
        attendances.add(attendanceSearchItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class AttendanceSearchItem extends Equatable{

  String employeeName;
  String facialGUID;
  String departmentName;
  String supervisorName;
  String province;
  String country;
  String guid;
  DateTime inTime;
  DateTime outTime;
  Duration dutyInTime;
  Duration dutyOutTime;
  DateTime dutyIn;
  DateTime dutyOut;
  bool late;
  bool early;
  String employeeId;
  int otMinutes;

  AttendanceSearchItem({this.employeeName, this.facialGUID, this.departmentName,
  this.supervisorName, this.province, this.country, this.guid, this.inTime,
    this.outTime, this.dutyInTime, this.dutyOutTime, this.dutyIn, this.dutyOut,
    this.late, this.early});
  @override
  List<Object> get props => [employeeName, facialGUID, departmentName,
    supervisorName, province, country, guid, inTime,
    outTime, dutyInTime, dutyOutTime, dutyIn, dutyOut, late, early];
  @override
  String toString() => this.employeeName;

  fromMap( Map<String, dynamic> data ){
    employeeName = data["employeeName"];
    facialGUID = data["facialGUID"];
    departmentName = data["departmentName"];
    supervisorName = data["supervisorName"];
    province = data["province"];
    country = data["country"];
    guid = data["guid"];
    inTime = data["inTime"]!=null? DateTime.parse(data["inTime"]) : null;
    outTime = data["outTime"]!=null?DateTime.parse(data["outTime"]) : null;

    dutyInTime =data["dutyInTime"]!=null?
    Duration(hours: int.parse(data["dutyInTime"].toString().substring(0, data["dutyInTime"].toString().indexOf(':'))),
        minutes: int.parse(data["dutyInTime"].toString().substring(data["dutyInTime"].toString().indexOf(':') + 1))
    ) : null;

    dutyOutTime = data["dutyOutTime"]!=null?
    Duration(hours: int.parse(data["dutyOutTime"].toString().substring(0, data["dutyOutTime"].toString().indexOf(':'))),
        minutes: int.parse(data["dutyOutTime"].toString().substring(data["dutyOutTime"].toString().indexOf(':') + 1))
    ) : null;


    dutyIn = data["dutyIn"]!=null? DateTime.parse(data["dutyIn"]) : null;
    dutyOut = data["dutyOut"]!=null?DateTime.parse(data["dutyOut"]) : null;

    late = data["late"];
    early = data["early"];
    employeeId = data["employeeId"];
    otMinutes = data["otMinutes"];
  }
}