import 'dart:convert';
import 'package:equatable/equatable.dart';



class EmployeeSearchResponse extends Equatable {

  List<EmployeeSearchItem> employees;
  int totalRecords;

  EmployeeSearchResponse({this.employees, this.totalRecords});

  @override
  List<Object> get props => [employees, totalRecords];

  @override
  String toString() => 'Total records=${this.totalRecords}';


  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final employeesData = data["employees"] as List;
      employees = new List<EmployeeSearchItem>();
      employeesData.forEach((element) {
        EmployeeSearchItem employeeSearchItem = new EmployeeSearchItem();
        employeeSearchItem.fromMap(element);
        employees.add(employeeSearchItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class EmployeeSearchItem extends Equatable{
  String departmentName;
  String supervisorName;
  String employeeId;
  String name;
  String gender;
  String jobTitleName;
  String roleName;
  Duration dutyInTime;
  Duration dutyOutTime;
  String faceGuid;
  String guid;
  String departmentGuid;
  String supervisorGUID;
  String userGUID;

  EmployeeSearchItem({this.departmentName, this.supervisorName, this.employeeId, this.name,
    this.gender, this.jobTitleName, this.roleName, this.dutyInTime, this.dutyOutTime,
    this.faceGuid, this.guid, this.departmentGuid, this.supervisorGUID, this.userGUID});

  @override
  List<Object> get props => [departmentName, supervisorName, employeeId, name,
    gender, jobTitleName, roleName, dutyInTime, dutyOutTime,
    faceGuid, guid, departmentGuid, supervisorGUID, userGUID];

  @override
  String toString() => this.name;
  fromMap( Map<String, dynamic> data ){
    departmentName = data["departmentName"];
    supervisorName = data["supervisorName"];
    employeeId = data["employeeId"];
    name = data["name"];
    gender = data["gender"];
    jobTitleName = data["jobTitleName"];
    roleName = data["roleName"];

    dutyInTime =data["dutyInTime"]!=null?
    Duration(hours: int.parse(data["dutyInTime"].toString().substring(0, data["dutyInTime"].toString().indexOf(':'))),
        minutes: int.parse(data["dutyInTime"].toString().substring(data["dutyInTime"].toString().indexOf(':') + 1))
        ) : null;

    dutyOutTime = data["dutyOutTime"]!=null?
    Duration(hours: int.parse(data["dutyOutTime"].toString().substring(0, data["dutyOutTime"].toString().indexOf(':'))),
        minutes: int.parse(data["dutyOutTime"].toString().substring(data["dutyOutTime"].toString().indexOf(':') + 1))
    ) : null;
    faceGuid = data["faceGuid"];
    guid = data["guid"];
    departmentGuid = data["departmentGuid"];
    supervisorGUID = data["supervisorGUID"];
    userGUID = data["userGUID"];
  }
}