import 'dart:convert';
import 'package:equatable/equatable.dart';



class GeofencingDetailResponse extends Equatable{
  String organizationGUID;
  String organization;
  String name;
  int restrictedType;
  double latitude;
  double longitude;
  int radius;
  String bssId;
  String ssId;
  String neighbouring;
  int wifiCondition;
  DateTime updatedOn;
  String status;
  String action;
  String guid;
  List<GeofencingEmployeeItem> items;

  GeofencingDetailResponse({this.organizationGUID, this.organization, this.name,
    this.restrictedType, this.latitude, this.longitude, this.radius, this.bssId,
    this.ssId, this.neighbouring, this.wifiCondition, this.updatedOn,
    this.status, this.action, this.guid, this.items});

  @override
  List<Object> get props => [organizationGUID, organization, name,
    restrictedType, latitude, longitude, radius, bssId,
    ssId, neighbouring, wifiCondition, updatedOn,
    status, action, guid, items];
  @override
  String toString() => 'Total records=${this.name}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];
      organizationGUID = data["organizationGUID"];
      organization = data["organization"];
      name = data["name"];
      restrictedType = data["restrictedType"];
      latitude = data["latitude"];
      longitude = data["longitude"];
      radius = data["radius"];
      bssId = data["bssId"];
      ssId = data["ssId"];
      neighbouring = data["neighbouring"];
      wifiCondition = data["wifiCondition"];
      updatedOn = data["updatedOn"]!=null? DateTime.parse(data["updatedOn"]) : data["updatedOn"];
      status = data["status"];
      action = data["action"];
      guid = data["guid"];

      final geofencings = data["items"] as List;
      items = new List<GeofencingEmployeeItem>();
      geofencings.forEach((element) {
        GeofencingEmployeeItem geofencingItem = new GeofencingEmployeeItem();
        geofencingItem.fromMap(element);
        items.add(geofencingItem);
      });

      return true;
    }else{
      return false;
    }
  }
}

class GeofencingEmployeeItem extends Equatable{
  String employee;
  String department;
  String jobTitle;
  String employeeGUID;
  String facialGUID;
  String guid;
  bool check = false;

  GeofencingEmployeeItem({this.employee, this.department, this.jobTitle,
    this.employeeGUID, this.facialGUID, this.guid, this.check});

  @override
  List<Object> get props => [employee, department, jobTitle, employeeGUID, facialGUID, guid, check];

  @override
  String toString() => this.employee;

  fromMap( Map<String, dynamic> data ){
    employee = data["employee"];
    department = data["department"];
    jobTitle = data["jobTitle"];
    employeeGUID = data["employeeGUID"];
    facialGUID = data["facialGUID"];
    guid = data["guid"];
    check = false;
  }
}