import 'dart:convert';
import 'package:equatable/equatable.dart';



class LeaveSearchResponse extends Equatable{

  List<LeaveSearchItem> leaves;
  int totalRecords;


  LeaveSearchResponse({this.leaves, this.totalRecords});
  @override
  List<Object> get props => [leaves, totalRecords];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  bool fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    if( resultData["success"]){
      Map<String, dynamic> data = resultData["data"];

      final attendancesData = data["leaves"] as List;
      leaves = new List<LeaveSearchItem>();
      attendancesData.forEach((element) {
        LeaveSearchItem attendanceSearchItem = new LeaveSearchItem();
        attendanceSearchItem.fromMap(element);
        leaves.add(attendanceSearchItem);
      });

      totalRecords = data["totalRecords"];
      return true;
    }else{
      return false;
    }
  }
}

class LeaveSearchItem extends Equatable{

  static const String STATUS_APPROVED = "Approved";
  static const String STATUS_PENDING = "Pending";
  static const String STATUS_REJECTED = "Rejected";
  static const String STATUS_CANCELLATION_REQUESTED = "Cancellation Requested";
  static const String STATUS_CANCELLED = "Cancelled";
  static const String STATUS_PROCESSING = "Processing";

  String employeeName;
  String employeeGuid;
  String facialGuid;
  String leaveType;
  DateTime startDate;
  DateTime endDate;
  String status;
  String guid;
  double total;
  String department;
  String details;
  String action;

  LeaveSearchItem({this.employeeName, this.employeeGuid, this.facialGuid, this.leaveType,
  this.startDate, this.endDate, this.status, this.guid, this.total, this.department,
    this.details, this.action});
  @override
  List<Object> get props => [employeeName, employeeGuid, facialGuid, leaveType,
    startDate, endDate, status, guid, total, department, details, action];
  @override
  String toString() => this.employeeName;

  fromMap( Map<String, dynamic> data ){
    employeeName = data["employeeName"];
    employeeGuid = data["employeeGuid"];
    facialGuid = data["facialGuid"];
    leaveType = data["leaveType"];
    startDate = data["startDate"]!=null? DateTime.parse(data["startDate"]) : null;
    endDate = data["endDate"]!=null?DateTime.parse(data["endDate"]) : null;
    status = data["status"];
    guid = data["guid"];
    total = data["total"];
    department = data["department"];
    details = data["details"];
    action = STATUS_PENDING;
  }
}