import 'dart:convert';
import 'package:equatable/equatable.dart';



class PayGradeResponse extends Equatable{
  List<PayGradeItem> items;
  int totalRecords;
  bool success;
  String message;


  PayGradeResponse({this.items, this.totalRecords, this.success, this.message});
  @override
  List<Object> get props => [items, totalRecords, success, message];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    success = resultData["success"];

    if( success ){
      Map<String, dynamic> data = resultData["data"];

        final geofencings = data["items"] as List;
        items = new List<PayGradeItem>();
        geofencings.forEach((element) {
          PayGradeItem item = new PayGradeItem();
          item.fromMap(element);
          items.add(item);
      });

      totalRecords = data["totalRecords"];
    }else{
      message = resultData["message"];
    }
  }

}

class PayGradeItem extends Equatable{

  String organizationGuid;
  String currencyTypeGuid;
  String currencyTypeName;
  String currencyTypeCode;
  String name;
  double minSalary;
  double maxSalary;
  DateTime createdOn;
  String guid;

  PayGradeItem({this.organizationGuid, this.currencyTypeGuid, this.currencyTypeName,
    this.currencyTypeCode, this.name, this.minSalary, this.maxSalary,
    this.createdOn, this.guid});

  @override
  List<Object> get props => [organizationGuid, currencyTypeGuid, currencyTypeName,
    currencyTypeCode, name, minSalary, maxSalary, createdOn, guid];

  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    organizationGuid = data["organizationGuid"];
    currencyTypeGuid = data["currencyTypeGuid"];
    currencyTypeName = data["currencyTypeName"];
    currencyTypeCode = data["currencyTypeCode"];
    name = data["name"];
    minSalary = data["minSalary"];
    maxSalary = data["maxSalary"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    guid = data["guid"];
  }

}