import 'dart:convert';
import 'package:equatable/equatable.dart';



class PayslipResponse extends Equatable{
  List<PayslipItem> items;
  int totalRecords;
  bool success;
  String message;


  PayslipResponse({this.items, this.totalRecords, this.success, this.message});
  @override
  List<Object> get props => [items, totalRecords, success, message];
  @override
  String toString() => 'Total records=${this.totalRecords}';

  fromJson(String json){
    if( json == null) return false;
    Map<String, dynamic> resultData = jsonDecode(json);
    success = resultData["success"];

    if( success ){
      Map<String, dynamic> data = resultData["data"];

        final geofencings = data["items"] as List;
        items = new List<PayslipItem>();
        geofencings.forEach((element) {
          PayslipItem geofencingItem = new PayslipItem();
          geofencingItem.fromMap(element);
          items.add(geofencingItem);
      });

      totalRecords = data["totalRecords"];
    }else{
      message = resultData["message"];
    }
  }

}

class PayslipItem extends Equatable{

  String name;
  String departmentName;
  String payFrequencyName;
  DateTime dateStart;
  DateTime dateEnd;
  String status;
  DateTime createdOn;
  String guid;

  PayslipItem({this.name, this.departmentName, this.payFrequencyName,
    this.dateStart, this.dateEnd, this.status, this.createdOn, this.guid});

  @override
  List<Object> get props => [name, departmentName, payFrequencyName, dateStart,
    dateEnd, status, createdOn, guid];

  @override
  String toString() => this.name;

  fromMap( Map<String, dynamic> data ){
    name = data["name"];
    departmentName = data["departmentName"];
    payFrequencyName = data["payFrequencyName"];
    dateStart = data["dateStart"]!=null ? DateTime.parse(data["dateStart"]) : null;
    dateEnd = data["dateEnd"]!=null? DateTime.parse(data["dateEnd"]) : null;
    status = data["status"];
    createdOn = data["createdOn"]!=null? DateTime.parse(data["createdOn"]) : null;
    guid = data["guid"];
  }

}