import 'dart:async';
import 'package:flutter/services.dart';

enum WifiState { error, success, already }

//https://pub.dev/packages/wifi#-readme-tab-
//https://github.com/once10301/wifi

class Wifi {
  static const MethodChannel _channel = const MethodChannel('plugins.kcc.com/wifi');

  static Future<String> get macAddress async {
    return await _channel.invokeMethod('mac');
  }

  static Future<String> get bssid async {
    return await _channel.invokeMethod('bssid');
  }

  static Future<String> get ssid async {
    return await _channel.invokeMethod('ssid');
  }

  static Future<int> get level async {
    return await _channel.invokeMethod('level');
  }
  static Future<int> get frequency async {
    return await _channel.invokeMethod('frequency');
  }

  static Future<String> get ip async {
    return await _channel.invokeMethod('ip');
  }

  static Future<List<WifiResult>> list(String key) async {
    final Map<String, dynamic> params = {
      'key': key,
    };
    var results = await _channel.invokeMethod('list', params);
    List<WifiResult> resultList = [];
    for (int i = 0; i < results.length; i++) {
      resultList.add(WifiResult(results[i]['bssid'], results[i]['ssid'], results[i]['level']));
    }
    return resultList;
  }

  static Future<List<Map<String, dynamic>>> listMap() async{
    List<Map<String, dynamic>> wifis = [];
    String connected = '';
    try{
      connected = await Wifi.bssid;
    }catch(e){}

    await Wifi.list('').then((List<WifiResult> result) {
      for (int i = 0; i < result.length; i++) {
        Map<String, dynamic> wifi = {
          'bssid': result[i].bssid,
          'ssid': result[i].ssid,
          'level': result[i].level,
          'frequency': 0,
          'condition': connected==result[i].bssid ? "connected" : "detected",
        };
        wifis.add(wifi);
      }
    });
    return wifis;

  }

  static Future<WifiState> connection(String ssid, String password) async {
    final Map<String, dynamic> params = {
      'ssid': ssid,
      'password': password,
    };
    int state = await _channel.invokeMethod('connection', params);
    switch (state) {
      case 0:
        return WifiState.error;
      case 1:
        return WifiState.success;
      case 2:
        return WifiState.already;
      default:
        return WifiState.error;
    }
  }
}

class WifiResult {
  String bssid;
  String ssid;
  int level;

  WifiResult(this.bssid, this.ssid, this.level);
}