import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/employee.dart';
import '/hrm/model/response/employee_search_response.dart';
import 'hrm_base_client.dart';
import 'token_service.dart';


class EmployeeService{

  Future<String> activeDefault(String organizationGUID, String facialGUID) async{
    TokenService tokenService = new TokenService( );
    String userAccessToken = await tokenService.faceLogin(facialGUID, false);

    Map<String, dynamic> data ={
      "organizationGUID": organizationGUID
    };

    String uri = "/v1/employee/active_default";
    String response = await postRequestWithAccessToken(uri, data, userAccessToken);

    return response;
  }

  Future<EmployeeSearchResponse> search( String userAccessToken, String name, int first, int max) async{

    if( name == null)
      name = '';

    String _uri = "/v1/employee/search?employeeName=${name}&first=${first}&max=${max}";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    EmployeeSearchResponse jobTitleListResponse = new EmployeeSearchResponse( );
    jobTitleListResponse.fromJson(response);

    return jobTitleListResponse;
  }

  Future<String> employeeRegister( Map<String, dynamic> employeeData, String currentFacialId ) async{

    String employeeGUID;
    TokenService tokenService = new TokenService( );
    String accessToken = await tokenService.faceLogin( currentFacialId, false );

    if( Constant.ENABLE_LOGS)
      print('Employee data: ${json.encode(employeeData)}');

    String uri = "/v1/employee/create";
    String jsonString = await postRequestWithAccessToken(uri, employeeData, accessToken);

    if( Constant.ENABLE_LOGS)
      print( jsonString );

    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      employeeGUID = registerResponse['data'];
    }

    return employeeGUID;
  }


  Future<Employee> retrieve(String accessToken, [String guid ='']) async{

    String _uri = "/v1/employee/retrieve?guid=${guid}";
    String response = await getRequestWithAccessToken(_uri, accessToken);
    if( Constant.ENABLE_LOGS)
      print(response);

    Employee employee= new Employee();
    if( ! await employee.fromJson(response))
      employee = null;

    return employee;
  }

  Future<bool> employeeUpdate( Map<String, dynamic> employeeData, String accessToken ) async{
    bool result = false;
    if( Constant.ENABLE_LOGS)
      print('Employee data: ${json.encode(employeeData)}');

    try{
      String uri = "/v1/employee/update";
      String jsonString = await putRequestWithAccessToken( uri, employeeData, accessToken );

      if( Constant.ENABLE_LOGS)
        print( jsonString );

      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];

    }catch(e){}
    return result;
  }

  Future<EmployeeSearchResponse> supervisor( String userAccessToken, String organizationGuid) async{

    if( organizationGuid == null)
      organizationGuid = '';

    String _uri = "/v1/employee/supervisor?organizationGuid=$organizationGuid";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    EmployeeSearchResponse jobTitleListResponse = new EmployeeSearchResponse( );
    jobTitleListResponse.fromJson(response);

    return jobTitleListResponse;
  }

  Future<bool> employeeTerminate( String employeeGuid, String accessToken ) async{
    bool result = false;
    if( Constant.ENABLE_LOGS)
      print('Employee guid: $employeeGuid');

    try{
      String uri = "/v1/employee/terminate/$employeeGuid";
      String jsonString = await deleteRequestWithAccessToken( uri, accessToken );

      if( Constant.ENABLE_LOGS)
        print( jsonString );

      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];

    }catch(e){}
    return result;
  }
}