import 'dart:convert';
import 'dart:io';
import '/hrm/config/constant.dart';
import '/hrm/model/request/face_request.dart';
import '/hrm/model/response/photos_list_by_face_response.dart';
import '/hrm/model/response/recognize_result.dart';
import 'hrm_base_client.dart';
import 'token_service.dart';


class FaceRecognizationService{

  Future<RecognizeResult> verify( String face, String facialGUID, bool onlyRemote, String organizationGUID ) async{
    RecognizeResult recognizeResult;
    int startTime = DateTime.now().millisecondsSinceEpoch;
    if (Platform.isAndroid && !onlyRemote){
        recognizeResult = await _verifyLocal(face, facialGUID);
      if( recognizeResult == null)
        recognizeResult = await _verifyRemote(face, facialGUID, organizationGUID);
    }else{
      recognizeResult = await _verifyRemote(face, facialGUID, organizationGUID);
    }

    int processingTime = DateTime.now().millisecondsSinceEpoch - startTime;
    if( Constant.ENABLE_LOGS )
      print('************************ processing=${processingTime}');

    return recognizeResult;
  }

  Future<RecognizeResult> _verifyRemote( String face, String facialGUID, String organizationGUID ) async{
    TokenService tokenService = new TokenService( );
    String accessToken = await tokenService.faceLogin( facialGUID, false );

    RecognizeResult _recognizeResult;
    Map<String, dynamic> data = {
      'flip': false,
      'preChcek': false,
      'demographic': false,
      'face': face
    };

    bool shareAttendance = await getSettingsAttendanceSharing( );
    String _faceRecognizeURI = shareAttendance? "/v1/recogser/recognition/verify_by_organization/$organizationGUID" : "/v1/recogser/recognition/verify";
    String jsonString = await postRequestWithAccessToken(_faceRecognizeURI, data, accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> recognizationResponse = jsonDecode(jsonString);
    bool success = recognizationResponse['success'];
    if( success == null){
      return _recognizeResult;
    }

    if( success ){
      Map<String, dynamic> dataResponse = recognizationResponse['data'];
      if( dataResponse['result']) {
        _recognizeResult = new RecognizeResult();
        _recognizeResult.fromJson( jsonString );
        }
      }

    return _recognizeResult;
  }

  @Deprecated('Performance issue')
  Future<RecognizeResult> _verifyLocal( String face, String facialGUID ) async{
    return null;
//    await init();
//    return await _recognitionService.recognize( face, null );
  }

  Future<RecognizeResult> recognize( String face, bool onlyRemote ) async{

    RecognizeResult recognizeResult;
    int startTime = DateTime.now().millisecondsSinceEpoch;
    if (Platform.isAndroid && !onlyRemote){
      recognizeResult = await _verifyLocal( face, null );
      if( recognizeResult == null)
        recognizeResult = await _recognizeRemote( face );
    }else{
      recognizeResult = await _recognizeRemote( face );
    }
    int processingTime = DateTime.now().millisecondsSinceEpoch - startTime;
    if( Constant.ENABLE_LOGS )
      print('************************ processing=${processingTime}');

    return recognizeResult;

  }

  Future<RecognizeResult> _recognizeRemote( String face ) async{

    TokenService tokenService = new TokenService();
    String accessToken = await tokenService.getDeviceToken(false);

    RecognizeResult _recognizeResult;
    Map<String, dynamic> data = {
      'flip': false,
      'preChcek': false,
      'demographic': false,
      'face': face
    };

    String _faceRecognizeURI = "/v1/recogser/recognition/recognize";
    String jsonString = await postRequestWithAccessToken(_faceRecognizeURI, data, accessToken);
    if( Constant.ENABLE_LOGS )
      print(jsonString);

    Map<String, dynamic> recognizationResponse = jsonDecode(jsonString);
    bool success = recognizationResponse['success'];
    if( success == null){
      return _recognizeResult;
    }

    if( success ){
      Map<String, dynamic> dataResponse = recognizationResponse['data'];
      if( dataResponse['result']) {
        _recognizeResult = new RecognizeResult();
        _recognizeResult.fromJson( jsonString );
      }
    }

    return _recognizeResult;
  }

  Future<RecognizeResult> recognizeByOrganization( String face, String organizationGUID ) async{

    TokenService tokenService = new TokenService();
    String accessToken = await tokenService.getDeviceToken(false);

    RecognizeResult _recognizeResult;
    Map<String, dynamic> data = {
      'flip': false,
      'preChcek': false,
      'demographic': false,
      'face': face
    };

    String _faceRecognizeURI = "/v1/recogser/recognition/verify_by_organization/${organizationGUID}";
    String jsonString = await postRequestWithAccessToken(_faceRecognizeURI, data, accessToken);
    if( Constant.ENABLE_LOGS )
      print(jsonString);

    Map<String, dynamic> recognizationResponse = jsonDecode(jsonString);
    bool success = recognizationResponse['success'];
    if( success == null){
      return _recognizeResult;
    }

    if( success ){
      Map<String, dynamic> dataResponse = recognizationResponse['data'];
      if( dataResponse['result']) {
        _recognizeResult = new RecognizeResult();
        _recognizeResult.fromJson( jsonString );
      }
    }

    return _recognizeResult;
  }

  Future<String> faceRegister(List<String> images,
      String name,
      String organizationGUID,
      String accessToken, String type) async{

    String faceGUID;
    List<FaceRequestItem> faces = [];
    FaceRequestItem faceRequestItem1 = new FaceRequestItem();
    faceRequestItem1.flip = false;
    faceRequestItem1.priority = 1;
    faceRequestItem1.preCheck = false;
    faceRequestItem1.face = images[1];
    faces.add(faceRequestItem1);

    if( images[2] != 'no') {
      FaceRequestItem faceRequestItem2 = new FaceRequestItem();
      faceRequestItem2.flip = false;
      faceRequestItem2.priority = 2;
      faceRequestItem2.preCheck = false;
      faceRequestItem2.face = images[2];
      faces.add(faceRequestItem2);
    }
    if( images[3] != 'no') {
      FaceRequestItem faceRequestItem3 = new FaceRequestItem();
      faceRequestItem3.flip = false;
      faceRequestItem3.priority = 2;
      faceRequestItem3.preCheck = false;
      faceRequestItem3.face = images[3];
      faces.add(faceRequestItem3);
    }
    FaceRequest faceRequest = new FaceRequest();
    faceRequest.name = name;
    faceRequest.organizationGUID = organizationGUID;
    faceRequest.profile = images[0];
    faceRequest.faces = faces;
    faceRequest.type = type;

    Map<String, dynamic> faceData = faceRequest.toMap();
    if( Constant.ENABLE_LOGS)
      print('Face data: ${json.encode(faceData)}');

    String uri = "/v1/recogser/recognition/register";
    String jsonString = await postRequestWithAccessToken(uri, faceData, accessToken);
    if( Constant.ENABLE_LOGS)
      print('Face response: ${jsonString}');

    try{
      if( jsonString != null){
        if( Constant.ENABLE_LOGS )
          print( jsonString );

        Map<String, dynamic> registerResponse = jsonDecode(jsonString);
        bool success = registerResponse['success'];
        // token failed condition
        if( success == null){
          return null;
        }

        if( success ){
          Map<String, dynamic> dataResponse = registerResponse['data'];
          if( dataResponse['result']) {
            faceGUID = dataResponse['faceGUID'];
          }
        }
      }
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }
    return faceGUID;
  }

  Future<PhotosListByFace> listPhotosByFace(String accessToken, String facialGUID) async{

    String _uri = "/v1/recogser/recognition/list_photos/${facialGUID}";
    String response = await getRequestWithAccessToken(_uri, accessToken);
    if( Constant.ENABLE_LOGS)
      print(response);

    PhotosListByFace listByFace= new PhotosListByFace();
    if( !listByFace.fromJson(response))
      listByFace = null;

    return listByFace;
  }

  Future<bool> updateProfile(String accessToken, String facialGUID, String image) async{
    bool result = false;

    try{
      Map<String, dynamic> faceData = {
        'flip': false,
        'priority': 1,
        'preChcek': false,
        'face': image
      };
      String uri = "/v1/recogser/recognition/upload_profile?guid=${facialGUID}";
      String jsonString = await postRequestWithAccessToken(uri, faceData, accessToken);
      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];
    }catch(e){}

    return result;
  }

  Future<bool> updatePhotos(String accessToken, String facialGUID, List<String> _images)async{
    bool result = false;

    try{
      List<Map<String, dynamic>> images = [];

      FaceRequestItem faceRequestItem1 = new FaceRequestItem();
      faceRequestItem1.flip = false;
      faceRequestItem1.priority = 0;
      faceRequestItem1.preCheck = false;
      faceRequestItem1.face = _images[1]=='no'?null : _images[1];
      images.add(faceRequestItem1.toMap());


      FaceRequestItem faceRequestItem2 = new FaceRequestItem();
      faceRequestItem2.flip = false;
      faceRequestItem2.priority = 1;
      faceRequestItem2.preCheck = false;
      faceRequestItem2.face = _images[2]=='no'?null : _images[2];
      images.add(faceRequestItem2.toMap());


      FaceRequestItem faceRequestItem3 = new FaceRequestItem();
      faceRequestItem3.flip = false;
      faceRequestItem3.priority = 2;
      faceRequestItem3.preCheck = false;
      faceRequestItem3.face = _images[3]=='no'?null : _images[3];
      images.add(faceRequestItem3.toMap());


      Map<String, dynamic> faceData = {
        'faceGUID': facialGUID,
        'faces': images
      };

      String uri = "/v1/recogser/recognition/upload_photos";
      String jsonString = await postRequestWithAccessToken(uri, faceData, accessToken);
      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];

    }catch(e){}

    return result;
  }

  Future<String> faceDetect( String image ) async{

    TokenService tokenService = new TokenService();
    String accessToken = await tokenService.getDeviceToken(false);

    String detectedFace;
    Map<String, dynamic> data = {
      'preChcek': true,
      'face': image
    };

    String _faceRecognizeURI = "/v1/recogser/recognition/detect";
    String jsonString = await postRequestWithAccessToken(_faceRecognizeURI, data, accessToken);
    if( Constant.ENABLE_LOGS )
      print(jsonString);

    Map<String, dynamic> recognizationResponse = jsonDecode(jsonString);
    bool success = recognizationResponse['success'];
    if( success == null){
      return detectedFace;
    }

    if( success ){
      detectedFace = recognizationResponse['data'];
    }

    return detectedFace;
  }
}