import '/hrm/config/constant.dart';
import '/hrm/database/dao/notification_dao.dart';
import '/hrm/database/entity/notification.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:shared_preferences/shared_preferences.dart';



final FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin = FlutterLocalNotificationsPlugin();

class FirebaseMessage{

  Future<void> initMessaging() async{

    await Firebase.initializeApp();
    final FirebaseMessaging _firebaseMessaging = FirebaseMessaging.instance;
    NotificationSettings settings = await _firebaseMessaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: true,
      sound: true,
    );

    FirebaseMessaging.onMessage.listen((RemoteMessage message) {
      save( message );
      if( Constant.ENABLE_LOGS)
        print('Got a message whilst in the foreground!');
      if( Constant.ENABLE_LOGS)
        print('Message data: ${message.data}');

      if (message.notification != null) {
        if( Constant.ENABLE_LOGS)
          print('Message also contained a notification: ${message.notification.title}');
      }
    });

    Future<void> _firebaseMessagingBackgroundHandler(RemoteMessage message) async {
      save( message );
      if( Constant.ENABLE_LOGS)
        print('Got a message whilst in the background!');
      if( Constant.ENABLE_LOGS)
        print('Message data: ${message.data}');

      if (message.notification != null) {
        if( Constant.ENABLE_LOGS)
          print('Message also contained a notification: ${message.notification.title}');
      }
    }
    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingBackgroundHandler);

//     _firebaseMessaging.configure(
//       onMessage: save,
// //          (Map<String, dynamic> message) async {
// //        print("onMessage: $message");
// //      },
//       onBackgroundMessage: save,
// //      onBackgroundMessage: myBackgroundMessageHandler,
//       onLaunch: save,
// //          (Map<String, dynamic> message) async {
// //        print("onLaunch: $message");
// //      },
//       onResume: save,
// //          (Map<String, dynamic> message) async {
// //        print("onResume: $message");
// //      },
//     );

    // _firebaseMessaging.requestNotificationPermissions(
    //     const IosNotificationSettings(
    //         sound: true, badge: true, alert: true, provisional: true));
    // _firebaseMessaging.onIosSettingsRegistered
    //     .listen((IosNotificationSettings settings) {
    //   if( Constant.ENABLE_LOGS)
    //     print("Settings registered: $settings");
    // });

    _firebaseMessaging.getToken().then((String token)async {
      if( token != null) {
        final SharedPreferences sharedPrefs = await SharedPreferences
            .getInstance();
        sharedPrefs.setString(Constant.FIREBASE_TOKEN_KEY, token);
      }
      if( Constant.ENABLE_LOGS)
        print('Push Messaging token: $token');

    });
  }

  Future<void> initLocalNotification() async{
    try {
      var initializationSettingsAndroid = AndroidInitializationSettings(
          'launch_background');
      var initializationSettingsIOS = IOSInitializationSettings(
          requestAlertPermission: true,
          requestBadgePermission: true,
          requestSoundPermission: true);
      var initializationSettings = InitializationSettings(
          android:initializationSettingsAndroid,
          iOS: initializationSettingsIOS
      );
      await flutterLocalNotificationsPlugin.initialize(initializationSettings,
          onSelectNotification: selectNotification);
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print('$e');
    }
  }

  Future selectNotification(String payload) async {
//    await Navigator.push(
//      context,
//      MaterialPageRoute(builder: (context) => SecondScreen(payload)),
//    );
    if( payload == Constant.NOTIFICATION_PAYLOAD_DEFAULT)
      flutterLocalNotificationsPlugin.cancel( 0 );
  }
}

Future<String> getFirebaseToken( ) async{
  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  return sharedPrefs.getString(Constant.FIREBASE_TOKEN_KEY);
}

Future<dynamic> myBackgroundMessageHandler(Map<String, dynamic> message) {
  if( Constant.ENABLE_LOGS)
    print('onBackground: $message');

  if (message.containsKey('data')) {
    // Handle data message
    final dynamic data = message['data'];
    if( Constant.ENABLE_LOGS)
      print('onBackground: $data');
  }

  if (message.containsKey('notification')) {
    // Handle notification message
    final dynamic notification = message['notification'];
    if( Constant.ENABLE_LOGS)
      print('onBackground: $notification');
  }

  // Or do other work.
}


Future<void> save( RemoteMessage remoteMessage ) async{
  Map<String, dynamic> message = remoteMessage.data;

  if( Constant.ENABLE_LOGS)
    print("onMessage: $message");
  try{
    NotificationDao notificationDao = new NotificationDao();
    await notificationDao.open();
    //onMessage: {from: 539663693477, targetId: 1, notification: {title: notificationTitle, e: 1, body: notificationBody}, campaignTypeId: 3, scheduleAt: 2020-07-24T23:50:48.168+0630, title: campaignTitle, campaignId: 23, collapse_key: com.kcc.ml.facialattendance, appId: HRM, description: campaignDescription}
    Notification notification = new Notification();

    if (message.containsKey('data')) {
      notification = await notification.fromMessage( message['data'].cast<String, dynamic>());
    }else
      notification = await notification.fromMessage(message);

    await notificationDao.insert(notification);
    await notificationDao.close();
    // if (remoteMessage.notification != null)
    //   await _showNotification( remoteMessage );
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }
}

// Future<void> save(Map<String, dynamic> message) async{
//   if( Constant.ENABLE_LOGS)
//     print("onMessage: $message");
//   try{
//     NotificationDao notificationDao = new NotificationDao();
//     await notificationDao.open();
//     //onMessage: {from: 539663693477, targetId: 1, notification: {title: notificationTitle, e: 1, body: notificationBody}, campaignTypeId: 3, scheduleAt: 2020-07-24T23:50:48.168+0630, title: campaignTitle, campaignId: 23, collapse_key: com.kcc.ml.facialattendance, appId: HRM, description: campaignDescription}
//     Notification notification = new Notification();
//
//     if (message.containsKey('data')) {
//       notification = await notification.fromMessage( message['data'].cast<String, dynamic>());
//     }else
//       notification = await notification.fromMessage(message);
//
//     await notificationDao.insert(notification);
//     await notificationDao.close();
//     await _showNotification(message);
//   }catch(e){
//     if( Constant.ENABLE_LOGS)
//       print(e);
//   }
// }

Future<void> _showNotification( RemoteMessage remoteMessage ) async {
  var androidPlatformChannelSpecifics = AndroidNotificationDetails(
      'ggt', 'Notification', channelDescription:'Background notification',
      importance: Importance.high, priority: Priority.high, ticker: 'ticker');
  var iOSPlatformChannelSpecifics = IOSNotificationDetails();
  var platformChannelSpecifics = NotificationDetails(
      android:androidPlatformChannelSpecifics, iOS: iOSPlatformChannelSpecifics);
  await flutterLocalNotificationsPlugin.show(
      0,
      remoteMessage.notification.title,
      remoteMessage.notification.body,
      platformChannelSpecifics,
      payload: Constant.NOTIFICATION_PAYLOAD_DEFAULT
  );
}

// Future<void> _showNotification(Map<String, dynamic> message) async {
//   var androidPlatformChannelSpecifics = AndroidNotificationDetails(
//       'hrm', 'Notification', 'Background notification',
//       importance: Importance.Max, priority: Priority.High, ticker: 'ticker');
//   var iOSPlatformChannelSpecifics = IOSNotificationDetails();
//   var platformChannelSpecifics = NotificationDetails(
//       androidPlatformChannelSpecifics, iOSPlatformChannelSpecifics);
//   await flutterLocalNotificationsPlugin.show( 0, message['notification']['title'], message['notification']['body'], platformChannelSpecifics, payload: Constant.NOTIFICATION_PAYLOAD_DEFAULT);
//
// }