import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/model/request/geofencing_add_request.dart';
import '/hrm/model/request/geofencing_search_request.dart';
import '/hrm/model/request/geofencing_update_request.dart';
import '/hrm/model/response/geofencing_detail_response.dart';
import '/hrm/model/response/geofencing_search_response.dart';
import 'hrm_base_client.dart';


class GeofencingService{

  Future<String> add(GeofencingAddRequest request, String userAccessToken) async{
    String guid;

    String _uri = "/v1/geofencing/add";
    String response = await postRequestWithAccessToken(_uri, request.toMap(), userAccessToken);
    if( Constant.ENABLE_LOGS)
      print(response);

    Map<String, dynamic> registerResponse = jsonDecode(response);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      guid = registerResponse['data'];
    }
    return guid;
  }

  Future<String> update(GeofencingUpdateRequest request, String userAccessToken) async{
    String guid;

    String _uri = "/v1/geofencing/update";
    String response = await postRequestWithAccessToken(_uri, request.toMap(), userAccessToken);
    if( Constant.ENABLE_LOGS)
      print(response);

    Map<String, dynamic> registerResponse = jsonDecode(response);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      guid = registerResponse['data'];
    }
    return guid;
  }

  Future<GeofencingSearchResponse> search( GeofencingSearchRequest request, String accessToken) async{
    GeofencingSearchResponse searchResponse;

    String uri = "/v1/geofencing/search";
    String jsonString = await postRequestWithAccessToken(uri, request.toMap(), accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    searchResponse = new GeofencingSearchResponse();
    if(!searchResponse.fromJson(jsonString))
      searchResponse = null;

    return searchResponse;
  }

  Future<GeofencingDetailResponse> detail( String userAccessToken, String guid) async{

    String _uri = "/v1/geofencing/detail/${guid}";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);

    if( Constant.ENABLE_LOGS)
      print(response);

    GeofencingDetailResponse detailResponse = new GeofencingDetailResponse();
    if( !detailResponse.fromJson(response) )
      detailResponse = null;

    return detailResponse;
  }

  Future<bool> changeStatus( String status, String guid, String accessToken) async{
    bool result = false;

    Map<String, dynamic> data={
      'guid': guid,
      'status': status
    };

    String uri = "/v1/geofencing/status";
    String jsonString = await postRequestWithAccessToken(uri, data, accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      result = registerResponse['data'];
    }

    return result;
  }

  Future<String> addGeofencee( List<String> employeeGUIDs, String geofencingGUID, String accessToken) async{
    String result;

    Map<String, dynamic> data={
      'geofencingGUID': geofencingGUID,
      'employeeGUIDs': employeeGUIDs
    };

    String uri = "/v1/geofencing/addGeofencee";
    String jsonString = await postRequestWithAccessToken(uri, data, accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      result = registerResponse['data'];
    }

    return result;
  }

  Future<bool> removeGeofencee( List<String> guids, String accessToken) async{
    bool result = false;

    Map<String, dynamic> data={
      'guids': guids,
    };

    String uri = "/v1/geofencing/removeGeofencee";
    String jsonString = await postRequestWithAccessToken(uri, data, accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      result = registerResponse['data'];
    }

    return result;
  }
}