import 'dart:convert';
import '/hrm/config/constant.dart';
import 'package:http/http.dart' as http;
import 'package:shared_preferences/shared_preferences.dart';
import 'dart:io';


HttpClient getHttpClient() {
  bool trustSelfSigned = true;
  HttpClient httpClient = new HttpClient()
    ..connectionTimeout = const Duration(seconds: 10)
    ..badCertificateCallback = ((X509Certificate cert, String host, int port) => trustSelfSigned);

  return httpClient;
}

Future<String> getRequest( String uri  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var response = await http.get('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {"Content-Type": "application/json;charset=UTF-8","Accept": "application/json"}
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> getRequestWithAccessToken( String uri, String accessToken  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.getUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   var response = await http.get('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       }
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }
  return reply;
}


Future<String> postRequest( String uri, Map jsonMap ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.postUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    // request.headers.set('Authorization', 'Bearer ${accessToken}');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();
    if( Constant.ENABLE_LOGS)
      print(response.statusCode);
    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   //encode Map to JSON
  //   var body = json.encode(jsonMap);
  //
  //   var response = await http.post('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //       "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json"
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> postRequestWithAccessToken( String uri, Map jsonMap, String accessToken ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.postUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();
    if( Constant.ENABLE_LOGS)
      print(response.statusCode);
    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }

  // try{
  //   //encode Map to JSON
  //   var body = json.encode(jsonMap);
  //   var response = await http.post('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> putRequestWithAccessToken( String uri, Map jsonMap, String accessToken ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.putUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }
  // try{
  //   //encode Map to JSON
  //   var body = json.encode(jsonMap);
  //   var response = await http.put('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       },
  //       body: body
  //   );
  //
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  //
  //   //return response;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }

  return reply;
}

Future<String> deleteRequestWithAccessToken( String uri, String accessToken  ) async{

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  String reply;

  try {
    HttpClient httpClient = getHttpClient();
    HttpClientRequest request = await httpClient.deleteUrl( Uri.parse('${getBaseURL(sharedPrefs)}${uri}') );
    request.headers.set('Content-Type', 'application/json;charset=UTF-8');
    request.headers.set('Accept', 'application/json');
    request.headers.set('Authorization', 'Bearer ${accessToken}');
    // request.add(utf8.encode(json.encode(jsonMap)));
    HttpClientResponse response = await request.close();

    if (response.statusCode == 200 || response.statusCode == 201)
      reply = await response.transform(utf8.decoder).join();

    httpClient.close();
  }catch(e){
    if( Constant.ENABLE_LOGS)
      print(e);
  }
  // try{
  //   var response = await http.delete('${getBaseURL(sharedPrefs)}${uri}',
  //       headers: {
  //         "Content-Type": "application/json;charset=UTF-8",
  //         "Accept": "application/json",
  //         "Authorization":'Bearer ${accessToken}'
  //       }
  //   );
  //   if( response.statusCode == 200 || response.statusCode==201)
  //     reply = response.body;
  // }catch(e){
  //   if( Constant.ENABLE_LOGS)
  //     print(e);
  // }
  return reply;
}

//https://stackoverflow.com/questions/44841729/how-to-upload-image-in-flutter
Future<String> uploadImageRequest( file, uri, accessToken) async {

  final SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
  Map<String, String> headers = {
    "Accept": "application/json",
    "Content-Type": "multipart/form-data; boundary=---acaca---",
    "Authorization": "Bearer " + accessToken
  };
  var request = http.MultipartRequest('POST', Uri.parse('${getBaseURL(sharedPrefs)}${uri}'));
  // add file to multipart
  request.files.add(await http.MultipartFile.fromPath( 'file', file.absolute.path, filename: 'faceupload.jpg',  ));
  //add headers
  request.headers.addAll(headers);
  var response = await request.send();

  if( Constant.ENABLE_LOGS)
    print('Response code=${response.statusCode}');

  response.stream.transform(utf8.decoder).listen((value) {
    if( Constant.ENABLE_LOGS)
      print(value);
    return value;
  });
  return response.reasonPhrase;


//  Dio dio = new Dio();
//  dio.options.baseUrl = Constant.BASE_URL;
//  dio.interceptors.add(LogInterceptor());
//  FormData formData = FormData.fromMap({
//    "file": await MultipartFile.fromFile( file.absolute.path,filename: basename(file.absolute.path) ),
//  });
//
//  Response response = await dio.post(
//      uri,
//      data: formData,
//      options: Options(
//        method: 'POST',
//        headers: headers,
//        responseType: ResponseType.json
//      ));
//  print('Response code=${response.statusCode}');
//  String val = response.toString();
//
//  return val;
}