import 'dart:convert';
import 'dart:io';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/kiosk_data.dart';
import '/hrm/model/response/kiosk_list_response.dart';
import '/hrm/service/token_service.dart';
import 'hrm_base_client.dart';


class KioskService{

  Future<KioskData> activate( String key, String firebaseToken ) async{
    KioskData kioskData;

    try{
      Map<String, dynamic> data = {
        "key": key,
        "firebaseToken": firebaseToken,
        "platform": Platform.isAndroid? Constant.PLATFORM_ANDROID : Constant.PLATFORM_IOS
      };

      String uri = "/v1/kiosk/register";
      String jsonString = await postRequest( uri, data );
      if( Constant.ENABLE_LOGS)
        print(jsonString);

      kioskData = new KioskData();
      if( !await kioskData.fromJson(jsonString, true))
        kioskData = null;

    }catch(e){}

    return kioskData;
  }

  Future<bool> exit(String guid, String facialGUID) async{
    try{
      TokenService tokenService = new TokenService();
      String token = await tokenService.faceLogin( facialGUID, false );

      String uri = "/v1/kiosk/exit/$guid";
      String jsonString = await getRequestWithAccessToken(uri, token);
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      if( resultData["success"]){
        return true;
      }else{
        return false;
      }
    }catch(e){}

    return false;
  }


  Future<KioskListResponse> list(String accessToken, int first, int max)async{
    KioskListResponse response;

    try{
      String uri = "/v1/kiosk/list?first=$first&max=$max";
      String jsonString = await getRequestWithAccessToken(uri, accessToken);
      if( Constant.ENABLE_LOGS)
        print(jsonString);

      response = KioskListResponse();
      response.fromJson(jsonString);

    }catch(e){}

    return response;
  }

  Future<bool> enable(String guid, String accessToken) async{
    try{
      String uri = "/v1/kiosk/enable/$guid";
      String jsonString = await getRequestWithAccessToken( uri, accessToken );
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      if( resultData["success"]){
        return true;
      }else{
        return false;
      }
    }catch(e){}

    return false;
  }

  Future<bool> disable(String guid, String accessToken) async{
    try{
      String uri = "/v1/kiosk/disable/$guid";
      String jsonString = await getRequestWithAccessToken( uri, accessToken );
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      if( resultData["success"]){
        return true;
      }else{
        return false;
      }
    }catch(e){}

    return false;
  }

  Future<String> generate( String accessToken ) async{
    try{
      String uri = "/v1/kiosk/generate";
      String jsonString = await getRequestWithAccessToken( uri, accessToken );
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      if( resultData["success"]){
        return resultData["data"];
      }else{
        return null;
      }
    }catch(e){}

    return null;
  }
}