import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/model/request/leave_apply_request.dart';
import '/hrm/model/request/leave_search_request.dart';
import '/hrm/model/response/leave_apply_response.dart';
import '/hrm/model/response/leave_balance_response.dart';
import '/hrm/model/response/leave_detail_response.dart';
import '/hrm/model/response/leave_search_response.dart';
import '/hrm/model/response/leave_type_response.dart' as ltype;
import 'hrm_base_client.dart';


class LeaveService{

  Future<LeaveSearchResponse> search(String accessToken, LeaveSearchRequest request) async{
    LeaveSearchResponse _attendanceResponse;

    String uri = "/v1/leave/list";
    String jsonString = await postRequestWithAccessToken(uri, request.toMap(), accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    _attendanceResponse = new LeaveSearchResponse();
    _attendanceResponse.fromJson(jsonString);
    return _attendanceResponse;

  }

  Future<String> changeStatus(String accessToken, String guid, String status, String comment) async{
    String result = 'Unknown error';

    Map<String, dynamic> data = {
      "guid": guid,
      "status": status,
      "comment": comment
    };
    String uri = "/v1/leave/change";
    String jsonString = await postRequestWithAccessToken( uri, data, accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    if( resultData["success"] ){
      result = resultData["message"];
    }

    return result;
  }

  Future<LeaveApplyResponse> apply(String accessToken, LeaveApplyRequest request) async{
    LeaveApplyResponse leaveApplyResponse;

    String uri = "/v1/leave/apply";
    String jsonString = await postRequestWithAccessToken(uri, request.toMap(), accessToken);
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    leaveApplyResponse = new LeaveApplyResponse();
    leaveApplyResponse.fromJson(jsonString);
    return leaveApplyResponse;
  }


  Future<LeaveDetailResponse> detail( String userAccessToken, String guid ) async{

    String _uri = "/v1/leave/detail/${guid}";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    LeaveDetailResponse leaveDetailResponse = new LeaveDetailResponse();
    leaveDetailResponse.fromJson(response);

    return leaveDetailResponse;
  }

  Future<LeaveBalanceResponse> balance(String accessToken, String employeeGuid) async{

    String _uri = "/v1/leave/balance/${employeeGuid}";
    String response = await getRequestWithAccessToken(_uri, accessToken);
    LeaveBalanceResponse leaveBalanceResponse = new LeaveBalanceResponse();
    leaveBalanceResponse.fromJson(response);

    return leaveBalanceResponse;
  }


  Future<ltype.LeaveTypeResponse> leaveType( String userAccessToken ) async{

    String _uri = "/v1/leaveType/type";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    if(Constant.ENABLE_LOGS)
      print(response);

    ltype.LeaveTypeResponse leaveTypeResponse = new ltype.LeaveTypeResponse();
    leaveTypeResponse.fromJson(response);
    return leaveTypeResponse;
  }

  Future<bool> leaveTypeUpdate(ltype.LeaveTypeItem item, String accessToken) async{

    Map<String, dynamic> data = {
      "name": item.name,
      "supervisorLeaveAssign": item.supervisorLeaveAssign,
      "employeeCanApply": item.employeeCanApply,
      "applyBeyondCurrent": item.applyBeyondCurrent,
      "guid": item.guid,
      "defaultPerYear": item.defaultPerYear,
      "departmentGuid": item.organizationGuid
    };

    String uri = "/v1/leaveType/update";
    String jsonString = await putRequestWithAccessToken( uri, data, accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    bool success = resultData["success"];
    if( success == null)
      return false;


    return success;
  }


  Future<bool> leaveTypeCreate(ltype.LeaveTypeItem item, String accessToken) async{

    Map<String, dynamic> data = {
      "name": item.name,
      "supervisorLeaveAssign": true,
      "employeeCanApply": true,
      "applyBeyondCurrent": false,
      "defaultPerYear": item.defaultPerYear,
      "departmentGuid": item.organizationGuid,
      "leaveAccrue": false,
      "carriedForward":false,
      "carriedForwardPercentage":0,
      "carriedForwardLeaveAvailability":365,
      "propotionateOnJoinedDate":false,
      "maxCarriedForwardAmount":0,
      "employeeLeavePeriod":false
    };

    String uri = "/v1/leaveType/create";
    String jsonString = await postRequestWithAccessToken( uri, data, accessToken );
    if( Constant.ENABLE_LOGS)
      print(jsonString);

    Map<String, dynamic> resultData = jsonDecode(jsonString);
    bool success = resultData["success"];
    if( success == null)
      return false;


    return success;
  }
}