import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/database/dao/notification_dao.dart';
import '/hrm/database/entity/notification.dart';
import '/hrm/model/response/notification_fetch_response.dart';
import 'hrm_base_client.dart';



class NotificationService{

  Future<NotificationFetchResponse> fetch(String userAccessToken, String deviceGUID,
      int lastCampaignId, int first, int max) async{
    NotificationFetchResponse fetchResponse;

    Map<String, dynamic> data = {
      "deviceGUID": deviceGUID,
      "lastCampaignId": lastCampaignId,
      "first": first,
      "max": max
    };
    if( Constant.ENABLE_LOGS)
      print(json.encode(data));

    String uri = "/v1/messaging/fetch";
    String jsonString = await postRequestWithAccessToken(uri, data, userAccessToken);

    if( Constant.ENABLE_LOGS)
      print(jsonString);

    fetchResponse = new NotificationFetchResponse();
    if( !fetchResponse.fromJson(jsonString)){
      fetchResponse = null;
    }

    return fetchResponse;
  }

  Future<bool> fetchAndUpdate( String userAccessToken, String deviceGUID )async{
    bool updated = false;
    bool hasNext = true;
    NotificationDao notificationDao = new NotificationDao();
    await notificationDao.open();

    try{
      while( hasNext ){
        int lastId = await getCampaignLastId( );
        NotificationFetchResponse fetchResponse = await fetch(userAccessToken, deviceGUID, lastId, 0, 20);
        if( fetchResponse != null ){
          hasNext = fetchResponse.hasNext;
          for( int i=0; i < fetchResponse.notifications.length; i++){
            Notification notification = fetchResponse.notifications[i];
            await notificationDao.insert(notification);
            lastId = notification.campaignId;
          }
          if( !updated ){
            updated = fetchResponse.notifications.isNotEmpty;
          }
        }else{
          hasNext = false;
        }

        await setCampaignLastId(lastId);
      }
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    await notificationDao.close();

    return updated;
  }

}