import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/model/response/organizations_by_face.dart';
import '/hrm/service/device_service.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'firebase_service.dart';
import 'hrm_base_client.dart';
import 'token_service.dart';
import 'user_service.dart';



class OrganizationService{

  Future<OrganizationsByFace> getListByFace( String userAccessToken, String facialGUID ) async{

    String _uri = '/v1/recogser/organization/list_by_face/${facialGUID}';
    String orgList = await getRequestWithAccessToken(_uri, userAccessToken);
    OrganizationsByFace organizationsByFace = new OrganizationsByFace();
    organizationsByFace.fromJson(orgList);

    return organizationsByFace;
  }

  Future<User> joinByFace(String organizationGUID, String facialGUID) async{
    TokenService tokenService = new TokenService( );
    String userAccessToken = await tokenService.faceLogin(facialGUID, false);

    Map<String, dynamic> data ={
      "organizationGUID": organizationGUID,
      "faceGUID": facialGUID
    };
    String uri = "/v1/recogser/organization/join_by_face";
    String response = await postRequestWithAccessToken(uri, data, userAccessToken);

    UserService userService = new UserService( );
    User user = await userService.getUserInformation(userAccessToken, null);
    AuthorityResponse authorityResponse = await userService.authority(userAccessToken);


    String firebaseToken = await getFirebaseToken();
    if( firebaseToken != null ) {
      DeviceService deviceService = new DeviceService();
      String deviceGUID = await deviceService.register(
          userAccessToken, user.guid, firebaseToken);

      if( deviceGUID != null ){
        SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
        sharedPrefs.setString(Constant.DEVICE_GUID, deviceGUID);
      }

    }
    return user;
  }

  Future<String> register(
      String name, String plan, String phone, String email,
      String address, String contact, int country ) async{

    String organizationGUID;
    Map<String, dynamic> orgData = {
      "name":name,
      "plan": plan,
      "phone": phone,
      "email": email,
      "address": address,
      "contact": contact,
      "country": country
    };

    if( Constant.ENABLE_LOGS)
      print('Org data: ${json.encode(orgData)}');

    String uri = "/v1/recogser/organization/register";
    String jsonString = await postRequest( uri, orgData );
    if( Constant.ENABLE_LOGS)
      print( jsonString );
    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      organizationGUID = registerResponse['data'];
    }

    return organizationGUID;
  }

  Future<bool> validate( String otp ) async{

    Map<String, dynamic> orgData = {
      "otp":otp
    };

    if( Constant.ENABLE_LOGS)
      print('OTP data: ${json.encode(orgData)}');

    String uri = "/v1/recogser/organization/validate_otp";
    String jsonString = await postRequest( uri, orgData );
    if( Constant.ENABLE_LOGS)
      print( jsonString );
    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return false;
    }
    return success;
  }

  Future<String> resend( String organizationGuid ) async{

    String otp;
    Map<String, dynamic> orgData = {
      "organizationGUID":organizationGuid
    };

    if( Constant.ENABLE_LOGS)
      print('org data: ${json.encode(orgData)}');

    String uri = "/v1/recogser/organization/resend_otp";
    String jsonString = await postRequest( uri, orgData );
    if( Constant.ENABLE_LOGS)
      print( jsonString );
    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }
    if( success ){
      otp = registerResponse['data'];
    }

    return otp;
  }

  Future<OrganizationDetailResponse> detail( String userAccessToken, String guid ) async{

    String _uri = '/v1/recogser/organization/detail/${guid}';
    String orgDetail = await getRequestWithAccessToken(_uri, userAccessToken);
    OrganizationDetailResponse organizationDetailResponse = new OrganizationDetailResponse();
    organizationDetailResponse.fromJson( orgDetail, true );
    return organizationDetailResponse;

  }

  Future<bool> profileUpdate( String image, String organizationGuid, String accessToken ) async{

    Map<String, dynamic> orgData = {
      "organizationGUID":organizationGuid,
      "logo":image
    };

    if( Constant.ENABLE_LOGS)
      print('Image data: ${json.encode(orgData)}');

    String uri = "/v1/recogser/organization/upload_logo";
    String jsonString = await postRequestWithAccessToken( uri, orgData, accessToken );
    if( Constant.ENABLE_LOGS)
      print( jsonString );
    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return false;
    }
    return success;
  }


  Future<bool> update(OrganizationDetailResponse detail, String organizationGuid,
      String accessToken ) async{

    Map<String, dynamic> orgData = {
      "guid":organizationGuid,
      "name":detail.name,
      "plan":detail.plan,
      "phone":detail.phone,
      "email":detail.email,
      "address":detail.address,
      "contact":detail.contact,
      "country":detail.country
    };

    if( Constant.ENABLE_LOGS)
      print('Update data: ${json.encode(orgData)}');

    String uri = "/v1/recogser/organization/update";
    String jsonString = await postRequestWithAccessToken( uri, orgData, accessToken );
    if( Constant.ENABLE_LOGS)
      print( jsonString );

    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return false;
    }
    return success;
  }
}