import '/hrm/config/constant.dart';
import '/hrm/model/response/token.dart';
import '/hrm/service/hrm_base_client.dart';
import 'package:shared_preferences/shared_preferences.dart';


class TokenService{

  Future<String> getDeviceToken( bool retry ) async{
    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    if( sharedPrefs.getInt(Constant.CURRENT_DEVICE_TOKEN_EXPIRED) != null && !retry){
      if(sharedPrefs.getInt(Constant.CURRENT_DEVICE_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
        accessToken = sharedPrefs.getString(Constant.CURRENT_DEVICE_TOKEN_VALUE);
        if( accessToken != null)
          return accessToken;
      }
    }

    if( sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED) != null ){
      if(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
        accessToken = sharedPrefs.getString(Constant.CURRENT_USER_TOKEN_VALUE);
        if( accessToken != null)
          return accessToken;
      }
    }

    String _deviceTokenURI = "/v1/recogser/device/";
    String response = await getRequest(_deviceTokenURI);
    Token token = new Token();
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(Constant.CURRENT_DEVICE_TOKEN_VALUE, accessToken);
      sharedPrefs.setInt(Constant.CURRENT_DEVICE_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));
    }

    if(Constant.ENABLE_LOGS)
      print('************************Device token value=${accessToken}');
    if(Constant.ENABLE_LOGS)
      print('************Device token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_DEVICE_TOKEN_EXPIRED))}');

    return accessToken;
  }


  Future<String> login(String username, String password, bool retry) async{
    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    if( sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED) != null && !retry){
      if(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
        accessToken = sharedPrefs.getString(Constant.CURRENT_USER_TOKEN_VALUE);
        if( accessToken != null)
          return accessToken;
      }
    }

    Map<String, dynamic> data = {
      'username': username,
      'password': password
    };

    String _loginURI = "/v1/recogser/login/";
    String response = await postRequest(_loginURI, data);
    Token token = new Token();
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(Constant.CURRENT_USER_TOKEN_VALUE, accessToken);
      sharedPrefs.setInt(Constant.CURRENT_USER_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));
    }

    if(Constant.ENABLE_LOGS)
      print('************************User token value=${accessToken}');
    if(Constant.ENABLE_LOGS)
      print('************User token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED))}');

    return accessToken;
  }

  Future<String> faceLogin(String guid, bool retry) async{
    String accessToken;
    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    if( sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED) != null && !retry){
      if(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED)> DateTime.now().millisecondsSinceEpoch){
        accessToken = sharedPrefs.getString(Constant.CURRENT_USER_TOKEN_VALUE);
        if( accessToken != null)
          return accessToken;
      }
    }

    Map<String, dynamic> data = {
      'guid': guid
    };

    String _faceLoginURI = "/v1/recogser/login/face";
    String response = await postRequest(_faceLoginURI, data);

    Token token = new Token();
    if(token.fromJson(response)){
      accessToken = token.accessToken;
      sharedPrefs.setString(Constant.CURRENT_USER_TOKEN_VALUE, accessToken);
      sharedPrefs.setInt(Constant.CURRENT_USER_TOKEN_EXPIRED, DateTime.now().millisecondsSinceEpoch + (token.expiresIn * 1000));
    }

    if(Constant.ENABLE_LOGS)
      print('************************Face token value=${accessToken}');
    if(Constant.ENABLE_LOGS)
      print('************Face token expired=${DateTime.fromMillisecondsSinceEpoch(sharedPrefs.getInt(Constant.CURRENT_USER_TOKEN_EXPIRED))}');

    return accessToken;
  }

  Future<String> test() async{
    String accessToken;
    String _faceLoginURI = "/v1/recogser/user/info/49e857b2-82c3-4260-829d-0d05b113f1fa";
    accessToken = await getRequestWithAccessToken(_faceLoginURI, '36b2c38f-5b6e-4cb0-b9ac-34dfcaed3482');
    return accessToken;
  }

  Future<String> postTest() async{
    String accessToken;
    String _faceLoginURI = "/v1/recogser/organization/list_faces?first=0&max=20";
    Map<String, dynamic> data = {
      'organizationGUID': "d9b3ea31-d666-4b3c-9576-c93bd520c799"
    };
    accessToken = await postRequestWithAccessToken(_faceLoginURI, data, '36b2c38f-5b6e-4cb0-b9ac-34dfcaed3482');
    return accessToken;
  }
}