import 'dart:convert';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/account_brief.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/user_role_list_response.dart';
import '/hrm/service/hrm_base_client.dart';
import '/hrm/service/token_service.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'device_service.dart';


class UserService{

  Future<AccountBrief> getBrief( String userAccessToken ) async{

    String _detailURI = "/v1/recogser/user/detail";
    String userDetail = await getRequestWithAccessToken(_detailURI, userAccessToken);
    AccountBrief accountBrief = new AccountBrief();
    accountBrief.fromJson(userDetail);
    if( Constant.ENABLE_LOGS)
      print(userDetail);
    return accountBrief;
  }

  Future<User> getUserInformation( String userAccessToken, String guid, [bool save=true] ) async{
    if( guid == null){
      AccountBrief accountBrief = await getBrief(userAccessToken);
      if( accountBrief == null) return null;

      guid = accountBrief.userGUID;
    }

    if ( userAccessToken != null && guid != null ){
      String _informationURI = "/v1/recogser/user/info/" + guid;
      String userInformation = await getRequestWithAccessToken(_informationURI, userAccessToken);
      User user = new User();
      user.fromJson(userInformation, save);

      return user;
    }

    return null;
  }

  Future<RoleListResponse> roles(String userAccessToken) async{

    String _uri = "/v1/recogser/user/roles";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    RoleListResponse jobTitleListResponse = new RoleListResponse();
    jobTitleListResponse.fromJson(response);

    return jobTitleListResponse;
  }

  Future<AuthorityResponse> authority(String userAccessToken) async{

    String _uri = "/v1/recogser/user/role_check";
    String response = await getRequestWithAccessToken(_uri, userAccessToken);
    if( Constant.ENABLE_LOGS)
      print(response);
    AuthorityResponse authorityResponse = new AuthorityResponse();
    await authorityResponse.fromJson(response, true);

    return authorityResponse;
  }


  Future<String> userRegister(
      String faceGUID,
      String name,
      String email,
      String phone,
      String roleGuid,
      String username,
      String password,
      String accessToken ) async{

    String userGUID;
    Map<String, dynamic> userData = {
      "name":name,
      "username": username,
      "facialGUID": faceGUID,
      "password": password,
      "email": email,
      "phone": phone,
      "roleGUID": roleGuid
    };

    if( Constant.ENABLE_LOGS)
      print('User data: ${json.encode(userData)}');

    String uri = "/v1/recogser/user/create";
    String jsonString = await postRequestWithAccessToken(uri, userData, accessToken);
    if( Constant.ENABLE_LOGS)
      print( jsonString );
    Map<String, dynamic> registerResponse = jsonDecode(jsonString);
    bool success = registerResponse['success'];

    if( success == null){
      return null;
    }

    if( success ){
      userGUID = registerResponse['data'];
    }

    return userGUID;
  }


  Future<void> logout() async{

    try{
      User user = new User( );
      await user.fromSharedPreferences();
      TokenService tokenService = new TokenService( );
      String accessToken = await tokenService.faceLogin( user.facialGUID, false );
      DeviceService deviceService = new DeviceService();
      await deviceService.logout(accessToken);

    }catch(e){}

    SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA);
    await sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
    await sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
    await sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
    await sharedPrefs.remove(Constant.DEVICE_GUID);
    await sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
    await sharedPrefs.remove(Constant.AUTHENTICATION_2FA);

  }

  Future<bool> userUpdate( Map<String, dynamic> userData, String accessToken ) async{
    bool result = false;
    if( Constant.ENABLE_LOGS)
      print('User data: ${json.encode(userData)}');

    try{
      String uri = "/v1/recogser/user/update";
      String jsonString = await postRequestWithAccessToken( uri, userData, accessToken );
      if( Constant.ENABLE_LOGS)
        print( jsonString );

      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];

    }catch(e){}
    return result;
  }

  Future<bool> changePassword( Map<String, dynamic> userData, String accessToken ) async{
    bool result = false;
    if( Constant.ENABLE_LOGS)
      print('Password data: ${json.encode(userData)}');

    try{
      String uri = "/v1/recogser/user/change_password";
      String jsonString = await postRequestWithAccessToken( uri, userData, accessToken );
      if( Constant.ENABLE_LOGS)
        print( jsonString );

      Map<String, dynamic> registerResponse = jsonDecode(jsonString);
      result = registerResponse['success'];

    }catch(e){}
    return result;
  }

  Future<bool> isAvailable( String username)async{
    bool response = false;

    try{
      String uri = "/v1/recogser/user/available/$username";
      String jsonString = await getRequest( uri );
      if( Constant.ENABLE_LOGS)
        print(jsonString);
      Map<String, dynamic> resultData = jsonDecode(jsonString);
      response = resultData["success"];
    }catch(e){}

    return response;
  }
}