import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/attendance_detail_response.dart';
import '/hrm/service/attendance_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:url_launcher/url_launcher.dart';



class AttendanceDetail extends StatefulWidget {

  AttendanceDetail({Key key, @required this.guid}) : super(key: key);
  final guid;
  @override
  _AttendanceDetailState createState() => _AttendanceDetailState();
}

class _AttendanceDetailState extends State<AttendanceDetail> {


  Color progressColor = Colors.white;
  User _currentUser = new User();
  String _accessToken;
  AttendanceDetailResponse _attendanceDetailResponse = new AttendanceDetailResponse();
  SharedPreferences _sharedPrefs;

  @override
  void initState() {
    super.initState();
    _getLoadData( );
  }

  Future<void> _getLoadData( ) async{
    _sharedPrefs = await SharedPreferences.getInstance();

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences();

    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    if( Constant.ENABLE_LOGS)
      print(_accessToken);
    AttendanceService attendanceService = new AttendanceService();
    _attendanceDetailResponse = await attendanceService.detail(_accessToken, widget.guid);

    progressColor = Colors.transparent;
    if (mounted) setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
            appBar: AppBar(
              title: Text(tr(LocaleKeys.titleAttendanceDetail)),
              bottom: _createProgressIndicator(),
            ),
            body: Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          Container(
                              padding: const EdgeInsets.only(left: 10.0, top: 0.0, right: 10.0),
                              width: MediaQuery.of(context).size.width,
                              child:Card(
                                color: Colors.white,
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      Container(
                                        margin: const EdgeInsets.only(left: 5.0, right: 5.0, top: 16),
                                        child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(
                                                      width: 120,
                                                      height: 120,
                                                      child: _attendanceDetailResponse.facialGUID!=null?
                                                      new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_attendanceDetailResponse.facialGUID}',
                                                        headers: {'Authorization':'Bearer ${_accessToken}'}, width: 48, height: 48, ) : Icon(Icons.person),
                                                  ),
                                                ),
                                              )
                                          ),
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.all(0.0),
                                        child: ListTile(
                                          title: Text(
                                            "${_attendanceDetailResponse.employeeName}",
                                            textAlign: TextAlign.center,
                                            style: TextStyle(
                                              color: Theme.of(context).primaryColor,
                                              fontSize: 24.0,
                                              fontWeight: FontWeight.bold,
                                            ),
                                          ),
                                          subtitle: Text(
                                            "${_attendanceDetailResponse.departmentName} Dept, ${_attendanceDetailResponse.province!=null?_attendanceDetailResponse.province:''} ${_attendanceDetailResponse.country!=null?_attendanceDetailResponse.country:''}",
                                            textAlign: TextAlign.center,
                                          ),
                                        ),
                                      ),
                                      _attendanceDetailResponse.supervisorName==null?Container() :Padding(
                                        padding: const EdgeInsets.only( bottom: 16),
                                        child: Text(
                                          "Supervisor: ${_attendanceDetailResponse.supervisorName}",
                                          textAlign: TextAlign.center,
                                        ),
                                      ),
                                    ]
                                ),
                              )
                          ),
                          Container(
                              padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                              width: MediaQuery.of(context).size.width,
                              child:Card(
                                color: Colors.white,
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      Padding(
                                        padding: const EdgeInsets.only(top:16),
                                        child: Text(
                                          tr(LocaleKeys.btnPunchIn),
                                          textAlign: TextAlign.left,
                                          style: TextStyle(
                                            color: Theme.of(context).primaryColor,
                                            fontSize: 24.0,
                                            fontWeight: FontWeight.bold,
                                          ),
                                        )
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.all(0.0),
                                        child: ListTile(
                                          title: Text(
                                              _attendanceDetailResponse.dutyIn!=null?
                                            "${tr(LocaleKeys.lblRegularIn)} ${new DateFormat('hh:mm a').format(_attendanceDetailResponse.dutyIn.toLocal())}"
                                            : '${tr(LocaleKeys.lblRegularIn)} Unknown',
                                          ),
                                          leading: Icon(Icons.alarm_on),
                                        ),
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.all(0.0),
                                        child: ListTile(
                                          title: Text(
                                              _attendanceDetailResponse.inTime!=null?
                                            "${new DateFormat('dd MMMM yyyy').format(_attendanceDetailResponse.inTime.toLocal())}"
                                            : '',
                                          ),
                                          subtitle: Text(
                                            _attendanceDetailResponse.inTime!=null?
                                            "${new DateFormat('hh:mm a').format(_attendanceDetailResponse.inTime.toLocal())}"
                                            : '',
                                          ),
                                          trailing: Text(
                                            _attendanceDetailResponse.late!=null?
                                            "${tr(LocaleKeys.lblLate)} ${_attendanceDetailResponse.late?'Yes':'No'}"
                                            :'',
                                          ),
                                          leading: Icon(Icons.arrow_forward),
                                        ),
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.only(bottom: 16),
                                        child: ListTile(
                                          title: Text(
                                            tr(LocaleKeys.lblLocation),
                                          ),
                                          subtitle: Text(
                                            "${_attendanceDetailResponse.inLatitude??'--.--------'}\n${_attendanceDetailResponse.inLongitude??'--.--------'}",
                                          ),
                                          leading: Icon(Icons.arrow_forward),
                                          trailing: Icon(Icons.map, color: Theme.of(context).primaryColor),
                                          onTap:_attendanceDetailResponse.inLatitude==null?null:  () async{
                                            try {
                                              final url = 'https://www.google.com/maps/search/?api=1&query=${_attendanceDetailResponse.inLatitude},${_attendanceDetailResponse.inLongitude}';
                                              if (await canLaunch(url)) {
                                                await launch(url);
                                              } else {
                                                throw 'Could not launch $url';
                                              }
                                            } on Exception catch (ex) {
                                              if( Constant.ENABLE_LOGS)
                                                print('${ex.toString()}');
                                            }
                                          },
                                        ),
                                      ),
                                    ]
                                ),
                              )
                          ),
                          Container(
                              padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                              width: MediaQuery.of(context).size.width,
                              child:Card(
                                color: Colors.white,
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      Padding(
                                          padding: const EdgeInsets.only(top:16),
                                          child: Text(
                                            tr(LocaleKeys.btnPunchOut),
                                            textAlign: TextAlign.left,
                                            style: TextStyle(
                                              color: Theme.of(context).primaryColor,
                                              fontSize: 24.0,
                                              fontWeight: FontWeight.bold,
                                            ),
                                          )
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.all(0.0),
                                        child: ListTile(
                                          title: Text(
                                            _attendanceDetailResponse.dutyOut!=null?
                                            "${tr(LocaleKeys.lblRegularOut)} ${new DateFormat('hh:mm a').format(_attendanceDetailResponse.dutyOut.toLocal())}"
                                                : '${tr(LocaleKeys.lblRegularOut)} Unknown',
                                          ),
                                          leading: Icon(Icons.alarm),
                                        ),
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.all(0.0),
                                        child: ListTile(
                                          title: Text(
                                            _attendanceDetailResponse.outTime!=null?
                                            "${new DateFormat('dd MMMM yyyy').format(_attendanceDetailResponse.outTime.toLocal())}"
                                                : '--/--/--',
                                          ),
                                          subtitle: Text(
                                            _attendanceDetailResponse.outTime!=null?
                                            "${new DateFormat('hh:mm a').format(_attendanceDetailResponse.outTime.toLocal())}"
                                                : '--:--',
                                          ),
                                          trailing: Text(
                                            _attendanceDetailResponse.early!=null?
                                            "${tr(LocaleKeys.lblEarlyOut)} ${_attendanceDetailResponse.early?'Yes':'No'}"
                                                :'${tr(LocaleKeys.lblEarlyOut)}',
                                          ),
                                          leading: Icon(Icons.arrow_back),
                                        ),
                                      ),
                                      Padding(
                                        padding: const EdgeInsets.only(top:16),
                                        child: ListTile(
                                          title: Text(
                                            tr(LocaleKeys.lblLocation),
                                          ),
                                          subtitle: Text(
                                            _attendanceDetailResponse.outLatitude!=null?
                                            "${_attendanceDetailResponse.outLatitude}\n${_attendanceDetailResponse.outLongitude}"
                                            :'--.--------\n--.--------',
                                          ),
                                          leading: Icon(Icons.arrow_back),
                                          trailing: Icon(Icons.map, color: Theme.of(context).primaryColor,),
                                          onTap: _attendanceDetailResponse.outLatitude==null?null: ()async{
                                            try {
                                              final url = 'https://www.google.com/maps/search/?api=1&query=${_attendanceDetailResponse.outLatitude},${_attendanceDetailResponse.outLongitude}';
                                              if (await canLaunch(url)) {
                                                await launch(url);
                                              } else {
                                                throw 'Could not launch $url';
                                              }
                                            } on Exception catch (ex) {
                                              if( Constant.ENABLE_LOGS)
                                                print('${ex.toString()}');
                                            }
                                          },
                                        ),
                                      ),
                                    ]
                                ),
                              )
                          ),
                        ]
                    )
                )
            ),
            backgroundColor: Colors.transparent,
        );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );
}