import 'dart:async';
import 'dart:convert';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/attendance_search_request.dart';
import '/hrm/model/response/attendance_search_response.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/department_search_response.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/service/attendance_service.dart';
import '/hrm/service/department_service.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'attendance_detail.dart';


class AttendanceSearch extends StatefulWidget {
  AttendanceSearch({Key key, this.employeeGuid}) : super(key: key);
  final String employeeGuid;

  @override
  _AttendanceSearchState createState() => _AttendanceSearchState();
}

class _AttendanceSearchState extends State<AttendanceSearch>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  AttendanceSearchResponse _attendanceSearchResponse;
  String _accessToken;
  AttendanceService _attendanceService = new AttendanceService();
  AttendanceSearchRequest _attendanceSearchRequest = new AttendanceSearchRequest();
  User _currentUser = new User();
  List<AttendanceSearchItem> _employees = [];
  SharedPreferences _sharedPrefs;

  DepartmentSearchResponse _departmentSearchResponse;
  DepartmentSearchItem _departmentSearchItem;
  EmployeeSearchResponse _employeeSearchResponse;
  EmployeeSearchItem _employeeSearchItem;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  Color _progressColor = Colors.transparent;
  bool _isSearch = true;

  @override
  void initState() {
    super.initState();
    _loadData( );
  }

  @override
  void deactivate() {
    super.deactivate();
  }

  Future<void> _getLoadData( bool refresh  ) async{
    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences();
    if( _sharedPrefs == null )
      _sharedPrefs = await SharedPreferences.getInstance();

    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

      _attendanceSearchRequest.first = _page;
      _attendanceSearchRequest.max = Constant.ROW_PER_PAGE;
      _attendanceSearchRequest.employeeName = '';
      _attendanceSearchRequest.startDate = _startDate;
      _attendanceSearchRequest.endDate = _endDate;

    if( _employeeSearchItem != null ){
      _attendanceSearchRequest.sharedGUID = _employeeSearchItem==null?null:_employeeSearchItem.guid;
      _attendanceSearchResponse = await _attendanceService.searchByEmployee(
          _accessToken, _attendanceSearchRequest);
    }else{
      _attendanceSearchRequest.sharedGUID = _departmentSearchItem==null? null : _departmentSearchItem.guid;
      _attendanceSearchResponse = await _attendanceService.search(
          _accessToken, _attendanceSearchRequest, _departmentSearchItem!=null);
    }

    if( Constant.ENABLE_LOGS)
      print(json.encode(_attendanceSearchRequest.toMap()));

      if (_attendanceSearchResponse != null &&
          _attendanceSearchResponse.attendances != null) {
        if (refresh && _employees.isNotEmpty)
          _employees.removeRange(0, _employees.length);

        _attendanceSearchResponse.attendances.forEach((element) {
          _employees.add(element);
        });
      }
      if (mounted) setState(() {});

  }

  Widget buildCtn(BuildContext context) {
    return
      ListView.separated(
        reverse: false,
        itemBuilder: (c, i) {
          return _getItem( context, i);
        },
        separatorBuilder: (context, index) {
          return Container(
            margin: EdgeInsets.only(left: 5, right: 5),
            height: 0.5,
            color: Colors.transparent,
          );
        },
        itemCount: _employees.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text(tr(LocaleKeys.titleAttendanceSearch)),
          bottom: _isSearch?_createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
          actions: <Widget>[
            FlatButton.icon(
                onPressed:(){
                  setState(() {
                    _isSearch = !_isSearch;
                  });
                },
                icon: _isSearch? Icon(Icons.cancel, color: Colors.white,)
                    : Icon(Icons.search, color: Colors.white,),
                label: Text(
                  _isSearch? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                  style:  TextStyle(color: Colors.white),
                )
            )
          ],
        ),
        body: _isSearch? Container() : SmartRefresher(
          controller: _refreshController,
          enablePullUp: true,
          child: buildCtn(context),
          physics: BouncingScrollPhysics(),
          header: WaterDropHeader(),
          footer: ClassicFooter(
            loadStyle: LoadStyle.ShowWhenLoading,
            completeDuration: Duration(milliseconds: 500),
          ),

          // scroll down
          onRefresh: () async {
            _page = 0;
            await _getLoadData( true );
    //        await Future.delayed(Duration(milliseconds: 1000));
            if (mounted) setState(() {});
            if( _attendanceSearchResponse.totalRecords==0)
              _refreshController.loadNoData();
            else
              _refreshController.refreshCompleted();
          },
          // scroll up
          onLoading: () async {
            ++_page;
            await _getLoadData( false );
    //        await Future.delayed(Duration(milliseconds: 1000));
            if (mounted) setState(() {});
            if( _attendanceSearchResponse.totalRecords==0)
              _refreshController.loadNoData();
            else
              _refreshController.loadComplete();
          },
        )
    );
  }

  Widget _getItem(BuildContext context, int index){
      return Container(
        //padding: const EdgeInsets.all(10),
        //height: 100,
          child: Card(
              elevation: 0,
              child: Container(
                //padding: EdgeInsets.only(top:10),
                  child: ListTile(
                    title: Text(_employees[index].employeeName,
                      style: TextStyle(color: Theme.of(context).primaryColor),
                    ),
                    subtitle: Text('${_employees[index].departmentName} Dept, ${_employees[index].province!=null?_employees[index].province:''}'),
                    leading: ClipOval(
                        child: Material(
                          elevation: 0,
                          color: Colors.white, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 64,
                              height: 64,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_employees[index].facialGUID}',
                                headers: {'Authorization':'Bearer ${_accessToken}'}, width: 48, height: 48, ),
                            ),
                          ),
                        )
                    ),
                    trailing: Wrap(
                        children:<Widget>[
                    _employees[index].outTime!=null &&
                        new DateFormat('dd-MM-yyyy').format(_employees[index].outTime.toLocal())
                            !=new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())
                        ?Container(
                      margin: EdgeInsets.only(top: 10),
                      child: Text(
                        '${
                            new DateFormat('yyyy').format(_employees[index].inTime.toLocal())
                                ==new DateFormat('yyyy').format(DateTime.now().toLocal())?
                            new DateFormat('dd MMM').format(_employees[index].inTime.toLocal())
                            : new DateFormat('dd/MM/yy').format(_employees[index].inTime.toLocal())
                        }',
                        style: TextStyle(
                            fontWeight: FontWeight.bold,
                            color:
                            _employees[index].late? Colors.red[700]
                                : Theme.of(context).primaryColor
                        ),
                        textAlign: TextAlign.center,
                      ),
                    )
                        :Column(
                      children: <Widget>[
                        Container(
                          margin: EdgeInsets.only(top: 10),
                          child: Text(
                              new DateFormat('dd-MM-yyyy').format(_employees[index].inTime.toLocal())
                                  ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                            '${new DateFormat('hh:mm a').format(_employees[index].inTime.toLocal())}'
                            : '${
                                  new DateFormat('yyyy').format(_employees[index].inTime.toLocal())
                                      ==new DateFormat('yyyy').format(DateTime.now().toLocal())?
                                  new DateFormat('dd MMM').format(_employees[index].inTime.toLocal())
                                  : new DateFormat('dd/MM/yy').format(_employees[index].inTime.toLocal())
                              }',
                            style: TextStyle(
                                fontWeight: FontWeight.bold,
                                color:
                                _employees[index].late? Colors.red[700]
                                    : Theme.of(context).primaryColor
                            ),
                            textAlign: TextAlign.center,
                          ),
                        ),
                        Container(
                          margin: EdgeInsets.only(top: 5),
                          child: Text(
                            _employees[index].outTime!=null?
                            '${new DateFormat('hh:mm a').format(_employees[index].outTime.toLocal())}'
                                :'--:--',
                            style: TextStyle(
                                fontWeight: FontWeight.bold,
                                color:
                                _employees[index].outTime!=null && _employees[index].early? Colors.red[700]
                                    : Theme.of(context).primaryColor
                            ),
                            textAlign: TextAlign.center,
                          ),
                        ),
                      ],
                    ),
                          Container(
                            padding: EdgeInsets.only(top: _employees[index].outTime!=null &&
                                new DateFormat('dd-MM-yyyy').format(_employees[index].outTime.toLocal())
                                    !=new DateFormat('dd-MM-yyyy').format(DateTime.now())?8:16),
                            child: Icon(Icons.keyboard_arrow_right),
                          )
                        ]
                    ),
                    onTap: (){
                      Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) => AttendanceDetail(
                                guid: _employees[index].guid
                            ),
                          )
                      );
                    },
                    contentPadding: EdgeInsets.only(left: 16.0, right: 0.0),
                  )
              )
          )
      );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
    preferredSize: Size(double.infinity, 500.0),
    child: SizedBox(
      height: 500.0,
        child: Card(
            margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
            elevation: 1,
            child: Padding(
                padding: EdgeInsets.all(10),
                child: Column(
                    children: <Widget>[
                      _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('ORGANIZATION_READ')?_deptSelection(context): new Container(),
                      _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ')?_employeeSelection(context) : new Container(),
                      Container(
                        margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                        child:new Row(children: <Widget>[
                          new Expanded(
                              child: new TextFormField(
                                decoration: new InputDecoration(
                                  icon: const Icon(Icons.calendar_today),
                                  hintText: 'yyyy-mm-dd',
                                  labelText: tr(LocaleKeys.lblStartDate),
                                ),
                                controller: _controller_start,
                                keyboardType: TextInputType.datetime,
                              )),
                          new IconButton(
                            icon: new Icon(Icons.mode_edit),
                            tooltip: 'Choose start date',
                            onPressed: (() {
                              _chooseDate(context, _controller_start.text, true);
                            }),
                          )
                        ]),
                      ),
                      Container(
                        margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                        child:new Row(children: <Widget>[
                          new Expanded(
                              child: new TextFormField(
                                decoration: new InputDecoration(
                                  icon: const Icon(Icons.calendar_today),
                                  hintText: 'yyyy-mm-dd',
                                  labelText: tr(LocaleKeys.lblEndDate),
                                ),
                                controller: _controller_end,
                                keyboardType: TextInputType.datetime,
                              )),
                          new IconButton(
                            icon: new Icon(Icons.mode_edit),
                            tooltip: 'Choose end date',
                            onPressed: (() {
                              _chooseDate(context, _controller_end.text, false);
                            }),
                          )
                        ]),
                      ),
                      Container(
                        margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          children: [
                            _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                            _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                          ],
                        ),
                      ),
                      Center(
                        child: Container(
                          width: MediaQuery.of(context).size.width * 0.9,
                          child: FlatButton.icon(
                              color: Theme.of(context).primaryColor,
                              onPressed: (){
                                openBrowser(Constant.BASE_URL_PRIMARY
                                    +'/v1/forwarder/attendance/'+_accessToken
                                    +'?eg='+ (_employeeSearchItem != null? _employeeSearchItem.guid : '')
                                    +'&d='+ (_employeeSearchItem == null && _departmentSearchItem!=null ? _departmentSearchItem.guid : '')
                                     + (_startDate!=null? '&sd=' +new DateFormat('yyyy-MM-dd').format(_startDate.toLocal()): '')
                                    + (_endDate!=null? '&ed=' +new DateFormat('yyyy-MM-dd').format(_endDate.toLocal()): '')
                                );
                                },
                              icon: Icon(Icons.file_upload,size: 32,color: Colors.white,),
                              label: Text('Export to Excel',
                                style:  TextStyle(color: Colors.white),
                              )
                          )
                        ),
                      ),
                    ]
                )
            )
        )
    )
  );

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _employeeSearchItem = null;
                _departmentSearchItem = null;
                _startDate = null;
                _controller_start.text = '';
                _endDate = null;
                _controller_end.text = '';
                _isSearch = !_isSearch;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                _isSearch = !_isSearch;
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Widget _deptSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<DepartmentSearchItem>(
          selectedItem: _departmentSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (DepartmentSearchItem i, DepartmentSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblDepartment),
          onFind: (String filter) => _deptSuggestion(filter),
          onChanged: (DepartmentSearchItem data) {
            _departmentSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, DepartmentSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text(tr(LocaleKeys.lblAllDepartments)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, DepartmentSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<DepartmentSearchItem>> _deptSuggestion(String query) async{
    List<DepartmentSearchItem> matches = List();
    if( _departmentSearchResponse.departments == null)
      return matches;

    matches.addAll(_departmentSearchResponse.departments);

    matches.retainWhere((s) {
      DepartmentSearchItem dept = s;
      if( dept.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _employeeSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeSearchItem>> _employeeSuggestion(String query) async{
    List<EmployeeSearchItem> matches = List();
    if( _employeeSearchResponse.employees == null)
      return matches;
    matches.addAll(_employeeSearchResponse.employees);

    matches.retainWhere((s) {
      EmployeeSearchItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  _loadData() async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    if( _currentUser.facialGUID == null){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
    }
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    DepartmentService _departmentService = new DepartmentService();
    _departmentSearchResponse = await _departmentService.search(_accessToken, _currentUser.organizationGUID, '');
    EmployeeService _employeeService = new EmployeeService();
    _employeeSearchResponse = await _employeeService.search(_accessToken, '', 0, 100000);

    if( _employeeSearchResponse != null && _employeeSearchResponse.employees!=null && widget.employeeGuid!=null){
      _employeeSearchResponse.employees.forEach((element) {
        if( element.guid == widget.employeeGuid){
          _employeeSearchItem = element;
          _isSearch = !_isSearch;
          return;
        }
      });
    }

    if( mounted)
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }


}
