import 'dart:async';
import 'dart:io';
import 'dart:convert';
import '/hrm/model/request/attendance_request.dart';
import '/hrm/model/response/attendance_response.dart';
import '/hrm/service/firebase_service.dart';
import 'package:flutter/services.dart';
import 'package:location/location.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/last_attendance_response.dart';
import '/hrm/model/response/recognize_result.dart';
import '/hrm/service/attendance_service.dart';
import '/hrm/service/face_detector_service.dart';
import '/hrm/service/face_recognization_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/foundation.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import '/hrm/imageprocessing/flutter_camera.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:intl/intl.dart';
import 'package:path_provider/path_provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import '/hrm/plugins/wifi.dart';



class FaceVerify extends StatefulWidget {
  FaceVerify({Key key}) : super(key: key);
  @override
  _FaceVerifyPageState createState() => _FaceVerifyPageState();
}

class _FaceVerifyPageState extends State<FaceVerify> {

  User _user = new User();
  String _accessToken;

//  List<Face> _faces = [];
  final _scanKey = GlobalKey<CameraMlVisionState>();
  bool _processing = false;
  bool willStop = false;
  bool _foundFace = false;
  bool _detected = false;
  Color progressColor = Colors.transparent;
  CameraLensDirection cameraLensDirection = CameraLensDirection.front;
  RecognizeResult _recognizeResult;
  FaceRecognizationService _faceRecognizationService;
  FaceDetectorService faceDetectorService = new FaceDetectorService();
  AttendanceService _attendanceService = new AttendanceService();
  String _action = LastAttendanceResponse.ACTION_IN;

  LocationData _startLocation;
  LocationData _currentLocation;
  Location _locationService  = new Location();
  bool _permission = false;
  String error;
  String _message = '';
  Color _messageColor = Colors.white;
  SharedPreferences _sharedPrefs;


  @override
  initState() {
    super.initState( );
     _getAccessToken();
    faceDetectorService.init();
    // listenThread( );
    _faceRecognizationService = new FaceRecognizationService();
    _initLocationService();
  }

  void _getAccessToken() async{
    await _user.fromSharedPreferences();
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _user.facialGUID, false );
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  void listenThread( ) async{
    var thread = new Thread(listen);
    await thread.start();
    await thread.join();
    if( Constant.ENABLE_LOGS)
      print("Thread terminated");
  }

  Future listen() async {

    while ( !willStop ) {
      await Thread.sleep( 50 );
      if( !_processing && mounted && !_foundFace)
        _processPhoto( );
      await Thread.sleep( 50 );
    }
    await Thread.sleep( 1 * 1000 );
    if( Constant.ENABLE_LOGS)
      print("Thread sleep");
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
      appBar: AppBar(
        title: Text(tr(LocaleKeys.mnuInOut)),
          bottom: _createProgressIndicator(),
      ),
      body: SizedBox.expand(
        child: CameraMlVision<List<Object>>(
          key: _scanKey,
          cameraLensDirection: cameraLensDirection,
          resolution: ResolutionPreset.high,
          overlayBuilder: (c) {
            return Positioned(
              bottom: 16.0,
              left: 16.0,
              right: 16.0,
              child: MaterialButton(
                onPressed: () async{await _processPhoto();},
                color: Theme.of(context).primaryColor,
                textColor: Colors.white,
                child: Icon(
                  Icons.camera_alt,
                  size: 32,
                ),
                padding: EdgeInsets.all(16),
                shape: CircleBorder(),
              )
            );
            return Container();
//              CustomPaint(
//              painter: FaceDetectorPainter(
//                  _scanKey.currentState.cameraValue.previewSize.flipped, _faces,
//                  reflection: cameraLensDirection == CameraLensDirection.front),
//            );
          },
          overlayBuilder1: (c){
            return _actionPanel( context );
          },
          onDispose: () {
          },
        ),
      ),
    )
    );
  }

  Widget _actionPanel( BuildContext context ){
    return _foundFace ? Positioned(
      bottom: 50.0,
      left: 10.0,
      right: 10.0,
      child: Card(
        color: Colors.transparent,
        elevation: 1.0,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Column(
          children: <Widget>[
            Container(
              padding: const EdgeInsets.all(10.0),
              child: ClipOval(
                  child: Material(
                    elevation: 2,
                    color: Colors.white, // button color
                    child: InkWell(
                      splashColor: Colors.red, // inkwell color
                      child: SizedBox(
                        width: 150,
                        height: 150,
                        child: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_recognizeResult.faceGUID}',
                          headers: {'Authorization':'Bearer ${_accessToken}'}, ),
                      ),
                    ),
                  )
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(0.0),
              child: Text(
                "${_recognizeResult.name}",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 24.0,
                  fontWeight: FontWeight.bold,
                ),
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(2.0),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Container(
                    child: Text(
                      '${new DateFormat('hh:mm').format(new DateTime.now())}',
                      style: TextStyle(
                          color: Colors.white,
                          fontWeight: FontWeight.normal,
                          fontSize: 48
                      ),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  Container(
                    margin: EdgeInsets.only(top: 20),
                    child: Text(
                      '${new DateFormat('a').format(new DateTime.now()).toLowerCase()}',
                      style: TextStyle(
                          color: Colors.white,
                          fontWeight: FontWeight.normal,
                          fontSize: 24
                      ),
                      textAlign: TextAlign.center,
                    ),
                  ),
                ],
              )
            ),
            Container(
              padding: const EdgeInsets.only(left:10.0,right: 10, bottom: 10),
              child: Divider(
                color: Colors.white,
              ),
            ),
            Container(
              margin: EdgeInsets.only(bottom: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  _buildButtons(
                      _action == LastAttendanceResponse.ACTION_IN? Colors.green[900] : Colors.grey,
                      SvgPicture.asset( 'assets/images/punch_in.svg',
                        semanticsLabel: tr(LocaleKeys.btnPunchIn),height: 40, width: 40, ),
                      tr(LocaleKeys.btnPunchIn)
                  ),
                  _buildButtons( _action == LastAttendanceResponse.ACTION_OUT? Colors.green[900] : Colors.grey,
                      SvgPicture.asset( 'assets/images/punch_out.svg',
                        semanticsLabel: tr(LocaleKeys.btnPunchOut),height: 40, width: 40, ),
                      tr(LocaleKeys.btnPunchOut)
                  ),
                ],
              ),
            ),
            Container(
              margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
              child: Text(
                _message,
                softWrap: true,
                style: TextStyle(
                  fontWeight: FontWeight.w400,
                  color: _messageColor,
                  fontSize: 18,
                ),
              ),
            ),
          ],
        ),
      ),
    )
        : Positioned(
      top: 20.0,
      left: 10.0,
      right: 10.0,
      child: !_detected ? new Container()
          : Card(
        color: Color.fromRGBO( 0, 00, 0, .7),
        elevation: 2.0,
        child:Column(
          children: <Widget>[
            Image(image: AssetImage('assets/images/face_scan.gif')),
            Container(
              margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
              child: ListTile(
                title:Text(
                  'Verifying your face ....',
                  softWrap: true,
                  style: TextStyle(
                    fontWeight: FontWeight.w400,
                    color: _messageColor,
                    fontSize: 14,
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Column _buildButtons(Color color, SvgPicture icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 3.0,
          icon: icon,
          color: color,
          onPressed:!_processing?() async{
            String _note = 'Face In';
            if( label == tr(LocaleKeys.btnPunchIn) ){
              _doAttend(_note, LastAttendanceResponse.ACTION_IN);
            }else{
              _note = 'Face Out';
              _doAttend(_note, LastAttendanceResponse.ACTION_OUT);
            }
          } : null,
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  @override
  void deactivate() {
    willStop = true;
    super.deactivate();
  }

  @override
  void dispose() {
    try {
      if (_scanKey.currentState.mounted) {
        _scanKey.currentState.dispose();
      }
    }catch(e){}
    willStop = true;
    try {
      faceDetectorService.dispose();
//      detector.close();
    }catch(e){}

    super.dispose();
  }

  Future<void> _processPhoto( ) async{
    try {
          if( _scanKey.currentState == null ) return;
          if( _processing ) return;
          if( !mounted ) return;

          _processing = true;
          _foundFace = false;
          setState(() {
            progressColor = Colors.white;
            _detected = false;
          });

          final Directory extDir = await getApplicationDocumentsDirectory();
          final String tmpDir = '${extDir.path}/Pictures/flutter_test';
          await Directory(tmpDir).create(recursive: true);
          String photoPath = '$tmpDir/${DateTime.now()}.jpg';

          if( _scanKey.currentState != null)
            photoPath = await _scanKey.currentState.takePhoto( );
print(photoPath);
            DetectionResult detectionResult = await faceDetectorService.detectedFaceAsBase64String(tmpDir, photoPath, mounted, true);

            if( detectionResult != null ){

              setState(() {
                progressColor = Colors.green;
                _detected = true;
              });
              _recognizeResult = await _faceRecognizationService.verify( detectionResult.detectedFace, _user.facialGUID, true, _user.organizationGUID);

              if( _recognizeResult == null){
//                await _getAccessToken();
                _foundFace = false;
                _processing = false;
                setState(() {
                  progressColor = Colors.red[600];
                  _detected = false;
                });
                return;
              }

              if( _recognizeResult.result ) {
                if( Constant.ENABLE_LOGS)
                  print(_recognizeResult.name);
                await _getLastAttendanceAction();
                setState(() {
                  progressColor = Colors.transparent;
                  _processing = false;
                  willStop = true;
                  _foundFace = true;
                });
              }else{
                setState(() {
                  progressColor = Colors.red[600];
                  _foundFace = false;
                  _processing = false;
                });
              }
            }else{
            try{
              new File( photoPath ).deleteSync();
            }catch(e){}
          }

    }catch( e){
      _processing = false;
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    _processing = false;
  }

  PreferredSize _createProgressIndicator( ) => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  Future<bool> _onWillPop( ) async {
    return true;
  }

  Future<void> _getLastAttendanceAction() async{
    LastAttendanceResponse lastAttendanceResponse = await _attendanceService.lastAction(_accessToken, _recognizeResult.faceGUID);
    if( lastAttendanceResponse == null || (lastAttendanceResponse!=null && lastAttendanceResponse.action==LastAttendanceResponse.ACTION_OUT) ){
      _action = LastAttendanceResponse.ACTION_IN;
    }else
      _action = LastAttendanceResponse.ACTION_OUT;
  }

  Future<void> _doAttend( String note, String action ) async{

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
    });

    _processing = true;
    AttendanceRequest attendanceRequest = new AttendanceRequest();
    attendanceRequest.action = action;
    attendanceRequest.note = note;
    attendanceRequest.faceGUID = _recognizeResult.faceGUID;
    attendanceRequest.firebase = await getFirebaseToken( );
    attendanceRequest.source = Platform.isIOS? Constant.PLATFORM_IOS : Constant.PLATFORM_ANDROID;

    try {
      if (_startLocation != null) {
        attendanceRequest.latitude = _startLocation.latitude;
        attendanceRequest.longitude = _startLocation.longitude;
        attendanceRequest.altitude = _startLocation.altitude;
        attendanceRequest.accuracy = _startLocation.accuracy;
        attendanceRequest.heading = _startLocation.heading;
        attendanceRequest.speed = _startLocation.speed;
      }
    }catch(e){}

    try {
      await Wifi.listMap().then((List<Map<String, dynamic>> result) {
          attendanceRequest.wifis = result;
      });
    }catch(e){}

    try {
      if( Constant.ENABLE_LOGS)
        print('${json.encode(attendanceRequest.toMap())}');

      AttendanceResponse attendanceResponse = await _attendanceService.attend(_accessToken, attendanceRequest.toMap());
      setState(() {
        if (attendanceResponse.success==null || !attendanceResponse.success) {
          progressColor = Colors.transparent;
          _message = 'Problem occur: ${attendanceResponse.error}';
          _messageColor = Colors.red[700];
          _processing = false;
        } else {
          progressColor = Colors.transparent;
          _messageColor = Colors.white;
          if (action == 'in') {
            _message = 'Punch-In successful, thank you';
          } else {
            _message = 'Punch-Out successful, thank you';
          }
          new ThreadTimer(new Duration(milliseconds: 3000), () {
                Navigator.pop(context, _recognizeResult);
          });
        }
      });
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }

  _initLocationService() async {
    await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
    LocationData location;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      bool serviceStatus = await _locationService.serviceEnabled();
      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      if( Constant.ENABLE_LOGS)
        print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }

    setState(() {
      _startLocation = location;
    });

  }

}