import 'dart:async';
import 'dart:convert';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/attendance_search_request.dart';
import '/hrm/model/response/attendance_search_response.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/service/attendance_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'attendance_detail.dart';
import 'attendance_search.dart';



class LatestForOrganization extends StatefulWidget {
  LatestForOrganization({Key key, @required this.onRefresh}) : super(key: key);
  final Function onRefresh;

  @override
  _LatestForOrganizationState createState() => _LatestForOrganizationState();
}

class _LatestForOrganizationState extends State<LatestForOrganization>{

  int _page = 0;
  int _selectType = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  AttendanceSearchResponse _attendanceSearchResponse;
  String _accessToken;
  AttendanceService _attendanceService = new AttendanceService();
  AttendanceSearchRequest _attendanceSearchRequest = new AttendanceSearchRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<AttendanceSearchItem> _employees = [];
  SharedPreferences _sharedPrefs;

  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  @override
  void deactivate() {
    super.deactivate();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null) {

      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();

    }

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _attendanceSearchRequest.first = _page;
    _attendanceSearchRequest.max = Constant.ROW_PER_PAGE;
    _attendanceSearchRequest.employeeName = '';

    if( _selectType == 1) {
    _attendanceSearchRequest.startDate = DateTime(
        DateTime.now().toLocal().year,
        DateTime.now().toLocal().month,
        DateTime.now().toLocal().day - 7,
        0,
        00,
        00);
    _attendanceSearchRequest.endDate = DateTime(
        DateTime.now().toLocal().year,
        DateTime.now().toLocal().month,
        DateTime.now().toLocal().day,
        23,
        59,
        59);
    }else{
      _attendanceSearchRequest.startDate = null;
      _attendanceSearchRequest.endDate = null;
    }
    if( Constant.ENABLE_LOGS)
      print(json.encode(_attendanceSearchRequest.toMap()));

    _attendanceSearchResponse = await _attendanceService.search(_accessToken, _attendanceSearchRequest, false);

    if( _attendanceSearchResponse!=null && _attendanceSearchResponse.attendances!=null){
      if( refresh && _employees.isNotEmpty)
        _employees.removeRange(0, _employees.length);

      _attendanceSearchResponse.attendances.forEach((element) {
        _employees.add(element);
      });

    }

    if (mounted) setState(() {});
  }

  Widget buildCtn(BuildContext context) {
    return ListView.separated(
      reverse: false,
      itemBuilder: (c, i) {
        return _getItem( context, i);
      },
      separatorBuilder: (context, index) {
        return Container(
          margin: EdgeInsets.only(left: 5, right: 5),
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _employees.length +1,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildCtn(context),
      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadFailed();
      },
    );
  }

  Widget _getItem(BuildContext context, int index){
    if (index == 0){
      return _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ')?
      _typeSelection( )
      : Container();
    }else{
      return Container(
        //padding: const EdgeInsets.all(10),
        //height: 100,
          child: Card(
              elevation: 0,
              child: Container(
                //padding: EdgeInsets.only(top:10),
                  child: ListTile(
                    title: Text(_employees[index -1].employeeName,
                      style: TextStyle(color: Theme.of(context).primaryColor),
                    ),
                    subtitle: Text('${_employees[index -1].departmentName} Dept, ${_employees[index -1].province!=null?_employees[index -1].province:''}'),
                    leading: ClipOval(
                        child: Material(
                          elevation: 0,
                          color: Colors.white, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_employees[index -1].facialGUID}',
                                headers: {'Authorization':'Bearer ${_accessToken}'}, width: 48, height: 48, ),
                            ),
                          ),
                        )
                    ),
                    trailing: Wrap(
                      children:<Widget>[
                        _employees[index -1].outTime!=null &&
                        new DateFormat('dd-MM-yyyy').format(_employees[index -1].outTime.toLocal())
                            !=new DateFormat('dd-MM-yyyy').format(DateTime.now())
                        ?Container(
                          margin: EdgeInsets.only(top: 10),
                          child: Text(
                            '${new DateFormat('dd MMM').format(_employees[index -1].inTime.toLocal())}',
                            style: TextStyle(
                                fontWeight: FontWeight.bold,
                                color:
                                _employees[index -1].late? Colors.red[700]
                                    : Theme.of(context).primaryColor
                            ),
                            textAlign: TextAlign.center,
                          ),
                        )
                        :Column(
                      children: <Widget>[
                        Container(
                          margin: EdgeInsets.only(top: 10),
                          child: Text(
                            new DateFormat('dd-MM-yyyy').format(_employees[index -1].inTime.toLocal())
                                ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())?
                            '${new DateFormat('hh:mm a').format(_employees[index -1].inTime.toLocal())}'
                                : '${new DateFormat('dd MMM').format(_employees[index -1].inTime.toLocal())}',
//                            '${new DateFormat('hh:mm a').format(_employees[index -2].inTime.toLocal())}',
                            style: TextStyle(
                                fontWeight: FontWeight.bold,
                                color:
                                _employees[index -1].late? Colors.red[700]
                                    : Theme.of(context).primaryColor
                            ),
                            textAlign: TextAlign.center,
                          ),
                        ),
                        Container(
                          margin: EdgeInsets.only(top: 5),
                          child: Text(
                            _employees[index -1].outTime!=null?
                            '${new DateFormat('hh:mm a').format(_employees[index -1].outTime.toLocal())}'
                                :'--:--',
                            style: TextStyle(
                              fontWeight: FontWeight.bold,
                                color:
                                _employees[index -1].outTime!=null && _employees[index -1].early? Colors.red[700]
                                    : Theme.of(context).primaryColor
                            ),
                            textAlign: TextAlign.center,
                          ),
                        ),
                      ],
                    ),
                        Container(
                          padding: EdgeInsets.only(top: _employees[index -1].outTime!=null &&
                              new DateFormat('dd-MM-yyyy').format(_employees[index -1].outTime.toLocal())
                                  !=new DateFormat('dd-MM-yyyy').format(DateTime.now())?8:16),
                         child: Icon(Icons.keyboard_arrow_right),
                        )
                      ]
                    ),
                    onTap: (){
                      Navigator.push(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) => AttendanceDetail(
                                guid: _employees[index -1].guid
                            ),
                          )
                      );
                    },
                    contentPadding: EdgeInsets.only(left: 16.0, right: 0.0),
                  )
              )
          )
      );
    }
  }

  Widget _typeSelection( ){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, left: 10.0),
      child:new Row(
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[
          new Text(
            tr(LocaleKeys.mnuInOut),
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 0,
            groupValue: _selectType,
            onChanged: ( value)async{
              setState(() { _selectType = value; });
              _page = 0;
              await _getLoadData( true);
            },
          ),
          new Text(
            tr(LocaleKeys.lblDay),
            style: new TextStyle(),
          ),
          new Radio(
            value: 1,
            groupValue: _selectType,
            onChanged: ( value) async{
              setState(() {_selectType = value;});
              _page = 0;
              await _getLoadData( true);
            },
          ),
          new Text(
            tr(LocaleKeys.lblWeek),
            style: new TextStyle(),
          ),new Radio(
            value: 2,
            groupValue: _selectType,
            onChanged: ( value){
              setState(() { _selectType = value;});
              _gotoAttendanceSearch( );
            },
          ),
          new Text(
            tr(LocaleKeys.mnuMore),
            style: new TextStyle(),
          ),
        ],
      ),
    );
  }

  void _gotoAttendanceSearch() async{
    OrganizationDetailResponse detailResponse = new OrganizationDetailResponse();
    await detailResponse.fromSharedPreferences();
    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }

    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => AttendanceSearch( ),
        )
    );
  }
  Future<bool> _showUpgradeDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Upgrade required!"),
          content: new Text("Thank you for using QuickQ HR, this feature is required to upgrade your plan. Please contact us: " + Constant.CONTACT_PHONE),
          actions: <Widget>[
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }
}
