import 'dart:io';
import 'dart:math';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/imageprocessing/base64image.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/face_request.dart';
import '/hrm/model/response/department_search_response.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/model/response/job_titles_response.dart';
import '/hrm/model/response/pay_grade_response.dart';
import '/hrm/model/response/user_role_list_response.dart';
import '/hrm/service/department_service.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/face_detector_service.dart';
import '/hrm/service/face_recognization_service.dart';
import '/hrm/service/job_titles_service.dart';
import '/hrm/service/pay_grade_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_datetime_picker/flutter_datetime_picker.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:image/image.dart' as imglib;
import 'package:intl/intl.dart';
import 'package:path_provider/path_provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';


class AddUser extends StatefulWidget {

  AddUser({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _AddUserPageState createState() => _AddUserPageState();

}

class _AddUserPageState extends State<AddUser> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  final _dateFormatTimeDisplay = new DateFormat('hh:mm a');
  final _dateFormatTimeOutput = new DateFormat('HH:mm');

  bool _processing = false;
  Color progressColor = Colors.transparent;
  String _name;
  String _gender;
  String _employeeId;
  DateTime _joinedDate;
  String _registeredFaceGUID=null;
  String _registeredUserGUID=null;
  String _registeredEmployeeGUID=null;
  bool _hasMore = false;
  DateTime inDate = DateTime(2020, 02, 01, 9, 00, 00);
  DateTime outDate = DateTime(2020, 02, 01, 17, 00, 00);
  final TextEditingController _controller_inTime = new TextEditingController();
  final TextEditingController _controller_outTime = new TextEditingController();
  final TextEditingController _controller_dept = new TextEditingController();
  final TextEditingController _controller_jobTitle = new TextEditingController();
  final TextEditingController _controller_supervisor = new TextEditingController();
  final TextEditingController _controller_role = new TextEditingController();
  final TextEditingController _controller_name = new TextEditingController();
  final TextEditingController _controller_email = new TextEditingController();
  final TextEditingController _controller_phone = new TextEditingController();
  final TextEditingController _controller_employeeId = new TextEditingController();
  final TextEditingController _controller_joinedDate = new TextEditingController();

  int _selectedPhoto = 0;
  List<String> _photoLabel = ["User profile image","Face photo 1","Face photo 2","Face photo 3"];
  List<String> _images = ['no','no','no','no'];
  String _cancelLabel = tr(LocaleKeys.btnCancel);

  DepartmentSearchResponse _departmentSearchResponse;
  DepartmentSearchItem _departmentSearchItem;

  JobTitleListResponse _jobTitleListResponse;
  JobTitleItem _jobTitleItem;

  RoleListResponse _roleListResponse;
  RoleItem _roleItem;

  EmployeeSearchResponse _employeeSearchResponse;
  EmployeeSearchItem _employeeSearchItem;

  EmployeeSearchItem _approver1;
  EmployeeSearchItem _approver2;
  EmployeeSearchItem _approver3;

  EmployeeService _employeeService;
  UserService _userService;
  User _currentUser = new User();
  String _accessToken;
  SharedPreferences _sharedPrefs;

  PayGradeResponse _payGradeResponse;
  PayGradeItem _payGradeItem;

  FaceDetectorService faceDetectorService = new FaceDetectorService();
  @override
  void initState() {
    super.initState();
    _getAccessToken( true );
    faceDetectorService.init();
  }

  void _getAccessToken( bool isFirstLoad) async{
    if( isFirstLoad) {
      _sharedPrefs = await SharedPreferences.getInstance();
      await _currentUser.fromSharedPreferences();
    }
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    if(isFirstLoad)
      _loadData();
  }

  _loadData() async{
    DepartmentService _departmentService = new DepartmentService();
    _departmentSearchResponse = await _departmentService.search(_accessToken, _currentUser.organizationGUID, '');

    JobTitlesService jobTitlesService = new JobTitlesService();
    _jobTitleListResponse = await jobTitlesService.search(_currentUser.organizationGUID, _accessToken, '');

    PayGradeService payGradeService = new PayGradeService();
    _payGradeResponse = await payGradeService.list( _currentUser.organizationGUID, _accessToken );

    _employeeService = new EmployeeService();
    _employeeSearchResponse = await _employeeService.supervisor(_accessToken, '' );

    _userService = new UserService( );
    _roleListResponse = await _userService.roles(_accessToken);
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            key: _scaffoldKey,
            resizeToAvoidBottomInset: true,
            // resizeToAvoidBottomPadding: true,
            appBar: AppBar(
              title: Text(widget.title),
              bottom: _createProgressIndicator(),
            ),
            body: Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          Container(
                            margin: const EdgeInsets.only(left: 20.0, right: 5.0, top: 10),
                            child:Text(
                              'One step and simple way, you can import all of your employees\' information from Excel file.',
                              style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                            ),
                          ),
                          Container(
                              margin: const EdgeInsets.only(left: 20.0, right: 5.0),
                              child: FlatButton.icon(
                                  color: Theme.of(context).primaryColor,
                                  onPressed: (){openBrowser( Constant.HRM_URL );},
                                  icon: Icon(Icons.file_download,size: 24,color: Colors.white,),
                                  label: Text('Import from Excel',
                                    style:  TextStyle(color: Colors.white),
                                  )
                              )
                          ),
                          Container(
                              padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                              child:Card(
                                color: Colors.grey[50],
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      Container(
                                        margin: const EdgeInsets.only(left: 5.0, right: 5.0),
                                        child: _images[_selectedPhoto] == 'no'
                                            ? Icon(Icons.account_circle, color: Colors.grey[100], size: 150,)
                                            : Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                      height: 150,
                                                      child: decode( _images[_selectedPhoto] )
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                        child:Text(
                                          _photoLabel[_selectedPhoto],
                                          style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                                        ),
                                      ),
                                      Container(
                                          margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 10),
                                          child: new Wrap(
                                              spacing: 10,
                                              runSpacing: 10,
                                              children: <Widget>[
                                                _images[0] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode(_images[0]),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[1] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[1] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[2] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[2] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[3] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[3] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                              ]
                                          )
                                      ),
                                      _photoSelection(),
                                      Container(
                                        margin: const EdgeInsets.only(top: 0.0, bottom: 5.0),
                                        child: Wrap(
                                          spacing: 20,
                                          runSpacing: 20,
                                          //mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                          children: [
                                            _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_camera, tr(LocaleKeys.btnTakePhoto)),
                                            _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_library, tr(LocaleKeys.btnBrowse)),
                                            //_images[_selectedPhoto] == 'no'
                                            //? Container() :
                                            _buildButtonColumn(_images[_selectedPhoto] == 'no' ? Colors.grey[100] : Theme.of(context).primaryColor, Icons.delete_forever, tr(LocaleKeys.btnRemove)),
                                          ],
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 20.0, right: 5.0),
                                        child:Text(
                                          'Photo must be included clear face, it should be taken with camera directly. It should NOT be scanned from paper or any media.',
                                          style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                                        ),
                                      ),
                                      new Container(
                                        margin: const EdgeInsets.only(left: 15.0, right: 15.0, bottom: 30),
                                        child: formData(context),
                                      ),

                                      //mainBodySecondWidgets(context),
                                    ]
                                ),
                              )
                          ),
                        ]
                    )
                )
            )
        )
    );
  }

  Widget _photoSelection( ){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, left: 0.0),
      child:new Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          new Text(
            tr(LocaleKeys.lblPhoto),
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 0,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value; });
            },
          ),
          new Text(
            'Profile',
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 1,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '1',
            style: new TextStyle(
              fontSize: 16.0,
            ),
          ),new Radio(
            value: 2,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '2',
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 3,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '3',
            style: new TextStyle(
              fontSize: 16.0,
            ),
          ),
        ],
      ),
    );
  }

  Widget topWidgets( BuildContext context) {
    return Container(
        color: Colors.white,
        padding: const EdgeInsets.only(left: 5.0, top: 0.0),
        height: 80,

        child: ListView(
          scrollDirection: Axis.horizontal,
          children: <Widget>[
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                color: Colors.grey[50],
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '1',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '2',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '3',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),

            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '4',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),

          ],
        )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel' || label == 'Close' || label == tr(LocaleKeys.btnCancel) ){
              Navigator.pop( context );
            }else{
              _submitForm( );
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  Column _buildButtonColumn(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        ClipOval(
            child: Material(
              color: color, // button color
              child: InkWell(
                splashColor: Colors.red, // inkwell color
                child: SizedBox(width: 36,
                    height: 36,
                    child: Icon(icon, color: Colors.white)
                ),
                onTap: () async{
                  if( label == tr(LocaleKeys.btnTakePhoto)){
                    await _getImage(true);
                  }else if( label == tr(LocaleKeys.btnBrowse)){
                    await _getImage(false);
                  }else if( label == tr(LocaleKeys.btnRemove)){
                    setState(() {
                      _images[_selectedPhoto] = 'no';
                    });
                  }
                },
              ),
            )
        ),
        Container(
          margin: const EdgeInsets.only(top: 5),
          child: Text(
            label,
            style: TextStyle(
              fontSize: 14,
              fontWeight: FontWeight.w400,
              color: color
            ),
          ),
        ),
      ],
    );
  }

  Form formData( BuildContext context ){

    return new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: new Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.person),
                  hintText: 'Employee/Student name',
                  labelText: tr(LocaleKeys.lblName),
                ),
                controller: _controller_name,
                validator: (val) => val.isEmpty ? 'Name is required' : null,
                onSaved: (val) => _name = val,
              ),
              Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child: new TextFormField(
                  decoration: InputDecoration(
                    icon: const Icon(Icons.phone),
                    hintText: '09 xxx xxx xxx',
                    labelText: tr(LocaleKeys.lblEmployeeMobileOptional),
                  ),
                  controller: _controller_phone,
                  keyboardType: TextInputType.phone,
                  validator: (val) => val.isEmpty ? 'Employee mobile number is required' : (isValidPhoneNumber(val) ? null : 'Please enter a valid phone number'),
                ),
              ),
              Container(
                margin: const EdgeInsets.only(left: 20.0, right: 5.0),
                child:Text(
                  'Employee mobile number will be validated on first time login with face recognition.',
                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                ),
              ),
              Container(
              margin: const EdgeInsets.only(top: 5.0, left: 5.0, right: 5.0),
                child:new Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: <Widget>[
                    new Text(
                      tr(LocaleKeys.lblGender),
                    style: new TextStyle( fontWeight: FontWeight.bold,),
                    ),
                    Expanded(
                      child: Center(
                        child:
                        RadioListTile(
                          value: 'Male',
                          groupValue: _gender,
                          title: Text(tr(LocaleKeys.lblMale)),
                          onChanged: ( value){
                            setState(() { _gender = value; });
                          },
                        )
                      )
                    ),
                    Expanded(
                        child: Center(
                            child:
                            RadioListTile(
                              value: 'Female',
                              groupValue: _gender,
                              title: Text(tr(LocaleKeys.lblFemale)),
                              onChanged: ( value){
                                setState(() { _gender = value; });
                              },
                            )
                        )
                    ),
                  ],
                ),
              ),
              Container(
                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5.0),
                child:
                new Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: <Widget>[
                    new Checkbox(
                      value: _hasMore,
                      onChanged: ( value){setState(() { _hasMore = value; });},
                    ),
                    new Text(
                      tr(LocaleKeys.lblMoreOptions),
                      style: new TextStyle(fontWeight: FontWeight.bold),
                    ),
                  ],
                ),
              ),
              _hasMore? Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child: new TextFormField(
                  decoration: InputDecoration(
                    icon: const Icon(Icons.perm_identity_sharp),
                    labelText: tr(LocaleKeys.lblEmployeeIdOptional),
                  ),
                  controller: _controller_employeeId,
                  onSaved: (val) => _employeeId = val,
                ),
              ) : Container(),
              _hasMore? _jobTitleSelection(context) : Container(),
              _hasMore? _deptSelection(context) : Container(),
              _hasMore? _rolesSelection(context) : Container(),
              _hasMore? _supervisorSelection(context) : Container(),
              _hasMore? _approver1Selection(context) : Container(),
              _hasMore? _approver2Selection(context) : Container(),
              _hasMore? _approver3Selection(context) : Container(),
              _hasMore? Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child:new Row(children: <Widget>[
                  new Expanded(
                      child: new TextFormField(
                        enabled: false,
                        decoration: new InputDecoration(
                          icon: const Icon(Icons.timer),
                          hintText: 'HH:mm',
                          labelText: tr(LocaleKeys.lblDutyInTimeOptional),
                        ),
                        controller: _controller_inTime,
                        keyboardType: TextInputType.datetime,
                      )),
                  new IconButton(
                    icon: new Icon(Icons.mode_edit),
                    tooltip: 'Choose time',
                    onPressed: (() {
                      DatePicker.showTimePicker(
                          context,
                          showTitleActions: true,
                          onChanged: (date) { },
                          onConfirm: (date) {
                            inDate = date;
                            _controller_inTime.text = _dateFormatTimeDisplay.format(date);
                          },
                          currentTime: inDate
                      );
                    }),
                  )
                ]),
              ) : Container(),
              _hasMore? Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child:new Row(children: <Widget>[
                  new Expanded(
                      child: new TextFormField(
                        enabled: false,
                        decoration: new InputDecoration(
                          icon: const Icon(Icons.access_time),
                          hintText: 'HH:mm',
                          labelText: tr(LocaleKeys.lblDutyOutTimeOptional),
                        ),
                        controller: _controller_outTime,
                        keyboardType: TextInputType.datetime,
                      )),
                  new IconButton(
                    icon: new Icon(Icons.mode_edit),
                    tooltip: 'Choose time',
                    onPressed: (() {
                      DatePicker.showTimePicker(
                          context,
                          showTitleActions: true,
                          onChanged: (date) { },
                          onConfirm: (date) {
                            outDate = date;
                            _controller_outTime.text = _dateFormatTimeDisplay.format(date);
                          },
                          currentTime: outDate
                      );
                    }),
                  )
                ]),
              ) : Container(),
              _hasMore? Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child: new TextFormField(
                  decoration: InputDecoration(
                    icon: const Icon(Icons.email),
                    hintText: 'name@domain.com',
                    labelText: tr(LocaleKeys.lblEmployeeEmailOptional),
                  ),
                  controller: _controller_email,
                  keyboardType: TextInputType.emailAddress,
                  validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                ),
              ) : Container(),
              _hasMore? Container(
                margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                child:new Row(children: <Widget>[
                  new Expanded(
                      child: new TextFormField(
                        decoration: new InputDecoration(
                          icon: const Icon(Icons.calendar_today),
                          hintText: 'yyyy-mm-dd',
                          labelText: tr(LocaleKeys.lblEmployeeJoinedOptional),
                        ),
                        controller: _controller_joinedDate,
                        keyboardType: TextInputType.datetime,
                      )),
                  new IconButton(
                    icon: new Icon(Icons.mode_edit),
                    tooltip: 'Choose date',
                    onPressed: (() {
                      _chooseDate(context, _controller_joinedDate.text );
                    }),
                  )
                ]),
              ) : Container(),
              _hasMore? _payGradeSelection(context) : Container(),

              Container(
                margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    _buildButtons( Theme.of(context).primaryColor, Icons.add, tr(LocaleKeys.btnAdd) ),
                    _buildButtons( Colors.transparent, Icons.cancel, _cancelLabel),
                  ],
                ),
              )
            ])
    );
  }

  Widget _jobTitleSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<JobTitleItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _jobTitleItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (JobTitleItem i, JobTitleItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblJobTitle),
          onFind: (String filter) => _jobTitleSuggestion(filter),
          onChanged: (JobTitleItem data) {
            _jobTitleItem = data;
          },
          dropdownBuilder: (BuildContext context, JobTitleItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.event_seat), title: Text(tr(LocaleKeys.lblJobTitleSelect)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, JobTitleItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<JobTitleItem>> _jobTitleSuggestion(String query) async{
    List<JobTitleItem> matches = List();
    if( _jobTitleListResponse.jobtitles == null)
      return matches;
    matches.addAll(_jobTitleListResponse.jobtitles);

    matches.retainWhere((s) {
      JobTitleItem jobTitleItem = s;
      if( jobTitleItem.name.toLowerCase().contains(query.toLowerCase()) || jobTitleItem.code.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _deptSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<DepartmentSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _departmentSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (DepartmentSearchItem i, DepartmentSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblDepartmentOptional),
          onFind: (String filter) => _deptSuggestion(filter),
          onChanged: (DepartmentSearchItem data) {
            _departmentSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, DepartmentSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text(tr(LocaleKeys.lblDepartmentSelect)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, DepartmentSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<DepartmentSearchItem>> _deptSuggestion(String query) async{
    List<DepartmentSearchItem> matches = List();
    if( _departmentSearchResponse.departments == null)
      return matches;

    matches.addAll(_departmentSearchResponse.departments);

    matches.retainWhere((s) {
      DepartmentSearchItem dept = s;
      if( dept.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _supervisorSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblSupervisorOptional),
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _employeeSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text(tr(LocaleKeys.lblSupervisorSelect)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }

  Widget _approver1Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver1,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblApprover1Optional),
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver1 = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text(tr(LocaleKeys.lblApprover1Select)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Widget _approver2Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver2,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblApprover2Optional),
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver2 = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text(tr(LocaleKeys.lblApprover2Select)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Widget _approver3Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver3,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblApprover3Optional),
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver3 = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text(tr(LocaleKeys.lblApprover3Select)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }

  Future<List<EmployeeSearchItem>> _supervisorSuggestion(String query) async{
    List<EmployeeSearchItem> matches = List();
    if( _employeeSearchResponse.employees == null)
      return matches;
    matches.addAll(_employeeSearchResponse.employees);

    matches.retainWhere((s) {
      EmployeeSearchItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _rolesSelection( BuildContext context){
    return Container(
      margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<RoleItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _roleItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (RoleItem i, RoleItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblRoleOptional),
          onFind: (String filter) => _rolesSuggestion(filter),
          onChanged: (RoleItem data) {
            _roleItem = data;
          },
          dropdownBuilder: (BuildContext context, RoleItem item ) =>
              Container(
                  child: (item?.name == null) ?
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    leading: Icon(Icons.security), title: Text(tr(LocaleKeys.lblRoleSelect)),
                  )
                      : ListTile(
                    contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  ),
            ),
          popupItemBuilder: (BuildContext context, RoleItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
      );
  }
  Future<List<RoleItem>> _rolesSuggestion(query) async {
    List<RoleItem> matches = List();
    if( _roleListResponse.roles == null)
      return matches;
    matches.addAll(_roleListResponse.roles);
    matches.retainWhere((s) {
      RoleItem roleItem = s;
      if( roleItem.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _payGradeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<PayGradeItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _payGradeItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (PayGradeItem i, PayGradeItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployeePayGradeOptional),
          onFind: (String filter) => _payGradeSuggestion(filter),
          onChanged: (PayGradeItem data) {
            _payGradeItem = data;
          },
          dropdownBuilder: (BuildContext context, PayGradeItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.monetization_on_outlined), title: Text(tr(LocaleKeys.lblEmployeePayGradeSelect)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, PayGradeItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<PayGradeItem>> _payGradeSuggestion(String query) async{
    List<PayGradeItem> matches = List();
    if( _payGradeResponse.items == null)
      return matches;

    matches.addAll( _payGradeResponse.items );

    matches.retainWhere((s) {
      PayGradeItem dept = s;
      if( dept.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void _submitForm() {
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      if( _controller_name.text.isEmpty)
        showMessage(tr(LocaleKeys.errorNameRequired));
      else
        showMessage(tr(LocaleKeys.errorMobileNumberRequired));
    } else {
      form.save();

      if( _images[0]=='no') {
        showMessage(tr(LocaleKeys.errorProfileImageRequired));
        setState(() {
          _selectedPhoto =0;
        });
        return;
      }
      if( _images[1]=='no') {
        showMessage(tr(LocaleKeys.errorOnePhotoRequired));
        setState(() {
          _selectedPhoto =1;
        });
        return;
      }

      if( _gender == null){
        showMessage(tr(LocaleKeys.errorGenderRequired));
        return;
      }
      _doRegister();
    }
  }

  Future<String> _employeeRegister( String userGUID ) async{
    Map<String, dynamic> employeeData = {
      "departmentGUID":_departmentSearchItem ==null? _currentUser.organizationGUID: _departmentSearchItem.guid,
      "supervisorGUID": _employeeSearchItem ==null? _currentUser.employeeGUID : _employeeSearchItem.guid,
      "employeeId": _controller_employeeId.text.isEmpty?null : _controller_employeeId.text,
      "firstName": _name.contains(" ")? _name.substring(0, _name.lastIndexOf(" ")) : _name,
      "middleName": null,
      "lastName": _name.contains(" ")? _name.substring(_name.lastIndexOf(" ") +1 ) : null,
      "gender": _gender,
      "jobTitleGUID": _jobTitleItem==null? _controller_jobTitle.text : _jobTitleItem.guid,
      "userGUID": userGUID,
      "approver1GUID": _approver1==null ? null : _approver1.guid,
      "approver2GUID": _approver2==null ? null : _approver2.guid,
      "approver3GUID": _approver3==null ? null : _approver3.guid,
      "dutyInTime": _dateFormatTimeOutput.format(inDate),
      "dutyOutTime": _dateFormatTimeOutput.format(outDate),
      "joinedDate": _joinedDate==null? null : new DateFormat('yyyy-MM-dd').format(_joinedDate.toLocal()),
      "mobilePhone": _controller_phone.text.isEmpty?null : _controller_phone.text,
      "privateEmail": _controller_email.text.isEmpty?null : _controller_email.text,
      "payGradeGuid": _payGradeItem ==null? null: _payGradeItem.guid,
    };

    return await _employeeService.employeeRegister(employeeData, _currentUser.facialGUID);

  }

  Future<void> _doRegister() async{
    if( _processing ) return;
    setState(() {
      _processing = true;
      progressColor = Colors.white;
    });

    if( _registeredFaceGUID == null){
      _getAccessToken( false);
      FaceRecognizationService recognizationService = new FaceRecognizationService();
      _registeredFaceGUID = await recognizationService.faceRegister(
          _images,
          _name,
          _currentUser.organizationGUID,
          _accessToken,
          FaceRequest.TYPE_ADD
        );
      }

    bool isSameWithLoginUser = false;
    if( _registeredFaceGUID !=null && _registeredFaceGUID == _currentUser.facialGUID){
      _registeredUserGUID = _currentUser.guid;
      isSameWithLoginUser = true;

    }else if( _registeredUserGUID == null && _registeredFaceGUID!=null){
        String orgName = _currentUser.organizationName.replaceAll(RegExp(r"[^a-zA-Z0-9]"), "");
        Random random = new Random();
        int randomNumber = random.nextInt(1000);
        String username = _name.toLowerCase().replaceAll(RegExp(r"[^a-zA-Z0-9]"), "")+ (randomNumber.toString().padLeft(3,'0')) +'@'+orgName;
        _registeredUserGUID = await _userService.userRegister(
            _registeredFaceGUID,
            _name,
            _controller_email.text.isEmpty?null : _controller_email.text,
            _controller_phone.text.isEmpty?null : _controller_phone.text,
            _roleItem==null ? null : _roleItem.guid,
            username.toLowerCase( ),
            null,
            _accessToken
        );
    }

    if( _registeredEmployeeGUID == null && _registeredUserGUID!=null)
      _registeredEmployeeGUID = await _employeeRegister(_registeredUserGUID);

    if( _registeredEmployeeGUID == null){
      showMessage('Failed to add new employee! please try again');
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
      });
    }else{

      if( isSameWithLoginUser ){
        EmployeeService employeeService = new EmployeeService();
        String response = await employeeService.activeDefault( _currentUser.organizationGUID, _currentUser.facialGUID);
        await _currentUser.fromJsonEmployee(response, true);
      }

      new ThreadTimer(new Duration(milliseconds: 1000), () {
        Navigator.pop(context, _registeredEmployeeGUID);
      });
    }
  }

  Future<void> _getImage( bool isCamera) async {
    _formKey.currentState.save();
    setState(() {progressColor = Colors.white;});

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/Pictures/flutter_test';
        await Directory(tmpDir).create(recursive: true);

        if( _selectedPhoto == 0 ){
          await _addImage(tmpDir, image.path);
        }else{
          // detecting face
          DetectionResult detectionResult = await faceDetectorService.detectedFaceAsBase64String(tmpDir, image.path, mounted, false, false);
          if( detectionResult == null )
            showMessage('No face detected on image');
          else{
            setState(() {_images[_selectedPhoto] = detectionResult.detectedFace;});
          }
        }
      }
    }catch(e){
      showMessage('Error occur, please select another photo');
      if( Constant.ENABLE_LOGS)
        print(e);
    }

    setState(() {progressColor = Colors.transparent;});
  }

  Future<void> _addImage(String tmpDir, String path) async{
    String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
    File rotatedFile = await FlutterImageCompress.compressAndGetFile(
      path, photoPath2,
      quality: 90,
      rotate: 0,
    );
    imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 192);
//      if( isCamera)
//        thumbnail = imglib.flipHorizontal( thumbnail );
    String cropFilePath = '$tmpDir/cropped.jpg';
    File(cropFilePath)
      ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 90));

    File smallFile  =File(cropFilePath);
    String encodedString = encode( smallFile );
    smallFile.deleteSync(recursive: false);
    rotatedFile.deleteSync(recursive: false);
    setState(() {_images[_selectedPhoto] = encodedString;});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  bool isValidPhoneNumber(String input) {
    if( input== null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r'^[0-9]{9,13}$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  Future _chooseDate(BuildContext context, String initialDateString ) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(1950),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      _controller_joinedDate.text = new DateFormat('yyyy-MM-dd').format(result);
        _joinedDate = result;
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }


  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  void dispose() {
    try {
      faceDetectorService.dispose();
    }catch(e){}
    super.dispose();
  }
}