
import 'dart:async';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/view/attendance/attendance_search.dart';
import '/hrm/view/more/geofencing/geofencing_list.dart';
import '/hrm/view/profile/profile_home.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'add_user.dart';


class EmployeeList extends StatefulWidget {
  EmployeeList({Key key, this.showAddButton}) : super(key: key);
  final bool showAddButton;
  @override
  _EmployeeListState createState() => _EmployeeListState();
}

class _EmployeeListState extends State<EmployeeList> {

  bool _search = false;
  bool _onSearch = false;
  final _searchQuery = new TextEditingController();
  Timer _debounce;
  String _searchText='';

  @override
  void initState() {
    super.initState();
    _searchQuery.addListener(_onSearchChanged);
  }

  @override
  void deactivate() {
    super.deactivate( );
  }

  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    _debounce.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {

    return Scaffold(
      appBar: AppBar(
        title: new Center(
            child: Container(
              //padding: const EdgeInsets.only(left: 50),
                child: new Text(
                    tr(LocaleKeys.mnuEmployees)
                )
            )
        ),
        elevation: 0,
        bottom: _search? PreferredSize(
            preferredSize: Size(double.infinity, 50.0),
            child: SizedBox(
                height: 50.0,
                child: new TextField(
                  decoration: new InputDecoration(
                    hintText: 'Employee name',
                    hintStyle: TextStyle(color: Colors.white54),
                    prefixIcon: const Icon(Icons.search, color: Colors.white,),
                  ),
                  keyboardType: TextInputType.text,
                  style: TextStyle(color: Colors.white),
                  controller: _searchQuery,
                  cursorColor: Colors.white,
                )
            )
        ): PreferredSize(
            preferredSize: Size(double.infinity, 0.0),
            child: SizedBox(
                height: 0.0,
                child: Container()
            )
        ),
        actions: <Widget>[
          FlatButton.icon(
              onPressed:(){
                setState(() {
                  if( _search ){
                    _searchText = '';
                    _searchQuery.text = '';
                  }
                  _search = !_search;
                });
              },
              icon: _search? Icon(Icons.cancel, color: Colors.white,)
                  : Icon(Icons.search, color: Colors.white,),
              label: Text(
                _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                style:  TextStyle(color: Colors.white),
              )
          )
        ],
      ),
      body: _createBody( context ),
      floatingActionButton: widget.showAddButton? FloatingActionButton.extended(
        onPressed: () async{
          print('Add user');
          String _registeredEmployeeGUID = await Navigator.push(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => AddUser(
                    title: tr(LocaleKeys.titleNewEmployee)
                ),
              )
          );
          if( _registeredEmployeeGUID != null){
            //TODO load the employee list
          }
          setState(() {
            _search = !_search;
            print(_search);
          });

          new ThreadTimer(new Duration(milliseconds: 1000), () {
            setState(() {
              _search = !_search;
              print(_search);
            });
          });
        },
        label: Text(
          tr(LocaleKeys.btnAddNewEmployee),
          style: TextStyle(color: Colors.white),
        ),
        icon: Icon(Icons.person_add),
        backgroundColor: Theme.of(context).primaryColor,
      ) : new Container(),
    );
  }

  Widget _createBody(BuildContext context){
    if(!_onSearch) {
      return EmployeeListHome(
        searchText: _searchText,
        onLoad: (){
          try{
            FocusScope.of(context).unfocus();
          }catch(e){}
        },
      );
    }else{
      return Container();
    }
  }

  _onSearchChanged() {
    setState(() {
      _onSearch = true;
    });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(milliseconds: 700), () {
      _searchText = _searchQuery.text.trim();
      setState(() {
        _onSearch = false;
      });
    });
  }

}

class EmployeeListHome extends StatefulWidget {
  EmployeeListHome({Key key, this.searchText, this.onLoad}) : super(key: key);
  final String searchText;
  final Function onLoad;

  @override
  _EmployeeListHomeState createState() => _EmployeeListHomeState();
}

class _EmployeeListHomeState extends State<EmployeeListHome>{

  int _page = 0;
  String _searchText= '';

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  EmployeeSearchResponse _employeeSearchResponse;
  String _accessToken;
  EmployeeService _employeeService = new EmployeeService( );
  User _currentUser = new User();
  List<EmployeeSearchItem> _employees = [];
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();
    _init();
    _searchText= widget.searchText ?? '';
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _employeeSearchResponse = await _employeeService.search(_accessToken, _searchText, _page,  Constant.ROW_PER_PAGE);
    if( _employeeSearchResponse!=null && _employeeSearchResponse.employees!=null){

      if( refresh && _employees.isNotEmpty)
        _employees.removeRange(0, _employees.length);

      _employeeSearchResponse.employees.forEach((element) {
        _employees.add(element);
      });
    }

    widget.onLoad();
    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => EmployeeResultItem(
        post: _employees[i],
        accessToken: _accessToken,
        sharedPrefs: _sharedPrefs,
        isEnd: i+1==_employees.length,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _employees.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class EmployeeResultItem extends StatefulWidget {

  EmployeeResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.isEnd}) : super(key: key);
  final EmployeeSearchItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final bool isEnd;


  @override
  _EmployeeResultItemState createState() => _EmployeeResultItemState();

}

class _EmployeeResultItemState extends State<EmployeeResultItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
          children: <Widget>[
            Card(
              elevation: 0,
              child: Container(
              //padding: EdgeInsets.only(top:10),
                  child: ListTile(
                    title: Text(widget.post.name, style: TextStyle(color: Theme.of(context).primaryColor),),
                    subtitle: Text(
                        '${widget.post.jobTitleName==null?'' : widget.post.jobTitleName+','} ${widget.post.departmentName==null?'' : widget.post.departmentName+" Dept"}'
                    ),
                    leading: ClipOval(
                        child: Material(
                          elevation: 1,
                          color: Colors.transparent, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 48,
                              height: 48,
                              child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.faceGuid}',
                                headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
                            ),
                          ),
                        )
                    ),
                    trailing: PopupMenuButton<String>(
                      onSelected: (String value) {
                        if( value == 'Profile'){
                          _gotoProfile();
                        }else if( value == 'Zone'){
                          _gotoGeofencing();
                        }else if( value == 'Attendance'){
                          _gotoAttendanceSearch(widget.post.guid);
                        }
                      },
                      child: Icon(Icons.menu, color: Theme.of(context).primaryColor, ),
                      itemBuilder: (BuildContext context) => <PopupMenuEntry<String>>[
                        const PopupMenuItem<String>(
                          value: 'Profile',
                          child: Text('Profile'),
                        ),
                        const PopupMenuItem<String>(
                          value: 'Zone',
                          child: Text('Mange attendance zones'),
                        ),
                        const PopupMenuItem<String>(
                          value: 'Attendance',
                          child: Text('Attendance list'),
                        ),
                      ],
                    ),
                    onTap: (){
                      _gotoProfile();
                    },
                  )
                )
            ),
            widget.isEnd? SizedBox(
              height: 150,
            ) : Container()
          ]
        )
    );

  }

  _gotoProfile(){

    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => ProfileHome(
            userGUID: widget.post.userGUID,
            employeeGUID: widget.post.guid,
            showExit: false,
            sharedPrefs: widget.sharedPrefs,
          ),
        )
    );
  }

  void _gotoGeofencing(){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => GeofencingListHome(),
        )
    );
  }

  void _gotoAttendanceSearch(String employeeGuid){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => AttendanceSearch(
            employeeGuid: employeeGuid,
          ),
        )
    );
  }
}