
import 'dart:async';
import 'dart:io';
import 'package:badges/badges.dart';
import '/hrm/config/constant.dart';
import '/hrm/database/dao/notification_dao.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/leave_search_request.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/leave_search_response.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/service/leave_service.dart';
import '/hrm/service/notification_service.dart';
import '/hrm/service/organization_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import '/hrm/view/employee/add_user.dart';
import '/hrm/view/attendance/latest_for_organization.dart';
import '/hrm/view/employee/employee_list_home.dart';
import '/hrm/view/attendance/face_verify.dart';
import '/hrm/view/leave/leave_home.dart';
import '/hrm/view/more/more_details.dart';
import '/hrm/view/login/pre_login.dart';
import '/hrm/view/organization/activate_organization.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:easy_localization/easy_localization.dart';
import 'package:threading/threading.dart';
import 'user_validate.dart';



class Home extends StatefulWidget {
  Home({Key key, this.title}) : super(key: key);
  final String title;
  @override
  _HomeState createState() => _HomeState();
}

class _HomeState extends State<Home> {
  final dateFormatDMY = new DateFormat('dd MMMMd yyyy');
  int _selectedIndex = 0;
  SharedPreferences _prefs;
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  NotificationDao notificationDao = new NotificationDao();
  int _notificationCount = 0;
  int _pendingLeaveCount = 0;
  bool _isFirstOpen = true;
  bool _notificationRefreshing = false;
  bool _shutterSoundNotice = true;
  String topTitle = '${new DateFormat('EEEE').format(new DateTime.now()).toUpperCase()}';
  bool showColumn = int.parse(new DateFormat('ss').format(new DateTime.now())) % 2 == 0;
  bool isStopped = false;
  bool _search = false;
  bool _onSearch = false;
  final _searchQuery = new TextEditingController();
  Timer _debounce;
  String _searchText='';

  @override
  void initState() {
    super.initState();
    Timer.periodic(Duration(seconds: 1), (timer) {
      if (isStopped ) {
        timer.cancel();
      }
      showColumn = int.parse(new DateFormat('ss').format(new DateTime.now())) % 2 == 0;
      if( new DateTime.now().millisecondsSinceEpoch % 3 == 0)
        _refreshNotifications();
      try{
        setState(() { });
      }catch(e){}

    });
    _searchQuery.addListener(_onSearchChanged);
    _loadProfile();
  }

  @override
  void deactivate() {
    isStopped=true;
    super.deactivate( );
  }

  @override
  void dispose() {
    _searchQuery.removeListener(_onSearchChanged);
    _searchQuery.dispose();
    _debounce.cancel();
    super.dispose();
  }

  void _loadProfile( )async{
    _prefs = await SharedPreferences.getInstance( );
    if( _prefs.getString(Constant.CURRENT_USER_DATA) == null ){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => LoginSelection(
              title: tr(LocaleKeys.login),
            ),
          )
      );
    }
    await _currentUser.fromSharedPreferences();
    await _authorityResponse.fromSharedPreferences();

    if( Platform.isIOS)
      _shutterSoundNotice = await getCameraShutterSoundNotice( );

    setState(() { });
    _checkOrganizationValidity( );
    _checkLeavePendingCount();
  }

  @override
  Widget build(BuildContext context) {

    return Scaffold(
          appBar: AppBar(
            title: new Center(
                child: Container(
                    //padding: const EdgeInsets.only(left: 50),
                    child: new Text(
                    '$topTitle'
                  )
                )
            ),
            elevation: 0,
            bottom: _search? PreferredSize(
              preferredSize: Size(double.infinity, 50.0),
                child: SizedBox(
                  height: 50.0,
                    child: new TextField(
                      decoration: new InputDecoration(
                        hintText: 'Employee name',
                        hintStyle: TextStyle(color: Colors.white54),
                        prefixIcon: const Icon(Icons.search, color: Colors.white,),
                      ),
                      keyboardType: TextInputType.text,
                      style: TextStyle(color: Colors.white),
                      controller: _searchQuery,
                      cursorColor: Colors.white,
                    )
                )
            ): (_selectedIndex==0? PreferredSize(
              preferredSize: Size(double.infinity, 90.0),
                child: SizedBox(
                    height: 90.0,
                    child: _header(context)
                )
            ):
            PreferredSize(
                preferredSize: Size(double.infinity, 0.0),
                child: SizedBox(
                    height: 0.0,
                    child: Container()
                )
            )),
            actions: <Widget>[
              _selectedIndex!=3? new Container() : FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      if( _search ){
                        _searchText = '';
                        _searchQuery.text = '';
                      }
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel, color: Colors.white,)
                      : Icon(Icons.search, color: Colors.white,),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
      body: _createBody( context ),
      bottomNavigationBar: BottomNavigationBar(
        items: <BottomNavigationBarItem>[
          BottomNavigationBarItem(
            icon: Icon(Icons.home),
            label: tr(LocaleKeys.mnuHome),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.alarm_on),
            label: tr(LocaleKeys.mnuInOut),
          ),
          BottomNavigationBarItem(
            icon: _authorityResponse.authorities!=null
                && _authorityResponse.authorities.contains('SUPERVISOR_WRITE')
                && _pendingLeaveCount>0? Badge(
                position: BadgePosition.topEnd(top: -20, end: -20),
                badgeContent: Text('$_pendingLeaveCount',style: TextStyle(color: Colors.white),),
                toAnimate: true,
                animationType: BadgeAnimationType.slide,
                child: Icon(Icons.drafts)
            ) : Icon(Icons.drafts),
            label: tr(LocaleKeys.mnuLeave),
          ),
          BottomNavigationBarItem(
            icon: Icon(Icons.group),
            label: tr(LocaleKeys.mnuEmployees),
          ),
          BottomNavigationBarItem(
            icon: _notificationCount>0? Badge(
              position: BadgePosition.topEnd(top: -20, end: -20),
              badgeContent: Text('$_notificationCount',style: TextStyle(color: Colors.white),),
              toAnimate: true,
              animationType: BadgeAnimationType.slide,
              child: Icon(Icons.menu)
            ) : Icon(Icons.menu),
            label: tr(LocaleKeys.mnuMore),
          ),
        ],
        currentIndex: _selectedIndex,
        selectedItemColor: Theme.of(context).primaryColor,
        unselectedItemColor: Colors.grey[700],
        showUnselectedLabels: true,
        showSelectedLabels: true,
        onTap: _onItemTapped,
      ),
      floatingActionButton:
      _selectedIndex==3 && _authorityResponse.authorities!=null
          && _authorityResponse.authorities.contains('MANAGER_WRITE') ?FloatingActionButton.extended(
        onPressed: () async{
          String _registeredEmployeeGUID = await Navigator.push(
              context,
              MaterialPageRoute(
                builder: (BuildContext context) => AddUser(
                  title: tr(LocaleKeys.titleNewEmployee)
                ),
              )
          );
          if( _registeredEmployeeGUID != null){
            setState(() {
              _onSearch = true;
            });
            new ThreadTimer(new Duration(milliseconds: 500), () {
              setState(() {
                _onSearch = false;
              });
            });
          }
        },
        label: Text(
          tr(LocaleKeys.btnAddNewEmployee),
          style: TextStyle(color: Colors.white),
        ),
        icon: Icon(Icons.person_add),
        backgroundColor: Theme.of(context).primaryColor,
      ) : new Container(),
    );
  }

  void _onItemTapped(int index) async{
    if( !await _show2FADialog(context)){
      return;
    }

    setState(() {
      _selectedIndex = index;
      if( _selectedIndex == 0){
        topTitle = '${new DateFormat('EEEE').format(new DateTime.now()).toUpperCase()}';
      }else if( _selectedIndex == 3){
        topTitle = tr(LocaleKeys.mnuEmployees);
      }else if( _selectedIndex == 4){
        topTitle = tr(LocaleKeys.mnuMore);
      }

      if( _selectedIndex != 3){
        _search = false;
        _searchQuery.text = '';
      }
    });

    if( _selectedIndex == 1 ){
      if( Platform.isIOS && _shutterSoundNotice ) {
        bool dont = await _showDialog(context);
        _shutterSoundNotice = !dont;
        if (dont) {
          await setCameraShutterSoundNotice(false);
        }
      }
      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => FaceVerify(),
          )
      );
      setState(() {
        _selectedIndex = 0;
        topTitle = '${new DateFormat('EEEE').format(new DateTime.now()).toUpperCase()}';
      });

    }else if( _selectedIndex == 2 ){

      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => LeaveHome(
              pendingLeaveCount: _pendingLeaveCount,
              onChange: (pendingLeaveCount){
                setState(() {
                  _pendingLeaveCount = pendingLeaveCount;
                });
              },
            ),
          )
      );
      setState(() {
        _selectedIndex = 0;
        topTitle = '${new DateFormat('EEEE').format(new DateTime.now()).toUpperCase()}';
      });
    }
  }


  Widget _header(BuildContext context){
    return Container(
      height: MediaQuery.of(context).orientation == Orientation.portrait ?
      MediaQuery.of(context).size.height * 0.12 : MediaQuery.of(context).size.height * 0.21,
      width: MediaQuery.of(context).size.width,
      child: Column(
          children: <Widget>[
            Container(
              child: Text(
                '${new DateFormat('dd MMMM yyyy').format(new DateTime.now()).toUpperCase()}',
                style: TextStyle(
                    color: Colors.white,
                    fontWeight: FontWeight.normal,
                    fontSize: 18
                ),
                textAlign: TextAlign.center,
              ),
            ),
            Container(
                margin: const EdgeInsets.only(top:0, bottom: 0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Container(
                      child: Text(
                        showColumn?
                        '${new DateFormat('hh:mm').format(new DateTime.now())}'
                            :'${new DateFormat('hh mm').format(new DateTime.now())}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 48
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                    Container(
                      margin: EdgeInsets.only(top: 20),
                      child: Text(
                        '${new DateFormat('a').format(new DateTime.now()).toLowerCase()}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 24
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ],
                )
            ),
          ]
      ),
      color: Theme.of(context).primaryColor,
    );
  }

  Future<bool> _showDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Camera usage!"),
          content: new Text("You may hear camera shutter sound frequently. The purpose of preventing to access the hidden camera."),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Don't show again"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Widget _createBody(BuildContext context){
    if (_selectedIndex == 0){
      return LatestForOrganization(
        onRefresh: () {
          // _refreshNotifications();
          },
      );
    }else if(_selectedIndex == 3 && !_onSearch) {
        return EmployeeListHome(
          searchText: _searchText,
          onLoad: (){
            try{
              FocusScope.of(context).unfocus();
            }catch(e){}
          },
        );

    }else if(_selectedIndex == 4) {
      return MoreDetails(
        notificationCount: _notificationCount,
        onNotificationChange: (count){
          setState(() {
            _notificationCount = count;
          });
        },
      );
    }else{
      return new Container();
    }
  }

  _onSearchChanged() {
    setState(() {
      _onSearch = true;
    });
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(const Duration(milliseconds: 700), () {
      _searchText = _searchQuery.text.trim();
      setState(() {
        _onSearch = false;
      });
    });
  }

  void _refreshNotifications( )async{
    if( _notificationRefreshing ) return;
    _notificationRefreshing = true;

    if( _isFirstOpen ) {
      TokenService tokenService = new TokenService();
      String _accessToken = await tokenService.faceLogin(
          _currentUser.facialGUID, false);
      SharedPreferences sharedPrefs = await SharedPreferences.getInstance();
      String deviceGUID = sharedPrefs.getString(Constant.DEVICE_GUID);
      NotificationService notificationService = new NotificationService();
      bool hasUpdate = await notificationService.fetchAndUpdate(
          _accessToken, deviceGUID);
      _isFirstOpen = false;
    }

    await notificationDao.open( );
    int count = await notificationDao.unreadCount();
    await notificationDao.close();

    if( count != _notificationCount){
      setState(() {
        _notificationCount = count;
      });
    }

    _notificationRefreshing = false;
    setState(() {});
  }

  void _checkOrganizationValidity( ) async{

    try{
        TokenService tokenService = new TokenService( );
        String accessToken = await tokenService.faceLogin( _currentUser.facialGUID, true );
        OrganizationService organizationService = new OrganizationService();
        OrganizationDetailResponse detailResponse = await organizationService.detail(accessToken, _currentUser.organizationGUID);

        if( detailResponse != null && detailResponse.success){
          if( detailResponse.status == OrganizationDetailResponse.ORGANIZATION_STATUS_PENDING){
            if( _authorityResponse.authorities!=null
                && _authorityResponse.authorities.contains('ORGANIZATION_READ') ) {
              Navigator.pushReplacement(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) =>
                        ActivateOrganization(
                          organizationGuid: _currentUser.organizationGUID,
                          gotoLogin: true,
                        ),
                  )
              );
            }else{
              UserService userService = new UserService( );
              await userService.logout( );
              Navigator.pushReplacement(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) => LoginSelection(
                        title: tr(LocaleKeys.login)
                    ),
                  )
              );
            }
          }else if( detailResponse.status == OrganizationDetailResponse.ORGANIZATION_STATUS_INACTIVE){
            UserService userService = new UserService( );
            await userService.logout( );
            Navigator.pushReplacement(
                context,
                MaterialPageRoute(
                  builder: (BuildContext context) => LoginSelection(
                      title: tr(LocaleKeys.login)
                  ),
                )
            );
          }
        }

    }catch(e){}

  }

  void _checkLeavePendingCount() async{
    try{
      TokenService tokenService = new TokenService( );
      String accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
      LeaveService _leaveService = new LeaveService();
      LeaveSearchRequest _leaveSearchRequest = new LeaveSearchRequest();
      _leaveSearchRequest.first = 0;
      _leaveSearchRequest.max = 1;
      _leaveSearchRequest.status = LeaveSearchItem.STATUS_PENDING;
      LeaveSearchResponse _leaveSearchResponse = await _leaveService.search(accessToken, _leaveSearchRequest);
      if( _leaveSearchResponse.totalRecords !=null){
        setState(() {
          _pendingLeaveCount = _leaveSearchResponse.totalRecords;
        });
      }
    }catch(e){}
  }

  Future<bool> _show2FADialog( BuildContext context )async {

    if( ! await get2FA() ){
      return true;
    }
    bool result = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => UserValidation(
            facialGUID: _currentUser.facialGUID,
          ),
        )
    );
    return result;
  }

}