

import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/organizations_by_face.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/organization_service.dart';
import '/hrm/view/home/home.dart';
import 'package:flutter/material.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/generated/locale_keys.g.dart';
import 'package:shared_preferences/shared_preferences.dart';


class OrganizationListByFace extends StatefulWidget{
  OrganizationListByFace({Key key, @required this.organizationsByFace,
    @required this.accessToken, @required this.facialGUID, @required this.sharedPrefs}) : super(key: key);
  final OrganizationsByFace organizationsByFace;
  final String accessToken;
  final String facialGUID;
  final SharedPreferences sharedPrefs;

  @override
  _OrganizationsByFacePageState createState() => _OrganizationsByFacePageState();
}

class _OrganizationsByFacePageState extends State<OrganizationListByFace>{

  Color progressColor = Colors.transparent;
  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Text(tr(LocaleKeys.titleOrgSelection)),
          bottom: _createProgressIndicator(),
        ),
        body: ListView.builder(
          shrinkWrap: false,
          itemBuilder: (BuildContext ctxt, int index) {
            return Container(
                padding: const EdgeInsets.all(10),
                height: 100,
                child: Card(
                  color: Theme.of(context).primaryColor,
                  child: Container(
                    padding: EdgeInsets.only(top:10),
                      child: ListTile(
                    title: Text(
                      '${widget.organizationsByFace.organizations[index].name}',
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                    leading: ClipOval(
                        child: Material(
                          elevation: 2,
                          color: Colors.white, // button color
                          child: InkWell(
                            splashColor: Colors.red, // inkwell color
                            child: SizedBox(
                              width: 64,
                              height: 64,
                              child:
                              widget.organizationsByFace.organizations[index].guid!=null? new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/organization/download_logo/${widget.organizationsByFace.organizations[index].guid}',
                                headers: {'Authorization':'Bearer ${widget.accessToken}'}, )
                              :
                              Icon(Icons.group_work, size: 48,),
                            ),
                          ),
                        )
                    ),
                    trailing: Icon(Icons.arrow_forward_ios, color: Colors.white,),
                        onTap: (){_setDefaultOrganization(widget.organizationsByFace.organizations[index]);},
                  )
                )
                )
            );
          },
          itemCount: widget.organizationsByFace.organizations.length,
        ),
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  void _setDefaultOrganization(OrganizationForFace organizationForFace) async{

    if( _processing) return;
    setState(() {
      _processing = true;
      progressColor = Colors.white;
    });
    OrganizationService organizationService = new OrganizationService();
    User user = await organizationService.joinByFace(organizationForFace.guid, widget.facialGUID);
    EmployeeService employeeService = new EmployeeService();
    String response = await employeeService.activeDefault(organizationForFace.guid, widget.facialGUID);
    await user.fromJsonEmployee(response, true);
    setState(() {
      progressColor = Colors.transparent;
    });
    Navigator.pushReplacement(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => Home(
            title: '${Constant.APP_NAME}',
          ),
        )
    );
  }
}