
import '/hrm/config/constant.dart';
import '/hrm/service/firebase_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/validation_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:threading/threading.dart';



class UserValidation extends StatefulWidget {
  UserValidation({Key key, @required this.facialGUID}) : super(key: key);
  final String facialGUID;

  @override
  _UserValidationState createState() => _UserValidationState( );
}

class _UserValidationState extends State<UserValidation> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  Color progressColor = Colors.transparent;
  String _code;
  bool _codeSent = false;
  int _resendCount = 0;
  IconData _topIcon = Icons.help_outline;

  @override
  void initState() {
    super.initState();
    _checkFireBaseRegistration();
    _resend( context );
  }

  void _checkFireBaseRegistration() async{
    try{
      if( await getFirebaseToken( ) == null){
        FirebaseMessage _firebaseMessage = FirebaseMessage( );
        await _firebaseMessage.initMessaging( );
      }
    }catch(e){}
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).primaryColor,
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Validation'),
            bottom: _createProgressIndicator(),
          ),
          body: Stack(
            fit: StackFit.passthrough,
            children: [
              _backgroundPanel( true ),
              Center(
                  child: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  color: Color.fromRGBO(250, 250, 250, .4),
                                  elevation: 1,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                          margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                          child: Text('Validation',
                                              style: TextStyle(
                                                  color: Theme.of(context).primaryColor,
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 18
                                              )),
                                        ),
                                        Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                    height: 150,
                                                    child: Icon( _topIcon, size: 100, color: Theme.of(context).primaryColor,),
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                          child: formData(context),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )
              )
            ]
          )
        )
    );
  }

  Form formData(BuildContext context ){
    return new Form(
        key: _formKey,
        autovalidate: true,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.done),
                  hintText: '6 digits code',
                  labelText: 'Validation code',
                ),
                validator: (val) => val.isEmpty ? 'Code required' : null,
                onSaved: (val) => _code = val,
                keyboardType: TextInputType.number,
                maxLength: 6,
                autofocus: true,
              ),
              Container(
                margin: const EdgeInsets.only(left: 20.0, right: 5.0, top: 10),
                child:Text(
                  "Code already sent to your mobile phone via SMS. If you didn't receive yet, press again on \"Resend\" button.",
                  style: new TextStyle(fontSize: 12.0, color: Colors.black87),
                ),
              ),
               new Wrap(
                children: [
                  Container(
                    padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                    child: RaisedButton.icon(
                      onPressed:_processing?null: () {_submitForm(context); },
                      elevation: 1,
                      icon: Icon(Icons.done, color: Colors.white),
                      label: Text(
                        'Validate',
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  ),
                  Container(
                    padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                    child: RaisedButton.icon(
                      onPressed:_codeSent?null: () {_resend(context); },
                      elevation: 1,
                      icon: Icon(Icons.refresh, color: Colors.white),
                      label: Text(
                        'Resend',
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  ),
                ]
               ),
            ])
    );
  }

  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
          image:
          MediaQuery.of(context).orientation == Orientation.portrait ?
          new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
              : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  void _submitForm( BuildContext context ) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Required validation code!', context);
      return;
    }
    form.save();

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
      _processing = true;
    });

    TokenService tokenService = new TokenService();
    String accessToken = await tokenService.faceLogin( widget.facialGUID, false );
    ValidationService validationService = new ValidationService();
    bool result = await validationService.validate( _code, accessToken );

    if( result ){
      await set2FA( false );
      setState(() {
        progressColor = Colors.transparent;
        _topIcon = Icons.done;
        showMessage("Validation successful, thank you.", context);
        new ThreadTimer(new Duration(milliseconds: 3000), () {
            Navigator.pop( context, true );

        });
      });
    }else{
      showMessage("Invalid validation code!", context);
      setState(() {
        progressColor = Colors.transparent;
        _processing = false;
      });
    }

  }

  void _resend( BuildContext context ) async{
    if( _codeSent || _processing ) return;
    setState(() {
      _codeSent = true;
      _processing = true;
    });

    TokenService tokenService = new TokenService();
    String accessToken = await tokenService.faceLogin( widget.facialGUID, false );
    ValidationService validationService = new ValidationService();
    bool otp = await validationService.sendToken( accessToken );
    if( otp != null ){
      ++_resendCount;
      if( _resendCount < 3 ) {
        new ThreadTimer(new Duration(milliseconds: 1000 * 10), () {
          setState(() {
            _codeSent = false;
          });
        });
      }
      showMessage("Validation code already sent to your registered mobile.", context);
    }else{
      showMessage("Validation code sending failed, please try again later!", context);
    }

    setState(() {
      _processing = false;
    });
  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}