
import '/hrm/model/entity/kiosk_data.dart';
import '/hrm/service/firebase_service.dart';
import '/hrm/service/kiosk_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';



class ActivateKiosk extends StatefulWidget {
  ActivateKiosk({Key key}) : super(key: key);

  @override
  _ActivateKioskState createState() => _ActivateKioskState( );
}

class _ActivateKioskState extends State<ActivateKiosk> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  Color progressColor = Colors.transparent;
  String _kioskKey;

  @override
  void initState() {
    super.initState();
    _checkFireBaseRegistration();
  }

  void _checkFireBaseRegistration() async{
    try{
      if( await getFirebaseToken( ) == null){
        FirebaseMessage _firebaseMessage = FirebaseMessage( );
        await _firebaseMessage.initMessaging( );
      }
    }catch(e){}
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).primaryColor,
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Activation'),
            bottom: _createProgressIndicator(),
          ),
          body: Stack(
            fit: StackFit.passthrough,
            children: [
              _backgroundPanel( true ),
              Center(
                  child: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  color: Color.fromRGBO(250, 250, 250, .4),
                                  elevation: 1,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                          margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                          child: Text('Stand mode activation',
                                              style: TextStyle(
                                                  color: Theme.of(context).primaryColor,
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 18
                                              )),
                                        ),
                                        Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                    height: 150,
                                                    child: new Image.asset("assets/images/stand.png"),
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                          child: formData(context),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )
              )
            ]
          )
        )
    );
  }

  Form formData(BuildContext context ){
    return new Form(
        key: _formKey,
        autovalidate: true,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.vpn_key),
                  hintText: '',
                  labelText: 'Activation code',
                ),
                validator: (val) => val.isEmpty ? 'Code required' : null,
                onSaved: (val) => _kioskKey = val,
              ),
              Container(
                margin: const EdgeInsets.only(left: 20.0, right: 5.0, top: 10),
                child:Text(
                  "Code will be available under 'More > Stand mode management' after login.",
                  style: new TextStyle(fontSize: 12.0, color: Colors.black87),
                ),
              ),
              Container(
                padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                width: double.infinity,
                child: RaisedButton(
                  padding: EdgeInsets.all(15),
                  onPressed: () {_submitForm(context); },
                  elevation: 1,
                  child: Text(
                    'Activate',
                    style: TextStyle(color: Colors.white),
                  ),
                    shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(30.0))
                ),
              ),
            ])
    );
  }

  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
          image:
          MediaQuery.of(context).orientation == Orientation.portrait ?
          new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
              : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  void _submitForm(BuildContext context) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Required activation code!', context);
      return;
    }
    form.save();

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
      _processing = true;
    });

    String firebaseToken = await getFirebaseToken();
    if( firebaseToken == null){
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
        showMessage('Device validation failed. Close the app and try again!', context);
      });
      return;
    }

    KioskService kioskService = new KioskService( );
    KioskData kioskData = await kioskService.activate(_kioskKey, firebaseToken);
    if( kioskData == null ){
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
        showMessage('Failed to activate, please try again!', context);
      });
      return;
    }

    if( !kioskData.success){
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
        showMessage(kioskData.message, context);
      });
      return;
    }

    Navigator.pop(context, kioskData);

  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}