import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/view/login/pre_login.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'kiosk_recognize.dart';
import 'kiosk_settings.dart';


class KioskHome extends StatefulWidget {
  KioskHome({Key key}) : super(key: key);

  @override
  _KioskHomeState createState() => _KioskHomeState();
}

class _KioskHomeState extends State<KioskHome>{

  bool _shutterSoundNotice = true;

  @override
  void initState() {
    super.initState();
    _loadSettings();
  }

  void _loadSettings( )async{
    if( Platform.isIOS)
      _shutterSoundNotice = await getCameraShutterSoundNotice( );

    setState(() { });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Text('${Constant.APP_NAME} Attendance'),
        ),
      body: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            mainBodyFirstWidgets(context),
            mainBodySecondWidgets(context)
          ]
        )
      )
    );
  }

  Widget mainBodyFirstWidgets(BuildContext context){
    return Container(
        color: Colors.white,
        padding: const EdgeInsets.only(left: 5.0, right: 5.0 ),
        height: 210,
        child: Row(
          children: <Widget>[
            Container(
              width: MediaQuery.of(context).size.width * 0.48,
              child: GestureDetector(
                child: Card(
                  color: Colors.white,
                  elevation: 1,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Container(
                        width: 128,
                        height: 128,
                        padding: const EdgeInsets.only(top: 15.0 ),
                        child: IconButton(
                          highlightColor: Colors.red,
                          icon: SvgPicture.asset( 'assets/images/attend.svg', semanticsLabel: 'In / Out',height: 128, width: 128, ),
                          onPressed: () async{_gotoAttendance();},
                          tooltip: tr(LocaleKeys.mnuInOut),
                        ),
                        //child: SvgPicture.asset( 'assets/images/add_user.svg', semanticsLabel: 'Add User',height: 128, width: 128, ),
                      ),
                      ListTile(
                        title: Text(
                          tr(LocaleKeys.mnuInOut),
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                        onTap: () async{_gotoAttendance();},
                      ),
                    ],
                  ),
                ),
                onTap: ()async{_gotoAttendance();},
              ),
            ),
            Container(
              width: MediaQuery.of(context).size.width * 0.48,
              child: GestureDetector(
                child: Card(
                  color: Colors.white,
                  elevation: 1,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Container(
                        width: 128,
                        height: 128,
                        padding: const EdgeInsets.only(top: 15.0 ),
                        child: IconButton(
                          highlightColor: Colors.red,
                          icon: Icon(Icons.settings,size: 128, color: Theme.of(context).primaryColor,),
                          onPressed: () {_gotoSettings();},
                          tooltip: tr(LocaleKeys.lblSettings),
                        ),
                        //child: SvgPicture.asset( 'assets/images/add_user.svg', semanticsLabel: 'Add User',height: 128, width: 128, ),
                      ),
                      ListTile(
                        title: Text(
                          tr(LocaleKeys.lblSettings),
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                        onTap: () { _gotoSettings();},
                      ),
                    ],

                  ),
                ),
                onTap: (){ _gotoSettings(); },
              ),
            ),
          ],
        )
    );
  }

  Widget mainBodySecondWidgets(BuildContext context){
    return Container(
        color: Colors.white,
        padding: const EdgeInsets.only(left: 5.0, right: 5.0 ),
        height: 210,
        child: Row(
          children: <Widget>[
            Container(
              width: MediaQuery.of(context).size.width * 0.48,
              child: GestureDetector(
                child: Card(
                  color: Colors.white,
                  elevation: 1,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Container(
                        width: 128,
                        height: 128,
                        padding: const EdgeInsets.only(top: 15.0 ),
                        child: IconButton(
                          highlightColor: Colors.red,
                          icon: Icon(Icons.help_outline, size: 128, color: Theme.of(context).primaryColor,),
                          onPressed: () {  makeCall( Constant.CONTACT_PHONE ); },
                          tooltip: tr(LocaleKeys.lblHelp),
                        ),
                        //child: SvgPicture.asset( 'assets/images/add_user.svg', semanticsLabel: 'Add User',height: 128, width: 128, ),
                      ),
                      ListTile(
                        title: Text(
                          tr(LocaleKeys.lblHelp),
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                        onTap: () { makeCall( Constant.CONTACT_PHONE ); },
                      ),
                    ],
                  ),
                ),
                onTap: () { makeCall( Constant.CONTACT_PHONE ); },
              ),
            ),
            Container(
              width: MediaQuery.of(context).size.width * 0.48,
              child: GestureDetector(
                child: Card(
                  color: Colors.white,
                  elevation: 1,
                  child: new Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Container(
                        width: 128,
                        height: 128,
                        padding: const EdgeInsets.only(top: 15.0 ),
                        child: IconButton(
                          highlightColor: Colors.red,
                          icon: Icon(Icons.contact_phone, size: 128, color: Theme.of(context).primaryColor,),
                          onPressed: () { makeCall( Constant.CONTACT_PHONE );  },
                          tooltip: tr(LocaleKeys.lblContact),
                        ),
                      ),
                      ListTile(
                        title: Text(
                          tr(LocaleKeys.lblContact),
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                        onTap: () { makeCall( Constant.CONTACT_PHONE ); },
                      ),
                    ],
                  ),
                ),
                onTap: (){ makeCall( Constant.CONTACT_PHONE ); },
              ),
            ),
          ],
        )
    );
  }
  void _gotoAttendance()async{

    if( Platform.isIOS && _shutterSoundNotice ) {
      bool dont = await _showDialog(context);
      _shutterSoundNotice = !dont;
      if (dont) {
        await setCameraShutterSoundNotice(false);
      }
    }
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => KioskRecognize(),
        )
    );
  }

  void _gotoSettings()async{
    String message = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => KioskSettings(),
        )
    );

    if( message!= null && message=='exit'){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => LoginSelection(title: tr(LocaleKeys.login))
          )
      );
    }
  }

  Future<bool> _showDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Camera usage!"),
          content: new Text("You may hear camera shutter sound frequently. The purpose of preventing to access the hidden camera."),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Don't show again"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

}