

import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/database/dao/logs_dao.dart';
import '/hrm/database/entity/log.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/kiosk_data.dart';
import '/hrm/model/request/attendance_request.dart';
import '/hrm/model/response/attendance_response.dart';
import '/hrm/model/response/last_attendance_response.dart';
import '/hrm/model/response/recognize_result.dart';
import '/hrm/service/attendance_service.dart';
import '/hrm/service/face_detector_service.dart';
import '/hrm/service/face_recognization_service.dart';
import '/hrm/service/firebase_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/foundation.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import '/hrm/imageprocessing/flutter_camera.dart';
import 'package:path_provider/path_provider.dart';
import 'package:screen/screen.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import '/hrm/imageprocessing/base64image.dart';
import 'package:flutter_svg/flutter_svg.dart';


class KioskRecognize extends StatefulWidget {
  KioskRecognize({Key key}) : super(key: key);
  @override
  _KioskRecognizePageState createState() => _KioskRecognizePageState();
}

class _KioskRecognizePageState extends State<KioskRecognize> {

  bool _processing = false;
  bool willStop = false;
  bool _foundFace = false;
  bool _faceError = false;
  Color progressColor = Colors.transparent;
  String _message = '';
  String _distanceMessage = '';
  Color _messageColor = Colors.white;
  String _accessToken;
  String _firebaseToken;
  bool _detected = false;
  String _action = LastAttendanceResponse.ACTION_IN;

  // Setting values
  int _intervalMinutes = 120;
  double _detectionDistance=1.0;
  bool _autoAttendance = true;
  bool _screenOn = true;
  bool _inOutMode = true;


  final _scanKey = GlobalKey<CameraMlVisionState>();
  CameraLensDirection cameraLensDirection = CameraLensDirection.front;
  AttendanceService _attendanceService = new AttendanceService();
  FaceDetectorService _faceDetectorService = new FaceDetectorService();
  RecognizeResult _recognizeResult;
  FaceRecognizationService _faceRecognizationService;
  KioskData _kioskData = new KioskData();
  LogDao logDao = new LogDao();
  SharedPreferences _sharedPrefs;


  @override
  initState() {
    super.initState( );
    _faceDetectorService.init();
    _faceRecognizationService = new FaceRecognizationService( );
    // listenThread( );
  }

  void listenThread( ) async{
    // TODO replace with device token service
    try{
      await _kioskData.fromSharedPreferences();
      await _getAccessToken( _kioskData.facialGUID );
      _sharedPrefs = await SharedPreferences.getInstance();
    }catch(e){}

    try{
      // TODO remove platform check condition
      if ( Platform.isAndroid ){
        _firebaseToken = await getFirebaseToken( );
        if( _firebaseToken == null){
          FirebaseMessage _firebaseMessage = FirebaseMessage();
          await _firebaseMessage.initMessaging();
        }
      }
    }catch(e){}
    _detectionDistance = await getKioskSettingsFaceDetectionDistance( );
    _autoAttendance = await getKioskSettingsAutoAttendance();
    _intervalMinutes = await getKioskSettingsTimeIntervalMinutes();
    _screenOn = await getKioskSettingsScreenAlwaysOn();
    _inOutMode = await getKioskSettingsInOutMode();
    await logDao.open();
    if( _screenOn ) {
      try {Screen.setBrightness(0.5);Screen.keepOn(true);} catch (e) {}
    }

    var thread = new Thread(listen);
    await thread.start();
    await thread.join();
    if( Constant.ENABLE_LOGS)
      print("Thread terminated");
  }

  Future listen( ) async {
    while ( !willStop ) {
      await Thread.sleep( 50 );
      if( !_processing && mounted && !_foundFace)
        _processPhoto( );
      await Thread.sleep( 50 );
    }
    await Thread.sleep( 1 * 1000 );
    if( Constant.ENABLE_LOGS)
      print("Thread sleep");
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
      appBar: AppBar(
        title: Text(tr(LocaleKeys.mnuInOut)),
          bottom: _createProgressIndicator(),
      ),
      body: SizedBox.expand(
        child: CameraMlVision<List<Object>>(
          key: _scanKey,
          cameraLensDirection: cameraLensDirection,
          resolution: ResolutionPreset.high,
          overlayBuilder: (c) {
            return Positioned(
                bottom: 16.0,
                left: 16.0,
                right: 16.0,
                child: MaterialButton(
                  onPressed: () async{await _processPhoto();},
                  color: Theme.of(context).primaryColor,
                  textColor: Colors.white,
                  child: Icon(
                    Icons.camera_alt,
                    size: 32,
                  ),
                  padding: EdgeInsets.all(16),
                  shape: CircleBorder(),
                )
            );
            // return Container();
          },
          overlayBuilder1: (c){
            return _actionPanel( context );
          },
          onDispose: () {},
        ),
      ),
    )
    );
  }

  Widget _actionPanel( BuildContext context ){
    return _foundFace ? _attendanceActionPanel(context)
    : Positioned(
      top: 10.0,
      left: 10.0,
      right: 10.0,
      child: !_detected ? new Container(
        child: !_detected && _faceError ?
        new Container(
          child: ListTile(
            title:Text(
              'Sorry',
//              'Sorry, your face was not be recognized',
              softWrap: true,
              style: TextStyle(
                fontWeight: FontWeight.w400,
                color: Colors.red[600],
                fontSize: 14,
              ),
            ),
            subtitle: Text(
              'Please look at me and try again!',
              softWrap: true,
              style: TextStyle(
                fontWeight: FontWeight.w400,
                color: Colors.red[600],
                fontSize: 12,
              ),
            ),
            leading: Icon(Icons.clear, color: Colors.red[600],),
          ),
        )
        : new Container()
      )
          : Card(
        color: Color.fromRGBO( 0, 00, 0, .7),
        elevation: 0.0,
        child:Column(
            children: <Widget>[
              Image(image: AssetImage('assets/images/face_scan.gif')),
              Container(
                margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
                child: ListTile(
                  title:Text(
                    'Recognizing your face ....',
                    softWrap: true,
                    style: TextStyle(
                      fontWeight: FontWeight.w400,
                      color: _messageColor,
                      fontSize: 14,
                    ),
                  ),
                  subtitle: Text(
                    _distanceMessage,
                    softWrap: true,
                    style: TextStyle(
                      fontWeight: FontWeight.w400,
                      color: _messageColor,
                      fontSize: 12,
                    ),
                  ),
                ),
              ),
            ],
        ),
      ),
    );
  }

  Widget _attendanceActionPanel(BuildContext context){
    return _autoAttendance ?
    Positioned(
      top: 0.0,
      left: 10.0,
      right: 10.0,
      child: Card(
        color: Color.fromRGBO( 0, 00, 0, .7),
        elevation: 2.0,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Column(
          children: <Widget>[
            Container(
              padding: const EdgeInsets.all(10.0),
              child: ClipOval(
                  child: Material(
                    elevation: 2,
                    color: Colors.white, // button color
                    child: InkWell(
                      splashColor: Colors.red, // inkwell color
                      child: SizedBox(
                        width: 150,
                        height: 150,
                        child: _recognizeResult.profile != null ? decode( _recognizeResult.profile ):
                        new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_recognizeResult.faceGUID}',
                          headers: {'Authorization':'Bearer ${_accessToken}'}, ),
                      ),
                    ),
                  )
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(0.0),
              child: Text(
                "${_recognizeResult.name}",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 24.0,
                  fontWeight: FontWeight.bold,
                ),
              ),
            ),
            Padding(
                padding: const EdgeInsets.all(2.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Container(
                      child: Text(
                        '${new DateFormat('hh:mm').format(new DateTime.now())}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 48
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                    Container(
                      margin: EdgeInsets.only(top: 20),
                      child: Text(
                        '${new DateFormat('a').format(new DateTime.now()).toLowerCase()}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 24
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ],
                )
            ),
            Container(
              padding: const EdgeInsets.only(left:10.0,right: 10, bottom: 10),
              child: Divider(
                color: Colors.white,
              ),
            ),
            Container(
              margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
              child: ListTile(
                title:Text(
                  _message,
                  softWrap: true,
                  style: TextStyle(
                    fontWeight: FontWeight.w400,
                    color: _messageColor,
                    fontSize: 18,
                  ),
                ),
                leading: SvgPicture.asset( 'assets/images/tick.svg', semanticsLabel: 'Recognize',height: 24, width: 24, ),
              ),
            ),
          ],
        ),
      ),
    )
        : Positioned(
      bottom: 50.0,
      left: 10.0,
      right: 10.0,
      child: Card(
        color: Colors.transparent,
        elevation: 1.0,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Column(
          children: <Widget>[
            Align(
              alignment: Alignment.topRight,
              child: Padding(
                padding: const EdgeInsets.all(5.0),
                child:Ink(
                  decoration: ShapeDecoration(
                    color: Colors.red[600],
                    shape: CircleBorder(),
                  ),
                  child: IconButton(
                    icon: Icon(Icons.close),
                    color: Colors.white,
                    onPressed:(){
                      setState(() {
                        _foundFace = false;
                        _processing = false;
                      });
                    },
                  ),
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.all(10.0),
              child: ClipOval(
                  child: Material(
                    elevation: 2,
                    color: Colors.white, // button color
                    child: InkWell(
                      splashColor: Colors.red, // inkwell color
                      child: SizedBox(
                        width: 150,
                        height: 150,
                        child: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${_recognizeResult.faceGUID}',
                          headers: {'Authorization':'Bearer ${_accessToken}'}, ),
                      ),
                    ),
                  )
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(0.0),
              child: Text(
                "${_recognizeResult.name}",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 24.0,
                  fontWeight: FontWeight.bold,
                ),
              ),
            ),
            Padding(
                padding: const EdgeInsets.all(2.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Container(
                      child: Text(
                        '${new DateFormat('hh:mm').format(new DateTime.now())}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 48
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                    Container(
                      margin: EdgeInsets.only(top: 20),
                      child: Text(
                        '${new DateFormat('a').format(new DateTime.now()).toLowerCase()}',
                        style: TextStyle(
                            color: Colors.white,
                            fontWeight: FontWeight.normal,
                            fontSize: 24
                        ),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ],
                )
            ),
            Container(
              padding: const EdgeInsets.only(left:10.0,right: 10, bottom: 10),
              child: Divider(
                color: Colors.white,
              ),
            ),
            Container(
              margin: EdgeInsets.only(bottom: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  _buildButtons(
                      _action == LastAttendanceResponse.ACTION_IN? Colors.green[900] : Colors.grey,
                      SvgPicture.asset( 'assets/images/punch_in.svg',
                        semanticsLabel: tr(LocaleKeys.btnPunchIn),height: 40, width: 40, ),
                      tr(LocaleKeys.btnPunchIn)
                  ),
                  _buildButtons( _action == LastAttendanceResponse.ACTION_OUT? Colors.green[900] : Colors.grey,
                      SvgPicture.asset( 'assets/images/punch_out.svg',
                        semanticsLabel: tr(LocaleKeys.btnPunchOut),height: 40, width: 40, ),
                      tr(LocaleKeys.btnPunchOut)
                  ),
                ],
              ),
            ),
            Container(
              margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
              child: Text(
                _message,
                softWrap: true,
                style: TextStyle(
                  fontWeight: FontWeight.w400,
                  color: _messageColor,
                  fontSize: 18,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  Column _buildButtons(Color color, SvgPicture icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 3.0,
          icon: icon,
          color: color,
          onPressed:!_processing?() async{
            String _note = 'Face In';
            if( label == tr(LocaleKeys.btnPunchIn) ){
              _doAttend(_note, LastAttendanceResponse.ACTION_IN);
            }else{
              _note = 'Face Out';
              _doAttend(_note, LastAttendanceResponse.ACTION_OUT);
            }
          } : null,
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  @override
  void deactivate() {
    willStop = true;
    super.deactivate();
  }

  @override
  void dispose() {
    try {
      if (_scanKey.currentState.mounted) {
        _scanKey.currentState.dispose();
      }
    }catch(e){}
    willStop = true;
    try {
      _faceDetectorService.dispose();
//      detector.close();
    }catch(e){}

    try{
      logDao.close();
    }catch(e){}

    super.dispose();
  }

  Future<void> _processPhoto( ) async{
    try {
          // state error
          if( _scanKey.currentState == null ){
            _exit('_scanKey.currentState == null');
            return;
          }
          if( _processing ) return;
          // media not mounted
          if( !mounted ){
            _exit('Not mounted');
            return;
          }

          setState(() {
            progressColor = Colors.white;
            _messageColor = Colors.white;
            _processing = true;
            _faceError = false;
            _detected = false;
            _message='';
          });

          final Directory extDir = await getApplicationDocumentsDirectory();
          final String tmpDir = '${extDir.path}/Pictures/flutter_test';
          await Directory(tmpDir).create(recursive: true);
          String photoPath = '$tmpDir/${DateTime.now()}-o.jpg';

          photoPath = await _scanKey.currentState.takePhoto( );
          // Camera error
          if( photoPath==null ){
            _exit('Camera failed');
            return;
          }

          DetectionResult detectionResult;
          try{
            detectionResult = await _faceDetectorService.detectedFaceAsBase64String(tmpDir, photoPath, mounted, true);
          }catch(e){}

          if( detectionResult != null ){

            if( detectionResult.isOnlineDetector ){
              _distanceMessage = 'Google service does not working';
              _message = 'Google service does not working';
              _messageColor = Colors.red[700];
              _exit('Google service does not working');
              return;
            }

            // checking min distance
            if( double.parse(detectionResult.distance) > _detectionDistance ){
              setState(() {
                progressColor = Colors.transparent;
                _foundFace = false;
                _processing = false;
              });
              return;
            }

            if(_screenOn) {Screen.setBrightness(0.8);}
            setState(() {
              _distanceMessage = '${detectionResult.distance} meters faraway';
              progressColor = Colors.green;
              _detected = true;
            });

            _recognizeResult = await _faceRecognizationService.recognizeByOrganization( detectionResult.detectedFace, _kioskData.organizationGUID );
            setState(() {
              _detected = false;
            });

            if( _recognizeResult == null ){
              setState(() {
                progressColor = Colors.red[600];
                _faceError = true;
              });

              new ThreadTimer(new Duration(milliseconds: 1000 * 5), (){
                setState(() {
                  _foundFace = false;
                  _processing = false;
                });
              });
              return;
            }

            // in/out interval checking
            if( _autoAttendance ) {
              Log log = await logDao.findByFacialGuid(
                  _recognizeResult.faceGUID);
              if (log != null &&
                  log.logTime.millisecondsSinceEpoch + (_intervalMinutes*60*1000) >
                      DateTime
                          .now()
                          .millisecondsSinceEpoch) {
                setState(() {
                  progressColor = Colors.transparent;
                  _foundFace = false;
                  _processing = false;
                });
                return;
              }
            }

            if( _recognizeResult.result ) {
              if( Constant.ENABLE_LOGS)
                print(_recognizeResult.name);
              setState(() {
                _foundFace = true;
                progressColor = Colors.transparent;
                _faceError = false;
                if(_autoAttendance)
                  _message='Verified';
              });

              _getLastAttendanceAction( _recognizeResult.faceGUID );
            }else{
              setState(() {
                progressColor = Colors.red[600];
                _foundFace = false;
                _processing = false;
              });
            }

          }else{
            if( _screenOn ){try{Screen.setBrightness(0.2);}catch(e){}}
            setState(() {
              _detected = false;
              progressColor = Colors.red[600];
              _foundFace = false;
              _processing = false;
            });
            return;
          }

    }catch( e){
      _processing = false;
      if( Constant.ENABLE_LOGS)
        print('Error from catch ->>>>>>${e}');
      _exit('Exception -->${e}');
    }
    _processing = false;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    setState(() {
      willStop = true;
    });
    if( _screenOn) {try {Screen.setBrightness(0.5);Screen.keepOn(false);} catch (e) {}}
    return true;
  }

  Future<void> _getAccessToken( String facialGUID ) async{
    TokenService tokenService = new TokenService();
    String token =  await tokenService.faceLogin( facialGUID, false );
    if( Constant.ENABLE_LOGS)
      print('This is access token =>$token');
    setState(() {
      _accessToken = token;
    });
  }

  Future<void> _getLastAttendanceAction(String facialGUID) async{
    String note = "Kiosk In";

    if( _inOutMode ) {
      await _getAccessToken(facialGUID);
      LastAttendanceResponse lastAttendanceResponse = await _attendanceService
          .lastAction(_accessToken, _recognizeResult.faceGUID);
      if (lastAttendanceResponse == null
          || (lastAttendanceResponse != null && lastAttendanceResponse.action ==
              LastAttendanceResponse.ACTION_OUT)) {
        _action = LastAttendanceResponse.ACTION_IN;
      } else {
        _action = LastAttendanceResponse.ACTION_OUT;
        note = "Kiosk Out";
      }
      setState(() {});

      // in/out interval checking
      if( _autoAttendance && lastAttendanceResponse != null ) {

        if ( lastAttendanceResponse.dateTime.millisecondsSinceEpoch + (_intervalMinutes*60*1000) >
                DateTime.now().millisecondsSinceEpoch) {
          setState(() {
            progressColor = Colors.transparent;
            _foundFace = false;
            _processing = false;
          });
          return;
        }
      }

    }// End of IF

    if( _autoAttendance )
      await _doAttend( note, _action );
  }

  Future<void> _doAttend( String note, String action ) async{

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
    });

    _processing = true;
    AttendanceRequest attendanceRequest = new AttendanceRequest();
    attendanceRequest.action = action;
    attendanceRequest.note = note;
    attendanceRequest.faceGUID = _recognizeResult.faceGUID;
    attendanceRequest.firebase = _firebaseToken;
    attendanceRequest.source = AttendanceRequest.SOURCE_KIOSK;
    attendanceRequest.kioskGUID = _kioskData.guid;
    // try {
    //  if (_startLocation != null) {
    //    attendanceRequest.latitude = _startLocation.latitude;
    //    attendanceRequest.longitude = _startLocation.longitude;
    //    attendanceRequest.altitude = _startLocation.altitude;
    //    attendanceRequest.accuracy = _startLocation.accuracy;
    //    attendanceRequest.heading = _startLocation.heading;
    //    attendanceRequest.speed = _startLocation.speed;
    //  }
    // }catch(e){}

   // try {
   //   await Wifi.listMap().then((List<Map<String, dynamic>> result) {
   //       attendanceRequest.wifis = result;
   //   });
   // }catch(e){}

    try {
      if( Constant.ENABLE_LOGS)
        print('${json.encode(attendanceRequest.toMap())}');

      AttendanceResponse attendanceResponse = await _attendanceService.attend(_accessToken, attendanceRequest.toMap());

        if (attendanceResponse.success==null || !attendanceResponse.success) {
          setState(() {
            progressColor = Colors.transparent;
            _message = 'Problem occur: ${attendanceResponse.error}';
            _messageColor = Colors.red[700];
            _processing = false;
          });

          // TODO next face resume threshold
          new ThreadTimer(new Duration(milliseconds: 1000 * 3), (){
            setState(() {
              _foundFace = false;
              _processing = false;
            });
          });
        } else {
          setState(() {
            progressColor = Colors.transparent;
            _messageColor = Colors.white;
            if (action == 'in') {
              _message = 'Punch-In successful, thank you';
            } else {
              _message = 'Punch-Out successful, thank you';
            }
          });

          Log log = await logDao.findByFacialGuid(_recognizeResult.faceGUID);
            if( log == null){
              log = new Log(id:null, logTime: DateTime.now(),note: null, facialGuid: _recognizeResult.faceGUID);
              await logDao.insertLog(log);
            }else{
              log.logTime = DateTime.now();
              await logDao.update(log, _recognizeResult.faceGUID);
            }

          // TODO next face resume threshold
          new ThreadTimer(new Duration(milliseconds: 1000 * 1), (){
            setState(() {
              _foundFace = false;
              _processing = false;
            });
          });
        }
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
  }


  void _exit( String reason ){
    if( Constant.ENABLE_LOGS)
      print('Exit reason=${reason}');
    setState(() {
      willStop = true;
    });
    if(_screenOn) {try {Screen.setBrightness(0.5);Screen.keepOn(false);} catch (e) {}}
    try{
      logDao.close();
    }catch(e){}
    new ThreadTimer(new Duration(milliseconds: 1000), (){Navigator.pop(context, false);});
  }
}