

import '/hrm/config/constant.dart';
import '/hrm/model/entity/kiosk_data.dart';
import '/hrm/service/kiosk_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';


class KioskSettings extends StatefulWidget {
  KioskSettings({Key key}) : super(key: key);

  @override
  _KioskSettingsState createState() => _KioskSettingsState( );
}

class _KioskSettingsState extends State<KioskSettings> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  bool _processing = false;
  Color _progressColor = Colors.transparent;
  double _detectionDistance=1.0;
  double _interval = 2.0;
  bool _autoAttendance = true;
  bool _screenOn = true;
  bool _multipleFaces = false;
  bool _inOutMode = true;

  @override
  void initState() {
    super.initState();
    _loadDefaultValues();
  }

  void _loadDefaultValues()async{
    _detectionDistance = await getKioskSettingsFaceDetectionDistance( );
    _interval = (await getKioskSettingsTimeIntervalMinutes() / 60).toDouble();
    _autoAttendance = await getKioskSettingsAutoAttendance();
    _screenOn = await getKioskSettingsScreenAlwaysOn();
    _multipleFaces = await getKioskSettingsMultipleFaces();
    _inOutMode = await getKioskSettingsInOutMode();

    setState(() {});
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Settings'),
            bottom: _createProgressIndicator(),
          ),
          body: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  elevation: 0,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Automatic In/Out'),
                                              subtitle: Text('The attendance will be done automatically after detecting the face'),
                                              trailing: Switch(
                                                value: _autoAttendance,
                                                onChanged:_multipleFaces? null: (value){
                                                  setState(() {
                                                    _autoAttendance=value;
                                                    setKioskSettingsAutoAttendance( _autoAttendance );
                                                  });
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Multiple faces processing'),
                                              subtitle: Text('All detected faces will be processed at the same time'),
                                              trailing: Switch(
                                                value: _multipleFaces,
                                                onChanged: null,
                                                // onChanged: (value){
                                                //   setState(() {
                                                //     _multipleFaces=value;
                                                //     setKioskSettingsMultipleFaces( _multipleFaces );
                                                //     if( _multipleFaces){
                                                //       _autoAttendance = true;
                                                //       setKioskSettingsAutoAttendance( _autoAttendance );
                                                //     }
                                                //   });
                                                // },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Screen always on'),
                                              subtitle: Text('The device screen never timeout while detecting face'),
                                              trailing: Switch(
                                                value: _screenOn,
                                                onChanged: (value){
                                                  setState(() {
                                                    _screenOn=value;
                                                    setKioskSettingsScreenAlwaysOn( _screenOn );
                                                  });
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('In/Out mode'),
                                              subtitle: Text('Punch-in and punch-out will be recorded when Enable(e.g. for work). Otherwise every record will be punch-in(e.g. for school, meeting, etc.).'),
                                              trailing: Switch(
                                                value: _inOutMode,
                                                onChanged: (value){
                                                  setState(() {
                                                    _inOutMode=value;
                                                    setKioskSettingsInOutMode( _inOutMode );
                                                  });
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                          child: ListTile(
                                            title: SliderTheme(
                                            data: SliderTheme.of(context).copyWith(
                                              activeTrackColor: Theme.of(context).primaryColor,
                                              inactiveTrackColor: Colors.blueGrey[50],
                                              trackShape: RoundedRectSliderTrackShape(),
                                              trackHeight: 5.0,
                                              thumbShape: RoundSliderThumbShape(enabledThumbRadius: 10.0),
                                              thumbColor: Theme.of(context).primaryColor,
                                              overlayColor: Colors.green[800].withAlpha(32),
                                              overlayShape: RoundSliderOverlayShape(overlayRadius: 28.0),
                                              tickMarkShape: RoundSliderTickMarkShape(),
                                              activeTickMarkColor: Colors.white,
                                              inactiveTickMarkColor: Colors.red[600],
                                              valueIndicatorShape: PaddleSliderValueIndicatorShape(),
                                              valueIndicatorColor: Theme.of(context).primaryColor,
                                              valueIndicatorTextStyle: TextStyle(
                                                color: Colors.white,
                                              ),
                                            ),
                                            child: Slider(
                                              value: _detectionDistance,
                                              min: 0.2,
                                              max: 2.0,
                                              divisions: 9,
                                              label: '$_detectionDistance meters',
                                              onChanged: (value) {
                                                setState(() {
                                                  _detectionDistance = value;
                                                  setKioskSettingsFaceDetectionDistance( _detectionDistance );
                                                });
                                              },
                                            ),
                                          ),
                                            subtitle: Text('Minimum face detection distance: $_detectionDistance meters'),
                                          )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                            child: ListTile(
                                              title: SliderTheme(
                                                data: SliderTheme.of(context).copyWith(
                                                  activeTrackColor: Theme.of(context).primaryColor,
                                                  inactiveTrackColor: Colors.blueGrey[50],
                                                  trackShape: RoundedRectSliderTrackShape(),
                                                  trackHeight: 5.0,
                                                  thumbShape: RoundSliderThumbShape(enabledThumbRadius: 10.0),
                                                  thumbColor: Theme.of(context).primaryColor,
                                                  overlayColor: Colors.green[800].withAlpha(32),
                                                  overlayShape: RoundSliderOverlayShape(overlayRadius: 28.0),
                                                  tickMarkShape: RoundSliderTickMarkShape(),
                                                  activeTickMarkColor: Colors.white,
                                                  inactiveTickMarkColor: Colors.red[600],
                                                  valueIndicatorShape: PaddleSliderValueIndicatorShape(),
                                                  valueIndicatorColor: Theme.of(context).primaryColor,
                                                  valueIndicatorTextStyle: TextStyle(
                                                    color: Colors.white,
                                                  ),
                                                ),
                                                child: Slider(
                                                  value: _interval,
                                                  min: 0.0,
                                                  max: 7.5,
                                                  divisions: 15,
                                                  label: '$_interval hours',
                                                  onChanged: (value) {
                                                    setState(() {
                                                      _interval = value;
                                                      setKioskSettingsTimeIntervalMinutes( _interval );
                                                    });
                                                  },
                                                ),
                                              ),
                                              subtitle: Text('Minimum In/Out time interval: $_interval hours'),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Logout'),
                                              subtitle: Text('Press here to exit the kiosk mode!'),
                                              trailing: Icon(Icons.exit_to_app, color: Theme.of(context).primaryColor ),
                                              onTap: (){_exit(context);},
                                            )
                                        ),
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )

        )
    );
  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

  Future<void> _exit( BuildContext context) async{
    // if( _processing) return;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    bool result = await _showConfirmDialog(context);
    if ( !result ){
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
      return;
    }

    KioskData kioskData = new KioskData();
    await kioskData.fromSharedPreferences();
    KioskService kioskService = new KioskService();
    result = await kioskService.exit(kioskData.deviceGUID, kioskData.facialGUID);

    if( !result){
      showMessage("Error: kiosk mode exit failed, please try again!", context);
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
    }else{
      await _sharedPrefs.remove(Constant.CURRENT_USER_DATA);
      await _sharedPrefs.remove(Constant.CURRENT_USER_TOKEN_VALUE);
      await _sharedPrefs.remove(Constant.CURRENT_USER_DATA_EXT_EMPLOYEE);
      await _sharedPrefs.remove(Constant.CURRENT_USER_DATA_AUTHORITY);
      await _sharedPrefs.remove(Constant.FIREBASE_TOKEN_KEY);
      await _sharedPrefs.remove(Constant.DEVICE_GUID);
      await _sharedPrefs.remove(Constant.CURRENT_KIOSK_DATA);
      await _sharedPrefs.remove(Constant.KIOSK_SETTINGS_AUTO_ATTENDANCE);
      await _sharedPrefs.remove(Constant.KIOSK_SETTINGS_DETECTION_DISTANCE);
      await _sharedPrefs.remove(Constant.KIOSK_SETTINGS_TIME_INTERVAL);
      Navigator.pop(context, 'exit');
    }

  }

  Future<bool> _showConfirmDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Exit confirm"),
          content: new Text("Are you sure to exit the kiosk mode?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

}