

import 'package:badges/badges.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/leave_apply_request.dart';
import '/hrm/model/response/leave_apply_response.dart';
import '/hrm/model/response/leave_balance_response.dart';
import '/hrm/service/leave_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:percent_indicator/linear_percent_indicator.dart';


class ApplyLeave extends StatefulWidget {
  ApplyLeave({Key key}) : super(key: key);
  @override
  _ApplyLeaveState createState() => _ApplyLeaveState();
}

class _ApplyLeaveState extends State<ApplyLeave> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  Color progressColor = Colors.transparent;
  Color _messageColor = Colors.green[800];
  String _message = '';
  final TextEditingController _controller_sd = new TextEditingController();
  final TextEditingController _controller_ed = new TextEditingController();
  final TextEditingController _controller_reason = new TextEditingController();
  DateTime _sdate = new DateTime.now();
  DateTime _edate = new DateTime.now();
  Map<String, dynamic> selectedType;
  String _closeLabel = tr(LocaleKeys.btnLater);

  LeaveService _leaveService = new LeaveService();

//  LeaveTypeResponse _leaveTypeResponse;
//  LeaveTypeItem _leaveTypeItem;


  LeaveBalanceResponse _balanceResponse;
  LeaveTypeItem _leaveTypeItem;

  User _currentUser = new User();
  String _accessToken;
  String _leaveDateType = 'Full Day';
  List<String> leaveTypeList = <String>['Full Day', 'Half Day - Morning', 'Half Day - Afternoon'];

  @override
  void initState() {
    super.initState();
    _loadLeaveTypes();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(tr(LocaleKeys.titleLeaveApplication)),
            bottom: _createProgressIndicator(),
          ),

          body: new SafeArea(
            top: false,
            bottom: false,
            child: new Form(
              key: _formKey,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              child: new ListView(
                padding: const EdgeInsets.symmetric(horizontal: 6.0),
                children: <Widget>[

                  Container(
                    child: Card(
                      color: Colors.grey[50],
                      elevation: 1,
                      child: new Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          ListTile(
                            title: Text(
                              tr(LocaleKeys.titleLeaveForm),
                              style: TextStyle(
                                color: Theme.of(context).primaryColor,
                              ),
                            ),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child: _balanceResponse==null?Container() : _leaveTypeSelection( context ),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child:new Row(children: <Widget>[
                              new Expanded(
                                  child: new TextFormField(
                                    decoration: new InputDecoration(
                                      icon: const Icon(Icons.calendar_today),
                                      hintText: 'yyyy-mm-dd',
                                      labelText: tr(LocaleKeys.lblLeaveStartDate),
                                    ),
                                    controller: _controller_sd,
                                    keyboardType: TextInputType.datetime,
                                    validator: (val) => isValidLeaveDate(val) ? null : 'Not a valid date',
                                    onSaved: (val) => _sdate = convertToDate(val),
                                  )),
                              new IconButton(
                                icon: new Icon(Icons.mode_edit),
                                tooltip: 'Choose date',
                                onPressed: (() {
                                  _chooseDate(context, _controller_sd.text, true);
                                }),
                              )
                            ]),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child:new Row(children: <Widget>[
                              new Expanded(
                                  child: new TextFormField(
                                    decoration: new InputDecoration(
                                      icon: const Icon(Icons.calendar_today),
                                      hintText: 'yyyy-mm-dd',
                                      labelText: tr(LocaleKeys.lblLeaveEndDate),
                                    ),
                                    controller: _controller_ed,
                                    keyboardType: TextInputType.datetime,
                                    validator: (val) => isValidLeaveDate(val) ? null : 'Not a valid date',
                                    onSaved: (val) => _edate = convertToDate(val),
                                  )),
                              new IconButton(
                                icon: new Icon(Icons.mode_edit),
                                tooltip: 'Choose date',
                                onPressed: (() {
                                  _chooseDate(context, _controller_ed.text, false);
                                }),
                              )
                            ]),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 15.0, left: 5.0, right: 5.0),
                            child: _sdate.compareTo(_edate)==0? _leaveDayTypeSelection(context) : Container(),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 15.0, left: 5.0, right: 5.0),
                            child: new TextFormField(
                              decoration: InputDecoration(
                                icon: const Icon(Icons.help_outline),
                                hintText: '',
                                labelText: tr(LocaleKeys.lblLeaveReason),
                              ),
                              controller: _controller_reason,
                            ),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                              children: [
                                _buildButtons( Theme.of(context).primaryColor, Icons.email, tr(LocaleKeys.btnApply) ),
                                _buildButtons( Colors.transparent, Icons.cancel, _closeLabel),
                              ],
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                  Container(
                    alignment: Alignment.center,
                    margin: const EdgeInsets.only( bottom: 5.0, top: 10, left: 20.0),
                    child: Text(
                      _message,
                      softWrap: true,
                      style: TextStyle(
                        fontWeight: FontWeight.w400,
                        color: _messageColor,
                        fontSize: 16,
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
        )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: ()async{
            if( label == 'Close' || label == tr(LocaleKeys.btnLater) ){
              Navigator.pop( context );
            }else{
              _submitForm();
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  Widget _leaveTypeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<LeaveTypeItem>(
          selectedItem: _leaveTypeItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (LeaveTypeItem i, LeaveTypeItem s) => i?.leaveTypeGuid==s?.leaveTypeGuid,
          label: tr(LocaleKeys.lblLeaveType),
          onFind: (String filter) => _leaveTypeSuggestion(filter),
          onChanged: (LeaveTypeItem data) {
            _leaveTypeItem = data;
          },
          dropdownBuilder: (BuildContext context, LeaveTypeItem item ) =>
              Container(
                child: (item?.leaveType == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.drafts), title: Text(tr(LocaleKeys.lblSelectLeaveType)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.leaveType),
                ),
              ),
          popupItemBuilder: (BuildContext context, LeaveTypeItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.leaveType),
                  subtitle: new LinearPercentIndicator(
                    animation: true,
                    lineHeight: 20.0,
                    animationDuration: 2500,
                    percent: 1 - ( (item.used+item.pending ) / item.total ),
                    center: Text(
                        "${item.total - (item.used+item.pending)} days available",
                        style: new TextStyle(color: Colors.white)
                    ),
                    linearStrokeCap: LinearStrokeCap.roundAll,
                    progressColor: Theme.of(context).primaryColor,
                  ),
                ),
              ),
        )
    );
  }
  Future<List<LeaveTypeItem>> _leaveTypeSuggestion(String query) async{
    List<LeaveTypeItem> matches = List();
    if( _balanceResponse == null)
      return matches;
    matches.addAll(_balanceResponse.items);

    matches.retainWhere((s) {
      LeaveTypeItem leaveTypeItem = s;
      if( leaveTypeItem.leaveType.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isAfter(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2030)//new DateTime.now()
    );

    if (result == null) return;

    setState(() {
      if( start ) {
        _controller_sd.text = new DateFormat('yyyy-MM-dd').format(result);
        _sdate = result;
      }else
        _controller_ed.text = new DateFormat('yyyy-MM-dd').format(result);
      _edate = result;
    });
  }
  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    }catch (e) {return null;}
  }
  bool isValidLeaveDate(String leaveDate) {
    if (leaveDate.isEmpty) return false;
    var d = convertToDate(leaveDate);
    //return d != null && d.isAfter(new DateTime.now());
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Future<void> _loadLeaveTypes( ) async {
    await _currentUser.fromSharedPreferences();
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    _balanceResponse = await _leaveService.balance( _accessToken, _currentUser.employeeGUID);
    setState(() {

    });
//    _leaveTypeResponse = await _leaveService.leaveType(_accessToken);
  }

  void _submitForm() {
    final FormState form = _formKey.currentState;

    if( _leaveTypeItem == null ){
      showMessage('Leave type required!');
      return;
    }

    if( _sdate == null && _controller_sd.text.isEmpty){
      showMessage('Start date required!');
      return;
    }
    if( _edate == null && _controller_ed.text.isEmpty){
      showMessage('End date required!');
      return;
    }

    _sdate = convertToDate(_controller_sd.text);
    _edate = convertToDate(_controller_ed.text);

    if( _sdate == null){
      showMessage('Invalid start date!');
      return;
    }
    if( _edate == null){
      showMessage('Invalid end date!');
      return;
    }

    if( _sdate.isAfter(_edate)){
      showMessage('Invalid start/end date!');
      return;
    }

    if (!form.validate()) {
      showMessage('Field required');
    }else {
      form.save();
      _doApply(  );
    }
  }

  Future<void> _doApply(  ) async{

    bool result = await _showConfirmDialog(context);
    if ( !result ){return;}

    if( _processing ) return;

    setState(() {
      _processing = true;
      progressColor = Colors.white;
      _messageColor = Theme.of(context).primaryColor;
    });

    try {
      LeaveApplyRequest leaveApplyRequest = new LeaveApplyRequest(
        employeeGuid: _currentUser.employeeGUID,
        startDate: _sdate,
        endDate: _edate,
        leaveTypeGuid: _leaveTypeItem.leaveTypeGuid,
        reason: _controller_reason.text,
        attachment: null,
        leaveDayType: _leaveDateType
      );

      TokenService tokenService = new TokenService( );
      _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
      LeaveApplyResponse leaveApplyResponse = await _leaveService.apply(_accessToken, leaveApplyRequest);

      if (leaveApplyResponse.success && leaveApplyResponse.total >0) {
        setState(() {
          progressColor = Colors.transparent;
          showMessage('Success: leave applied for ${leaveApplyResponse.total.toStringAsFixed(1)} days');
          _message = 'Success: leave applied for ${leaveApplyResponse.total.toStringAsFixed(1)} days';
          _closeLabel = 'Close';
        });
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => ApplyLeaveSuccessResponse(
                leaveApplyResponse: leaveApplyResponse,
                sDate: _sdate,
                eDate: _edate,
                leaveDateType: _leaveDateType,
              ),
            )
        );
//          new ThreadTimer(new Duration(milliseconds: 1000), (){Navigator.pop(context,'OK');});

      } else {
        setState(() {
          progressColor = Colors.transparent;
          showMessage('Problem occur, ${leaveApplyResponse.message}');
          _processing = false;
          _message = 'Problem occur, ${leaveApplyResponse.message}';
          _messageColor = Colors.red[600];
        });
      }


    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    _processing = false;

  }

  Future<bool> _showConfirmDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm"),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

  Widget _leaveDayTypeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<String>(
          selectedItem: _leaveDateType,
          mode: Mode.MENU,
//          showSearchBox: true,
//          showClearButton: true,
//          autoFocusSearchBox: true,
//           showSelectedItem: true,
          compareFn: (String i, String s) => i==s,
          label: tr(LocaleKeys.lblLeaveDayType),
          onFind: (String filter) => _suggestion(filter),
          onChanged: (String data) {
            _leaveDateType = data;
          },
          dropdownBuilder: (BuildContext context, String item ) =>
              Container(
                child: (item == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.calendar_today),
                  title: Text( tr(LocaleKeys.lblSelectLeaveDayType) ),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0),
                  title: Text(item),
                  leading: Icon(Icons.calendar_today),
                ),
              ),
          popupItemBuilder: (BuildContext context, String item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item)
                ),
              ),
        )
    );
  }

  Future<List<String>> _suggestion(String query) async{
    List<String> matches = List();
    matches.addAll(leaveTypeList);

    matches.retainWhere((s) {
      String dayType = s;
      if( dayType.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

}

class ApplyLeaveSuccessResponse extends StatefulWidget {
  ApplyLeaveSuccessResponse({Key key, @required this.leaveApplyResponse,
    @required this.sDate, @required this.eDate, @required this.leaveDateType}) : super(key: key);
  final LeaveApplyResponse leaveApplyResponse;
  final DateTime sDate;
  final DateTime eDate;
  final String leaveDateType;

  @override
  _ApplyLeaveSuccessResponseState createState() => _ApplyLeaveSuccessResponseState();
}

class _ApplyLeaveSuccessResponseState extends State<ApplyLeaveSuccessResponse> {


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Text(tr(LocaleKeys.titleLeaveApplicationSuccess)),
        ),
      body: ListView.builder(
        shrinkWrap: false,
        itemBuilder: (BuildContext ctxt, int index) {
          return _getItem( ctxt, index);
        },
        itemCount: widget.leaveApplyResponse.leaves.length+1,
      ),
    );
  }

  Widget _getItem(BuildContext context, int index){
    if( index == 0){
      return Card(
        elevation: 1,
        child: Wrap(
            children: <Widget>[
              Container(
                padding: const EdgeInsets.all(10.0),
                child: Center(
                  child: ClipOval(
                      child: Material(
                        color: Colors.transparent,
                        elevation: 0, // button color
                        child: InkWell(
                          child: SizedBox(
                            width: 120,
                            height: 120,
                            child: Icon(Icons.done_outline, color: Theme.of(context).primaryColor, size: 120,)
                          ),
                        ),
                      )
                  )
                ),
              ),
              Container(
                padding: const EdgeInsets.all(10.0),
                child: ListTile(
                  leading: Badge(
                    badgeContent: Text(
                      '${widget.leaveApplyResponse.status}',
                      style: TextStyle(color: Colors.white),
                    ),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    shape: BadgeShape.square,
                    badgeColor: Theme.of(context).primaryColor,
                    borderRadius: BorderRadius.circular(3),
                    elevation: 3,
                  ),
                  title: Text('${widget.leaveDateType}'),
                  subtitle: Text(
                      '${new DateFormat('dd/MM/yyyy').format(widget.sDate.toLocal())} - ${new DateFormat('dd/MM/yyyy').format(widget.eDate.toLocal())}'
                  ),
                  trailing: Text(
                      widget.leaveApplyResponse.total>1?
                      '${widget.leaveApplyResponse.total.toStringAsFixed(1)} days'
                          :'${widget.leaveApplyResponse.total.toStringAsFixed(1)} day'
                  ),
                ),
              ),
            ]
        ),
      );
    }else{
      return Card(
        elevation: 1,
        child: Container(
          padding: const EdgeInsets.all(10.0),
          child: ListTile(
            title: Text(
                '${new DateFormat('dd/MM/yyyy').format(widget.leaveApplyResponse.leaves[index-1].date.toLocal())}'
            ),
            leading: Text(
                '${widget.leaveApplyResponse.leaves[index-1].type}'
            ),
            trailing: Text(
                widget.leaveApplyResponse.leaves[index-1].status==LeaveApplyItem.STATUS_SUCCESS?
                    'leave':'${widget.leaveApplyResponse.leaves[index-1].status}'
            ),
          ),
        ),
      );
    }
  }
}