
import 'package:badges/badges.dart';
import '/hrm/config/constant.dart';
import '/hrm/model/response/leave_apply_response.dart';
import '/hrm/model/response/leave_detail_response.dart';
import '/hrm/service/leave_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';



class LeaveDetail extends StatefulWidget {
  LeaveDetail({Key key, @required this.guid, this.accessToken,
  this.sharedPrefs}) : super(key: key);
  final String guid;
  final String accessToken;
  final SharedPreferences sharedPrefs;

  @override
  _LeaveDetailState createState() => _LeaveDetailState();
}

class _LeaveDetailState extends State<LeaveDetail> {

  LeaveDetailResponse _detailResponse;
  LeaveService _leaveService = new LeaveService();

  @override
  void initState() {
    super.initState();
    _loadData();
  }

  _loadData() async{
    _detailResponse = await _leaveService.detail(widget.accessToken, widget.guid);
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Leave Detail'),
      ),
      body: _detailResponse==null? Container() : ListView.builder(
        shrinkWrap: false,
        itemBuilder: (BuildContext ctxt, int index) {
          return _getItem( ctxt, index);
        },
        itemCount: _detailResponse.leaves.length+1,
      ),
    );
  }

  Widget _getItem(BuildContext context, int index){
    if( index == 0){
      return Card(
        elevation: 1,
        child: Wrap(
            children: <Widget>[
              Container(
                padding: const EdgeInsets.all(10.0),
                child: Center(
                  child:ClipOval(
                    child: Material(
                      elevation: 8,
                      color: Colors.white, // button color
                      child: InkWell(
                        splashColor: Colors.red, // inkwell color
                        child: SizedBox(
                          width: 120,
                          height: 120,
                          child: _detailResponse.leaveItem.facialGuid!=null?
                          new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${_detailResponse.leaveItem.facialGuid}',
                            headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ) : Icon(Icons.person),
                        ),
                      ),
                    )
                  )
                ),
              ),
              Container(
                margin: EdgeInsets.only(top: 0),
                child: ListTile(
                  title: Text(
                    "${_detailResponse.leaveItem.employeeName}",
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                      fontSize: 24.0,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                  subtitle: Text(
                    _detailResponse.leaveItem.department!=null?
                    "${_detailResponse.leaveItem.department} department" : '',
                  ),
                  leading:Badge(
                    badgeContent: Text(
                        '${_detailResponse.leaveItem.status}',
                          style: TextStyle(color: Colors.white),
                    ),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    shape: BadgeShape.square,
                    badgeColor: Theme.of(context).primaryColor,
                    borderRadius: BorderRadius.circular(3),
                    elevation: 3,
                  ),
                ),
              ),
              Container(
                padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
                child: Wrap(
                  children: <Widget>[
                    Badge(
                      badgeContent: Text(
                        _detailResponse.leaveItem.total>1?
                        '${_detailResponse.leaveItem.total.toStringAsFixed(1)} days'
                            :'${_detailResponse.leaveItem.total.toStringAsFixed(1)} day',
                        style: TextStyle(color: Colors.white),
                      ),
                      toAnimate: true,
                      animationType: BadgeAnimationType.slide,
                      shape: BadgeShape.square,
                      badgeColor: Theme.of(context).primaryColor,
                      borderRadius: BorderRadius.circular(3),
                      elevation: 3,
                    ),
                    Badge(
                      badgeContent: Text(
                          new DateFormat('yyyy').format(_detailResponse.leaveItem.startDate) == new DateFormat('yyyy').format(DateTime.now().toLocal())?
                          '${new DateFormat('dd MMM').format(_detailResponse.leaveItem.startDate)} - ${new DateFormat('dd MMM').format(_detailResponse.leaveItem.endDate)}'
                              :'${new DateFormat('dd/MM/yy').format(_detailResponse.leaveItem.startDate)} - ${new DateFormat('dd/MM/yy').format(_detailResponse.leaveItem.endDate)}'
                      ),
                      toAnimate: true,
                      animationType: BadgeAnimationType.slide,
                      shape: BadgeShape.square,
                      badgeColor: Colors.white,
                      borderRadius: BorderRadius.circular(0),
                      elevation: 0,
                    ),
                    Badge(
                      badgeContent: Text(
                        '${_detailResponse.leaveItem.leaveType}',
                        style: TextStyle(fontWeight: FontWeight.w600),
                      ),
                      toAnimate: true,
                      animationType: BadgeAnimationType.slide,
                      shape: BadgeShape.square,
                      badgeColor: Colors.white,
                      borderRadius: BorderRadius.circular(0),
                      elevation: 0,
                    ),
                  ],
                ),
              ),
              Container(
                margin: EdgeInsets.only(top: 0),
                child: ListTile(
                  leading: Icon(Icons.info_outline),
                  title: Text('Reason'),
                  subtitle: Text(
                      _detailResponse.leaveItem.details==null?_detailResponse.leaveItem.leaveType:_detailResponse.leaveItem.details
                  ),
                ),
              ),
            ]
        ),
      );
    }else{
      return Card(
        elevation: 1,
        child: Container(
          padding: const EdgeInsets.all(10.0),
          child: ListTile(
            leading: Text(
                '${new DateFormat('dd/MM/yyyy').format(_detailResponse.leaves[index-1].date.toLocal())}'
            ),
            title: Text(
                '${_detailResponse.leaves[index-1].type}'
            ),
            trailing: Text(
                _detailResponse.leaves[index-1].status==LeaveApplyItem.STATUS_SUCCESS?
                'leave':'${_detailResponse.leaves[index-1].status}'
            ),
          ),
        ),
      );
    }
  }
}