

import '/hrm/model/response/leave_balance_response.dart';
import '/hrm/service/leave_service.dart';
import 'package:flutter/material.dart';
import 'package:percent_indicator/circular_percent_indicator.dart';


class LeaveEntitlement extends StatefulWidget {
  LeaveEntitlement({Key key, this.accessToken, this.employeeGuid}) : super(key: key);
  final String accessToken;
  final String employeeGuid;

  @override
  _LeaveEntitlementState createState() => _LeaveEntitlementState();
}

class _LeaveEntitlementState extends State<LeaveEntitlement>{

  LeaveService _leaveService = new LeaveService();
  LeaveBalanceResponse _balanceResponse;
  List<LeaveTypeItem> _items=[];


  @override
  void initState() {
    super.initState();
    _loadData();
  }

  _loadData() async{
    _balanceResponse = await _leaveService.balance( widget.accessToken, widget.employeeGuid);
    if( _balanceResponse.success){
      setState(() {
        _items = _balanceResponse.items;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 0, right: 0),
      itemBuilder: (c, i) => _item(_items[i], c, i+1 == _items.length),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _items.length,
    );
  }

  Widget _item(LeaveTypeItem typeItem, BuildContext context, bool isEnd){
    return
      Container(
        margin: EdgeInsets.all(8),
        child:  Wrap(
          children: <Widget>[
            Card(
              elevation: 1,
              child:Container(
                padding: EdgeInsets.only(top: 8, bottom: 16),
              child: Wrap(
                children: <Widget>[
                  ListTile(
                    title: new Text(
                    "${typeItem.leaveType}",
                    ),
                    subtitle: Text(
                        'Leave days for current leave period is ${typeItem.total} days. ${typeItem.used} days have been used, ${typeItem.pending} days are pending and ${typeItem.total - typeItem.used+typeItem.pending} days are available.'
                    ),
                  ),
                  Center(
                    child: new CircularPercentIndicator(
                      radius: 120.0,
                      lineWidth: 15.0,
                      animation: true,
                      animationDuration: 2000,
                      percent: 1 - ( (typeItem.used+typeItem.pending ) / typeItem.total ),
                      center: new Text(
                        "${typeItem.total - (typeItem.used+typeItem.pending)} days",
                        style: new TextStyle(fontWeight: FontWeight.bold, fontSize: 16),
                      ),
                      circularStrokeCap: CircularStrokeCap.round,
                      progressColor: Theme.of(context).primaryColor,
                      footer: new Text(
                        "Total days: ${typeItem.total} days",
                        style: new TextStyle(),
                      ),
                    ),
                  ),
                ]
              )
              )
            ),
            isEnd? SizedBox(
              height: 330,
            ) : Container()
          ]
        )
    );
  }
}