

import 'package:badges/badges.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'apply_leave.dart';
import 'leave_approved.dart';
import 'leave_entitlement.dart';
import 'leave_pending.dart';
import 'leave_rejected.dart';



class LeaveHome extends StatefulWidget {
  LeaveHome({Key key, this.pendingLeaveCount, this.onChange}) : super(key: key);
  final int pendingLeaveCount;
  final Function onChange;
  @override
  _LeaveHomeState createState() => _LeaveHomeState();
}

class _LeaveHomeState extends State<LeaveHome> {

  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  EmployeeSearchResponse _employeeSearchResponse;
  EmployeeSearchItem _employeeSearchItem;
  String _accessToken;
  bool _search = false;
  DateTime _startDate;
  DateTime _endDate;
  String _employeeGuid;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  int _selectedIndex = 0;
  String _title = tr(LocaleKeys.lblLeaveApproved);
  int _pendingLeaveCount = 0;

  @override
  void initState() {
    super.initState();
    _loadData();
  }

  _loadData() async{
    if( _currentUser.facialGUID == null){
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
    }
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    EmployeeService _employeeService = new EmployeeService();
    _employeeSearchResponse = await _employeeService.search(_accessToken, '', 0, 100000);

    setState(() {_pendingLeaveCount = widget.pendingLeaveCount;});

  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(_title),
            bottom: _search?_createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            actions: <Widget>[
              _selectedIndex==3? new Container() : FlatButton.icon(
                  onPressed:(){
                    setState(() {
                      _reset();
                      _search = !_search;
                    });
                  },
                  icon: _search? Icon(Icons.cancel, color: Colors.white,)
                      : Icon(Icons.search, color: Colors.white,),
                  label: Text(
                    _search? tr(LocaleKeys.btnClose) : tr(LocaleKeys.btnSearch),
                    style:  TextStyle(color: Colors.white),
                  )
              )
            ],
          ),
          body: _createBody( ),
          bottomNavigationBar:
          BottomNavigationBar(
            items: <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.done_all),
                label: tr(LocaleKeys.lblLeaveApproved),
              ),
              BottomNavigationBarItem(
                icon: _authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('SUPERVISOR_WRITE')
                    && _pendingLeaveCount>0? Badge(
                    position: BadgePosition.topEnd(top: -20, end: -20),
                    badgeContent: Text('$_pendingLeaveCount',style: TextStyle(color: Colors.white),),
                    toAnimate: true,
                    animationType: BadgeAnimationType.slide,
                    child: Icon(Icons.help)
                ) : Icon(Icons.help),
                label: tr(LocaleKeys.lblLeavePending),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.do_not_disturb_on),
                label: tr(LocaleKeys.lblLeaveRejected),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.calendar_today),
                label: tr(LocaleKeys.lblLeaveEntitlement),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            onTap: _onItemTapped,
          ),
          floatingActionButton: FloatingActionButton.extended(
            onPressed: () async{
              String result = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) => ApplyLeave( ),
                  )
              );
              if( result != null && result == 'OK'){
                setState(() {
                  _selectedIndex = 1;
                });
              }
            },
            label: Text(
              tr(LocaleKeys.lblApplyLeave),
              style: TextStyle(color: Colors.white),
            ),
            icon: Icon(Icons.email),
            backgroundColor: Theme.of(context).primaryColor,
          ),
        )
    );
  }

  Widget _createBody(){
    if( _selectedIndex == 0 && !_search ){
      return LeaveApproved(
        startDate: _startDate,
        endDate: _endDate,
        employeeGuid: _employeeGuid,
      );
    }else if( _selectedIndex == 1 && !_search ){
      return LeavePending(
        startDate: _startDate,
        endDate: _endDate,
        employeeGuid: _employeeGuid,
        onChange: (pendingLeaveCount){
          setState(() {
            _pendingLeaveCount = pendingLeaveCount;
            widget.onChange(pendingLeaveCount);
          });
        },
      );
    }else if( _selectedIndex == 2  && !_search){
      return LeaveRejected(
        startDate: _startDate,
        endDate: _endDate,
        employeeGuid: _employeeGuid,
      );
    }else if( _selectedIndex == 3  && !_search){
      return LeaveEntitlement(
        accessToken: _accessToken,
        employeeGuid: _currentUser.employeeGUID,
      );
    }else{
      return new Container();
    }

  }

  void _onItemTapped(int index) {
    setState(() {
      _selectedIndex = index;
      if( _selectedIndex == 0 ){
        _title = tr(LocaleKeys.lblLeaveApproved);
        _reset();
      }else if( _selectedIndex == 1){
        _title = tr(LocaleKeys.lblLeavePending);
        _reset();
      }else if( _selectedIndex == 2){
        _title = tr(LocaleKeys.lblLeaveRejected);
        _reset();
      }else{
        _title = tr(LocaleKeys.lblLeaveEntitlement);
        _reset();
        _search = false;
      }
    });
  }

  _reset(){
    _employeeSearchItem = null;
    _employeeGuid = null;
    _startDate = null;
    _controller_start.text = '';
    _endDate = null;
    _controller_end.text = '';
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 400.0),
      child: SizedBox(
          height: 400.0,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 0,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child:new Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ')?_employeeSelection(context) : new Container(),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.search, tr(LocaleKeys.btnSearch) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnReset)),
                            ],
                          ),
                        ),
                        Center(
                          child: Container(
                              width: MediaQuery.of(context).size.width * 0.9,
                              child: FlatButton.icon(
                                  color: Theme.of(context).primaryColor,
                                  onPressed: (){showMessage("Current version doesn't support yet!");},
                                  icon: Icon(Icons.file_upload,size: 32,color: Colors.white,),
                                  label: Text('Export to Excel',
                                    style:  TextStyle(color: Colors.white),
                                  )
                              )
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

  bool isValidDob(String dob) {
    if (dob.isEmpty) return false;
    var d = convertToDate(dob);
    return d != null && d.isAfter(new DateTime.fromMillisecondsSinceEpoch( new DateTime.now().millisecondsSinceEpoch - (1000 * 60 * 60 * 25) ));
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == 'Cancel' || label == tr(LocaleKeys.btnReset) || label == 'Later' ){
              setState(() {
                _reset();
                _search = !_search;
              });
            }else if( label==tr(LocaleKeys.btnSearch)){
              setState(() {
                if( _controller_start.text.isEmpty || _controller_end.text.isEmpty){
                  _startDate = null;
                  _endDate = null;
                }
                _search = !_search;
              });
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 5, left: 10, right: 10),
        child: DropdownSearch<EmployeeSearchItem>(
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _employeeSearchItem = data;
            if( _employeeSearchItem != null){
              _employeeGuid = _employeeSearchItem.guid;
            }else{
              _employeeGuid = null;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeSearchItem>> _employeeSuggestion(String query) async{
    List<EmployeeSearchItem> matches = List();
    if( _employeeSearchResponse.employees == null)
      return matches;
    matches.addAll(_employeeSearchResponse.employees);

    matches.retainWhere((s) {
      EmployeeSearchItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }
}
