

import 'package:badges/badges.dart';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/leave_search_request.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/leave_search_response.dart';
import '/hrm/service/leave_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'leave_detail.dart';


class LeavePending extends StatefulWidget {
  LeavePending({Key key, this.startDate, this.endDate, this.employeeGuid,
    this.onChange}) : super(key: key);
  final DateTime startDate;
  final DateTime endDate;
  final String employeeGuid;
  final Function onChange;

  @override
  _LeavePendingState createState() => _LeavePendingState();
}

class _LeavePendingState extends State<LeavePending>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  LeaveSearchResponse _leaveSearchResponse;
  String _accessToken;
  LeaveService _leaveService = new LeaveService();
  LeaveSearchRequest _leaveSearchRequest = new LeaveSearchRequest();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  List<LeaveSearchItem> _leaves = [];
  SharedPreferences _sharedPrefs;

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null) {
      await _currentUser.fromSharedPreferences();
      await _authorityResponse.fromSharedPreferences();
      _sharedPrefs = await SharedPreferences.getInstance();
    }
    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _leaveSearchRequest.first = _page;
    _leaveSearchRequest.max = Constant.ROW_PER_PAGE;
    _leaveSearchRequest.status = LeaveSearchItem.STATUS_PENDING;

    if( widget.startDate!=null && widget.endDate!=null) {
      _leaveSearchRequest.startDate = widget.startDate;
      _leaveSearchRequest.endDate = widget.endDate;
    }
    _leaveSearchRequest.employeeGuid = widget.employeeGuid;

    _leaveSearchResponse = await _leaveService.search(_accessToken, _leaveSearchRequest);
    if( _leaveSearchResponse!=null && _leaveSearchResponse.leaves!=null){

      if( refresh && _leaves.isNotEmpty)
        _leaves.removeRange(0, _leaves.length);

      _leaveSearchResponse.leaves.forEach((element) {
        _leaves.add(element);
      });
    }

    try{
      if( _leaveSearchResponse != null)
        widget.onChange(_leaveSearchResponse.totalRecords);
    }catch(e){}

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => LeaveResultItem(
        post: _leaves[i],
        accessToken: _accessToken,
        authorityResponse: _authorityResponse,
        leaveService: _leaveService,
        isEnd: i+1 == _leaves.length,
        currentEmployeeGuid: _currentUser.employeeGUID,
        onChange: () async{
          _page = 0;
          await _getLoadData( true );
        },
        sharedPrefs: _sharedPrefs,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _leaves.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class LeaveResultItem extends StatefulWidget {

  LeaveResultItem({Key key, @required this.post, @required this.accessToken, 
    @required this.authorityResponse, @required this.leaveService, this.onChange,
    @required this.sharedPrefs, @required this.isEnd, @required this.currentEmployeeGuid}) : super(key: key);
  final LeaveSearchItem post;
  final String accessToken;
  final AuthorityResponse authorityResponse;
  final LeaveService leaveService;
  final Function onChange;
  SharedPreferences sharedPrefs;
  final bool isEnd;
  final String currentEmployeeGuid;

  @override
  _LeaveResultItemState createState() => _LeaveResultItemState();

}

class _LeaveResultItemState extends State<LeaveResultItem> {

  final TextEditingController _controller_comment = new TextEditingController();
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(

        child: Wrap(
            children: <Widget>[
              Card(
                  elevation: 1,
                  child: Wrap(
                    children: <Widget>[
                      ListTile(
                          title: Text(widget.post.employeeName, style: TextStyle(color: Theme.of(context).primaryColor),),
                          subtitle: Text(
                              widget.post.department!=null?
                              '${widget.post.department} department'
                                  :''
                          ),
                          leading: ClipOval(
                              child: Material(
                                elevation: 1,
                                color: Colors.transparent, // button color
                                child: InkWell(
                                  splashColor: Colors.red, // inkwell color
                                  child: SizedBox(
                                    width: 48,
                                    height: 48,
                                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.facialGuid}',
                                      headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
                                  ),
                                ),
                              )
                          ),
                          trailing: Icon(Icons.keyboard_arrow_right),
                        onTap: (){
                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (BuildContext context) => LeaveDetail(
                                  accessToken: widget.accessToken,
                                  guid: widget.post.guid,
                                  sharedPrefs: widget.sharedPrefs,
                                ),
                              )
                          );
                        },
                        ),
                      Container(
                        padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
                          child: Wrap(
                            children: <Widget>[
                              Badge(
                                badgeContent: Text(
                                  widget.post.total>1?
                                  '${widget.post.total.toStringAsFixed(1)} days'
                                      :'${widget.post.total.toStringAsFixed(1)} day',
                                  style: TextStyle(color: Colors.white),
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Theme.of(context).primaryColor,
                                borderRadius: BorderRadius.circular(3),
                                elevation: 3,
                              ),
                              Badge(
                                badgeContent: Text(
                                    new DateFormat('yyyy').format(widget.post.startDate) == new DateFormat('yyyy').format(DateTime.now().toLocal())?
                                    '${new DateFormat('dd MMM').format(widget.post.startDate)} - ${new DateFormat('dd MMM').format(widget.post.endDate)}'
                                        :'${new DateFormat('dd/MM/yy').format(widget.post.startDate)} - ${new DateFormat('dd/MM/yy').format(widget.post.endDate)}'
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Colors.white,
                                borderRadius: BorderRadius.circular(0),
                                elevation: 0,
                              ),
                              Badge(
                                badgeContent: Text(
                                  '${widget.post.leaveType}',
                                  style: TextStyle(fontWeight: FontWeight.w600),
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Colors.white,
                                borderRadius: BorderRadius.circular(0),
                                elevation: 0,
                              ),
                            ],
                          ),
                      ),
                      _createActionButtons( context, widget.post ),
                      _showConfirm( context, widget.post ),
                    ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );

  }

  Widget _createActionButtons( BuildContext context, LeaveSearchItem post ){
    if( widget.authorityResponse.authorities !=null
        && widget.authorityResponse.authorities.contains('SUPERVISOR_READ')
        && post.action == LeaveSearchItem.STATUS_PENDING ) {
      return ButtonBar(
        children: <Widget>[
          _buildButtons(
              Theme.of(context).primaryColor, Icons.done_all, 'Approve', context, post),
          _buildButtons(
              Colors.red[600], Icons.do_not_disturb_on, 'Reject', context, post),
          widget.post.employeeGuid==widget.currentEmployeeGuid?
          _buildButtons(
              Colors.black54, Icons.cancel, 'Cancel', context, post) : Container(),
        ],
      );
    }else if( widget.authorityResponse.authorities !=null
        && !widget.authorityResponse.authorities.contains('SUPERVISOR_READ')
        && post.action == LeaveSearchItem.STATUS_PENDING ){
      return ButtonBar(
        children: <Widget>[
          _buildButtons(
              Colors.black54, Icons.cancel, 'Cancel', context, post),
        ],
      );
    }else{
      return Container();
    }
  }

  Column _buildButtons(Color color, IconData icon, String label, BuildContext context, LeaveSearchItem post) {
    return Column(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        RaisedButton.icon(
          elevation: 0.0,
          icon: Icon(icon, color: color),
          color: Colors.grey[200],
          onPressed:() async{
            _actionButtons( label, context, post);
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.black87,
            ),
          ),
        ),
      ],
    );

  }

  Future<void> _actionButtons( String label, BuildContext context, LeaveSearchItem post) async {

    if( label == 'Approve'){
      DateTime current = DateTime(
        DateTime.now().toLocal().year, DateTime.now().toLocal().month, DateTime.now().toLocal().day, 0, 00, 00);
      if( post.endDate.isBefore( current )){
        showMessage('Leave application expired on ${new DateFormat('dd/MM/yy').format(post.endDate)}');
      }else {
        setState(() {
          post.action = LeaveSearchItem.STATUS_APPROVED;
        });
      }
    }else if( label == 'Reject' ){
      setState(() {
        post.action = LeaveSearchItem.STATUS_REJECTED;
      });
    }else if( label == 'Cancel' ){
      setState(() {
        post.action = LeaveSearchItem.STATUS_CANCELLED;
      });
    }else if( label == 'Confirm' ){
      _actionCompleted( context, post);
    }else if( label == 'Later' ){
      setState(() {
        post.action = LeaveSearchItem.STATUS_PENDING;
      });
    }
  }

  Widget _showConfirm( BuildContext context, LeaveSearchItem post ){

    if( post.action == LeaveSearchItem.STATUS_APPROVED || post.action == LeaveSearchItem.STATUS_REJECTED || post.action == LeaveSearchItem.STATUS_CANCELLED){
      return new Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Container(
              margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
              child: new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.done),
                  hintText: '',
                  labelText: 'Comment',
                ),
                //onSaved: (val) => _name = val,
                keyboardType: TextInputType.text,
                //validator: (val) => val.isEmpty ? _controller_labour.text = '0' : (double.parse(val)>=0 ? null : 'Invalid data'),
                controller: _controller_comment,
                onChanged: (val){ },
                //onEditingComplete: (){},
              ),
            ),

            ButtonBar(
              children: <Widget>[
                _buildButtons( Theme.of(context).primaryColor, Icons.done, 'Confirm', context, post),
                _buildButtons( Colors.grey[600], Icons.settings_backup_restore, 'Later', context, post),
              ],
            )
          ]
      );
    }else{
      return new Container();
    }
  }

  Future<void> _actionCompleted( BuildContext context, LeaveSearchItem post) async {
    bool result = await _showConfirmDialog(context);
    if ( ! result ){
      setState(() {post.action = LeaveSearchItem.STATUS_PENDING;});
      return;
    }

    String msgTitle = await widget.leaveService.changeStatus(widget.accessToken, post.guid, post.action, _controller_comment.text);

        if ( msgTitle.toLowerCase().contains('successful')) {
          setState(() {
            post.status = post.action;
            post.action = '';
          });
          widget.onChange();
        }else {
          setState(() {
            post.action = LeaveSearchItem.STATUS_PENDING;
          });
        }


    final snackBar = SnackBar(
      content: Text(msgTitle),
      duration: const Duration(seconds: 3),
      action: SnackBarAction(
        label: 'OK',
        onPressed: () { },
      ),
    );
    Scaffold.of(context).showSnackBar(snackBar);
    setState(() { });
  }
  
  Future<bool> _showConfirmDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm"),
          content: new Text("Are you sure to perform this action?"),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }
  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
}