

import 'package:badges/badges.dart';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/leave_search_request.dart';
import '/hrm/model/response/leave_search_response.dart';
import '/hrm/service/leave_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'leave_detail.dart';


class LeaveRejected extends StatefulWidget {
  LeaveRejected({Key key, this.startDate, this.endDate, this.employeeGuid}) : super(key: key);
  final DateTime startDate;
  final DateTime endDate;
  final String employeeGuid;

  @override
  _LeaveRejectedState createState() => _LeaveRejectedState();
}

class _LeaveRejectedState extends State<LeaveRejected>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  LeaveSearchResponse _leaveSearchResponse;
  String _accessToken;
  LeaveService _leaveService = new LeaveService();
  LeaveSearchRequest _leaveSearchRequest = new LeaveSearchRequest();
  User _currentUser = new User();
  List<LeaveSearchItem> _leaves = [];
  SharedPreferences _sharedPrefs;

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );
    if(_sharedPrefs == null)
      _sharedPrefs = await SharedPreferences.getInstance();

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _leaveSearchRequest.first = _page;
    _leaveSearchRequest.max = Constant.ROW_PER_PAGE;
    _leaveSearchRequest.status = LeaveSearchItem.STATUS_REJECTED;

    if( widget.startDate!=null && widget.endDate!=null) {
      _leaveSearchRequest.startDate = widget.startDate;
      _leaveSearchRequest.endDate = widget.endDate;
    }
    _leaveSearchRequest.employeeGuid = widget.employeeGuid;

    _leaveSearchResponse = await _leaveService.search(_accessToken, _leaveSearchRequest);
    if( _leaveSearchResponse!=null && _leaveSearchResponse.leaves!=null){

      if( refresh && _leaves.isNotEmpty)
        _leaves.removeRange(0, _leaves.length);

      _leaveSearchResponse.leaves.forEach((element) {
        _leaves.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => LeaveResultItem(
        post: _leaves[i],
        accessToken: _accessToken,
        sharedPrefs: _sharedPrefs,
        isEnd: i+1 == _leaves.length,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Colors.transparent,
        );
      },
      itemCount: _leaves.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class LeaveResultItem extends StatefulWidget {

  LeaveResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.isEnd}) : super(key: key);
  final LeaveSearchItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final bool isEnd;

  @override
  _LeaveResultItemState createState() => _LeaveResultItemState();

}

class _LeaveResultItemState extends State<LeaveResultItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                  elevation: 1,
                  child: Wrap(
                      children: <Widget>[
                        ListTile(
                          title: Text(widget.post.employeeName, style: TextStyle(color: Theme.of(context).primaryColor),),
                          subtitle: Text(
                              widget.post.department!=null?
                              '${widget.post.department} department'
                                  :'Unknown department'
                          ),
                          leading: ClipOval(
                              child: Material(
                                elevation: 1,
                                color: Colors.transparent, // button color
                                child: InkWell(
                                  splashColor: Colors.red, // inkwell color
                                  child: SizedBox(
                                    width: 48,
                                    height: 48,
                                    child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.facialGuid}',
                                      headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
                                  ),
                                ),
                              )
                          ),
                          trailing: Icon(Icons.do_not_disturb_on, color: Colors.red[600],),
                          onTap: (){
                            Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (BuildContext context) => LeaveDetail(
                                    accessToken: widget.accessToken,
                                    guid: widget.post.guid,
                                    sharedPrefs: widget.sharedPrefs,
                                  ),
                                )
                            );
                          },
                        ),
                        Container(
                          padding: const EdgeInsets.only(left: 16, right: 16, bottom: 16),
                          child: Wrap(
                            children: <Widget>[
                              Badge(
                                badgeContent: Text(
                                  widget.post.total>1?
                                  '${widget.post.total.toStringAsFixed(1)} days'
                                      :'${widget.post.total.toStringAsFixed(1)} day',
                                  style: TextStyle(color: Colors.white),
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Theme.of(context).primaryColor,
                                borderRadius: BorderRadius.circular(3),
                                elevation: 3,
                              ),
                              Badge(
                                badgeContent: Text(
                                    new DateFormat('yyyy').format(widget.post.startDate) == new DateFormat('yyyy').format(DateTime.now().toLocal())?
                                    '${new DateFormat('dd MMM').format(widget.post.startDate)} - ${new DateFormat('dd MMM').format(widget.post.endDate)}'
                                        :'${new DateFormat('dd/MM/yy').format(widget.post.startDate)} - ${new DateFormat('dd/MM/yy').format(widget.post.endDate)}'
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Colors.white,
                                borderRadius: BorderRadius.circular(0),
                                elevation: 0,
                              ),
                              Badge(
                                badgeContent: Text(
                                  '${widget.post.leaveType}',
                                  style: TextStyle(fontWeight: FontWeight.w600),
                                ),
                                toAnimate: true,
                                animationType: BadgeAnimationType.slide,
                                shape: BadgeShape.square,
                                badgeColor: Colors.white,
                                borderRadius: BorderRadius.circular(0),
                                elevation: 0,
                              ),
                            ],
                          ),
                        )
                      ]
                  )
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )

      //padding: const EdgeInsets.all(10),
      //height: 100,
//        child: Card(
//          elevation: 0,
//          child: Wrap(
//            children: <Widget>[
//              Container(
//              //padding: EdgeInsets.only(top:10),
//                child: ListTile(
//                  title: Text(widget.post.employeeName, style: TextStyle(color: Theme.of(context).primaryColor),),
//                  subtitle: Text(
//                      '${widget.post.leaveType}: ${new DateFormat('dd/MM/yy').format(widget.post.startDate)}~${new DateFormat('dd/MM/yy').format(widget.post.endDate)}'
//                  ),
//                  leading: ClipOval(
//                      child: Material(
//                        elevation: 1,
//                        color: Colors.transparent, // button color
//                        child: InkWell(
//                          splashColor: Colors.red, // inkwell color
//                          child: SizedBox(
//                            width: 48,
//                            height: 48,
//                            child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.facialGuid}',
//                              headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
//                          ),
//                        ),
//                      )
//                  ),
//                  trailing: Icon(Icons.do_not_disturb_on, color: Colors.red[600],),
//                  onTap: (){},
//                )
//              ),
//              widget.isEnd? SizedBox(
//                height: 150,
//              ) : Container()
//            ]
//          )
//        )
    );

  }
}