

import 'dart:async';
import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/response/recognize_result.dart';
import '/hrm/service/face_detector_service.dart';
import '/hrm/service/face_recognization_service.dart';
import 'package:flutter/foundation.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import '/hrm/imageprocessing/flutter_camera.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:path_provider/path_provider.dart';
import 'package:threading/threading.dart';


class FaceRecognize extends StatefulWidget {
  FaceRecognize({Key key}) : super(key: key);
  @override
  _FaceRecognizePageState createState() => _FaceRecognizePageState();
}

class _FaceRecognizePageState extends State<FaceRecognize> {

  final _scanKey = GlobalKey<CameraMlVisionState>();
  bool _processing = false;
  bool willStop = false;
  bool _foundFace = false;
  Color progressColor = Colors.transparent;
  CameraLensDirection cameraLensDirection = CameraLensDirection.front;
//  FaceDetector detector;
  RecognizeResult _recognizeResult;
  FaceRecognizationService _faceRecognizationService;
  FaceDetectorService faceDetectorService = new FaceDetectorService();
  bool _detected = false;

  @override
  initState() {
    super.initState( );
    faceDetectorService.init();
    _faceRecognizationService = new FaceRecognizationService( );
    // listenThread( );
  }

  void listenThread( ) async{
    var thread = new Thread(listen);
    await thread.start();
    await thread.join();
    if( Constant.ENABLE_LOGS)
      print("Thread terminated");
  }

  Future listen( ) async {

    while ( !willStop ) {
      await Thread.sleep( 50 );
      if( !_processing && mounted && !_foundFace)
        _processPhoto( );
      await Thread.sleep( 50 );
    }
    await Thread.sleep( 1 * 1000 );
    if( Constant.ENABLE_LOGS)
      print("Thread sleep");
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
      appBar: AppBar(
        title: Text(tr(LocaleKeys.msgFaceRecognizing)),
          bottom: _createProgressIndicator(),
      ),
      body: SizedBox.expand(
        child: CameraMlVision<List<Object>>(
          key: _scanKey,
          cameraLensDirection: cameraLensDirection,
          resolution: ResolutionPreset.high,
          overlayBuilder: (c) {
            return Positioned(
                bottom: 16.0,
                left: 16.0,
                right: 16.0,
                child: MaterialButton(
                  onPressed: () async{await _processPhoto();},
                  color: Theme.of(context).primaryColor,
                  textColor: Colors.white,
                  child: Icon(
                    Icons.camera_alt,
                    size: 32,
                  ),
                  padding: EdgeInsets.all(16),
                  shape: CircleBorder(),
                )
            );
            // return Container();
          },
          overlayBuilder1: (c){
            return _actionPanel( context );
          },
          onDispose: () { },
        ),
      ),
    )
    );
  }

  Widget _actionPanel( BuildContext context ){
    return _recognizeResult!=null && _recognizeResult.name != null ?
    Container(
      margin: EdgeInsets.only(top: (MediaQuery.of(context).size.height * 0.07), bottom: 50, left: 10, right: 10),
      child: Card(
        color: Color.fromRGBO(250, 250, 250, .7),
        elevation: 1,
        child: new Column(
          children: <Widget>[
            Container(
              padding: const EdgeInsets.all(10.0),
              margin: EdgeInsets.only(top: 100),
              child: ClipOval(
                  child: Material(
                    color: Colors.transparent,
                    elevation: 0, // button color
                    child: InkWell(
                      child: SizedBox(width: 120,
                        height: 120,
                        child: SvgPicture.asset( 'assets/images/tick.svg', semanticsLabel: 'Recognize',height: 120, width: 120, ),
                      ),
                    ),
                  )
              ),
            ),
            Container(
              padding: const EdgeInsets.only(top: 20.0),
              child: Text('${_recognizeResult.name}', softWrap: true,
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                  color: Theme.of(context).primaryColor,
                  fontSize: 24,
                ),
              ),
            ),
            Container(
              padding: const EdgeInsets.all(10.0),
              child: Divider(
                color: Theme.of(context).primaryColor,
                thickness: 1,
              ),
            ),
          ],
        ),
      ),
    )
        : Positioned(
      top: 20.0,
      left: 10.0,
      right: 10.0,
      child: !_detected ? new Container()
          : Card(
        color: Color.fromRGBO( 0, 00, 0, .7),
        elevation: 2.0,
        child:Column(
          children: <Widget>[
            Image(image: AssetImage('assets/images/face_scan.gif')),
            Container(
              margin: const EdgeInsets.only( bottom: 15.0, left: 5, right: 5),
              child: ListTile(
                title:Text(
                  'Recognizing your face ....',
                  softWrap: true,
                  style: TextStyle(
                    fontWeight: FontWeight.w400,
                    color: Colors.white,
                    fontSize: 14,
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  @override
  void deactivate() {
    willStop = true;
    super.deactivate();
  }

  @override
  void dispose() {
    try {
      if (_scanKey.currentState.mounted) {
        _scanKey.currentState.dispose();
      }
    }catch(e){}
    willStop = true;
    try {
      faceDetectorService.dispose();
    }catch(e){}

    super.dispose();
  }


  Future<void> _processPhoto( ) async{
    try {
          if( _scanKey.currentState == null ) return;
          if( _processing ) return;
          if( !mounted ) return;

          setState(() {
            progressColor = Colors.white;
            _processing = true;
            _detected = false;
          });

          final Directory extDir = await getApplicationDocumentsDirectory();
          final String tmpDir = '${extDir.path}/Pictures/flutter_test';
          await Directory(tmpDir).create(recursive: true);
          String photoPath = '$tmpDir/${DateTime.now()}.jpg';

          if( _scanKey.currentState != null)
            photoPath = await _scanKey.currentState.takePhoto( );


          DetectionResult detectionResult;
          try{
            detectionResult = await faceDetectorService.detectedFaceAsBase64String(tmpDir, photoPath, mounted, true);
          }catch(e){}

          if( detectionResult != null ){
            setState(() {
              progressColor = Colors.green;
              _detected = true;
            });
            _recognizeResult = await _faceRecognizationService.recognize( detectionResult.detectedFace, true );

            if( _recognizeResult == null ){
              _foundFace = false;
              _processing = false;
              setState(() {
                progressColor = Colors.red[600];
              });
              return;
            }

            if( _recognizeResult.result ) {
              if( Constant.ENABLE_LOGS)
                print(_recognizeResult.name);
              setState(() {
                willStop = true;
                _foundFace = true;
                progressColor = Colors.transparent;
              });

              new ThreadTimer(new Duration(milliseconds: 1000), (){Navigator.pop(context, _recognizeResult);});

            }else{
              setState(() {
                progressColor = Colors.red[600];
                _foundFace = false;
                _processing = false;
              });
            }

          }else{
            try{
              new File( photoPath ).deleteSync();
            }catch(e){}
            setState(() {
              progressColor = Colors.red[600];
              _foundFace = false;
              _processing = false;
            });
          }
    }catch( e){
      _processing = false;
      if( Constant.ENABLE_LOGS)
        print(e);
    }
    _processing = false;
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}