import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/services.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/kiosk_data.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/account_brief.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/organizations_by_face.dart';
import '/hrm/model/response/recognize_result.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/firebase_service.dart';
import '/hrm/service/organization_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import '/hrm/view/home/home.dart';
import '/hrm/view/login/user_login.dart';
import '/hrm/view/organization/activate_organization.dart';
import '/hrm/view/organization/organization_register.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'face_recognize.dart';
import '../kiosk/active_kiosk.dart';
import '../kiosk/kiosk_home.dart';
import '../home/organization_list_by_face.dart';
import 'dart:io';
import 'package:local_auth/local_auth.dart';
import 'setup_private_connection.dart';
import '/delivery/service/user_service.dart' as deliUser;



class LoginSelection extends StatefulWidget {
  LoginSelection({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _LoginSelectionPageState createState() => _LoginSelectionPageState( );
}

class _LoginSelectionPageState extends State<LoginSelection> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  Language selectedLanguage;
  List<Language> languages = <Language> [new Language("မြန်မာ", SvgPicture.asset( 'assets/images/myanmar.svg', semanticsLabel: 'My',height: 24, width: 24, )),
    Language('English', SvgPicture.asset( 'assets/images/uk.svg', semanticsLabel: 'En',height: 24, width: 24, ))];

  Color progressColor = Colors.transparent;
  FirebaseMessage _firebaseMessage = FirebaseMessage( );
  bool _processDone = false;
  bool _shutterSoundNotice = true;
  SharedPreferences _sharedPrefs;
  int _doubleTapCount = 0;

  @override
  void initState() {
    super.initState();
    _checkLogin( );
    selectedLanguage = languages[1];
  }

  void _checkLogin() async{

    setState(() {
      progressColor = Colors.white;
      _processDone = true;
    });

    try {
      if( await getFirebaseToken( ) == null)
        await _firebaseMessage.initMessaging();
    }catch(e){}

    _sharedPrefs = await SharedPreferences.getInstance();
    if( _sharedPrefs.getString(Constant.CURRENT_USER_DATA) != null ){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: ( BuildContext context ) => Home(
              title: '${Constant.APP_NAME}',
            ),
          )
      );
    }

    deliUser.UserService userService = new deliUser.UserService( );
    String userAccessToken = await userService.hrAuthorization( );

    if( userAccessToken != null ){
      setState(() {
        progressColor = Colors.white;
        _processDone = true;
      });
      showMessage('Login success, a moment please', context);

      UserService userService = new UserService( );
      AccountBrief accountBrief =await userService.getBrief(userAccessToken);
      if( Constant.ENABLE_LOGS)
        print(userAccessToken);
      await gotoOrgSelection( userAccessToken, accountBrief.facialGUID );

    }else{
      setState(() {
        progressColor = Colors.transparent;
        _processDone = false;
      });
      showMessage('Login failed! you have no authority to use HR Services. Please contact your system administrator.', context);
    }


    // if( _sharedPrefs.getString(Constant.CURRENT_KIOSK_DATA) != null ){
    //   Navigator.pushReplacement(
    //       context,
    //       MaterialPageRoute(
    //         builder: (BuildContext context) => KioskHome( ),
    //       )
    //   );
    // }

    try{
      String lang = await getLanguage();
        setState(() {
          if( lang == 'en')
            selectedLanguage = languages[1];
          else
            selectedLanguage = languages[0];
        });

    }catch(e){}

    if( Platform.isIOS)
      _shutterSoundNotice = await getCameraShutterSoundNotice( );
  }

  //https://www.developerlibs.com/2019/09/flutter-drop-down-menu-list-example.html
  Widget dropdownWidget() {
    return new DropdownButtonHideUnderline(
      child: DropdownButton<Language>(
      //hint:  Text("Select item"),
      value: selectedLanguage,
      onChanged: (Language value) {
        setState(() {
          selectedLanguage = value;
          if( value.name != 'English'){
            context.locale = Locale('my','');
          }else{
            context.locale = Locale('en','');
          }
        });
      },
      items: languages.map((Language lang) {
        return  DropdownMenuItem<Language>(
          value: lang,
          child: Row(
            children: <Widget>[
              lang.icon,
              SizedBox(width: 10,),
              Text(
                lang.name,
                style:  TextStyle(color: Colors.white),
              ),
            ],
          ),
        );
      }).toList(),
    ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Theme.of(context).primaryColor,
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text('HR Management login', style: TextStyle(color: Colors.white)),
          bottom: _createProgressIndicator(),
          actions: <Widget>[
            new Theme(
              data: Theme.of(context).copyWith(
                canvasColor: Theme.of(context).primaryColor,
              ),
              child:dropdownWidget()
            ),
            FlatButton.icon(
                onPressed:_processDone?null : (){_gotoKiosk();},
                icon: SvgPicture.asset( 'assets/images/kiosk.svg', semanticsLabel: 'Kiosk',height: 24, width: 24, ),
                label: Text('Stand mode',
                  style:  TextStyle(color: Colors.white),
                )
            )
          ],
        ),
        body: Stack(
            fit: StackFit.passthrough,
            children: [
              _backgroundPanel( true ),
              Center(
                  child: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  color: Color.fromRGBO(250, 250, 250, .0),
                                  elevation: 0,
                                  child: Column(
                                      children: <Widget>[
                                        Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: GestureDetector(
                                            child:ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                    height: 150,
                                                    child: new Image.asset("assets/images/logo.png"),
                                                  ),
                                                ),
                                              )
                                            ),
                                            onDoubleTap: (){
                                              _changeBaseUrl();
                                              },
                                          ),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                          child: Text('${Constant.APP_NAME}: Professional HRM',
                                              style: TextStyle(
                                                  color: Theme.of(context).primaryColor,
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 18
                                              )),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                          child: Form(
                                              key: _formKey,
                                              autovalidate: true,
                                              child: new Column(
                                                  children: <Widget>[
                                                    Container(
                                                      margin: const EdgeInsets.only( bottom: 5.0, top: 10),
                                                      child: Text('',
                                                        softWrap: true,
                                                        style: TextStyle(
                                                          fontWeight: FontWeight.w400,
                                                          color: Colors.transparent,
                                                          fontSize: 14,
                                                        ),
                                                      ),
                                                    ),
                                                    Container(
                                                      padding: const EdgeInsets.only(left: 0.0, bottom: 0.0, right: 0.0, top: 25),
                                                      width: double.infinity,
                                                      child: RaisedButton(
                                                        padding: EdgeInsets.all(15),
                                                        onPressed:_processDone?null : () { _gotoRecognization();},
                                                        elevation: 1,
                                                        child: Text(
                                                          tr(LocaleKeys.btnFaceLogin),
                                                          style: TextStyle(color: Colors.white),
                                                        ),
                                                          shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(30.0))
                                                      ),
                                                    ),
                                                  ]
                                              )
                                          ),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                            Container(
                              padding: const EdgeInsets.only(left: 25.0, bottom: 10.0, right: 25.0, top: 20),
                              width: double.infinity,
                              child: RaisedButton(
                                padding: EdgeInsets.all(15),
                                onPressed:_processDone?null : () { _gotoLogin(); },
                                elevation: 1,
                                child: Text(
                                  tr(LocaleKeys.btnLogin),
                                  style: TextStyle(color: Colors.white),
                                ),
                                  shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(30.0))
                              ),
                            ),

                            // Container(
                            //   padding: const EdgeInsets.only(left: 25.0, bottom: 25.0, right: 25.0, top: 15),
                            //   width: double.infinity,
                            //   child: RaisedButton(
                            //     padding: EdgeInsets.all(15),
                            //     onPressed:_processDone?null : () {
                            //       Navigator.push(
                            //           context,
                            //           MaterialPageRoute(
                            //             builder: (BuildContext context) => OrganizationRegister(
                            //               title: tr(LocaleKeys.titleBusinessRegistration),
                            //             ),
                            //           )
                            //       );
                            //     },
                            //     elevation: 1,
                            //     child: Text(
                            //       tr(LocaleKeys.btnRegister),
                            //       style: TextStyle(color: Colors.white),
                            //     ),
                            //       shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(30.0))
                            //   ),
                            // ),
                          ]
                      )
                  )
              )
            ]
        )
    );
  }

  Future<void> _gotoRecognization( ) async {

    if( Platform.isIOS ){
      bool result = await _localAuthImpl();
      if( !result ){
        await _showBiometricsFailed( context );
        _gotoLogin();
        return;
      }
    }

    if( Platform.isIOS && _shutterSoundNotice ) {
      bool dont = await _showDialog(context);
      _shutterSoundNotice = !dont;
      if (dont) {
        await setCameraShutterSoundNotice(false);
      }
    }

    RecognizeResult _recognizeResult = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => FaceRecognize(),
        )
    );
    if( Constant.ENABLE_LOGS)
      print(_recognizeResult.toString());
    if( _recognizeResult == null) return;

    setState(() {
      progressColor = Colors.white;
      _processDone = true;
    });
    showMessage('Login success, a moment please', context);

    // 2FA checking
    await set2FA( _recognizeResult.enable2FA );

    if( Constant.ENABLE_LOGS)
      print('Load user data ${_recognizeResult.faceGUID}');
    TokenService tokenService = new TokenService( );
    String userAccessToken = await tokenService.faceLogin(_recognizeResult.faceGUID, false);
    if( Constant.ENABLE_LOGS)
      print(userAccessToken);
    await gotoOrgSelection( userAccessToken, _recognizeResult.faceGUID );

  }

  Future<void> _gotoLogin() async{
    String userAccessToken = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => UserLogin(title: tr(LocaleKeys.login),),
        )
    );

    if( userAccessToken != null){
      setState(() {
        progressColor = Colors.white;
        _processDone = true;
      });
      showMessage('Login success, a moment please', context);

      UserService userService = new UserService( );
      AccountBrief accountBrief =await userService.getBrief(userAccessToken);
      if( Constant.ENABLE_LOGS)
        print(userAccessToken);
      await gotoOrgSelection( userAccessToken, accountBrief.facialGUID );
    }
  }

  Future<void> _gotoKiosk() async{

    KioskData kioskData = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => ActivateKiosk(),
        )
    );

    if( kioskData == null){
      return;
    }

    setState(() {
      progressColor = Colors.white;
      _processDone = true;
    });
    showMessage('Activation success, a moment please', context);

    Navigator.pushReplacement(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => KioskHome( ),
        )
    );
  }

  Future<void> gotoOrgSelection(String userAccessToken, String facialGUID) async{

    await _checkFireBaseRegistration();
    OrganizationService organizationService = new OrganizationService();
    OrganizationsByFace organizationsByFace = await organizationService.getListByFace(userAccessToken, facialGUID);

    if( organizationsByFace.organizations == null){
      showMessage('Account was not validated. Please try to validate!', context);
      _shouldValidate( userAccessToken, facialGUID );
      return;
    }

    if( Constant.ENABLE_LOGS)
      print(organizationsByFace.toString());

    // more than one org
    if( organizationsByFace.organizations.length > 1) {
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) =>
                OrganizationListByFace(
                  organizationsByFace: organizationsByFace,
                  accessToken: userAccessToken,
                  facialGUID: facialGUID,
                  sharedPrefs: _sharedPrefs,
                ),
          )
      );

    }else{
      User user = await organizationService.joinByFace(organizationsByFace.organizations[0].guid, facialGUID);
      EmployeeService employeeService = new EmployeeService();
      String response = await employeeService.activeDefault(organizationsByFace.organizations[0].guid, facialGUID);
      await user.fromJsonEmployee(response, true);
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => Home(
              title: '${Constant.APP_NAME}',
            ),
          )
      );
    }

  }

  Future<void> _checkFireBaseRegistration() async{
    try{
      if( await getFirebaseToken( ) == null){
        await _firebaseMessage.initMessaging( );
      }
    }catch(e){}
  }

  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
          image:
          MediaQuery.of(context).orientation == Orientation.portrait ?
          new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
              : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor)
      )
  );

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _showDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Camera usage!"),
          content: new Text("You may hear camera shutter sound frequently. The purpose of preventing to access the hidden camera."),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Don't show again"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<bool> _localAuthImpl() async{
    bool result = true;
    try {
      var localAuth = LocalAuthentication();
      bool canCheckBiometrics = await localAuth.canCheckBiometrics;
      if (!canCheckBiometrics) {
        return canCheckBiometrics;
      }

      try{
        List<BiometricType> availableBiometrics = await localAuth
            .getAvailableBiometrics();
        bool didAuthenticate =
        await localAuth.authenticateWithBiometrics(
            localizedReason: 'Please authenticate before login to the QuickQ HR.');

        result = didAuthenticate;
      }on PlatformException catch(e){
        if( Constant.ENABLE_LOGS)
          print('LocalAuth Error Code=${e.code}');
      }
    }catch(e){
      if( Constant.ENABLE_LOGS)
        print(e.toString());
    }

    return result;
  }

  Future<void> _showBiometricsFailed( BuildContext context )async {

    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Biometric authentication failed!"),
          content: new Text("QuickQ HR can't process biometric authentication. Please login with username and password."),
          actions: <Widget>[
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
  }

  Future<void> _changeBaseUrl() async{
    ++_doubleTapCount;
    if( _doubleTapCount == 1 ) {
      new ThreadTimer(new Duration(milliseconds: 1000 * 5), () {
        _doubleTapCount = 0;
      });
    }

    if( _doubleTapCount >= 3){
      _doubleTapCount = 0;
      await Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => SetupPrivateConnection(sharedPrefs: _sharedPrefs,),
          )
      );
    }
  }

  Future<void> _shouldValidate(String userAccessToken, String facialGUID) async{

    UserService userService = new UserService( );
    User user = await userService.getUserInformation(userAccessToken, null);
    AuthorityResponse authorityResponse = await userService.authority(userAccessToken);
    if( authorityResponse.authorities!=null
        && authorityResponse.authorities.contains('ORGANIZATION_READ') ) {

      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) =>
                ActivateOrganization(
                  organizationGuid: user.organizationGUID,
                  gotoLogin: true,
                ),
          )
      );
    }

  }
}

class Language {
  const Language(this.name,this.icon);
  final String name;
  final SvgPicture icon;
}