import '/hrm/config/constant.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';



class SetupPrivateConnection extends StatefulWidget {
  SetupPrivateConnection({Key key, @required this.sharedPrefs}) : super(key: key);
  final SharedPreferences sharedPrefs;

  @override
  _SetupPrivateConnectionState createState() => _SetupPrivateConnectionState( );
}

class _SetupPrivateConnectionState extends State<SetupPrivateConnection> {
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _processing = false;
  Color progressColor = Colors.transparent;
  String _serverURL;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          backgroundColor: Theme.of(context).primaryColor,
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Private connection'),
            bottom: _createProgressIndicator(),
          ),
          body: Stack(
            fit: StackFit.passthrough,
            children: [
              _backgroundPanel( true ),
              Center(
                  child: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  color: Color.fromRGBO(250, 250, 250, .4),
                                  elevation: 1,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                          margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                          child: Text('Setting private connection',
                                              style: TextStyle(
                                                  color: Theme.of(context).primaryColor,
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 18
                                              )),
                                        ),
                                        Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                    height: 150,
                                                    child: Icon(Icons.lock, size: 120,),
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                        new Container(
                                          margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                          child: formData(context),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )
              )
            ]
          )
        )
    );
  }

  Form formData(BuildContext context ){
    return new Form(
        key: _formKey,
        autovalidate: true,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.language),
                  hintText: '',
                  labelText: 'Server url',
                ),
                validator: (val) => val.isEmpty ? 'Server URL required' : null,
                onSaved: (val) => _serverURL = val,
                keyboardType: TextInputType.multiline,
                minLines: 1,
                maxLines: 5,
              ),
              Wrap(
                  children: <Widget>[
                    Container(
                      padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                      width: double.infinity,
                      child: RaisedButton(
                        padding: EdgeInsets.all(15),
                        onPressed:_processing?null: () {_submitForm(context); },
                        elevation: 1,
                        child: Text(
                          'Save',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                    Container(
                      padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 0),
                      width: double.infinity,
                      child: RaisedButton(
                        padding: EdgeInsets.all(15),
                        onPressed:_processing?null: () {
                          _reset( );
                          },
                        elevation: 1,
                        child: Text(
                          'Reset to default',
                          style: TextStyle(color: Colors.white),
                        ),
                      ),
                    ),
                  ]
              ),
            ])
    );
  }

  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
          image:
          MediaQuery.of(context).orientation == Orientation.portrait ?
          new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
              : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  void _submitForm(BuildContext context) async{

    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Required server URL', context);
      return;
    }
    form.save();

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
      _processing = true;
    });

    if( !_serverURL.startsWith('http')){
      _serverURL = 'http://$_serverURL';
    }

    if( _serverURL.endsWith('/') || _serverURL.endsWith('\\') || _serverURL.endsWith('.'))
      _serverURL = _serverURL.substring(0, _serverURL.length -1);

    if(Constant.ENABLE_LOGS)
      print('New server url=$_serverURL');

    String defaultBaseURL = getBaseURL(widget.sharedPrefs);
    if(Constant.ENABLE_LOGS)
      print('Default base url=$defaultBaseURL');
    await setBaseURL( _serverURL.trim().toLowerCase() );

    String accessToken;
    try {
      TokenService tokenService = new TokenService();
      accessToken = await tokenService.getDeviceToken(true);
    }catch(e){}

    if( accessToken != null && accessToken.length > 10){

      showMessage('Congratulation! connection success.', context);
      new ThreadTimer(new Duration(milliseconds: 1000 * 5), () {
        Navigator.pop(context);
      });

    }else{
      await setBaseURL( defaultBaseURL );
      showMessage('Connection failed, please check the server url.', context);
    }

    setState(() {
      progressColor = Colors.transparent;
      _processing = false;
    });

  }

  void _reset( ) async{

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
      _processing = true;
    });

    String defaultBaseURL = getBaseURL(widget.sharedPrefs);
    await setBaseURL( Constant.BASE_URL_PRIMARY );
    String accessToken;
    try {
      TokenService tokenService = new TokenService();
      accessToken = await tokenService.getDeviceToken(true);
    }catch(e){}

    if( accessToken != null && accessToken.length > 10){

      showMessage('Connection success with default server.', context);
      new ThreadTimer(new Duration(milliseconds: 1000 * 5), () {
        Navigator.pop(context);
      });

    }else{
      await setBaseURL( defaultBaseURL );
      showMessage('Connection failed, please check the server url.', context);
    }

    setState(() {
      progressColor = Colors.transparent;
      _processing = false;
    });

  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}