

import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';


class UserLogin extends StatefulWidget {
  UserLogin({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _LoginUserUserPageState createState() => _LoginUserUserPageState( );
}

class _LoginUserUserPageState extends State<UserLogin> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool _processing = false;
  Color progressColor = Colors.transparent;
  String _name;
  String _password;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
        backgroundColor: Theme.of(context).primaryColor,
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(widget.title),
            bottom: _createProgressIndicator(),
          ),
          body: Stack(
              fit: StackFit.passthrough,
              children: [
                _backgroundPanel( true ),
                  Center(
                      child: SingleChildScrollView(
                          child: Column(
                              children: <Widget>[
                                Container(
                                    padding: const EdgeInsets.only(left: 20.0, top: 5.0, right: 20.0),
                                    child:Card(
                                      color: Color.fromRGBO(250, 250, 250, .4),
                                      elevation: 1,
                                      child: Column(
                                          children: <Widget>[
                                            Container(
                                              padding: const EdgeInsets.only(left:5.0, right: 5, bottom: 5, top: 20),

                                              child: ClipOval(
                                                  child: Material(
                                                    elevation: 8,
                                                    color: Colors.white, // button color
                                                    child: InkWell(
                                                      splashColor: Colors.red, // inkwell color
                                                      child: SizedBox(width: 150,
                                                        height: 150,
                                                        child: new Image.asset("assets/images/logo.png"),
                                                      ),
                                                    ),
                                                  )
                                              ),
                                            ),
                                            new Container(
                                              margin: const EdgeInsets.only(top:20, left: 15.0, right: 15.0, bottom: 20),
                                              child: Text('${Constant.APP_NAME}: Professional HRM',
                                                  style: TextStyle(
                                                      color: Theme.of(context).primaryColor,
                                                      fontWeight: FontWeight.bold,
                                                      fontSize: 18
                                                  )),
                                            ),
                                            new Container(
                                              margin: const EdgeInsets.only(left: 15.0, right: 15.0),
                                              child: formData(context),
                                            ),
                                          ]
                                      ),
                                    )
                                ),
                              ]
                          )
                      )
                  )
              ]
          )
      )
    );
  }


  Widget _backgroundPanel( bool home ){
    return new Container(
      decoration: new BoxDecoration(
        image:
            MediaQuery.of(context).orientation == Orientation.portrait ?
            new DecorationImage(image: new AssetImage('assets/images/bg3.jpg'), fit: BoxFit.cover,)
                : new DecorationImage(image: new AssetImage('assets/images/bg_landscape.jpg'), fit: BoxFit.cover,)
      ),
    );
  }

  Form formData(BuildContext context ){

    return new Form(
        key: _formKey,
        autovalidate: true,
        child: new Column(
            children: <Widget>[
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.account_circle),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblUsername),
                ),
                validator: (val) => val.isEmpty ? 'Username required' : null,
                onSaved: (val) => _name = val,
              ),
              new TextFormField(
                obscureText: true,
                decoration: InputDecoration(
                  icon: const Icon(Icons.star),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblPassword),
                ),
                validator: (val) => val.isEmpty ? 'Password required' : null,
                onSaved: (val) => _password = val,
              ),
              Container(
                padding: const EdgeInsets.only(left: 10.0, bottom: 25.0, right: 10.0, top: 25),
                width: double.infinity,
                child: RaisedButton(
                  padding: EdgeInsets.all(15),
                  onPressed: () { _submitForm(context);},
                  elevation: 1,
                  child: Text(
                    tr(LocaleKeys.btnLoginMain),
                    style: TextStyle(color: Colors.white),
                  ),
                    shape: RoundedRectangleBorder(borderRadius: new BorderRadius.circular(30.0))
                ),
              ),
            ])
    );
  }

  void _submitForm(BuildContext context) {
    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Required username and password!', context);
    } else {
      form.save(); //This invokes each onSaved event
      _doLogin( context );
    }
  }

  Future<void> _doLogin(BuildContext context  ) async{

    if( _processing ) return;
    setState(() {
      progressColor = Colors.white;
      _processing = true;
    });

    TokenService tokenService = new TokenService();
    String userAccessToken = await tokenService.login(_name, _password, true);

    try {
      if( userAccessToken == null){
        setState(() {
          _processing = false;
          progressColor = Colors.transparent;
          showMessage('Invalid username or password.', context);
        });
      }else{
        Navigator.pop(context, userAccessToken);
      }

    }catch(e){
      if(Constant.ENABLE_LOGS)
        print(e);
    }
    setState(() {
      _processing = false;
      progressColor = Colors.transparent;
    });
  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}
