import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/department_search_response.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/service/department_service.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';



class AdminReportHome extends StatefulWidget {
  AdminReportHome({Key key, this.currentUser, this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final SharedPreferences sharedPrefs;

  @override
  _AdminReportHomeState createState() => _AdminReportHomeState();
}

class _AdminReportHomeState extends State<AdminReportHome>{

  bool _isSearch = false;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  bool showDate = false;
  DepartmentSearchResponse _departmentSearchResponse;
  DepartmentSearchItem _departmentSearchItem;
  EmployeeSearchResponse _employeeSearchResponse;
  EmployeeSearchItem _employeeSearchItem;
  bool _showEmployee = false;
  List<Map<String, String>> _leaveStatus = [];
  Map<String, String> _selectedLeaveStatus = {'name':'All'};
  bool showLeaveStatus = false;
  bool _processing = false;
  Color _progressColor = Colors.transparent;
  String _accessToken;
  String _title = "Reporting";
  String _reportName = '';

  @override
  initState() {
    super.initState( );
    //[{"name":"All"},{"name":'Approved'},{"name":'Pending'},{"name":'Rejected'},{"name":'Cancelled'}];
    _leaveStatus = new List<Map<String, String>>();
    _leaveStatus.add({"name":"All"});
    _leaveStatus.add({"name":"Approved"});
    _leaveStatus.add({"name":"Pending"});
    _leaveStatus.add({"name":"Rejected"});
    _leaveStatus.add({"name":"Cancelled"});
    _loadData();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text(_title),
              bottom: _isSearch?_createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            ),
            body: ListView(
              children: <Widget>[
                Container(
                    margin: EdgeInsets.all(16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(
                              Icons.download_sharp,
                              size: 48,
                              color: Theme.of(context).primaryColor
                          ),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeDetailsReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee details and you can filter employees by department, job title.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeDetailsReport);
                              _reportName = 'Employee details report';
                              showLeaveStatus = false;
                              showDate = false;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(
                              Icons.download_sharp,
                              size: 48,
                              color: Theme.of(context).primaryColor
                          ),
                          title: Text(
                            tr(LocaleKeys.lblActiveEmployeeReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list employees who are currently active based on joined date and termination date.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblActiveEmployeeReport);
                              _reportName = 'Active employee report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = false;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblNewHiresEmployeeReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list employees who are joined between given two dates.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblNewHiresEmployeeReport);
                              _reportName = 'New hires employee report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = false;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblTerminatedEmployeeReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list employees who are terminated between given two dates.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblTerminatedEmployeeReport);
                              _reportName = 'Terminated employee report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = false;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeLeavesReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee leaves by employee, date range and leave status.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeLeavesReport);
                              _reportName = 'Employee leaves report';
                              showLeaveStatus = true;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeLeaveEntitlement),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list employees leave entitlement for current leave period by department or by employee.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeLeaveEntitlement);
                              _reportName = 'Employee leave entitlement';
                              showLeaveStatus = false;
                              showDate = false;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeAttendanceReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee attendance entries by employee and date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeAttendanceReport);
                              _reportName = 'Employee attendance report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeLateInReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee attendance entries late-in by employee and date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeLateInReport);
                              _reportName = 'Employee late-in report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeEarlyOutReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee attendance entries early-out by employee and date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeEarlyOutReport);
                              _reportName = 'Employee early-out report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeOvertimeReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee attendance entries by employee with overtime calculations.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeOvertimeReport);
                              _reportName = 'Overtime report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeOvertimeSummaryReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee attendance entries by employee with overtime calculation summary.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeOvertimeSummaryReport);
                              _reportName = 'Overtime summary report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeePayrollReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee payrolls.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeePayrollReport);
                              _reportName = 'Payroll report';
                              showLeaveStatus = false;
                              showDate = true;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEmployeeSalaryComponentsReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all employee salary components.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEmployeeSalaryComponentsReport);
                              _reportName = 'Employee salary components report';
                              showLeaveStatus = false;
                              showDate = false;
                              _showEmployee = true;
                            });
                          },
                        )
                    )
                ),
              ],
            )
        )
    );
  }


  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, MediaQuery.of(context).size.height * 0.7),
      child: SizedBox(
          height: MediaQuery.of(context).size.height * 0.7,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 1,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child: Column(
                      children: <Widget>[
                        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('ORGANIZATION_READ')?_deptSelection(context): new Container(),
                        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ') && _showEmployee?_employeeSelection(context) : new Container(),
                        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ') && showLeaveStatus?_statusSelection(context) : new Container(),
                        showDate? Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ) : Container(),
                        showDate? Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ) : Container(),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.download_sharp, tr(LocaleKeys.btnDownload) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnCancel)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }


  _loadData() async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    await _authorityResponse.fromSharedPreferences();

    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( widget.currentUser.facialGUID, false );

    DepartmentService _departmentService = new DepartmentService();
    _departmentSearchResponse = await _departmentService.search(_accessToken, widget.currentUser.organizationGUID, '');
    EmployeeService _employeeService = new EmployeeService();
    _employeeSearchResponse = await _employeeService.search(_accessToken, '', 0, 100000);

    if( mounted)
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
  }

  Widget _deptSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<DepartmentSearchItem>(
          selectedItem: _departmentSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (DepartmentSearchItem i, DepartmentSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblDepartment),
          onFind: (String filter) => _deptSuggestion(filter),
          onChanged: (DepartmentSearchItem data) {
            _departmentSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, DepartmentSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text(tr(LocaleKeys.lblAllDepartments)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, DepartmentSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<DepartmentSearchItem>> _deptSuggestion(String query) async{
    List<DepartmentSearchItem> matches = List();
    if( _departmentSearchResponse.departments == null)
      return matches;

    matches.addAll(_departmentSearchResponse.departments);

    matches.retainWhere((s) {
      DepartmentSearchItem dept = s;
      if( dept.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _employeeSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: tr(LocaleKeys.lblEmployee),
          onFind: (String filter) => _employeeSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _employeeSearchItem = data;
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.account_circle), title: Text(tr(LocaleKeys.lblAllEmployees)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Future<List<EmployeeSearchItem>> _employeeSuggestion(String query) async{
    List<EmployeeSearchItem> matches = List();
    if( _employeeSearchResponse.employees == null)
      return matches;
    matches.addAll(_employeeSearchResponse.employees);

    matches.retainWhere((s) {
      EmployeeSearchItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _statusSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<Map<String, String>>(
          selectedItem: _selectedLeaveStatus,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (Map<String, String> i, Map<String, String> s) => i['name']==s['name'],
          label: tr(LocaleKeys.lblLeaveType),
          onFind: (String filter) => _statusSuggestion(filter),
          onChanged: (Map<String, String> data) {
            _selectedLeaveStatus = data;
          },
          dropdownBuilder: (BuildContext context, Map<String, String> item ) =>
              Container(
                child: (item['name'] == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text(tr(LocaleKeys.lblAllDepartments)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item['name']),
                ),
              ),
          popupItemBuilder: (BuildContext context, Map<String, String> item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item['name']),
                ),
              ),
        )
    );
  }
  Future<List<Map<String, String>>> _statusSuggestion(String query) async{
    List<Map<String, String>> matches = List();
    if( _leaveStatus == null)
      return matches;

    matches.addAll(_leaveStatus);

    matches.retainWhere((s) {
      Map<String, String> dept = s;
      if( dept['name'].toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == tr(LocaleKeys.btnCancel) ){
              reset();
            }else if( label==tr(LocaleKeys.btnDownload)){
              openBrowser(Constant.BASE_URL_PRIMARY
                  +'/v1/forwarder/reports/'+_accessToken
                  +'?n=' + Uri.encodeFull(_reportName)
                  +'&d='+ (_departmentSearchItem!=null ? _departmentSearchItem.guid : '')
                  +'&eg='+ (_employeeSearchItem != null? _employeeSearchItem.guid : '')
                  + (_startDate!=null? '&sd=' +new DateFormat('yyyy-MM-dd').format(_startDate.toLocal()): '')
                  + (_endDate!=null? '&ed=' +new DateFormat('yyyy-MM-dd').format(_endDate.toLocal()): '')
                  +(_selectedLeaveStatus !=null? '&s=' +_selectedLeaveStatus['name'] : '')
              );
              reset();
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  void reset(){
    setState(() {
      _employeeSearchItem = null;
      _departmentSearchItem = null;
      _startDate = null;
      _controller_start.text = '';
      _endDate = null;
      _controller_end.text = '';
      _isSearch = !_isSearch;
      _selectedLeaveStatus = {'name':'All'};
      _reportName = '';
      _title = "Reporting";
    });
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }

}