import 'dart:io';
import 'package:dropdown_search/dropdown_search.dart';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/geofencing_add_request.dart' as add;
import '/hrm/model/request/geofencing_update_request.dart';
import '/hrm/model/response/geofencing_detail_response.dart';
import '/hrm/plugins/wifi.dart';
import '/hrm/service/geofencing_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:location/location.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'geofencing_map.dart';
import 'selected_employees.dart';



class AddGeofencing extends StatefulWidget {
  AddGeofencing({Key key, @required this.title, this.guid, this.selectedIndex}) : super(key: key);
  final String title;
  final String guid;
  final int selectedIndex;
  @override
  _AddGeofencingState createState() => _AddGeofencingState();
}

class _AddGeofencingState extends State<AddGeofencing> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  final TextEditingController _controller_name = new TextEditingController();
  final TextEditingController _controller_latitude = new TextEditingController();
  final TextEditingController _controller_longitude = new TextEditingController();
  final TextEditingController _controller_bssid = new TextEditingController();
  int _restrictedType = 1;
  String _action="both";
  double _radius = 150;
  int _wifiCondition = 2;


  List<Map<String, dynamic>> _wifis=[];
  Map<String, dynamic> _wifi;
  List<GeofencingEmployeeItem> _items=[];
  bool _processing = false;
  Color _progressColor = Colors.transparent;
  User _currentUser = new User();
  GeofencingService _geofencingService = new GeofencingService();
  GeofencingDetailResponse _response;
  String _accessToken;
  SharedPreferences _sharedPrefs;
  LocationData _startLocation;
  LocationData _currentLocation;
  Location _locationService  = new Location();
  bool _permission = false;
  String error;
  int _selectedIndex = 0;
  int _selectedType = 0;
  bool _deleteMode = false;

  @override
  void initState() {
    super.initState();
    _selectedIndex = widget.selectedIndex??0;
    _getAccessToken( true );
    _initLocationService();
  }

  void _getAccessToken( bool isFirstLoad) async{
    if( isFirstLoad) {
      _sharedPrefs = await SharedPreferences.getInstance();
      await _currentUser.fromSharedPreferences();
    }
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    if(widget.guid != null && isFirstLoad)
      _loadData();

    _loadWifi(true);
  }

  void _loadData() async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    _response = await _geofencingService.detail(_accessToken, widget.guid);
    if( _response != null ){
      _controller_name.text = _response.name;
      _restrictedType = _response.restrictedType;
      _action = _response.action;
      if( _restrictedType == add.GeofencingAddRequest.RESTRICTED_TYPE_GPS){
        _radius = _response.radius.toDouble();
        _controller_latitude.text = _response.latitude.toString();
        _controller_longitude.text = _response.longitude.toString();
      }else{
        _wifi = {"bssid":_response.bssId,"ssid":_response.ssId};
        _wifiCondition = _response.wifiCondition;
        _controller_bssid.text = _response.bssId;
      }
      _items = _response.items;
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
    }
  }

  void _loadWifi(bool firstLoad) async{
    try {
      if(Platform.isAndroid)
        await Wifi.listMap().then((List<Map<String, dynamic>> result) {
          _wifis = result;
          if( (_wifis == null || _wifis.isEmpty) && !firstLoad){
            showMessage('Please turn on your Wifi to select from available wifi list!');
          }
        });
    }catch(e){}
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            key: _scaffoldKey,
            appBar: AppBar(
              title: Text(widget.title),
              bottom: _deleteMode? _createSelectionIndicator() : _createProgressIndicator(),
            ),
            body: _createBody(context),
            floatingActionButton: _selectedIndex==1? FloatingActionButton.extended(
              label: Text(
                'Add users',
                style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.person_add),
              backgroundColor: Theme.of(context).primaryColor,
              onPressed: _processing?null: () async{
               _addEmployees();
              },
            ) : new Container(),
          bottomNavigationBar: BottomNavigationBar(
            items: const <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.location_on),
                title: Text('Details'),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.supervised_user_circle),
                title: Text('Selected users'),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Colors.grey[700],
            showUnselectedLabels: true,
            showSelectedLabels: true,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  void _onItemTapped(int index) {
    setState(() {
      _selectedIndex = index;
      if( _selectedIndex ==0)
        _deleteMode = false;
    });
  }

  Widget _createBody(BuildContext context){
    if( _selectedIndex == 0 ){
      return Center(
        child: SingleChildScrollView(
          child: Column(
              children: <Widget>[
                Container(
                    padding: const EdgeInsets.only(left: 5.0, top: 5.0, right: 5.0),
                    child:Card(
                      color: Colors.grey[50],
                      elevation: 1,
                      child: Column(
                        children: <Widget>[
                          new Container(
                            margin: const EdgeInsets.only(left: 5.0, right: 5.0, bottom: 30),
                            child: formData(context),
                          ),
                        ],
                      ),
                    )
                ),
              ]
          ),
        ),
      );
    }else{
      return _buildContent(context);
    }

  }
  Form formData( BuildContext context ){

    return new Form(
        key: _formKey,
        autovalidate: true,
        child: new Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              new TextFormField(
                decoration: const InputDecoration(
                  icon: const Icon(Icons.location_on),
                  hintText: 'Head Office',
                  labelText: 'Zone name',
                ),
                controller: _controller_name,
                validator: (val) => val.isEmpty ? 'Zone name is required' : null,
//                onSaved: (val) => _name = val,
              ),
              Container(
                margin: const EdgeInsets.only(left: 40.0, right: 5.0, top:5),
                child:Text(
                  'Zone name should be your employees understandable name. e.g. Head office, Mandalay branch, Myanmar Plaza, Bogyoke market, etc.',
                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                ),
              ),
              Container(
                margin: const EdgeInsets.only(top: 15.0, left: 0.0, right: 0.0),
                child: ListTile(
                  title: Text('Attendance restricted for'),
                subtitle:new Wrap(
                  children: <Widget>[
//                    Expanded(
//                      child: Center(
//                        child:
                    RadioListTile(
                          value: 'in',
                          groupValue: _action,
                          title: Text('Punch-in'),
                      subtitle: Text(
                        "Selected employees must punch-in within specific zone",
                        style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                      ),
                          onChanged: ( value){
                            setState(() { _action = value; });
                          },
//                        )
//                      )
                    ),
//                    Expanded(
//                      child: Center(
//                        child:
                    RadioListTile(
                          value: 'out',
                          groupValue: _action,
                          title: Text('Punch-out'),
                      subtitle: Text(
                        "Selected employees must punch-out within specific zone",
                        style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                      ),
                          onChanged: ( value){
                            setState(() { _action = value; });
                          },
//                        )
//                      )
                    ),
//                    Expanded(
//                      child: Center(
//                        child:
                    RadioListTile(
                          value: 'both',
                          groupValue: _action,
                          title: Text('Both'),
                      subtitle: Text(
                        "Selected employees must punch-in/punch-out within specific zone",
                        style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                      ),
                          onChanged: ( value){
                            setState(() { _action = value; });
                          },
//                        )
//                      )
                    ),
                  ],
                )
    ),
              ),
              Container(
                margin: const EdgeInsets.only(top: 5.0, left: 5.0, right: 5.0),
                child:ListTile(
                title:Text('Restricted by'),
                subtitle: new Wrap(
                  children: <Widget>[
//                    Expanded(
//                      child: Center(
//                        child:
                        RadioListTile(
                          value: 1,
                          groupValue: _restrictedType,
                          title: Text('GPS location'),
                          subtitle: Text(
                            "Employee's device location service must be turn on before In/Out",
                            style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                          ),
                          onChanged: widget.guid==null? ( value){
                            setState(() { _restrictedType = value; });
                          } : null,
//                        )
//                      )
                    ),
//                    Expanded(
//                        child: Center(
//                            child:
                            RadioListTile(
                              value: 2,
                              groupValue: _restrictedType,
                              title: Text('Wifi signal'),
                              subtitle: Text(
                                  'Notice: Wifi restriction may NOT be working on iOS devices!',
                                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                              ),
                              onChanged: widget.guid==null? ( value){
                                setState(() { _restrictedType = value; });
                                _loadWifi(false);
                              } : null,
//                            )
//                        )
                    ),
                  ],
                ),
                ),
              ),
//              _restrictedType==2? Container(
//                margin: const EdgeInsets.only(left: 5.0, right: .0),
//                child:Text(
//                  'Notice: Wifi restriction may NOT be working on iOS devices!',
//                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
//                ),
//              ) : Container(),
              _restrictedType==1?Container(
                margin: const EdgeInsets.only(top: 0.0, left: 0.0, right: 0.0),
                child:new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    new Container(
                        margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                        child: ListTile(
                          subtitle: SliderTheme(
                            data: SliderTheme.of(context).copyWith(
                              activeTrackColor: Theme.of(context).primaryColor,
                              inactiveTrackColor: Colors.blueGrey[50],
                              trackShape: RoundedRectSliderTrackShape(),
                              trackHeight: 5.0,
                              thumbShape: RoundSliderThumbShape(enabledThumbRadius: 10.0),
                              thumbColor: Theme.of(context).primaryColor,
                              overlayColor: Colors.green[800].withAlpha(32),
                              overlayShape: RoundSliderOverlayShape(overlayRadius: 28.0),
                              tickMarkShape: RoundSliderTickMarkShape(),
                              activeTickMarkColor: Colors.white,
                              inactiveTickMarkColor: Colors.red[600],
                              valueIndicatorShape: PaddleSliderValueIndicatorShape(),
                              valueIndicatorColor: Theme.of(context).primaryColor,
                              valueIndicatorTextStyle: TextStyle(
                                color: Colors.white,
                              ),
                            ),
                            child: Slider(
                              value: _radius,
                              min: 50,
                              max: 300,
                              divisions:5,
                              label: '${_radius.toInt()} meters',
                              onChanged: (value) {
                                setState(() {
                                  _radius = value;
                                });
                              },
                            ),
                          ),
                          title: Text('Coverage distance in meters'),
                          trailing: Text('${_radius.toInt()}m'),
                        )
                    ),
                    new TextFormField(
                      decoration: const InputDecoration(
                        icon: Icon(Icons.gps_fixed),
                        labelText: 'Latitude',
                      ),
                      controller: _controller_latitude,
                      keyboardType: TextInputType.number,
                    ),
                    new TextFormField(
                      decoration: const InputDecoration(
                        icon: Icon(Icons.gps_fixed),
                        labelText: 'Longitude',
                      ),
                      controller: _controller_longitude,
                      keyboardType: TextInputType.number,
                    ),
                    Container(
                      margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 40),
                      child: RaisedButton.icon(
                        elevation: 2.0,
                        icon: Icon(Icons.map, color: Colors.white),
                        color: Theme.of(context).primaryColor,
                        onPressed:_processing ? null: () async{
                          LocationData locData = _startLocation;
                          if( _controller_latitude.text.isNotEmpty || _controller_longitude.text.isNotEmpty){
                          }
                              Map<String, dynamic> data= await Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (BuildContext context) => GeofencingMap(
                                    lat: _controller_latitude.text.isNotEmpty? double.parse(_controller_latitude.text) : (_startLocation!=null? _startLocation.latitude : null),
                                    lon: _controller_longitude.text.isNotEmpty? double.parse(_controller_longitude.text) : (_startLocation!=null? _startLocation.longitude : null),
                                    title: 'Get location',
                                  ),
                                )
                              );
                              if( data != null){
                                _controller_latitude.text = data['lat'].toString();
                                _controller_longitude.text = data['lon'].toString();
                              }
                        },
                        label: Text(
                          'Get location from map',
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w400,
                            color: Colors.white,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ):
              Container(
                margin: const EdgeInsets.only(top: 0.0, left: 0.0, right: 0.0),
                child:new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Platform.isAndroid? _wifisSelection(context) : Container(),
                    new TextFormField(
                      decoration: const InputDecoration(
                        icon: Icon(Icons.network_wifi),
                        hintText: 'xx:xx:xx:xx:xx:xx',
                        labelText: "Wifi device's MAC address",
                      ),
                      controller: _controller_bssid,
                    ),
                    Container(
                      margin: const EdgeInsets.only(top: 25.0, left: 5.0, right: 5.0),
                      child:ListTile(
                        title:Text('Wifi condition'),
                        subtitle:new Wrap(
                          children: <Widget>[
                            RadioListTile(
                              value: 1,
                              groupValue: _wifiCondition,
                              title: Text('While connected'),
                              subtitle: Text(
                                  "Employee's device must be joined with specific Wifi",
                                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                              ),
                              onChanged: ( value){
                                setState(() { _wifiCondition = value; });
                              },
                            ),
                            RadioListTile(
                              value: 2,
                              groupValue: _wifiCondition,
                              title: Text('When detected'),
                              subtitle: Text(
                                  "Employee's device Wifi must be turn on before In/Out",
                                  style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                              ),
                              onChanged: ( value){
                                setState(() { _wifiCondition = value; });
                              },
                            ),
                          ],
                        )
                      ),
                    ),
                  ],
                ),
              ),
              Container(
                margin: const EdgeInsets.only(top: 35.0, bottom: 15.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    _buildButtons( Theme.of(context).primaryColor, Icons.add, widget.guid==null?'Save':'Update' ),
                    _buildButtons( Colors.transparent, Icons.cancel, widget.guid==null?'Cancel':'Later'),
                  ],
                ),
              )
            ])
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel' || label == 'Close' || label == 'Later' ){
              Navigator.pop( context );
            }else{
              if( label == 'Save'){
                _save();
              }else{
                _update();
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  Widget _wifisSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<Map<String, dynamic>>(
          selectedItem: _wifi,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
//          showSelectedItem: true,
//          compareFn: (Map<String, dynamic> i, Map<String, dynamic> s) => i['bssid']==s['bssid'],
          label: "Wifi list",
          onFind: (String filter) => _wifiSuggestion(filter),
          onChanged: (Map<String, dynamic> data) {
            _wifi = data;
            if(_wifi != null){
              _controller_bssid.text = _wifi['bssid'];
            }else{
              _controller_bssid.text = '';
            }
          },
          dropdownBuilder: (BuildContext context, Map<String, dynamic> item ) =>
              Container(
                child: (item == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.wifi), title: Text("Select the Wifi"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item['ssid']),
                ),
              ),
          popupItemBuilder: (BuildContext context, Map<String, dynamic> item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item['ssid']),
                ),
              ),
        )
    );
  }
  Future<List<Map<String, dynamic>>> _wifiSuggestion(String query) async{
    List<Map<String, dynamic>> matches = List();
    if( _wifis == null)
      return matches;
    matches.addAll(_wifis);

    matches.retainWhere((s) {
      Map<String, dynamic> jobTitleItem = s;
      if( jobTitleItem['ssid'].toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 0, right: 0),
      itemBuilder: (c, i) => EmployeeResultItem(
        post: _items[i],
        isEnd: i+1==_items.length,
        accessToken: _accessToken,
        sharedPrefs: _sharedPrefs,
        onDelete: (){
          setState(() {
            _deleteMode = true;
          });
        },
        deleteMode: _deleteMode,
        isCheck: (value){
          int checkCount = 0;
          _items.forEach((element) {
            if( element.check)
              ++checkCount;
          });
          setState(() {
            if( checkCount == _items.length){
              _selectedType = 2;
            }else if( checkCount> 0 && checkCount != _items.length){
              _selectedType = 1;
            }else
              _selectedType = 0;
          });
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _items.length,
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  PreferredSize _createSelectionIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 60.0),
      child: SizedBox(
          height: 60.0,
          child: Container(
              child: Ink(
                  color: Colors.white,
                  child: ListTile(
                      leading: RaisedButton.icon(
                        elevation: 1.0,
                        icon: Icon(Icons.clear, ),
                        color: Colors.white,
                        onPressed:_processing ? null: () {
                          setState(() {
                            _deleteMode = false;
                            _selectedType = 0;
                            _selectEmployees(false);
                          });
                        },
                        label: Text(
                          'Cancel',
                        ),
                      ),
                      title: RaisedButton.icon(
                        elevation: 1.0,
                        icon: Icon(Icons.delete_forever, color: Colors.red[600]),
                        color: Colors.white,
                        onPressed:_processing ? null: () {
                          _removeEmployees();
                        },
                        label: Text(
                          'Remove',
                        ),
                      ),
                      trailing: _checkBox()
                  )
              )
          )
      )
  );

  _addEmployees()async{
    _selectEmployees(false);
    _deleteMode = false;
    List<GeofencingEmployeeItem> _employees = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => SelectedEmployees(
            selectedEmployees: _items,
          ),
        )
    );

    if( _employees != null && _employees.isNotEmpty){
      if( widget.guid == null ) {
        setState(() {
          _items.addAll(_employees);
          _selectEmployees(false);
          if (_employees.isNotEmpty) {
            showMessage("${_employees.length} user${_employees.length > 1
                ? 's'
                : ''} added");
            new ThreadTimer(new Duration(milliseconds: 5000), () {
              setState(() {
                if (_selectedIndex == 1)
                  _selectedIndex = 0;
              });
            });
          }
        });
      }else{
        setState(() {
          _processing = true;
          _progressColor = Colors.white;
        });

        List<String> employeeGUIDs = [];
        _employees.forEach((element) {
          employeeGUIDs.add(element.employeeGUID);
        });

        try{
          String result = await _geofencingService.addGeofencee(employeeGUIDs, _response.guid, _accessToken);
          if( result != null ){
            showMessage("${employeeGUIDs.length} user${employeeGUIDs.length>1?'s':''} added into ${_controller_name.text} zone.");
            _items.addAll(_employees);
            _selectEmployees(false);
          }else{
            showMessage('Error: adding failed!');
          }
        }catch(e){}

        setState(() {
          _processing = false;
          _progressColor = Colors.transparent;
        });
      }

    }
  }

  _removeEmployees() async{

    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    // for update to server
    List<GeofencingEmployeeItem> removedEmployees = [];
    removedEmployees.addAll(_items);
    removedEmployees.retainWhere((element) {
      if( element.check)
        return true;
      else
        return false;
    });

    if( removedEmployees.isEmpty){
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
        _deleteMode = false;
        _selectedType = 0;
      });
      return;
    }

    bool result = true;
    try {
      if (widget.guid != null ) {
        List<String> guids = [];
        removedEmployees.forEach((element) {
          guids.add(element.guid);
        });
        result = await _geofencingService.removeGeofencee(guids, _accessToken);
        if( result ){
          showMessage("${guids.length} user${guids.length>1?'s':''} removed from ${_controller_name.text} zone.");
        }else{
          showMessage('Error: removing failed!');
        }
      }
      // updating the list
      if (result) {
        _items.removeWhere((element) {
          if (element.check)
            return true;
          else
            return false;
        });
      }
    }catch(e){}

    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
      _deleteMode = false;
      _selectedType = 0;
    });
  }
  
  Widget _checkBox(){
    if( _selectedType == 0){
      return IconButton(
        icon: Icon(Icons.check_box_outline_blank, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 2;
            _selectEmployees(true);
          });
        },
      );
    }else if( _selectedType == 1){
      return IconButton(
        icon: Icon(Icons.indeterminate_check_box, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 2;
            _selectEmployees(true);
          });
        },
      );
    }else{
      return IconButton(
        icon: Icon(Icons.check_box, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 0;
            _selectEmployees(false);
          });
        },
      );
    }
  }

  _selectEmployees( bool check ){
    for( int i=0; i < _items.length; i++){
      _items[i].check = check;
    }
  }
  _initLocationService() async {
    await _locationService.changeSettings(accuracy: LocationAccuracy.high, interval: 1000);
    LocationData location;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      bool serviceStatus = await _locationService.serviceEnabled();

      if (serviceStatus) {
        PermissionStatus _permissionGranted;
        _permissionGranted = await _locationService.hasPermission();
        if (_permissionGranted == PermissionStatus.denied) {
          _permissionGranted = await _locationService.requestPermission();
          if (_permissionGranted != PermissionStatus.granted) {
            return;
          }
        }

        location = await _locationService.getLocation();
        _locationService.onLocationChanged.listen((LocationData result) async {
          if(mounted){
            _currentLocation = result;
            _startLocation = _currentLocation;
          }
        });

      } else {
        bool serviceStatusResult = await _locationService.requestService();
        if(serviceStatusResult){
          _initLocationService();
        }
      }
    } on PlatformException catch (e) {
      if( Constant.ENABLE_LOGS)
        print(e);
      if (e.code == 'PERMISSION_DENIED') {
        error = e.message;
      } else if (e.code == 'SERVICE_STATUS_ERROR') {
        error = e.message;
      }
      location = null;
    }

    setState(() {
      _startLocation = location;
    });

  }

  bool _validate(){

    if( _controller_name.text.trim().isEmpty){
      showMessage('Zone name is required!');
      return false;
    }

    if( _restrictedType == add.GeofencingAddRequest.RESTRICTED_TYPE_GPS){
      if( _controller_latitude.text.trim().isEmpty || _controller_longitude.text.trim().isEmpty){
        showMessage('Latitude and longitude are required!');
        return false;
      }

      try {
        if (double.parse(_controller_latitude.text.trim()) > 90 ||
            double.parse(_controller_latitude.text.trim()) < -90) {
          showMessage('Invalid latitude value!');
          return false;
        }

        if (double.parse(_controller_longitude.text.trim()) > 180 ||
            double.parse(_controller_longitude.text.trim()) < -180) {
          showMessage('Invalid longitude value!');
          return false;
        }
      }catch(e){return false;}

    }else{
      if( _controller_bssid.text.trim().isEmpty){
        showMessage("Wifi device's MAC address is required!");
        return false;
      }

      final RegExp regex = new RegExp(r'(([0-9A-Fa-f]{2}[:-]?){6})');
      bool result= regex.hasMatch(_controller_bssid.text.trim());
      if( !result){
        showMessage("Invalid MAC address, it must be \"6 groups of 2 hexadecimal digits, separated by hyphens - or colons :\"");
        return result;
      }else{
        if( _controller_bssid.text.trim().contains("-") ){
          String bssid = _controller_bssid.text.trim();
          _controller_bssid.text = bssid.replaceAll("-", ":");
        }
      }
    }

    if( _items.isEmpty){
      setState(() {
        _selectedIndex = 1;
      });
      showMessage("Users should be added into this zone!");
      return false;
    }

    return true;
  }

  Future<void> _save() async{
    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Zone name is required!');
      return;
    }
    form.save();

    if( _validate() ){
      setState(() {
        _processing = true;
        _progressColor = Colors.white;
      });
      try{
        add.GeofencingAddRequest request;
        if( _restrictedType == add.GeofencingAddRequest.RESTRICTED_TYPE_GPS){
          request = new add.GeofencingAddRequest(
            organizationGUID: _currentUser.organizationGUID,
            name: _controller_name.text.trim(),
            restrictedType: _restrictedType,
            latitude: double.parse(_controller_latitude.text.trim()),
            longitude: double.parse(_controller_longitude.text.trim()),
            radius: _radius.toInt(),
            action: _action,
            employees: _items
          );
        }else{
          request = new add.GeofencingAddRequest(
              organizationGUID: _currentUser.organizationGUID,
              name: _controller_name.text.trim(),
              restrictedType: _restrictedType,
              action: _action,
              bssId: _controller_bssid.text.trim(),
              ssId: _wifi!=null?_wifi['ssid']:null,
              wifiCondition: _wifiCondition,
              employees: _items
          );
        }

        String guid = await _geofencingService.add(request, _accessToken);
        if( guid != null && guid.length > 10){
          showMessage('${_controller_name.text} zone created with successful');
          new ThreadTimer(new Duration(milliseconds: 1000), (){Navigator.pop(context, guid);});
        }else{
          showMessage('Sorry, new zone creating failed!');
          setState(() {
            _processing = false;
            _progressColor = Colors.transparent;
          });
        }
      }catch(e){}
    }

  }

  Future<void> _update() async{
    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      showMessage('Zone name is required!');
      return;
    }
    form.save();

    if( _validate() ){
      setState(() {
        _processing = true;
        _progressColor = Colors.white;
      });
      try{
        GeofencingUpdateRequest request;
        if( _restrictedType == add.GeofencingAddRequest.RESTRICTED_TYPE_GPS){
          request = new GeofencingUpdateRequest(
            name: _controller_name.text.trim(),
            restrictedType: _restrictedType,
            latitude: double.parse(_controller_latitude.text.trim()),
            longitude: double.parse(_controller_longitude.text.trim()),
            radius: _radius.toInt(),
            action: _action,
            guid: _response.guid
          );
        }else{
          request = new GeofencingUpdateRequest(
              name: _controller_name.text.trim(),
              restrictedType: _restrictedType,
              action: _action,
              bssId: _controller_bssid.text.trim(),
              ssId: _wifi!=null?_wifi['ssid']:null,
              wifiCondition: _wifiCondition,
            guid: _response.guid
          );
        }

        String guid = await _geofencingService.update(request, _accessToken);
        if( guid != null && guid.length > 10){
          showMessage('${_controller_name.text} zone updated with successful');
          new ThreadTimer(new Duration(milliseconds: 1000), (){Navigator.pop(context, guid);});
        }else{
          showMessage('Sorry, ${_controller_name.text} zone updating failed!');
          setState(() {
            _processing = false;
            _progressColor = Colors.transparent;
          });
        }
      }catch(e){}
    }

  }

  Future<bool> _showConfirmDialog( BuildContext context, String message )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm!"),
          content: new Text(message),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }
}

class EmployeeResultItem extends StatefulWidget {

  EmployeeResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.onDelete, this.deleteMode,
    this.isCheck, @required this.isEnd}) : super(key: key);
  final GeofencingEmployeeItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final Function onDelete;
  final bool deleteMode;
  final Function isCheck;
  final bool isEnd;

  @override
  _EmployeeResultItemState createState() => _EmployeeResultItemState();

}

class _EmployeeResultItemState extends State<EmployeeResultItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
          children: <Widget>[
            Card(
              elevation: 0,
              child: Container(
                  //padding: EdgeInsets.only(top:10),
                    child: ListTile(
                      title: Text(widget.post.employee, style: TextStyle(color: Theme.of(context).primaryColor),),
                      subtitle: Text('${widget.post.jobTitle??''}, ${widget.post.department} Dept'),
                      leading: ClipOval(
                          child: Material(
                            elevation: 1,
                            color: Colors.transparent, // button color
                            child: InkWell(
                              splashColor: Colors.red, // inkwell color
                              child: SizedBox(
                                width: 48,
                                height: 48,
                                child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.facialGUID}',
                                  headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
                              ),
                            ),
                          )
                      ),
                      trailing: !widget.deleteMode? IconButton(
                        icon: Icon(Icons.delete_forever,),
                        color: Colors.red[60],
                        onPressed:(){ widget.onDelete(); },
                      )
                      :
                      (widget.post.check?
                      IconButton(
                        icon: Icon(Icons.check_box, ),
                        color: Theme.of(context).primaryColor,
                        onPressed:(){
                          setState(() {
                            widget.post.check = !widget.post.check;
                          });
                          widget.isCheck(false);
                        },
                      )
                          :
                      IconButton(
                        icon: Icon(Icons.check_box_outline_blank,),
                        color: Theme.of(context).primaryColor,
                        onPressed:(){
                          setState(() {
                            widget.post.check = !widget.post.check;
                          });
                          widget.isCheck(true);
                        },
                      )
                      ),
                      onTap: (){
                        if( widget.deleteMode){
                          setState(() {
                            widget.post.check = !widget.post.check;
                            widget.isCheck(widget.post.check);
                          });
                        }
                      },
                    )
                )
            ),
            widget.isEnd? SizedBox(
              height: 150,
            ) : Container()
          ]
        )
    );
  }

}