import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/request/geofencing_add_request.dart';
import '/hrm/model/request/geofencing_search_request.dart';
import '/hrm/model/response/geofencing_search_response.dart';
import '/hrm/service/geofencing_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'add_geofencing.dart';


class GeofencingListHome extends StatefulWidget {
  GeofencingListHome({Key key}) : super(key: key);
  @override
  _GeofencingListHomeState createState() => _GeofencingListHomeState();
}


class _GeofencingListHomeState extends State<GeofencingListHome> {

  bool _waiting = false;
  Color _progressColor = Colors.transparent;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Attendance zone restrictions'),
              bottom: _createProgressIndicator(),
            ),
            body: _waiting? new Container() : GeofencingListFragment(
              onUpdate: (value){
                setState(() {
                  _waiting = value;
                });
              },
              onChange: (color){
                setState(() {
                  _progressColor = color;
                });
              },
            ),
            floatingActionButton:FloatingActionButton.extended(
              label: Text(
                'Add new zone',
                style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_location),
              backgroundColor: Theme.of(context).primaryColor,
              onPressed: () async{
                setState(() {
                  _waiting = true;
                });
                String guid = await Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) => AddGeofencing(
                      title: 'New zone',
                    ),
                  )
                );
                setState(() {
                  _waiting = false;
                });
              },
            )
        )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}

class GeofencingListFragment extends StatefulWidget {
  GeofencingListFragment({Key key, @required this.onUpdate, this.onChange}) : super(key: key);
  final Function onUpdate;
  final Function onChange;

  @override
  _GeofencingListFragmentState createState() => _GeofencingListFragmentState();
}

class _GeofencingListFragmentState extends State<GeofencingListFragment>{

  int _page = 0;
  String _searchText='';

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  GeofencingSearchResponse _searchResponse;
  String _accessToken;
  GeofencingService _geofencingService = new GeofencingService();
  User _currentUser = new User();
  List<GeofencingItem> _geofencingItem = [];
  GeofencingSearchRequest _request = new GeofencingSearchRequest();
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    _request.organizationGUID = _currentUser.organizationGUID;
    _request.first = _page;
    _request.max = Constant.ROW_PER_PAGE;


    _searchResponse = await _geofencingService.search(_request, _accessToken );
    if( _searchResponse!=null && _searchResponse.items!=null){

      if( refresh && _geofencingItem.isNotEmpty)
        _geofencingItem.removeRange(0, _geofencingItem.length);

      _searchResponse.items.forEach((element) {
        _geofencingItem.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => GeofencingResultItem(
        post: _geofencingItem[i],
        accessToken: _accessToken,
        geofencingService: _geofencingService,
        sharedPrefs: _sharedPrefs,
        isEnd: i+1 == _geofencingItem.length,
        onUpdate: (value){
          widget.onUpdate(value);
        },
        onChange: (color){
          widget.onChange(color);
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _geofencingItem.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class GeofencingResultItem extends StatefulWidget {

  GeofencingResultItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.onUpdate, this.onChange,
    @required this.geofencingService, @required this.isEnd}) : super(key: key);
  final GeofencingItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final Function onUpdate;
  final Function onChange;
  final GeofencingService geofencingService;
  final bool isEnd;

  @override
  _GeofencingResultItemState createState() => _GeofencingResultItemState();

}

class _GeofencingResultItemState extends State<GeofencingResultItem> {

  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(

        child: Wrap(
            children: <Widget>[
              Card(
                elevation: 0,
                child: Container(
                //padding: EdgeInsets.only(top:10),
                  child: ListTile(
                    leading: widget.post.restrictedType == GeofencingAddRequest.RESTRICTED_TYPE_GPS? Icon(Icons.gps_fixed) : Icon(Icons.wifi),
                    title: Text(
                      widget.post.name,
                      style: TextStyle(color: Theme.of(context).primaryColor),
                    ),
                    subtitle: new Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Container(
                              margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                              child: new Wrap(
                                  children: <Widget>[
                                    Container(
                                      child: Text('Restricted for: '),
                                      width: MediaQuery.of(context).size.width*0.25,
                                    ),
                                    Container(
                                      child: Text(
                                        '${widget.post.action==GeofencingAddRequest.ACTION_BOTH?'Punch-in/Punch-out': 'Punch-'+widget.post.action}',
                                        style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                      ),
                                    ),
                                  ]
                              )
                          ),
                          Container(
                              margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                              child: new Wrap(
                                  children: <Widget>[
                                    Container(
                                      child: Text('Updated On: '),
                                      width: MediaQuery.of(context).size.width*0.25,
                                    ),
                                    Container(
                                      child: Text(
                                        '${new DateFormat('dd/MM/yy hh:mm a').format(widget.post.updatedOn.toLocal())}',
                                        style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                      ),
                                    ),
                                  ]
                              )
                          ),
                          Container(
                              margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                              child: new Wrap(
                                  children: <Widget>[
                                    Container(
                                      child: RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.edit, color: Theme.of(context).primaryColor),
                                        color: Colors.grey[200],
                                        onPressed:_processing?null: () {
                                          _gotoAddOrUpdate(widget.post.name, widget.post.guid, null);
                                        },
                                        label: Text(
                                          'Edit',
                                          style: TextStyle(
                                          ),
                                        ),
                                      ),
                                    ),
                                    Container(
                                      margin: const EdgeInsets.only(left: 10.0, right: 0.0, top: 0),
                                      child: RaisedButton.icon(
                                        elevation: 0.0,
                                        icon: Icon(Icons.person_add, color: Theme.of(context).primaryColor),
                                        color: Colors.grey[200],
                                        onPressed:_processing?null: () {
                                          _gotoAddOrUpdate(widget.post.name, widget.post.guid, 1);
                                        },
                                        label: Text(
                                          'Add user',
                                          style: TextStyle(
                                          ),
                                        ),
                                      ),
                                    ),
                                  ]
                              )
                          ),
                        ]
                    ),
  //                  subtitle: Text(
  //                      'Restricted for: ${widget.post.action==GeofencingAddRequest.ACTION_BOTH?'Punch-in/Punch-out': 'Punch-'+widget.post.action}'
  //                  ),
                    trailing: widget.post.status== GeofencingAddRequest.STATUS_ACTIVE?
                    IconButton(
                      icon: Icon(Icons.check_box, ),
                      color: Theme.of(context).primaryColor,
                      onPressed:_processing?null: (){
                        _changeStatus(context, false);
                        },
                    )
                        :
                    IconButton(
                      icon: Icon(Icons.check_box_outline_blank,),
                      color: Theme.of(context).primaryColor,
                      onPressed:_processing?null: (){
                        _changeStatus(context, true);
                      },
                    ),
                    onTap:_processing?null: (){
                      _gotoAddOrUpdate(widget.post.name, widget.post.guid, null);
                    },
                  )
                ),
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

  void _changeStatus(BuildContext context, bool isEnable )async{
    String message = "Are you sure to ${isEnable?'turn on':'turn off'} the ${widget.post.name} zone?";
    bool result = await _showConfirmDialog(context, message);
    if( result ){
      setState(() {
        _processing = true;
      });
      widget.onChange(Colors.white);

      try{
        result = await widget.geofencingService.changeStatus(
            isEnable? GeofencingAddRequest.STATUS_ACTIVE : GeofencingAddRequest.STATUS_INACTIVE,
            widget.post.guid,
            widget.accessToken
        );

        if( result ){
          setState(() {
            _processing = false;
            widget.post.status = isEnable? GeofencingAddRequest.STATUS_ACTIVE : GeofencingAddRequest.STATUS_INACTIVE;
          });
          widget.onChange(Colors.transparent);
        }

      }catch(e){
        if(Constant.ENABLE_LOGS)
          print(e);
      }
    }
  }

  void _gotoAddOrUpdate(String title, String guid, int selectedInde) async{
    widget.onUpdate(true);
    String updatedGuid = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => AddGeofencing(
            title: title,
            guid: guid,
            selectedIndex: selectedInde,
          ),
        )
    );
    widget.onUpdate(false);
  }

  Future<bool> _showConfirmDialog( BuildContext context, String message )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Action confirm!"),
          content: new Text(message),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }
}