import 'dart:async';
import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';



class GeofencingMap extends StatefulWidget {
  GeofencingMap({@required this.lat, @required this.lon, @required this.title});
  final double lat;
  final double lon;
  final title;
  @override
  _GeofencingMapState createState() => _GeofencingMapState();
}

class _GeofencingMapState extends State<GeofencingMap> {

  MapType _mapType = MapType.normal;
  bool _rotateGesturesEnabled = true;
  bool _scrollGesturesEnabled = true;
  bool _tiltGesturesEnabled = true;
  bool _zoomGesturesEnabled = true;
  bool _indoorViewEnabled = true;
  bool _myLocationEnabled = true;
  bool _myLocationButtonEnabled = true;
  bool _isMapCreated = false;
  bool _isMoving = false;
  bool _compassEnabled = true;
  bool _mapToolbarEnabled = true;

  GoogleMapController _controller;
  CameraPosition _kGooglePlex;
  Map<MarkerId, Marker> markers = <MarkerId, Marker>{};
  MarkerId selectedMarker;
  BitmapDescriptor _markerIcon;
  final MarkerId markerId = MarkerId('123');

  @override
  void initState() {
    super.initState();
    _kGooglePlex = CameraPosition(
      target: widget.lat==null ? LatLng(16.813538,96.171205) : LatLng(widget.lat, widget.lon),
      zoom: 14.4746,
    );
  }

  Future<void> _createMarkerImageFromAsset(BuildContext context) async {
    if (_markerIcon == null) {
      final ImageConfiguration imageConfiguration =
      createLocalImageConfiguration(context);
      BitmapDescriptor.fromAssetImage(
          imageConfiguration, 'assets/images/map_marker.png')
          .then(_updateBitmap);
    }
  }
  void _updateBitmap(BitmapDescriptor bitmap) {
    setState(() {
      _markerIcon = bitmap;
    });
  }

  void _add( LatLng position) {
    // creating a new MARKER
    final Marker marker = Marker(
      markerId: markerId,
      position: position,
      infoWindow: InfoWindow(title: 'Marker position', snippet: '${position.latitude},${position.longitude}'),
      onTap: () {
        _onMarkerTapped(markerId);
      },
    );

    setState(() {
      markers.clear( );
      // adding a new marker to map
      markers[markerId] = marker;
    });
  }

  void _onMapCreated(GoogleMapController controller) {
    this._controller = controller;
    _add( _kGooglePlex.target );
  }

  void _onMarkerTapped(MarkerId markerId) {
    final Marker tappedMarker = markers[markerId];
    if (tappedMarker != null) {
      setState(() {
        if (markers.containsKey(selectedMarker)) {
          final Marker resetOld = markers[selectedMarker]
              .copyWith(iconParam: BitmapDescriptor.defaultMarker);
          markers[selectedMarker] = resetOld;
        }
        selectedMarker = markerId;
        final Marker newMarker = tappedMarker.copyWith(
          iconParam: BitmapDescriptor.defaultMarkerWithHue(
            BitmapDescriptor.hueGreen,
          ),
        );
        markers[markerId] = newMarker;
      });
    }
  }


  @override
  void dispose() {
    super.dispose();
  }


  @override
  Widget build(BuildContext context) {
    _createMarkerImageFromAsset(context);
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
        appBar: AppBar(
          title: Text(widget.title),
            bottom: _createProgressIndicator(),
        ),
        body: Center(
          child:Container(
            //padding: const EdgeInsets.only(left: 5.0, top: 10.0, right: 5.0),
            child: GoogleMap(
              mapType: _mapType,
              initialCameraPosition: _kGooglePlex,
              onMapCreated: _onMapCreated,
              compassEnabled: _compassEnabled,
              mapToolbarEnabled: _mapToolbarEnabled,
              rotateGesturesEnabled: _rotateGesturesEnabled,
              scrollGesturesEnabled: _scrollGesturesEnabled,
              tiltGesturesEnabled: _tiltGesturesEnabled,
              zoomGesturesEnabled: _zoomGesturesEnabled,
              indoorViewEnabled: _indoorViewEnabled,
              myLocationEnabled: _myLocationEnabled,
              myLocationButtonEnabled: _myLocationButtonEnabled,
              markers: Set<Marker>.of(markers.values),
              onTap: (LatLng pos) {
                setState(() {_add( pos);});
              },
            ),
          )
        ),
      )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 60.0),
      child: SizedBox(
          height: 60.0,
          child: Container(
            child: Ink(
            color: Colors.white,
            child: ListTile(
              title: Text(
                  'Tap on map and press here',
                style: TextStyle(color: Theme.of(context).primaryColor),
              ),
              leading: Icon(Icons.info_outline, color: Theme.of(context).primaryColor),
              trailing: Icon(Icons.add_location, color: Theme.of(context).primaryColor),
              onTap: (){
                if( markers.isEmpty){

                }else{
                  Marker tappedMarker = markers[markerId];
                  Navigator.pop(context,{'lat':tappedMarker.position.latitude,'lon':tappedMarker.position.longitude});
                }
              },
            )
            )
          )
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}