

import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/model/response/geofencing_detail_response.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';



class SelectedEmployees extends StatefulWidget {
  SelectedEmployees({Key key, @required this.selectedEmployees}) : super(key: key);
  List<GeofencingEmployeeItem> selectedEmployees;
  @override
  _SelectedEmployeesState createState() => _SelectedEmployeesState();
}


class _SelectedEmployeesState extends State<SelectedEmployees> {

  int _selectedIndex = 0;
  int _selectedType = 0;
  List<GeofencingEmployeeItem> _employees = [];

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Employee list'),
              bottom: _createSelectionIndicator(),
            ),
            body: SelectedEmployeeList(
              selectedEmployees: widget.selectedEmployees,
              checkType: (value){
                setState(() {
                  _selectedType = value;
                });
              },
              employees: (value){
                _employees = value;
                },
            ),
          bottomNavigationBar: BottomNavigationBar(
            items: const <BottomNavigationBarItem>[
              BottomNavigationBarItem(
                icon: Icon(Icons.done_all),
                title: Text('OK'),
              ),
              BottomNavigationBarItem(
                icon: Icon(Icons.cancel),
                title: Text('Cancel'),
              ),
            ],
            currentIndex: _selectedIndex,
            selectedItemColor: Theme.of( context ).primaryColor,
            unselectedItemColor: Theme.of( context ).primaryColor,
            showUnselectedLabels: true,
            showSelectedLabels: true,
            onTap: _onItemTapped,
          ),
        )
    );
  }

  void _onItemTapped(int index) {
    setState(() {
      _selectedIndex = index;
    });
    if( _selectedIndex == 1){
      Navigator.pop(context);
    }else{
      List<GeofencingEmployeeItem> selectedEmployees = [];
      selectedEmployees.addAll(_employees);
      selectedEmployees.retainWhere((element) {
        if( element.check)
          return true;
        else
          return false;
      });
      Navigator.pop(context, selectedEmployees);
    }
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  PreferredSize _createSelectionIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 70.0),
      child: SizedBox(
          height: 70.0,
          child: Container(
              child: Ink(
                  color: Colors.white,
                  child: ListTile(
                    title: new TextFormField(
                      decoration: const InputDecoration(
                        icon: const Icon(Icons.search),
                        hintText: 'Employee name',
                        labelText: 'Search',
                      ),
                    ),
                    trailing: _checkBox()
                  )
              )
          )
      )
  );

  Widget _checkBox(){
    if( _selectedType == 0){
      return IconButton(
        icon: Icon(Icons.check_box_outline_blank, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 2;
            _selectEmployees(true);
          });
        },
      );
    }else if( _selectedType == 1){
      return IconButton(
        icon: Icon(Icons.indeterminate_check_box, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 2;
            _selectEmployees(true);
          });
        },
      );
    }else{
      return IconButton(
        icon: Icon(Icons.check_box, ),
        color: Theme.of(context).primaryColor,
        onPressed:(){
          setState(() {
            _selectedType = 0;
            _selectEmployees(false);
          });
        },
      );
    }
  }

  _selectEmployees( bool check ){
    for( int i=0; i < _employees.length; i++){
      _employees[i].check = check;
    }
  }


}


class SelectedEmployeeList extends StatefulWidget {
  SelectedEmployeeList({Key key, @required this.checkType, @required this.employees, @required this.selectedEmployees}) : super(key: key);
  final Function checkType;
  final Function employees;
  List<GeofencingEmployeeItem> selectedEmployees;
  @override
  _SelectedEmployeeListState createState() => _SelectedEmployeeListState();
}

class _SelectedEmployeeListState extends State<SelectedEmployeeList>{

  int _checkCount = 0;
  int _page = 0;

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  EmployeeSearchResponse _employeeSearchResponse;
  String _accessToken;
  EmployeeService _employeeService = new EmployeeService();
  User _currentUser = new User();
  List<GeofencingEmployeeItem> _employees = [];
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();

    _init();
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _employeeSearchResponse = await _employeeService.search(_accessToken, null, _page,  100000);
    if( _employeeSearchResponse!=null && _employeeSearchResponse.employees!=null){

      if( refresh && _employees.isNotEmpty)
        _employees.removeRange(0, _employees.length);

      _employeeSearchResponse.employees.forEach((element) {
        bool existing = false;
        if( widget.selectedEmployees != null){
          widget.selectedEmployees.forEach((element1) {
            if( element1.employeeGUID == element.guid){
              existing = true;
              return;
            }
          });
        }

        if( !existing) {
          GeofencingEmployeeItem geofencingEmployeeItem = new GeofencingEmployeeItem(
              employee: element.name,
              department: element.departmentName,
              jobTitle: element.jobTitleName,
              employeeGUID: element.guid,
              facialGUID: element.faceGuid,
              guid: null,
              check: false
          );
          _employees.add(geofencingEmployeeItem);
        }
      });

      widget.employees(_employees);
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 0, right: 0),
      itemBuilder: (c, i) => EmployeeResultItem(
        post: _employees[i],
        accessToken: _accessToken,
        sharedPrefs: _sharedPrefs,
        isCheck: (value){
          _checkCount = 0;
          _employees.forEach((element) {
            if( element.check)
              ++_checkCount;
          });
          if( _checkCount == _employees.length){
            widget.checkType(2);
          }else if( _checkCount> 0 && _checkCount != _employees.length){
            widget.checkType(1);
          }else
            widget.checkType(0);
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _employees.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class EmployeeResultItem extends StatefulWidget {

  EmployeeResultItem({Key key, @required this.post, @required this.accessToken, @required this.sharedPrefs, @required this.isCheck}) : super(key: key);
  final GeofencingEmployeeItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final Function isCheck;

  @override
  _EmployeeResultItemState createState() => _EmployeeResultItemState();

}

class _EmployeeResultItemState extends State<EmployeeResultItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      //padding: const EdgeInsets.all(10),
      //height: 100,
        child: Card(
            elevation: 0,
            child: Container(
              //padding: EdgeInsets.only(top:10),
                child: ListTile(
                  title: Text(widget.post.employee, style: TextStyle(color: Theme.of(context).primaryColor),),
                  subtitle: Text('${widget.post.jobTitle??''}, ${widget.post.department} Dept'),
                  leading: ClipOval(
                      child: Material(
                        elevation: 1,
                        color: Colors.transparent, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 48,
                            height: 48,
                            child: new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${widget.post.facialGUID}',
                              headers: {'Authorization':'Bearer ${widget.accessToken}'}, width: 48, height: 48, ),
                          ),
                        ),
                      )
                  ),
                  trailing: widget.post.check?
                  IconButton(
                    icon: Icon(Icons.check_box, ),
                    color: Theme.of(context).primaryColor,
                    onPressed:(){
                      setState(() {
                        widget.post.check = !widget.post.check;
                        widget.isCheck(false);
                      });
                      },
                  )
                      :
                  IconButton(
                    icon: Icon(Icons.check_box_outline_blank,),
                    color: Theme.of(context).primaryColor,
                    onPressed:(){
                      setState(() {
                        widget.post.check = !widget.post.check;
                        widget.isCheck(true);
                      });
                      },
                  ),
                  onTap: (){
                    setState(() {
                      widget.post.check = !widget.post.check;
                      widget.isCheck(widget.post.check);
                    });
                  },
                )
            )
        )
    );

  }
}