

import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/kiosk_list_response.dart';
import '/hrm/service/kiosk_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';



class KioskListHome extends StatefulWidget {
  KioskListHome({Key key}) : super(key: key);
  @override
  _KioskListHomeState createState() => _KioskListHomeState();
}


class _KioskListHomeState extends State<KioskListHome> {

  bool _waiting = false;
  Color _progressColor = Colors.transparent;
  KioskService _kioskService;
  String _accessToken;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Stand mode codes'),
              bottom: _createProgressIndicator(),
            ),
            body: _waiting? new Container() : KioskListFragment(
              onUpdate: (value){
                setState(() {
                  _waiting = value;
                });
              },
              onChange: (color){
                setState(() {
                  _progressColor = color;
                });
              },
              onCallBack: (service, token){
                _kioskService = service;
                _accessToken = token;
              },
            ),
            floatingActionButton:FloatingActionButton.extended(
              label: Text(
                'New code',
                style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.vpn_key),
              backgroundColor: Theme.of(context).primaryColor,
              onPressed: () async{
                setState(() {
                  _waiting = true;
                });

                String message = "Are you sure to generate new code?";
                bool result = await _showConfirmDialog(context, message);
                if( result )
                  String key = await _kioskService.generate( _accessToken );

                setState(() {
                  _waiting = false;
                });
              },
            )
        )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}

class KioskListFragment extends StatefulWidget {
  KioskListFragment({Key key, @required this.onUpdate, this.onChange, this.onCallBack}) : super(key: key);
  final Function onUpdate;
  final Function onChange;
  final Function onCallBack;

  @override
  _KioskListFragmentState createState() => _KioskListFragmentState();
}

class _KioskListFragmentState extends State<KioskListFragment>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  KioskListResponse _searchResponse;
  String _accessToken;
  KioskService _kioskService = new KioskService();
  User _currentUser = new User();
  List<KioskListItem> kioskListItems = [];
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    widget.onCallBack( _kioskService, _accessToken );


    _searchResponse = await _kioskService.list(_accessToken, _page, Constant.ROW_PER_PAGE);
    if( _searchResponse!=null && _searchResponse.items!=null){

      if( refresh && kioskListItems.isNotEmpty)
        kioskListItems.removeRange(0, kioskListItems.length);

      _searchResponse.items.forEach((element) {
        kioskListItems.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => KioskItem(
        post: kioskListItems[i],
        accessToken: _accessToken,
        kioskService: _kioskService,
        sharedPrefs: _sharedPrefs,
        isEnd: i+1 == kioskListItems.length,
        onUpdate: (value){
          widget.onUpdate(value);
        },
        onChange: (color){
          widget.onChange(color);
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: kioskListItems.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class KioskItem extends StatefulWidget {

  KioskItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.onUpdate, this.onChange,
    @required this.kioskService, @required this.isEnd}) : super(key: key);
  final KioskListItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final Function onUpdate;
  final Function onChange;
  final KioskService kioskService;
  final bool isEnd;

  @override
  _KioskListItemState createState() => _KioskListItemState();

}

class _KioskListItemState extends State<KioskItem> {

  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
          children: <Widget>[
            Card(
              elevation: 0,
              child: Container(
              //padding: EdgeInsets.only(top:10),
                child: ListTile(
                  leading: Icon(Icons.vpn_key),
                  title: Text(
                    'Code: ${widget.post.token}',
                    style: TextStyle(color: widget.post.usedBy==null ? Theme.of(context).primaryColor : Colors.black54),
                  ),
                  subtitle: Text('${new DateFormat('dd/MM/yy hh:mm a').format(widget.post.createdOn.toLocal())}'),
                  trailing: widget.post.active?
                  IconButton(
                    icon: Icon(Icons.check_box, ),
                    color: Theme.of(context).primaryColor,
                    onPressed:_processing?null: (){
                      _changeStatus(context, false);
                      },
                  )
                      :
                  IconButton(
                    icon: Icon(Icons.check_box_outline_blank,),
                    color: Theme.of(context).primaryColor,
                    onPressed:_processing?null: (){
                      _changeStatus(context, true);
                    },
                  ),
                )
              ),
            ),
            widget.isEnd? SizedBox(
              height: 200,
            ) : Container()
          ]
        )
    );
  }

  void _changeStatus(BuildContext context, bool isEnable )async{
    String message = "Are you sure to ${isEnable?'enable':'disable'} the key code: ${widget.post.token}?";
    bool result = await _showConfirmDialog(context, message);
    if( result ){
      setState(() {
        _processing = true;
      });
      widget.onChange(Colors.white);

      try{

        bool result = false;
        if( isEnable)
          result = await widget.kioskService.enable( widget.post.guid, widget.accessToken );
        else
          result = await widget.kioskService.disable( widget.post.guid, widget.accessToken );

        if( result ){
          setState(() {
            _processing = false;
            widget.post.active = isEnable;
          });
          widget.onChange(Colors.transparent);
        }

      }catch(e){
        if(Constant.ENABLE_LOGS)
          print(e);
      }
    }
  }
}
Future<bool> _showConfirmDialog( BuildContext context, String message )async {
  bool result = false;
  await showDialog(
    context: context,
    builder: (BuildContext context) {
      // return object of type Dialog
      return AlertDialog(
        title: new Text("Action confirm!"),
        content: new Text(message),
        actions: <Widget>[
          // usually buttons at the bottom of the dialog
          new FlatButton(
            child: new Text("Yes"),
            onPressed: () { result = true;Navigator.of(context).pop();},
          ),
          new FlatButton(
            child: new Text("No"),
            onPressed: () { result = false; Navigator.of(context).pop();},
          ),
        ],
      );
    },
  );
  return result;
}