import 'package:badges/badges.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/database/dao/notification_dao.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/view/attendance/attendance_search.dart';
import '/hrm/view/organization/organization_home.dart';
import '/hrm/view/profile/profile_home.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:shared_preferences/shared_preferences.dart';
import '../login/pre_login.dart';
import 'geofencing/geofencing_list.dart';
import 'kiosk/kiosk_list.dart';
import 'notification/notification_list_home.dart';
import 'payslip/payslip.dart';
import 'settings/settings.dart';
import 'userreport/user_report_home.dart';


class MoreDetails extends StatefulWidget {
  MoreDetails({Key key, @required this.notificationCount, @required this.onNotificationChange}) : super(key: key);
  final int notificationCount;
  final Function onNotificationChange;

  @override
  _MoreDetailsState createState() => _MoreDetailsState();
}

class _MoreDetailsState extends State<MoreDetails>{
  User _currentUser = new User();
  NotificationDao notificationDao = new NotificationDao();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  SharedPreferences _sharedPrefs;
  int notificationCount = 0;
  OrganizationDetailResponse detailResponse = new OrganizationDetailResponse();

  @override
  initState() {
    super.initState( );
    _loadProfile();
  }

  void _loadProfile( )async{
    // loading login user for token generating
    await _currentUser.fromSharedPreferences( );
    await _authorityResponse.fromSharedPreferences();
    _sharedPrefs = await SharedPreferences.getInstance();
    notificationCount = widget.notificationCount;
    await detailResponse.fromSharedPreferences();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return ListView(
      children: <Widget>[
        Container(
          margin: EdgeInsets.all(16),
          child: Card(
            elevation: 1,
            child: ListTile(
              leading: Icon(
                  Icons.person,
                  size: 48,
                  color: Theme.of(context).primaryColor
              ),
              title: Text(
                tr(LocaleKeys.lblProfile),
                style: TextStyle(
                  color: Theme.of(context).primaryColor,
                ),
              ),
              subtitle: Text('Manage your personal information, logout, etc.'),
              trailing: Icon(Icons.keyboard_arrow_right),
              onTap: () {
                _gotoProfile();
              },
            )
          )
        ),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(
                      Icons.notifications_active,
                      size: 48,
                      color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    tr(LocaleKeys.lblNotifications),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('Pieces of information for you'),
                  trailing: Badge(
                    shape: BadgeShape.square,
                    borderRadius: BorderRadius.circular(20),
                    toAnimate: true,
                    badgeContent:
                    Text(
                      notificationCount<10? ' ${notificationCount} ' : '${notificationCount}',
                      style: TextStyle(
                          color: Colors.white
                      ),
                    ),
                  ),
                  onTap: () {
                    _gotoNotifications();
                  },
                )
            )
        ),
        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('ORGANIZATION_READ')? Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(
                      Icons.group_work,
                      size: 48,
                      color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    tr(LocaleKeys.lblAdministration),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text("Manage your organization. Configure the leave policies, setup the employee qualifications, download the administrative reports"),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoOrganization(context);
                  },
                )
            )
        ) : new Container(),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(
                      Icons.monetization_on,
                      size: 48,
                      color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    tr(LocaleKeys.lblPayslips),//tr(LocaleKeys.mnuEmployees),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text("Check and download your payslips."),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoPayslip(context);
                  },
                )
            )
        ),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(
                      Icons.assignment,
                      size: 48,
                      color: Theme.of(context).primaryColor
                  ),
                  title: Text(
                    tr(LocaleKeys.lblDownloadReports),//tr(LocaleKeys.mnuEmployees),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text("Download your attendance, leaves, overtime, salary packages reports."),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoReports(context);
                  },
                )
            )
        ),
        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('ORGANIZATION_READ')? Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: SvgPicture.asset(
                    'assets/images/kiosk.svg',
                    semanticsLabel: 'Stand mode management',
                    height: 48,
                    width: 48,
                    color: Theme.of(context).primaryColor,
                  ),
                  title: Text(
                    tr(LocaleKeys.lblStandMode),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('Stand & Go: automatic In/Out system, no need to touch like a fingerprint. One device works for every employees'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotokioskList();
                  },
                )
            )
        ) : new Container(),
        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ')? Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: SvgPicture.asset(
                    'assets/images/protection.svg',
                    semanticsLabel: 'Geofencing',
                    height: 48,
                    width: 48,
                  ),
                  title: Text(
                    tr(LocaleKeys.lblAttendanceZone),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('Restrict your employees punch-in / punch-out locations'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoGeofencing(context);
                  },
                )
            )
        ) : new Container(),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: SvgPicture.asset(
                    'assets/images/attend.svg',
                    semanticsLabel: 'Attendance',
                    height: 48,
                    width: 48,
                  ),
                  title: Text(
                    tr(LocaleKeys.lblTimeManagement),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('Report the attendances by employee or date range.'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoAttendanceSearch();
                  },
                )
            )
        ),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(Icons.settings, size: 48, color: Theme.of(context).primaryColor,),
                  title: Text(
                    tr(LocaleKeys.lblSettings),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('More convenience functions'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    _gotoSettings();
                  },
                )
            )
        ),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(Icons.help, size: 48, color: Theme.of(context).primaryColor,),
                  title: Text(
                    tr(LocaleKeys.lblHelp),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('We are always at your service'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    makeCall( Constant.CONTACT_PHONE );
                  },
                )
            )
        ),
        Container(
            margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
            child: Card(
                elevation: 1,
                child: ListTile(
                  leading: Icon(Icons.web, size: 48, color: Theme.of(context).primaryColor,),
                  title: Text(
                    tr(LocaleKeys.lblWebportal),
                    style: TextStyle(
                      color: Theme.of(context).primaryColor,
                    ),
                  ),
                  subtitle: Text('Mobile app included only few management features, full management features are available over website. hrtest.quickdoor2door.com'),
                  trailing: Icon(Icons.keyboard_arrow_right),
                  onTap: () {
                    openBrowser(Constant.HRM_URL);
                  },
                )
            )
        ),

      ],
    );
  }

  void _gotoAttendanceSearch() async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => AttendanceSearch( ),
        )
    );
  }


  void _gotoGeofencing(BuildContext context) async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }

      Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => GeofencingListHome(),
          )
      );
  }

  void _gotoOrganization(BuildContext context){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => OrganizationHome(
            currentUser: _currentUser,
            sharedPrefs: _sharedPrefs,
          ),
        )
    );
  }

  void _gotoPayslip(BuildContext context) async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => PayslipHome(),
        )
    );
  }
  void _gotoReports(BuildContext context) async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => UserReportHome(
            currentUser: _currentUser,
            sharedPrefs: _sharedPrefs,
          ),
        )
    );
  }

  void _gotoNotifications() async{
    await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => NotificationListHome( ),
        )
    );
    int count = await _refreshNotifications();
    notificationCount = count;
    widget.onNotificationChange(notificationCount);
  }

  void _gotoProfile() async{

    String message = await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => ProfileHome(
            userGUID: _currentUser.guid,
            employeeGUID: _currentUser.employeeGUID,
            showExit: true,
            sharedPrefs: _sharedPrefs,
          ),
        )
    );

    if( message!= null && message=='exit'){
      Navigator.pushReplacement(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => LoginSelection(title: tr(LocaleKeys.login))
          )
      );
    }
  }

  void _gotoSettings(){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => Settings( ),
        )
    );
  }

  void _gotokioskList()async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }

      Navigator.push(
          context,
          MaterialPageRoute(
            builder: (BuildContext context) => KioskListHome(),
          )
      );

  }

  void showMessage(String message) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _showUpgradeDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Upgrade required!"),
          content: new Text("Thank you for using QuickQ HR, this feature is required to upgrade your plan. Please contact us: " + Constant.CONTACT_PHONE),
          actions: <Widget>[
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

  Future<int> _refreshNotifications( )async{

    await notificationDao.open( );
    int notificationCount = await notificationDao.unreadCount();
    await notificationDao.close();

    return notificationCount;
  }

}