

import 'dart:async';
import 'dart:convert';
import '/hrm/config/constant.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';
import '/hrm/database/entity/notification.dart' as msg;



class NotificationDetail extends StatefulWidget {
  NotificationDetail({Key key, @required this.notification}) : super(key: key);
  final msg.Notification notification;

  @override
  _NotificationDetailState createState() => _NotificationDetailState();
}

class _NotificationDetailState extends State<NotificationDetail> {
  final Completer<WebViewController> _controller = Completer<WebViewController>();
  String contentBase64;

  @override
  void initState() {
    super.initState();
    if( widget.notification.url == null || (widget.notification.url!=null && widget.notification.url.trim().isEmpty) ) {
      contentBase64 = 'data:text/html;base64,${base64Encode(
          const Utf8Encoder().convert(
              """<!DOCTYPE html>
              <html>
                <head><meta name="viewport" content="width=device-width, initial-scale=1.0"></head>
                <body style='"margin: 0; padding: 0;'>
                  <div>${widget.notification.description}</div>
                </body>
              </html>""" ))}';
    }else
      contentBase64 = widget.notification.url;
  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(
            '${widget.notification.title}'
        ),
      ),
      body: Builder(builder: (BuildContext context) {
        return WebView(
          initialUrl: '$contentBase64',
          javascriptMode: JavascriptMode.unrestricted,
          onWebViewCreated: (WebViewController webViewController) {
            _controller.complete(webViewController);
          },
          // TODO Remove this when collection literals makes it to stable.
          // ignore: prefer_collection_literals
          javascriptChannels: <JavascriptChannel>[
            _toasterJavascriptChannel(context),
          ].toSet(),
          navigationDelegate: (NavigationRequest request) {
            if (request.url.startsWith('https://www.youtube.com/')) {
              if( Constant.ENABLE_LOGS)
                print('blocking navigation to $request}');
              return NavigationDecision.prevent;
            }
            if( Constant.ENABLE_LOGS)
              print('allowing navigation to $request');
            return NavigationDecision.navigate;
          },
          onPageStarted: (String url) {
            if( Constant.ENABLE_LOGS)
              print('Page started loading: $url');
          },
          onPageFinished: (String url) {
            if( Constant.ENABLE_LOGS)
              print('Page finished loading: $url');
          },
          gestureNavigationEnabled: true,
        );
      }),
    );
  }
  JavascriptChannel _toasterJavascriptChannel(BuildContext context) {
    return JavascriptChannel(
        name: 'Toaster',
        onMessageReceived: (JavascriptMessage message) {
          Scaffold.of(context).showSnackBar(
            SnackBar(content: Text(message.message)),
          );
        });
  }
}