

import '/hrm/config/constant.dart';
import '/hrm/database/dao/notification_dao.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:launch_review/launch_review.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import '/hrm/database/entity/notification.dart' as msg;
import 'notification_detail.dart';



class NotificationListHome extends StatefulWidget {
  NotificationListHome({Key key}) : super(key: key);
  @override
  _NotificationListHomeState createState() => _NotificationListHomeState();
}


class _NotificationListHomeState extends State<NotificationListHome> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
            title: Text('Notifications'),
          ),
          body: NotificationList()
        )
    );
  }

  Future<bool> _onWillPop() async {
    return true;
  }

}


class NotificationList extends StatefulWidget {
  NotificationList({Key key}) : super(key: key);
  @override
  _NotificationListState createState() => _NotificationListState();
}

class _NotificationListState extends State<NotificationList>{

  int _page = 0;
  String _searchText='';

  RefreshController _refreshController = RefreshController(initialRefresh: true);
  NotificationDao notificationDao = new NotificationDao();
  List<msg.Notification> _notifications = [];

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{
    await notificationDao.open();
    List<msg.Notification> notifications = await notificationDao.notifications(Constant.ROW_PER_PAGE, _page * Constant.ROW_PER_PAGE);
    if( refresh && _notifications.isNotEmpty)
      _notifications.removeRange(0, _notifications.length);

    notifications.forEach((element) {
    _notifications.add(element);
    });
    await notificationDao.close();
    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => NotificationListItem(
        post: _notifications[i],
        notificationDao: notificationDao,
        index: i,
        notifications: _notifications,
        onUndo: (index, notification){
//          setState(() {
//            _notifications.insert(index, notification);
//          });

        },
        onDelete: (notification) async{
          await notificationDao.open();
          await notificationDao.deleteById(notification.campaignId);
          await notificationDao.close();
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: _notifications.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class NotificationListItem extends StatefulWidget {

  NotificationListItem({Key key, @required this.post, @required this.notificationDao, this.index, this.notifications, this.onUndo, this.onDelete}) : super(key: key);
  final msg.Notification post;
  final NotificationDao notificationDao;
  final int index;
  final List<msg.Notification> notifications;
  final Function onUndo;
  final Function onDelete;

  @override
  _NotificationListItemState createState() => _NotificationListItemState();

}

class _NotificationListItemState extends State<NotificationListItem> {

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Dismissible(
          background: stackBehindDismiss(),
          key: ObjectKey(widget.post),
          confirmDismiss: (direction) async{
            if (direction == DismissDirection.endToStart) {
              Scaffold.of(context).showSnackBar(SnackBar(
                  content: Text("Notification deleted!"),
//                  action: SnackBarAction(
//                      label: "Undo",
//                      onPressed: () {
//                        widget.onUndo(widget.index, widget.post);
//                      }
//                  )
              )
              );
              widget.onDelete(widget.post);
              return true;
            }else{
              return false;
            }
          },
          child: Container(
        //padding: const EdgeInsets.all(10),
        //height: 100,
          child: Card(
              elevation: 0,
              child: Container(
                //padding: EdgeInsets.only(top:10),
                  child: ListTile(
                    title: Text(
                      '${widget.post.title}',
                      style: TextStyle(
                          color: widget.post.status==msg.Notification.STATUS_UNREAD? Theme.of(context).primaryColor : Colors.black,
                      ),
                    ),
                    subtitle: Text(
                        '${widget.post.brief!=null? widget.post.brief : (widget.post.description!=null? widget.post.description : widget.post.url)}',
                        overflow: TextOverflow.ellipsis,
                        maxLines: 2,
                    ),
                    //leading: widget.post.status==msg.Notification.STATUS_UNREAD? Icon(Icons.email) : Icon(Icons.drafts),
                    trailing: _dateDisplay(context),
                    onTap: ()async {
                      _updateStatus(widget.post);
                      if( widget.post.campaignTypeId == msg.Notification.TYPE_UPDATE){
                        LaunchReview.launch();
                      }else {
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (BuildContext context) =>
                                  NotificationDetail(
                                    notification: widget.post,
                                  ),
                            )
                        );
                      }
                    },
                  )
              )
          )
        )
      );
  }

  void deleteItem(index) {
    /*
    By implementing this method, it ensures that upon being dismissed from our widget tree,
    the item is removed from our list of items and our list is updated, hence
    preventing the "Dismissed widget still in widget tree error" when we reload.
    */
    setState(() {
      widget.notifications.removeAt(index);
    });
  }

  Widget stackBehindDismiss() {
    return Container(
      alignment: Alignment.centerRight,
      padding: EdgeInsets.only(right: 20.0),
      color: Colors.red,
      child: Icon(
        Icons.delete,
        color: Colors.white,
      ),
    );
  }

  Widget _dateDisplay(BuildContext context){
    String date = '';
    if( new DateFormat('dd-MM-yyyy').format(widget.post.scheduleAt.toLocal()) ==new DateFormat('dd-MM-yyyy').format(DateTime.now().toLocal())){
      date = new DateFormat('hh:mm a').format(widget.post.scheduleAt.toLocal());
    }else if( new DateFormat('yyyy').format(widget.post.scheduleAt.toLocal()) !=new DateFormat('yyyy').format(DateTime.now().toLocal())){
      date = new DateFormat('dd/MM/yy').format(widget.post.scheduleAt.toLocal());
    }else{
      date = new DateFormat('dd MMM').format(widget.post.scheduleAt.toLocal());
    }
    return Text(
      '$date',
      style: TextStyle(
          fontWeight: widget.post.status==msg.Notification.STATUS_UNREAD? FontWeight.bold : FontWeight.normal,
          color: widget.post.status==msg.Notification.STATUS_UNREAD? Theme.of(context).primaryColor : Colors.black,
      ),
    );

//    return Row(
//      children: <Widget>[
//        Container(
//          margin: EdgeInsets.only(top: 0),
//          child: Text(
//            '$date>',
//            style: TextStyle(
//                fontWeight: FontWeight.bold,
//                color:Theme.of(context).primaryColor
//            ),
//          ),
//        ),
//        Container(
//          margin: EdgeInsets.only(top: 0),
//          child: Icon(Icons.arrow_forward_ios,size: 12,)
//        ),
//      ],
//    );
  }

  void _updateStatus(msg.Notification notification) async{
    try{
      notification.status = msg.Notification.STATUS_READ;
      await widget.notificationDao.open();
      await widget.notificationDao.update(notification );
      await widget.notificationDao.close();
      setState(() {});
    }catch(e){print(e);}
  }
}