

import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/payslip_response.dart';
import '/hrm/service/payslip_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';



class PayslipHome extends StatefulWidget {
  PayslipHome({Key key}) : super(key: key);
  @override
  _PayslipHomeState createState() => _PayslipHomeState();
}


class _PayslipHomeState extends State<PayslipHome> {

  bool _waiting = false;
  Color _progressColor = Colors.transparent;


  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Payslips'),
              bottom: _createProgressIndicator(),
            ),
            body: _waiting? new Container() : PayslipFragment(),
        )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}

class PayslipFragment extends StatefulWidget {
  PayslipFragment({Key key}) : super(key: key);

  @override
  _PayslipFragmentState createState() => _PayslipFragmentState();
}

class _PayslipFragmentState extends State<PayslipFragment>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  PayslipResponse _searchResponse;
  String _accessToken;
  PayslipService _payslipService = new PayslipService();
  User _currentUser = new User();
  List<PayslipItem> payslipListItems = [];
  SharedPreferences _sharedPrefs;


  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() async{
    _sharedPrefs = await SharedPreferences.getInstance();
  }

  Future<void> _getLoadData( bool refresh ) async{

    if( _currentUser.facialGUID == null)
      await _currentUser.fromSharedPreferences( );

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );

    _searchResponse = await _payslipService.list(_currentUser.employeeGUID,
        _accessToken, _page, Constant.ROW_PER_PAGE);
    if( _searchResponse!=null && _searchResponse.items!=null){

      if( refresh && payslipListItems.isNotEmpty)
        payslipListItems.removeRange(0, payslipListItems.length);

      _searchResponse.items.forEach((element) {
        payslipListItems.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => PayslipViewItem(
        post: payslipListItems[i],
        accessToken: _accessToken,
        sharedPrefs: _sharedPrefs,
        isEnd: i+1 == payslipListItems.length,
        employeeGuid: _currentUser.employeeGUID,
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: payslipListItems.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class PayslipViewItem extends StatefulWidget {

  PayslipViewItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.isEnd, @required this.employeeGuid}) : super(key: key);
  final PayslipItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final bool isEnd;
  final String employeeGuid;

  @override
  _PayslipItemState createState() => _PayslipItemState();

}

class _PayslipItemState extends State<PayslipViewItem> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
            children: <Widget>[
              Card(
                elevation: 0,
                child: Container(
                  //padding: EdgeInsets.only(top:10),
                    child: ListTile(
                      leading: Icon(Icons.attach_money),
                      title: Text(
                        '${new DateFormat('dd/MM/yy').format(widget.post.dateStart.toLocal())} - ${new DateFormat('dd/MM/yy').format(widget.post.dateEnd.toLocal())}',
                        style: TextStyle(color: Theme.of(context).primaryColor),
                      ),
                      subtitle: Text('Pay frequency: ${widget.post.payFrequencyName}'),
                      trailing: Icon(Icons.file_download, color: Theme.of(context).primaryColor ),
                      onTap: (){
                        openBrowser(Constant.BASE_URL_PRIMARY
                            +'/v1/forwarder/eps/${widget.post.guid}?eg=${widget.employeeGuid}&at=${widget.accessToken}');
                      },
                    )
                ),
              ),
              widget.isEnd? SizedBox(
                height: 200,
              ) : Container()
            ]
        )
    );
  }

}
Future<bool> _showConfirmDialog( BuildContext context, String message )async {
  bool result = false;
  await showDialog(
    context: context,
    builder: (BuildContext context) {
      // return object of type Dialog
      return AlertDialog(
        title: new Text("Action confirm!"),
        content: new Text(message),
        actions: <Widget>[
          // usually buttons at the bottom of the dialog
          new FlatButton(
            child: new Text("Yes"),
            onPressed: () { result = true;Navigator.of(context).pop();},
          ),
          new FlatButton(
            child: new Text("No"),
            onPressed: () { result = false; Navigator.of(context).pop();},
          ),
        ],
      );
    },
  );
  return result;
}