import '/hrm/config/constant.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/notification_user_settings_response.dart';
import '/hrm/service/settings_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/view/login/pre_login.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:easy_localization/easy_localization.dart';


class Settings extends StatefulWidget {
  Settings({Key key}) : super(key: key);

  @override
  _SettingsState createState() => _SettingsState( );
}

class _SettingsState extends State<Settings> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  SharedPreferences _sharedPrefs;
  bool _processing = false;
  Color _progressColor = Colors.transparent;

  SettingsService _settingsService = new SettingsService();
  NotificationUserSettingsResponse _fetchResponse;
  TokenService _tokenService = new TokenService();
  User _currentUser = new User();


  bool _shareAttendance = false;
  bool _punchIn = true;
  bool _punchOut = true;
  bool _leaveApplied = true;
  bool _leaveRejected = true;
  bool _leaveApproved = true;
  bool _userLogin = true;
  bool _userLogout = true;
  double _dutyInReminder = 15;
  double _dutyOutReminder = 5;
  Language selectedLanguage;
  List<Language> languages = <Language> [new Language("မြန်မာ", SvgPicture.asset( 'assets/images/myanmar.svg', semanticsLabel: 'My',height: 32, width: 32, )),
    Language('English', SvgPicture.asset( 'assets/images/uk.svg', semanticsLabel: 'En',height: 32, width: 32, ))];


  @override
  void initState() {
    super.initState();
    _loadDefaultValues();
  }

  void _loadDefaultValues() async{
    _shareAttendance = await getSettingsAttendanceSharing( );
    _dutyInReminder = await getSettingsPunchInReminder();
    _dutyOutReminder = await getSettingsPunchOutReminder();
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });
    _sharedPrefs = await SharedPreferences.getInstance();


    await _currentUser.fromSharedPreferences();
    String _accessToken = await _tokenService.faceLogin(_currentUser.facialGUID, false);
    _fetchResponse = await _settingsService.fetchNotificationUserSettings(_accessToken, null);

    await _findByKey( NotificationUserSettingsItem.ATTENDANCE_IN ).then((element) =>
      _punchIn = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.ATTENDANCE_OUT ).then((element) =>
      _punchOut = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.LEAVE_APPLIED ).then((element) =>
      _leaveApplied = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.LEAVE_REJECTED ).then((element) =>
      _leaveRejected = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.LEAVE_APPROVED ).then((element) =>
      _leaveApproved = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.USER_LOGIN ).then((element) =>
      _userLogin = element.value.toLowerCase() == 'true'
    );
    await _findByKey( NotificationUserSettingsItem.USER_LOGOUT ).then((element) =>
      _userLogout = element.value.toLowerCase() == 'true'
    );

    String lang = await getLanguage();
      if( lang == 'en')
        selectedLanguage = languages[1];
      else
        selectedLanguage = languages[0];

    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  Future<NotificationUserSettingsItem> _findByKey( String key )async{
    NotificationUserSettingsItem settingsItem;
    _fetchResponse.settings.forEach((element) {
      if( element.key ==  key){
        settingsItem = element;
        return;
      }
    });

    return settingsItem;
  }

  Future<bool> _update(String key, bool value, BuildContext context) async{
    bool result = false;
    if( _processing ) return result;

    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    String _accessToken = await _tokenService.faceLogin(_currentUser.facialGUID, false);
    NotificationUserSettingsItem settingsItem = await _findByKey(key);
    settingsItem.value = value.toString();
    result = await _settingsService.updateNotificationUserSettings(settingsItem, _accessToken);

    if( !result ){
      showMessage('Failed to save! please try again later.', context);
    }

    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
//            backgroundColor: Colors.transparent,
            title: Text('Settings'),
            bottom: _createProgressIndicator(),
          ),
          body: SingleChildScrollView(
                      child: Column(
                          children: <Widget>[
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  elevation: 0,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Attendance sharing'),
                                              subtitle: Text('Allow other colleagues to attend over my device.'),
                                              trailing: Switch(
                                                value: _shareAttendance,
                                                onChanged:(value){
                                                  setState(() {
                                                    _shareAttendance=value;
                                                    setSettingsAttendanceSharing(_shareAttendance);
                                                  });
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        /*
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                            child: ListTile(
                                              title: SliderTheme(
                                                data: SliderTheme.of(context).copyWith(
                                                  activeTrackColor: Theme.of(context).primaryColor,
                                                  inactiveTrackColor: Colors.blueGrey[50],
                                                  trackShape: RoundedRectSliderTrackShape(),
                                                  trackHeight: 5.0,
                                                  thumbShape: RoundSliderThumbShape(enabledThumbRadius: 10.0),
                                                  thumbColor: Theme.of(context).primaryColor,
                                                  overlayColor: Colors.green[800].withAlpha(32),
                                                  overlayShape: RoundSliderOverlayShape(overlayRadius: 28.0),
                                                  tickMarkShape: RoundSliderTickMarkShape(),
                                                  activeTickMarkColor: Colors.white,
                                                  inactiveTickMarkColor: Colors.red[600],
                                                  valueIndicatorShape: PaddleSliderValueIndicatorShape(),
                                                  valueIndicatorColor: Theme.of(context).primaryColor,
                                                  valueIndicatorTextStyle: TextStyle(
                                                    color: Colors.white,
                                                  ),
                                                ),
                                                child: Slider(
                                                  value: _dutyInReminder,
                                                  min: 0,
                                                  max: 60,
                                                  divisions: 12,
                                                  label: '${_dutyInReminder.toInt()} minutes',
                                                  onChanged: (value) {
                                                    setState(() {
                                                      _dutyInReminder = value;
                                                      setSettingsPunchInReminder(_dutyInReminder);
                                                    });
                                                  },
                                                ),
                                              ),
                                              subtitle: Text('Punch-in notify before ${_dutyInReminder.toInt()} minutes'),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                            child: ListTile(
                                              title: SliderTheme(
                                                data: SliderTheme.of(context).copyWith(
                                                  activeTrackColor: Theme.of(context).primaryColor,
                                                  inactiveTrackColor: Colors.blueGrey[50],
                                                  trackShape: RoundedRectSliderTrackShape(),
                                                  trackHeight: 5.0,
                                                  thumbShape: RoundSliderThumbShape(enabledThumbRadius: 10.0),
                                                  thumbColor: Theme.of(context).primaryColor,
                                                  overlayColor: Colors.green[800].withAlpha(32),
                                                  overlayShape: RoundSliderOverlayShape(overlayRadius: 28.0),
                                                  tickMarkShape: RoundSliderTickMarkShape(),
                                                  activeTickMarkColor: Colors.white,
                                                  inactiveTickMarkColor: Colors.red[600],
                                                  valueIndicatorShape: PaddleSliderValueIndicatorShape(),
                                                  valueIndicatorColor: Theme.of(context).primaryColor,
                                                  valueIndicatorTextStyle: TextStyle(
                                                    color: Colors.white,
                                                  ),
                                                ),
                                                child: Slider(
                                                  value: _dutyOutReminder,
                                                  min: 0,
                                                  max: 60,
                                                  divisions: 12,
                                                  label: '${_dutyOutReminder.toInt()} minutes',
                                                  onChanged: (value) {
                                                    setState(() {
                                                      _dutyOutReminder = value;
                                                      setSettingsPunchOutReminder( _dutyOutReminder );
                                                    });
                                                  },
                                                ),
                                              ),
                                              subtitle: Text('Punch-out notify before ${_dutyOutReminder.toInt()} minutes'),
                                            )
                                        ),
                                        */
                                      ]
                                  ),
                                )
                            ),
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0),
                                child:Card(
                                  elevation: 0,
                                  child: Column(
                                      children: <Widget>[
                                        ListTile(
                                          title: Text('Language'),
                                          subtitle: _dropdownWidget(),
                                          leading: Icon(Icons.language),
                                        ),
                                      ]
                                  ),
                                )
                            ),
                            Container(
                                padding: const EdgeInsets.only(left: 20.0, top: 20.0, right: 20.0, bottom: 20.0),
                                child:Card(
                                  elevation: 0,
                                  child: Column(
                                      children: <Widget>[
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify punch-in'),
                                              subtitle: Text('Notification will be shown when your subordinates punch-in.'),
                                              trailing: Switch(
                                                value: _punchIn,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _punchIn = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.ATTENDANCE_IN, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify punch-out'),
                                              subtitle: Text('Notification will be shown when your subordinates punch-out.'),
                                              trailing: Switch(
                                                value: _punchOut,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _punchOut = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.ATTENDANCE_OUT, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify leave applied'),
                                              subtitle: Text('Notification will be shown when your subordinates applied the leave.'),
                                              trailing: Switch(
                                                value: _leaveApplied,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _leaveApplied = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.LEAVE_APPLIED, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify leave rejected'),
                                              subtitle: Text('Notification will be shown when your senior rejected the leave.'),
                                              trailing: Switch(
                                                value: _leaveRejected,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _leaveRejected = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.LEAVE_REJECTED, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify leave approved'),
                                              subtitle: Text('Notification will be shown when your leave application has been approved.'),
                                              trailing: Switch(
                                                value: _leaveApproved,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _leaveApproved = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.LEAVE_APPROVED, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify login'),
                                              subtitle: Text('Notification will be shown when your subordinates login.'),
                                              trailing: Switch(
                                                value: _userLogin,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _userLogin = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.USER_LOGIN, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        ),
                                        new Container(
                                            margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 20),
                                            child: ListTile(
                                              title: Text('Notify logout'),
                                              subtitle: Text('Notification will be shown when your subordinates logout.'),
                                              trailing: Switch(
                                                value: _userLogout,
                                                onChanged:_processing?null :(value) async{
                                                  setState(() {
                                                    _userLogout = value;
                                                  });
                                                  bool result = await _update( NotificationUserSettingsItem.USER_LOGOUT, value, context);
                                                },
                                                activeTrackColor: Colors.green[500],
                                                activeColor: Theme.of(context).primaryColor,
                                              ),
                                            )
                                        )
                                      ]
                                  ),
                                )
                            ),
                          ]
                      )
                  )

        )
    );
  }

  //https://www.developerlibs.com/2019/09/flutter-drop-down-menu-list-example.html
  Widget _dropdownWidget() {
    return new DropdownButtonHideUnderline(
      child: DropdownButton<Language>(
        //hint:  Text("Select item"),
        value: selectedLanguage,
        onChanged: (Language value){
          setState(() {
            selectedLanguage = value;
            if( value.name != 'English'){
              context.locale = Locale('my','');
              setLanguage( 'my' );
            }else{
              context.locale = Locale('en','');
              setLanguage( 'en' );
            }
          });
        },
        items: languages.map((Language lang) {
          return  DropdownMenuItem<Language>(
            value: lang,
            child: Row(
              children: <Widget>[
                lang.icon,
                SizedBox(width: 15,),
                Text(
                  lang.name,
//                  style:  TextStyle(color: Colors.white),
                ),
              ],
            ),
          );
        }).toList(),
      ),
    );
  }

  void showMessage(String message, BuildContext context) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }

}