

import 'package:dropdown_search/dropdown_search.dart';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';



class UserReportHome extends StatefulWidget {
  UserReportHome({Key key, this.currentUser, this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final SharedPreferences sharedPrefs;

  @override
  _UserReportHomeState createState() => _UserReportHomeState();
}

class _UserReportHomeState extends State<UserReportHome>{

  bool _isSearch = false;
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  DateTime _startDate;
  DateTime _endDate;
  final TextEditingController _controller_start = new TextEditingController();
  final TextEditingController _controller_end = new TextEditingController();
  bool showDate = false;
  List<Map<String, String>> _leaveStatus = [];
  Map<String, String> _selectedLeaveStatus = {'name':'All'};
  bool showLeaveStatus = false;
  bool _processing = false;
  Color _progressColor = Colors.transparent;
  String _accessToken;
  String _title = "Reporting";
  String _reportName = '';

  @override
  initState() {
    super.initState( );
    _leaveStatus = new List<Map<String, String>>();
    _leaveStatus.add({"name":"All"});
    _leaveStatus.add({"name":"Approved"});
    _leaveStatus.add({"name":"Pending"});
    _leaveStatus.add({"name":"Rejected"});
    _leaveStatus.add({"name":"Cancelled"});
    _loadData();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text(_title),
              bottom: _isSearch?_createProgressIndicator() : PreferredSize(preferredSize: Size(double.infinity, 1.0), child: SizedBox(height: 1.0,)),
            ),
            body: ListView(
              children: <Widget>[

                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                              tr(LocaleKeys.lblLeavesReports),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all leaves by date range and leave status.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblLeavesReports);
                              _reportName = 'Employee leaves report';
                              showLeaveStatus = true;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblAttendanceReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list attendance by date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblAttendanceReport);
                              _reportName = 'Employee attendance report';
                              showLeaveStatus = false;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblLateInReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list late-in from attendance by date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblLateInReport);
                              _reportName = 'Employee late-in report';
                              showLeaveStatus = false;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblEarlyOutReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list early-out from attendance by date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblEarlyOutReport);
                              _reportName = 'Employee early-out report';
                              showLeaveStatus = false;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblOvertimeReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list overtime calculations from attendance by date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblOvertimeReport);
                              _reportName = 'Overtime report';
                              showLeaveStatus = false;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblOvertimeSummaryReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list overtime calculation summary from attendance by date range.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblOvertimeSummaryReport);
                              _reportName = 'Overtime summary report';
                              showLeaveStatus = false;
                              showDate = true;
                            });
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.download_sharp, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblSalaryComponentsReport),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('This report list all of your salary components.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            setState(() {
                              _isSearch = true;
                              _title = tr(LocaleKeys.lblSalaryComponentsReport);
                              _reportName = 'Employee salary components report';
                              showLeaveStatus = false;
                              showDate = false;
                            });
                          },
                        )
                    )
                ),
              ],
            )
        )
    );
  }


  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, MediaQuery.of(context).size.height * 0.7),
      child: SizedBox(
          height: MediaQuery.of(context).size.height * 0.7,
          child: Card(
              margin: EdgeInsets.only(left: 0, right: 0, bottom: 0, top: 0),
              elevation: 1,
              child: Padding(
                  padding: EdgeInsets.all(10),
                  child: Column(
                      children: <Widget>[
                        _authorityResponse.authorities!=null && _authorityResponse.authorities.contains('SUPERVISOR_READ') && showLeaveStatus?_statusSelection(context) : new Container(),
                        showDate? Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblStartDate),
                                  ),
                                  controller: _controller_start,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose start date',
                              onPressed: (() {
                                _chooseDate(context, _controller_start.text, true);
                              }),
                            )
                          ]),
                        ) : Container(),
                        showDate? Container(
                          margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                          child:new Row(children: <Widget>[
                            new Expanded(
                                child: new TextFormField(
                                  decoration: new InputDecoration(
                                    icon: const Icon(Icons.calendar_today),
                                    hintText: 'yyyy-mm-dd',
                                    labelText: tr(LocaleKeys.lblEndDate),
                                  ),
                                  controller: _controller_end,
                                  keyboardType: TextInputType.datetime,
                                )),
                            new IconButton(
                              icon: new Icon(Icons.mode_edit),
                              tooltip: 'Choose end date',
                              onPressed: (() {
                                _chooseDate(context, _controller_end.text, false);
                              }),
                            )
                          ]),
                        ) : Container(),
                        Container(
                          margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              _buildButtons( Theme.of(context).primaryColor, Icons.download_sharp, tr(LocaleKeys.btnDownload) ),
                              _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnCancel)),
                            ],
                          ),
                        ),
                      ]
                  )
              )
          )
      )
  );

  Future _chooseDate(BuildContext context, String initialDateString, bool start) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(2020),
        lastDate: new DateTime(2100)
    );

    if (result == null) return;

    setState(() {
      if( start ){
        _controller_start.text = new DateFormat('yyyy-MM-dd').format(result);
        _startDate = result;
      }
      else{
        _controller_end.text = new DateFormat('yyyy-MM-dd').format(result);
        _endDate = result;
      }
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }


  _loadData() async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    await _authorityResponse.fromSharedPreferences();

    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( widget.currentUser.facialGUID, false );

    if( mounted)
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
  }

  Widget _statusSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<Map<String, String>>(
          selectedItem: _selectedLeaveStatus,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (Map<String, String> i, Map<String, String> s) => i['name']==s['name'],
          label: tr(LocaleKeys.lblLeaveType),
          onFind: (String filter) => _statusSuggestion(filter),
          onChanged: (Map<String, String> data) {
            _selectedLeaveStatus = data;
          },
          dropdownBuilder: (BuildContext context, Map<String, String> item ) =>
              Container(
                child: (item['name'] == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text(tr(LocaleKeys.lblAllDepartments)),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item['name']),
                ),
              ),
          popupItemBuilder: (BuildContext context, Map<String, String> item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item['name']),
                ),
              ),
        )
    );
  }
  Future<List<Map<String, String>>> _statusSuggestion(String query) async{
    List<Map<String, String>> matches = List();
    if( _leaveStatus == null)
      return matches;

    matches.addAll(_leaveStatus);

    matches.retainWhere((s) {
      Map<String, String> dept = s;
      if( dept['name'].toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed: () async{
            if( label == tr(LocaleKeys.btnCancel) ){
              reset();
            }else if( label==tr(LocaleKeys.btnDownload)){
              openBrowser(Constant.BASE_URL_PRIMARY
                  +'/v1/forwarder/reports/'+_accessToken
                  +'?n=' + Uri.encodeFull(_reportName)
                  +'&type=user'
                  + (_startDate!=null? '&sd=' +new DateFormat('yyyy-MM-dd').format(_startDate.toLocal()): '')
                  + (_endDate!=null? '&ed=' +new DateFormat('yyyy-MM-dd').format(_endDate.toLocal()): '')
                  +(_selectedLeaveStatus !=null? '&s=' +_selectedLeaveStatus['name'] : '')
              );
              reset();
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  void reset(){
    setState(() {
      _startDate = null;
      _controller_start.text = '';
      _endDate = null;
      _controller_end.text = '';
      _isSearch = !_isSearch;
      _selectedLeaveStatus = {'name':'All'};
      _reportName = '';
      _title = "Reporting";
    });
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<bool> _onWillPop() async {
    return true;
  }

}