

import '/hrm/model/response/department_search_response.dart';
import 'package:flutter/material.dart';



class EditDepartment extends StatefulWidget {
  EditDepartment({Key key, @required this.post, this.onDispose}) : super(key: key);
  final DepartmentSearchItem post;
  final Function onDispose;
  @override
  _EditDepartmentState createState() => _EditDepartmentState();
}

class _EditDepartmentState extends State<EditDepartment>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerPhone = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerDescription = new TextEditingController();
  bool _processing = false;

  @override
  initState() {
    super.initState( );
    _controllerName.text = widget.post.name ?? '';
    _controllerPhone.text = widget.post.phone ?? '';
    _controllerEmail.text = widget.post.email ?? '';
    _controllerAddress.text = widget.post.address ?? '';
    _controllerDescription.text = widget.post.description ?? '';
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
          appBar: AppBar(
            title: Text(widget.post.name ?? 'New department'),
          ),
          body: ListView(
            children: <Widget>[
              new Form(
                key: _formKey,
                autovalidate: true,
                child: Container(
                    padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                    child:Card(
                        color: Colors.grey[50],
                        elevation: 1,
                        child: Container(
                            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                            child:new Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: <Widget>[
                                  new TextFormField(
                                    decoration: InputDecoration(
                                      icon: const Icon(Icons.supervised_user_circle),
                                      hintText: '',
                                      labelText: 'Department name',
                                    ),
                                    controller: _controllerName,
                                    keyboardType: TextInputType.name,
                                    validator: (val) => val.isEmpty ? 'Name is required' : null,
                                    onSaved: (val) => widget.post.name = val.isEmpty? null : val,
                                  ),

                                  new TextFormField(
                                    decoration: InputDecoration(
                                      icon: const Icon(Icons.phone),
                                      hintText: '',
                                      labelText: 'Phone',
                                    ),
                                    controller: _controllerPhone,
                                    keyboardType: TextInputType.phone,
                                    onSaved: (val) => widget.post.phone = val.isEmpty? null : val,
                                  ),
                                  new TextFormField(
                                    decoration: InputDecoration(
                                      icon: const Icon(Icons.email),
                                      hintText: 'name@domain.com',
                                      labelText: 'Email address',
                                    ),
                                    controller: _controllerEmail,
                                    keyboardType: TextInputType.emailAddress,
                                    validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                                    onSaved: (val) => widget.post.email = val.isEmpty? null : val,
                                  ),
                                  new TextFormField(
                                    decoration: const InputDecoration(
                                      icon: const Icon(Icons.alternate_email),
                                      hintText: '',
                                      labelText: 'Address',
                                    ),
                                    keyboardType: TextInputType.multiline,
                                    controller: _controllerAddress,
                                    onSaved: (val) => widget.post.address = val.isEmpty? null : val,
                                  ),
                                  new TextFormField(
                                    decoration: const InputDecoration(
                                      icon: const Icon(Icons.alternate_email),
                                      hintText: '',
                                      labelText: 'Description',
                                    ),
                                    keyboardType: TextInputType.multiline,
                                    controller: _controllerDescription,
                                    onSaved: (val) => widget.post.description = val.isEmpty? null : val,
                                  ),
                                  Container(
                                    margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                                    child: Row(
                                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                      children: [
                                        _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                                        _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                                      ],
                                    ),
                                  )
                                ]
                            )
                        )
                    )
                )
              )
            ]
          )
        )
    );


  }

  Future<bool> _onWillPop() async {
    return true;
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.post, false);
              Navigator.pop(context);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.post, true);
                Navigator.pop(context);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

}