

import '/hrm/model/response/organization_detail_response.dart';
import 'package:flutter/material.dart';



class EditOrganizationDetail extends StatefulWidget {
  EditOrganizationDetail({Key key, @required this.detailResponse, this.onDispose}) : super(key: key);
  final OrganizationDetailResponse detailResponse;
  final Function onDispose;
  @override
  _EditOrganizationDetailState createState() => _EditOrganizationDetailState();
}

class _EditOrganizationDetailState extends State<EditOrganizationDetail>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  bool _processing = false;

  @override
  initState() {
    super.initState( );
    _controllerEmail.text = widget.detailResponse.email ?? '';
    _controllerAddress.text = widget.detailResponse.address ?? '';
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return new Form(
      key: _formKey,
      autovalidate: true,
      child: Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                new TextFormField(
                  decoration: InputDecoration(
                    icon: const Icon(Icons.email),
                    hintText: 'name@domain.com',
                    labelText: 'Email address',
                  ),
                  controller: _controllerEmail,
                  keyboardType: TextInputType.emailAddress,
                  validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                  onSaved: (val) => widget.detailResponse.email = val.isEmpty? null : val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'Address',
                  ),
                  keyboardType: TextInputType.multiline,
                  controller: _controllerAddress,
                  validator: (val) => val.isEmpty ? 'Address is required' : null,
                  onSaved: (val) => widget.detailResponse.address = val.isEmpty? null : val,
                ),
                Container(
                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                      _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                    ],
                  ),
                )
              ]
            )
          )
        )
      )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.detailResponse, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.detailResponse, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

}