

import '/hrm/model/entity/user.dart';
import '/hrm/model/response/leave_type_response.dart';
import '/hrm/service/leave_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'edit_leave_type.dart';



class LeaveTypeList extends StatefulWidget {
  LeaveTypeList({Key key, this.currentUser, this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final SharedPreferences sharedPrefs;


  @override
  _LeaveTypeListState createState() => _LeaveTypeListState();
}


class _LeaveTypeListState extends State<LeaveTypeList> {

  bool _waiting = false;
  Color _progressColor = Colors.transparent;
  LeaveService _leaveService;
  String _accessToken;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Leave types'),
              bottom: _createProgressIndicator(),
            ),
            body: _waiting? new Container() : DepartmentListFragment(
              onUpdate: (value){
                setState(() {
                  _waiting = value;
                });
              },
              onChange: (color){
                setState(() {
                  _progressColor = color;
                });
              },
              onCallBack: (service, token){
                _leaveService = service;
                _accessToken = token;
              },
              currentUser: widget.currentUser,
              sharedPrefs: widget.sharedPrefs,
            ),
            floatingActionButton:FloatingActionButton.extended(
              label: Text(
                'Add new',
                style: TextStyle(color: Colors.white),
              ),
              icon: Icon(Icons.add_circle_outline),
              backgroundColor: Theme.of(context).primaryColor,
              onPressed: (){
                _create();
              },
            )
        )
    );
  }

  void _create() async{
    LeaveTypeItem item = new LeaveTypeItem(organizationGuid: widget.currentUser.organizationGUID);
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (BuildContext context) => EditLeaveType(
          post: item,
          onDispose: ( item, action ) async{
            if( action ){
              setState(() {
                _waiting = true;
                _progressColor = Colors.white;
              });
              await _leaveService.leaveTypeCreate( item, _accessToken );
              setState(() {
                _waiting = false;
                _progressColor = Colors.transparent;
              });
            }
          },
        ),
      )
    );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor)
      )
  );

  Future<bool> _onWillPop() async {
    return true;
  }
}

class DepartmentListFragment extends StatefulWidget {
  DepartmentListFragment({Key key, @required this.onUpdate, this.onChange,
    this.onCallBack, this.currentUser, this.sharedPrefs}) : super(key: key);
  final Function onUpdate;
  final Function onChange;
  final Function onCallBack;
  final User currentUser;
  final SharedPreferences sharedPrefs;

  @override
  _DepartmentListFragmentState createState() => _DepartmentListFragmentState();
}

class _DepartmentListFragmentState extends State<DepartmentListFragment>{

  int _page = 0;
  RefreshController _refreshController = RefreshController(initialRefresh: true);
  String _accessToken;
  List<LeaveTypeItem> leaveTypes = [];
  LeaveService _leaveService = new LeaveService();
  LeaveTypeResponse _leaveTypeResponse;

  @override
  void initState() {
    super.initState();
  }

  Future<void> _getLoadData( bool refresh ) async{

    TokenService tokenService = new TokenService( );
    _accessToken = await tokenService.faceLogin( widget.currentUser.facialGUID, false );
    widget.onCallBack( _leaveService, _accessToken );


    _leaveTypeResponse = await _leaveService.leaveType(_accessToken);
    if( _leaveTypeResponse!=null && _leaveTypeResponse.leaveTypes !=null){

      if( leaveTypes.isNotEmpty)
        leaveTypes.removeRange(0, leaveTypes.length);

      _leaveTypeResponse.leaveTypes.forEach((element) {
        leaveTypes.add(element);
      });
    }

    if (mounted) setState(() {});
  }

  Widget buildContent(BuildContext context) {
    return ListView.separated(
      reverse: false,
      padding: EdgeInsets.only(left: 5, right: 5),
      itemBuilder: (c, i) => DepartmentItem(
        post: leaveTypes[i],
        accessToken: _accessToken,
        leaveService: _leaveService,
        sharedPrefs: widget.sharedPrefs,
        isEnd: i+1 == leaveTypes.length,
        onUpdate: (value){
          widget.onUpdate(value);
        },
        onChange: (color){
          widget.onChange(color);
        },
      ),
      separatorBuilder: (context, index) {
        return Container(
          height: 0.5,
          color: Theme.of(context).primaryColor,
        );
      },
      itemCount: leaveTypes.length,
    );
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return SmartRefresher(
      controller: _refreshController,
      enablePullUp: true,
      child: buildContent(context),

      physics: BouncingScrollPhysics(),
      header: WaterDropHeader(),
      footer: ClassicFooter(
        loadStyle: LoadStyle.ShowWhenLoading,
        completeDuration: Duration(milliseconds: 500),
      ),

      // scroll down
      onRefresh: () async {
        _page = 0;
        await _getLoadData( true);
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.refreshCompleted();

        /*
        if(failed){
         _refreshController.refreshFailed();
        }
      */
      },
      // scroll up
      onLoading: () async {
        ++_page;
        await _getLoadData( false );
//        await Future.delayed(Duration(milliseconds: 1000));
        if (mounted) setState(() {});
        _refreshController.loadComplete();
      },
    );
  }

}

class DepartmentItem extends StatefulWidget {

  DepartmentItem({Key key, @required this.post, @required this.accessToken,
    @required this.sharedPrefs, @required this.onUpdate, this.onChange,
    @required this.leaveService, @required this.isEnd}) : super(key: key);
  final LeaveTypeItem post;
  final String accessToken;
  final SharedPreferences sharedPrefs;
  final Function onUpdate;
  final Function onChange;
  final LeaveService leaveService;
  final bool isEnd;

  @override
  _DepartmentItemState createState() => _DepartmentItemState();

}

class _DepartmentItemState extends State<DepartmentItem> {
  bool _processing = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        child: Wrap(
          children: <Widget>[
            Card(
              elevation: 0,
              child: Container(
              //padding: EdgeInsets.only(top:10),
                child: ListTile(
                  leading: Icon(Icons.calendar_today),
                  title: Text(
                    '${widget.post.name}',
                    style: TextStyle(color: Theme.of(context).primaryColor),
                  ),
                  subtitle: Text("Days per year: ${widget.post.defaultPerYear??''}"),
                    trailing: widget.post.organizationGuid!=null && widget.post.organizationGuid.isNotEmpty ?IconButton(
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Theme.of(context).primaryColor,
                      onPressed:_processing?null : (){
                        _edit();
                      },
                    ) : Icon(Icons.do_not_disturb_on, color: Colors.black12,)
                )
              ),
            ),
            widget.isEnd? SizedBox(
              height: 200,
            ) : Container()
          ]
        )
    );
  }

  void _edit() async{
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => EditLeaveType(
            post: widget.post,
            onDispose: ( post, action) async{
              if( action ){
                _processing = true;
                widget.onUpdate(true);
                widget.onChange(Colors.white);

                await widget.leaveService.leaveTypeUpdate(post, widget.accessToken);

                _processing = false;
                widget.onChange(Colors.transparent);
                widget.onUpdate(false);

              }
            },
          ),
        )
    );
  }

}

Future<bool> _showConfirmDialog( BuildContext context, String message )async {
  bool result = false;
  await showDialog(
    context: context,
    builder: (BuildContext context) {
      // return object of type Dialog
      return AlertDialog(
        title: new Text("Action confirm!"),
        content: new Text(message),
        actions: <Widget>[
          // usually buttons at the bottom of the dialog
          new FlatButton(
            child: new Text("Yes"),
            onPressed: () { result = true;Navigator.of(context).pop();},
          ),
          new FlatButton(
            child: new Text("No"),
            onPressed: () { result = false; Navigator.of(context).pop();},
          ),
        ],
      );
    },
  );
  return result;
}