

import 'dart:io';
import '/hrm/config/constant.dart';
import '/hrm/imageprocessing/base64image.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/service/organization_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:path_provider/path_provider.dart';
import 'package:image/image.dart' as imglib;

import 'edit_organization_detail.dart';



class OrganizationDetail extends StatefulWidget {
  OrganizationDetail({Key key, @required this.currentUser,
    @required this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final SharedPreferences sharedPrefs;


  @override
  _OrganizationDetailState createState() => _OrganizationDetailState();
}


class _OrganizationDetailState extends State<OrganizationDetail>{

  TokenService tokenService = new TokenService( );
  OrganizationService organizationService = new OrganizationService();
  OrganizationDetailResponse _detailResponse;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  Color _progressColor = Colors.transparent;
  bool _processing = false;
  bool _editOrganization = false;
  String _accessToken;
  String _profileImage;

  @override
  initState() {
    super.initState( );
    _loadData( );
  }

  void _loadData( ) async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });
    _accessToken = await tokenService.faceLogin( widget.currentUser.facialGUID, false );
    _detailResponse = await organizationService.detail( _accessToken, widget.currentUser.organizationGUID );
    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            key: _scaffoldKey,
            resizeToAvoidBottomInset: true,
            // resizeToAvoidBottomPadding: true,
            appBar: AppBar(
              title: Text('Organization'),
              bottom: _createProgressIndicator(),
            ),
            body: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          _editOrganization? _organizationEditPanel(context) : _organizationPanel(context),
                        ]
                    )
                )
        )
    );
  }

  Widget _organizationPanel( BuildContext context ){
    return _detailResponse!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:_processing?null : () async{
                        setState(() {
                          _editOrganization = true;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                Container(
                  padding: const EdgeInsets.only(top:0.0),
                  child: ClipOval(
                      child: Material(
                        elevation: 2,
                        color: Colors.white, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 150,
                            height: 150,
                            child: _detailResponse !=null?
                            ( _profileImage==null? new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/organization/download_logo/${widget.currentUser.organizationGUID}',
                              headers: {'Authorization':'Bearer ${_accessToken}'},)
                            : decode( _profileImage ) )
                                : Icon(Icons.group_work, size: 120,),
                          ),
                        ),
                      )
                  ),
                ),
                Container(
                  margin: const EdgeInsets.only(top: 0.0, bottom: 5.0),
                  child: Wrap(
                    spacing: 20,
                    runSpacing: 20,
                    //mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_camera, 'Take photo'),
                      _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_library, 'Browse'),
                    ],
                  ),
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('${_detailResponse.name}'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Phone: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_detailResponse.phone??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Email: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_detailResponse.email??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Address: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_detailResponse.address??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Registered on: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${new DateFormat('dd/MM/yyyy').format(_detailResponse.createdOn)??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Status: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_detailResponse.status??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Info: '),
                                        width: MediaQuery.of(context).size.width*0.25,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_detailResponse.info??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _organizationEditPanel(BuildContext context){
    return _detailResponse!= null ? EditOrganizationDetail(
      detailResponse: _detailResponse,
      onDispose: (organization, isSave) async{
        setState(() {
          _editOrganization = false;
        });
        if( isSave ){
          await _update();
        }
      },
    ) : new Container();
  }

  Column _buildButtonColumn(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        ClipOval(
            child: Material(
              color: color, // button color
              child: InkWell(
                splashColor: Colors.red, // inkwell color
                child: SizedBox(width: 36,
                    height: 36,
                    child: Icon(icon, color: Colors.white)
                ),
                onTap:_processing?null : () async{
                  if( label == 'Take photo'){
                    await _getImage(true);
                  }else if( label == 'Browse'){
                    await _getImage(false);
                  }
                },
              ),
            )
        ),
        Container(
          margin: const EdgeInsets.only(top: 5),
          child: Text(
            label,
            style: TextStyle(
                fontSize: 14,
                fontWeight: FontWeight.w400,
                color: color
            ),
          ),
        ),
      ],
    );
  }

  Future<void> _getImage( bool isCamera) async {
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/Pictures/flutter_test';
        await Directory(tmpDir).create(recursive: true);
        await _addImage(tmpDir, image.path);
      }
    }catch(e){showMessage('Error occur, please select another photo');print(e);}

    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  Future<void> _addImage(String tmpDir, String path) async{
    String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
    File rotatedFile = await FlutterImageCompress.compressAndGetFile(
      path, photoPath2,
      quality: 90,
      rotate: 0,
    );
    imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 512);
    String cropFilePath = '$tmpDir/cropped.jpg';
    File(cropFilePath)
      ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 90));

    File smallFile  =File(cropFilePath);
    String encodedString = encode( smallFile );
    smallFile.deleteSync(recursive: false);
    rotatedFile.deleteSync(recursive: false);
    setState(() {_profileImage = encodedString;});

    await organizationService.profileUpdate(
        _profileImage,
        widget.currentUser.organizationGUID,
        _accessToken
    );

  }


  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  void deactivate() {
    super.deactivate( );
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  Future<void> _update() async{
    if( _processing) return null;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    bool result = await organizationService.update(_detailResponse, widget.currentUser.organizationGUID, _accessToken);

    if( result ){
      showMessage('Update successful.');
    }else{
      showMessage('Update failed!');
      _detailResponse = await organizationService.detail( _accessToken, widget.currentUser.organizationGUID );
    }

    setState((){
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }
}