

import 'package:easy_localization/easy_localization.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/view/more/adminreport/admin_report_home.dart';
import '/hrm/view/organization/organization_detail.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'department_list.dart';
import 'leave_type_list.dart';
import '/hrm/model/response/organization_detail_response.dart';
import '/hrm/config/constant.dart';



class OrganizationHome extends StatefulWidget {
  OrganizationHome({Key key, this.currentUser, this.sharedPrefs}) : super(key: key);
  final User currentUser;
  final SharedPreferences sharedPrefs;

  @override
  _OrganizationHomeState createState() => _OrganizationHomeState();
}

class _OrganizationHomeState extends State<OrganizationHome>{

  OrganizationDetailResponse detailResponse = new OrganizationDetailResponse();

  @override
  initState() {
    super.initState( );
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
            appBar: AppBar(
              title: Text('Administration'),
            ),
            body: ListView(
              children: <Widget>[
                Container(
                    margin: EdgeInsets.all(16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(
                              Icons.group_work,
                              size: 48,
                              color: Theme.of(context).primaryColor
                          ),
                          title: Text(
                            tr(LocaleKeys.lblOrgProfile),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('Manage your organization information.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            _gotoOrganization();
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(
                              Icons.supervised_user_circle,
                              size: 48,
                              color: Theme.of(context).primaryColor
                          ),
                          title: Text(
                            tr(LocaleKeys.lblDepartments),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('Manage your organization department.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            _gotoDepartment();
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.settings, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblLeaveSettings),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('Setup leave type, etc.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            _gotoLeaveType();
                          },
                        )
                    )
                ),
                Container(
                    margin: EdgeInsets.only(left: 16, right: 16, bottom: 16),
                    child: Card(
                        elevation: 1,
                        child: ListTile(
                          leading: Icon(Icons.assignment, size: 48, color: Theme.of(context).primaryColor,),
                          title: Text(
                            tr(LocaleKeys.lblAdminReports),
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                          subtitle: Text('Download the reports such attendance, leave, overtime, payroll, etc.'),
                          trailing: Icon(Icons.keyboard_arrow_right),
                          onTap: () {
                            _gotoReports();
                          },
                        )
                    )
                ),
              ],
            )
        )
    );
  }

  void _gotoOrganization(){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => OrganizationDetail(
            currentUser: widget.currentUser,
            sharedPrefs: widget.sharedPrefs,
          ),
        )
    );
  }

  void _gotoDepartment(){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => DepartmentList(
            currentUser: widget.currentUser,
            sharedPrefs: widget.sharedPrefs,
          ),
        )
    );
  }

  void _gotoLeaveType(){
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => LeaveTypeList(
            currentUser: widget.currentUser,
            sharedPrefs: widget.sharedPrefs,
          ),
        )
    );
  }

  void _gotoReports() async{

    if( detailResponse.plan!=null && detailResponse.plan == Constant.PLAN_BASIC){
      await _showUpgradeDialog(context);
      return;
    }
    Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => AdminReportHome(
            currentUser: widget.currentUser,
            sharedPrefs: widget.sharedPrefs,
          ),
        )
    );
  }

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    Scaffold.of(context).showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
  Future<bool> _showUpgradeDialog( BuildContext context )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text("Upgrade required!"),
          content: new Text("Thank you for using QuickQ HR, this feature is required to upgrade your plan. Please contact us: " + Constant.CONTACT_PHONE),
          actions: <Widget>[
            new FlatButton(
              child: new Text("OK"),
              onPressed: () { Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }
  Future<bool> _onWillPop() async {
    return true;
  }

}