import 'dart:io';
import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/generated/locale_keys.g.dart';
import '/hrm/imageprocessing/base64image.dart';
import '/hrm/model/request/face_request.dart';
import '/hrm/service/face_detector_service.dart';
import '/hrm/service/face_recognization_service.dart';
import '/hrm/service/organization_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:image_picker/image_picker.dart';
import 'package:image/image.dart' as imglib;
import 'package:path_provider/path_provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:threading/threading.dart';
import 'activate_organization.dart';


class OrganizationRegister extends StatefulWidget {

  OrganizationRegister({Key key, @required this.title}) : super(key: key);
  final String title;
  @override
  _OrganizationRegisterState createState() => _OrganizationRegisterState();

}

class _OrganizationRegisterState extends State<OrganizationRegister> {

  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool _processing = false;
  Color progressColor = Colors.transparent;

  final TextEditingController _controllerConfirmPassword = new TextEditingController();
  final TextEditingController _controllerPassword = new TextEditingController();
  final TextEditingController _controllerUsername = new TextEditingController();
  final TextEditingController _controllerAddress = new TextEditingController();
  final TextEditingController _controllerOrganization = new TextEditingController();
  final TextEditingController _controllerName = new TextEditingController();
  final TextEditingController _controllerEmail = new TextEditingController();
  final TextEditingController _controllerPhone = new TextEditingController();

  int _selectedPhoto = 0;
  List<String> _photoLabel = ["Your profile image","Face photo 1","Face photo 2","Face photo 3"];
  List<String> _images = ['no','no','no','no'];
  String _registeredFaceGUID=null;
  String _registeredUserGUID=null;
  String _registeredOrganizationGUID=null;
  TokenService tokenService = new TokenService();
  String _accessToken;

  UserService _userService = new UserService();
  SharedPreferences _sharedPrefs;

  FaceDetectorService faceDetectorService = new FaceDetectorService();
  @override
  void initState() {
    super.initState();
    _getAccessToken( true );
    faceDetectorService.init();
  }

  void _getAccessToken( bool isFirstLoad) async{
    if( isFirstLoad) {
      _sharedPrefs = await SharedPreferences.getInstance();
    }
    _accessToken = await tokenService.getDeviceToken(true);
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
        onWillPop: _onWillPop,
        child: Scaffold(
            key: _scaffoldKey,
            resizeToAvoidBottomInset: true,
            // resizeToAvoidBottomPadding: true,
            appBar: AppBar(
              title: Text(widget.title),
              bottom: _createProgressIndicator(),
            ),
            body: Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          Container(
                              padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                              child:Card(
                                color: Colors.grey[50],
                                elevation: 1,
                                child: Column(
                                    children: <Widget>[
                                      new Container(
                                        margin: const EdgeInsets.only(left: 16.0, right: 16.0, bottom: 0),
                                        child: formData(context),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 16.0, right: 0.0, top: 30),
                                        child:ListTile(
                                          title: Text(
                                              'Face login information',
                                            style: TextStyle(color: Theme.of(context).primaryColor),
                                          ),
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 5.0, right: 5.0),
                                        child: _images[_selectedPhoto] == 'no'
                                            ? Icon(Icons.account_circle, color: Colors.grey[100], size: 150,)
                                            : Container(
                                          padding: const EdgeInsets.all(5.0),
                                          child: ClipOval(
                                              child: Material(
                                                elevation: 8,
                                                color: Colors.white, // button color
                                                child: InkWell(
                                                  splashColor: Colors.red, // inkwell color
                                                  child: SizedBox(width: 150,
                                                      height: 150,
                                                      child: decode( _images[_selectedPhoto] )
                                                  ),
                                                ),
                                              )
                                          ),
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 0.0, right: 0.0),
                                        child:Text(
                                          _photoLabel[_selectedPhoto],
                                          style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                                        ),
                                      ),
                                      Container(
                                          margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 10),
                                          child: new Wrap(
                                              spacing: 10,
                                              runSpacing: 10,
                                              children: <Widget>[
                                                _images[0] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode(_images[0]),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[1] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[1] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[2] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[2] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                                _images[3] == 'no'
                                                    ? Icon(Icons.account_circle, color: Colors.grey[100], size: 50,)
                                                    : Container(
                                                  child: ClipOval(
                                                      child: Material(
                                                        elevation: 1,
                                                        color: Colors.white, // button color
                                                        child: InkWell(
                                                          splashColor: Colors.red, // inkwell color
                                                          child: SizedBox(
                                                            width: 50,
                                                            height: 50,
                                                            child: decode( _images[3] ),
                                                          ),
                                                        ),
                                                      )
                                                  ),
                                                ),
                                              ]
                                          )
                                      ),
                                      _photoSelection(),
                                      Container(
                                        margin: const EdgeInsets.only(top: 0.0, bottom: 5.0),
                                        child: Wrap(
                                          spacing: 20,
                                          runSpacing: 20,
                                          //mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                          children: [
                                            _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_camera, tr(LocaleKeys.btnTakePhoto)),
                                            _buildButtonColumn( Theme.of(context).primaryColor, Icons.photo_library, tr(LocaleKeys.btnBrowse)),
                                            //_images[_selectedPhoto] == 'no'
                                            //? Container() :
                                            _buildButtonColumn(_images[_selectedPhoto] == 'no' ? Colors.grey[100] : Theme.of(context).primaryColor, Icons.delete_forever, tr(LocaleKeys.btnRemove)),
                                          ],
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(left: 20.0, right: 5.0),
                                        child:Text(
                                          'Photo must be included clear face, it should be taken with camera directly. It should NOT be scanned from paper or any media.',
                                          style: new TextStyle(fontSize: 12.0, color: Colors.blueGrey),
                                        ),
                                      ),
                                      Container(
                                        margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                                        child: Row(
                                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                          children: [
                                            _buildButtons( Theme.of(context).primaryColor, Icons.done, tr(LocaleKeys.btnRegisterAccount) ),
                                            _buildButtons( Colors.transparent, Icons.cancel, tr(LocaleKeys.btnLater)),
                                          ],
                                        ),
                                      )
                                      //mainBodySecondWidgets(context),
                                    ]
                                ),
                              )
                          ),
                        ]
                    )
                )
            )
        )
    );
  }

  Widget _photoSelection( ){
    return Container(
      margin: const EdgeInsets.only(top: 0.0, left: 0.0),
      child:new Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          new Text(
            'Photo',
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 0,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value; });
            },
          ),
          new Text(
            'Profile',
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 1,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '1',
            style: new TextStyle(
              fontSize: 16.0,
            ),
          ),new Radio(
            value: 2,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '2',
            style: new TextStyle(fontSize: 16.0),
          ),
          new Radio(
            value: 3,
            groupValue: _selectedPhoto,
            onChanged: ( value){
              setState(() { _selectedPhoto = value;});
            },
          ),
          new Text(
            '3',
            style: new TextStyle(
              fontSize: 16.0,
            ),
          ),
        ],
      ),
    );
  }

  Widget topWidgets( BuildContext context) {
    return Container(
        color: Colors.white,
        padding: const EdgeInsets.only(left: 5.0, top: 0.0),
        height: 80,

        child: ListView(
          scrollDirection: Axis.horizontal,
          children: <Widget>[
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                color: Colors.grey[50],
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '1',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '2',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),
            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '3',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),

            Container(
              width: 90.0,
              child: Card(
                elevation: 1,
                child: new Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    const ListTile(
                      //leading: Icon(Icons.album),
                        title: Text(
                          'photo',
                        ),
                        subtitle: Text(
                          '4',
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontSize: 24,
                            fontWeight: FontWeight.bold,
                            color: Colors.teal,
                          ),
                        )
                    ),
                  ],
                ),
              ),
            ),

          ],
        )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel' || label == 'Close' || label == tr(LocaleKeys.btnLater) ){
              Navigator.pop( context );
            }else{
              _submitForm( );
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );

  }

  Column _buildButtonColumn(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        ClipOval(
            child: Material(
              color: color, // button color
              child: InkWell(
                splashColor: Colors.red, // inkwell color
                child: SizedBox(width: 36,
                    height: 36,
                    child: Icon(icon, color: Colors.white)
                ),
                onTap: () async{
                  if( label == tr(LocaleKeys.btnTakePhoto)){
                    await _getImage(true);
                  }else if( label == tr(LocaleKeys.btnBrowse)){
                    await _getImage(false);
                  }else if( label == tr(LocaleKeys.btnRemove)){
                    setState(() {
                      _images[_selectedPhoto] = 'no';
                    });
                  }
                },
              ),
            )
        ),
        Container(
          margin: const EdgeInsets.only(top: 5),
          child: Text(
            label,
            style: TextStyle(
                fontSize: 14,
                fontWeight: FontWeight.w400,
                color: color
            ),
          ),
        ),
      ],
    );
  }

  Form formData( BuildContext context ){

    return new Form(
        key: _formKey,
        autovalidateMode: AutovalidateMode.onUserInteraction,
        child: new Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.person),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblYourName),
                ),
                controller: _controllerName,
                validator: (val) => val.isEmpty ? 'Name is required' : null,
              ),
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.group_work),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblOrganizationName),
                ),
                controller: _controllerOrganization,
                validator: (val) => val.isEmpty ? 'Organization name is required' : null,
              ),
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.phone),
                  hintText: '09 xxx xxx xxx',
                  labelText: tr(LocaleKeys.lblMobilePhone),
                ),
                controller: _controllerPhone,
                keyboardType: TextInputType.phone,
                validator: (val) => isValidPhoneNumber(val) ? null : 'Please enter a valid phone number',
              ),
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.email),
                  hintText: 'name@domain.com',
                  labelText: tr(LocaleKeys.lblEmailAddress),
                ),
                controller: _controllerEmail,
                keyboardType: TextInputType.emailAddress,
                validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
              ),
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.alternate_email),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblAddress),
                ),
                keyboardType: TextInputType.multiline,
                controller: _controllerAddress,
                validator: (val) => val.isEmpty ? 'Address is required' : null,
              ),

              Container(
                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 30),
                child:ListTile(
                  title: Text(
                      'Login information',
                      style: TextStyle(color: Theme.of(context).primaryColor),
                  ),
                ),
              ),
              new TextFormField(
                decoration: InputDecoration(
                  icon: const Icon(Icons.account_circle),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblUsername),
                ),
                keyboardType: TextInputType.multiline,
                controller: _controllerUsername,
                validator: (val) => val.isEmpty ? 'Username is required' : (val.contains(" ")? 'Character space not allow!' : (val.length<8? 'Minimum length is 8' : null)),
              ),
              new TextFormField(
                obscureText: true,
                decoration: InputDecoration(
                  icon: const Icon(Icons.star),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblPassword),
                ),
                controller: _controllerPassword,
                validator: (val) => val.isEmpty ? 'Password required' : (val.length<6? 'Minimum password length is 6' : null),
              ),
              new TextFormField(
                obscureText: true,
                decoration: InputDecoration(
                  icon: const Icon(Icons.star),
                  hintText: '',
                  labelText: tr(LocaleKeys.lblConfirmPassword),
                ),
                controller: _controllerConfirmPassword,
                validator: (val) => val.isEmpty ? 'Password required' : (val!= _controllerPassword.text ? 'Password does not match!' : null),
              ),
            ]
        )
    );
  }

  void _submitForm() {

    final FormState form = _formKey.currentState;

    if (!form.validate()) {
      showMessage('Fill the required fields!');
    } else if (_controllerUsername.text.trim().toLowerCase() == 'administrator'
        || _controllerUsername.text.trim().toLowerCase() == 'superadmin') {
      showMessage( '${_controllerUsername.text.trim()} is not available. Please try with another username.');
    }else {
      form.save();

      if( _controllerPassword.text != _controllerConfirmPassword.text){
        showMessage('Password does not match!');
        return;
      }

      if( _images[0]=='no') {
        showMessage('Profile image is required!');
        setState(() {
          _selectedPhoto =0;
        });
        return;
      }
      if( _images[1]=='no') {
        showMessage('At least 1 face photo is required!');
        setState(() {
          _selectedPhoto =1;
        });
        return;
      }
      _checkUsername();
    }
  }

  Future<void> _checkUsername()async{

    if( _processing ) return;
    setState(() {
      _processing = true;
      progressColor = Colors.white;
    });

    bool available = await _userService.isAvailable(_controllerUsername.text.trim());
    if( available ){
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
      });
      _doRegister();
    }else{
      showMessage( '${_controllerUsername.text.trim()} is already used. Please try with another username.');
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
      });
    }
  }

  Future<void> _doRegister() async{
    if( _processing ) return;
    setState(() {
      _processing = true;
      progressColor = Colors.white;
    });

    _getAccessToken( false );

    if( _registeredOrganizationGUID == null ){
      OrganizationService organizationService = new OrganizationService();
      _registeredOrganizationGUID = await organizationService.register(
          _controllerOrganization.text.trim(),
          Constant.DEFAULT_ORGANIZATION_PLAN_PRO,
          _controllerPhone.text.trim(),
          _controllerEmail.text.isEmpty?null : _controllerEmail.text,
          _controllerAddress.text.trim(),
          _controllerName.text.trim(),
          null);
    }

    if( _registeredFaceGUID == null && _registeredOrganizationGUID!=null){
      _getAccessToken( false );
      FaceRecognizationService recognizationService = new FaceRecognizationService();
      _registeredFaceGUID = await recognizationService.faceRegister(
          _images,
          _controllerName.text,
          _registeredOrganizationGUID,
          _accessToken,
          FaceRequest.TYPE_REGISTER
      );
    }

    if( _registeredUserGUID == null && _registeredFaceGUID!=null){
      _registeredUserGUID = await _userService.userRegister(
          _registeredFaceGUID,
          _controllerName.text.trim(),
          _controllerEmail.text.isEmpty?null : _controllerEmail.text,
          _controllerPhone.text.trim(),
          Constant.DEFAULT_ORGANIZATION_ROLE_GUID,
          _controllerUsername.text.trim(),
          _controllerConfirmPassword.text,
          _accessToken
      );
    }

    if( _registeredOrganizationGUID == null || _registeredFaceGUID == null || _registeredUserGUID==null){
      showMessage('Business account registration failed, please contact ${Constant.CONTACT_PHONE}');
      setState(() {
        _processing = false;
        progressColor = Colors.transparent;
      });
    }else{
      showMessage('Registration successful, please validate your account.');
      new ThreadTimer(new Duration(milliseconds: 2000), () {
        Navigator.pushReplacement(
            context,
            MaterialPageRoute(
              builder: (BuildContext context) => ActivateOrganization(
                organizationGuid: _registeredOrganizationGUID,
                gotoLogin: false,
              ),
            )
        );
      });
    }
  }

  Future<void> _getImage( bool isCamera) async {
    _formKey.currentState.save();
    setState(() {progressColor = Colors.white;});

    try{
      final picker = ImagePicker();
      final image = await picker.getImage(source: isCamera? ImageSource.camera : ImageSource.gallery);
      if( image != null ) {
        final Directory extDir = await getApplicationDocumentsDirectory();
        final String tmpDir = '${extDir.path}/Pictures/flutter_test';
        await Directory(tmpDir).create(recursive: true);

        if( _selectedPhoto == 0 ){
          await _addImage(tmpDir, image.path);
        }else{
          // detecting face
          DetectionResult detectionResult = await faceDetectorService.detectedFaceAsBase64String(tmpDir, image.path, mounted, false, false);
          if( detectionResult == null )
            showMessage('No face detected on image');
          else{
            setState(() {_images[_selectedPhoto] = detectionResult.detectedFace;});
          }
        }
      }
    }catch(e){showMessage('Error occur, please select another photo');print(e);}

    setState(() {progressColor = Colors.transparent;});
  }

  Future<void> _addImage(String tmpDir, String path) async{
    String photoPath2 = '$tmpDir/${DateTime.now()}-2.jpg';
    File rotatedFile = await FlutterImageCompress.compressAndGetFile(
      path, photoPath2,
      quality: 90,
      rotate: 0,
    );
    imglib.Image thumbnail = imglib.copyResize( imglib.decodeImage( rotatedFile.readAsBytesSync()), width: 192);
//      if( isCamera)
//        thumbnail = imglib.flipHorizontal( thumbnail );
    String cropFilePath = '$tmpDir/cropped.jpg';
    File(cropFilePath)
      ..writeAsBytesSync(imglib.encodeJpg(thumbnail, quality: 90));

    File smallFile  =File(cropFilePath);
    String encodedString = encode( smallFile );
    smallFile.deleteSync(recursive: false);
    rotatedFile.deleteSync(recursive: false);
    setState(() {_images[_selectedPhoto] = encodedString;});
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: progressColor,)
      )
  );

  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }

  bool isValidPhoneNumber(String input) {
    if( input== null || input.isEmpty) return false;
    final RegExp regex = new RegExp(r'^[0-9]{9,13}$');//new RegExp(r'(?:[0-9]\d{10,13})$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

  Future<bool> _onWillPop() async {
    return true;
  }

  @override
  void dispose() {
    try {
      faceDetectorService.dispose();
    }catch(e){}
    super.dispose();
  }
}