
import '/hrm/model/entity/employee.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';



class EditAbout extends StatefulWidget {
  EditAbout({Key key, @required this.employee, this.onDispose}) : super(key: key);
  final Employee employee;
  final Function onDispose;
  @override
  _EditAboutState createState() => _EditAboutState();
}

class _EditAboutState extends State<EditAbout>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_firstName = new TextEditingController();
  final TextEditingController _controller_lastName = new TextEditingController();
  final TextEditingController _controller_dob = new TextEditingController();
  final TextEditingController _controller_nationality = new TextEditingController();
  final TextEditingController _controller_maritalStatus = new TextEditingController();
  final TextEditingController _controller_nrc = new TextEditingController();
  String _gender;
  bool _processing = false;

  @override
  initState() {
    super.initState( );
    _controller_firstName.text = widget.employee.firstName;
    _controller_lastName.text = widget.employee.lastName ?? '';
    _controller_nationality.text = widget.employee.nationality ?? '';
    _controller_maritalStatus.text = widget.employee.maritalStatus ?? '';
    _controller_nrc.text = widget.employee.nicNum ?? '';
    _gender =  widget.employee.gender ?? '';
    _controller_dob.text = widget.employee.birthday!=null?new DateFormat('yyyy-MM-dd').format(widget.employee.birthday) : '';
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return new Form(
      key: _formKey,
      autovalidate: true,
      child: Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.person),
                    hintText: '',
                    labelText: 'First name',
                  ),
                  controller: _controller_firstName,
                  validator: (val) => val.isEmpty ? 'First name is required' : null,
                  onSaved: (val) => widget.employee.firstName = val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.person),
                    hintText: '',
                    labelText: 'Last name',
                  ),
                  controller: _controller_lastName,
                  onSaved: (val) => widget.employee.lastName = val.isEmpty? null : val,
                ),
                Container(
                  margin: const EdgeInsets.only(top: 5.0, left: 5.0, right: 5.0),
                  child:new Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: <Widget>[
                      new Text(
                        'Gender',
                        style: new TextStyle( fontWeight: FontWeight.bold,),
                      ),
                      new Radio(
                        value: 'Male',
                        groupValue: _gender,
                        onChanged: ( value){
                          setState(() { _gender = value; });
                        },
                      ),
                      new Text(
                        'Male',
//                      style: new TextStyle(fontSize: 16.0),
                      ),
                      new Radio(
                        value: 'Female',
                        groupValue: _gender,
                        onChanged: ( value){
                          setState(() { _gender = value; });
                        },
                      ),
                      new Text(
                        'Female',
//                      style: new TextStyle(fontSize: 16.0,),
                      ),
                    ],
                  ),
                ),
                Container(
                  margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                  child:new Row(children: <Widget>[
                    new Expanded(
                        child: new TextFormField(
                          decoration: new InputDecoration(
                            icon: const Icon(Icons.calendar_today),
                            hintText: 'yyyy-mm-dd',
                            labelText: 'Date of birth',
                          ),
                          controller: _controller_dob,
                          keyboardType: TextInputType.datetime,
                        )),
                    new IconButton(
                      icon: new Icon(Icons.mode_edit),
                      tooltip: 'Choose date of birth',
                      onPressed: (() {
                        _chooseDate(context, _controller_dob.text );
                      }),
                    )
                  ]),
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.ac_unit),
                    hintText: '',
                    labelText: 'Nationality',
                  ),
                  controller: _controller_nationality,
                  onSaved: (val) => widget.employee.nationality = val.isEmpty?null:val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.all_inclusive),
                    hintText: '',
                    labelText: 'Marital status',
                  ),
                  controller: _controller_maritalStatus,
                  onSaved: (val) => widget.employee.maritalStatus = val.isEmpty?null:val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.credit_card),
                    hintText: '',
                    labelText: 'NRC',
                  ),
                  controller: _controller_nrc,
                  onSaved: (val) => widget.employee.nicNum = val.isEmpty?null:val,
                ),
                Container(
                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                      _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                    ],
                  ),
                )
              ]
            )
          )
        )
      )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.employee, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.employee.gender = _gender;
                widget.onDispose(widget.employee, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString ) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1900 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(1900),
        lastDate: new DateTime.now()
    );

    if (result == null) return;

    setState(() {
      _controller_dob.text = new DateFormat('yyyy-MM-dd').format(result);
      widget.employee.birthday = result;
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

}