
import '/hrm/model/entity/employee.dart';
import 'package:flutter/material.dart';



class EditAddress extends StatefulWidget {
  EditAddress({Key key, @required this.employee, this.onDispose}) : super(key: key);
  final Employee employee;
  final Function onDispose;
  @override
  _EditAddressState createState() => _EditAddressState( );
}

class _EditAddressState extends State<EditAddress>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_address1 = new TextEditingController();
  final TextEditingController _controller_address2 = new TextEditingController();
  final TextEditingController _controller_city = new TextEditingController();
  final TextEditingController _controller_state = new TextEditingController();
  final TextEditingController _controller_country = new TextEditingController();
  final TextEditingController _controller_postalCode = new TextEditingController();
  bool _processing = false;

  @override
  initState() {
    super.initState( );
    _controller_address1.text = widget.employee.address1 ?? '';
    _controller_address2.text = widget.employee.address2 ?? '';
    _controller_city.text = widget.employee.city ?? '';
    _controller_state.text = widget.employee.province ?? '';
    _controller_country.text = widget.employee.country ?? '';
    _controller_postalCode.text = widget.employee.postalCode ?? '';
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new Form(
      key: _formKey,
      autovalidate: true,
      child: Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'Address 1',
                  ),
                  keyboardType: TextInputType.multiline,
                  maxLines: null,
                  controller: _controller_address1,
                  onSaved: (val) => widget.employee.address1 = val.isEmpty? null : val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'Address 2',
                  ),
                  keyboardType: TextInputType.multiline,
                  maxLines: null,
                  controller: _controller_address2,
                  onSaved: (val) => widget.employee.address2 = val.isEmpty? null : val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'City',
                  ),
                  keyboardType: TextInputType.text,
                  controller: _controller_city,
                  onSaved: (val) => widget.employee.city = val.isEmpty?null:val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'State/Division',
                  ),
                  keyboardType: TextInputType.text,
                  controller: _controller_state,
                  onSaved: (val) => widget.employee.province = val.isEmpty?null:val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'Country',
                  ),
                  keyboardType: TextInputType.text,
                  controller: _controller_country,
                  onSaved: (val) => widget.employee.country = val.isEmpty?null:val,
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: '',
                    labelText: 'Postal code',
                  ),
                  keyboardType: TextInputType.number,
                  controller: _controller_postalCode,
                  onSaved: (val) => widget.employee.postalCode = val.isEmpty?null:val,
                ),
                Container(
                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                      _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                    ],
                  ),
                )
              ]
            )
          )
        )
      )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.employee, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.employee, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  bool isValidPhoneNumber(String input) {
    if( input== null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r'^[0-9]{9,13}$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

}