
import '/hrm/model/entity/employee.dart';
import 'package:flutter/material.dart';



class EditContact extends StatefulWidget {
  EditContact({Key key, @required this.employee, this.onDispose}) : super(key: key);
  final Employee employee;
  final Function onDispose;
  @override
  _EditContactState createState() => _EditContactState( );
}

class _EditContactState extends State<EditContact>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_mobile = new TextEditingController();
  final TextEditingController _controller_homePhone = new TextEditingController();
  final TextEditingController _controller_workPhone = new TextEditingController();
  final TextEditingController _controller_privateEmail = new TextEditingController();
  final TextEditingController _controller_workEmail = new TextEditingController();
  bool _processing = false;

  @override
  initState() {
    super.initState( );
    _controller_mobile.text = widget.employee.mobilePhone ?? '';
    _controller_homePhone.text = widget.employee.homePhone ?? '';
    _controller_workPhone.text = widget.employee.workPhone ?? '';
    _controller_privateEmail.text = widget.employee.privateEmail ?? '';
    _controller_workEmail.text = widget.employee.workEmail ?? '';
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new Form(
      key: _formKey,
      autovalidate: true,
      child: Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.phone),
                    hintText: '09 xxx xxx xxx',
                    labelText: 'Mobile phone',
                  ),
                  keyboardType: TextInputType.phone,
                  controller: _controller_mobile,
                  onSaved: (val) => widget.employee.mobilePhone = val.isEmpty? null : val,
                  validator: (val) => isValidPhoneNumber(val) ? null : 'Please enter a valid phone number',
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.phone),
                    hintText: '09 xxx xxx xxx',
                    labelText: 'Home phone',
                  ),
                  keyboardType: TextInputType.phone,
                  controller: _controller_homePhone,
                  onSaved: (val) => widget.employee.homePhone = val.isEmpty? null : val,
                  validator: (val) => isValidHomePhone(val) ? null : 'Please enter a valid phone number',
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.phone),
                    hintText: '09 xxx xxx xxx',
                    labelText: 'Work phone',
                  ),
                  keyboardType: TextInputType.phone,
                  controller: _controller_workPhone,
                  onSaved: (val) => widget.employee.workPhone = val.isEmpty?null:val,
                  validator: (val) => isValidHomePhone(val) ? null : 'Please enter a valid phone number',
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.email),
                    hintText: 'name@domain.com',
                    labelText: 'Private email',
                  ),
                  keyboardType: TextInputType.emailAddress,
                  controller: _controller_privateEmail,
                  onSaved: (val) => widget.employee.privateEmail = val.isEmpty?null:val,
                  validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.alternate_email),
                    hintText: 'name@domain.com',
                    labelText: 'Work email',
                  ),
                  keyboardType: TextInputType.emailAddress,
                  controller: _controller_workEmail,
                  onSaved: (val) => widget.employee.workEmail = val.isEmpty?null:val,
                  validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                ),
                Container(
                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                      _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                    ],
                  ),
                )
              ]
            )
          )
        )
      )
    );
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.employee, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.employee, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  bool isValidPhoneNumber(String input) {
    if( input== null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r'^[0-9]{9,13}$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidHomePhone(String input) {
    if( input== null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r'^[0-9]{8,13}$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }

}