import 'package:dropdown_search/dropdown_search.dart';
import '/hrm/config/constant.dart';
import '/hrm/model/entity/employee.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/department_search_response.dart';
import '/hrm/model/response/employee_search_response.dart';
import '/hrm/model/response/job_titles_response.dart';
import '/hrm/service/department_service.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/job_titles_service.dart';
import '/hrm/service/token_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_datetime_picker/flutter_datetime_picker.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';



class EditOrganization extends StatefulWidget {
  EditOrganization({Key key, @required this.employee, @required this.user, this.onDispose}) : super(key: key);
  final Employee employee;
  final User user;
  final Function onDispose;
  @override
  _EditOrganizationState createState() => _EditOrganizationState();
}

class _EditOrganizationState extends State<EditOrganization>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_joined_date = new TextEditingController();
  final TextEditingController _controller_inTime = new TextEditingController();
  final TextEditingController _controller_outTime = new TextEditingController();
  final TextEditingController _controller_employeeId = new TextEditingController();

  DepartmentSearchResponse _departmentSearchResponse;
  DepartmentSearchItem _departmentSearchItem;
  JobTitleListResponse _jobTitleListResponse;
  JobTitleItem _jobTitleItem;
  EmployeeSearchResponse _employeeSearchResponse;
  EmployeeSearchItem _employeeSearchItem;
  EmployeeSearchItem _approver1;
  EmployeeSearchItem _approver2;
  EmployeeSearchItem _approver3;
  EmployeeService _employeeService;
  bool _processing = false;
  String _accessToken;
  User _currentUser = new User();
  DateTime inDate = DateTime(2020, 02, 01, 9, 00, 00);
  DateTime outDate = DateTime(2020, 02, 01, 17, 00, 00);
  final _dateFormatTimeDisplay = new DateFormat('hh:mm a');
  final _dateFormatTimeOutput = new DateFormat('HH:mm');
  SharedPreferences _sharedPrefs;

  @override
  initState() {
    super.initState( );
    _getAccessToken( true );
    _controller_employeeId.text = widget.employee.employeeId ?? '';
    _controller_joined_date.text = widget.employee.joinedDate!=null?new DateFormat('yyyy-MM-dd').format(widget.employee.joinedDate) : '';
    if( widget.employee.dutyInTime != null){
      inDate = DateTime(2020, 02, 01, widget.employee.dutyInTime.inHours, widget.employee.dutyInTime.inMinutes.remainder(60), 00);
      _controller_inTime.text = _dateFormatTimeDisplay.format(inDate );
    }
    if( widget.employee.dutyOutTime != null){
      outDate = DateTime(2020, 02, 01, widget.employee.dutyOutTime.inHours, widget.employee.dutyOutTime.inMinutes.remainder(60), 00);
      _controller_outTime.text = _dateFormatTimeDisplay.format(outDate );
    }
    setState(() {});
  }

  void _getAccessToken( bool isFirstload) async{
    if( _sharedPrefs == null)
      _sharedPrefs = await SharedPreferences.getInstance();

    await _currentUser.fromSharedPreferences();
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    if(isFirstload)
      _loadData();
  }

  _loadData() async{
    DepartmentService _departmentService = new DepartmentService();
    _departmentSearchResponse = await _departmentService.search(_accessToken, _currentUser.organizationGUID, '');
    _departmentSearchResponse.departments.forEach((element) {
      if( widget.employee.departmentGUID == element.guid){
        _departmentSearchItem = element;
        return;
      }
    });

    JobTitlesService jobTitlesService = new JobTitlesService();
    _jobTitleListResponse = await jobTitlesService.search(_currentUser.organizationGUID, _accessToken, '');
    _jobTitleListResponse.jobtitles.forEach((element) {
      if( widget.employee.jobTitlesGUID == element.guid){
        _jobTitleItem = element;
        return;
      }
    });

    _employeeService = new EmployeeService();
    _employeeSearchResponse = await _employeeService.supervisor(_accessToken, '' );
    _employeeSearchResponse.employees.forEach((element) {
      if( widget.employee.supervisorGUID == element.guid){
        _employeeSearchItem = element;
      }else if( widget.employee.approver1GUID == element.guid){
        _approver1 = element;
      }else if( widget.employee.approver2GUID == element.guid){
        _approver2 = element;
      }else if( widget.employee.approver3GUID == element.guid){
        _approver3 = element;
      }
    });
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return new Form(
        key: _formKey,
        autovalidate: true,
        child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:Card(
                color: Colors.grey[50],
                elevation: 1,
                child: Container(
                    padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
                    child:new Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          new TextFormField(
                            decoration: const InputDecoration(
                              icon: const Icon(Icons.account_box),
                              hintText: '',
                              labelText: 'Employee Id',
                            ),
                            controller: _controller_employeeId,
                            onSaved: (val) => widget.employee.employeeId = val.isEmpty? null : val,
                          ),
                          _jobTitleSelection(context),
                          _deptSelection(context),
                          _supervisorSelection(context),
                          _approver1Selection(context),
                          _approver2Selection(context),
                          _approver3Selection(context),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child:new Row(children: <Widget>[
                              new Expanded(
                                  child: new TextFormField(
                                    decoration: new InputDecoration(
                                      icon: const Icon(Icons.calendar_today),
                                      hintText: 'yyyy-mm-dd',
                                      labelText: 'Joined date',
                                    ),
                                    controller: _controller_joined_date,
                                    keyboardType: TextInputType.datetime,
                                  )),
                              new IconButton(
                                icon: new Icon(Icons.mode_edit),
                                tooltip: 'Choose joined date',
                                onPressed: (() {
                                  _chooseDate(context, _controller_joined_date.text );
                                }),
                              )
                            ]),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child:new Row(children: <Widget>[
                              new Expanded(
                                  child: new TextFormField(
                                    enabled: false,
                                    decoration: new InputDecoration(
                                      icon: const Icon(Icons.timer),
                                      hintText: 'HH:mm',
                                      labelText: 'Duty in time (Optional)',
                                    ),
                                    controller: _controller_inTime,
                                    keyboardType: TextInputType.datetime,
                                  )),
                              new IconButton(
                                icon: new Icon(Icons.mode_edit),
                                tooltip: 'Choose date',
                                onPressed: (() {
                                  DatePicker.showTimePicker(
                                      context,
                                      showTitleActions: true,
                                      onChanged: (date) { },
                                      onConfirm: (date) {
                                        inDate = date;
                                        _controller_inTime.text = _dateFormatTimeDisplay.format(date);
                                        widget.employee.dutyInTime = Duration(hours:inDate.hour, minutes: inDate.minute );
                                      },
                                      currentTime: inDate
                                  );
                                }),
                              )
                            ]),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 5.0, bottom: 5.0, left: 5.0, right: 5.0),
                            child:new Row(children: <Widget>[
                              new Expanded(
                                  child: new TextFormField(
                                    enabled: false,
                                    decoration: new InputDecoration(
                                      icon: const Icon(Icons.access_time),
                                      hintText: 'HH:mm',
                                      labelText: 'Duty out time (Optional)',
                                    ),
                                    controller: _controller_outTime,
                                    keyboardType: TextInputType.datetime,
                                  )),
                              new IconButton(
                                icon: new Icon(Icons.mode_edit),
                                tooltip: 'Choose date',
                                onPressed: (() {
                                  DatePicker.showTimePicker(
                                      context,
                                      showTitleActions: true,
                                      onChanged: (date) { },
                                      onConfirm: (date) {
                                        outDate = date;
                                        _controller_outTime.text = _dateFormatTimeDisplay.format(date);
                                        widget.employee.dutyOutTime = Duration(hours:outDate.hour, minutes: outDate.minute );
                                      },
                                      currentTime: outDate
                                  );
                                }),
                              )
                            ]),
                          ),
                          Container(
                            margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                            child: Row(
                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                              children: [
                                _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                                _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                              ],
                            ),
                          )
                        ]
                    )
                )
            )
        )
    );
  }

  Widget _jobTitleSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<JobTitleItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _jobTitleItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (JobTitleItem i, JobTitleItem s) => i?.guid==s?.guid,
          label: "Job Title",
          onFind: (String filter) => _jobTitleSuggestion(filter),
          onChanged: (JobTitleItem data) {
            _jobTitleItem = data;
            if( _jobTitleItem != null ){
              widget.employee.jobTitles = _jobTitleItem.name;
              widget.employee.jobTitlesGUID = _jobTitleItem.guid;
            }else{
              widget.employee.jobTitles = null;
              widget.employee.jobTitlesGUID = null;
            }
          },
          dropdownBuilder: (BuildContext context, JobTitleItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.event_seat), title: Text("Select the job title"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, JobTitleItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<JobTitleItem>> _jobTitleSuggestion(String query) async{
    List<JobTitleItem> matches = List();
    if( _jobTitleListResponse.jobtitles == null)
      return matches;
    matches.addAll(_jobTitleListResponse.jobtitles);

    matches.retainWhere((s) {
      JobTitleItem jobTitleItem = s;
      if( jobTitleItem.name.toLowerCase().contains(query.toLowerCase()) || jobTitleItem.code.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _deptSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<DepartmentSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _departmentSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (DepartmentSearchItem i, DepartmentSearchItem s) => i?.guid==s?.guid,
          label: "Department",
          onFind: (String filter) => _deptSuggestion(filter),
          onChanged: (DepartmentSearchItem data) {
            _departmentSearchItem = data;
            if( _departmentSearchItem == null ){
              widget.employee.departmentGUID = widget.user.organizationGUID;
              widget.employee.department = widget.user.organizationName;
            }else{
              widget.employee.departmentGUID = _departmentSearchItem.guid;
              widget.employee.department = _departmentSearchItem.name;
            }
          },
          dropdownBuilder: (BuildContext context, DepartmentSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.place), title: Text("Select the Department"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, DepartmentSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<DepartmentSearchItem>> _deptSuggestion(String query) async{
    List<DepartmentSearchItem> matches = List();
    if( _departmentSearchResponse.departments == null)
      return matches;

    matches.addAll(_departmentSearchResponse.departments);

    matches.retainWhere((s) {
      DepartmentSearchItem dept = s;
      if( dept.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Widget _supervisorSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _employeeSearchItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: "Supervisor",
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _employeeSearchItem = data;
            if( _employeeSearchItem == null ){
              widget.employee.supervisor = null;
              widget.employee.supervisorGUID = null;
            }else{
              widget.employee.supervisor = _employeeSearchItem.name;
              widget.employee.supervisorGUID = _employeeSearchItem.guid;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text("Select the Supervisor"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }

  Widget _approver1Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver1,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: "Approver 1",
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver1 = data;
            if( _approver1 == null ){
              widget.employee.approver1GUID = null;
            }else{
              widget.employee.approver1GUID = _approver1.guid;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text("Select the approver 1"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Widget _approver2Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver2,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: "Approver 2",
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver2 = data;
            if( _approver2 == null ){
              widget.employee.approver2GUID = null;
            }else{
              widget.employee.approver2GUID = _approver2.guid;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text("Select the approver 2"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }
  Widget _approver3Selection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<EmployeeSearchItem>(
//          isFilteredOnline: true,
//          autoValidate: true,
//          validator: (RoleItem i) => i == null ? "Field is required " : null,
//          hint: "Select a role",
//          popupTitle: Text('Select a role'),
          selectedItem: _approver3,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (EmployeeSearchItem i, EmployeeSearchItem s) => i?.guid==s?.guid,
          label: "Approver 3",
          onFind: (String filter) => _supervisorSuggestion(filter),
          onChanged: (EmployeeSearchItem data) {
            _approver3 = data;
            if( _approver3 == null ){
              widget.employee.approver3GUID = null;
            }else{
              widget.employee.approver3GUID = _approver3.guid;
            }
          },
          dropdownBuilder: (BuildContext context, EmployeeSearchItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.supervised_user_circle), title: Text("Select the approver 3"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
          popupItemBuilder: (BuildContext context, EmployeeSearchItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected,
                  title: Text(item.name),
                  subtitle: Text('${item.jobTitleName}\n${item.departmentName}'),
//                  trailing: Text('${item.departmentName}'),
                  leading: new Image.network( '${getBaseURL(_sharedPrefs)}/v1/recogser/recognition/download_profile/${item.faceGuid}',
                    headers: {'Authorization':'Bearer ${_accessToken}'}, width: 42, height: 42, ),
                ),
              ),
        )
    );
  }

  Future<List<EmployeeSearchItem>> _supervisorSuggestion(String query) async{
    List<EmployeeSearchItem> matches = List();
    if( _employeeSearchResponse.employees == null)
      return matches;
    matches.addAll(_employeeSearchResponse.employees);

    matches.retainWhere((s) {
      EmployeeSearchItem employeeSearchItem = s;
      if( employeeSearchItem.name.toLowerCase().contains(query.toLowerCase()))
        return true;
      else
        return false;
    }
    );
    return matches;
  }

  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.employee, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.employee, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }

  Future _chooseDate(BuildContext context, String initialDateString ) async {
    var now = new DateTime.now();
    var initialDate = convertToDate(initialDateString) ?? now;
    initialDate = (initialDate.year >= 1950 && initialDate.isBefore(now) ? initialDate : now);

    var result = await showDatePicker(
        context: context,
        initialDate: initialDate,
        firstDate: new DateTime(1950),
        lastDate: new DateTime.now()
    );

    if (result == null) return;

    setState(() {
      _controller_joined_date.text = new DateFormat('yyyy-MM-dd').format(result);
      widget.employee.joinedDate = result;
    });
  }

  DateTime convertToDate(String input) {
    try {
      var d = new DateFormat('yyyy-MM-dd').parseStrict(input);
      return d;
    } catch (e) {
      return null;
    }
  }

}