import 'package:dropdown_search/dropdown_search.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/user_role_list_response.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import 'package:flutter/material.dart';



class EditUser extends StatefulWidget {
  EditUser({Key key, @required this.currentUser, this.onDispose}) : super(key: key);
  final User currentUser;
  final Function onDispose;
  @override
  _EditUserState createState() => _EditUserState();
}

class _EditUserState extends State<EditUser>{
  final GlobalKey<FormState> _formKey = new GlobalKey<FormState>();
  final TextEditingController _controller_email = new TextEditingController();
  final TextEditingController _controller_phone = new TextEditingController();
  bool _processing = false;
  User _currentUser = new User();
  String _accessToken;
  RoleListResponse _roleListResponse;
  RoleItem _roleItem;
  UserService _userService;

  @override
  initState() {
    super.initState( );
    _controller_email.text = widget.currentUser.email ?? '';
    _controller_phone.text = widget.currentUser.phone ?? '';
    setState(() {});
    _getAccessToken( true );
  }

  _loadData() async{
    _userService = new UserService( );
    _roleListResponse = await _userService.roles(_accessToken);
    _roleListResponse.roles.forEach((element) {
      if( widget.currentUser.roleGUID == element.guid){
        setState(() {_roleItem = element;});
        return;
      }
    });
  }

  void _getAccessToken( bool isFirstload ) async{
    await _currentUser.fromSharedPreferences();
    TokenService tokenService = new TokenService();
    _accessToken = await tokenService.faceLogin( _currentUser.facialGUID, false );
    if(isFirstload)
      _loadData();
  }


  @override
  Widget build(BuildContext context) {
    return new Form(
      key: _formKey,
      autovalidateMode: AutovalidateMode.onUserInteraction,
      child: Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Container(
            padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
            child:new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                _rolesSelection(context),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.person),
                    hintText: '',
                    labelText: 'Email address',
                  ),
                  controller: _controller_email,
                  onSaved: (val) => widget.currentUser.email = val.isEmpty? null : val,
                  validator: (value) => isValidEmail(value) ? null : 'Please enter a valid email address',
                ),
                new TextFormField(
                  decoration: const InputDecoration(
                    icon: const Icon(Icons.person),
                    hintText: '',
                    labelText: 'Phone number',
                  ),
                  controller: _controller_phone,
                  onSaved: (val) => widget.currentUser.phone = val.isEmpty? null : val,
                  validator: (val) => isValidPhoneNumber(val) ? null : 'Please enter a valid phone number',
                ),
                Container(
                  margin: const EdgeInsets.only(top: 15.0, bottom: 15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      _buildButtons( Theme.of(context).primaryColor, Icons.save, 'Save' ),
                      _buildButtons( Colors.transparent, Icons.cancel, 'Cancel'),
                    ],
                  ),
                )
              ]
            )
          )
        )
      )
    );
  }

  Widget _rolesSelection( BuildContext context){
    return Container(
        margin: EdgeInsets.only(top: 20),
        child: DropdownSearch<RoleItem>(
          selectedItem: _roleItem,
          mode: Mode.DIALOG,
          showSearchBox: true,
          showClearButton: true,
          // autoFocusSearchBox: true,
          // showSelectedItem: true,
          compareFn: (RoleItem i, RoleItem s) => i?.guid==s?.guid,
          label: "Role",
          onFind: (String filter) => _rolesSuggestion(filter),
          onChanged: (RoleItem data) {
            _roleItem = data;
            if( _roleItem != null ){
              widget.currentUser.roleName = _roleItem.name;
              widget.currentUser.roleGUID = _roleItem.guid;
            }
          },
          dropdownBuilder: (BuildContext context, RoleItem item ) =>
              Container(
                child: (item?.name == null) ?
                ListTile(
                  contentPadding: EdgeInsets.all(0),
                  leading: Icon(Icons.security), title: Text("Select the role"),
                )
                    : ListTile(
                  contentPadding: EdgeInsets.all(0), title: Text(item.name),
                ),
              ),
          popupItemBuilder: (BuildContext context, RoleItem item, bool isSelected) =>
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8),
                decoration: !isSelected ? null
                    : BoxDecoration(
                  border: Border.all(color: Theme.of(context).primaryColor),
                  borderRadius: BorderRadius.circular(5),
                  color: Colors.white,
                ),
                child: ListTile(
                  selected: isSelected, title: Text(item.name),
                ),
              ),
        )
    );
  }
  Future<List<RoleItem>> _rolesSuggestion(query) async {
    List<RoleItem> matches = List();
    if( _roleListResponse.roles == null)
      return matches;
    matches.addAll(_roleListResponse.roles);
    matches.retainWhere((s) {
      RoleItem roleItem = s;
      if( roleItem.name.toLowerCase().contains(query.toLowerCase()) )
        return true;
      else
        return false;
    }
    );
    return matches;
  }


  Column _buildButtons(Color color, IconData icon, String label) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        RaisedButton.icon(
          elevation: 2.0,
          icon: Icon(icon, color: Colors.white),
          color: color,
          onPressed:_processing ? null: () async{
            if( label == 'Cancel'){
              widget.onDispose(widget.currentUser, false);
            }else{
              final FormState form = _formKey.currentState;
              if (!form.validate()) {
              } else {
                form.save();
                widget.onDispose(widget.currentUser, true);
              }
            }
          },
          label: Text(
            label,
            style: TextStyle(
              fontSize: 16,
              fontWeight: FontWeight.w400,
              color: Colors.white,
            ),
          ),
        ),
      ],
    );
  }
  bool isValidPhoneNumber(String input) {
    if( input== null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r'^[0-9]{9,13}$');//new RegExp(r'^\(\d\d\d\)\d\d\d\-\d\d\d\d$');
    return regex.hasMatch(input);
  }

  bool isValidEmail(String input) {
    if( input==null || input.isEmpty) return true;
    final RegExp regex = new RegExp(r"^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,253}[a-zA-Z0-9])?)*$");
    return regex.hasMatch(input);
  }
}