import 'package:easy_localization/easy_localization.dart';
import '/hrm/config/constant.dart';
import '/hrm/imageprocessing/base64image.dart';
import '/hrm/model/entity/employee.dart';
import '/hrm/model/entity/user.dart';
import '/hrm/model/response/authorities_response.dart';
import '/hrm/model/response/photos_list_by_face_response.dart';
import '/hrm/service/employee_service.dart';
import '/hrm/service/face_recognization_service.dart';
import '/hrm/service/token_service.dart';
import '/hrm/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'edit_about.dart';
import 'edit_address.dart';
import 'edit_contact.dart';
import 'edit_organization.dart';
import 'edit_password.dart';
import 'edit_photos.dart';
import 'edit_user.dart';



class ProfileHome extends StatefulWidget {
  ProfileHome({Key key, @required this.userGUID, @required this.employeeGUID,
    @required this.showExit, @required this.sharedPrefs}) : super(key: key);
  final String userGUID;
  final String employeeGUID;
  final bool showExit;
  final SharedPreferences sharedPrefs;

  @override
  _ProfileHomeState createState() => _ProfileHomeState();
}

class _ProfileHomeState extends State<ProfileHome>{

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  User _currentUser = new User();
  AuthorityResponse _authorityResponse = new AuthorityResponse();
  UserService _userService = new UserService();
  EmployeeService _employeeService = new EmployeeService();
  Employee _employee;
  FaceRecognizationService _faceRecognizationService = new FaceRecognizationService();
  PhotosListByFace _photosListByFace;


  Color _progressColor = Colors.transparent;
  bool _processing = false;
  String _accessToken;
  bool _editAbout = false;
  bool _editOrganization = false;
  bool _editContact = false;
  bool _editAddress = false;
  bool _editPhotos = false;
  bool _editUser = false;
  bool _editPassword = false;


  @override
  initState() {
    super.initState( );
    _loadProfile();
  }

  void _loadProfile( )async{
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    // loading login user for token generating
    await _currentUser.fromSharedPreferences( );
    await _authorityResponse.fromSharedPreferences();
    await _getAccessToken(_currentUser.facialGUID, false);

    // loading required user data
    _currentUser = await _userService.getUserInformation(_accessToken, widget.userGUID, false);
    setState(() {});
    _employee = await _employeeService.retrieve( _accessToken, widget.employeeGUID );
    setState(() {});
    _photosListByFace = await _faceRecognizationService.listPhotosByFace( _accessToken, _currentUser.facialGUID );
    setState(() {
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  Future<void> _getAccessToken( String facialGUID, bool force ) async{
    TokenService tokenService = new TokenService();
    String token =  await tokenService.faceLogin( facialGUID, force );
    setState(() {
      _accessToken = token;
    });
  }


  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: _onWillPop,
        child: Scaffold(
          key: _scaffoldKey,
          resizeToAvoidBottomInset: true,
          // resizeToAvoidBottomPadding: true,
          appBar: AppBar(
            title: Text('Profile'),
            bottom: _createProgressIndicator(),
          ),
          body: Center(
                child: SingleChildScrollView(
                    child: Column(
                        children: <Widget>[
                          _editPhotos? _photosEditPanel(context) : _photosPanel(context),
                          _editUser? _userEditPanel(context) : _userPanel(context),
                          _editAbout? _aboutEditPanel(context) : _aboutPanel( context ),
                          _editOrganization? _organizationEditPanel(context) : _organizationPanel(context),
                          _editContact ? _contactEditPanel(context) : _contactPanel(context),
                          _editAddress ? _addressEditPanel(context) : _addressPanel(context),
                          _editPassword ? _loginEditPanel(context) : _loginPanel(context),
                          widget.showExit?Container(
                              margin: const EdgeInsets.only(left: 10.0, right: 10.0, top: 20),
                              child: Card(
                                  child: ListTile(
                                title: Text('Logout'),
                                subtitle: Text('Press here to exit your account!'),
                                trailing: Icon(Icons.exit_to_app, color: Theme.of(context).primaryColor ),
                                onTap: () async{
                                  _exit( context );
                                },
                              )
                              )
                          ) : new Container(),
                          _authorityResponse.authorities!=null
                              && _authorityResponse.authorities.contains('MANAGER_WRITE')
                              && !widget.showExit && _employee !=null
                              && _employee.status==Constant.EMPLOYEE_STATUS_ACTIVE? Container(
                              margin: const EdgeInsets.only(left: 10.0, right: 10.0, top: 20),
                              child: Card(
                                  child: ListTile(
                                    title: Text(
                                        'Terminate ${_currentUser.name}',
                                      style: TextStyle(
                                        color: Colors.red[500],
                                      ),
                                    ),
                                    subtitle: Text(
                                        'Press here if you want to terminate "${_currentUser.name}" from your organization',
                                    ),
                                    leading: Icon(Icons.delete_forever, color: Colors.red[500] ),
                                    onTap: () async{
                                      _terminate( context );
                                    },
                                  ),
                              )
                          ) : new Container(),
                        ]
                    )
                )
          )
        )
    );
  }

  Widget _photosPanel( BuildContext context ){
    return Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Container(
                  padding: const EdgeInsets.only(top:0.0),
                  child: ClipOval(
                      child: Material(
                        elevation: 2,
                        color: Colors.white, // button color
                        child: InkWell(
                          splashColor: Colors.red, // inkwell color
                          child: SizedBox(
                            width: 150,
                            height: 150,
                            child: _currentUser !=null && _currentUser.facialGUID!=null? new Image.network( '${getBaseURL(widget.sharedPrefs)}/v1/recogser/recognition/download_profile/${_currentUser.facialGUID}',
                              headers: {'Authorization':'Bearer ${_accessToken}'},)
                            : Icon(Icons.account_circle, size: 120,),
                          ),
                        ),
                      )
                  ),
                ),
                ListTile(
                  title: Text('Face recognition'),
                  subtitle: _photosListByFace!=null? Container(
                      margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 10),
                      child: new Wrap(
                          spacing: 10,
                          runSpacing: 10,
                          children: <Widget>[
                            Container(
                              child: ClipOval(
                                  child: Material(
                                    elevation: 1,
                                    color: Colors.white, // button color
                                    child: InkWell(
                                      splashColor: Colors.red, // inkwell color
                                      child: SizedBox(
                                        width: 50,
                                        height: 50,
                                        child: _photosListByFace.face1!=null? decode( _photosListByFace.face1 ): Icon(Icons.account_circle, color: Colors.grey[50],),
                                      ),
                                    ),
                                  )
                              ),
                            ),
                            Container(
                              child: ClipOval(
                                  child: Material(
                                    elevation: 1,
                                    color: Colors.white, // button color
                                    child: InkWell(
                                      splashColor: Colors.red, // inkwell color
                                      child: SizedBox(
                                        width: 50,
                                        height: 50,
                                        child: _photosListByFace.face2!=null? decode( _photosListByFace.face2 ): Icon(Icons.account_circle, color: Colors.grey[50],),
                                      ),
                                    ),
                                  )
                              ),
                            ),
                            Container(
                              child: ClipOval(
                                  child: Material(
                                    elevation: 1,
                                    color: Colors.white, // button color
                                    child: InkWell(
                                      splashColor: Colors.red, // inkwell color
                                      child: SizedBox(
                                        width: 50,
                                        height: 50,
                                        child: _photosListByFace.face3!=null? decode( _photosListByFace.face3 ): Icon(Icons.account_circle, color: Colors.grey[50],),
                                      ),
                                    ),
                                  )
                              ),
                            ),
                          ]
                      )
                  ) : Container(),
                  trailing: Container(
                      child: RaisedButton.icon(
                        elevation: 0.0,
                        icon: Icon(Icons.photo_library, color: Theme.of(context).primaryColor,),
                        color: Colors.grey[50],
                        onPressed:() async{
                          setState(() {
                            _editPhotos = true;
                            _editAbout = false;
                            _editOrganization = false;
                            _editContact = false;
                            _editAddress = false;
                            _editUser = false;
                            _editPassword = false;
                          });
                        },
                        label: Text(
                          'Edit',
                          style: TextStyle(
                            fontWeight: FontWeight.w400,
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                      )
                  ),
                ),
              ]
          ),
        )
    );
  }
  Widget _photosEditPanel(BuildContext context){
    return _photosListByFace!= null ? EditPhotos(
      photosListByFace: _photosListByFace,
      accessToken: _accessToken,
      faceGuid: _currentUser.facialGUID,
      onDispose: (images, isSave) async{
        setState(() {
          _editPhotos = false;
        });
        if( isSave ){
          setState(() {
            _processing = true;
            _progressColor = Colors.white;
          });
          if ( images[0]!='no' && images[0] != _photosListByFace.profile){

            await _getAccessToken(_currentUser.facialGUID, false);
            bool result = await _faceRecognizationService.updateProfile(_accessToken, _currentUser.facialGUID, images[0]);
            if( !result){
              showMessage('Profile image updating failed!');
              setState(() {
                _processing = false;
                _progressColor = Colors.transparent;
              });
              return;
            }else{
              showMessage('Update successful.');
            }
          }

          if( images[1]==_photosListByFace.face1
              && images[2]==_photosListByFace.face2
              && images[3]==_photosListByFace.face3){
            setState(() {
              _processing = false;
              _progressColor = Colors.transparent;
            });
            return;
          }

          if( images[1]=='no' && images[2]=='no' && images[3]=='no'){
            showMessage('At least 1 face photo must be uploaded!');
            setState(() {
              _processing = false;
              _progressColor = Colors.transparent;
            });
            return;
          }

          await _getAccessToken(_currentUser.facialGUID, false);
          bool result = await _faceRecognizationService.updatePhotos(_accessToken, _currentUser.facialGUID, images);
          if( !result )
            showMessage('Face photos updating failed!');
          else
            showMessage('Update successful.');
          _photosListByFace = await _faceRecognizationService.listPhotosByFace( _accessToken, _currentUser.facialGUID );

          setState(() {
            _processing = false;
            _progressColor = Colors.transparent;
          });
        }
      },
      onChanged: (){
        setState(() {
          _processing = true;
          _progressColor = Colors.white;
        });
      },
      onCompleted: (message){
        if( message != 'OK')
          showMessage(message);
        setState(() {
          _processing = false;
          _progressColor = Colors.transparent;
        });
      },
    ) : new Container();
  }

  Widget _userPanel( BuildContext context ){
    return Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                _authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('MANAGER_WRITE')? Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() async{
                        setState(() {
                          _editUser = true;
                          _editAbout = false;
                          _editOrganization = false;
                          _editContact = false;
                          _editAddress = false;
                          _editPhotos = false;
                          _editPassword = false;
                        });
                        },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ) : new Container(),
                _currentUser!=null? new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0),
                    child: ListTile(
                      title: Text('${_currentUser.name??''}'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Organization: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.organizationName??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Roles: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.roleName??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Email: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.email??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Phone: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.phone??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ) : Container(),
              ]
          ),
        )
    );
  }
  Widget _userEditPanel(BuildContext context){
    return _currentUser!= null ? EditUser(
      currentUser: _currentUser,
      onDispose: (user, isSave) async{
        setState(() {
          _editUser = false;
        });
        if( isSave ){
          _updateUser( );
        }
      },
    ) : new Container();
  }

  Widget _aboutPanel( BuildContext context ){
    return _employee!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() async{
                        setState(() {
                          _editAbout = true;
                          _editOrganization = false;
                          _editContact = false;
                          _editAddress = false;
                          _editPhotos = false;
                          _editUser = false;
                          _editPassword = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('About'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('First name: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.firstName??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Last name: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.lastName??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Gender: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.gender??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Date of birth: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          _employee.birthday!=null?"${new DateFormat('dd MMMM yyyy').format(_employee.birthday.toLocal())}" : '',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Nationality: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.nationality??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Marital status: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.maritalStatus??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('NRC No.: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.nicNum??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _aboutEditPanel(BuildContext context){
    return _employee!= null ? EditAbout(
      employee: _employee,
      onDispose: (employee, isSave) async{
        setState(() {
          _editAbout = false;
        });
        if( isSave ){
          await _update();
        }
      },
    ) : new Container();
  }

  Widget _organizationPanel( BuildContext context ){
    return _employee!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                _authorityResponse.authorities!=null
                    && _authorityResponse.authorities.contains('MANAGER_WRITE')? Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() async{
                        setState(() {
                          _editOrganization = true;
                          _editAbout = false;
                          _editContact = false;
                          _editAddress = false;
                          _editPhotos = false;
                          _editUser = false;
                          _editPassword = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ) : new Container(),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('Organization'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Title: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.jobTitles??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Department: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.department??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Employee Id: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.employeeId??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Supervisor: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.supervisor??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Approver1: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.approver1Name??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Approver2: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.approver2Name??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Approver3: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.approver3Name??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Duty time in: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          _employee.dutyInTime!=null?
                                          "${_employee.dutyInTime.inHours.toString().padLeft(2, "0")}:${_employee.dutyInTime.inMinutes.remainder(60).toString().padLeft(2, "0")}":'',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Duty time out: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          _employee.dutyOutTime!=null?
                                          "${_employee.dutyOutTime.inHours.toString().padLeft(2, "0")}:${_employee.dutyOutTime.inMinutes.remainder(60).toString().padLeft(2, "0")}":'',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Joined Date: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                          child: Text(
                                            _employee.joinedDate!=null?"${new DateFormat('dd MMMM yyyy').format(_employee.joinedDate.toLocal())}" : '',
                                            style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                          )
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Status: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.status??''}',
                                          style: TextStyle(color: _employee.status==Constant.EMPLOYEE_STATUS_TERMINATED? Colors.red[700] : Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _organizationEditPanel(BuildContext context){
    return _employee!= null ? EditOrganization(
      employee: _employee,
      user: _currentUser,
      onDispose: (employee, isSave) async{
        setState(() {
          _editOrganization = false;
        });
        if( isSave ){
          await _update();
        }
      },
    ) : new Container();
  }

  Widget _contactPanel( BuildContext context ){
    return _employee!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() {
                        setState(() {
                          _editAbout = false;
                          _editOrganization = false;
                          _editContact = true;
                          _editAddress = false;
                          _editPhotos = false;
                          _editUser = false;
                          _editPassword = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('Contact'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Mobile: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.mobilePhone??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Home phone: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.homePhone??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Work phone: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.workPhone??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Private email: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.privateEmail??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Work email: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.workEmail??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _contactEditPanel(BuildContext context){
    return _employee!= null ? EditContact(
      employee: _employee,
      onDispose: (employee, isSave) async{
        setState(() {
          _editContact = false;
        });
        if( isSave ){
          await _update();
        }
      },
    ) : new Container();
  }

  Widget _addressPanel( BuildContext context ){
    return _employee!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() {
                        setState(() {
                          _editAddress = true;
                          _editAbout = false;
                          _editOrganization = false;
                          _editContact = false;
                          _editPhotos = false;
                          _editUser = false;
                          _editPassword = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('Address'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Address1: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.address1??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Address2: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.address2??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('City: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.city??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('State/division: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.province??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Country: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.country??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Postal code: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_employee.postalCode??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _addressEditPanel(BuildContext context){
    return _employee!= null ? EditAddress(
      employee: _employee,
      onDispose: (employee, isSave) async{
        setState(() {
          _editAddress = false;
        });
        if( isSave ){
          await _update();
        }
      },
    ) : new Container();
  }

  Widget _loginPanel( BuildContext context ){
    return _currentUser!= null ? Container(
        padding: const EdgeInsets.only(left: 10.0, top: 10.0, right: 10.0),
        child:Card(
          color: Colors.grey[50],
          elevation: 1,
          child: Column(
              children: <Widget>[
                Align(
                    alignment: Alignment.topRight,
                    child: RaisedButton.icon(
                      elevation: 0.0,
                      icon: Icon(Icons.edit, color: Theme.of(context).primaryColor,),
                      color: Colors.grey[50],
                      onPressed:() {
                        setState(() {
                          _editPassword = true;
                          _editAbout = false;
                          _editOrganization = false;
                          _editContact = false;
                          _editAddress = false;
                          _editPhotos = false;
                          _editUser = false;
                        });
                      },
                      label: Text(
                        'Edit',
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                    )
                ),
                new Container(
                    margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 0.0),
                    child: ListTile(
                      title: Text('Login security'),
                      subtitle: new Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('username: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '${_currentUser.username??''}',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                            Container(
                                margin: const EdgeInsets.only(left: 0.0, right: 0.0, top: 5),
                                child: new Wrap(
                                    children: <Widget>[
                                      Container(
                                        child: Text('Password: '),
                                        width: MediaQuery.of(context).size.width*0.235,
                                      ),
                                      Container(
                                        child: Text(
                                          '*** *** ***',
                                          style: TextStyle(color: Colors.black, fontWeight: FontWeight.w400,),
                                        ),
                                      ),
                                    ]
                                )
                            ),
                          ]
                      ),
                    )
                ),
              ]
          ),
        )
    ) : new Container();
  }
  Widget _loginEditPanel(BuildContext context){
    return _currentUser!= null ? EditPassword(
      onDispose: (password, isSave, message) async{
        if( isSave ){
          if( message != 'OK'){
            showMessage(message);
          }else{
            setState(() {
              _editPassword = false;
            });
            await _changePassword(password);
          }
        }else{
          setState(() {
            _editPassword = false;
          });
        }
      },
    ) : new Container();
  }

  PreferredSize _createProgressIndicator() => PreferredSize(
      preferredSize: Size(double.infinity, 4.0),
      child: SizedBox(
          height: 4.0,
          child: LinearProgressIndicator(backgroundColor: _progressColor,)
      )
  );
  void showMessage(String message, [MaterialColor color = Colors.blueGrey]) {
    _scaffoldKey.currentState.showSnackBar(new SnackBar(backgroundColor: Theme.of(context).primaryColor, content: new Text(message)));
  }
  Future<bool> _onWillPop() async {
    return true;
  }
  @override
  void deactivate() {
    super.deactivate( );
  }

  Future<void> _update() async{
    if( _processing) return null;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });
    await _getAccessToken(_currentUser.facialGUID, false);
    bool result = await _employeeService.employeeUpdate(_employee.toMap(), _accessToken);

    if( result ){
      showMessage('Update successful.');
    }else{
      showMessage('Update failed!');
      _employee = await _employeeService.retrieve( _accessToken );
    }

    setState((){
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  Future<void> _updateUser() async{
    if( _processing) return null;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });
    await _getAccessToken(_currentUser.facialGUID, false);
    Map<String, dynamic> data = {
      'email': _currentUser.email,
      'phone': _currentUser.phone,
      'roleGUID': _currentUser.roleGUID,
      'guid': _currentUser.guid
    };
    bool result = await _userService.userUpdate( data, _accessToken );

    if( result ){
      showMessage('Update successful.');
    }else{
      showMessage('Update failed!');
    }
    _currentUser = await _userService.getUserInformation(_accessToken, _currentUser.guid, false);
    setState((){
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }

  Future<void> _changePassword(String password) async{
    if( _processing) return null;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });
    await _getAccessToken(_currentUser.facialGUID, true);
    Map<String, dynamic> data = {
      'newPassword': password,
      'guid': _currentUser.guid
    };
    bool result = await _userService.changePassword( data, _accessToken );

    if( result ){
      showMessage('Update successful.');
    }else{
      showMessage('Update failed!');
    }
    setState((){
      _processing = false;
      _progressColor = Colors.transparent;
    });
  }
  Future<void> _exit( BuildContext context) async{
    // if( _processing) return;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    bool result = await _showConfirmDialog(context, "Logout confirm", "Are you sure to exit from App?");
    if ( !result ){
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
      return;
    }

    UserService userService = new UserService();
    await userService.logout( );
    Navigator.pop(context, 'exit');
  }

  Future<void> _terminate( BuildContext context) async{
    if( _processing) return;
    setState(() {
      _processing = true;
      _progressColor = Colors.white;
    });

    bool result = await _showConfirmDialog(context, 'Termination confirm', 'Are you sure to terminate "${_currentUser.name}" from your organization?');
    if ( !result ){
      setState(() {
        _processing = false;
        _progressColor = Colors.transparent;
      });
      return;
    }

    result = await _employeeService.employeeTerminate( _employee.guid, _accessToken );
    if( !result )
      showMessage('Problem, termination failed.');
    else
      Navigator.pop(context);
  }

  Future<bool> _showConfirmDialog( BuildContext context, String title, String message )async {
    bool result = false;
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        // return object of type Dialog
        return AlertDialog(
          title: new Text(title),
          content: new Text(message),
          actions: <Widget>[
            // usually buttons at the bottom of the dialog
            new FlatButton(
              child: new Text("Yes"),
              onPressed: () { result = true;Navigator.of(context).pop();},
            ),
            new FlatButton(
              child: new Text("No"),
              onPressed: () { result = false; Navigator.of(context).pop();},
            ),
          ],
        );
      },
    );
    return result;
  }

}