import 'dart:io';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_downloader/flutter_downloader.dart';
import 'package:provider/provider.dart';
import 'package:qq_delivery/theme.dart';
import 'package:responsive_framework/responsive_framework.dart';
import 'delivery/config/constant.dart';
import 'delivery/generated/locale_keys.g.dart';
import 'delivery/service/firebase_service.dart';
import 'delivery/view/landing.dart';
import 'delivery/view/login.dart';

class MyHttpOverrides extends HttpOverrides{
  @override
  HttpClient createHttpClient(SecurityContext context){
    return super.createHttpClient(context)
      ..connectionTimeout = const Duration(seconds: 10)
      ..badCertificateCallback = (X509Certificate cert, String host, int port)=> true;
  }
}

void main() async{

  WidgetsFlutterBinding.ensureInitialized();
  try {
    await EasyLocalization.ensureInitialized();
  }catch(e){}
  try {
    await FlutterDownloader.initialize(debug: Constant.ENABLE_LOGS);
  }catch(e){}
  try{
    FirebaseMessage _firebaseMessage = FirebaseMessage( );
    await _firebaseMessage.initMessaging( );
    // await _firebaseMessage.initLocalNotification();
  }catch(e){}

  HttpOverrides.global = new MyHttpOverrides();
  runApp(AppWithTheme( ));
}

class AppWithTheme extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => ThemeChanger( getDefaultTheme(Constant.THEME_NAME_BRANDED_DEFAULT) )),
      ],
      child: EasyLocalization(
          supportedLocales: [Locale('en', ''), Locale('my', '')],
          path: 'assets/translations', // <-- change patch to your
          fallbackLocale: Locale('en', ''),
          child: MyApp()
      ),
    );
  }
}


class MyApp extends StatelessWidget {

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeChanger>(context);
    return MaterialApp(
      builder: (context, widget) => ResponsiveWrapper.builder(
        BouncingScrollWrapper.builder(context, widget),
        maxWidth: 1200,
        minWidth: 480,
        defaultScale: true,
        breakpoints: [
          ResponsiveBreakpoint.resize(450, name: MOBILE),
          ResponsiveBreakpoint.autoScale(800, name: TABLET),
          ResponsiveBreakpoint.autoScale(1000, name: TABLET),
          ResponsiveBreakpoint.resize(1200, name: DESKTOP),
          ResponsiveBreakpoint.autoScale(2460, name: "4K"),
        ],),
      title: '${Constant.APP_NAME}',
      theme: theme.getTheme,
      debugShowCheckedModeBanner: false,
      localizationsDelegates: context.localizationDelegates,
      supportedLocales: context.supportedLocales,
      locale: context.locale,
      home: Welcome(),
    );
  }
}
